--
-- PostgreSQL database dump
--

SET client_encoding = 'LATIN1';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

--
-- Name: acesso; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA acesso;


--
-- Name: alimentos; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA alimentos;


--
-- Name: cadastro; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA cadastro;


--
-- Name: consistenciacao; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA consistenciacao;


--
-- Name: historico; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA historico;


--
-- Name: modules; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA modules;


--
-- Name: pmiacoes; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA pmiacoes;


--
-- Name: pmicontrolesis; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA pmicontrolesis;


--
-- Name: pmidrh; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA pmidrh;


--
-- Name: pmieducar; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA pmieducar;


--
-- Name: pmiotopic; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA pmiotopic;


--
-- Name: portal; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA portal;


--
-- Name: urbano; Type: SCHEMA; Schema: -; Owner: -
--

CREATE SCHEMA urbano;


--
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: -; Owner: -
--

CREATE PROCEDURAL LANGUAGE plpgsql;


SET search_path = public, pg_catalog;

--
-- Name: typ_idlog; Type: TYPE; Schema: public; Owner: -
--

CREATE TYPE typ_idlog AS (
	idlog integer
);


--
-- Name: typ_idpes; Type: TYPE; Schema: public; Owner: -
--

CREATE TYPE typ_idpes AS (
	idpes integer
);


SET search_path = alimentos, pg_catalog;

--
-- Name: fcn_calcular_qtde_cardapio(character varying, integer, integer, numeric); Type: FUNCTION; Schema: alimentos; Owner: -
--

CREATE FUNCTION fcn_calcular_qtde_cardapio(character varying, integer, integer, numeric) RETURNS character varying
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_cli ALIAS for $1;
  v_id_car ALIAS for $2;
  v_id_pro ALIAS for $3;
  v_qtde_per_capita ALIAS for $4;
  reg_unidade RECORD;
  v_inscritos char(1);
  v_correcao numeric:=0;
  v_coccao numeric:=0;
  v_qtde numeric:= 0;
  v_qtde_retorno varchar(20);
BEGIN
  SELECT INTO v_inscritos inscritos
  FROM alimentos.cliente
  WHERE alimentos.cliente.idcli = v_id_cli;
  SELECT INTO v_correcao,v_coccao fator_correcao,fator_coccao
  FROM alimentos.produto
  WHERE alimentos.produto.idpro = v_id_pro;
  FOR reg_unidade IN SELECT unf.num_inscritos as insc, unf.num_matriculados as matr
        FROM alimentos.cardapio_faixa_unidade car,
                     alimentos.unidade_faixa_etaria unf
        WHERE car.idfeu = unf.idfeu
        AND   car.idcar = v_id_car LOOP
    IF v_inscritos = 'S' THEN
      v_qtde := v_qtde + (v_qtde_per_capita * reg_unidade.insc * v_correcao * v_coccao);
    ELSE
      v_qtde := v_qtde + (v_qtde_per_capita * reg_unidade.matr * v_correcao * v_coccao);
    END IF;
  END LOOP;

  v_qtde_retorno := trim(to_char(v_qtde, '9999999999.99'));
  RETURN v_qtde_retorno;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_calcular_qtde_percapita(character varying, integer, integer, integer, integer); Type: FUNCTION; Schema: alimentos; Owner: -
--

CREATE FUNCTION fcn_calcular_qtde_percapita(character varying, integer, integer, integer, integer) RETURNS numeric
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_cli ALIAS for $1;
  v_id_uni ALIAS for $2;
  v_id_pro ALIAS for $3;
  v_data_inicial ALIAS for $4;
  v_data_final ALIAS for $5;
  v_sql_qtde_produto text;
  v_reg_qtde_produto RECORD;
  v_qtde_produto_periodo numeric;
  v_existe_unidade integer;
BEGIN
  v_sql_qtde_produto := 'SELECT car.dt_cardapio
    ,cap.quantidade as qtde
    ,car.idcar as idcar
  FROM alimentos.cardapio car
      ,alimentos.cardapio_produto cap
  WHERE car.idcar = cap.idcar
  AND cap.idpro = ' || v_id_pro || '
  AND car.finalizado = ''S''
  AND car.idcli = ''' ||  v_id_cli || '''
  AND TO_NUMBER(TO_CHAR(car.dt_cardapio,''YYYYMMDD''),''99999999'') BETWEEN ' || v_data_inicial || ' AND  ' || v_data_final;
  v_qtde_produto_periodo := 0;
  v_existe_unidade := 0;
  FOR v_reg_qtde_produto IN EXECUTE v_sql_qtde_produto LOOP
    --
    -- Verifica se o cardpio atende a unidade em questo
    --
    SELECT INTO v_existe_unidade distinct uni.iduni
    FROM alimentos.cardapio car
      ,alimentos.cardapio_faixa_unidade caf
      ,alimentos.unidade_atendida uni
      ,alimentos.unidade_faixa_etaria ufa
    WHERE car.idcar = caf.idcar
    AND caf.idfeu = ufa.idfeu
    AND ufa.iduni = uni.iduni
    AND uni.iduni = v_id_uni
    AND car.idcar = v_reg_qtde_produto.idcar;
    IF v_existe_unidade > 0 THEN
      v_qtde_produto_periodo := v_qtde_produto_periodo  + v_reg_qtde_produto.qtde;
    END IF;
  END LOOP;
  RETURN v_qtde_produto_periodo;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_calcular_qtde_unidade(character varying, integer, integer, numeric, integer, integer); Type: FUNCTION; Schema: alimentos; Owner: -
--

CREATE FUNCTION fcn_calcular_qtde_unidade(character varying, integer, integer, numeric, integer, integer) RETURNS numeric
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_cli ALIAS for $1;
  v_id_uni ALIAS for $2;
  v_id_pro ALIAS for $3;
  v_qtde_per_capita ALIAS for $4;
  v_data_inicial ALIAS for $5;
  v_data_final ALIAS for $6;
  v_reg_faixa RECORD;
  v_inscritos char(1);
  v_correcao numeric:=0;
  v_coccao numeric:=0;
  v_qtde numeric:= 0;
  v_peso numeric:=0;
  v_qtde_produto_periodo numeric:=0;
  v_num_inscritos integer;
  v_num_matriculados integer;
BEGIN
  SELECT INTO v_inscritos inscritos
  FROM alimentos.cliente
  WHERE alimentos.cliente.idcli = v_id_cli;
  SELECT INTO v_correcao,v_coccao,v_peso pro.fator_correcao, pro.fator_coccao, unp.peso
  FROM alimentos.produto pro, alimentos.unidade_produto unp
  WHERE   pro.idunp = unp.idunp
  AND unp.idcli = v_id_cli
  AND     pro.idpro = v_id_pro;
  --
  -- Obtm inscritos e matriculados para a unidade
  --
  v_num_inscritos:=0;
  v_num_matriculados:=0;
  FOR v_reg_faixa IN SELECT distinct ufa.idfae as idfae
      ,ufa.num_inscritos as num_inscritos
      ,ufa.num_matriculados as num_matriculados
    FROM  alimentos.cardapio car
      ,alimentos.cardapio_faixa_unidade caf
      ,alimentos.unidade_atendida uni
      ,alimentos.unidade_faixa_etaria ufa
    WHERE car.idcar = caf.idcar
    AND   caf.idfeu = ufa.idfeu
    AND ufa.iduni = uni.iduni
    AND     uni.iduni = v_id_uni
    AND     car.finalizado = 'S'
    AND car.idcli = v_id_cli
    AND TO_NUMBER(TO_CHAR(car.dt_cardapio,'YYYYMMDD'),'99999999') BETWEEN v_data_inicial AND v_data_final LOOP
    v_num_inscritos:=v_num_inscritos + v_reg_faixa.num_inscritos;
    v_num_matriculados:=v_num_matriculados + v_reg_faixa.num_matriculados;
  END LOOP;
  --
  -- Calcula a qtde percapita necessria do produto para o perodo
  --
  select INTO v_qtde_produto_periodo
  alimentos.fcn_calcular_qtde_percapita (v_id_cli,v_id_uni,v_id_pro,v_data_inicial,v_data_final);
  IF v_inscritos = 'S' THEN
    v_qtde := v_qtde_produto_periodo * v_num_inscritos * v_correcao * v_coccao;
  ELSE
    v_qtde := v_qtde_produto_periodo * v_num_matriculados * v_correcao * v_coccao;
  END IF;
  RETURN round(v_qtde/v_peso);
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_gerar_guia_remessa(text, text, integer, integer, character varying, character varying, character varying, integer); Type: FUNCTION; Schema: alimentos; Owner: -
--

CREATE FUNCTION fcn_gerar_guia_remessa(text, text, integer, integer, character varying, character varying, character varying, integer) RETURNS text
    AS $_$DECLARE
   -- Parmetro recebidos
   v_array_fornecedor ALIAS for $1;
   v_array_unidade ALIAS for $2;
   v_data_inicial ALIAS for $3;
   v_data_final ALIAS for $4;
   v_classe ALIAS for $5;
   v_id_cliente ALIAS for $6;
   v_login ALIAS for $7;
   v_id_log ALIAS for $8;
   v_sql_unidade text;
   v_reg_unidade RECORD;
   v_reg_faixa RECORD;
   v_sql_produto text;
   v_reg_produto RECORD;
   v_sql_fornecedor text;
   v_reg_fornecedor RECORD;
   v_num_inscritos integer;
   v_num_matriculados integer;
   v_inscritos char(1);
   v_qtde_necessaria numeric;         -- quantidade total necessaria para o produto
   v_qtde_necessaria_saldo numeric;   -- saldo do produto a fornecer
   v_qtde_nec_saldo_ant numeric;      -- saldo anterior do produto
   v_qtde_guia_produto numeric;       -- qtde do produto com guia de remessa gerada
   v_qtde_disponivel numeric;         -- qtde total disponvel no(s) fornecedor(res)
   v_qtde_total_contrato numeric;     -- qtde total contratada com o fornecedor
   v_percentual_forn numeric;         -- percentual que pode ser fornecido pelo forn
   -- quantidade fornecida para o produto pelo fornecedor na guia
   v_qtde_forn numeric;
   v_qtde_guia integer:=0;
   v_qtde_produto_periodo numeric:=0; -- qtde de produto necessria para o perodo
              --(somatria da qtde percapita da receita)

   v_id_guia integer:=0;
   v_id_guia_produto integer:=0;
   v_sequencial integer:=0;         -- sequencial da guia de remessa gerada
   v_num_inscr_matr integer:=0;
   v_num_refeicao integer:=0;
   v_dt_cardapio_ini_x date;
   v_dt_cardapio_ini_y integer:=0;
   v_dt_cardapio_ini date;
   v_dt_cardapio_fim date;
   v_classe_aux VARCHAR(2);
   v_peso numeric;
   v_existe_forn integer:=0;
   v_existe_produto integer:=0;
   v_existe_estoque integer:=0;
   --v_id_log integer:=0;

BEGIN
   --
   -- Converte data numrica invertida para o formato DD/MM/YYYY
   --
   v_dt_cardapio_ini := TO_DATE(substr(TRIM(TO_CHAR(v_data_inicial,'99999999')),7,2)|| '/' || substr(TRIM(TO_CHAR(v_data_inicial,'99999999')),5,2)|| '/' || substr(TRIM(TO_CHAR(v_data_inicial,'99999999')),1,4),'DD/MM/YYYY');
   v_dt_cardapio_ini_x := v_dt_cardapio_ini;
   v_dt_cardapio_fim := TO_DATE(substr(TRIM(TO_CHAR(v_data_final,'99999999')),7,2)|| '/' || substr(TRIM(TO_CHAR(v_data_final,'99999999')),5,2)|| '/' || substr(TRIM(TO_CHAR(v_data_final,'99999999')),1,4),'DD/MM/YYYY');
   --
   -- INSERE LOG
   --
   --v_id_log:=nextval('alimentos.log_guia_remessa_idlogguia_seq'::text);
   INSERT INTO alimentos.log_guia_remessa
    (idlogguia
    ,login
    ,idcli
    ,dt_inicial
    ,dt_final
    ,unidade
    ,fornecedor
    ,classe
    ,dt_geracao
    ,mensagem)
  VALUES
    (v_id_log
    ,v_login
    ,v_id_cliente
    ,v_dt_cardapio_ini
    ,v_dt_cardapio_fim
    ,v_array_unidade
    ,v_array_fornecedor
    ,v_classe
    ,CURRENT_TIMESTAMP
    ,'*** INCIO ***');
   --
   -- Obtm indicao de inscritos ou matriculados para o cliente
   --
   SELECT INTO v_inscritos inscritos
   FROM alimentos.cliente
   WHERE alimentos.cliente.idcli = v_id_cliente;
   RAISE NOTICE 'Inscritos(%)',v_inscritos;
   --
   -- GRAVA LOG
   --
   UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n Inscritos: ' || v_inscritos WHERE idlogguia = v_id_log;
   --
   -- Obtm o nmero de tipos de refeies que sero atendidas
   -- (total caf da manh, total almoo, etc)
   --
   SELECT INTO v_num_refeicao COUNT(distinct idtre)
   FROM alimentos.cardapio car
   WHERE car.idcli = v_id_cliente
   AND car.finalizado = 'S'
   AND TO_NUMBER(TO_CHAR(car.dt_cardapio,'YYYYMMDD'),'99999999') BETWEEN v_data_inicial AND v_data_final;
   WHILE v_dt_cardapio_ini_x <= v_dt_cardapio_fim LOOP

      RAISE NOTICE '---------------->>1 Loop Data(%)',v_dt_cardapio_ini_x;

      --
      -- GRAVA LOG
      --
      UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      ----- 1 Loop Data ' || TO_CHAR(v_dt_cardapio_ini_x,'DD/MM/YYYY') || ' -----' WHERE idlogguia = v_id_log;
      --
      -- Prepara SELECT para obter as unidades que atendem aos filtros informados
      --
      v_sql_unidade := 'SELECT distinct uni.iduni as iduni
      ,TRIM(uni.nome) as nome
      FROM alimentos.cardapio car
       ,alimentos.cardapio_faixa_unidade caf
       ,alimentos.unidade_atendida uni
       ,alimentos.unidade_faixa_etaria ufa
      WHERE car.idcar = caf.idcar
      AND caf.idfeu = ufa.idfeu
      AND ufa.iduni = uni.iduni';

      IF v_array_unidade <> '0' THEN
         v_sql_unidade := v_sql_unidade || ' AND uni.iduni IN (' || v_array_unidade || ')';
      END IF;

      v_sql_unidade := v_sql_unidade || ' AND car.finalizado = ''S''
      AND car.idcli = ''' || v_id_cliente || '''
      AND car.dt_cardapio = ''' || v_dt_cardapio_ini_x || '''';

      --
      -- Executa SELECT para obter as unidades
      --
      FOR v_reg_unidade IN EXECUTE v_sql_unidade LOOP

         RAISE NOTICE '                                          ';
         RAISE NOTICE '2 Loop Unidade(%)',v_reg_unidade.nome;

         --
         -- GRAVA LOG
         --
         UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      ----- 2 Loop Unidade ' || TRIM(TO_CHAR(v_reg_unidade.iduni, '9999999999')) || '-' || v_reg_unidade.nome || ' -----' WHERE idlogguia = v_id_log;
         v_num_inscritos:=0;
   v_num_matriculados:=0;

         --
         -- Obtm inscritos e matriculados para a unidade
         --

         FOR v_reg_faixa IN SELECT distinct ufa.idfae as idfae
                   ,ufa.num_inscritos as num_inscritos
             ,ufa.num_matriculados as num_matriculados
      FROM alimentos.cardapio car
    ,alimentos.cardapio_faixa_unidade caf
    ,alimentos.unidade_atendida uni
    ,alimentos.unidade_faixa_etaria ufa
      WHERE car.idcar = caf.idcar
      AND caf.idfeu = ufa.idfeu
      AND ufa.iduni = uni.iduni
      AND uni.iduni = v_reg_unidade.iduni
      AND car.finalizado = 'S'
      AND car.idcli = v_id_cliente
      AND car.dt_cardapio = v_dt_cardapio_ini_x LOOP

            v_num_inscritos:=v_num_inscritos + v_reg_faixa.num_inscritos;
      v_num_matriculados:=v_num_matriculados + v_reg_faixa.num_matriculados;

   END LOOP;
   RAISE NOTICE 'Inscritos(%)',v_num_inscritos;
   RAISE NOTICE 'Matriculados(%)',v_num_matriculados;

         --
         -- GRAVA LOG
         --
         UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      N Inscritos: ' || TRIM(TO_CHAR(v_num_inscritos, '9999999999')) WHERE idlogguia = v_id_log;
   UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      N Matriculados: ' || TRIM(TO_CHAR(v_num_matriculados, '9999999999')) WHERE idlogguia = v_id_log;
   --
   -- Prepara SELECT para carregar produtos existentes no cardpio
   --
         v_sql_produto := 'SELECT distinct cap.idpro as idpro
        ,cap.quantidade as qtde
        ,pro.fator_correcao as fator_correcao
        ,pro.fator_coccao as fator_coccao
        ,TRIM(pro.nome_compra) as nome_compra
        ,unp.peso as peso
      FROM  alimentos.cardapio car
        ,alimentos.cardapio_produto cap
        ,alimentos.produto pro
        ,alimentos.unidade_produto unp
        ,alimentos.cardapio_faixa_unidade caf
        ,alimentos.unidade_atendida uni
        ,alimentos.unidade_faixa_etaria ufa
      WHERE car.idcar = cap.idcar
      AND cap.idpro = pro.idpro
      AND     pro.idunp = unp.idunp
      AND     unp.idcli = ''' || v_id_cliente || '''
      AND car.idcar = caf.idcar
      AND   caf.idfeu = ufa.idfeu
      AND ufa.iduni = uni.iduni';
   IF v_classe is not null THEN
      v_sql_produto := v_sql_produto || ' AND pro.classe = ''' || v_classe || '''';
         END IF;

         --
   -- Seleciona somente os produtos do cardpio que o fornecedor
         -- pode fornecedor
   --
         IF v_array_fornecedor <> '0' THEN
      v_sql_produto := v_sql_produto || ' AND pro.idpro IN (
         SELECT cop.idpro
            FROM alimentos.contrato con
          ,alimentos.contrato_produto cop
                  WHERE cop.idcon = con.idcon
                AND con.cancelado = ''N''
                AND con.finalizado = ''S''
                AND con.ultimo_contrato = ''S''
                AND con.idfor IN (' || v_array_fornecedor || ')
                  AND cop.qtde_remessa < cop.qtde_contratada
                  AND cop.idpro = pro.idpro)';
         END IF;

         v_sql_produto := v_sql_produto || '
      AND uni.iduni = ' || v_reg_unidade.iduni || '
            AND car.finalizado = ''S''
            AND car.idcli = ''' || v_id_cliente || '''
            AND car.dt_cardapio = ''' || v_dt_cardapio_ini_x || '''
            ORDER BY TRIM(pro.nome_compra)';

         --
         -- Obtm os produtos do cardpio utilizados pela unidade
         --

         FOR v_reg_produto IN EXECUTE v_sql_produto LOOP

            RAISE NOTICE '                                          ';
            RAISE NOTICE '3 Loop Produto(%)',v_reg_produto.nome_compra;

            --
            -- GRAVA LOG
            --
            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n      3 Loop Produto ' || TRIM(TO_CHAR(v_reg_produto.idpro, '9999999999')) || '-' || v_reg_produto.nome_compra WHERE idlogguia = v_id_log;

            v_existe_produto := v_existe_produto + 1;

            --
            -- Calcula a qtde percapita necessria do produto para o perodo
            --

            v_dt_cardapio_ini_y := TO_NUMBER(TO_CHAR(v_dt_cardapio_ini_x, 'YYYYMMDD'),'99999999');

            select INTO v_qtde_produto_periodo alimentos.fcn_calcular_qtde_percapita (v_id_cliente, v_reg_unidade.iduni, v_reg_produto.idpro, v_dt_cardapio_ini_y, v_dt_cardapio_ini_y);

            --
            --
            -- Calcula a qtde necessria para o produto e para a unidade considerando
            -- os inscritos/matriculados
            --
            if v_inscritos = 'S' THEN
               v_qtde_necessaria := v_qtde_produto_periodo * v_num_inscritos * v_reg_produto.fator_correcao * v_reg_produto.fator_coccao;
            ELSE
               v_qtde_necessaria := v_qtde_produto_periodo * v_num_matriculados * v_reg_produto.fator_correcao * v_reg_produto.fator_coccao;
            END IF;

            --
            -- Divide qtde total em gramas pelo peso da unidade do produto
            --
            v_qtde_necessaria := ROUND(v_qtde_necessaria / v_reg_produto.peso);

            --
            -- Obtm qtde do produto j gerado em outras guias
            --
            v_qtde_guia_produto := 0;

            SELECT INTO v_qtde_guia_produto COALESCE(SUM(qtde),0)
               FROM alimentos.guia_produto_diario
               WHERE idpro = v_reg_produto.idpro
               AND iduni = v_reg_unidade.iduni
               AND dt_guia = v_dt_cardapio_ini_x;

            v_qtde_necessaria_saldo:=v_qtde_necessaria - v_qtde_guia_produto;


            --
            -- GRAVA LOG
            --
            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      Qtde per capita: ' || TRIM(TO_CHAR(v_qtde_produto_periodo, '9999999999.99')) WHERE idlogguia = v_id_log;

            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      Fator correo: ' || TRIM(TO_CHAR(v_reg_produto.fator_correcao, '9999999999.99')) WHERE idlogguia = v_id_log;

            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      Fator coco: ' || TRIM(TO_CHAR(v_reg_produto.fator_coccao, '9999999999.99')) WHERE idlogguia = v_id_log;

            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      Peso: ' || TRIM(TO_CHAR(v_reg_produto.peso, '9999999999.99')) WHERE idlogguia = v_id_log;
            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      Qtde necessria: ' || TRIM(TO_CHAR(v_qtde_necessaria, '9999999999.99')) WHERE idlogguia = v_id_log;

            RAISE NOTICE 'Qtde per capita(%)',v_qtde_produto_periodo;
            RAISE NOTICE 'Fator correo(%)',v_reg_produto.fator_correcao;
            RAISE NOTICE 'Fator coco(%)',v_reg_produto.fator_coccao;
            RAISE NOTICE 'Peso(%)',v_reg_produto.peso;
            RAISE NOTICE 'Qtde Necessria(%)',v_qtde_necessaria;

            --
            -- Obtm qtde do produto em contrato, incluindo
            -- todos os fornecedores autorizados a fornecer para a unidade
            -- e que tenham produtos a fornecer (qtde remessa < qtde contrato)
            -- ou qtde remessa = qtde_contrato (estoque zerado) e contrato vigente porque
            -- possuem outros produtos a fornecer
            --
            v_sql_fornecedor := 'SELECT con.idfor as idfor
               ,COALESCE(SUM(cop.qtde_contratada-cop.qtde_remessa),0) as qtde
               ,SUM(cop.qtde_contratada) as qtde_contratada
               FROM alimentos.contrato con
                   ,alimentos.contrato_produto cop
               WHERE con.idcon = cop.idcon
         AND con.cancelado = ''N''
         AND con.finalizado = ''S''
         AND con.ultimo_contrato = ''S''
         AND con.idfor IN (SELECT idfor
                  FROM alimentos.fornecedor_unidade_atendida
                  WHERE iduni = ' || v_reg_unidade.iduni || ')
               AND (cop.qtde_remessa < cop.qtde_contratada
               OR  (cop.qtde_remessa  = cop.qtde_contratada
               AND (SELECT COALESCE(SUM(cop2.qtde_contratada-cop2.qtde_remessa),0)
                       FROM alimentos.contrato_produto cop2
                       WHERE cop2.idcon = con.idcon
                       AND cop2.idpro <> ' ||  v_reg_produto.idpro || ' ) > 0 ))
               AND cop.idpro = ' || v_reg_produto.idpro || '
               GROUP BY con.idfor ';

            v_qtde_total_contrato := 0;
            v_existe_forn := 0;

            FOR v_reg_fornecedor IN EXECUTE v_sql_fornecedor LOOP
               v_existe_forn := 1;
               v_qtde_total_contrato:= v_qtde_total_contrato + v_reg_fornecedor.qtde_contratada;
            END LOOP;

            RAISE NOTICE 'Qtde total contrato(%)',v_qtde_total_contrato;

            --
            -- GRAVA LOG
            --
            UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      Qtde total contrato: ' || TRIM(TO_CHAR(v_qtde_total_contrato, '9999999999.99')) WHERE idlogguia = v_id_log;

            --
            -- Testa se fornecedor tem contrato e autorizao para fornecer para a
            -- unidade
            --
            IF v_existe_forn = 0 THEN

               --
               -- GRAVA LOG
               --
               UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n      *** existe forn: N ***' WHERE idlogguia = v_id_log;

               RETURN '-1 Produto: ' || v_reg_produto.nome_compra || ' no tem contrato ou o fornecedor contratado no est autorizado a fornecer para a unidade ' || v_reg_unidade.nome || '.';

            END IF;

            --
            -- Qtde necessria deve ser > 0
            --
            IF v_qtde_necessaria > 0 THEN

               --
               -- Calcula a quantidade a ser fornecida por cada fornecedor
               --

               -- PREPARA SELECT
               v_sql_fornecedor := 'SELECT con.idcon as idcon
                  ,con.idfor as idfor
                  ,TRIM(con.codigo) as codigo
                  ,cop.qtde_contratada as qtde_contratada
                  ,cop.qtde_remessa as qtde_remessa
                  ,cop.idcop as idcop
                  ,TRIM(forn.nome_fantasia) as nome
                  FROM  alimentos.contrato con
                       ,alimentos.contrato_produto cop
                       ,alimentos.fornecedor forn
                  WHERE con.idcon = cop.idcon
                  AND con.idfor = forn.idfor
                  AND     con.cancelado = ''N''
                  AND     con.finalizado = ''S''
                  AND con.ultimo_contrato = ''S''';

               IF v_array_fornecedor <> '0' THEN
                  v_sql_fornecedor := v_sql_fornecedor || ' AND con.idfor IN (' || v_array_fornecedor || ')';
               END IF;

               v_sql_fornecedor := v_sql_fornecedor || ' AND con.idfor IN
                  (SELECT idfor
                      FROM alimentos.fornecedor_unidade_atendida
                      WHERE iduni = ' || v_reg_unidade.iduni || ')
                      AND cop.qtde_remessa < cop.qtde_contratada
                      AND cop.idpro = ' || v_reg_produto.idpro;

               v_qtde_disponivel := 0;
               v_existe_estoque := 0;

               -- EXECUTA SELECT PREPARADO
               FOR v_reg_fornecedor IN EXECUTE v_sql_fornecedor LOOP

                  RAISE NOTICE '                                          ';
                  RAISE NOTICE '4 Loop Fornecedor(%)',v_reg_fornecedor.nome;

                  --
                  -- GRAVA LOG
                  --
                  UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n           4 Loop Fornecedor ' || TRIM(TO_CHAR(v_reg_fornecedor.idfor, '9999999999')) || '-' || v_reg_fornecedor.nome || ' Contrato: ' || v_reg_fornecedor.codigo  WHERE idlogguia = v_id_log;

                  v_existe_estoque := 1;

                  v_qtde_disponivel:=COALESCE(v_reg_fornecedor.qtde_contratada- v_reg_fornecedor.qtde_remessa,0);

                  v_percentual_forn:=(v_reg_fornecedor.qtde_contratada * 100) / v_qtde_total_contrato;

                  v_qtde_forn:=ROUND((v_qtde_necessaria * v_percentual_forn) / 100 );

                  --
                  -- Verifica quantidade de saldo a ser fornecida  < 0
                  -- Deve ser verificado por questes de arredondamento,
                  -- evitando enviar uma qtde maior que a necessria

                  v_qtde_nec_saldo_ant := v_qtde_necessaria_saldo;
                  v_qtde_necessaria_saldo:=v_qtde_necessaria_saldo - v_qtde_forn;

                  RAISE NOTICE 'Qtde disponvel(%)',v_qtde_disponivel;
                  RAISE NOTICE 'Percentual(%)',v_percentual_forn;
                  RAISE NOTICE 'Qtde a fornecer(%)',v_qtde_forn;

                  --
                  -- GRAVA LOG
                  --
                  UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || '  \n           Qtde disponvel: ' || TRIM(TO_CHAR(v_qtde_disponivel, '9999999999.99')) WHERE idlogguia = v_id_log;

                  UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || '  \n           Percentual: ' || TRIM(TO_CHAR(v_percentual_forn, '9999999999.9999999999')) WHERE idlogguia = v_id_log;

                  UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || '  \n           Qtde a fornecer: ' || TRIM(TO_CHAR(v_qtde_forn, '9999999999')) WHERE idlogguia = v_id_log;

                  IF v_qtde_necessaria_saldo < 0 THEN

                     v_qtde_forn := v_qtde_nec_saldo_ant;
                     RAISE NOTICE 'Qtde a fornecer ajustada(%)',v_qtde_forn;

                     --
                     -- GRAVA LOG
                     --
                     UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || '  \n           Qtde a fornecer ajustada: ' || TRIM(TO_CHAR(v_qtde_forn, '9999999999')) WHERE idlogguia = v_id_log;

                  END IF;

                  --
                  -- Qtde a ser fornecida do produto deve ser maior que zero
                  -- Qtde disponvel > qtde a ser fornecida

                  IF v_qtde_forn > 0 AND v_qtde_disponivel >= v_qtde_forn THEN

                     --
                     -- Grava a Guia de Remessa para a Unidade, Forn, Produto
                     -- e Contrato
                     --
                     v_id_guia:=0;
                     -- Verifica se existe guia gravada
                     SELECT INTO v_id_guia gui.idgui
                        FROM alimentos.guia_remessa gui
                        WHERE gui.iduni = v_reg_unidade.iduni
                        AND gui.idfor = v_reg_fornecedor.idfor
                        AND gui.idcon = v_reg_fornecedor.idcon
                        AND gui.classe_produto = COALESCE (v_classe, gui.classe_produto)
                        AND gui.situacao = 'E'
                        AND gui.dt_cardapio_inicial  = v_dt_cardapio_ini
                        AND gui.dt_cardapio_final = v_dt_cardapio_fim FOR UPDATE;

                     IF v_id_guia IS NULL THEN

                        -- Obtm ID prxima guia
                        v_id_guia := nextval( 'alimentos.guia_remessa_idgui_seq'::text);

                        IF v_inscritos = 'S' THEN
                           v_num_inscr_matr := v_num_inscritos;
                        ELSE
                           v_num_inscr_matr := v_num_matriculados;
                        END IF;

                        v_classe_aux := v_classe;
                        IF v_classe IS NULL THEN
                           v_classe_aux := 'PN';
                        END IF;

                        -- Obtm o prximo sequencial anual para guia
                        SELECT INTO v_sequencial
                           COALESCE(MAX(sequencial),0) + 1
                           FROM alimentos.guia_remessa gui
                           WHERE gui.ano = TO_NUMBER(TO_CHAR(CURRENT_TIMESTAMP, 'YYYY'),'9999')
                           AND gui.idcli = v_id_cliente;

                        --
                        -- GRAVA LOG
                        --
                        UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n           ID Nova guia: ' || TRIM(TO_CHAR(v_id_guia, '9999999999')) || ' Sequencial: ' || TRIM(TO_CHAR(v_sequencial, '9999999999')) WHERE idlogguia = v_id_log;

                        INSERT INTO alimentos.guia_remessa
         (idgui
                           ,idcon
                           ,login_emissao
                           ,idfor
                           ,iduni
                           ,idcli
                           ,dt_emissao
                           ,ano
                           ,sequencial
                           ,dt_cardapio_inicial
                           ,dt_cardapio_final
                           ,num_inscr_matr
                           ,num_refeicao
                           ,situacao
                           ,classe_produto)
                           VALUES (v_id_guia
                           ,v_reg_fornecedor.idcon
                           ,v_login
                           ,v_reg_fornecedor.idfor
                           ,v_reg_unidade.iduni
                           ,v_id_cliente
                           ,CURRENT_TIMESTAMP
                           ,TO_NUMBER(TO_CHAR(CURRENT_TIMESTAMP,'YYYY'),'9999')
                           ,v_sequencial
                           ,v_dt_cardapio_ini
                           ,v_dt_cardapio_fim
                           ,v_num_inscr_matr
                           ,v_num_refeicao
                           ,'E'
                           ,v_classe_aux);

                        v_qtde_guia := v_qtde_guia + 1;

                     END IF;

                     v_id_guia_produto:=0;
                     -- Verifica se produto j existe na guia
                     SELECT INTO v_id_guia_produto gup.idgup
                        FROM alimentos.guia_remessa_produto gup
                            ,alimentos.guia_remessa gui
                        WHERE gui.idgui = gup.idgui
                        AND gup.idpro = v_reg_produto.idpro
                        AND TRIM(gui.login_emissao) = TRIM(v_login)
                        AND gui.idgui = v_id_guia FOR UPDATE;

                     v_peso := v_reg_produto.peso;

                     IF v_peso = 0 THEN
                        v_peso := 1;
                     END IF;

                     IF v_id_guia_produto IS NULL THEN

                        --
                        -- GRAVA LOG
                        --
                        UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n           *** Novo produto na guia ***   ID Guia:  ' || TRIM(TO_CHAR(v_id_guia, '9999999999')) || ' *** ' WHERE idlogguia = v_id_log;

                        -- Insere novo produto na guia
                        INSERT INTO alimentos.guia_remessa_produto (
                            idgui
                           ,idpro
                           ,qtde_per_capita
                           ,qtde_guia
                           ,peso
                           ,qtde_recebida
                           ,peso_total)
                           VALUES(v_id_guia
                           ,v_reg_produto.idpro
                           ,v_reg_produto.qtde
                           ,v_qtde_forn
                           ,v_peso
                           ,0
                           ,ROUND((v_qtde_forn*v_peso),3));

                     ELSE

                        --
                        -- GRAVA LOG
                        --
                        UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n           Update ID guia_produto: ' || TRIM(TO_CHAR(v_id_guia_produto, '9999999999')) || '   ID Guia: ' || TRIM(TO_CHAR(v_id_guia, '9999999999')) WHERE idlogguia = v_id_log;

                        UPDATE alimentos.guia_remessa_produto
                           SET qtde_guia = (qtde_guia + v_qtde_forn)
                           ,peso_total = ROUND(((qtde_guia + v_qtde_forn)*v_peso),3)
                           WHERE idgup = v_id_guia_produto;

                     END IF;

                     --
                     -- Grava guia_produto_diario.
                     -- Armazena a qtde a ser fornecida do produto em uma data
                     -- e unidade, ou seja, diariamente.
                     --
                     INSERT INTO alimentos.guia_produto_diario
                        (idgui
                        ,idpro
                        ,iduni
                        ,dt_guia
                        ,qtde)
                        VALUES
                        (v_id_guia
                        ,v_reg_produto.idpro
                        ,v_reg_unidade.iduni
                        ,v_dt_cardapio_ini_x
                        ,v_qtde_forn);

                     --
                     -- Diminui estoque do produto
                     --
                     UPDATE alimentos.produto
                        SET qtde_estoque = ROUND((qtde_estoque - v_qtde_forn),2)
                        WHERE idpro = v_reg_produto.idpro;

                     --
                     -- Aumenta quantidade de remessa emitida para o contrato
                     --
                     UPDATE alimentos.contrato_produto
                        SET qtde_remessa = ROUND((qtde_remessa + v_qtde_forn),2)
                        WHERE idcop = v_reg_fornecedor.idcop;

                  ELSE

                     IF v_qtde_disponivel < v_qtde_forn THEN

                        --
                        -- GRAVA LOG
                        --
                        UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n           *** Estoque insuficiente ***' WHERE idlogguia = v_id_log;

                        RETURN '-1 Fornecedor: ' ||  v_reg_fornecedor.nome || ' est com estoque insuficiente para o produto: ' || v_reg_produto.nome_compra || '.';

                     END IF;

                  END IF;

               END LOOP;

               --
               -- Identifica que existe contrato com fornecedor, porm
               -- contrato est com estoque zerado.
               IF v_existe_estoque = 0 THEN

                  --
                  -- GRAVA LOG
                  --
                  UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n      *** Estoque zerado ***' WHERE idlogguia = v_id_log;

                  RETURN '-1 Produto: ' ||  v_reg_produto.nome_compra || ' est com estoque zerado.';

               END IF;

            END IF;

         END LOOP;

      END LOOP;

      v_dt_cardapio_ini_x := v_dt_cardapio_ini_x + interval '1 day';

   END LOOP;

   IF v_existe_produto = 0 AND v_array_fornecedor <> '0' THEN

      --
      -- GRAVA LOG
      --
      UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n     *** Fornecedor no possui produtos para fornecer no perodo. ***' WHERE idlogguia = v_id_log;

      RETURN '-1 Fornecedor no possui produtos para fornecer no perodo. Verifique os contratos elaborados com o fornecedor.';

   ELSIF v_qtde_guia > 0 THEN

      --
      -- GRAVA LOG
      --
      UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n *** GUIAS GERADAS: ' ||  TRIM(TO_CHAR(v_qtde_guia, '9999999999')) || ' ***' WHERE idlogguia = v_id_log;

      RETURN 'Foram geradas ' || REPLACE(TRIM(TO_CHAR(v_qtde_guia,'9,999,999,999')),',','.') || ' guias de remessas.';

   ELSE

      --
      -- GRAVA LOG
      --
      UPDATE alimentos.log_guia_remessa SET mensagem = mensagem || ' \n \n     *** No existem dados para gerar guias de acordo com os filtros informados. ***' WHERE idlogguia = v_id_log;

      RETURN 'No existem dados para gerar guias de acordo com os filtros informados.';

   END IF;

END;$_$
    LANGUAGE plpgsql;


SET search_path = cadastro, pg_catalog;

--
-- Name: fcn_aft_documento(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_documento() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes   numeric;
  BEGIN
    v_idpes := NEW.idpes;
    EXECUTE 'DELETE FROM cadastro.documento WHERE ( (rg = 0 OR rg IS NULL) AND (idorg_exp_rg IS NULL) AND data_exp_rg IS NULL AND (sigla_uf_exp_rg IS NULL OR length(trim(sigla_uf_exp_rg))=0) AND (tipo_cert_civil = 0 OR tipo_cert_civil IS NULL) AND (num_termo = 0 OR num_termo IS NULL) AND (num_livro = 0 OR num_livro IS NULL) AND (num_livro = 0 OR num_livro IS NULL) AND (num_folha = 0 OR num_folha IS NULL) AND data_emissao_cert_civil IS NULL AND (sigla_uf_cert_civil IS NULL OR length(trim(sigla_uf_cert_civil))=0) AND (sigla_uf_cart_trabalho IS NULL OR length(trim(sigla_uf_cart_trabalho))=0) AND (cartorio_cert_civil IS NULL OR length(trim(cartorio_cert_civil))=0) AND (num_cart_trabalho = 0 OR num_cart_trabalho IS NULL) AND (serie_cart_trabalho = 0 OR serie_cart_trabalho IS NULL) AND data_emissao_cart_trabalho IS NULL AND (num_tit_eleitor = 0 OR num_tit_eleitor IS NULL) AND (zona_tit_eleitor = 0 OR zona_tit_eleitor IS NULL) AND (secao_tit_eleitor = 0 OR secao_tit_eleitor IS NULL) ) AND idpes='||quote_literal(v_idpes)||'';
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_documento_provisorio(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_documento_provisorio() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_rg        numeric;
  v_uf_expedicao      text;
  v_verificacao_provisorio  numeric;

  v_comando     text;
  v_registro      record;

  BEGIN
    v_idpes     := NEW.idpes;
    v_rg      := COALESCE(NEW.rg, -1);
    v_uf_expedicao    := TRIM(COALESCE(NEW.sigla_uf_exp_rg, ''));

    v_verificacao_provisorio:= 0;

    -- verificar se a situao do cadastro da pessoa  provisrio
    FOR v_registro IN SELECT situacao FROM cadastro.pessoa WHERE idpes=v_idpes LOOP
      IF v_registro.situacao = 'P' THEN
        v_verificacao_provisorio := 1;
      END IF;
    END LOOP;

    -- Verificao para atualizar ou no a situao do cadastro da pessoa para Ativo
    IF LENGTH(v_uf_expedicao) > 0 AND v_rg > 0 AND v_verificacao_provisorio = 1 THEN
      EXECUTE 'UPDATE cadastro.pessoa SET situacao='||quote_literal('A')||'WHERE idpes='||quote_literal(v_idpes)||';';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_fisica(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_fisica() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_idpes_mae     numeric;
  v_idpes_pai     numeric;
  v_idpes_responsavel   numeric;
  v_idpes_conjuge     numeric;

  v_nome_mae      text;
  v_nome_pai      text;
  v_nome_responsavel    text;
  v_nome_conjuge      text;

  v_verificacao_mae   numeric;
  v_verificacao_pai   numeric;
  v_verificacao_conjuge   numeric;
  v_verificacao_responsavel numeric;

  v_num_aviso_mae     numeric;
  v_num_aviso_pai     numeric;
  v_num_aviso_conjuge   numeric;
  v_num_aviso_responsavel   numeric;

  v_existe_aviso_mae    numeric;
  v_existe_aviso_pai    numeric;
  v_existe_aviso_conjuge    numeric;
  v_existe_aviso_responsavel  numeric;

  v_comando     text;
  v_registro      record;

  BEGIN
    v_idpes     := NEW.idpes;
    v_idpes_mae   := NEW.idpes_mae;
    v_idpes_pai   := NEW.idpes_pai;
    v_idpes_responsavel := NEW.idpes_responsavel;
    v_idpes_conjuge   := NEW.idpes_con;
    v_nome_mae    := TRIM(NEW.nome_mae);
    v_nome_pai    := TRIM(NEW.nome_pai);
    v_nome_responsavel  := TRIM(NEW.nome_responsavel);
    v_nome_conjuge    := TRIM(NEW.nome_conjuge);

    v_num_aviso_mae   := 1;
    v_num_aviso_pai   := 2;
    v_num_aviso_conjuge := 3;
    v_num_aviso_responsavel := 4;

    v_verificacao_mae   := 0;
    v_verificacao_pai   := 0;
    v_verificacao_conjuge   := 0;
    v_verificacao_responsavel := 0;

    v_existe_aviso_mae    := 0;
    v_existe_aviso_pai    := 0;
    v_existe_aviso_conjuge    := 0;
    v_existe_aviso_responsavel  := 0;

    -- obter os avisos j existentes para a pessoa
    FOR v_registro IN SELECT aviso FROM cadastro.aviso_nome WHERE idpes=v_idpes LOOP
      IF v_registro.aviso = 1 THEN
        v_existe_aviso_mae := 1;
      ELSIF v_registro.aviso = 2 THEN
        v_existe_aviso_pai := 1;
      ELSIF v_registro.aviso = 3 THEN
        v_existe_aviso_conjuge := 1;
      ELSIF v_registro.aviso = 4 THEN
        v_existe_aviso_responsavel := 1;
      END IF;
    END LOOP;

    -- MAE
    IF v_idpes_mae > 0 AND v_idpes_mae IS NOT NULL AND LENGTH(v_nome_mae) > 0 AND v_nome_mae IS NOT NULL THEN
      FOR v_registro IN SELECT * from public.fcn_compara_nome_pessoa_fonetica(v_nome_mae, v_idpes_mae) LOOP
        v_verificacao_mae := v_registro.fcn_compara_nome_pessoa_fonetica;
      END LOOP;
    ELSE
      v_verificacao_mae := 1;
    END IF;

    -- PAI
    IF v_idpes_pai > 0 AND v_idpes_pai IS NOT NULL AND LENGTH(v_nome_pai) > 0 AND v_nome_pai IS NOT NULL THEN
      FOR v_registro IN SELECT * from public.fcn_compara_nome_pessoa_fonetica(v_nome_pai, v_idpes_pai) LOOP
        v_verificacao_pai := v_registro.fcn_compara_nome_pessoa_fonetica;
      END LOOP;
    ELSE
      v_verificacao_pai := 1;
    END IF;

    -- CONJUGE
    IF v_idpes_conjuge > 0 AND v_idpes_conjuge IS NOT NULL AND LENGTH(v_nome_conjuge) > 0 AND v_nome_conjuge IS NOT NULL THEN
      FOR v_registro IN SELECT * from public.fcn_compara_nome_pessoa_fonetica(v_nome_conjuge, v_idpes_conjuge) LOOP
        v_verificacao_conjuge := v_registro.fcn_compara_nome_pessoa_fonetica;
      END LOOP;
    ELSE
      v_verificacao_conjuge := 1;
    END IF;

    -- RESPONSAVEL
    IF v_idpes_responsavel > 0 AND v_idpes_responsavel IS NOT NULL AND LENGTH(v_nome_responsavel) > 0 AND v_nome_responsavel IS NOT NULL THEN
      FOR v_registro IN SELECT * from public.fcn_compara_nome_pessoa_fonetica(v_nome_responsavel, v_idpes_responsavel) LOOP
        v_verificacao_responsavel := v_registro.fcn_compara_nome_pessoa_fonetica;
      END LOOP;
    ELSE
      v_verificacao_responsavel := 1;
    END IF;
    -- Inserir ou Deletar aviso da MAE
    IF v_verificacao_mae = 0 AND v_existe_aviso_mae = 0 THEN
      EXECUTE 'INSERT INTO cadastro.aviso_nome (idpes, aviso) VALUES ('||quote_literal(v_idpes)||','||v_num_aviso_mae||');';
    ELSIF v_verificacao_mae = 1 AND v_existe_aviso_mae = 1 THEN
      EXECUTE 'DELETE FROM cadastro.aviso_nome WHERE idpes='||quote_literal(v_idpes)||' AND aviso='||v_num_aviso_mae||';';
    END IF;

    -- Inserir ou Deletar aviso do PAI
    IF v_verificacao_pai = 0 AND v_existe_aviso_pai = 0 THEN
      EXECUTE 'INSERT INTO cadastro.aviso_nome (idpes, aviso) VALUES ('||quote_literal(v_idpes)||','||v_num_aviso_pai||');';
    ELSIF v_verificacao_pai = 1 AND v_existe_aviso_pai = 1 THEN
      EXECUTE 'DELETE FROM cadastro.aviso_nome WHERE idpes='||quote_literal(v_idpes)||' AND aviso='||v_num_aviso_pai||';';
    END IF;

    -- Inserir ou Deletar aviso do CONJUGE
    IF v_verificacao_conjuge = 0 AND v_existe_aviso_conjuge = 0 THEN
      EXECUTE 'INSERT INTO cadastro.aviso_nome (idpes, aviso) VALUES ('||quote_literal(v_idpes)||','||v_num_aviso_conjuge||');';
    ELSIF v_verificacao_conjuge = 1 AND v_existe_aviso_conjuge = 1 THEN
      EXECUTE 'DELETE FROM cadastro.aviso_nome WHERE idpes='||quote_literal(v_idpes)||' AND aviso='||v_num_aviso_conjuge||';';
    END IF;

    -- Inserir ou Deletar aviso do RESPONSAVEL
    IF v_verificacao_responsavel = 0 AND v_existe_aviso_responsavel = 0 THEN
      EXECUTE 'INSERT INTO cadastro.aviso_nome (idpes, aviso) VALUES ('||quote_literal(v_idpes)||','||v_num_aviso_responsavel||');';
    ELSIF v_verificacao_responsavel = 1 AND v_existe_aviso_responsavel = 1 THEN
      EXECUTE 'DELETE FROM cadastro.aviso_nome WHERE idpes='||quote_literal(v_idpes)||' AND aviso='||v_num_aviso_responsavel||';';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_fisica_cpf_provisorio(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_fisica_cpf_provisorio() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_cpf       numeric;
  v_verificacao_provisorio  numeric;

  v_comando     text;
  v_registro      record;

  BEGIN
    v_idpes     := NEW.idpes;
    v_cpf     := COALESCE(NEW.cpf, -1);

    v_verificacao_provisorio:= 0;

    -- verificar se a situao do cadastro da pessoa  provisrio
    FOR v_registro IN SELECT situacao FROM cadastro.pessoa WHERE idpes=v_idpes LOOP
      IF v_registro.situacao = 'P' THEN
        v_verificacao_provisorio := 1;
      END IF;
    END LOOP;

    -- Verificao para atualizar ou no a situao do cadastro da pessoa para Ativo
    IF v_cpf > 0 AND v_verificacao_provisorio = 1 THEN
      EXECUTE 'UPDATE cadastro.pessoa SET situacao='||quote_literal('A')||'WHERE idpes='||quote_literal(v_idpes)||';';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_fisica_provisorio(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_fisica_provisorio() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_idpes_mae     numeric;
  v_nome_mae      text;
  v_data_nascimento   text;
  v_verificacao_provisorio  numeric;

  v_comando     text;
  v_registro      record;

  BEGIN
    v_idpes     := NEW.idpes;
    v_idpes_mae   := COALESCE(NEW.idpes_mae, -1);
    v_nome_mae    := TRIM(COALESCE(NEW.nome_mae, ''));
    v_data_nascimento := COALESCE(TO_CHAR(NEW.data_nasc, 'DD/MM/YYYY'), '');

    v_verificacao_provisorio:= 0;

    -- verificar se a situao do cadastro da pessoa  provisrio
    FOR v_registro IN SELECT situacao FROM cadastro.pessoa WHERE idpes=v_idpes LOOP
      IF v_registro.situacao = 'P' THEN
        v_verificacao_provisorio := 1;
      END IF;
    END LOOP;

    -- Verificao para atualizar ou no a situao do cadastro da pessoa para Ativo
    IF v_data_nascimento <> '' AND (LENGTH(v_nome_mae) > 0 OR v_idpes_mae > 0) AND v_verificacao_provisorio = 1 THEN
      EXECUTE 'UPDATE cadastro.pessoa SET situacao='||quote_literal('A')||'WHERE idpes='||quote_literal(v_idpes)||';';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_ins_endereco_externo(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_ins_endereco_externo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes   numeric;
  v_tipo_endereco text;
  BEGIN
    v_idpes   := NEW.idpes;
    v_tipo_endereco := NEW.tipo;
    EXECUTE 'DELETE FROM cadastro.endereco_pessoa WHERE idpes='||quote_literal(v_idpes)||' AND tipo='||v_tipo_endereco||';';
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_ins_endereco_pessoa(); Type: FUNCTION; Schema: cadastro; Owner: -
--

CREATE FUNCTION fcn_aft_ins_endereco_pessoa() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes   numeric;
  v_tipo_endereco text;
  BEGIN
    v_idpes   := NEW.idpes;
    v_tipo_endereco := NEW.tipo;
    EXECUTE 'DELETE FROM cadastro.endereco_externo WHERE idpes='||quote_literal(v_idpes)||' AND tipo='||v_tipo_endereco||';';
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


SET search_path = consistenciacao, pg_catalog;

--
-- Name: fcn_delete_temp_cadastro_unificacao_cmf(integer); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_delete_temp_cadastro_unificacao_cmf(integer) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpes ALIAS for $1;
BEGIN
  -- Deleta dados da tabela temp_cadastro_unificacao_cmf
  DELETE FROM consistenciacao.temp_cadastro_unificacao_cmf WHERE idpes = v_idpes;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_temp_cadastro_unificacao_siam(integer); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_delete_temp_cadastro_unificacao_siam(integer) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpes ALIAS for $1;
BEGIN
  -- Deleta dados da tabela temp_cadastro_unificacao_siam
  DELETE FROM consistenciacao.temp_cadastro_unificacao_siam WHERE idpes = v_idpes;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_documento_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_documento_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes         numeric;

  v_data_expedicao_rg_nova    date;
  v_data_expedicao_rg_antiga    date;
  v_data_emissao_cert_civil_nova    date;
  v_data_emissao_cert_civil_antiga  date;
  v_data_emissao_cart_trabalho_nova date;
  v_data_emissao_cart_trabalho_antiga date;
  v_orgao_expedicao_rg_novo   numeric;
  v_orgao_expedicao_rg_antigo   numeric;
  v_numero_rg_novo      numeric;
  v_numero_rg_antigo      numeric;
  v_numero_titulo_eleitor_novo    numeric;
  v_numero_titulo_eleitor_antigo    numeric;
  v_numero_zona_titulo_novo   numeric;
  v_numero_zona_titulo_antigo   numeric;
  v_numero_secao_titulo_novo    numeric;
  v_numero_secao_titulo_antigo    numeric;
  v_numero_cart_trabalho_novo   numeric;
  v_numero_cart_trabalho_antigo   numeric;
  v_numero_serie_cart_trabalho_novo numeric;
  v_numero_serie_cart_trabalho_antigo numeric;
  v_tipo_certidao_civil_novo    numeric;
  v_tipo_certidao_civil_antigo    numeric;
  v_numero_termo_certidao_civil_novo  numeric;
  v_numero_termo_certidao_civil_antigo  numeric;
  v_numero_livro_certidao_civil_novo  varchar;
  v_numero_livro_certidao_civil_antigo  varchar;
  v_numero_folha_certidao_civil_novo  numeric;
  v_numero_folha_certidao_civil_antigo  numeric;
  v_cartorio_certidao_civil_novo    text;
  v_cartorio_certidao_civil_antigo  text;
  v_uf_expedicao_rg_novo      text;
  v_uf_expedicao_rg_antigo    text;
  v_uf_emissao_certidao_civil_novo  text;
  v_uf_emissao_certidao_civil_antigo  text;
  v_uf_emissao_carteira_trabalho_novo text;
  v_uf_emissao_carteira_trabalho_antigo text;

  v_comando     text;
  v_origem_gravacao   text;

  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;

  v_nova_credibilidade    numeric;

  v_registro      record;
  v_aux_data_nova     text;
  v_aux_data_antiga   text;

  -- ID dos campos
  v_idcam_numero_rg     numeric;
  v_idcam_orgao_expedidor_rg    numeric;
  v_idcam_data_expedicao_rg   numeric;
  v_idcam_uf_expedicao_rg     numeric;
  v_idcam_tipo_certidao_civil   numeric;
  v_idcam_numero_termo_certidao_civil numeric;
  v_idcam_numero_livro_certidao_civil numeric;
  v_idcam_numero_folha_certidao_civil numeric;
  v_idcam_data_emissao_certidao_civil numeric;
  v_idcam_cartorio_certidao_civil        numeric;
  v_idcam_uf_emissao_certidao_civil numeric;
  v_idcam_numero_carteira_trabalho  numeric;
  v_idcam_numero_serie_carteira_trabalho  numeric;
  v_idcam_data_emissao_carteira_trabalho  numeric;
  v_idcam_uf_emissao_carteira_trabalho  numeric;
  v_idcam_numero_titulo_eleitor   numeric;
  v_idcam_numero_zona_titulo_eleitor  numeric;
  v_idcam_numero_secao_titulo_eleitor numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Maxima, 2 = Alta, 3 = Media, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migracao, U = Usuario, C = Rotina de confrontacao, O = Oscar
  */
  BEGIN
    v_idpes         := NEW.idpes;

    v_data_expedicao_rg_nova    := NEW.data_exp_rg;
    v_data_emissao_cert_civil_nova    := NEW.data_emissao_cert_civil;
    v_data_emissao_cart_trabalho_nova := NEW.data_emissao_cart_trabalho;
    v_orgao_expedicao_rg_novo   := NEW.idorg_exp_rg;
    v_numero_rg_novo      := NEW.rg;
    v_numero_titulo_eleitor_novo    := NEW.num_tit_eleitor;
    v_numero_zona_titulo_novo   := NEW.zona_tit_eleitor;
    v_numero_secao_titulo_novo    := NEW.secao_tit_eleitor;
    v_numero_cart_trabalho_novo   := NEW.num_cart_trabalho;
    v_numero_serie_cart_trabalho_novo := NEW.serie_cart_trabalho;
    v_tipo_certidao_civil_novo    := NEW.tipo_cert_civil;
    v_numero_termo_certidao_civil_novo  := NEW.num_termo;
    v_numero_livro_certidao_civil_novo  := NEW.num_livro;
    v_numero_folha_certidao_civil_novo  := NEW.num_folha;
    v_cartorio_certidao_civil_novo    := NEW.cartorio_cert_civil;
    v_uf_expedicao_rg_novo      := NEW.sigla_uf_exp_rg;
    v_uf_emissao_certidao_civil_novo  := NEW.sigla_uf_cert_civil;
    v_uf_emissao_carteira_trabalho_novo := NEW.sigla_uf_cart_trabalho;

    IF TG_OP <> 'UPDATE' THEN
      v_data_expedicao_rg_antiga    := NULL;
      v_data_emissao_cert_civil_antiga  := NULL;
      v_data_emissao_cart_trabalho_antiga := NULL;
      v_orgao_expedicao_rg_antigo   := 0;
      v_numero_rg_antigo      := 0;
      v_numero_titulo_eleitor_antigo    := 0;
      v_numero_zona_titulo_antigo   := 0;
      v_numero_secao_titulo_antigo    := 0;
      v_numero_cart_trabalho_antigo   := 0;
      v_numero_serie_cart_trabalho_antigo := 0;
      v_tipo_certidao_civil_antigo    := 0;
      v_numero_termo_certidao_civil_antigo  := 0;
      v_numero_livro_certidao_civil_antigo  := '';
      v_numero_folha_certidao_civil_antigo  := 0;
      v_cartorio_certidao_civil_antigo  := '';
      v_uf_expedicao_rg_antigo    := '';
      v_uf_emissao_certidao_civil_antigo  := '';
      v_uf_emissao_carteira_trabalho_antigo := '';
    ELSE
      v_data_expedicao_rg_antiga    := OLD.data_exp_rg;
      v_data_emissao_cert_civil_antiga  := OLD.data_emissao_cert_civil;
      v_data_emissao_cart_trabalho_antiga := OLD.data_emissao_cart_trabalho;
      v_orgao_expedicao_rg_antigo   := COALESCE(OLD.idorg_exp_rg, 0);
      v_numero_rg_antigo      := COALESCE(OLD.rg, 0);
      v_numero_titulo_eleitor_antigo    := COALESCE(OLD.num_tit_eleitor, 0);
      v_numero_zona_titulo_antigo   := COALESCE(OLD.zona_tit_eleitor, 0);
      v_numero_secao_titulo_antigo    := COALESCE(OLD.secao_tit_eleitor, 0);
      v_numero_cart_trabalho_antigo   := COALESCE(OLD.num_cart_trabalho, 0);
      v_numero_serie_cart_trabalho_antigo := COALESCE(OLD.serie_cart_trabalho, 0);
      v_tipo_certidao_civil_antigo    := COALESCE(OLD.tipo_cert_civil, 0);
      v_numero_termo_certidao_civil_antigo  := COALESCE(OLD.num_termo, 0);
      v_numero_livro_certidao_civil_antigo  := OLD.num_livro;
      v_numero_folha_certidao_civil_antigo  := COALESCE(OLD.num_folha, 0);
      v_cartorio_certidao_civil_antigo  := COALESCE(OLD.cartorio_cert_civil, '');
      v_uf_expedicao_rg_antigo    := COALESCE(OLD.sigla_uf_exp_rg, '');
      v_uf_emissao_certidao_civil_antigo  := COALESCE(OLD.sigla_uf_cert_civil, '');
      v_uf_emissao_carteira_trabalho_antigo := COALESCE(OLD.sigla_uf_cart_trabalho, '');
    END IF;

    v_idcam_numero_rg     := 6;
    v_idcam_orgao_expedidor_rg    := 7;
    v_idcam_data_expedicao_rg   := 8;
    v_idcam_uf_expedicao_rg     := 9;
    v_idcam_tipo_certidao_civil   := 10;
    v_idcam_numero_termo_certidao_civil := 11;
    v_idcam_numero_livro_certidao_civil := 12;
    v_idcam_numero_folha_certidao_civil := 13;
    v_idcam_data_emissao_certidao_civil := 14;
    v_idcam_cartorio_certidao_civil   := 15;
    v_idcam_uf_emissao_certidao_civil := 16;
    v_idcam_numero_carteira_trabalho  := 17;
    v_idcam_numero_serie_carteira_trabalho  := 18;
    v_idcam_data_emissao_carteira_trabalho  := 19;
    v_idcam_uf_emissao_carteira_trabalho  := 20;
    v_idcam_numero_titulo_eleitor   := 21;
    v_idcam_numero_zona_titulo_eleitor  := 22;
    v_idcam_numero_secao_titulo_eleitor := 23;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;

    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usuario ou pelo usuario do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migracao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;

    IF v_nova_credibilidade > 0 THEN

      -- DATA DE EXPEDICAO DO RG
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_expedicao_rg_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_expedicao_rg_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_expedicao_rg||','||v_nova_credibilidade||');';
      END IF;

      -- DATA DE EMISSAO CERTIDAO CIVIL
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_emissao_cert_civil_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_emissao_cert_civil_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_emissao_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- DATA DE EMISSAO CARTEIRA DE TRABALHO
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_emissao_cart_trabalho_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_emissao_cart_trabalho_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_emissao_carteira_trabalho||','||v_nova_credibilidade||');';
      END IF;

      -- ORGAO EXPEDIDOR DO RG
      IF v_orgao_expedicao_rg_novo <> v_orgao_expedicao_rg_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_orgao_expedidor_rg||','||v_nova_credibilidade||');';
      END IF;

      -- RG
      IF v_numero_rg_novo <> v_numero_rg_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_rg||','||v_nova_credibilidade||');';
      END IF;

      -- TITULO ELEITOR
      IF v_numero_titulo_eleitor_novo <> v_numero_titulo_eleitor_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_titulo_eleitor||','||v_nova_credibilidade||');';
      END IF;

      -- ZONA TITULO ELEITOR
      IF v_numero_zona_titulo_novo <> v_numero_zona_titulo_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_zona_titulo_eleitor||','||v_nova_credibilidade||');';
      END IF;

      -- SECAO TITULO ELEITOR
      IF v_numero_secao_titulo_novo <> v_numero_secao_titulo_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_secao_titulo_eleitor||','||v_nova_credibilidade||');';
      END IF;

      -- CARTEIRA DE TRABALHO
      IF v_numero_cart_trabalho_novo <> v_numero_cart_trabalho_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_carteira_trabalho||','||v_nova_credibilidade||');';
      END IF;

      -- SERIE CARTEIRA DE TRABALHO
      IF v_numero_serie_cart_trabalho_novo <> v_numero_serie_cart_trabalho_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_serie_carteira_trabalho||','||v_nova_credibilidade||');';
      END IF;

      -- TIPO CERTIDAO CIVIL
      IF v_tipo_certidao_civil_novo <> v_tipo_certidao_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_tipo_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- NUMERO TERMO CERTIDAO CIVIL
      IF v_numero_termo_certidao_civil_novo <> v_numero_termo_certidao_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_termo_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- NUMERO LIVRO CERTIDAO CIVIL
      IF v_numero_livro_certidao_civil_novo <> v_numero_livro_certidao_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_livro_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- NUMERO FOLHA CERTIDAO CIVIL
      IF v_numero_folha_certidao_civil_novo <> v_numero_folha_certidao_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_folha_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- CARTORIO CERTIDAO CIVIL
      IF v_cartorio_certidao_civil_novo <> v_cartorio_certidao_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cartorio_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- UF EXPEDICAO RG
      IF v_uf_expedicao_rg_novo <> v_uf_expedicao_rg_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_uf_expedicao_rg||','||v_nova_credibilidade||');';
      END IF;

      -- UF EMISSAO CERTIDAO CIVIL
      IF v_uf_emissao_certidao_civil_novo <> v_uf_emissao_certidao_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_uf_emissao_certidao_civil||','||v_nova_credibilidade||');';
      END IF;

      -- UF EMISSAO CARTEIRA DE TRABALHO
      IF v_uf_emissao_carteira_trabalho_novo <> v_uf_emissao_carteira_trabalho_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_uf_emissao_carteira_trabalho||','||v_nova_credibilidade||');';
      END IF;

    END IF;
    -- Verificar os campos Vazios ou Nulos
    -- DATA DE EXPEDICAO DO RG
    IF TRIM(v_data_expedicao_rg_nova)='' OR v_data_expedicao_rg_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_expedicao_rg||','||v_sem_credibilidade||');';
    END IF;
    -- DATA DE EMISSAO CERTIDAO CIVIL
    IF TRIM(v_data_emissao_cert_civil_nova)='' OR v_data_emissao_cert_civil_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_emissao_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- DATA DE EMISSAO CARTEIRA DE TRABALHO
    IF TRIM(v_data_emissao_cart_trabalho_nova)='' OR v_data_emissao_cart_trabalho_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_emissao_carteira_trabalho||','||v_sem_credibilidade||');';
    END IF;
    -- ORGAO EXPEDIDOR DO RG
    IF v_orgao_expedicao_rg_novo <= 0 OR v_orgao_expedicao_rg_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_orgao_expedidor_rg||','||v_sem_credibilidade||');';
    END IF;
    -- RG
    IF v_numero_rg_novo <= 0 OR v_numero_rg_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_rg||','||v_sem_credibilidade||');';
    END IF;
    -- TITULO ELEITOR
    IF v_numero_titulo_eleitor_novo <= 0 OR v_numero_titulo_eleitor_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_titulo_eleitor||','||v_sem_credibilidade||');';
    END IF;
    -- ZONA TITULO ELEITOR
    IF v_numero_zona_titulo_novo <= 0 OR v_numero_zona_titulo_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_zona_titulo_eleitor||','||v_sem_credibilidade||');';
    END IF;
    -- SECAO TITULO ELEITOR
    IF v_numero_secao_titulo_novo <= 0 OR v_numero_secao_titulo_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_secao_titulo_eleitor||','||v_sem_credibilidade||');';
    END IF;
    -- CARTEIRA DE TRABALHO
    IF v_numero_cart_trabalho_novo <= 0 OR v_numero_cart_trabalho_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_carteira_trabalho||','||v_sem_credibilidade||');';
    END IF;
    -- SERIE CARTEIRA DE TRABALHO
    IF v_numero_serie_cart_trabalho_novo <= 0 OR v_numero_serie_cart_trabalho_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_serie_carteira_trabalho||','||v_sem_credibilidade||');';
    END IF;
    -- TIPO CERTIDAO CIVIL
    IF v_tipo_certidao_civil_novo <= 0 OR v_tipo_certidao_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_tipo_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- NUMERO TERMO CERTIDAO CIVIL
    IF v_numero_termo_certidao_civil_novo <= 0 OR v_numero_termo_certidao_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_termo_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- NUMERO LIVRO CERTIDAO CIVIL
    IF v_numero_livro_certidao_civil_novo = '' OR v_numero_livro_certidao_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_livro_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- NUMERO FOLHA CERTIDAO CIVIL
    IF v_numero_folha_certidao_civil_novo <= 0 OR v_numero_folha_certidao_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero_folha_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- CARTORIO CERTIDAO CIVIL
    IF TRIM(v_cartorio_certidao_civil_novo)='' OR v_cartorio_certidao_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cartorio_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- UF EXPEDICAO RG
    IF TRIM(v_uf_expedicao_rg_novo)='' OR v_uf_expedicao_rg_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_uf_expedicao_rg||','||v_sem_credibilidade||');';
    END IF;
    -- UF EMISSAO CERTIDAO CIVIL
    IF TRIM(v_uf_emissao_certidao_civil_novo)='' OR v_uf_emissao_certidao_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_uf_emissao_certidao_civil||','||v_sem_credibilidade||');';
    END IF;
    -- UF EMISSAO CARTEIRA DE TRABALHO
    IF TRIM(v_uf_emissao_carteira_trabalho_novo)='' OR v_uf_emissao_carteira_trabalho_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_uf_emissao_carteira_trabalho||','||v_sem_credibilidade||');';
    END IF;

  RETURN NEW;
END;$$
    LANGUAGE plpgsql;


--
-- Name: fcn_endereco_externo_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_endereco_externo_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;

  v_sigla_uf_antiga   text;
  v_sigla_uf_nova     text;
  v_id_tipo_logradouro_antigo text;
  v_id_tipo_logradouro_novo text;
  v_logradouro_antigo   text;
  v_logradouro_novo   text;
  v_numero_antigo     numeric;
  v_numero_novo     numeric;
  v_letra_antiga      text;
  v_letra_nova      text;
  v_complemento_antigo    text;
  v_complemento_novo    text;
  v_bairro_antigo     text;
  v_bairro_novo     text;
  v_cep_antigo      numeric;
  v_cep_novo      numeric;
  v_cidade_antiga     text;
  v_cidade_nova     text;

  v_tipo_endereco     numeric;

  v_comando     text;
  v_origem_gravacao   text;

  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;

  v_nova_credibilidade    numeric;

  v_registro      record;

  -- ID dos campos
  v_idcam_sigla_uf_correspondencia    numeric;
  v_idcam_id_tipo_logradouro_correspondencia  numeric;
  v_idcam_logradouro_correspondencia    numeric;
  v_idcam_numero_correspondencia      numeric;
  v_idcam_letra_correspondencia     numeric;
  v_idcam_complemento_correspondencia   numeric;
  v_idcam_bairro_correspondencia      numeric;
  v_idcam_cep_correspondencia     numeric;
  v_idcam_cidade_correspondencia      numeric;

  v_idcam_sigla_uf_residencial      numeric;
  v_idcam_id_tipo_logradouro_residencial    numeric;
  v_idcam_logradouro_residencial      numeric;
  v_idcam_numero_residencial      numeric;
  v_idcam_letra_residencial     numeric;
  v_idcam_complemento_residencial     numeric;
  v_idcam_bairro_residencial      numeric;
  v_idcam_cep_residencial       numeric;
  v_idcam_cidade_residencial      numeric;

  v_idcam_sigla_uf_comercial      numeric;
  v_idcam_id_tipo_logradouro_comercial    numeric;
  v_idcam_logradouro_comercial      numeric;
  v_idcam_numero_comercial      numeric;
  v_idcam_letra_comercial       numeric;
  v_idcam_complemento_comercial     numeric;
  v_idcam_bairro_comercial      numeric;
  v_idcam_cep_comercial       numeric;
  v_idcam_cidade_comercial      numeric;

  v_idcam_sigla_uf    numeric;
  v_idcam_id_tipo_logradouro  numeric;
  v_idcam_logradouro    numeric;
  v_idcam_numero      numeric;
  v_idcam_letra     numeric;
  v_idcam_complemento   numeric;
  v_idcam_bairro      numeric;
  v_idcam_cep     numeric;
  v_idcam_cidade      numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Mxima, 2 = Alta, 3 = Mdia, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migrao, U = Usurio, C = Rotina de confrontao, O = Oscar
  */
  BEGIN
    v_idpes       := NEW.idpes;
    v_tipo_endereco     := NEW.tipo;

    v_sigla_uf_nova     := NEW.sigla_uf;
    v_id_tipo_logradouro_novo := NEW.idtlog;
    v_logradouro_novo   := NEW.logradouro;
    v_numero_novo     := NEW.numero;
    v_letra_nova      := NEW.letra;
    v_complemento_novo    := NEW.complemento;
    v_bairro_novo     := NEW.bairro;
    v_cep_novo      := NEW.cep;
    v_cidade_nova     := NEW.cidade;

    IF TG_OP <> 'UPDATE' THEN
      v_sigla_uf_antiga   := '';
      v_id_tipo_logradouro_antigo := '';
      v_logradouro_antigo   := '';
      v_numero_antigo     := 0;
      v_letra_antiga      := '';
      v_complemento_antigo    := '';
      v_bairro_antigo     := '';
      v_cep_antigo      := 0;
      v_cidade_antiga     := '';
    ELSE
      v_sigla_uf_antiga   := COALESCE(OLD.sigla_uf, '');
      v_id_tipo_logradouro_antigo := COALESCE(OLD.idtlog, '');
      v_logradouro_antigo   := COALESCE(OLD.logradouro, '');
      v_numero_antigo     := COALESCE(OLD.numero, 0);
      v_letra_antiga      := COALESCE(OLD.letra, '');
      v_complemento_antigo    := COALESCE(OLD.complemento, '');
      v_bairro_antigo     := COALESCE(OLD.bairro, '');
      v_cep_antigo      := COALESCE(OLD.cep, 0);
      v_cidade_antiga     := COALESCE(OLD.cidade, '');
    END IF;

    v_idcam_sigla_uf_correspondencia    := 55;
    v_idcam_id_tipo_logradouro_correspondencia  := 48;
    v_idcam_logradouro_correspondencia    := 47;
    v_idcam_numero_correspondencia      := 49;
    v_idcam_letra_correspondencia     := 50;
    v_idcam_complemento_correspondencia   := 51;
    v_idcam_bairro_correspondencia      := 52;
    v_idcam_cep_correspondencia     := 53;
    v_idcam_cidade_correspondencia      := 54;
    v_idcam_sigla_uf_residencial      := 64;
    v_idcam_id_tipo_logradouro_residencial    := 57;
    v_idcam_logradouro_residencial      := 56;
    v_idcam_numero_residencial      := 58;
    v_idcam_letra_residencial     := 59;
    v_idcam_complemento_residencial     := 60;
    v_idcam_bairro_residencial      := 61;
    v_idcam_cep_residencial       := 62;
    v_idcam_cidade_residencial      := 63;
    v_idcam_sigla_uf_comercial      := 73;
    v_idcam_id_tipo_logradouro_comercial    := 66;
    v_idcam_logradouro_comercial      := 65;
    v_idcam_numero_comercial      := 67;
    v_idcam_letra_comercial       := 68;
    v_idcam_complemento_comercial     := 69;
    v_idcam_bairro_comercial      := 70;
    v_idcam_cep_comercial       := 71;
    v_idcam_cidade_comercial      := 72;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;

    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usurio ou usurio do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migrao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;

    IF v_tipo_endereco = 1 THEN
      v_idcam_sigla_uf    := v_idcam_sigla_uf_correspondencia;
      v_idcam_id_tipo_logradouro  := v_idcam_id_tipo_logradouro_correspondencia;
      v_idcam_logradouro    := v_idcam_logradouro_correspondencia;
      v_idcam_numero      := v_idcam_numero_correspondencia;
      v_idcam_letra     := v_idcam_letra_correspondencia;
      v_idcam_complemento   := v_idcam_complemento_correspondencia;
      v_idcam_bairro      := v_idcam_bairro_correspondencia;
      v_idcam_cep     := v_idcam_cep_correspondencia;
      v_idcam_cidade      := v_idcam_cidade_correspondencia;
    ELSIF v_tipo_endereco = 2 THEN
      v_idcam_sigla_uf    := v_idcam_sigla_uf_residencial;
      v_idcam_id_tipo_logradouro  := v_idcam_id_tipo_logradouro_residencial;
      v_idcam_logradouro    := v_idcam_logradouro_residencial;
      v_idcam_numero      := v_idcam_numero_residencial;
      v_idcam_letra     := v_idcam_letra_residencial;
      v_idcam_complemento   := v_idcam_complemento_residencial;
      v_idcam_bairro      := v_idcam_bairro_residencial;
      v_idcam_cep     := v_idcam_cep_residencial;
      v_idcam_cidade      := v_idcam_cidade_residencial;
    ELSIF v_tipo_endereco = 3 THEN
      v_idcam_sigla_uf    := v_idcam_sigla_uf_comercial;
      v_idcam_id_tipo_logradouro  := v_idcam_id_tipo_logradouro_comercial;
      v_idcam_logradouro    := v_idcam_logradouro_comercial;
      v_idcam_numero      := v_idcam_numero_comercial;
      v_idcam_letra     := v_idcam_letra_comercial;
      v_idcam_complemento   := v_idcam_complemento_comercial;
      v_idcam_bairro      := v_idcam_bairro_comercial;
      v_idcam_cep     := v_idcam_cep_comercial;
      v_idcam_cidade      := v_idcam_cidade_comercial;
    END IF;

    IF v_nova_credibilidade > 0 THEN
      -- UF
      IF v_sigla_uf_nova <> v_sigla_uf_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_sigla_uf||','||v_nova_credibilidade||');';
      END IF;

      -- ID TIPO LOGRADOURO
      IF v_id_tipo_logradouro_novo <> v_id_tipo_logradouro_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_id_tipo_logradouro||','||v_nova_credibilidade||');';
      END IF;

      -- LOGRADOURO
      IF v_logradouro_novo <> v_logradouro_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_logradouro||','||v_nova_credibilidade||');';
      END IF;

      -- NUMERO
      IF v_numero_novo <> v_numero_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero||','||v_nova_credibilidade||');';
      END IF;

      -- LETRA
      IF v_letra_nova <> v_letra_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_letra||','||v_nova_credibilidade||');';
      END IF;

      -- COMPLEMENTO
      IF v_complemento_novo <> v_complemento_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_complemento||','||v_nova_credibilidade||');';
      END IF;

      -- BAIRRO
      IF v_bairro_novo <> v_bairro_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_bairro||','||v_nova_credibilidade||');';
      END IF;

      -- CEP
      IF v_cep_novo <> v_cep_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cep||','||v_nova_credibilidade||');';
      END IF;

      -- CIDADE
      IF v_cidade_nova <> v_cidade_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cidade||','||v_nova_credibilidade||');';
      END IF;

    END IF;

    -- Verificar os campos Vazios ou Nulos

    -- UF
    IF TRIM(v_sigla_uf_nova)='' OR v_sigla_uf_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_sigla_uf||','||v_sem_credibilidade||');';
    END IF;

    -- ID TIPO LOGRADOURO
    IF TRIM(v_id_tipo_logradouro_novo)='' OR v_id_tipo_logradouro_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_id_tipo_logradouro||','||v_sem_credibilidade||');';
    END IF;

    -- LOGRADOURO
    IF TRIM(v_logradouro_novo)='' OR v_logradouro_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_logradouro||','||v_sem_credibilidade||');';
    END IF;
    -- NUMERO
    IF v_numero_novo <= 0 OR v_numero_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero||','||v_sem_credibilidade||');';
    END IF;

    -- LETRA
    IF TRIM(v_letra_nova)='' OR v_letra_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_letra||','||v_sem_credibilidade||');';
    END IF;

    -- COMPLEMENTO
    IF TRIM(v_complemento_novo)='' OR v_complemento_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_complemento||','||v_sem_credibilidade||');';
    END IF;

    -- BAIRRO
    IF TRIM(v_bairro_novo)='' OR v_bairro_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_bairro||','||v_sem_credibilidade||');';
    END IF;
    -- CEP
    IF v_cep_novo <= 0 OR v_cep_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cep||','||v_sem_credibilidade||');';
    END IF;

    -- CIDADE
    IF TRIM(v_cidade_nova)='' OR v_cidade_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cidade||','||v_sem_credibilidade||');';
    END IF;

  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_endereco_pessoa_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_endereco_pessoa_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_cep_antigo      numeric;
  v_cep_novo      numeric;
  v_id_logradouro_antigo    numeric;
  v_id_logradouro_novo    numeric;
  v_id_bairro_antigo    numeric;
  v_id_bairro_novo    numeric;
  v_numero_antigo     numeric;
  v_numero_novo     numeric;
  v_letra_antiga      text;
  v_letra_nova      text;
  v_complemento_antigo    text;
  v_complemento_novo    text;
  v_tipo_endereco     numeric;
  v_comando     text;
  v_origem_gravacao   text;
  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;
  v_nova_credibilidade    numeric;

  v_registro      record;

  -- ID dos campos
  v_idcam_cep_correspondencia     numeric;
  v_idcam_id_logradouro_correspondencia   numeric;
  v_idcam_id_bairro_correspondencia   numeric;
  v_idcam_numero_correspondencia      numeric;
  v_idcam_letra_correspondencia     numeric;
  v_idcam_complemento_correspondencia   numeric;

  v_idcam_cep_residencial       numeric;
  v_idcam_id_logradouro_residencial   numeric;
  v_idcam_id_bairro_residencial     numeric;
  v_idcam_numero_residencial      numeric;
  v_idcam_letra_residencial     numeric;
  v_idcam_complemento_residencial     numeric;

  v_idcam_cep_comercial       numeric;
  v_idcam_id_logradouro_comercial     numeric;
  v_idcam_id_bairro_comercial     numeric;
  v_idcam_numero_comercial      numeric;
  v_idcam_letra_comercial       numeric;
  v_idcam_complemento_comercial     numeric;

  v_idcam_cep     numeric;
  v_idcam_id_logradouro   numeric;
  v_idcam_id_bairro   numeric;
  v_idcam_numero      numeric;
  v_idcam_letra     numeric;
  v_idcam_complemento   numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Mxima, 2 = Alta, 3 = Mdia, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migrao, U = Usurio, C = Rotina de confrontao, O = Oscar
  */
  BEGIN
    v_idpes       := NEW.idpes;
    v_tipo_endereco     := NEW.tipo;

    v_cep_novo      := NEW.cep;
    v_id_logradouro_novo    := NEW.idlog;
    v_id_bairro_novo    := NEW.idbai;
    v_numero_novo     := NEW.numero;
    v_letra_nova      := NEW.letra;
    v_complemento_novo    := NEW.complemento;

    IF TG_OP <> 'UPDATE' THEN
      v_cep_antigo    := 0;
      v_id_logradouro_antigo  := 0;
      v_id_bairro_antigo  := 0;
      v_numero_antigo   := 0;
      v_letra_antiga    := '';
      v_complemento_antigo  := '';
    ELSE
      v_cep_antigo    := COALESCE(OLD.cep, 0);
      v_id_logradouro_antigo  := COALESCE(OLD.idlog, 0);
      v_id_bairro_antigo  := COALESCE(OLD.idbai, 0);
      v_numero_antigo   := COALESCE(OLD.numero, 0);
      v_letra_antiga    := COALESCE(OLD.letra, '');
      v_complemento_antigo  := COALESCE(OLD.complemento, '');
    END IF;

    v_idcam_cep_correspondencia     := 53;
    v_idcam_id_logradouro_correspondencia   := 47;
    v_idcam_id_bairro_correspondencia   := 52;
    v_idcam_numero_correspondencia      := 49;
    v_idcam_letra_correspondencia     := 50;
    v_idcam_complemento_correspondencia   := 51;
    v_idcam_cep_residencial       := 62;
    v_idcam_id_logradouro_residencial   := 56;
    v_idcam_id_bairro_residencial     := 61;
    v_idcam_numero_residencial      := 58;
    v_idcam_letra_residencial     := 59;
    v_idcam_complemento_residencial     := 60;
    v_idcam_cep_comercial       := 71;
    v_idcam_id_logradouro_comercial     := 65;
    v_idcam_id_bairro_comercial     := 70;
    v_idcam_numero_comercial      := 67;
    v_idcam_letra_comercial       := 68;
    v_idcam_complemento_comercial     := 69;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;
    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usurio ou pelo usurio do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migrao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;
    IF v_tipo_endereco = 1 THEN
      v_idcam_cep     := v_idcam_cep_correspondencia;
      v_idcam_id_logradouro   := v_idcam_id_logradouro_correspondencia;
      v_idcam_id_bairro   := v_idcam_id_bairro_correspondencia;
      v_idcam_numero      := v_idcam_numero_correspondencia;
      v_idcam_letra     := v_idcam_letra_correspondencia;
      v_idcam_complemento   := v_idcam_complemento_correspondencia;
    ELSIF v_tipo_endereco = 2 THEN
      v_idcam_cep     := v_idcam_cep_residencial;
      v_idcam_id_logradouro   := v_idcam_id_logradouro_residencial;
      v_idcam_id_bairro   := v_idcam_id_bairro_residencial;
      v_idcam_numero      := v_idcam_numero_residencial;
      v_idcam_letra     := v_idcam_letra_residencial;
      v_idcam_complemento   := v_idcam_complemento_residencial;
    ELSIF v_tipo_endereco = 3 THEN
      v_idcam_cep     := v_idcam_cep_comercial;
      v_idcam_id_logradouro   := v_idcam_id_logradouro_comercial;
      v_idcam_id_bairro   := v_idcam_id_bairro_comercial;
      v_idcam_numero      := v_idcam_numero_comercial;
      v_idcam_letra     := v_idcam_letra_comercial;
      v_idcam_complemento   := v_idcam_complemento_comercial;
    END IF;
    IF v_nova_credibilidade > 0 THEN
      -- CEP
      IF v_cep_novo <> v_cep_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cep||','||v_nova_credibilidade||');';
      END IF;
      -- ID LOGRADOURO
      IF v_id_logradouro_novo <> v_id_logradouro_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_id_logradouro||','||v_nova_credibilidade||');';
      END IF;
      -- BAIRRO
      IF v_id_bairro_novo <> v_id_bairro_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_id_bairro||','||v_nova_credibilidade||');';
      END IF;

      -- NUMERO
      IF v_numero_novo <> v_numero_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero||','||v_nova_credibilidade||');';
      END IF;

      -- LETRA
      IF v_letra_nova <> v_letra_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_letra||','||v_nova_credibilidade||');';
      END IF;

      -- COMPLEMENTO
      IF v_complemento_novo <> v_complemento_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_complemento||','||v_nova_credibilidade||');';
      END IF;
    END IF;

    -- Verificar os campos Vazios ou Nulos

    -- CEP
    IF v_cep_novo <= 0 OR v_cep_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cep||','||v_sem_credibilidade||');';
    END IF;

    -- ID LOGRADOURO
    IF v_id_logradouro_novo <= 0 OR v_id_logradouro_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_id_logradouro||','||v_sem_credibilidade||');';
    END IF;

    -- BAIRRO
    IF v_id_bairro_novo <= 0 OR v_id_bairro_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_id_bairro||','||v_sem_credibilidade||');';
    END IF;

    -- NUMERO
    IF v_numero_novo <= 0 OR v_numero_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_numero||','||v_sem_credibilidade||');';
    END IF;

    -- LETRA
    IF TRIM(v_letra_nova)='' OR v_letra_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_letra||','||v_sem_credibilidade||');';
    END IF;

    -- COMPLEMENTO
    IF TRIM(v_complemento_novo)='' OR v_complemento_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_complemento||','||v_sem_credibilidade||');';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_fisica_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_fisica_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_data_nasc_nova    date;
  v_data_nasc_antiga    date;
  v_sexo_novo     text;
  v_sexo_antigo     text;
  v_nome_mae_novo     text;
  v_nome_mae_antigo   text;
  v_nome_pai_novo     text;
  v_nome_pai_antigo   text;
  v_nome_conjuge_novo   text;
  v_nome_conjuge_antigo   text;
  v_nome_responsavel_novo   text;
  v_nome_responsavel_antigo text;
  v_nome_ultima_empresa_novo  text;
  v_nome_ultima_empresa_antigo  text;
  v_id_ocupacao_novo    numeric;
  v_id_ocupacao_antigo    numeric;
  v_id_escolaridade_novo    numeric;
  v_id_escolaridade_antigo  numeric;
  v_id_estado_civil_novo    numeric;
  v_id_estado_civil_antigo  numeric;
  v_id_pais_origem_novo   numeric;
  v_id_pais_origem_antigo   numeric;
  v_data_chegada_brasil_nova  date;
  v_data_chegada_brasil_antiga  date;
  v_data_obito_nova   date;
  v_data_obito_antiga   date;
  v_data_uniao_nova   date;
  v_data_uniao_antiga   date;

  v_comando     text;
  v_origem_gravacao   text;

  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;

  v_nova_credibilidade    numeric;

  v_registro      record;
  v_aux_data_nova     text;
  v_aux_data_antiga   text;

  -- ID dos campos
  v_idcam_data_nasc   numeric;
  v_idcam_sexo      numeric;
  v_idcam_nome_mae    numeric;
  v_idcam_nome_pai    numeric;
  v_idcam_nome_conjuge    numeric;
  v_idcam_nome_responsavel  numeric;
  v_idcam_nome_ultima_empresa numeric;
  v_idcam_ocupacao    numeric;
  v_idcam_escolaridade    numeric;
  v_idcam_estado_civil    numeric;
  v_idcam_pais_origem   numeric;
  v_idcam_data_chegada_brasil numeric;
  v_idcam_data_obito    numeric;
  v_idcam_data_uniao    numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Mxima, 2 = Alta, 3 = Mdia, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migrao, U = Usurio, C = Rotina de confrontao, O = Oscar
  */
  BEGIN
    v_idpes       := NEW.idpes;
    v_data_nasc_nova    := NEW.data_nasc;
    v_sexo_novo     := NEW.sexo;
    v_nome_mae_novo     := NEW.nome_mae;
    v_nome_pai_novo     := NEW.nome_pai;
    v_nome_conjuge_novo   := NEW.nome_conjuge;
    v_nome_responsavel_novo   := NEW.nome_responsavel;
    v_nome_ultima_empresa_novo  := NEW.ultima_empresa;
    v_id_ocupacao_novo    := NEW.idocup;
    v_id_escolaridade_novo    := NEW.idesco;
    v_id_estado_civil_novo    := NEW.ideciv;
    v_id_pais_origem_novo   := NEW.idpais_estrangeiro;
    v_data_chegada_brasil_nova  := NEW.data_chegada_brasil;
    v_data_obito_nova   := NEW.data_obito;
    v_data_uniao_nova   := NEW.data_uniao;

    IF TG_OP <> 'UPDATE' THEN
      v_data_nasc_antiga    := NULL;
      v_sexo_antigo     := '';
      v_nome_mae_antigo   := '';
      v_nome_pai_antigo   := '';
      v_nome_conjuge_antigo   := '';
      v_nome_responsavel_antigo := '';
      v_nome_ultima_empresa_antigo  := '';
      v_id_ocupacao_antigo    := 0;
      v_id_escolaridade_antigo  := 0;
      v_id_estado_civil_antigo  := 0;
      v_id_pais_origem_antigo   := 0;
      v_data_chegada_brasil_antiga  := NULL;
      v_data_obito_antiga   := NULL;
      v_data_uniao_antiga   := NULL;
    ELSE
      v_data_nasc_antiga    := OLD.data_nasc;
      v_sexo_antigo     := COALESCE(OLD.sexo, '');
      v_nome_mae_antigo   := COALESCE(OLD.nome_mae, '');
      v_nome_pai_antigo   := COALESCE(OLD.nome_pai, '');
      v_nome_conjuge_antigo   := COALESCE(OLD.nome_conjuge, '');
      v_nome_responsavel_antigo := COALESCE(OLD.nome_responsavel, '');
      v_nome_ultima_empresa_antigo  := COALESCE(OLD.ultima_empresa, '');
      v_id_ocupacao_antigo    := COALESCE(OLD.idocup, 0);
      v_id_escolaridade_antigo  := COALESCE(OLD.idesco, 0);
      v_id_estado_civil_antigo  := COALESCE(OLD.ideciv, 0);
      v_id_pais_origem_antigo   := COALESCE(OLD.idpais_estrangeiro, 0);
      v_data_chegada_brasil_antiga  := OLD.data_chegada_brasil;
      v_data_obito_antiga   := OLD.data_obito;
      v_data_uniao_antiga   := OLD.data_uniao;
    END IF;

    v_idcam_data_nasc   := 5;
    v_idcam_sexo      := 26;
    v_idcam_nome_mae    := 27;
    v_idcam_nome_pai    := 28;
    v_idcam_nome_conjuge    := 29;
    v_idcam_nome_responsavel  := 30;
    v_idcam_nome_ultima_empresa := 31;
    v_idcam_ocupacao    := 32;
    v_idcam_escolaridade    := 33;
    v_idcam_estado_civil    := 34;
    v_idcam_pais_origem   := 35;
    v_idcam_data_chegada_brasil := 36;
    v_idcam_data_obito    := 37;
    v_idcam_data_uniao    := 38;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;

    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usurio ou pelo usurio do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migrao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;

    IF v_nova_credibilidade > 0 THEN

      -- DATA DE NASCIMENTO
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_nasc_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_nasc_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_nasc||','||v_nova_credibilidade||');';
      END IF;

      -- DATA DE UNIO
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_uniao_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_uniao_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_uniao||','||v_nova_credibilidade||');';
      END IF;

      -- DATA DE BITO
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_obito_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_obito_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_obito||','||v_nova_credibilidade||');';
      END IF;

      -- DATA DE CHEGADA AO BRASIL
      v_aux_data_nova := COALESCE(TO_CHAR (v_data_chegada_brasil_nova, 'DD/MM/YYYY'), '');
      v_aux_data_antiga := COALESCE(TO_CHAR (v_data_chegada_brasil_antiga, 'DD/MM/YYYY'), '');

      IF v_aux_data_nova <> v_aux_data_antiga THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_chegada_brasil||','||v_nova_credibilidade||');';
      END IF;

      -- NOME DA ME
      IF v_nome_mae_novo <> v_nome_mae_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_mae||','||v_nova_credibilidade||');';
      END IF;

      -- NOME DO PAI
      IF v_nome_pai_novo <> v_nome_pai_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_pai||','||v_nova_credibilidade||');';
      END IF;

      -- NOME DO CONJUGE
      IF v_nome_conjuge_novo <> v_nome_conjuge_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_conjuge||','||v_nova_credibilidade||');';
      END IF;

      -- NOME DO RESPONSAVEL
      IF v_nome_responsavel_novo <> v_nome_responsavel_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_responsavel||','||v_nova_credibilidade||');';
      END IF;

      -- NOME LTIMA EMPRESA
      IF v_nome_ultima_empresa_novo <> v_nome_ultima_empresa_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_ultima_empresa||','||v_nova_credibilidade||');';
      END IF;

      -- SEXO
      IF v_sexo_novo <> v_sexo_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_sexo||','||v_nova_credibilidade||');';
      END IF;

      -- ID OCUPAO PROFISSIONAL
      IF v_id_ocupacao_novo <> v_id_ocupacao_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_ocupacao||','||v_nova_credibilidade||');';
      END IF;

      -- ID ESCOLARIDADE
      IF v_id_escolaridade_novo <> v_id_escolaridade_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_escolaridade||','||v_nova_credibilidade||');';
      END IF;

      -- ID ESTADO CIVIL
      IF v_id_estado_civil_novo <> v_id_estado_civil_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_estado_civil||','||v_nova_credibilidade||');';
      END IF;

      -- ID PAIS ORIGEM
      IF v_id_pais_origem_novo <> v_id_pais_origem_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_pais_origem||','||v_nova_credibilidade||');';
      END IF;

    END IF;

    -- Verificar os campos Vazios ou Nulos
    -- DATA DE NASCIMENTO
    IF TRIM(v_data_nasc_nova)='' OR v_data_nasc_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_nasc||','||v_sem_credibilidade||');';
    END IF;
    -- DATA DE UNIO
    IF TRIM(v_data_uniao_nova)='' OR v_data_uniao_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_uniao||','||v_sem_credibilidade||');';
    END IF;
    -- DATA DE BITO
    IF TRIM(v_data_obito_nova)='' OR v_data_obito_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_obito||','||v_sem_credibilidade||');';
    END IF;
    -- DATA DE CHEGADA AO BRASIL
    IF TRIM(v_data_chegada_brasil_nova)='' OR v_data_chegada_brasil_nova IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_data_chegada_brasil||','||v_sem_credibilidade||');';
    END IF;
    -- NOME DA ME
    IF TRIM(v_nome_mae_novo)='' OR v_nome_mae_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_mae||','||v_sem_credibilidade||');';
    END IF;
    -- NOME DO PAI
    IF TRIM(v_nome_pai_novo)='' OR v_nome_pai_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_pai||','||v_sem_credibilidade||');';
    END IF;
    -- NOME DO CONJUGE
    IF TRIM(v_nome_conjuge_novo)='' OR v_nome_conjuge_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_conjuge||','||v_sem_credibilidade||');';
    END IF;
    -- NOME DO RESPONSAVEL
    IF TRIM(v_nome_responsavel_novo)='' OR v_nome_responsavel_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_responsavel||','||v_sem_credibilidade||');';
    END IF;
    -- NOME LTIMA EMPRESA
    IF TRIM(v_nome_ultima_empresa_novo)='' OR v_nome_ultima_empresa_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome_ultima_empresa||','||v_sem_credibilidade||');';
    END IF;
    -- SEXO
    IF TRIM(v_sexo_novo)='' OR v_sexo_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_sexo||','||v_sem_credibilidade||');';
    END IF;
    -- ID OCUPAO PROFISSIONAL
    IF v_id_ocupacao_novo <= 0 OR v_id_ocupacao_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_ocupacao||','||v_sem_credibilidade||');';
    END IF;
    -- ID ESCOLARIDADE
    IF v_id_escolaridade_novo <= 0 OR v_id_escolaridade_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_escolaridade||','||v_sem_credibilidade||');';
    END IF;
    -- ID ESTADO CIVIL
    IF v_id_estado_civil_novo <= 0 OR v_id_estado_civil_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_estado_civil||','||v_sem_credibilidade||');';
    END IF;
    -- ID PAIS ORIGEM
    IF v_id_pais_origem_novo <= 0 OR v_id_pais_origem_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_pais_origem||','||v_sem_credibilidade||');';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_fone_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_fone_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;

  v_ddd_antigo      numeric;
  v_ddd_novo      numeric;
  v_fone_antigo     numeric;
  v_fone_novo     numeric;
  v_tipo_fone     numeric;

  v_comando     text;
  v_origem_gravacao   text;

  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;

  v_nova_credibilidade    numeric;

  v_registro      record;

  -- ID dos campos
  v_idcam_ddd_fone_residencial  numeric;
  v_idcam_fone_residencial  numeric;
  v_idcam_ddd_fone_comercial  numeric;
  v_idcam_fone_comercial    numeric;
  v_idcam_ddd_fone_celular  numeric;
  v_idcam_fone_celular    numeric;
  v_idcam_ddd_fax     numeric;
  v_idcam_fax     numeric;

  v_idcam_ddd     numeric;
  v_idcam_fone      numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Mxima, 2 = Alta, 3 = Mdia, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migrao, U = Usurio, C = Rotina de confrontao, O = Oscar
  */
  BEGIN
    v_idpes   := NEW.idpes;
    v_tipo_fone := NEW.tipo;
    v_ddd_novo  := NEW.ddd;
    v_fone_novo := NEW.fone;

    IF TG_OP <> 'UPDATE' THEN
      v_ddd_antigo  := 0;
      v_fone_antigo := 0;
    ELSE
      v_ddd_antigo  := COALESCE(OLD.ddd, 0);
      v_fone_antigo := COALESCE(OLD.fone, 0);
    END IF;

    v_idcam_ddd_fone_residencial  := 39;
    v_idcam_fone_residencial  := 40;
    v_idcam_ddd_fone_comercial  := 41;
    v_idcam_fone_comercial    := 42;
    v_idcam_ddd_fone_celular  := 43;
    v_idcam_fone_celular    := 44;
    v_idcam_ddd_fax     := 45;
    v_idcam_fax     := 46;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;

    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usurio ou usurio do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migrao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;

    IF v_tipo_fone = 1 THEN
      v_idcam_ddd := v_idcam_ddd_fone_residencial;
      v_idcam_fone := v_idcam_fone_residencial;
    ELSIF v_tipo_fone = 2 THEN
      v_idcam_ddd := v_idcam_ddd_fone_comercial;
      v_idcam_fone := v_idcam_fone_comercial;
    ELSIF v_tipo_fone = 3 THEN
      v_idcam_ddd := v_idcam_ddd_fone_celular;
      v_idcam_fone := v_idcam_fone_celular;
    ELSIF v_tipo_fone = 4 THEN
      v_idcam_ddd := v_idcam_ddd_fax;
      v_idcam_fone := v_idcam_fax;
    END IF;

    IF v_nova_credibilidade > 0 THEN
      IF v_ddd_novo <> v_ddd_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_ddd||','||v_nova_credibilidade||');';
      END IF;

      IF v_fone_novo <> v_fone_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_fone||','||v_nova_credibilidade||');';
      END IF;
    END IF;

    -- Verificar os campos Vazios ou Nulos
    IF v_ddd_novo <= 0 OR v_ddd_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_ddd||','||v_sem_credibilidade||');';
    END IF;
    IF v_fone_novo <= 0 OR v_fone_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_fone||','||v_sem_credibilidade||');';
    END IF;

  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_gravar_historico_campo(numeric, numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_gravar_historico_campo(numeric, numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  v_idpes ALIAS for $1;
  v_idcam ALIAS for $2;
  v_credibilidade ALIAS for $3;

  v_comando   text;
  v_existe_historico  numeric;
  v_registro    record;

  BEGIN
    -- verificar se j existe histrico para o campo
    v_comando := 'SELECT idcam FROM consistenciacao.historico_campo WHERE idpes='||quote_literal(v_idpes)||' AND idcam='||quote_literal(v_idcam)||';';
    FOR v_registro IN EXECUTE v_comando LOOP
      v_existe_historico := v_registro.idcam;
    END LOOP;
    IF v_existe_historico > 0 THEN
      EXECUTE 'UPDATE consistenciacao.historico_campo SET credibilidade='||v_credibilidade||', data_hora=CURRENT_TIMESTAMP WHERE idpes='||quote_literal(v_idpes)||' AND idcam='||quote_literal(v_idcam)||';';
    ELSE
      EXECUTE 'INSERT INTO consistenciacao.historico_campo(idpes, idcam, credibilidade, data_hora) VALUES ('||quote_literal(v_idpes)||','|| quote_literal(v_idcam)||', '||quote_literal(v_credibilidade)||', CURRENT_TIMESTAMP);';
    END IF;
  RETURN 1;
END; $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_juridica_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_juridica_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_cnpj_novo     numeric;
  v_cnpj_antigo     numeric;
  v_fantasia_novo     text;
  v_fantasia_antigo   text;
  v_inscricao_estadual_novo numeric;
  v_inscricao_estadual_antigo numeric;

  v_comando     text;
  v_origem_gravacao   text;

  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;

  v_nova_credibilidade    numeric;

  v_registro      record;

  -- ID dos campos
  v_idcam_cnpj      numeric;
  v_idcam_fantasia    numeric;
  v_idcam_inscricao_estadual  numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Mxima, 2 = Alta, 3 = Mdia, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migrao, U = Usurio, C = Rotina de confrontao, O = Oscar
  */
  BEGIN
    v_idpes       := NEW.idpes;
    v_cnpj_novo     := NEW.cnpj;
    v_fantasia_novo     := NEW.fantasia;
    v_inscricao_estadual_novo := NEW.insc_estadual;

    IF TG_OP <> 'UPDATE' THEN
      v_cnpj_antigo     := 0;
      v_fantasia_antigo   := '';
      v_inscricao_estadual_antigo := 0;
    ELSE
      v_cnpj_antigo     := COALESCE(OLD.cnpj, 0);
      v_fantasia_antigo   := COALESCE(OLD.fantasia, '');
      v_inscricao_estadual_antigo := COALESCE(OLD.insc_estadual, 0);
    END IF;

    v_idcam_cnpj      := 1;
    v_idcam_fantasia    := 24;
    v_idcam_inscricao_estadual  := 25;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;

    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usurio ou pelo usurio do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migrao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;

    IF v_nova_credibilidade > 0 THEN

      -- CNPJ
      IF v_cnpj_novo <> v_cnpj_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cnpj||','||v_nova_credibilidade||');';
      END IF;

      -- NOME FANTASIA
      IF v_fantasia_novo <> v_fantasia_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_fantasia||','||v_nova_credibilidade||');';
      END IF;

      -- INSCRIO ESTADUAL
      IF v_inscricao_estadual_novo <> v_inscricao_estadual_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_inscricao_estadual||','||v_nova_credibilidade||');';
      END IF;
    END IF;

    -- Verificar os campos Vazios ou Nulos
    -- CNPJ
    IF v_cnpj_novo <= 0 OR v_cnpj_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_cnpj||','||v_sem_credibilidade||');';
    END IF;
    -- NOME FANTASIA
    IF TRIM(v_fantasia_novo)='' OR v_fantasia_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_fantasia||','||v_sem_credibilidade||');';
    END IF;

    -- INSCRIO ESTADUAL
    IF v_inscricao_estadual_novo <= 0 OR v_inscricao_estadual_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_inscricao_estadual||','||v_sem_credibilidade||');';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_pessoa_historico_campo(); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_pessoa_historico_campo() RETURNS "trigger"
    AS $$
DECLARE
  v_idpes       numeric;
  v_nome_novo     text;
  v_nome_antigo     text;
  v_email_novo      text;
  v_email_antigo      text;
  v_url_novo      text;
  v_url_antigo      text;

  v_comando     text;
  v_origem_gravacao   text;

  v_credibilidade_maxima    numeric;
  v_credibilidade_alta    numeric;
  v_sem_credibilidade   numeric;

  v_nova_credibilidade    numeric;

  v_registro      record;

  -- ID dos campos
  v_idcam_nome      numeric;
  v_idcam_email     numeric;
  v_idcam_url     numeric;

  /*
  consistenciacao.historico_campo.credibilidade: 1 = Mxima, 2 = Alta, 3 = Mdia, 4 = Baixa, 5 = Sem credibilidade
  cadastro.pessoa.origem_gravacao: M = Migrao, U = Usurio, C = Rotina de confrontao, O = Oscar
  */
  BEGIN
    v_idpes   := NEW.idpes;
    v_nome_novo := NEW.nome;
    v_email_novo  := NEW.email;
    v_url_novo  := NEW.url;

    IF TG_OP <> 'UPDATE' THEN
      v_nome_antigo := '';
      v_email_antigo  := '';
      v_url_antigo  := '';
    ELSE
      v_nome_antigo := COALESCE(OLD.nome, '');
      v_email_antigo  := COALESCE(OLD.email, '');
      v_url_antigo  := COALESCE(OLD.url, '');
    END IF;

    v_idcam_nome  := 2;
    v_idcam_email := 3;
    v_idcam_url := 4;

    v_nova_credibilidade := 0;
    v_credibilidade_maxima := 1;
    v_credibilidade_alta := 2;
    v_sem_credibilidade := 5;
    v_comando := 'SELECT origem_gravacao FROM cadastro.pessoa WHERE idpes='||quote_literal(v_idpes)||';';

    FOR v_registro IN EXECUTE v_comando LOOP
      v_origem_gravacao := v_registro.origem_gravacao;
    END LOOP;

    IF v_origem_gravacao = 'U' OR v_origem_gravacao = 'O' THEN -- os dados foram editados pelo usurio ou pelo usurio do Oscar
      v_nova_credibilidade := v_credibilidade_maxima;
    ELSIF v_origem_gravacao = 'M' THEN -- os dados foram originados por migrao
      v_nova_credibilidade := v_credibilidade_alta;
    END IF;

    IF v_nova_credibilidade > 0 THEN

      -- NOME
      IF v_nome_novo <> v_nome_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome||','||v_nova_credibilidade||');';
      END IF;

      -- E-MAIL
      IF v_email_novo <> v_email_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_email||','||v_nova_credibilidade||');';
      END IF;

      -- URL
      IF v_url_novo <> v_url_antigo THEN
        EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_url||','||v_nova_credibilidade||');';
      END IF;
    END IF;

    -- Verificar os campos Vazios ou Nulos
    -- NOME
    IF TRIM(v_nome_novo)='' OR v_nome_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_nome||','||v_sem_credibilidade||');';
    END IF;
    -- E-MAIL
    IF TRIM(v_email_novo)='' OR v_email_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_email||','||v_sem_credibilidade||');';
    END IF;
    -- URL
    IF TRIM(v_url_novo)='' OR v_url_novo IS NULL THEN
      EXECUTE 'SELECT consistenciacao.fcn_gravar_historico_campo('||v_idpes||','||v_idcam_url||','||v_sem_credibilidade||');';
    END IF;
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_cadastro(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_cadastro(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpesVelho     ALIAS for $1;
  v_idpesNovo      ALIAS for $2;
  v_cpfAux           NUMERIC;
  v_cpfIdpesNovo     NUMERIC;
  v_reg          record;
  v_regAux         record;
BEGIN
  --Unificando registros das tabelas do cadastros de pessoas--
  SET search_path = cadastro, pg_catalog;
  --FONE_PESSOA--
  FOR v_reg IN SELECT *
       FROM fone_pessoa
       WHERE
       idpes = v_idpesVelho AND
       tipo NOT IN (SELECT tipo FROM fone_pessoa WHERE idpes = v_idpesNovo) LOOP
    INSERT INTO fone_pessoa (idpes, tipo, ddd, fone) VALUES (v_idpesNovo,v_reg.tipo, v_reg.ddd, v_reg.fone);
  END LOOP;
  --Atualizando os telefones do idpeNovo que tenham o DDD e o FONE igual a zero--
  FOR v_reg IN SELECT * FROM fone_pessoa WHERE idpes = v_idpesNovo AND ddd = 0 AND fone = 0 LOOP
    DELETE FROM fone_pessoa WHERE idpes = v_reg.idpes AND tipo = v_reg.tipo;
    FOR v_regAux IN SELECT * FROM fone_pessoa WHERE idpes = v_idpesVelho AND tipo = v_reg.tipo LOOP
      INSERT INTO fone_pessoa(idpes,tipo,ddd,fone) VALUES (v_reg.idpes,v_regAux.tipo,v_regAux.ddd,v_regAux.fone);
    END LOOP;
  END LOOP;
  DELETE FROM fone_pessoa WHERE idpes = v_idpesVelho;
  --JURIDICA--
  DELETE FROM juridica WHERE idpes = v_idpesVelho;
  --HISTORICO_CARTAO--
  UPDATE historico_cartao SET idpes_cidadao = v_idpesNovo WHERE idpes_cidadao = v_idpesVelho;
  --ENDERECO_PESSOA--
  FOR v_reg IN SELECT *
         FROM endereco_pessoa
         WHERE
         idpes = v_idpesVelho AND
         tipo NOT IN (SELECT tipo FROM endereco_pessoa WHERE idpes = v_idpesNovo) LOOP
    INSERT INTO endereco_pessoa (idpes, tipo, cep, idlog, idbai, numero, letra, complemento, reside_desde)
    VALUES (v_idpesNovo,v_reg.tipo, v_reg.cep, v_reg.idlog, v_reg.idbai, v_reg.numero, v_reg.letra, v_reg.complemento, v_reg.reside_desde);
  END LOOP;
  DELETE FROM endereco_pessoa WHERE idpes = v_idpesVelho;
  --ENDERECO_EXTERNO--
  FOR v_reg IN SELECT *
     FROM endereco_externo
     WHERE
     idpes = v_idpesVelho AND
     tipo NOT IN (SELECT tipo FROM endereco_externo WHERE idpes = v_idpesNovo) LOOP
    INSERT INTO endereco_externo (idpes, tipo, idtlog, logradouro, numero, letra, complemento, bairro, cep, cidade, sigla_uf,reside_desde)
    VALUES (v_idpesNovo, v_reg.tipo, v_reg.idtlog, v_reg.logradouro, v_reg.numero, v_reg.letra, v_reg.complemento, v_reg.bairro, v_reg.cep, v_reg.cidade, v_reg.sigla_uf,v_reg.reside_desde);
  END LOOP;
  DELETE FROM endereco_externo WHERE idpes = v_idpesVelho;
  --FISICA_CPF--
  --Obtendo enventual CPF da pessoa antiga para ser inserido na pessoa nova--
  FOR v_reg IN SELECT cpf FROM fisica_cpf WHERE idpes = v_idpesVelho AND idpes <> v_idpesNovo LOOP
    v_cpfAux := v_reg.cpf;
  END LOOP;
  DELETE FROM fisica_cpf WHERE idpes = v_idpesVelho;
  IF v_cpfAux IS NOT NULL THEN
    FOR v_reg IN SELECT cpf FROM fisica_cpf WHERE idpes = v_idpesNovo LOOP
      v_cpfIdpesNovo := v_reg.cpf;
    END LOOP;
    --Verificando se o idpess j possu um CPF--
    IF v_cpfIdpesNovo IS NULL THEN
      INSERT INTO fisica_cpf (idpes, cpf) VALUES (v_idpesNovo,v_cpfAux);
    END IF;
  END IF;
  --FUNCIONARIO--
  UPDATE funcionario SET idpes = v_idpesNovo WHERE idpes = v_idpesVelho;
  --DOCUMENTO--
  DELETE FROM documento WHERE idpes = v_idpesVelho;
  --FISICA--
  DELETE FROM fisica WHERE idpes = v_idpesVelho;
  --PESSOA--
  DELETE FROM pessoa WHERE idpes = v_idpesVelho;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_cmf(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_cmf(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpesVelho ALIAS for $1;
  v_idpesNovo ALIAS for $2;

BEGIN
  --Unificando registros do SECM (Sistema de Emissao de Certidoes Municipais)--
  EXECUTE 'SELECT consistenciacao.fcn_unifica_secm('||v_idpesVelho||','||v_idpesNovo||');';
  --Unificando as tabelas do sistema de SGDO (Sistema de Gestao de Despesas Orcamentarias)--
  EXECUTE 'SELECT consistenciacao.fcn_unifica_sgdo('||v_idpesVelho||','||v_idpesNovo||');';
  --Unificando as tabelas do sistema de SGP (Sistema de Gerenciamento de Protocolo)--
  EXECUTE 'SELECT consistenciacao.fcn_unifica_sgp('||v_idpesVelho||','||v_idpesNovo||');';
  --Unificando as tabelas do sistema de SGPA (Sistema de Gestao da Praca de Atendimento)--
  EXECUTE 'SELECT consistenciacao.fcn_unifica_sgpa('||v_idpesVelho||','||v_idpesNovo||');';
  --Unificando as tabelas do sistema de SGSP (Sistema de Gerenciamento de Servicos consistenciacaoos)--
  EXECUTE 'SELECT consistenciacao.fcn_unifica_sgsp('||v_idpesVelho||','||v_idpesNovo||');';
      --Unificando registros das tabelas do SCD (Sistema de Consintenciacao de Dados)--
   EXECUTE 'SELECT consistenciacao.fcn_unifica_scd('||v_idpesVelho||','||v_idpesNovo||');';
  --Unificando registros das tabelas do SCA (Controle de acesso)--
  EXECUTE 'SELECT consistenciacao.fcn_unifica_sca('||v_idpesVelho||','||v_idpesNovo||');';
  --Unificando registros das tabelas de cadastro--
   EXECUTE 'SELECT consistenciacao.fcn_unifica_cadastro('||v_idpesVelho||','||v_idpesNovo||');';
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_sca(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_sca(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  --Parametro recebidos--
  v_idpesVelho ALIAS for $1;
  v_idpesNovo  ALIAS for $2;
  v_loginAux   varchar;
  v_reg      record;
BEGIN
  --Unificando registros das tabelas do SCA (Sistema de Controle de Acesso)--
  SET search_path = acesso, pg_catalog;
  --PESSOA_INSTITUICAO--
  --Inserindo as instituicoes do idvelho para o id novo--
  FOR v_reg IN SELECT *
               FROM pessoa_instituicao
               WHERE
         idpes = v_idpesVelho AND
         idins NOT IN
        (SELECT idins FROM
         pessoa_instituicao
         WHERE idpes = v_idpesNovo) LOOP
    INSERT INTO pessoa_instituicao (idpes,idins) VALUES (v_idpesNovo, v_reg.idins);
  END LOOP;
  --Apagando os registro do id velho--
  EXECUTE 'DELETE FROM pessoa_instituicao WHERE idpes = '||v_idpesVelho||';';
  --LOG_ACESSO--
  EXECUTE 'UPDATE log_acesso SET idpes ='||v_idpesNovo||' WHERE idpes = '||v_idpesVelho||';';
  --USUARIO_GRUPO--
  --Obtendo o login do novo idpes para ser usado na unificacao da tabela usuario_grupo--
  FOR v_reg IN SELECT login FROM usuario WHERE idpes = v_idpesNovo LOOP
    v_loginAux := v_reg.login;
  END LOOP;
  --Inserindo os grupos do idpes velho para o idpes novo--
  FOR v_reg IN SELECT *
         FROM usuario_grupo
         WHERE
         login IN (SELECT login FROM usuario WHERE idpes = v_idpesVelho) AND
         idgrp NOT IN (SELECT idgrp
                       FROM usuario_grupo
           WHERE
           login = v_loginAux) LOOP
    INSERT INTO usuario_grupo (idgrp,login) VALUES (v_reg.idgrp, v_loginAux);
  END LOOP;
  --Deletando os registros do idpes velho--
  EXECUTE 'DELETE FROM usuario_grupo
            WHERE login IN
         (SELECT login FROM usuario WHERE idpes ='|| v_idpesVelho ||');';
  --LOG_ERRO--
  EXECUTE 'UPDATE log_erro SET idpes = '||v_idpesNovo||' WHERE idpes = '||v_idpesVelho||';';

  --USUARIO--
  EXECUTE 'DELETE FROM usuario WHERE idpes ='|| v_idpesVelho||';';
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_scd(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_scd(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpesVelho ALIAS for $1;
  v_idpesNovo  ALIAS for $2;
BEGIN
  --Unificando registros das tabelas do SCD (Sistema de Consintenciacao de Dados)--
  SET search_path = consistenciacao, pg_catalog;
  --INCOERENCIA--
  UPDATE incoerencia_pessoa_possivel SET idpes =  v_idpesNovo WHERE idpes = v_idpesVelho;
  --CONFRONTACAO--
  UPDATE confrontacao SET idpes =  v_idpesNovo WHERE idpes = v_idpesVelho;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_sgp(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_sgp(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpesVelho ALIAS for $1;
  v_idpesNovo ALIAS for $2;
BEGIN
  --Unificando as tabelas do sistema de SGP (Sistema de Gerenciamento de Protocolo)--
  SET search_path = protocolo, pg_catalog;
  --ATIVIDADE--
  UPDATE atividade SET idpes_ini = v_idpesNovo WHERE idpes_ini = v_idpesVelho;
  UPDATE atividade SET idpes_fim = v_idpesNovo WHERE idpes_fim = v_idpesVelho;
  --LOG_FUNC_PROCESSO--
  UPDATE log_func_processo SET idpes = v_idpesNovo WHERE idpes = v_idpesVelho;
  --PROCESSO--
  UPDATE processo SET idpesreq = v_idpesNovo WHERE idpesreq = v_idpesVelho;
  UPDATE processo SET idpesfav = v_idpesNovo WHERE idpesfav = v_idpesVelho;
  --TEMP_ATIVIDADE--
  UPDATE temp_atividade SET idpes = v_idpesNovo WHERE idpes = v_idpesVelho;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_sgpa(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_sgpa(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpesVelho  ALIAS for $1;
  v_idpesNovo   ALIAS for $2;
  v_reg       record;
  v_idAtnVelho  numeric;
  v_idAtnNovo numeric;
BEGIN
  --Unificando as tabelas do sistema de SGPA (Sistema de getao da Praca de Atendimento)--
  SET search_path = praca, pg_catalog;
  --Obtendo o id do atendente velho para poder fazer UPDATE nas tabelas para o id do atendeentente novo--
  FOR v_reg IN SELECT idatn FROM atendente WHERE idpes = v_idpesVelho LOOP
    v_idAtnVelho := v_reg.idatn;
  END LOOP;
  FOR v_reg IN SELECT idatn FROM atendente WHERE idpes = v_idpesNovo LOOP
    v_idAtnNovo := v_reg.idatn;
  END LOOP;
  --ATENDIMENTO--
  UPDATE atendimento SET idpes = v_idpesNovo WHERE idpes = v_idpesVelho;
  UPDATE atendimento SET idatn = v_idAtnNovo WHERE idatn = v_idAtnVelho;
  --TURNO_ATENDENTE--
  FOR v_reg IN SELECT * FROM turno_atendente
         WHERE
         idatn = v_idAtnVelho AND
         idtur NOT IN (SELECT idtur FROM turno_atendente WHERE idatn = v_idAtnNovo) LOOP
    INSERT INTO turno_atendente (idmes,idins,idatn,idtur) VALUES (v_reg.idmes, v_reg.idins, v_idAtnNovo,v_reg.idtur);
  END LOOP;
  DELETE FROM turno_atendente WHERE idatn = v_idAtnVelho;
  --AUSENCIA--
  UPDATE ausencia SET idatn = v_idAtnNovo WHERE idatn = v_idAtnVelho;
  --SITUCAO_ATENDENTE--
  UPDATE situacao_atendente SET idatn = v_idAtnNovo WHERE idatn = v_idAtnVelho;
  --PRODUTIVIDADE--
  UPDATE produtividade SET idatn = v_idAtnNovo WHERE idatn = v_idAtnVelho;
  --ATENDENTE--
  DELETE FROM atendente WHERE idpes = v_idpesVelho;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_unifica_sgsp(numeric, numeric); Type: FUNCTION; Schema: consistenciacao; Owner: -
--

CREATE FUNCTION fcn_unifica_sgsp(numeric, numeric) RETURNS integer
    AS $_$
DECLARE
  -- Parametro recebidos
  v_idpesVelho ALIAS for $1;
  v_idpesNovo  ALIAS for $2;
  v_reg      record;
BEGIN
  --Unificando registros das tabelas do SGSP (Sistema de Gererenciamento de Servios Pblicos)--
  SET search_path = servicos, pg_catalog;
  --FUNCIONARIO_AUTORIZADO--
  FOR v_reg IN SELECT *
         FROM funcionario_autorizado
         WHERE
         idpes = v_idpesVelho AND
         idpes <> v_idpesNovo LOOP
    INSERT INTO funcionario_autorizado (idpes, idins) VALUES (v_idpesNovo, v_reg.idins);
  END LOOP;
  --SOLICITACAO_SERVICO--
  UPDATE solicitacao_servico SET idpes_atendente = v_idpesNovo WHERE idpes_atendente = v_idpesVelho;
  UPDATE solicitacao_servico SET idpes_planejamento = v_idpesNovo WHERE idpes_planejamento = v_idpesVelho;
  --SOLICITANTE--
  UPDATE solicitante SET idpes = v_idpesNovo WHERE idpes = v_idpesVelho;
  --ORDEM_SERVICO--
  UPDATE ordem_servico SET idpes = v_idpesNovo WHERE idpes = v_idpesVelho;
  --APROVACAO--
  UPDATE aprovacao SET idpes_resposta = v_idpesNovo WHERE idpes_resposta = v_idpesVelho;
  --AUTORIZACAO_DIARIA--
  UPDATE autorizacao_diaria SET idpes_usuario = v_idpesNovo WHERE idpes_usuario = v_idpesVelho;
  UPDATE autorizacao_diaria SET idpes_autorizacao = v_idpesNovo WHERE idpes_autorizacao = v_idpesVelho;
  --FUNCIONARIO_AUTORIZADO--
  DELETE FROM funcionario_autorizado WHERE idpes = v_idpesVelho;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


SET search_path = historico, pg_catalog;

--
-- Name: fcn_delete_grava_historico_bairro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_bairro() RETURNS "trigger"
    AS $$
DECLARE
   v_idbai    numeric;
   v_idmun    numeric;
   v_nome   varchar;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
BEGIN
   v_idbai    := OLD.idbai;
   v_idmun    := OLD.idmun;
   v_nome   := OLD.nome;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA BAIRRO
      INSERT INTO historico.bairro
      (idbai, idmun, nome, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idbai, v_idmun, v_nome, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_cep_logradouro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_cep_logradouro() RETURNS "trigger"
    AS $$
DECLARE
   v_cep    numeric;
   v_idlog    numeric;
   v_nroini   numeric;
   v_nrofin   numeric;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
BEGIN
   v_cep    := OLD.cep;
   v_idlog    := OLD.idlog;
   v_nroini   := OLD.nroini;
   v_nrofin   := OLD.nrofin;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA CEP_LOGRADOURO
      INSERT INTO historico.cep_logradouro
      (cep, idlog, nroini, nrofin, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_cep, v_idlog, v_nroini, v_nrofin, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_cep_logradouro_bairro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_cep_logradouro_bairro() RETURNS "trigger"
    AS $$
DECLARE
   v_idbai    numeric;
   v_idlog    numeric;
   v_cep    numeric;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idbai    := OLD.idbai;
   v_idlog    := OLD.idlog;
   v_cep    := OLD.cep;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA CEP_LOGRADOURO_BAIRRO
      INSERT INTO historico.cep_logradouro_bairro
      (idbai, idlog, cep, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idbai, v_idlog, v_cep, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_documento(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_documento() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_sigla_uf_exp_rg  char(2);
   v_rg     numeric;
   v_data_exp_rg  date;
   v_tipo_cert_civil  numeric;
   v_num_termo    numeric;
   v_num_livro    varchar;
   v_num_folha    numeric;
   v_data_emissao_cert_civil  date;
   v_sigla_uf_cert_civil  char(2);
   v_sigla_uf_cart_trabalho char(2);
   v_cartorio_cert_civil  varchar;
   v_num_cart_trabalho    numeric;
   v_serie_cart_trabalho  numeric;
   v_data_emissao_cart_trabalho date;
   v_num_tit_eleitor  numeric;
   v_zona_tit_eleitor numeric;
   v_secao_tit_eleitor  numeric;
   v_idorg_exp_rg   numeric;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
BEGIN
   v_idpes      := OLD.idpes;
   v_sigla_uf_exp_rg    := OLD.sigla_uf_exp_rg;
   v_rg       := OLD.rg;
   v_data_exp_rg    := OLD.data_exp_rg;
   v_tipo_cert_civil    := OLD.tipo_cert_civil;
   v_num_termo      := OLD.num_termo;
   v_num_livro      := OLD.num_livro;
   v_num_folha      := OLD.num_folha;
   v_data_emissao_cert_civil  := OLD.data_emissao_cert_civil;
   v_sigla_uf_cert_civil  := OLD.sigla_uf_cert_civil;
   v_sigla_uf_cart_trabalho := OLD.sigla_uf_cart_trabalho;
   v_cartorio_cert_civil  := OLD.cartorio_cert_civil;
   v_num_cart_trabalho    := OLD.num_cart_trabalho;
   v_serie_cart_trabalho  := OLD.serie_cart_trabalho;
   v_data_emissao_cart_trabalho := OLD.data_emissao_cart_trabalho;
   v_num_tit_eleitor    := OLD.num_tit_eleitor;
   v_zona_tit_eleitor   := OLD.zona_tit_eleitor;
   v_secao_tit_eleitor    := OLD.secao_tit_eleitor;
   v_idorg_exp_rg   := OLD.idorg_exp_rg;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA DOCUMENTO
      INSERT INTO historico.documento
      (idpes, sigla_uf_exp_rg, rg, data_exp_rg, tipo_cert_civil, num_termo, num_livro, num_folha, data_emissao_cert_civil, sigla_uf_cert_civil, sigla_uf_cart_trabalho, cartorio_cert_civil, num_cart_trabalho, serie_cart_trabalho, data_emissao_cart_trabalho, num_tit_eleitor, zona_tit_eleitor, secao_tit_eleitor, idorg_exp_rg, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_sigla_uf_exp_rg, v_rg, v_data_exp_rg, v_tipo_cert_civil, v_num_termo, v_num_livro, v_num_folha, v_data_emissao_cert_civil, v_sigla_uf_cert_civil, v_sigla_uf_cart_trabalho, v_cartorio_cert_civil, v_num_cart_trabalho, v_serie_cart_trabalho, v_data_emissao_cart_trabalho, v_num_tit_eleitor, v_zona_tit_eleitor, v_secao_tit_eleitor, v_idorg_exp_rg, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_endereco_externo(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_endereco_externo() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_tipo   numeric;
   v_sigla_uf   char(2);
   v_idtlog   varchar;
   v_logradouro   varchar;
   v_numero   numeric;
   v_letra    char(1);
   v_complemento  varchar;
   v_bairro   varchar;
   v_cep    numeric;
   v_cidade   varchar;
   v_reside_desde date;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
BEGIN
   v_idpes      := OLD.idpes;
   v_tipo     := OLD.tipo;
   v_sigla_uf     := OLD.sigla_uf;
   v_idtlog     := OLD.idtlog;
   v_logradouro     := OLD.logradouro;
   v_numero     := OLD.numero;
   v_letra      := OLD.letra;
   v_complemento    := OLD.complemento;
   v_bairro     := OLD.bairro;
   v_cep      := OLD.cep;
   v_cidade     := OLD.cidade;
   v_reside_desde   := OLD.reside_desde;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA ENDERECO_EXTERNO
      INSERT INTO historico.endereco_externo
      (idpes, tipo, sigla_uf, idtlog, logradouro, numero, letra, complemento, bairro, cep, cidade, reside_desde, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_tipo, v_sigla_uf, v_idtlog, v_logradouro, v_numero, v_letra, v_complemento, v_bairro, v_cep, v_cidade, v_reside_desde, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_endereco_pessoa(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_endereco_pessoa() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_tipo   numeric;
   v_cep    numeric;
   v_idlog    numeric;
   v_idbai    numeric;
   v_numero   numeric;
   v_letra    char(1);
   v_complemento  varchar;
   v_reside_desde date;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
BEGIN
   v_idpes    := OLD.idpes;
   v_tipo   := OLD.tipo;
   v_cep    := OLD.cep;
   v_idlog    := OLD.idlog;
   v_idbai    := OLD.idbai;
   v_numero   := OLD.numero;
   v_letra    := OLD.letra;
   v_complemento  := OLD.complemento;
   v_reside_desde := OLD.reside_desde;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA ENDERECO_PESSOA
      INSERT INTO historico.endereco_pessoa
      (idpes, tipo, cep, idlog, idbai, numero, letra, complemento, reside_desde, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_tipo, v_cep, v_idlog, v_idbai, v_numero, v_letra, v_complemento, v_reside_desde, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_fisica(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_fisica() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_data_nasc      date;
   v_sexo     char(1);
   v_idpes_mae      numeric;
   v_idpes_pai      numeric;
   v_idpes_responsavel    numeric;
   v_idesco     numeric;
   v_ideciv     numeric;
   v_idpes_con      numeric;
   v_data_uniao     date;
   v_data_obito     date;
   v_nacionalidade    numeric;
   v_idpais_estrangeiro   numeric;
   v_data_chegada_brasil  date;
   v_idmun_nascimento   numeric;
   v_ultima_empresa   varchar;
   v_idocup     numeric;
   v_nome_mae     varchar;
   v_nome_pai     varchar;
   v_nome_conjuge   varchar;
   v_nome_responsavel   varchar;
   v_justificativa_provisorio varchar;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idpes      := OLD.idpes;
   v_data_nasc      := OLD.data_nasc;
   v_sexo     := OLD.sexo;
   v_idpes_mae      := OLD.idpes_mae;
   v_idpes_pai      := OLD.idpes_pai;
   v_idpes_responsavel    := OLD.idpes_responsavel;
   v_idesco     := OLD.idesco;
   v_ideciv     := OLD.ideciv;
   v_idpes_con      := OLD.idpes_con;
   v_data_uniao     := OLD.data_uniao;
   v_data_obito     := OLD.data_obito;
   v_nacionalidade    := OLD.nacionalidade;
   v_idpais_estrangeiro   := OLD.idpais_estrangeiro;
   v_data_chegada_brasil  := OLD.data_chegada_brasil;
   v_idmun_nascimento   := OLD.idmun_nascimento;
   v_ultima_empresa   := OLD.ultima_empresa;
   v_idocup     := OLD.idocup;
   v_nome_mae     := OLD.nome_mae;
   v_nome_pai     := OLD.nome_pai;
   v_nome_conjuge   := OLD.nome_conjuge;
   v_nome_responsavel   := OLD.nome_responsavel;
   v_justificativa_provisorio := OLD.justificativa_provisorio;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FISICA
      INSERT INTO historico.fisica
      (idpes, data_nasc, sexo, idpes_mae, idpes_pai, idpes_responsavel, idesco, ideciv, idpes_con, data_uniao, data_obito, nacionalidade, idpais_estrangeiro, data_chegada_brasil, idmun_nascimento, ultima_empresa, idocup, nome_mae, nome_pai, nome_conjuge, nome_responsavel, justificativa_provisorio, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_data_nasc, v_sexo, v_idpes_mae, v_idpes_pai, v_idpes_responsavel, v_idesco, v_ideciv, v_idpes_con, v_data_uniao, v_data_obito, v_nacionalidade, v_idpais_estrangeiro, v_data_chegada_brasil, v_idmun_nascimento, v_ultima_empresa, v_idocup, v_nome_mae, v_nome_pai, v_nome_conjuge, v_nome_responsavel, v_justificativa_provisorio, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_fisica_cpf(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_fisica_cpf() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_cpf      numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idpes      := OLD.idpes;
   v_cpf      := OLD.cpf;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FISICA_CPF
      INSERT INTO historico.fisica_cpf
      (idpes, cpf, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_cpf, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_fone_pessoa(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_fone_pessoa() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_tipo     numeric;
   v_ddd      numeric;
   v_fone     numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idpes      := OLD.idpes;
   v_tipo     := OLD.tipo;
   v_ddd      := OLD.ddd;
   v_fone     := OLD.fone;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FONE_PESSOA
      INSERT INTO historico.fone_pessoa
      (idpes, tipo, ddd, fone, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_tipo, v_ddd, v_fone, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_funcionario(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_funcionario() RETURNS "trigger"
    AS $$
DECLARE
   v_matricula      numeric;
   v_idins      numeric;
   v_idset      numeric;
   v_idpes      numeric;
   v_situacao     char(1);
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_matricula      := OLD.matricula;
   v_idins      := OLD.idins;
   v_idset      := OLD.idset;
   v_idpes      := OLD.idpes;
   v_situacao     := OLD.situacao;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FUNCIONARIO
      INSERT INTO historico.funcionario
      (matricula, idins, idset, idpes, situacao, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_matricula, v_idins, v_idset, v_idpes, v_situacao, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_juridica(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_juridica() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_cnpj     numeric;
   v_fantasia     varchar;
   v_insc_estadual    numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idpes      := OLD.idpes;
   v_cnpj     := OLD.cnpj;
   v_fantasia     := OLD.fantasia;
   v_insc_estadual    := OLD.insc_estadual;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA JURIDICA
      INSERT INTO historico.juridica
      (idpes, cnpj, fantasia, insc_estadual, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_cnpj, v_fantasia, v_insc_estadual, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_logradouro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_logradouro() RETURNS "trigger"
    AS $$
DECLARE
   v_idlog      numeric;
   v_idtlog     varchar;
   v_nome     varchar;
   v_idmun      numeric;
   v_ident_oficial    char(1);
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idlog      := OLD.idlog;
   v_idtlog     := OLD.idtlog;
   v_nome     := OLD.nome;
   v_idmun      := OLD.idmun;
   v_ident_oficial    := OLD.ident_oficial;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA LOGRADOURO
      INSERT INTO historico.logradouro
      (idlog, idtlog, nome, idmun, ident_oficial, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idlog, v_idtlog, v_nome, v_idmun, v_ident_oficial, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_municipio(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_municipio() RETURNS "trigger"
    AS $$
DECLARE
   v_idmun      numeric;
   v_sigla_uf     char(2);
   v_nome     varchar;
   v_area_km2     numeric;
   v_idmreg     numeric;
   v_idasmun      numeric;
   v_cod_ibge     numeric;
   v_geom     TEXT;
   v_tipo     char(1);
   v_idmun_pai      numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idmun      := OLD.idmun;
   v_sigla_uf     := OLD.sigla_uf;
   v_nome     := OLD.nome;
   v_area_km2     := OLD.area_km2;
   v_idmreg     := OLD.idmreg;
   v_idasmun      := OLD.idasmun;
   v_cod_ibge     := OLD.cod_ibge;
   v_geom     := OLD.geom;
   v_tipo     := OLD.tipo;
   v_idmun_pai      := OLD.idmun_pai;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA MUNICIPIO
      INSERT INTO historico.municipio
      (idmun, sigla_uf, nome, area_km2, idmreg, idasmun, cod_ibge, geom, tipo, idmun_pai, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idmun, v_sigla_uf, v_nome, v_area_km2, v_idmreg, v_idasmun, v_cod_ibge, v_geom, v_tipo, v_idmun_pai, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_pessoa(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_pessoa() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_nome   TEXT;
   v_idpes_cad    numeric;
   v_data_cad   timestamp;
   v_url    TEXT;
   v_tipo   char;
   v_idpes_rev    numeric;
   v_data_rev   timestamp;
   v_email    TEXT;
   v_situacao   char;
   v_origem_gravacao  char;
   v_idsis_rev    numeric;
   v_idsis_cad    numeric;
BEGIN
   v_idpes    := OLD.idpes;
   v_nome   := OLD.nome;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_url    := OLD.url;
   v_tipo   := OLD.tipo;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_email    := OLD.email;
   v_situacao   := OLD.situacao;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idsis_cad    := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA PESSOA
      INSERT INTO historico.pessoa
      (idpes, nome, idpes_cad, data_cad, url, tipo, idpes_rev, data_rev, email, situacao, origem_gravacao, idsis_rev, idsis_cad, operacao) VALUES
      (v_idpes, v_nome, v_idpes_cad, v_data_cad, v_url, v_tipo, v_idpes_rev, v_data_rev, v_email, v_situacao, v_origem_gravacao, v_idsis_rev, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_grava_historico_socio(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_delete_grava_historico_socio() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes_juridica   numeric;
   v_idpes_fisica   numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
BEGIN
   v_idpes_juridica   := OLD.idpes_juridica;
   v_idpes_fisica   := OLD.idpes_fisica;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA SOCIO
      INSERT INTO historico.socio
      (idpes_juridica, idpes_fisica, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes_juridica, v_idpes_fisica, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, 'E');

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_bairro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_bairro() RETURNS "trigger"
    AS $$
DECLARE
   v_idbai    numeric;
   v_idmun    numeric;
   v_nome   varchar;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idbai    := OLD.idbai;
   v_idmun    := OLD.idmun;
   v_nome   := OLD.nome;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;
   v_operacao   := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA BAIRRO
      INSERT INTO historico.bairro
      (idbai, idmun, nome, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idbai, v_idmun, v_nome, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_cep_logradouro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_cep_logradouro() RETURNS "trigger"
    AS $$
DECLARE
   v_cep    numeric;
   v_idlog    numeric;
   v_nroini   numeric;
   v_nrofin   numeric;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_cep    := OLD.cep;
   v_idlog    := OLD.idlog;
   v_nroini   := OLD.nroini;
   v_nrofin   := OLD.nrofin;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;
   v_operacao   := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA CEP_LOGRADOURO
      INSERT INTO historico.cep_logradouro
      (cep, idlog, nroini, nrofin, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_cep, v_idlog, v_nroini, v_nrofin, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_cep_logradouro_bairro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_cep_logradouro_bairro() RETURNS "trigger"
    AS $$
DECLARE
   v_idbai    numeric;
   v_idlog    numeric;
   v_cep    numeric;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idbai    := OLD.idbai;
   v_idlog    := OLD.idlog;
   v_cep    := OLD.cep;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;
   v_operacao   := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA CEP_LOGRADOURO_BAIRRO
      INSERT INTO historico.cep_logradouro_bairro
      (idbai, idlog, cep, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idbai, v_idlog, v_cep, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_documento(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_documento() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_sigla_uf_exp_rg  char(2);
   v_rg     numeric;
   v_data_exp_rg  date;
   v_tipo_cert_civil  numeric;
   v_num_termo    numeric;
   v_num_livro    varchar;
   v_num_folha    numeric;
   v_data_emissao_cert_civil  date;
   v_sigla_uf_cert_civil  char(2);
   v_sigla_uf_cart_trabalho char(2);
   v_cartorio_cert_civil  varchar;
   v_num_cart_trabalho    numeric;
   v_serie_cart_trabalho  numeric;
   v_data_emissao_cart_trabalho date;
   v_num_tit_eleitor  numeric;
   v_zona_tit_eleitor numeric;
   v_secao_tit_eleitor  numeric;
   v_idorg_exp_rg   numeric;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idpes      := OLD.idpes;
   v_sigla_uf_exp_rg    := OLD.sigla_uf_exp_rg;
   v_rg       := OLD.rg;
   v_data_exp_rg    := OLD.data_exp_rg;
   v_tipo_cert_civil    := OLD.tipo_cert_civil;
   v_num_termo      := OLD.num_termo;
   v_num_livro      := OLD.num_livro;
   v_num_folha      := OLD.num_folha;
   v_data_emissao_cert_civil  := OLD.data_emissao_cert_civil;
   v_sigla_uf_cert_civil  := OLD.sigla_uf_cert_civil;
   v_sigla_uf_cart_trabalho := OLD.sigla_uf_cart_trabalho;
   v_cartorio_cert_civil  := OLD.cartorio_cert_civil;
   v_num_cart_trabalho    := OLD.num_cart_trabalho;
   v_serie_cart_trabalho  := OLD.serie_cart_trabalho;
   v_data_emissao_cart_trabalho := OLD.data_emissao_cart_trabalho;
   v_num_tit_eleitor    := OLD.num_tit_eleitor;
   v_zona_tit_eleitor   := OLD.zona_tit_eleitor;
   v_secao_tit_eleitor    := OLD.secao_tit_eleitor;
   v_idorg_exp_rg   := OLD.idorg_exp_rg;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA DOCUMENTO
      INSERT INTO historico.documento
      (idpes, sigla_uf_exp_rg, rg, data_exp_rg, tipo_cert_civil, num_termo, num_livro, num_folha, data_emissao_cert_civil, sigla_uf_cert_civil, sigla_uf_cart_trabalho, cartorio_cert_civil, num_cart_trabalho, serie_cart_trabalho, data_emissao_cart_trabalho, num_tit_eleitor, zona_tit_eleitor, secao_tit_eleitor, idorg_exp_rg, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_sigla_uf_exp_rg, v_rg, v_data_exp_rg, v_tipo_cert_civil, v_num_termo, v_num_livro, v_num_folha, v_data_emissao_cert_civil, v_sigla_uf_cert_civil, v_sigla_uf_cart_trabalho, v_cartorio_cert_civil, v_num_cart_trabalho, v_serie_cart_trabalho, v_data_emissao_cart_trabalho, v_num_tit_eleitor, v_zona_tit_eleitor, v_secao_tit_eleitor, v_idorg_exp_rg, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_endereco_externo(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_endereco_externo() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_tipo   numeric;
   v_sigla_uf   char(2);
   v_idtlog   varchar;
   v_logradouro   varchar;
   v_numero   numeric;
   v_letra    char(1);
   v_complemento  varchar;
   v_bairro   varchar;
   v_cep    numeric;
   v_cidade   varchar;
   v_reside_desde date;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idpes    := OLD.idpes;
   v_tipo   := OLD.tipo;
   v_sigla_uf   := OLD.sigla_uf;
   v_idtlog   := OLD.idtlog;
   v_logradouro   := OLD.logradouro;
   v_numero   := OLD.numero;
   v_letra    := OLD.letra;
   v_complemento  := OLD.complemento;
   v_bairro   := OLD.bairro;
   v_cep    := OLD.cep;
   v_cidade   := OLD.cidade;
   v_reside_desde := OLD.reside_desde;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;
   v_operacao   := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA ENDERECO_EXTERNO
      INSERT INTO historico.endereco_externo
      (idpes, tipo, sigla_uf, idtlog, logradouro, numero, letra, complemento, bairro, cep, cidade, reside_desde, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_tipo, v_sigla_uf, v_idtlog, v_logradouro, v_numero, v_letra, v_complemento, v_bairro, v_cep, v_cidade, v_reside_desde, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_endereco_pessoa(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_endereco_pessoa() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_tipo   numeric;
   v_cep    numeric;
   v_idlog    numeric;
   v_idbai    numeric;
   v_numero   numeric;
   v_letra    char(1);
   v_complemento  varchar;
   v_reside_desde date;
   v_idpes_rev    numeric;
   v_data_rev   TIMESTAMP;
   v_origem_gravacao  char(1);
   v_idsis_rev    numeric;
   v_idpes_cad    numeric;
   v_data_cad   TIMESTAMP;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idpes    := OLD.idpes;
   v_tipo   := OLD.tipo;
   v_cep    := OLD.cep;
   v_idlog    := OLD.idlog;
   v_idbai    := OLD.idbai;
   v_numero   := OLD.numero;
   v_letra    := OLD.letra;
   v_complemento  := OLD.complemento;
   v_reside_desde := OLD.reside_desde;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_idsis_cad    := OLD.idsis_cad;
   v_operacao   := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA ENDERECO_PESSOA
      INSERT INTO historico.endereco_pessoa
      (idpes, tipo, cep, idlog, idbai, numero, letra, complemento, reside_desde, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_tipo, v_cep, v_idlog, v_idbai, v_numero, v_letra, v_complemento, v_reside_desde, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_fisica(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_fisica() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_data_nasc      date;
   v_sexo     char(1);
   v_idpes_mae      numeric;
   v_idpes_pai      numeric;
   v_idpes_responsavel    numeric;
   v_idesco     numeric;
   v_ideciv     numeric;
   v_idpes_con      numeric;
   v_data_uniao     date;
   v_data_obito     date;
   v_nacionalidade    numeric;
   v_idpais_estrangeiro   numeric;
   v_data_chegada_brasil  date;
   v_idmun_nascimento   numeric;
   v_ultima_empresa   varchar;
   v_idocup     numeric;
   v_nome_mae     varchar;
   v_nome_pai     varchar;
   v_nome_conjuge   varchar;
   v_nome_responsavel   varchar;
   v_justificativa_provisorio varchar;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idpes      := OLD.idpes;
   v_data_nasc      := OLD.data_nasc;
   v_sexo     := OLD.sexo;
   v_idpes_mae      := OLD.idpes_mae;
   v_idpes_pai      := OLD.idpes_pai;
   v_idpes_responsavel    := OLD.idpes_responsavel;
   v_idesco     := OLD.idesco;
   v_ideciv     := OLD.ideciv;
   v_idpes_con      := OLD.idpes_con;
   v_data_uniao     := OLD.data_uniao;
   v_data_obito     := OLD.data_obito;
   v_nacionalidade    := OLD.nacionalidade;
   v_idpais_estrangeiro   := OLD.idpais_estrangeiro;
   v_data_chegada_brasil  := OLD.data_chegada_brasil;
   v_idmun_nascimento   := OLD.idmun_nascimento;
   v_ultima_empresa   := OLD.ultima_empresa;
   v_idocup     := OLD.idocup;
   v_nome_mae     := OLD.nome_mae;
   v_nome_pai     := OLD.nome_pai;
   v_nome_conjuge   := OLD.nome_conjuge;
   v_nome_responsavel   := OLD.nome_responsavel;
   v_justificativa_provisorio := OLD.justificativa_provisorio;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FISICA
      INSERT INTO historico.fisica
      (idpes, data_nasc, sexo, idpes_mae, idpes_pai, idpes_responsavel, idesco, ideciv, idpes_con, data_uniao, data_obito, nacionalidade, idpais_estrangeiro, data_chegada_brasil, idmun_nascimento, ultima_empresa, idocup, nome_mae, nome_pai, nome_conjuge, nome_responsavel, justificativa_provisorio, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_data_nasc, v_sexo, v_idpes_mae, v_idpes_pai, v_idpes_responsavel, v_idesco, v_ideciv, v_idpes_con, v_data_uniao, v_data_obito, v_nacionalidade, v_idpais_estrangeiro, v_data_chegada_brasil, v_idmun_nascimento, v_ultima_empresa, v_idocup, v_nome_mae, v_nome_pai, v_nome_conjuge, v_nome_responsavel, v_justificativa_provisorio, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_fisica_cpf(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_fisica_cpf() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_cpf      numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idpes      := OLD.idpes;
   v_cpf      := OLD.cpf;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FISICA_CPF
      INSERT INTO historico.fisica_cpf
      (idpes, cpf, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_cpf, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_fone_pessoa(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_fone_pessoa() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_tipo     numeric;
   v_ddd      numeric;
   v_fone     numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idpes      := OLD.idpes;
   v_tipo     := OLD.tipo;
   v_ddd      := OLD.ddd;
   v_fone     := OLD.fone;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FONE_PESSOA
      INSERT INTO historico.fone_pessoa
      (idpes, tipo, ddd, fone, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_tipo, v_ddd, v_fone, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_funcionario(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_funcionario() RETURNS "trigger"
    AS $$
DECLARE
   v_matricula      numeric;
   v_idins      numeric;
   v_idset      numeric;
   v_idpes      numeric;
   v_situacao     char(1);
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_matricula      := OLD.matricula;
   v_idins      := OLD.idins;
   v_idset      := OLD.idset;
   v_idpes      := OLD.idpes;
   v_situacao     := OLD.situacao;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA FUNCIONARIO
      INSERT INTO historico.funcionario
      (matricula, idins, idset, idpes, situacao, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_matricula, v_idins, v_idset, v_idpes, v_situacao, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_juridica(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_juridica() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes      numeric;
   v_cnpj     numeric;
   v_fantasia     varchar;
   v_insc_estadual    numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idpes      := OLD.idpes;
   v_cnpj     := OLD.cnpj;
   v_fantasia     := OLD.fantasia;
   v_insc_estadual    := OLD.insc_estadual;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA JURIDICA
      INSERT INTO historico.juridica
      (idpes, cnpj, fantasia, insc_estadual, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes, v_cnpj, v_fantasia, v_insc_estadual, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_logradouro(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_logradouro() RETURNS "trigger"
    AS $$
DECLARE
   v_idlog      numeric;
   v_idtlog     varchar;
   v_nome     varchar;
   v_idmun      numeric;
   v_ident_oficial    char(1);
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idlog      := OLD.idlog;
   v_idtlog     := OLD.idtlog;
   v_nome     := OLD.nome;
   v_idmun      := OLD.idmun;
   v_ident_oficial    := OLD.ident_oficial;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA LOGRADOURO
      INSERT INTO historico.logradouro
      (idlog, idtlog, nome, idmun, ident_oficial, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idlog, v_idtlog, v_nome, v_idmun, v_ident_oficial, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_municipio(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_municipio() RETURNS "trigger"
    AS $$
DECLARE
   v_idmun      numeric;
   v_sigla_uf     char(2);
   v_nome     varchar;
   v_area_km2     numeric;
   v_idmreg     numeric;
   v_idasmun      numeric;
   v_cod_ibge     numeric;
   v_geom     TEXT;
   v_tipo     char(1);
   v_idmun_pai      numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idmun      := OLD.idmun;
   v_sigla_uf     := OLD.sigla_uf;
   v_nome     := OLD.nome;
   v_area_km2     := OLD.area_km2;
   v_idmreg     := OLD.idmreg;
   v_idasmun      := OLD.idasmun;
   v_cod_ibge     := OLD.cod_ibge;
   v_geom     := OLD.geom;
   v_tipo     := OLD.tipo;
   v_idmun_pai      := OLD.idmun_pai;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA MUNICIPIO
      INSERT INTO historico.municipio
      (idmun, sigla_uf, nome, area_km2, idmreg, idasmun, cod_ibge, geom, tipo, idmun_pai, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idmun, v_sigla_uf, v_nome, v_area_km2, v_idmreg, v_idasmun, v_cod_ibge, v_geom, v_tipo, v_idmun_pai, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_pessoa(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_pessoa() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes    numeric;
   v_nome   TEXT;
   v_idpes_cad    numeric;
   v_data_cad   timestamp;
   v_url    TEXT;
   v_tipo   char;
   v_idpes_rev    numeric;
   v_data_rev   timestamp;
   v_email    TEXT;
   v_situacao   char;
   v_origem_gravacao  char;
   v_idsis_rev    numeric;
   v_idsis_cad    numeric;
   v_operacao   char(1);
BEGIN
   v_idpes    := OLD.idpes;
   v_nome   := OLD.nome;
   v_idpes_cad    := OLD.idpes_cad;
   v_data_cad   := OLD.data_cad;
   v_url    := OLD.url;
   v_tipo   := OLD.tipo;
   v_idpes_rev    := OLD.idpes_rev;
   v_data_rev   := OLD.data_rev;
   v_email    := OLD.email;
   v_situacao   := OLD.situacao;
   v_origem_gravacao  := OLD.origem_gravacao;
   v_idsis_rev    := OLD.idsis_rev;
   v_idsis_cad    := OLD.idsis_cad;
   v_operacao   := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA PESSOA
      INSERT INTO historico.pessoa
      (idpes, nome, idpes_cad, data_cad, url, tipo, idpes_rev, data_rev, email, situacao, origem_gravacao, idsis_rev, idsis_cad, operacao) VALUES
      (v_idpes, v_nome, v_idpes_cad, v_data_cad, v_url, v_tipo, v_idpes_rev, v_data_rev, v_email, v_situacao, v_origem_gravacao, v_idsis_rev, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


--
-- Name: fcn_grava_historico_socio(); Type: FUNCTION; Schema: historico; Owner: -
--

CREATE FUNCTION fcn_grava_historico_socio() RETURNS "trigger"
    AS $$
DECLARE
   v_idpes_juridica   numeric;
   v_idpes_fisica   numeric;
   v_idpes_rev      numeric;
   v_data_rev     TIMESTAMP;
   v_origem_gravacao    char(1);
   v_idsis_rev      numeric;
   v_idpes_cad      numeric;
   v_data_cad     TIMESTAMP;
   v_idsis_cad      numeric;
   v_operacao     char(1);
BEGIN
   v_idpes_juridica   := OLD.idpes_juridica;
   v_idpes_fisica   := OLD.idpes_fisica;
   v_idpes_rev      := OLD.idpes_rev;
   v_data_rev     := OLD.data_rev;
   v_origem_gravacao    := OLD.origem_gravacao;
   v_idsis_rev      := OLD.idsis_rev;
   v_idpes_cad      := OLD.idpes_cad;
   v_data_cad     := OLD.data_cad;
   v_idsis_cad      := OLD.idsis_cad;
   v_operacao     := OLD.operacao;

  IF v_data_rev IS NULL THEN
          v_data_rev := CURRENT_TIMESTAMP;
        END IF;

      -- GRAVA HISTRICO PARA TABELA SOCIO
      INSERT INTO historico.socio
      (idpes_juridica, idpes_fisica, idpes_rev, data_rev, origem_gravacao, idsis_rev, idpes_cad, data_cad, idsis_cad, operacao) VALUES
      (v_idpes_juridica, v_idpes_fisica, v_idpes_rev, v_data_rev, v_origem_gravacao, v_idsis_rev, v_idpes_cad, v_data_cad, v_idsis_cad, v_operacao);

   RETURN NEW;

END; $$
    LANGUAGE plpgsql;


SET search_path = pmieducar, pg_catalog;

--
-- Name: fcn_aft_update(); Type: FUNCTION; Schema: pmieducar; Owner: -
--

CREATE FUNCTION fcn_aft_update() RETURNS "trigger"
    AS $$
DECLARE
  nm_tabela   varchar(255);
  alteracoes    text;
  data_cadastro   TIMESTAMP;
  v_insercao    int2;


  BEGIN
    v_insercao    := 0;
    nm_tabela   := TG_RELNAME;
    alteracoes    := NEW;
    data_cadastro   := CURRENT_TIMESTAMP;
    IF TG_OP = 'INSERT' THEN
      v_insercao := 1;
    END IF;
    insert into pmieducar.historico_educar (tabela, alteracao, data, insercao) values (nm_tabela, alteracoes, data_cadastro, v_insercao);
  RETURN NEW;
END; $$
    LANGUAGE plpgsql;


SET search_path = public, pg_catalog;

--
-- Name: fcn_aft_logradouro_fonetiza(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_aft_logradouro_fonetiza() RETURNS "trigger"
    AS $$
   DECLARE
    v_idlog    bigint;
    v_nome_n   text;
    v_nome_v   text;
    v_fonema   text;
    v_reg_fon  record;
   BEGIN
    IF TG_OP = 'INSERT' THEN
     v_idlog  := NEW.idlog;
     v_nome_n := NEW.nome;
     FOR v_reg_fon IN SELECT DISTINCT * from public.fcn_fonetiza(v_nome_n) LOOP
      v_fonema := v_reg_fon.fcn_fonetiza;
      EXECUTE 'INSERT INTO public.logradouro_fonetico (fonema,idlog) VALUES ('||quote_literal(v_fonema)||','||quote_literal(v_idlog)||');';
     END LOOP;
    ELSIF TG_OP = 'UPDATE' THEN
     v_idlog  := NEW.idlog;
     v_nome_n := NEW.nome;
     v_nome_v := OLD.nome;
     IF v_nome_n <> v_nome_v THEN
      EXECUTE 'DELETE FROM public.logradouro_fonetico WHERE idlog = '||quote_literal(v_idlog)||';';
      FOR v_reg_fon IN SELECT DISTINCT * from public.fcn_fonetiza(v_nome_n) LOOP
       v_fonema := v_reg_fon.fcn_fonetiza;
       EXECUTE 'INSERT INTO public.logradouro_fonetico (fonema,idlog) VALUES ('||quote_literal(v_fonema)||','||quote_literal(v_idlog)||');';
      END LOOP;
     END IF;
    END IF;
    RETURN NEW;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_aft_pessoa_fonetiza(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_aft_pessoa_fonetiza() RETURNS "trigger"
    AS $$
   DECLARE
    v_idpes    bigint;
    v_nome_n   text;
    v_nome_v   text;
    v_fonema   text;
    v_reg_fon  record;
   BEGIN
    IF TG_OP = 'INSERT' THEN
     v_idpes  := NEW.idpes;
     v_nome_n := NEW.nome;
     FOR v_reg_fon IN SELECT DISTINCT * from public.fcn_fonetiza(v_nome_n) LOOP
      v_fonema := v_reg_fon.fcn_fonetiza;
      EXECUTE 'INSERT INTO cadastro.pessoa_fonetico (fonema,idpes) VALUES ('||quote_literal(v_fonema)||','||quote_literal(v_idpes)||');';
     END LOOP;
    ELSIF TG_OP = 'UPDATE' THEN
     v_idpes  := NEW.idpes;
     v_nome_n := NEW.nome;
     v_nome_v := OLD.nome;
     IF v_nome_n <> v_nome_v THEN
      EXECUTE 'DELETE FROM cadastro.pessoa_fonetico WHERE idpes = '||quote_literal(v_idpes)||';';
      FOR v_reg_fon IN SELECT DISTINCT * from public.fcn_fonetiza(v_nome_n) LOOP
       v_fonema := v_reg_fon.fcn_fonetiza;
       EXECUTE 'INSERT INTO cadastro.pessoa_fonetico (fonema,idpes) VALUES
 ('||quote_literal(v_fonema)||','||quote_literal(v_idpes)||');';
      END LOOP;
     END IF;
    END IF;
    RETURN NEW;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_bef_ins_fisica(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_bef_ins_fisica() RETURNS "trigger"
    AS $$
   DECLARE
    v_idpes    cadastro.fisica.idpes%TYPE;
    v_contador integer;
   BEGIN
    SELECT INTO v_contador count(idpes) from cadastro.juridica where idpes = NEW.idpes;
    IF v_contador = 1 THEN
     RAISE EXCEPTION 'O Identificador % j est cadastrado como Pessoa Jurdica', NEW.idpes;
    END IF;
    RETURN NEW;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_bef_ins_juridica(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_bef_ins_juridica() RETURNS "trigger"
    AS $$
   DECLARE
    v_idpes    cadastro.juridica.idpes%TYPE;
    v_contador integer;
   BEGIN
    SELECT INTO v_contador count(idpes) from cadastro.fisica where idpes = NEW.idpes;
    IF v_contador = 1 THEN
     RAISE EXCEPTION 'O Identificador % j est cadastrado como Pessoa Fsica', NEW.idpes;
    END IF;
    RETURN NEW;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_bef_logradouro_fonetiza(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_bef_logradouro_fonetiza() RETURNS "trigger"
    AS $$
   DECLARE
    v_idlog    bigint;
   BEGIN
    v_idlog := OLD.idlog;
    EXECUTE 'DELETE FROM public.logradouro_fonetico WHERE idlog = '||quote_literal(v_idlog)||';';
    RETURN OLD;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_bef_pessoa_fonetiza(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_bef_pessoa_fonetiza() RETURNS "trigger"
    AS $$
   DECLARE
    v_idpes    bigint;
   BEGIN
    v_idpes := OLD.idpes;
    EXECUTE 'DELETE FROM cadastro.pessoa_fonetico WHERE idpes = '||quote_literal(v_idpes)||';';
    RETURN OLD;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_compara_nome_pessoa_fonetica(text, numeric); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_compara_nome_pessoa_fonetica(text, numeric) RETURNS integer
    AS $_$
DECLARE
  v_nome_parametro      ALIAS FOR $1;
  v_idpes_parametro     ALIAS FOR $2;
  v_nome_pessoa_1       text;
  v_nome_pessoa_2       text;
  v_registro        record;
  v_nome_primeiro_ultimo_pessoa_1   text;
  v_nome_primeiro_ultimo_pessoa_2   text;
  v_cont     integer;
  v_fonema   text;
  v_comando  text;

  BEGIN

  -- obter o nome da pessoa referente ao IDPES passado como parametro
  v_comando := 'SELECT nome FROM cadastro.pessoa WHERE idpes = '||quote_literal(v_idpes_parametro)||';';
  FOR v_registro IN EXECUTE v_comando LOOP
    v_nome_pessoa_1 := v_registro.nome;
  END LOOP;

  v_nome_pessoa_2 := v_nome_parametro;

  v_nome_primeiro_ultimo_pessoa_1 := '';
  v_nome_primeiro_ultimo_pessoa_2 := '';
  v_cont := 0;

  -- primeiro e ltimo nome da pessoa com fontica
  FOR v_registro IN SELECT * FROM public.fcn_fonetiza(public.fcn_obter_primeiro_ultimo_nome(v_nome_pessoa_1)) LOOP
    v_cont := v_cont + 1;
    v_fonema := v_registro.fcn_fonetiza;

    IF v_cont > 1 THEN
      v_nome_primeiro_ultimo_pessoa_1 := v_nome_primeiro_ultimo_pessoa_1 || ' ';
    END IF;
    v_nome_primeiro_ultimo_pessoa_1 := v_nome_primeiro_ultimo_pessoa_1 || v_fonema;
  END LOOP;

  v_cont := 0;
  FOR v_registro IN SELECT * FROM public.fcn_fonetiza(public.fcn_obter_primeiro_ultimo_nome(v_nome_pessoa_2)) LOOP
    v_cont := v_cont + 1;
    v_fonema := v_registro.fcn_fonetiza;

    IF v_cont > 1 THEN
      v_nome_primeiro_ultimo_pessoa_2 := v_nome_primeiro_ultimo_pessoa_2 || ' ';
    END IF;
    v_nome_primeiro_ultimo_pessoa_2 := v_nome_primeiro_ultimo_pessoa_2 || v_fonema;
  END LOOP;

  IF v_nome_primeiro_ultimo_pessoa_1 = v_nome_primeiro_ultimo_pessoa_2 THEN
    RETURN 1;
  ELSE
    RETURN 0;
  END IF;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_cons_log_fonetica(text, bigint); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_cons_log_fonetica(text, bigint) RETURNS SETOF typ_idlog
    AS $_$
   DECLARE
    v_texto    ALIAS FOR $1;
    v_idmun    ALIAS FOR $2;
    v_fonema   text;
    v_comando  text;
    v_idlog    bigint;
    v_reg_fon  record;
    v_cont     integer;
    retorno    typ_idlog%ROWTYPE;
   BEGIN
    v_cont := 0;
    v_comando := 'select public.logradouro_fonetico.idlog from public.logradouro_fonetico, public.logradouro where (fonema = ';
    FOR v_reg_fon IN SELECT DISTINCT * FROM public.fcn_fonetiza(v_texto) LOOP v_cont := v_cont + 1;
     v_fonema := v_reg_fon.fcn_fonetiza;
     IF v_cont > 1 THEN
      v_comando := v_comando||' or fonema = ';
     END IF;
     v_comando := v_comando||quote_literal(v_fonema);
    END LOOP;
    v_comando := v_comando||') AND public.logradouro.idlog = public.logradouro_fonetico.idlog';
    v_comando := v_comando||' AND public.logradouro.idmun = '||v_idmun;
    v_comando := v_comando||' group by public.logradouro_fonetico.idlog having count(fonema) = '||quote_literal(v_cont)||';';
    FOR retorno IN EXECUTE v_comando LOOP
     RETURN NEXT retorno;
    END LOOP;
    RETURN;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_consulta_fonetica(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_consulta_fonetica(text) RETURNS SETOF typ_idpes
    AS $_$
   DECLARE
    v_texto    ALIAS FOR $1;
    v_fonema   text;
    v_comando  text;
    v_idpes    bigint;
    v_reg_fon  record;
    v_cont     integer;
    retorno typ_idpes%ROWTYPE;
   BEGIN
    v_cont := 0;
    v_comando := 'select idpes from cadastro.pessoa_fonetico where fonema = ';
    FOR v_reg_fon IN SELECT DISTINCT * FROM public.fcn_fonetiza(v_texto) LOOP
     v_cont := v_cont + 1;
     v_fonema := v_reg_fon.fcn_fonetiza;
     IF v_cont > 1 THEN
      v_comando := v_comando||' or fonema = ';
     END IF;
     v_comando := v_comando||quote_literal(v_fonema);
    END LOOP;
    v_comando := v_comando||' group by idpes having count(fonema) = '||quote_literal(v_cont)||';';
    FOR retorno IN EXECUTE v_comando LOOP
     RETURN NEXT retorno;
    END LOOP;
    RETURN;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_endereco_externo(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_delete_endereco_externo(integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_idpes ALIAS for $1;
  v_tipo ALIAS for $2;

BEGIN
  -- Deleta dados da tabela endereco_externo
  DELETE FROM cadastro.endereco_externo WHERE idpes = v_idpes AND tipo = v_tipo;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_endereco_pessoa(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_delete_endereco_pessoa(integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_idpes ALIAS for $1;
  v_tipo ALIAS for $2;

BEGIN
  -- Deleta dados da tabela endereco_pessoa
  DELETE FROM cadastro.endereco_pessoa WHERE idpes = v_idpes AND tipo = v_tipo;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_fone_pessoa(integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_delete_fone_pessoa(integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;

BEGIN
  -- Deleta dados da tabela fone_pessoa
  DELETE FROM cadastro.fone_pessoa WHERE idpes = v_id_pes;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_delete_funcionario(integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_delete_funcionario(integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_matricula ALIAS for $1;
  v_id_ins ALIAS for $2;

BEGIN
  -- Deleta dados da tabela funcionrio
  DELETE FROM cadastro.funcionario WHERE matricula = v_matricula AND idins = v_id_ins;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_dia_util(date, date); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_dia_util(date, date) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_dt_ini ALIAS for $1;
  v_dt_fim ALIAS for $2;
  v_dt_ini_x date;
  v_qtde integer;
BEGIN
  v_qtde := 0;
  v_dt_ini_x := v_dt_ini;
  WHILE v_dt_ini_x <= v_dt_fim LOOP
    IF to_char(v_dt_ini_x,'D') NOT IN ('1','7') THEN
      IF NOT EXISTS(SELECT idfer
             from servicos.feriado
             WHERE to_char(data,'DD/MM/YYYY') =
             to_char(v_dt_ini_x,'DD/MM/YYYY')) THEN
        v_qtde := v_qtde + 1;
      END IF;
    END IF;
    v_dt_ini_x := v_dt_ini_x + interval '1 day';
  END LOOP;
  RETURN v_qtde;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_fonetiza(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_fonetiza(text) RETURNS SETOF text
    AS $_$
   DECLARE
    v_array         ALIAS FOR $1;
    orig            text := '';
    dest            text := '';
    ori             integer := 1;
    v_campo         integer;
   BEGIN
    orig := public.fcn_upper_nrm(v_array)||'  ';
    FOR ori IN 1..152 LOOP
     IF substr(orig, ori, 1) = ' ' THEN
      IF substr(orig, ori + 1, 1) = ' ' THEN
       IF dest IN ('DA','DAS','DOS','DU','DUS','DI','E','S/A','LTDA','OTIDA','S.A','DO','DE','-','AVI','I','OA','A','SA') THEN
        dest := '';
       END IF;
       IF dest <> ' ' AND
          dest <> '' THEN
        RETURN NEXT trim(dest);
        dest := '';
       END IF;
       ori := 152;
       EXIT;
      ELSE
       IF dest IN ('DA','DAS','DOS','DU','DUS','DI','E','S/A','LTDA','OTIDA','S.A','DO','DE','-','AVI','I','OA','A','SA') THEN
        dest := '';
       END IF;
       IF dest <> ' ' AND
          dest <> '' THEN
        RETURN NEXT trim(dest);
        dest := '';
       END IF;
       ori := ori + 1;
      END IF;
     ELSIF substr(orig,ori + 1,1) = '-' OR
           substr(orig,ori + 1,1) = '/' THEN
      IF substr(orig,ori,1) <> ' ' AND
         substr(orig,ori + 2,1) <> ' ' THEN
       orig = substr(orig,1,ori)||' '||substr(orig,ori + 1,42);
      END IF;
     END IF;
     -- Numero
     IF substr(orig,ori,1) >= '0' and
        substr(orig,ori,1) <= '9' THEN
      dest := dest||substr(orig,ori,1);
     -- Letra Igual
     ELSIF substr(orig,ori,1) = substr(orig,ori + 1,1) THEN
      IF length(dest) = 1 THEN
       dest := '';
      END IF;
     -- Letras A, I ou O
     ELSIF substr(orig,ori,1) = 'A' OR
           substr(orig,ori,1) = 'I' OR
           substr(orig,ori,1) = 'O' THEN
      dest := dest||substr(orig,ori,1);
     -- Letra E
     ELSIF substr(orig,ori,1) = 'E' THEN
      dest := dest||'I';
     -- Letra R
     ELSIF substr(orig,ori,1) = 'R' THEN
       dest := dest||'H';
     -- Letra S
     ELSIF substr(orig,ori,1) = 'S' THEN
      IF substr(orig,ori + 1,1) NOT IN ('A','E','I','O','U','Y') AND
         length(dest) = 0 THEN
       dest := dest||'IS';
      ELSIF substr(orig,ori + 1,1) = 'C' AND
            substr(orig,ori + 2,1) = 'H' THEN
       IF length(dest) = 1 THEN
        dest := '';
       END IF;
      ELSIF substr(orig,ori + 1,1) = 'H' THEN
       dest := dest||'KS';
       ori := ori + 1;
      ELSE
       dest := dest||substr(orig,ori,1);
      END IF;
     -- Letra N
     ELSIF substr(orig,ori,1) = 'N' THEN
      IF substr(orig,ori + 1,1) = 'H' THEN
       dest := dest||'NI';
      ELSE
       IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||substr(orig,ori,1);
       ELSE
        dest := dest||'M';
       END IF;
      END IF;
     -- Letra L
     ELSIF substr(orig,ori,1) = 'L' THEN
      IF substr(orig,ori + 1,1) = 'H' THEN
       dest := dest||'LI';
      ELSIF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'O';
      END IF;
     -- Letra D
     ELSIF substr(orig,ori,1) = 'D' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'DI';
      END IF;
     -- Letra C
     ELSIF substr(orig,ori,1) = 'C' THEN
      IF substr(orig,ori + 1,1) = 'H' THEN
       IF substr(orig,ori + 2,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||'KS';
        ori := ori + 1;
       END IF;
      ELSIF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       IF substr(orig,ori + 1,1) IN ('E','I','Y') THEN
        dest := dest||'S';
       ELSE
        dest := dest||'K';
       END IF;
      ELSE
       IF length(dest) = 1 THEN
        dest := '';
       END IF;
      END IF;
     -- Letra M
     ELSIF substr(orig,ori,1) = 'M' THEN
      IF substr(orig,ori + 1,1) = 'N' THEN
       IF length(dest) = 1 THEN
        dest := '';
       END IF;
      ELSE
       dest := dest||substr(orig,ori,1);
      END IF;
     -- Letra T
     ELSIF substr(orig,ori,1) = 'T' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'TI';
      END IF;
     -- Letra U
     ELSIF substr(orig,ori,1) = 'U' THEN
      dest := dest||'O';
     -- Letra V
     ELSIF substr(orig,ori,1) = 'V' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'VI';
      END IF;
     -- Letra G
     ELSIF substr(orig,ori,1) = 'G' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       IF substr(orig,ori + 1,1) = 'U' AND
          substr(orig,ori + 2,1) IN ('I','E','Y') THEN
        dest := dest||'J';
        ori := ori + 1;
       ELSE
        dest := dest||'J';
       END IF;
      ELSE
       dest := dest||'JI';
      END IF;
     -- Letra B
     ELSIF substr(orig,ori,1) = 'B' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'BI';
      END IF;
     -- Letra P
     ELSIF substr(orig,ori,1) = 'P' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       IF substr(orig,ori + 1,1) = 'H' THEN
        dest := dest||'F';
       ELSE
        dest := dest||'PI';
       END IF;
      END IF;
     -- Letra Z
     ELSIF substr(orig,ori,1) = 'Z' THEN
       dest := dest||'S';
     -- Letra F
     ELSIF substr(orig,ori,1) = 'F' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'FI';
      END IF;
     -- Letra J
     ELSIF substr(orig,ori,1) = 'J' THEN
      dest := dest||'J';
     -- Letra K
     ELSIF substr(orig,ori,1) = 'K' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'KI';
      END IF;
     -- Letra Y
     ELSIF substr(orig,ori,1) = 'Y' THEN
      dest := dest||'I';
     -- Letra W
     ELSIF substr(orig,ori,1) = 'W' THEN
      IF ori = 1 THEN
       IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||'V';
       ELSE
        dest := dest||'VI';
       END IF;
      ELSIF substr(orig,ori - 1,1) IN ('E','I') THEN
       IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||'V';
       ELSE
        dest := dest||'O';
       END IF;
      ELSE
       dest := dest||'V';
      END IF;
     -- Letra Q
     ELSIF substr(orig,ori,1) = 'Q' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y',' ') THEN
       dest := dest||'K';
       IF substr(orig,ori + 1,1) = 'U' AND
          substr(orig,ori + 2,1) IN ('I','E','Y') THEN
        ori := ori + 1;
       END IF;
      ELSE
       dest := dest||'QI';
      END IF;
     -- Letra X
     ELSIF substr(orig,ori,1) = 'X' THEN
      dest := dest||'KS';
     END IF;
    END LOOP;
    dest := NULL;
    RETURN;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_fonetiza_logr_geral(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_fonetiza_logr_geral() RETURNS text
    AS $$
   DECLARE
    v_fonema   text;
    v_nomlog   text;
    v_idlog    bigint;
    v_reg_log  record;
    v_reg_fon  record;
    v_cont     integer;
   BEGIN
    FOR v_reg_log IN SELECT idlog, nome FROM public.logradouro LOOP
     v_nomlog := v_reg_log.nome;
     v_idlog  := v_reg_log.idlog;
     FOR v_reg_fon IN SELECT DISTINCT * FROM public.fcn_fonetiza(v_nomlog) LOOP
      v_fonema := v_reg_fon.fcn_fonetiza;
      EXECUTE 'INSERT INTO public.logradouro_fonetico (fonema,idlog) VALUES ('||quote_literal(v_fonema)||','||quote_literal(v_idlog)||');';
     END LOOP;
    END LOOP;
    SELECT count(idlog) INTO v_cont FROM public.logradouro_fonetico;
    v_fonema := 'Foram gravados '||to_char(v_cont,'9999999')||' registros em logradouro_fonetico';
    RETURN v_fonema;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_fonetiza_palavra(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_fonetiza_palavra(text) RETURNS text
    AS $_$
DECLARE
    v_array         ALIAS FOR $1;
    orig            text := '';
    dest            text := '';
    ori             integer := 1;
    v_campo         integer;
   BEGIN
    orig := public.fcn_upper_nrm(v_array)||'  ';
    FOR ori IN 1..152 LOOP
     IF substr(orig, ori, 1) = ' ' THEN
      IF substr(orig, ori + 1, 1) = ' ' THEN
       IF dest IN ('DA','DAS','DOS','DU','DUS','DI','E','S/A','LTDA','OTIDA','S.A','DO','DE','-','AVI','I','OA','A','SA') THEN
        dest := '';
       END IF;
       IF dest <> ' ' AND
          dest <> '' THEN
        RETURN trim(dest);
        dest := '';
       END IF;
       ori := 152;
       EXIT;
      ELSE
       IF dest IN ('DA','DAS','DOS','DU','DUS','DI','E','S/A','LTDA','OTIDA','S.A','DO','DE','-','AVI','I','OA','A','SA') THEN
        dest := '';
       END IF;
       IF dest <> ' ' AND
          dest <> '' THEN
        RETURN trim(dest);
        dest := '';
       END IF;
       ori := ori + 1;
      END IF;
     ELSIF substr(orig,ori + 1,1) = '-' OR
           substr(orig,ori + 1,1) = '/' THEN
      IF substr(orig,ori,1) <> ' ' AND
         substr(orig,ori + 2,1) <> ' ' THEN
       orig = substr(orig,1,ori)||' '||substr(orig,ori + 1,42);
      END IF;
     END IF;
     -- Numero
     IF substr(orig,ori,1) >= '0' and
        substr(orig,ori,1) <= '9' THEN
      dest := dest||substr(orig,ori,1);
     -- Letra Igual
     ELSIF substr(orig,ori,1) = substr(orig,ori + 1,1) THEN
      IF length(dest) = 1 THEN
       dest := '';
      END IF;
     -- Letras A, I ou O
     ELSIF substr(orig,ori,1) = 'A' OR
           substr(orig,ori,1) = 'I' OR
           substr(orig,ori,1) = 'O' THEN
      dest := dest||substr(orig,ori,1);
     -- Letra E
     ELSIF substr(orig,ori,1) = 'E' THEN
      dest := dest||'I';
     -- Letra R
     ELSIF substr(orig,ori,1) = 'R' THEN
       dest := dest||'H';
     -- Letra S
     ELSIF substr(orig,ori,1) = 'S' THEN
      IF substr(orig,ori + 1,1) NOT IN ('A','E','I','O','U','Y') AND
         length(dest) = 0 THEN
       dest := dest||'IS';
      ELSIF substr(orig,ori + 1,1) = 'C' AND
            substr(orig,ori + 2,1) = 'H' THEN
       IF length(dest) = 1 THEN
        dest := '';
       END IF;
      ELSIF substr(orig,ori + 1,1) = 'H' THEN
       dest := dest||'KS';
       ori := ori + 1;
      ELSE
       dest := dest||substr(orig,ori,1);
      END IF;
     -- Letra N
     ELSIF substr(orig,ori,1) = 'N' THEN
      IF substr(orig,ori + 1,1) = 'H' THEN
       dest := dest||'NI';
      ELSE
       IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||substr(orig,ori,1);
       ELSE
        dest := dest||'M';
       END IF;
      END IF;
     -- Letra L
     ELSIF substr(orig,ori,1) = 'L' THEN
      IF substr(orig,ori + 1,1) = 'H' THEN
       dest := dest||'LI';
      ELSIF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'O';
      END IF;
     -- Letra D
     ELSIF substr(orig,ori,1) = 'D' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'DI';
      END IF;
     -- Letra C
     ELSIF substr(orig,ori,1) = 'C' THEN
      IF substr(orig,ori + 1,1) = 'H' THEN
       IF substr(orig,ori + 2,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||'KS';
        ori := ori + 1;
       END IF;
      ELSIF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       IF substr(orig,ori + 1,1) IN ('E','I','Y') THEN
        dest := dest||'S';
       ELSE
        dest := dest||'K';
       END IF;
      ELSE
       IF length(dest) = 1 THEN
        dest := '';
       END IF;
      END IF;
     -- Letra M
     ELSIF substr(orig,ori,1) = 'M' THEN
      IF substr(orig,ori + 1,1) = 'N' THEN
       IF length(dest) = 1 THEN
        dest := '';
       END IF;
      ELSE
       dest := dest||substr(orig,ori,1);
      END IF;
     -- Letra T
     ELSIF substr(orig,ori,1) = 'T' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'TI';
      END IF;
     -- Letra U
     ELSIF substr(orig,ori,1) = 'U' THEN
      dest := dest||'O';
     -- Letra V
     ELSIF substr(orig,ori,1) = 'V' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'VI';
      END IF;
     -- Letra G
     ELSIF substr(orig,ori,1) = 'G' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       IF substr(orig,ori + 1,1) = 'U' AND
          substr(orig,ori + 2,1) IN ('I','E','Y') THEN
        dest := dest||'J';
        ori := ori + 1;
       ELSE
        dest := dest||'J';
       END IF;
      ELSE
       dest := dest||'JI';
      END IF;
     -- Letra B
     ELSIF substr(orig,ori,1) = 'B' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'BI';
      END IF;
     -- Letra P
     ELSIF substr(orig,ori,1) = 'P' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       IF substr(orig,ori + 1,1) = 'H' THEN
        dest := dest||'F';
       ELSE
        dest := dest||'PI';
       END IF;
      END IF;
     -- Letra Z
     ELSIF substr(orig,ori,1) = 'Z' THEN
       dest := dest||'S';
     -- Letra F
     ELSIF substr(orig,ori,1) = 'F' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'FI';
      END IF;
     -- Letra J
     ELSIF substr(orig,ori,1) = 'J' THEN
      dest := dest||'J';
     -- Letra K
     ELSIF substr(orig,ori,1) = 'K' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') OR
         (substr(orig,ori + 1,1) = 'H' AND
          substr(orig,ori + 2,1) <> ' ') THEN
       dest := dest||substr(orig,ori,1);
      ELSE
       dest := dest||'KI';
      END IF;
     -- Letra Y
     ELSIF substr(orig,ori,1) = 'Y' THEN
      dest := dest||'I';
     -- Letra W
     ELSIF substr(orig,ori,1) = 'W' THEN
      IF ori = 1 THEN
       IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||'V';
       ELSE
        dest := dest||'VI';
       END IF;
      ELSIF substr(orig,ori - 1,1) IN ('E','I') THEN
       IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y') THEN
        dest := dest||'V';
       ELSE
        dest := dest||'O';
       END IF;
      ELSE
       dest := dest||'V';
      END IF;
     -- Letra Q
     ELSIF substr(orig,ori,1) = 'Q' THEN
      IF substr(orig,ori + 1,1) IN ('A','E','I','O','U','Y',' ') THEN
       dest := dest||'K';
       IF substr(orig,ori + 1,1) = 'U' AND
          substr(orig,ori + 2,1) IN ('I','E','Y') THEN
        ori := ori + 1;
       END IF;
      ELSE
       dest := dest||'QI';
      END IF;
     -- Letra X
     ELSIF substr(orig,ori,1) = 'X' THEN
      dest := dest||'KS';
     END IF;
    END LOOP;
    dest := NULL;
    RETURN;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_fonetiza_pessoa_geral(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_fonetiza_pessoa_geral() RETURNS text
    AS $$
   DECLARE
    v_fonema   text;
    v_nome     text;
    v_idpes    bigint;
    v_reg_pes  record;
    v_reg_fon  record;
    v_cont     integer;
   BEGIN
    FOR v_reg_pes IN SELECT idpes, nome FROM cadastro.pessoa LOOP
     v_nome  := v_reg_pes.nome;
     v_idpes := v_reg_pes.idpes;
     FOR v_reg_fon IN SELECT DISTINCT * FROM public.fcn_fonetiza(v_nome) LOOP
      v_fonema := v_reg_fon.fcn_fonetiza;
      EXECUTE 'INSERT INTO cadastro.pessoa_fonetico (fonema,idpes) VALUES ('||quote_literal(v_fonema)||','||quote_literal(v_idpes)||');';
     END LOOP;
    END LOOP;
    SELECT count(idpes) INTO v_cont FROM cadastro.pessoa_fonetico;
    v_fonema := 'Foram gravados '||to_char(v_cont,'999999')||' registros em pessoa_fonetico';
    RETURN v_fonema;
   END;
  $$
    LANGUAGE plpgsql;


--
-- Name: fcn_fonetiza_primeiro_ultimo_nome(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_fonetiza_primeiro_ultimo_nome(text) RETURNS text
    AS $_$
DECLARE
  v_nome_parametro    ALIAS FOR $1;
  v_registro      record;
  v_nome_primeiro_ultimo_pessoa text;
  v_cont          integer;
  v_fonema        text;
  BEGIN
  v_nome_primeiro_ultimo_pessoa := '';
  v_cont := 0;
  -- primeiro e ltimo nome da pessoa com fontica
  FOR v_registro IN SELECT * FROM public.fcn_fonetiza(public.fcn_obter_primeiro_ultimo_nome(v_nome_parametro)) LOOP
    v_cont := v_cont + 1;
    v_fonema := v_registro.fcn_fonetiza;
    IF v_cont > 1 THEN
      v_nome_primeiro_ultimo_pessoa := v_nome_primeiro_ultimo_pessoa || ' ';
    END IF;
    v_nome_primeiro_ultimo_pessoa := v_nome_primeiro_ultimo_pessoa || v_fonema;
  END LOOP;
  RETURN v_nome_primeiro_ultimo_pessoa;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_documento(integer, character varying, character varying, character varying, character varying, integer, integer, integer, integer, character varying, character varying, character varying, character varying, integer, integer, character varying, character varying, integer, integer, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_documento(integer, character varying, character varying, character varying, character varying, integer, integer, integer, integer, character varying, character varying, character varying, character varying, integer, integer, character varying, character varying, integer, integer, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_rg ALIAS for $2;
  v_orgao_exp_rg ALIAS for $3;
  v_data_exp_rg ALIAS for $4;
  v_sigla_uf_exp_rg ALIAS for $5;
      v_tipo_cert_civil ALIAS for $6;
      v_num_termo ALIAS for $7;
  v_num_livro ALIAS for $8;
  v_num_folha ALIAS for $9;
      v_data_emissao_cert_civil ALIAS for $10;
      v_sigla_uf_cert_civil ALIAS for $11;
  v_sigla_uf_cart_trabalho ALIAS for $12;
  v_cartorio_cert_civil ALIAS for $13;
  v_num_cart_trabalho ALIAS for $14;
      v_serie_cart_trabalho ALIAS for $15;
      v_data_emissao_cart_trabalho ALIAS for $16;
  v_num_tit_eleitor ALIAS for $17;
  v_zona_tit_eleitor ALIAS for $18;
  v_secao_tit_eleitor ALIAS for $19;
  v_origem_gravacao ALIAS for $20;
  v_idpes_cad ALIAS for $21;
  v_idsis_cad ALIAS for $22;

    -- Outras variveis
      v_rg_aux varchar(10);
      v_orgao_exp_rg_aux varchar(10);
      v_sigla_uf_exp_rg_aux varchar(2);
  v_tipo_cert_civil_aux integer;
  v_num_termo_aux integer;
  v_num_livro_aux integer;
  v_num_folha_aux integer;
      v_sigla_uf_cert_civil_aux varchar(2);
  v_sigla_uf_cart_trabalho_aux varchar(2);
      v_cartorio_cert_civil_aux varchar(150);
      v_num_cart_trabalho_aux integer;
      v_serie_cart_trabalho_aux integer;
      v_num_tit_eleitor_aux varchar(13);
      v_zona_tit_eleitor_aux integer;
      v_secao_tit_eleitor_aux integer;
BEGIN
    IF v_rg = '' THEN
        v_rg_aux := NULL;
    ELSE
        v_rg_aux := v_rg;
    END IF;
    IF v_orgao_exp_rg = '' THEN
        v_orgao_exp_rg_aux := NULL;
    ELSE
        v_orgao_exp_rg_aux := v_orgao_exp_rg;
    END IF;
    IF v_sigla_uf_exp_rg = '' THEN
        v_sigla_uf_exp_rg_aux := NULL;
    ELSE
        v_sigla_uf_exp_rg_aux := v_sigla_uf_exp_rg;
    END IF;
    IF v_tipo_cert_civil = 0 THEN
        v_tipo_cert_civil_aux := NULL;
    ELSE
        v_tipo_cert_civil_aux := v_tipo_cert_civil;
    END IF;
    IF v_num_termo = 0 THEN
        v_num_termo_aux := NULL;
    ELSE
        v_num_termo_aux := v_num_termo;
    END IF;
    IF v_num_livro = 0 THEN
        v_num_livro_aux := NULL;
    ELSE
        v_num_livro_aux := v_num_livro;
    END IF;
    IF v_num_folha = 0 THEN
        v_num_folha_aux := NULL;
    ELSE
        v_num_folha_aux := v_num_folha;
    END IF;
    IF v_sigla_uf_cert_civil = '' THEN
        v_sigla_uf_cert_civil_aux := NULL;
    ELSE
        v_sigla_uf_cert_civil_aux := v_sigla_uf_cert_civil;
    END IF;
    IF v_sigla_uf_cart_trabalho = '' THEN
        v_sigla_uf_cart_trabalho_aux := NULL;
    ELSE
        v_sigla_uf_cart_trabalho_aux := v_sigla_uf_cart_trabalho;
    END IF;
    IF v_cartorio_cert_civil = '' THEN
        v_cartorio_cert_civil_aux := NULL;
    ELSE
        v_cartorio_cert_civil_aux := v_cartorio_cert_civil;
    END IF;
    IF v_num_cart_trabalho = 0 THEN
        v_num_cart_trabalho_aux := NULL;
    ELSE
        v_num_cart_trabalho_aux := v_num_cart_trabalho;
    END IF;
    IF v_serie_cart_trabalho = 0 THEN
        v_serie_cart_trabalho_aux := NULL;
    ELSE
        v_serie_cart_trabalho_aux := v_serie_cart_trabalho;
    END IF;
    IF v_num_tit_eleitor = '' THEN
        v_num_tit_eleitor_aux := NULL;
    ELSE
        v_num_tit_eleitor_aux := v_num_tit_eleitor;
    END IF;
    IF v_zona_tit_eleitor = 0 THEN
        v_zona_tit_eleitor_aux := NULL;
    ELSE
        v_zona_tit_eleitor_aux := v_zona_tit_eleitor;
    END IF;
    IF v_secao_tit_eleitor = 0 THEN
        v_secao_tit_eleitor_aux := NULL;
    ELSE
        v_secao_tit_eleitor_aux := v_secao_tit_eleitor;
    END IF;

  -- Insere dados na tabela funcionrio
    INSERT INTO cadastro.documento (idpes, rg, idorg_exp_rg,
                                data_exp_rg, sigla_uf_exp_rg,
                                tipo_cert_civil, num_termo,
                                num_livro, num_folha,
                                data_emissao_cert_civil, sigla_uf_cert_civil,
                                sigla_uf_cart_trabalho, cartorio_cert_civil,
                                num_cart_trabalho, serie_cart_trabalho,
                                data_emissao_cart_trabalho, num_tit_eleitor,                                              zona_tit_eleitor, secao_tit_eleitor, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
    VALUES(v_id_pes, to_number(v_rg_aux,9999999999), to_number(v_orgao_exp_rg_aux,9999999999),
           to_date(v_data_exp_rg,'DD/MM/YYYY'), v_sigla_uf_exp_rg_aux,
           v_tipo_cert_civil_aux, v_num_termo_aux,
           v_num_livro_aux, v_num_folha_aux,
           to_date(v_data_emissao_cert_civil,'DD/MM/YYYY'), v_sigla_uf_cert_civil_aux,
           v_sigla_uf_cart_trabalho_aux, v_cartorio_cert_civil_aux,
           v_num_cart_trabalho_aux, v_serie_cart_trabalho_aux,
           to_date(v_data_emissao_cart_trabalho,'DD/MM/YYYY'), to_number(v_num_tit_eleitor_aux,9999999999999),
           v_zona_tit_eleitor_aux, v_secao_tit_eleitor_aux, v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_endereco_externo(integer, integer, character varying, character varying, character varying, integer, character varying, character varying, character varying, integer, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_endereco_externo(integer, integer, character varying, character varying, character varying, integer, character varying, character varying, character varying, integer, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_tipo ALIAS for $2;
  v_sigla_uf ALIAS for $3;
  v_idtlog ALIAS for $4;
    v_logradouro ALIAS for $5;
    v_numero ALIAS for $6;
    v_letra ALIAS for $7;
    v_complemento ALIAS for $8;
    v_bairro ALIAS for $9;
    v_cep ALIAS for $10;
    v_cidade ALIAS for $11;
    v_reside_desde ALIAS for $12;
    v_origem_gravacao ALIAS for $13;
    v_idpes_cad ALIAS for $14;
    v_idsis_cad ALIAS for $15;

BEGIN
  -- Atualiza dados na tabela endereco_externo
    INSERT INTO cadastro.endereco_externo(idpes, tipo, sigla_uf, idtlog, logradouro, numero, letra, complemento, bairro, cep, cidade, reside_desde, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
    VALUES (v_id_pes, v_tipo, v_sigla_uf, v_idtlog, public.fcn_upper(v_logradouro), v_numero, public.fcn_upper(v_letra), public.fcn_upper(v_complemento), public.fcn_upper(v_bairro), v_cep, public.fcn_upper(v_cidade), to_date(v_reside_desde,'DD/MM/YYYY'), v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');

  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_endereco_pessoa(integer, integer, integer, integer, integer, integer, character varying, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_endereco_pessoa(integer, integer, integer, integer, integer, integer, character varying, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_tipo ALIAS for $2;
  v_cep ALIAS for $3;
  v_idlog ALIAS for $4;
  v_idbai ALIAS for $5;
  v_numero ALIAS for $6;
  v_letra ALIAS for $7;
  v_complemento ALIAS for $8;
  v_reside_desde ALIAS for $9;
  v_origem_gravacao ALIAS for $10;
      v_idpes_cad ALIAS for $11;
      v_idsis_cad ALIAS for $12;
BEGIN
  -- Insere dados na tabela endereco_pessoa
    INSERT INTO cadastro.endereco_pessoa (idpes,tipo,cep,idlog,idbai,numero,letra,complemento,reside_desde, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
  VALUES(v_id_pes,v_tipo,v_cep,v_idlog,v_idbai,v_numero,public.fcn_upper(v_letra),public.fcn_upper(v_complemento),to_date(v_reside_desde,'DD/MM/YYYY'), v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_fisica(integer, character varying, character varying, integer, integer, integer, integer, integer, integer, character varying, character varying, integer, integer, character varying, integer, character varying, integer, character varying, character varying, character varying, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_fisica(integer, character varying, character varying, integer, integer, integer, integer, integer, integer, character varying, character varying, integer, integer, character varying, integer, character varying, integer, character varying, character varying, character varying, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_data_nasc ALIAS for $2;
  v_sexo ALIAS for $3;
  v_id_pes_mae ALIAS for $4;
      v_id_pes_pai ALIAS for $5;
      v_id_pes_responsavel ALIAS for $6;
  v_id_esco ALIAS for $7;
  v_id_eciv ALIAS for $8;
  v_id_pes_con ALIAS for $9;
      v_data_uniao ALIAS for $10;
      v_data_obito ALIAS for $11;
  v_nacionalidade ALIAS for $12;
  v_id_pais_estrangeiro ALIAS for $13;
  v_data_chegada ALIAS for $14;
      v_id_mun_nascimento ALIAS for $15;
      v_ultima_empresa ALIAS for $16;
  v_id_ocup ALIAS for $17;
  v_nome_mae ALIAS for $18;
  v_nome_pai ALIAS for $19;
      v_nome_conjuge ALIAS for $20;
      v_nome_responsavel ALIAS for $21;
      v_justificativa_provisorio ALIAS for $22;
  v_origem_gravacao ALIAS for $23;
      v_idpes_cad ALIAS for $24;
      v_idsis_cad ALIAS for $25;
      -- Outras variveis
      v_id_pes_mae_aux integer;
      v_id_pes_pai_aux integer;
      v_id_pes_responsavel_aux integer;
  v_id_esco_aux integer;
  v_id_eciv_aux integer;
  v_id_pes_con_aux integer;
      v_nacionalidade_aux integer;
  v_id_pais_estrangeiro_aux integer;
      v_id_mun_nascimento_aux integer;
      v_id_ocup_aux integer;
      v_sexo_aux text;
BEGIN
    IF v_id_pes_mae = 0 THEN
        v_id_pes_mae_aux := NULL;
    ELSE
        v_id_pes_mae_aux := v_id_pes_mae;
    END IF;
    IF v_id_pes_pai = 0 THEN
        v_id_pes_pai_aux := NULL;
    ELSE
        v_id_pes_pai_aux := v_id_pes_pai;
    END IF;
    IF v_id_pes_responsavel = 0 THEN
        v_id_pes_responsavel_aux := NULL;
    ELSE
        v_id_pes_responsavel_aux := v_id_pes_responsavel;
    END IF;
    IF v_id_esco = 0 THEN
        v_id_esco_aux := NULL;
    ELSE
        v_id_esco_aux := v_id_esco;
    END IF;
    IF v_id_eciv = 0 THEN
        v_id_eciv_aux := NULL;
    ELSE
        v_id_eciv_aux := v_id_eciv;
    END IF;
    IF v_id_pes_con = 0 THEN
        v_id_pes_con_aux := NULL;
    ELSE
        v_id_pes_con_aux := v_id_pes_con;
    END IF;
    IF v_nacionalidade = 0 THEN
        v_nacionalidade_aux := NULL;
    ELSE
        v_nacionalidade_aux := v_nacionalidade;
    END IF;
    IF v_id_pais_estrangeiro = 0 THEN
        v_id_pais_estrangeiro_aux := NULL;
    ELSE
        v_id_pais_estrangeiro_aux := v_id_pais_estrangeiro;
    END IF;
    IF v_id_mun_nascimento = 0 THEN
        v_id_mun_nascimento_aux := NULL;
    ELSE
        v_id_mun_nascimento_aux := v_id_mun_nascimento;
    END IF;
    IF v_id_ocup = 0 THEN
        v_id_ocup_aux := NULL;
    ELSE
        v_id_ocup_aux := v_id_ocup;
    END IF;
    IF TRIM(v_sexo) = '' THEN
        v_sexo_aux := NULL;
    ELSE
        v_sexo_aux := public.fcn_upper(v_sexo);
    END IF;
  -- Insere dados na tabela funcionrio
    INSERT INTO cadastro.fisica (idpes, data_nasc, sexo,
                                idpes_mae,idpes_pai, idpes_responsavel,
                                idesco, ideciv, idpes_con,
                                data_uniao, data_obito, nacionalidade,
                                idpais_estrangeiro, data_chegada_brasil, idmun_nascimento,
                                ultima_empresa, idocup, nome_mae,
                                nome_pai, nome_conjuge,
                                nome_responsavel, justificativa_provisorio, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
    VALUES(v_id_pes,to_date(v_data_nasc,'DD/MM/YYYY'),v_sexo_aux,
           v_id_pes_mae_aux, v_id_pes_pai_aux, v_id_pes_responsavel_aux,
           v_id_esco_aux, v_id_eciv_aux, v_id_pes_con_aux,
           to_date(v_data_uniao,'DD/MM/YYYY'), to_date(v_data_obito,'DD/MM/YYYY'), v_nacionalidade_aux,
           v_id_pais_estrangeiro_aux, to_date(v_data_chegada,'DD/MM/YYYY'), v_id_mun_nascimento_aux,             public.fcn_upper(v_ultima_empresa), v_id_ocup_aux, public.fcn_upper(v_nome_mae),
           public.fcn_upper(v_nome_pai), public.fcn_upper(v_nome_conjuge),
           public.fcn_upper(v_nome_responsavel), v_justificativa_provisorio, v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');

  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_fisica_cpf(integer, text, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_fisica_cpf(integer, text, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_cpf ALIAS for $2;
  v_origem_gravacao ALIAS for $3;
      v_idpes_cad ALIAS for $4;
      v_idsis_cad ALIAS for $5;
BEGIN
  INSERT INTO cadastro.fisica_cpf (idpes,cpf, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao) VALUES (v_id_pes,to_number(v_cpf,99999999999), v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_fone_pessoa(integer, integer, integer, integer, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_fone_pessoa(integer, integer, integer, integer, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_tipo ALIAS for $2;
  v_ddd ALIAS for $3;
  v_fone ALIAS for $4;
  v_origem_gravacao ALIAS for $5;
      v_idpes_cad ALIAS for $6;
      v_idsis_cad ALIAS for $7;
BEGIN
  -- Insere dados na tabela fone_pessoa
  INSERT INTO cadastro.fone_pessoa (idpes,tipo,ddd,fone, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
    VALUES (v_id_pes,v_tipo,v_ddd,v_fone, v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_funcionario(integer, integer, integer, integer, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_funcionario(integer, integer, integer, integer, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_matricula ALIAS for $1;
  v_id_ins ALIAS for $2;
  v_id_set ALIAS for $3;
  v_id_pes ALIAS for $4;
      v_situacao ALIAS for $5;
  v_origem_gravacao ALIAS for $6;
      v_idpes_cad ALIAS for $7;
      v_idsis_cad ALIAS for $8;
    -- Outras variveis
    v_id_set_aux integer;
BEGIN
    IF v_id_set = 0 THEN
        v_id_set_aux := NULL;
    ELSE
        v_id_set_aux := v_id_set;
    END IF;
  -- Insere dados na tabela funcionrio
    INSERT INTO cadastro.funcionario (matricula,idins,idset,idpes,situacao, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
    VALUES(v_matricula,v_id_ins,v_id_set_aux,v_id_pes,public.fcn_upper(v_situacao), v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_juridica(integer, character varying, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_juridica(integer, character varying, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_cnpj ALIAS for $2;
  v_fantasia ALIAS for $3;
  v_inscr_estadual ALIAS for $4;
  v_origem_gravacao ALIAS for $5;
      v_idpes_cad ALIAS for $6;
      v_idsis_cad ALIAS for $7;

BEGIN
  -- Insere dados na tabela juridica
    INSERT INTO cadastro.juridica (idpes,cnpj,fantasia,insc_estadual, origem_gravacao, idpes_cad, idsis_cad, data_cad, operacao)
    VALUES (v_id_pes,to_number(v_cnpj,99999999999999),public.fcn_upper(v_fantasia),to_number(v_inscr_estadual,9999999999), v_origem_gravacao, v_idpes_cad, v_idsis_cad, CURRENT_TIMESTAMP, 'I');
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_insert_pessoa(integer, character varying, character varying, character varying, character varying, integer, character varying, character varying, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_insert_pessoa(integer, character varying, character varying, character varying, character varying, integer, character varying, character varying, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_razao_social ALIAS for $2;
  v_url ALIAS for $3;
  v_email ALIAS for $4;
  v_situacao ALIAS for $5;
  v_id_pes_logado ALIAS for $6;
  v_tipo ALIAS for $7;
  v_origem_gravacao ALIAS for $8;
      v_idpes_cad ALIAS for $9;
      v_idsis_cad ALIAS for $10;

  idpes_logado integer;
BEGIN
  idpes_logado := v_id_pes_logado;
  IF (idpes_logado <= 0) THEN
    INSERT INTO cadastro.pessoa (idpes,nome,idpes_cad,data_cad,url,tipo,email,situacao,origem_gravacao, idsis_cad, operacao) VALUES (v_id_pes,public.fcn_upper(v_razao_social),NULL,CURRENT_TIMESTAMP,v_url,v_tipo,v_email,v_situacao,v_origem_gravacao, v_idsis_cad, 'I');
  ELSE
    INSERT INTO cadastro.pessoa (idpes,nome,idpes_cad,data_cad,url,tipo,email,situacao,origem_gravacao, idsis_cad, operacao) VALUES (v_id_pes,public.fcn_upper(v_razao_social),idpes_logado,CURRENT_TIMESTAMP,v_url,v_tipo,v_email,v_situacao,v_origem_gravacao, v_idsis_cad, 'I');
  END IF;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_obter_primeiro_ultimo_nome(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_obter_primeiro_ultimo_nome(text) RETURNS text
    AS $_$
DECLARE
  v_nome_parametro    ALIAS FOR $1;
  v_nome        text;
  v_primeiro_nome     text;
  v_ultimo_nome     text;
  v_posicao_espaco_primeiro_nome  integer;
  v_posicao_espaco_ultimo_nome  integer;
  v_cont        integer;
  v_fonema_junior     text;
  v_fonema_sobrinho   text;
  v_fonema_sobrinha   text;
  v_fonema_filho      text;
  v_fonema_filha      text;
  v_fonema_ultimo_nome    text;
  v_reg       record;
  v_total_caracteres    integer;
  BEGIN
  v_primeiro_nome := '';
  v_ultimo_nome := '';
  v_nome := TRIM(v_nome_parametro);
  v_total_caracteres := LENGTH(v_nome);
  -- obter somente o primeiro e o ltimo nome das pessoas
  IF v_total_caracteres > 0 THEN
    -- retirar os espaos duplicados
    WHILE POSITION('  ' IN v_nome) > 0 LOOP
      v_nome := REPLACE(UPPER(v_nome),'  ', ' ');
    END LOOP;
    -- retirar ocorrncias que devem ser ignoradas no nome
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUTROS', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUTRAS', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUTRO', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUTRA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUTRS', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OTS', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUTA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OUT', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E SM', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E S/M', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E OT', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' S/M', ''));
    v_total_caracteres := LENGTH(v_nome);
    IF v_total_caracteres = (POSITION(' OUTRO' IN v_nome) + 5) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUTRO', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OUTRA' IN v_nome) + 5) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUTRA', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OUTROS' IN v_nome) + 6) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUTROS', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OUTRAS' IN v_nome) + 6) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUTRAS', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OTS' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OTS', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OUTA' IN v_nome) + 4) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUTA', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OUT' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUT', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' OUTRS' IN v_nome) + 5) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' OUTRS', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' SM' IN v_nome) + 2) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' SM', ''));
    END IF;
    v_nome := TRIM(v_nome);
    -- posio do espao em branco para obter o primeiro nome
    v_posicao_espaco_primeiro_nome := POSITION(' ' IN v_nome);
    IF v_posicao_espaco_primeiro_nome > 0 THEN
      v_primeiro_nome := SUBSTR(v_nome, 1, (v_posicao_espaco_primeiro_nome - 1));
    ELSE
      v_primeiro_nome := v_nome;
    END IF;
    -- obter o ltimo nome
    v_posicao_espaco_ultimo_nome := 0;
    IF v_posicao_espaco_primeiro_nome > 0 THEN
      v_cont := v_posicao_espaco_ultimo_nome + 1;
      -- obter posio do espao em branco anterior ao ltimo nome
      WHILE v_cont < LENGTH(v_nome) LOOP
        IF SUBSTR(v_nome, v_cont, 1) = ' ' THEN
          v_posicao_espaco_ultimo_nome = v_cont;
        END IF;
        v_cont := v_cont + 1;
      END LOOP;
      v_ultimo_nome := SUBSTR(v_nome, (v_posicao_espaco_ultimo_nome + 1));
      -- fonema do ltimo nome
      FOR v_reg IN SELECT * FROM public.fcn_fonetiza(v_ultimo_nome) LOOP
        v_fonema_ultimo_nome := v_reg.fcn_fonetiza;
      END LOOP;
      -- verificar se o ltimo nome termina com Junior, Sobrinho ou Filho e outros
      FOR v_reg IN SELECT * FROM public.fcn_fonetiza('junior') LOOP
        v_fonema_junior := v_reg.fcn_fonetiza;
      END LOOP;
      FOR v_reg IN SELECT * FROM public.fcn_fonetiza('sobrinho') LOOP
        v_fonema_sobrinho := v_reg.fcn_fonetiza;
      END LOOP;
      FOR v_reg IN SELECT * FROM public.fcn_fonetiza('filho') LOOP
        v_fonema_filho := v_reg.fcn_fonetiza;
      END LOOP;
      FOR v_reg IN SELECT * FROM public.fcn_fonetiza('filha') LOOP
        v_fonema_filha := v_reg.fcn_fonetiza;
      END LOOP;
      FOR v_reg IN SELECT * FROM public.fcn_fonetiza('sobrinha') LOOP
        v_fonema_sobrinha := v_reg.fcn_fonetiza;
      END LOOP;
      IF v_fonema_ultimo_nome = v_fonema_junior OR
                          v_fonema_ultimo_nome = v_fonema_sobrinho OR
                          v_fonema_ultimo_nome = v_fonema_filho OR
                          v_fonema_ultimo_nome = v_fonema_filha OR
                          v_fonema_ultimo_nome = v_fonema_sobrinha
                        THEN
        v_nome := TRIM(SUBSTR(v_nome, 1, (LENGTH(v_nome) - LENGTH(v_ultimo_nome) - 1)));
        v_primeiro_nome := '';
        v_ultimo_nome := '';
        -- obter novamente o primeiro nome
        v_posicao_espaco_primeiro_nome := POSITION(' ' IN v_nome);
        IF v_posicao_espaco_primeiro_nome > 0 THEN
          v_primeiro_nome := SUBSTR(v_nome, 1, (v_posicao_espaco_primeiro_nome - 1));
        END IF;

        -- obter o penultimo nome
        IF v_posicao_espaco_primeiro_nome > 0 THEN
          v_posicao_espaco_ultimo_nome := 0;
          v_cont := 1;

          -- obter posio do espao em branco anterior ao ltimo nome
          WHILE v_cont < LENGTH(v_nome) LOOP
          IF SUBSTR(v_nome, v_cont, 1) = ' ' THEN
            v_posicao_espaco_ultimo_nome = v_cont;
          END IF;
          v_cont := v_cont + 1;
          END LOOP;
          v_ultimo_nome := SUBSTR(v_nome, (v_posicao_espaco_ultimo_nome + 1));
        END IF;
      END IF;
    END IF;
  END IF;
  RETURN v_primeiro_nome || ' ' || v_ultimo_nome;
  END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_obter_primeiro_ultimo_nome_juridica(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_obter_primeiro_ultimo_nome_juridica(text) RETURNS text
    AS $_$
DECLARE
  v_nome_parametro    ALIAS FOR $1;
  v_nome        text;
  v_primeiro_nome     text;
  v_ultimo_nome     text;
  v_posicao_espaco_primeiro_nome  integer;
  v_posicao_espaco_ultimo_nome  integer;
  v_cont        integer;
  v_fonema_ultimo_nome    text;
  v_reg       record;
  v_total_caracteres    integer;
  BEGIN
  v_primeiro_nome := '';
  v_ultimo_nome := '';
  v_nome := TRIM(v_nome_parametro);
  v_total_caracteres := LENGTH(v_nome);
  -- obter somente o primeiro e o ultimo nome das pessoas
  IF v_total_caracteres > 0 THEN
    -- retirar os espaos duplicados
    WHILE POSITION('  ' IN v_nome) > 0 LOOP
      v_nome := REPLACE(UPPER(v_nome),'  ', ' ');
    END LOOP;
    -- retirar ocorrncias que devem ser ignoradas no nome
    v_nome := TRIM(REPLACE(UPPER(v_nome),'&', ' '));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E CIA LTDA ME', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E CIA LTDA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' LTDA E OUTRA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA LTDA ME', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA LTDA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA. LTDA.', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA.LTDA.', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA.LTDA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA LTDA.', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' CIA. LTDA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' SC LT', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' S/C LT', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' S/C L', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' SC L', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' S/C LTD', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' SC LTD', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' E CIA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' LTDA ME', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' LTDA. ME', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),' LTDA.ME', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),'LTDA', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),'LTDA.', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),'S/C', ''));
    v_nome := TRIM(REPLACE(UPPER(v_nome),'S/A', ''));
    v_total_caracteres := LENGTH(v_nome);
    IF v_total_caracteres = (POSITION(' LT' IN v_nome) + 2) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' LT', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' LT.' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' LT.', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' LTD' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' LTD', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' LTD.' IN v_nome) + 4) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' LTD.', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' ME.' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' ME.', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' ME' IN v_nome) + 2) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' ME', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' -ME' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' -ME', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' -ME.' IN v_nome) + 4) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' -ME.', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' SA' IN v_nome) + 2) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' SA', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' S.A.' IN v_nome) + 4) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' S.A.', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' S.A' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' S.A', ''));
    END IF;
    IF v_total_caracteres = (POSITION(' S A' IN v_nome) + 3) THEN
      v_nome := TRIM(REPLACE(UPPER(v_nome),' S A', ''));
    END IF;
    v_nome := TRIM(v_nome);
    -- posio do espaco em branco para obter o primeiro nome
    v_posicao_espaco_primeiro_nome := POSITION(' ' IN v_nome);
    IF v_posicao_espaco_primeiro_nome > 0 THEN
      v_primeiro_nome := SUBSTR(v_nome, 1, (v_posicao_espaco_primeiro_nome - 1));
    ELSE
      v_primeiro_nome := v_nome;
    END IF;
    -- obter o ultimo nome
    v_posicao_espaco_ultimo_nome := 0;
    IF v_posicao_espaco_primeiro_nome > 0 THEN
      v_cont := v_posicao_espaco_ultimo_nome + 1;
      -- obter posicao do espaco em branco anterior ao ultimo nome
      WHILE v_cont < LENGTH(v_nome) LOOP
        IF SUBSTR(v_nome, v_cont, 1) = ' ' THEN
          v_posicao_espaco_ultimo_nome = v_cont;
        END IF;
        v_cont := v_cont + 1;
      END LOOP;
      v_ultimo_nome := SUBSTR(v_nome, (v_posicao_espaco_ultimo_nome + 1));
    END IF;
  END IF;
  --Fonetizando o primeiro nome--
  FOR v_reg IN SELECT * FROM public.fcn_fonetiza(v_primeiro_nome) LOOP
    v_primeiro_nome := v_reg.fcn_fonetiza;
  END LOOP;
  --Fonetizando o ultimo nome nome--
  FOR v_reg IN SELECT * FROM public.fcn_fonetiza(v_ultimo_nome) LOOP
    v_ultimo_nome := v_reg.fcn_fonetiza;
  END LOOP;
  RETURN v_primeiro_nome || ' ' || v_ultimo_nome;
  END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_documento(integer, character varying, character varying, character varying, character varying, integer, integer, integer, integer, character varying, character varying, character varying, character varying, integer, integer, character varying, character varying, integer, integer, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_documento(integer, character varying, character varying, character varying, character varying, integer, integer, integer, integer, character varying, character varying, character varying, character varying, integer, integer, character varying, character varying, integer, integer, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_rg ALIAS for $2;
  v_orgao_exp_rg ALIAS for $3;
  v_data_exp_rg ALIAS for $4;
  v_sigla_uf_exp_rg ALIAS for $5;
    v_tipo_cert_civil ALIAS for $6;
    v_num_termo ALIAS for $7;
  v_num_livro ALIAS for $8;
  v_num_folha ALIAS for $9;
    v_data_emissao_cert_civil ALIAS for $10;
    v_sigla_uf_cert_civil ALIAS for $11;
  v_sigla_uf_cart_trabalho ALIAS for $12;
  v_cartorio_cert_civil ALIAS for $13;
  v_num_cart_trabalho ALIAS for $14;
    v_serie_cart_trabalho ALIAS for $15;
    v_data_emissao_cart_trabalho ALIAS for $16;
  v_num_tit_eleitor ALIAS for $17;
  v_zona_tit_eleitor ALIAS for $18;
  v_secao_tit_eleitor ALIAS for $19;
  v_origem_gravacao ALIAS for $20;
  v_idpes_rev ALIAS for $21;
  v_idsis_rev ALIAS for $22;

    -- Outras variveis
    v_rg_aux varchar(10);
    v_orgao_exp_rg_aux varchar(10);
    v_sigla_uf_exp_rg_aux varchar(2);
    v_tipo_cert_civil_aux integer;
  v_num_termo_aux integer;
  v_num_livro_aux integer;
  v_num_folha_aux integer;
    v_sigla_uf_cert_civil_aux varchar(2);
  v_sigla_uf_cart_trabalho_aux varchar(2);
    v_cartorio_cert_civil_aux varchar(150);
    v_num_cart_trabalho_aux integer;
    v_serie_cart_trabalho_aux integer;
    v_num_tit_eleitor_aux varchar(13);
    v_zona_tit_eleitor_aux integer;
    v_secao_tit_eleitor_aux integer;
BEGIN
    IF v_rg = '' THEN
        v_rg_aux := NULL;
    ELSE
        v_rg_aux := v_rg;
    END IF;
    IF v_orgao_exp_rg = '' THEN
        v_orgao_exp_rg_aux := NULL;
    ELSE
        v_orgao_exp_rg_aux := v_orgao_exp_rg;
    END IF;
    IF v_sigla_uf_exp_rg = '' THEN
        v_sigla_uf_exp_rg_aux := NULL;
    ELSE
        v_sigla_uf_exp_rg_aux := v_sigla_uf_exp_rg;
    END IF;
    IF v_tipo_cert_civil = 0 THEN
        v_tipo_cert_civil_aux := NULL;
    ELSE
        v_tipo_cert_civil_aux := v_tipo_cert_civil;
    END IF;
    IF v_num_termo = 0 THEN
        v_num_termo_aux := NULL;
    ELSE
        v_num_termo_aux := v_num_termo;
    END IF;
    IF v_num_livro = 0 THEN
        v_num_livro_aux := NULL;
    ELSE
        v_num_livro_aux := v_num_livro;
    END IF;
    IF v_num_folha = 0 THEN
        v_num_folha_aux := NULL;
    ELSE
        v_num_folha_aux := v_num_folha;
    END IF;
    IF v_sigla_uf_cert_civil = '' THEN
        v_sigla_uf_cert_civil_aux := NULL;
    ELSE
        v_sigla_uf_cert_civil_aux := v_sigla_uf_cert_civil;
    END IF;
    IF v_sigla_uf_cart_trabalho = '' THEN
        v_sigla_uf_cart_trabalho_aux := NULL;
    ELSE
        v_sigla_uf_cart_trabalho_aux := v_sigla_uf_cart_trabalho;
    END IF;
    IF v_cartorio_cert_civil = '' THEN
        v_cartorio_cert_civil_aux := NULL;
    ELSE
        v_cartorio_cert_civil_aux := v_cartorio_cert_civil;
    END IF;
    IF v_num_cart_trabalho = 0 THEN
        v_num_cart_trabalho_aux := NULL;
    ELSE
        v_num_cart_trabalho_aux := v_num_cart_trabalho;
    END IF;
    IF v_serie_cart_trabalho = 0 THEN
        v_serie_cart_trabalho_aux := NULL;
    ELSE
        v_serie_cart_trabalho_aux := v_serie_cart_trabalho;
    END IF;
    IF v_num_tit_eleitor = '' THEN
        v_num_tit_eleitor_aux := NULL;
    ELSE
        v_num_tit_eleitor_aux := v_num_tit_eleitor;
    END IF;
    IF v_zona_tit_eleitor = 0 THEN
        v_zona_tit_eleitor_aux := NULL;
    ELSE
        v_zona_tit_eleitor_aux := v_zona_tit_eleitor;
    END IF;
    IF v_secao_tit_eleitor = 0 THEN
        v_secao_tit_eleitor_aux := NULL;
    ELSE
        v_secao_tit_eleitor_aux := v_secao_tit_eleitor;
    END IF;

  -- Insere dados na tabela funcionrio
    UPDATE cadastro.documento
    SET rg = to_number(v_rg_aux,9999999999),
        idorg_exp_rg = to_number(v_orgao_exp_rg_aux,9999999999),
        data_exp_rg = to_date(v_data_exp_rg,'DD/MM/YYYY'),
        sigla_uf_exp_rg = v_sigla_uf_exp_rg_aux,
        tipo_cert_civil = v_tipo_cert_civil_aux,
        num_termo = v_num_termo_aux,
        num_livro = v_num_livro_aux,
        num_folha = v_num_folha_aux,
        data_emissao_cert_civil = to_date(v_data_emissao_cert_civil,'DD/MM/YYYY'),
        sigla_uf_cert_civil = v_sigla_uf_cert_civil_aux,
        sigla_uf_cart_trabalho = v_sigla_uf_cart_trabalho_aux,
        cartorio_cert_civil = v_cartorio_cert_civil_aux,
        num_cart_trabalho = v_num_cart_trabalho_aux,
        serie_cart_trabalho = v_serie_cart_trabalho_aux,
        data_emissao_cart_trabalho = to_date(v_data_emissao_cart_trabalho,'DD/MM/YYYY'),
        num_tit_eleitor = to_number(v_num_tit_eleitor_aux,9999999999999),                                              zona_tit_eleitor = v_zona_tit_eleitor_aux,
        secao_tit_eleitor = v_secao_tit_eleitor_aux,
  origem_gravacao = v_origem_gravacao,
  idpes_rev = v_idpes_rev,
  idsis_rev = v_idsis_rev,
  data_rev = CURRENT_TIMESTAMP,
  operacao = 'A'
    WHERE idpes = v_id_pes;


  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_endereco_externo(integer, integer, character varying, character varying, character varying, integer, character varying, character varying, character varying, integer, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_endereco_externo(integer, integer, character varying, character varying, character varying, integer, character varying, character varying, character varying, integer, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_tipo ALIAS for $2;
  v_sigla_uf ALIAS for $3;
  v_idtlog ALIAS for $4;
    v_logradouro ALIAS for $5;
    v_numero ALIAS for $6;
    v_letra ALIAS for $7;
    v_complemento ALIAS for $8;
    v_bairro ALIAS for $9;
    v_cep ALIAS for $10;
    v_cidade ALIAS for $11;
    v_reside_desde ALIAS for $12;
    v_origem_gravacao ALIAS for $13;
    v_idpes_rev ALIAS for $14;
    v_idsis_rev ALIAS for $15;

BEGIN
  -- Atualiza dados na tabela endereco_externo
  UPDATE cadastro.endereco_externo
    SET sigla_uf = v_sigla_uf,
        idtlog = v_idtlog,
        logradouro = public.fcn_upper(v_logradouro),
        numero = v_numero,
        letra = public.fcn_upper(v_letra),
        complemento = public.fcn_upper(v_complemento),
        bairro = public.fcn_upper(v_bairro),
        cep = v_cep,
        cidade = public.fcn_upper(v_cidade),
        reside_desde = to_date(v_reside_desde,'DD/MM/YYYY'),
  origem_gravacao = v_origem_gravacao,
  idpes_rev = v_idpes_rev,
  idsis_rev = v_idsis_rev,
  data_rev = CURRENT_TIMESTAMP,
  operacao = 'A'
    WHERE idpes = v_id_pes
    AND tipo = v_tipo;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_endereco_pessoa(integer, integer, integer, integer, integer, integer, character varying, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_endereco_pessoa(integer, integer, integer, integer, integer, integer, character varying, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
    -- Parmetro recebidos
    v_id_pes ALIAS for $1;
    v_tipo ALIAS for $2;
    v_cep ALIAS for $3;
    v_idlog ALIAS for $4;
    v_idbai ALIAS for $5;
    v_numero ALIAS for $6;
    v_letra ALIAS for $7;
    v_complemento ALIAS for $8;
    v_reside_desde ALIAS for $9;
    v_origem_gravacao ALIAS for $10;
    v_idpes_rev ALIAS for $11;
    v_idsis_rev ALIAS for $12;
    v_sql_update text;
BEGIN
    -- Atualiza dados na tabela endereco_pessoa
    UPDATE cadastro.endereco_pessoa
    SET cep = v_cep,
        idlog = v_idlog,
        idbai = v_idbai,
        numero = v_numero,
        letra = public.fcn_upper(v_letra),
        complemento = public.fcn_upper(v_complemento),
        reside_desde = to_date(v_reside_desde,'DD/MM/YYYY'),
  origem_gravacao = v_origem_gravacao,
  idpes_rev = v_idpes_rev,
  idsis_rev = v_idsis_rev,
  data_rev = CURRENT_TIMESTAMP,
  operacao = 'A'
    WHERE idpes = v_id_pes
    AND tipo = v_tipo;
    RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_fisica(integer, character varying, character varying, integer, integer, integer, integer, integer, integer, character varying, character varying, integer, integer, character varying, integer, character varying, integer, character varying, character varying, character varying, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_fisica(integer, character varying, character varying, integer, integer, integer, integer, integer, integer, character varying, character varying, integer, integer, character varying, integer, character varying, integer, character varying, character varying, character varying, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_data_nasc ALIAS for $2;
  v_sexo ALIAS for $3;
  v_id_pes_mae ALIAS for $4;
      v_id_pes_pai ALIAS for $5;
      v_id_pes_responsavel ALIAS for $6;
  v_id_esco ALIAS for $7;
  v_id_eciv ALIAS for $8;
  v_id_pes_con ALIAS for $9;
      v_data_uniao ALIAS for $10;
      v_data_obito ALIAS for $11;
  v_nacionalidade ALIAS for $12;
  v_id_pais_estrangeiro ALIAS for $13;
  v_data_chegada ALIAS for $14;
      v_id_mun_nascimento ALIAS for $15;
      v_ultima_empresa ALIAS for $16;
  v_id_ocup ALIAS for $17;
  v_nome_mae ALIAS for $18;
  v_nome_pai ALIAS for $19;
      v_nome_conjuge ALIAS for $20;
      v_nome_responsavel ALIAS for $21;
      v_justificativa_provisorio ALIAS for $22;
      v_origem_gravacao ALIAS for $23;
      v_idpes_rev ALIAS for $24;
      v_idsis_rev ALIAS for $25;

      -- Outras variveis
      v_id_pes_mae_aux integer;
      v_id_pes_pai_aux integer;
      v_id_pes_responsavel_aux integer;
  v_id_esco_aux integer;
  v_id_eciv_aux integer;
  v_id_pes_con_aux integer;
      v_nacionalidade_aux integer;
  v_id_pais_estrangeiro_aux integer;
      v_id_mun_nascimento_aux integer;
      v_id_ocup_aux integer;
  v_sexo_aux text;
BEGIN
    IF v_id_pes_mae = 0 THEN
        v_id_pes_mae_aux := NULL;
    ELSE
        v_id_pes_mae_aux := v_id_pes_mae;
    END IF;
    IF v_id_pes_pai = 0 THEN
        v_id_pes_pai_aux := NULL;
    ELSE
        v_id_pes_pai_aux := v_id_pes_pai;
    END IF;
    IF v_id_pes_responsavel = 0 THEN
        v_id_pes_responsavel_aux := NULL;
    ELSE
        v_id_pes_responsavel_aux := v_id_pes_responsavel;
    END IF;
    IF v_id_esco = 0 THEN
        v_id_esco_aux := NULL;
    ELSE
        v_id_esco_aux := v_id_esco;
    END IF;
    IF v_id_eciv = 0 THEN
        v_id_eciv_aux := NULL;
    ELSE
        v_id_eciv_aux := v_id_eciv;
    END IF;
    IF v_id_pes_con = 0 THEN
        v_id_pes_con_aux := NULL;
    ELSE
        v_id_pes_con_aux := v_id_pes_con;
    END IF;
    IF v_nacionalidade = 0 THEN
        v_nacionalidade_aux := NULL;
    ELSE
        v_nacionalidade_aux := v_nacionalidade;
    END IF;
    IF v_id_pais_estrangeiro = 0 THEN
        v_id_pais_estrangeiro_aux := NULL;
    ELSE
        v_id_pais_estrangeiro_aux := v_id_pais_estrangeiro;
    END IF;
    IF v_id_mun_nascimento = 0 THEN
        v_id_mun_nascimento_aux := NULL;
    ELSE
        v_id_mun_nascimento_aux := v_id_mun_nascimento;
    END IF;
    IF v_id_ocup = 0 THEN
        v_id_ocup_aux := NULL;
    ELSE
        v_id_ocup_aux := v_id_ocup;
    END IF;
    IF TRIM(v_sexo) = '' THEN
        v_sexo_aux := NULL;
    ELSE
        v_sexo_aux := public.fcn_upper(v_sexo);
    END IF;
  -- Insere dados na tabela funcionrio
    UPDATE cadastro.fisica
    SET data_nasc = to_date(v_data_nasc,'DD/MM/YYYY'),
        sexo = v_sexo_aux,
        idpes_mae = v_id_pes_mae_aux,
        idpes_pai = v_id_pes_pai_aux,
        idpes_responsavel = v_id_pes_responsavel_aux,
        idesco = v_id_esco_aux,
        ideciv = v_id_eciv_aux,
        idpes_con = v_id_pes_con_aux,
        data_uniao = to_date(v_data_uniao,'DD/MM/YYYY'),
        data_obito = to_date(v_data_obito,'DD/MM/YYYY'),
        nacionalidade = v_nacionalidade_aux,
        idpais_estrangeiro = v_id_pais_estrangeiro_aux,
        data_chegada_brasil = to_date(v_data_chegada,'DD/MM/YYYY'),
        idmun_nascimento = v_id_mun_nascimento_aux,
        ultima_empresa = public.fcn_upper(v_ultima_empresa),
        idocup = v_id_ocup_aux,
        nome_mae = public.fcn_upper(v_nome_mae),
        nome_pai = public.fcn_upper(v_nome_pai),
        nome_conjuge = public.fcn_upper(v_nome_conjuge),
        nome_responsavel = public.fcn_upper(v_nome_responsavel),
        justificativa_provisorio = v_justificativa_provisorio,
  origem_gravacao = v_origem_gravacao,
  idpes_rev = v_idpes_rev,
  idsis_rev = v_idsis_rev,
  data_rev = CURRENT_TIMESTAMP,
  operacao = 'A'
    WHERE idpes = v_id_pes;

  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_fisica_cpf(integer, text, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_fisica_cpf(integer, text, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_cpf ALIAS for $2;
  v_origem_gravacao ALIAS for $3;
  v_idpes_rev ALIAS for $4;
  v_idsis_rev ALIAS for $5;

BEGIN
  -- Atualiza dados na tabela fisica_cpf
  UPDATE cadastro.fisica_cpf SET
    origem_gravacao = v_origem_gravacao,
    idpes_rev = v_idpes_rev,
    idsis_rev = v_idsis_rev,
    data_rev = CURRENT_TIMESTAMP,
    operacao = 'A',
    cpf = to_number(v_cpf,99999999999)
    WHERE cadastro.fisica_cpf.idpes = v_id_pes;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_fone_pessoa(integer, integer, integer, integer, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_fone_pessoa(integer, integer, integer, integer, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_tipo ALIAS for $2;
  v_ddd ALIAS for $3;
  v_fone ALIAS for $4;
  v_origem_gravacao ALIAS for $5;
      v_idpes_rev ALIAS for $6;
      v_idsis_rev ALIAS for $7;

BEGIN
  -- Atualiza dados na tabela fone_pessoa
  UPDATE cadastro.fone_pessoa
    SET ddd = v_ddd,
        fone = v_fone,
  origem_gravacao = v_origem_gravacao,
  idpes_rev = v_idpes_rev,
  idsis_rev = v_idsis_rev,
  data_rev = CURRENT_TIMESTAMP,
  operacao = 'A'
    WHERE idpes = v_id_pes
    AND tipo = v_tipo;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_funcionario(numeric, integer, integer, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_funcionario(numeric, integer, integer, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_matricula ALIAS for $1;
  v_id_ins ALIAS for $2;
  v_id_set ALIAS for $3;
  v_situacao ALIAS for $4;
      v_origem_gravacao ALIAS for $5;
      v_idpes_rev ALIAS for $6;
      v_idsis_rev ALIAS for $7;
  -- Outras variveis
  v_id_set_aux integer;
  v_matricula_aux numeric;
  v_id_ins_aux integer;
  v_situacao_aux varchar(1);
BEGIN
  v_matricula_aux := v_matricula;
  v_id_ins_aux := v_id_ins;
  v_situacao_aux := v_situacao;
  IF v_id_set <= 0 THEN
    v_id_set_aux := NULL;
  ELSE
    v_id_set_aux := v_id_set;
  END IF;

  IF v_id_set_aux IS NULL AND v_id_set = -1 THEN
    -- Sql utilizado para ativar e desativar o registro na tabela funcionrio
     UPDATE cadastro.funcionario
      SET situacao=v_situacao_aux,
      origem_gravacao = v_origem_gravacao,
      idpes_rev = v_idpes_rev,
      idsis_rev = v_idsis_rev,
      data_rev = CURRENT_DATE,
      operacao = 'A'
      WHERE
          matricula=v_matricula_aux AND
          idins=v_id_ins_aux;
  ELSE
    UPDATE cadastro.funcionario
      SET idset=v_id_set_aux,
          situacao=v_situacao_aux,
          origem_gravacao = v_origem_gravacao,
          idpes_rev = v_idpes_rev,
          idsis_rev = v_idsis_rev,
          data_rev = CURRENT_TIMESTAMP,
            operacao = 'A'
      WHERE
          matricula=v_matricula_aux AND
          idins=v_id_ins_aux;
  END IF;

  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_juridica(integer, character varying, character varying, character varying, character, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_juridica(integer, character varying, character varying, character varying, character, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_cnpj ALIAS for $2;
  v_fantasia ALIAS for $3;
  v_inscr_estadual ALIAS for $4;
      v_origem_gravacao ALIAS for $5;
      v_idpes_rev ALIAS for $6;
      v_idsis_rev ALIAS for $7;

BEGIN
  -- Atualiza dados na tabela juridica
  UPDATE cadastro.juridica
    SET cnpj = to_number(v_cnpj,99999999999999),
        fantasia = public.fcn_upper(v_fantasia),
        insc_estadual = to_number(v_inscr_estadual,9999999999),
  origem_gravacao = v_origem_gravacao,
  idpes_rev = v_idpes_rev,
  idsis_rev = v_idsis_rev,
  data_rev = CURRENT_TIMESTAMP,
  operacao = 'A'
    WHERE idpes = v_id_pes;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_update_pessoa(integer, text, character varying, character varying, character varying, integer, character varying, integer, integer); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_update_pessoa(integer, text, character varying, character varying, character varying, integer, character varying, integer, integer) RETURNS integer
    AS $_$
DECLARE
  -- Parmetro recebidos
  v_id_pes ALIAS for $1;
  v_razao_social ALIAS for $2;
  v_url ALIAS for $3;
  v_email ALIAS for $4;
  v_situacao ALIAS for $5;
  v_id_pes_logado ALIAS for $6;
  v_origem_gravacao ALIAS for $7;
      v_idpes_rev ALIAS for $8;
      v_idsis_rev ALIAS for $9;

  idpes_logado integer;
BEGIN
  idpes_logado := v_id_pes_logado;
  IF (idpes_logado <= 0) THEN
    -- Atualiza dados na tabela pessoa
    UPDATE cadastro.pessoa SET
        nome = public.fcn_upper(v_razao_social),
        url = v_url,
        email = v_email,
        situacao = v_situacao,
        idpes_rev = NULL,
        data_rev = CURRENT_TIMESTAMP,
              origem_gravacao = v_origem_gravacao,
        idsis_rev = v_idsis_rev,
        operacao = 'A'
        WHERE idpes = v_id_pes;
  ELSE
    -- Atualiza dados na tabela pessoa
    UPDATE cadastro.pessoa SET
        nome = public.fcn_upper(v_razao_social),
        url = v_url,
        email = v_email,
        situacao = v_situacao,
        idpes_rev = idpes_logado,
        data_rev = CURRENT_TIMESTAMP,
              origem_gravacao = v_origem_gravacao,
        idsis_rev = v_idsis_rev,
        operacao = 'A'
        WHERE idpes = v_id_pes;
    END IF;
  RETURN 0;
END;$_$
    LANGUAGE plpgsql;


--
-- Name: fcn_upper(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_upper(text) RETURNS text
    AS $_$
   DECLARE
    v_texto     ALIAS FOR $1;
    v_retorno   text := '';
   BEGIN
    IF v_texto IS NOT NULL THEN
     SELECT translate(upper(v_texto),'','') INTO v_retorno;
    END IF;
    RETURN v_retorno;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: fcn_upper_nrm(text); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION fcn_upper_nrm(text) RETURNS text
    AS $_$
   DECLARE
    v_texto     ALIAS FOR $1;
    v_retorno   text := '';
   BEGIN
    IF v_texto IS NOT NULL THEN
     SELECT translate(upper(v_texto),'','AEIOUYAEIOUAOAEIOUAEIOUYCAEIOUYAEIOUAOAEIOUAEIOUC') INTO v_retorno;
    END IF;
    RETURN v_retorno;
   END;
  $_$
    LANGUAGE plpgsql;


--
-- Name: plpgsql_call_handler(); Type: FUNCTION; Schema: public; Owner: -
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


SET search_path = acesso, pg_catalog;

--
-- Name: funcao_idfunc_seq; Type: SEQUENCE; Schema: acesso; Owner: -
--

CREATE SEQUENCE funcao_idfunc_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: funcao_idfunc_seq; Type: SEQUENCE SET; Schema: acesso; Owner: -
--

SELECT pg_catalog.setval('funcao_idfunc_seq', 1, false);


SET default_tablespace = '';

SET default_with_oids = true;

--
-- Name: funcao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE funcao (
    idfunc integer DEFAULT nextval('funcao_idfunc_seq'::regclass) NOT NULL,
    idsis integer NOT NULL,
    idmen integer NOT NULL,
    nome character varying(100) NOT NULL,
    situacao character(1) NOT NULL,
    url character varying(250) NOT NULL,
    ordem numeric(2,0) NOT NULL,
    descricao character varying(250) NOT NULL,
    CONSTRAINT ck_funcao_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: grupo_idgrp_seq; Type: SEQUENCE; Schema: acesso; Owner: -
--

CREATE SEQUENCE grupo_idgrp_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: grupo_idgrp_seq; Type: SEQUENCE SET; Schema: acesso; Owner: -
--

SELECT pg_catalog.setval('grupo_idgrp_seq', 1, false);


--
-- Name: grupo; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE grupo (
    idgrp integer DEFAULT nextval('grupo_idgrp_seq'::regclass) NOT NULL,
    nome character varying(40) NOT NULL,
    situacao character(1) NOT NULL,
    descricao character varying(250),
    CONSTRAINT ck_grupo_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: grupo_funcao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE grupo_funcao (
    idmen integer NOT NULL,
    idsis integer NOT NULL,
    idgrp integer NOT NULL,
    idfunc integer NOT NULL
);


--
-- Name: grupo_menu; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE grupo_menu (
    idgrp integer NOT NULL,
    idsis integer NOT NULL,
    idmen integer NOT NULL
);


--
-- Name: grupo_operacao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE grupo_operacao (
    idfunc integer NOT NULL,
    idgrp integer NOT NULL,
    idsis integer NOT NULL,
    idmen integer NOT NULL,
    idope integer NOT NULL
);


--
-- Name: grupo_sistema; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE grupo_sistema (
    idsis integer NOT NULL,
    idgrp integer NOT NULL
);


--
-- Name: historico_senha; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE historico_senha (
    login character varying(16) NOT NULL,
    senha character varying(60) NOT NULL,
    data_cad timestamp without time zone NOT NULL
);


--
-- Name: instituicao_idins_seq; Type: SEQUENCE; Schema: acesso; Owner: -
--

CREATE SEQUENCE instituicao_idins_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: instituicao_idins_seq; Type: SEQUENCE SET; Schema: acesso; Owner: -
--

SELECT pg_catalog.setval('instituicao_idins_seq', 1, false);


--
-- Name: instituicao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE instituicao (
    idins integer DEFAULT nextval('instituicao_idins_seq'::regclass) NOT NULL,
    nome character varying(60) NOT NULL,
    situacao character(1) NOT NULL,
    CONSTRAINT ck_instituicao_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: log_acesso; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE log_acesso (
    data timestamp without time zone NOT NULL,
    idpes numeric(8,0) NOT NULL,
    idsis integer,
    idins integer,
    idcli character varying(10),
    operacao character(1) NOT NULL,
    CONSTRAINT ck_log_acesso_situacao CHECK (((operacao = 'I'::bpchar) OR (operacao = 'O'::bpchar)))
);


--
-- Name: log_erro; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE log_erro (
    data timestamp without time zone NOT NULL,
    idpes numeric(8,0),
    idsis integer,
    idmen integer,
    idfunc integer,
    idope integer,
    msg_erro text NOT NULL
);


--
-- Name: menu_idmen_seq; Type: SEQUENCE; Schema: acesso; Owner: -
--

CREATE SEQUENCE menu_idmen_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: menu_idmen_seq; Type: SEQUENCE SET; Schema: acesso; Owner: -
--

SELECT pg_catalog.setval('menu_idmen_seq', 1, false);


--
-- Name: menu; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE menu (
    idmen integer DEFAULT nextval('menu_idmen_seq'::regclass) NOT NULL,
    idsis integer NOT NULL,
    menu_idsis integer,
    menu_idmen integer,
    nome character varying(40) NOT NULL,
    descricao character varying(250) NOT NULL,
    situacao character(1) NOT NULL,
    ordem numeric(2,0) NOT NULL,
    CONSTRAINT ck_menu_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: operacao_idope_seq; Type: SEQUENCE; Schema: acesso; Owner: -
--

CREATE SEQUENCE operacao_idope_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: operacao_idope_seq; Type: SEQUENCE SET; Schema: acesso; Owner: -
--

SELECT pg_catalog.setval('operacao_idope_seq', 1, false);


--
-- Name: operacao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE operacao (
    idope integer DEFAULT nextval('operacao_idope_seq'::regclass) NOT NULL,
    idsis integer,
    nome character varying(40) NOT NULL,
    situacao character(1) NOT NULL,
    descricao character varying(250) NOT NULL,
    CONSTRAINT ck_operacao_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: operacao_funcao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE operacao_funcao (
    idmen integer NOT NULL,
    idsis integer NOT NULL,
    idfunc integer NOT NULL,
    idope integer NOT NULL
);


--
-- Name: pessoa_instituicao; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE pessoa_instituicao (
    idins integer NOT NULL,
    idpes numeric(8,0) NOT NULL
);


--
-- Name: sistema_idsis_seq; Type: SEQUENCE; Schema: acesso; Owner: -
--

CREATE SEQUENCE sistema_idsis_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: sistema_idsis_seq; Type: SEQUENCE SET; Schema: acesso; Owner: -
--

SELECT pg_catalog.setval('sistema_idsis_seq', 17, true);


--
-- Name: sistema; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE sistema (
    idsis integer DEFAULT nextval('sistema_idsis_seq'::regclass) NOT NULL,
    nome character varying(60) NOT NULL,
    descricao character varying(100) NOT NULL,
    contexto character varying(30) NOT NULL,
    situacao character(1) NOT NULL,
    CONSTRAINT ck_sistema_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: usuario; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE usuario (
    login character varying(16) NOT NULL,
    idpes numeric(8,0) NOT NULL,
    idpes_sga numeric(8,0),
    senha character varying(60) NOT NULL,
    datacad date NOT NULL,
    lastlogin timestamp without time zone NOT NULL,
    dica character varying(60),
    situacao character(1) NOT NULL,
    data_alt_senha date NOT NULL,
    exp_senha character(1),
    mudar_senha character(1),
    num_sessao_atual numeric(2,0) NOT NULL,
    prazo_exp numeric(3,0),
    estilo_menu character(1),
    CONSTRAINT ck_usuario_estilo_menu CHECK (((estilo_menu = 'C'::bpchar) OR (estilo_menu = 'D'::bpchar))),
    CONSTRAINT ck_usuario_exp_senha CHECK (((exp_senha = 'S'::bpchar) OR (exp_senha = 'N'::bpchar))),
    CONSTRAINT ck_usuario_mudar_senha CHECK (((mudar_senha = 'S'::bpchar) OR (mudar_senha = 'N'::bpchar))),
    CONSTRAINT ck_usuario_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: usuario_grupo; Type: TABLE; Schema: acesso; Owner: -; Tablespace:
--

CREATE TABLE usuario_grupo (
    idgrp integer NOT NULL,
    login character varying(16) NOT NULL
);


SET search_path = alimentos, pg_catalog;

--
-- Name: baixa_guia_produto_idbap_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE baixa_guia_produto_idbap_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: baixa_guia_produto_idbap_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('baixa_guia_produto_idbap_seq', 1, false);


--
-- Name: baixa_guia_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE baixa_guia_produto (
    idbap integer DEFAULT nextval('baixa_guia_produto_idbap_seq'::regclass) NOT NULL,
    idgup integer NOT NULL,
    idbai integer NOT NULL,
    dt_validade date,
    qtde_recebida numeric NOT NULL,
    dt_operacao date NOT NULL,
    login_baixa character varying(80) NOT NULL
);


--
-- Name: baixa_guia_remessa_idbai_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE baixa_guia_remessa_idbai_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: baixa_guia_remessa_idbai_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('baixa_guia_remessa_idbai_seq', 1, false);


--
-- Name: baixa_guia_remessa; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE baixa_guia_remessa (
    idbai integer DEFAULT nextval('baixa_guia_remessa_idbai_seq'::regclass) NOT NULL,
    login_baixa character varying(80) NOT NULL,
    idgui integer NOT NULL,
    dt_recebimento date NOT NULL,
    nome_recebedor character varying(40) NOT NULL,
    cargo_recebedor character varying(40) NOT NULL,
    dt_operacao date NOT NULL
);


--
-- Name: calendario_idcad_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE calendario_idcad_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: calendario_idcad_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('calendario_idcad_seq', 1, false);


--
-- Name: calendario; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE calendario (
    idcad integer DEFAULT nextval('calendario_idcad_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    ano integer NOT NULL,
    descricao character varying(40) NOT NULL
);


--
-- Name: cardapio_idcar_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE cardapio_idcar_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: cardapio_idcar_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('cardapio_idcar_seq', 1, false);


--
-- Name: cardapio; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE cardapio (
    idcar integer DEFAULT nextval('cardapio_idcar_seq'::regclass) NOT NULL,
    login_inclusao character varying(80) NOT NULL,
    login_alteracao character varying(80),
    idcli character varying(10) NOT NULL,
    idtre integer NOT NULL,
    dt_cardapio date NOT NULL,
    dt_inclusao timestamp without time zone NOT NULL,
    dt_ultima_alteracao timestamp without time zone,
    valor numeric NOT NULL,
    finalizado character(1) NOT NULL,
    CONSTRAINT ck_cardapio_finalizado CHECK (((finalizado = 'S'::bpchar) OR (finalizado = 'N'::bpchar)))
);


--
-- Name: cardapio_faixa_unidade; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE cardapio_faixa_unidade (
    idfeu integer NOT NULL,
    idcar integer NOT NULL
);


--
-- Name: cardapio_produto_idcpr_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE cardapio_produto_idcpr_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: cardapio_produto_idcpr_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('cardapio_produto_idcpr_seq', 1, false);


--
-- Name: cardapio_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE cardapio_produto (
    idcpr integer DEFAULT nextval('cardapio_produto_idcpr_seq'::regclass) NOT NULL,
    idpro integer NOT NULL,
    idcar integer NOT NULL,
    quantidade numeric NOT NULL,
    valor numeric NOT NULL
);


--
-- Name: cardapio_receita; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE cardapio_receita (
    idcar integer NOT NULL,
    idrec integer NOT NULL
);


--
-- Name: cliente; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE cliente (
    idcli character varying(10) NOT NULL,
    nome character varying(40) NOT NULL,
    cnpj character varying(14) NOT NULL,
    endereco character varying(60) NOT NULL,
    bairro character varying(30) NOT NULL,
    cidade character varying(18) NOT NULL,
    cep character varying(8),
    uf character varying(2) NOT NULL,
    telefone character varying(11) NOT NULL,
    fax character varying(11),
    email character varying(40),
    prefeito character varying(40) NOT NULL,
    educacao character varying(40) NOT NULL,
    administracao character varying(40) NOT NULL,
    coordenacao character varying(40) NOT NULL,
    inscritos character(1) NOT NULL,
    idpes integer NOT NULL,
    identificacao character varying(20) NOT NULL,
    tab_produtos character(1),
    CONSTRAINT ck_tab_produtos CHECK (((tab_produtos = '1'::bpchar) OR (tab_produtos = '2'::bpchar)))
);


--
-- Name: composto_quimico_idcom_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE composto_quimico_idcom_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: composto_quimico_idcom_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('composto_quimico_idcom_seq', 1, false);


--
-- Name: composto_quimico; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE composto_quimico (
    idcom integer DEFAULT nextval('composto_quimico_idcom_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    idgrpq integer NOT NULL,
    descricao character varying(50) NOT NULL,
    unidade character varying(5) NOT NULL
);


--
-- Name: contrato_idcon_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE contrato_idcon_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: contrato_idcon_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('contrato_idcon_seq', 1, false);


--
-- Name: contrato; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE contrato (
    idcon integer DEFAULT nextval('contrato_idcon_seq'::regclass) NOT NULL,
    codigo character varying(20) NOT NULL,
    idcli character varying(10) NOT NULL,
    login character varying(80) NOT NULL,
    num_aditivo integer NOT NULL,
    idfor integer NOT NULL,
    dt_vigencia date NOT NULL,
    tipo character(1) NOT NULL,
    vlr_atual numeric NOT NULL,
    cancelado character(1) NOT NULL,
    dt_cancelamento timestamp without time zone,
    dt_inclusao timestamp without time zone NOT NULL,
    ultimo_contrato character(1) NOT NULL,
    vlr_original numeric NOT NULL,
    finalizado character(1) NOT NULL,
    CONSTRAINT ck_contrato_cancelado CHECK (((cancelado = 'S'::bpchar) OR (cancelado = 'N'::bpchar))),
    CONSTRAINT ck_contrato_finalizado CHECK (((finalizado = 'S'::bpchar) OR (finalizado = 'N'::bpchar))),
    CONSTRAINT ck_contrato_tipo CHECK (((tipo = 'C'::bpchar) OR (tipo = 'A'::bpchar))),
    CONSTRAINT ck_contrato_ultimo_contrato CHECK (((ultimo_contrato = 'S'::bpchar) OR (ultimo_contrato = 'N'::bpchar)))
);


--
-- Name: contrato_produto_idcop_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE contrato_produto_idcop_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: contrato_produto_idcop_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('contrato_produto_idcop_seq', 1, false);


--
-- Name: contrato_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE contrato_produto (
    idcop integer DEFAULT nextval('contrato_produto_idcop_seq'::regclass) NOT NULL,
    idcon integer NOT NULL,
    idpro integer NOT NULL,
    qtde_contratada numeric NOT NULL,
    vlr_unitario_atual numeric NOT NULL,
    qtde_remessa numeric NOT NULL,
    qtde_recebida numeric NOT NULL,
    qtde_aditivo numeric NOT NULL,
    vlr_unitario_original numeric NOT NULL,
    operacao character(1) NOT NULL,
    ajuste numeric,
    CONSTRAINT ck_contrato_operacao CHECK (((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar)) OR (operacao = 'N'::bpchar)))
);


--
-- Name: evento_ideve_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE evento_ideve_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: evento_ideve_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('evento_ideve_seq', 1, false);


--
-- Name: evento; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE evento (
    ideve integer DEFAULT nextval('evento_ideve_seq'::regclass) NOT NULL,
    idcad integer NOT NULL,
    mes integer NOT NULL,
    dia integer NOT NULL,
    dia_util character(1) NOT NULL,
    descricao character varying(50) NOT NULL,
    CONSTRAINT ck_evento_dia_util CHECK (((dia_util = 'S'::bpchar) OR (dia_util = 'N'::bpchar)))
);


--
-- Name: faixa_composto_quimico_idfcp_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE faixa_composto_quimico_idfcp_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: faixa_composto_quimico_idfcp_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('faixa_composto_quimico_idfcp_seq', 1, false);


--
-- Name: faixa_composto_quimico; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE faixa_composto_quimico (
    idfcp integer DEFAULT nextval('faixa_composto_quimico_idfcp_seq'::regclass) NOT NULL,
    idcom integer NOT NULL,
    idfae integer NOT NULL,
    quantidade numeric NOT NULL,
    qtde_max_min character(3) NOT NULL,
    CONSTRAINT ck_qtde_max_min CHECK (((qtde_max_min = 'MAX'::bpchar) OR (qtde_max_min = 'MIN'::bpchar)))
);


--
-- Name: faixa_etaria_idfae_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE faixa_etaria_idfae_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: faixa_etaria_idfae_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('faixa_etaria_idfae_seq', 1, false);


--
-- Name: faixa_etaria; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE faixa_etaria (
    idfae integer DEFAULT nextval('faixa_etaria_idfae_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(50) NOT NULL,
    vlr_base_refeicao numeric NOT NULL
);


--
-- Name: fornecedor_idfor_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE fornecedor_idfor_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: fornecedor_idfor_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('fornecedor_idfor_seq', 1, false);


--
-- Name: fornecedor; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE fornecedor (
    idfor integer DEFAULT nextval('fornecedor_idfor_seq'::regclass) NOT NULL,
    idpes integer NOT NULL,
    idcli character varying(10) NOT NULL,
    razao_social character varying(50) NOT NULL,
    nome_fantasia character varying(50) NOT NULL,
    endereco character varying(40) NOT NULL,
    complemento character varying(30),
    bairro character varying(30) NOT NULL,
    cep character varying(8),
    cidade character varying(18) NOT NULL,
    uf character varying(2) NOT NULL,
    telefone character varying(11) NOT NULL,
    fax character varying(11),
    email character varying(40),
    contato character varying(40) NOT NULL,
    cpf_cnpj character varying(14) NOT NULL,
    inscr_estadual character varying(20),
    inscr_municipal character varying(20),
    tipo character(1) NOT NULL,
    CONSTRAINT ck_fornecedor CHECK (((tipo = 'F'::bpchar) OR (tipo = 'J'::bpchar)))
);


--
-- Name: fornecedor_unidade_atendida; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE fornecedor_unidade_atendida (
    iduni integer NOT NULL,
    idfor integer NOT NULL
);


--
-- Name: grupo_quimico_idgrpq_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE grupo_quimico_idgrpq_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: grupo_quimico_idgrpq_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('grupo_quimico_idgrpq_seq', 1, false);


--
-- Name: grupo_quimico; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE grupo_quimico (
    idgrpq integer DEFAULT nextval('grupo_quimico_idgrpq_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(50) NOT NULL
);


--
-- Name: guia_produto_diario_idguiaprodiario_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE guia_produto_diario_idguiaprodiario_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: guia_produto_diario_idguiaprodiario_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('guia_produto_diario_idguiaprodiario_seq', 1, false);


--
-- Name: guia_produto_diario; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE guia_produto_diario (
    idguiaprodiario integer DEFAULT nextval('guia_produto_diario_idguiaprodiario_seq'::regclass) NOT NULL,
    idgui integer NOT NULL,
    idpro integer NOT NULL,
    iduni integer NOT NULL,
    dt_guia date NOT NULL,
    qtde numeric NOT NULL
);


--
-- Name: guia_remessa_idgui_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE guia_remessa_idgui_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: guia_remessa_idgui_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('guia_remessa_idgui_seq', 1, false);


--
-- Name: guia_remessa; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE guia_remessa (
    idgui integer DEFAULT nextval('guia_remessa_idgui_seq'::regclass) NOT NULL,
    idcon integer NOT NULL,
    login_cancelamento character varying(80),
    login_emissao character varying(80) NOT NULL,
    idfor integer NOT NULL,
    iduni integer NOT NULL,
    idcli character varying(10) NOT NULL,
    dt_emissao timestamp without time zone NOT NULL,
    ano integer NOT NULL,
    sequencial integer NOT NULL,
    dt_cardapio_inicial date NOT NULL,
    dt_cardapio_final date NOT NULL,
    num_inscr_matr integer NOT NULL,
    num_refeicao integer NOT NULL,
    situacao character(1) NOT NULL,
    dt_cancelamento timestamp without time zone,
    justificativa_cancelamento character varying(300),
    classe_produto character varying(2) NOT NULL,
    CONSTRAINT ck_guia_remessa_classe_produto CHECK (((((classe_produto)::text = 'P'::text) OR ((classe_produto)::text = 'N'::text)) OR ((classe_produto)::text = 'PN'::text))),
    CONSTRAINT ck_guia_remessa_situacao CHECK (((((situacao = 'E'::bpchar) OR (situacao = 'R'::bpchar)) OR (situacao = 'C'::bpchar)) OR (situacao = 'P'::bpchar)))
);


--
-- Name: guia_remessa_produto_idgup_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE guia_remessa_produto_idgup_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: guia_remessa_produto_idgup_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('guia_remessa_produto_idgup_seq', 1, false);


--
-- Name: guia_remessa_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE guia_remessa_produto (
    idgup integer DEFAULT nextval('guia_remessa_produto_idgup_seq'::regclass) NOT NULL,
    idgui integer NOT NULL,
    idpro integer NOT NULL,
    qtde_per_capita numeric NOT NULL,
    qtde_guia numeric NOT NULL,
    peso numeric NOT NULL,
    qtde_recebida numeric NOT NULL,
    peso_total numeric NOT NULL
);


--
-- Name: log_guia_remessa_idlogguia_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE log_guia_remessa_idlogguia_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: log_guia_remessa_idlogguia_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('log_guia_remessa_idlogguia_seq', 1, false);


--
-- Name: log_guia_remessa; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE log_guia_remessa (
    idlogguia integer DEFAULT nextval('log_guia_remessa_idlogguia_seq'::regclass) NOT NULL,
    login character varying(80) NOT NULL,
    idcli character varying(10) NOT NULL,
    dt_inicial date NOT NULL,
    dt_final date NOT NULL,
    unidade character varying(80) NOT NULL,
    fornecedor character varying(80) NOT NULL,
    classe character(2),
    dt_geracao timestamp without time zone NOT NULL,
    mensagem text NOT NULL
);


--
-- Name: medidas_caseiras; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE medidas_caseiras (
    idmedcas character varying(20) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(50) NOT NULL
);


--
-- Name: pessoa_idpes_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE pessoa_idpes_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: pessoa_idpes_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('pessoa_idpes_seq', 1, false);


--
-- Name: pessoa; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE pessoa (
    idpes integer DEFAULT nextval('pessoa_idpes_seq'::regclass) NOT NULL,
    tipo character varying(1) NOT NULL,
    CONSTRAINT ck_pessoa CHECK (((((tipo)::text = 'C'::text) OR ((tipo)::text = 'F'::text)) OR ((tipo)::text = 'U'::text)))
);


--
-- Name: produto_idpro_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE produto_idpro_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: produto_idpro_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('produto_idpro_seq', 1, false);


--
-- Name: produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE produto (
    idpro integer DEFAULT nextval('produto_idpro_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    idunp character varying(20) NOT NULL,
    idfnde character varying(15) NOT NULL,
    idtip integer NOT NULL,
    descricao character varying(60) NOT NULL,
    nome_compra character varying(60) NOT NULL,
    referencia_ceasa character varying(1) NOT NULL,
    fator_coccao numeric NOT NULL,
    fator_correcao numeric NOT NULL,
    vlr_unitario numeric NOT NULL,
    penultimo_vlr_unitario numeric,
    dt_ultima_compra timestamp without time zone,
    dt_penultima_compra timestamp without time zone,
    qtde_estoque numeric NOT NULL,
    classe character varying(1) NOT NULL,
    desc_composto character varying(300),
    idfor integer,
    CONSTRAINT ck_produto_classe CHECK ((((classe)::text = 'P'::text) OR ((classe)::text = 'N'::text))),
    CONSTRAINT ck_produto_referencia_ceasa CHECK ((((referencia_ceasa)::text = '1'::text) OR ((referencia_ceasa)::text = '0'::text)))
);


--
-- Name: produto_composto_quimico_idpcq_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE produto_composto_quimico_idpcq_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: produto_composto_quimico_idpcq_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('produto_composto_quimico_idpcq_seq', 1, false);


--
-- Name: produto_composto_quimico; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE produto_composto_quimico (
    idpcq integer DEFAULT nextval('produto_composto_quimico_idpcq_seq'::regclass) NOT NULL,
    idpro integer NOT NULL,
    idcom integer NOT NULL,
    quantidade numeric NOT NULL
);


--
-- Name: produto_fornecedor_idprf_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE produto_fornecedor_idprf_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: produto_fornecedor_idprf_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('produto_fornecedor_idprf_seq', 1, false);


--
-- Name: produto_fornecedor; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE produto_fornecedor (
    idprf integer DEFAULT nextval('produto_fornecedor_idprf_seq'::regclass) NOT NULL,
    idfor integer NOT NULL,
    idpro integer NOT NULL,
    codigo_ean character varying(18) NOT NULL
);


--
-- Name: produto_medida_caseira_idpmc_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE produto_medida_caseira_idpmc_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: produto_medida_caseira_idpmc_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('produto_medida_caseira_idpmc_seq', 1, false);


--
-- Name: produto_medida_caseira; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE produto_medida_caseira (
    idpmc integer DEFAULT nextval('produto_medida_caseira_idpmc_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    idmedcas character varying(20) NOT NULL,
    idpro integer NOT NULL,
    peso numeric NOT NULL
);


--
-- Name: receita_idrec_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE receita_idrec_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: receita_idrec_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('receita_idrec_seq', 1, false);


--
-- Name: receita; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE receita (
    idrec integer DEFAULT nextval('receita_idrec_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    valor numeric NOT NULL,
    descricao character varying(60) NOT NULL,
    modo_preparo text NOT NULL,
    rendimento integer NOT NULL,
    valor_percapita numeric NOT NULL
);


--
-- Name: receita_composto_quimico_idrcq_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE receita_composto_quimico_idrcq_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: receita_composto_quimico_idrcq_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('receita_composto_quimico_idrcq_seq', 1, false);


--
-- Name: receita_composto_quimico; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE receita_composto_quimico (
    idrcq integer DEFAULT nextval('receita_composto_quimico_idrcq_seq'::regclass) NOT NULL,
    idcom integer NOT NULL,
    idrec integer NOT NULL,
    quantidade numeric NOT NULL
);


--
-- Name: receita_produto_idrpr_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE receita_produto_idrpr_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: receita_produto_idrpr_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('receita_produto_idrpr_seq', 1, false);


--
-- Name: receita_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE receita_produto (
    idrpr integer DEFAULT nextval('receita_produto_idrpr_seq'::regclass) NOT NULL,
    idpro integer NOT NULL,
    idrec integer NOT NULL,
    idmedcas character varying(20),
    quantidade numeric NOT NULL,
    valor numeric NOT NULL,
    qtdemedidacaseira integer NOT NULL,
    valor_percapita numeric NOT NULL
);


--
-- Name: tipo_produto_idtip_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE tipo_produto_idtip_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_produto_idtip_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('tipo_produto_idtip_seq', 1, false);


--
-- Name: tipo_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE tipo_produto (
    idtip integer DEFAULT nextval('tipo_produto_idtip_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(50) NOT NULL
);


--
-- Name: tipo_refeicao_idtre_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE tipo_refeicao_idtre_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_refeicao_idtre_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('tipo_refeicao_idtre_seq', 1, false);


--
-- Name: tipo_refeicao; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE tipo_refeicao (
    idtre integer DEFAULT nextval('tipo_refeicao_idtre_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(30) NOT NULL
);


--
-- Name: tipo_unidade_idtip_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE tipo_unidade_idtip_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_unidade_idtip_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('tipo_unidade_idtip_seq', 1, false);


--
-- Name: tipo_unidade; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE tipo_unidade (
    idtip integer DEFAULT nextval('tipo_unidade_idtip_seq'::regclass) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(50) NOT NULL
);


--
-- Name: unidade_atendida_iduni_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE unidade_atendida_iduni_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: unidade_atendida_iduni_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('unidade_atendida_iduni_seq', 1, false);


--
-- Name: unidade_atendida; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE unidade_atendida (
    iduni integer DEFAULT nextval('unidade_atendida_iduni_seq'::regclass) NOT NULL,
    idcad integer NOT NULL,
    idtip integer NOT NULL,
    codigo character varying(10) NOT NULL,
    idcli character varying(10) NOT NULL,
    nome character varying(40) NOT NULL,
    endereco character varying(60) NOT NULL,
    complemento character varying(30),
    bairro character varying(30) NOT NULL,
    cep character varying(8),
    telefone character varying(11) NOT NULL,
    fax character varying(11),
    email character varying(40),
    idpes integer NOT NULL,
    diretor character varying(40) NOT NULL
);


--
-- Name: unidade_faixa_etaria_idfeu_seq; Type: SEQUENCE; Schema: alimentos; Owner: -
--

CREATE SEQUENCE unidade_faixa_etaria_idfeu_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: unidade_faixa_etaria_idfeu_seq; Type: SEQUENCE SET; Schema: alimentos; Owner: -
--

SELECT pg_catalog.setval('unidade_faixa_etaria_idfeu_seq', 1, false);


--
-- Name: unidade_faixa_etaria; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE unidade_faixa_etaria (
    idfeu integer DEFAULT nextval('unidade_faixa_etaria_idfeu_seq'::regclass) NOT NULL,
    iduni integer NOT NULL,
    idfae integer NOT NULL,
    num_inscritos integer NOT NULL,
    num_matriculados integer NOT NULL
);


--
-- Name: unidade_produto; Type: TABLE; Schema: alimentos; Owner: -; Tablespace:
--

CREATE TABLE unidade_produto (
    idunp character varying(20) NOT NULL,
    idcli character varying(10) NOT NULL,
    descricao character varying(50) NOT NULL,
    peso numeric NOT NULL
);


SET search_path = cadastro, pg_catalog;

--
-- Name: aviso_nome; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE aviso_nome (
    idpes numeric(8,0) NOT NULL,
    aviso numeric(1,0) NOT NULL,
    CONSTRAINT ck_aviso_nome_aviso CHECK (((aviso >= (1)::numeric) AND (aviso <= (4)::numeric)))
);


--
-- Name: deficiencia_cod_deficiencia_seq; Type: SEQUENCE; Schema: cadastro; Owner: -
--

CREATE SEQUENCE deficiencia_cod_deficiencia_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: deficiencia_cod_deficiencia_seq; Type: SEQUENCE SET; Schema: cadastro; Owner: -
--

SELECT pg_catalog.setval('deficiencia_cod_deficiencia_seq', 1, false);


--
-- Name: deficiencia; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE deficiencia (
    cod_deficiencia integer DEFAULT nextval('deficiencia_cod_deficiencia_seq'::regclass) NOT NULL,
    nm_deficiencia character varying(70) NOT NULL
);


--
-- Name: documento; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE documento (
    idpes numeric(8,0) NOT NULL,
    rg numeric(20,0),
    data_exp_rg date,
    sigla_uf_exp_rg character(2),
    tipo_cert_civil numeric(2,0),
    num_termo numeric(8,0),
    num_livro character varying(8),
    num_folha numeric(4,0),
    data_emissao_cert_civil date,
    sigla_uf_cert_civil character(2),
    cartorio_cert_civil character varying(150),
    num_cart_trabalho numeric(9,0),
    serie_cart_trabalho numeric(5,0),
    data_emissao_cart_trabalho date,
    sigla_uf_cart_trabalho character(2),
    num_tit_eleitor numeric(13,0),
    zona_tit_eleitor numeric(4,0),
    secao_tit_eleitor numeric(4,0),
    idorg_exp_rg integer,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_documento_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_documento_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_documento_tipo_cert CHECK (((tipo_cert_civil >= (91)::numeric) AND (tipo_cert_civil <= (92)::numeric)))
);


--
-- Name: endereco_externo; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE endereco_externo (
    idpes numeric(8,0) NOT NULL,
    tipo numeric(1,0) NOT NULL,
    idtlog character varying(5) NOT NULL,
    logradouro character varying(150) NOT NULL,
    numero numeric(6,0),
    letra character(1),
    complemento character varying(20),
    bairro character varying(40),
    cep numeric(8,0),
    cidade character varying(60) NOT NULL,
    sigla_uf character(2) NOT NULL,
    reside_desde date,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    bloco character varying(20),
    andar numeric(2,0),
    apartamento numeric(6,0),
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    zona_localizacao integer DEFAULT 1,
    CONSTRAINT ck_endereco_externo_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_endereco_externo_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_endereco_externo_tipo CHECK (((tipo >= (1)::numeric) AND (tipo <= (3)::numeric)))
);


--
-- Name: endereco_pessoa; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE endereco_pessoa (
    idpes numeric(8,0) NOT NULL,
    tipo numeric(1,0) NOT NULL,
    cep numeric(8,0) NOT NULL,
    idlog numeric(6,0) NOT NULL,
    numero numeric(6,0),
    letra character(1),
    complemento character varying(20),
    reside_desde date,
    idbai numeric(6,0) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    bloco character varying(20),
    andar numeric(2,0),
    apartamento numeric(6,0),
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_endereco_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_endereco_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_endereco_pessoa_tipo CHECK (((tipo >= (1)::numeric) AND (tipo <= (3)::numeric)))
);


--
-- Name: escolaridade; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE escolaridade (
    idesco numeric(2,0) NOT NULL,
    descricao character varying(60) NOT NULL
);


--
-- Name: estado_civil; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE estado_civil (
    ideciv numeric(1,0) NOT NULL,
    descricao character varying(15) NOT NULL
);


--
-- Name: fisica; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fisica (
    idpes numeric(8,0) NOT NULL,
    data_nasc date,
    sexo character(1),
    idpes_mae numeric(8,0),
    idpes_pai numeric(8,0),
    idpes_responsavel numeric(8,0),
    idesco numeric(2,0),
    ideciv numeric(1,0),
    idpes_con numeric(8,0),
    data_uniao date,
    data_obito date,
    nacionalidade numeric(1,0),
    idpais_estrangeiro numeric(3,0),
    data_chegada_brasil date,
    idmun_nascimento numeric(6,0),
    ultima_empresa character varying(150),
    idocup numeric(6,0),
    nome_mae character varying(150),
    nome_pai character varying(150),
    nome_conjuge character varying(150),
    nome_responsavel character varying(150),
    justificativa_provisorio character varying(150),
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    ref_cod_sistema integer,
    cpf numeric(11,0),
    ref_cod_religiao integer,
    CONSTRAINT ck_fisica_nacionalidade CHECK (((nacionalidade >= (1)::numeric) AND (nacionalidade <= (3)::numeric))),
    CONSTRAINT ck_fisica_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_fisica_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_fisica_sexo CHECK (((sexo = 'M'::bpchar) OR (sexo = 'F'::bpchar)))
);


--
-- Name: fisica_cpf; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fisica_cpf (
    idpes numeric(8,0) NOT NULL,
    cpf numeric(11,0) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_fisica_cpf_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_fisica_cpf_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: fisica_deficiencia; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fisica_deficiencia (
    ref_idpes integer NOT NULL,
    ref_cod_deficiencia integer NOT NULL
);


--
-- Name: fisica_foto; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fisica_foto (
    idpes integer NOT NULL,
    caminho character varying(255)
);


--
-- Name: fisica_raca; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fisica_raca (
    ref_idpes integer NOT NULL,
    ref_cod_raca integer NOT NULL
);


--
-- Name: fisica_sangue; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fisica_sangue (
    idpes numeric(8,0) NOT NULL,
    grupo character(2) NOT NULL,
    rh smallint NOT NULL
);


--
-- Name: fone_pessoa; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE fone_pessoa (
    idpes numeric(8,0) NOT NULL,
    tipo numeric(1,0) NOT NULL,
    ddd numeric(3,0) NOT NULL,
    fone numeric(11,0) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_fone_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_fone_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_fone_pessoa_tipo CHECK (((tipo >= (1)::numeric) AND (tipo <= (4)::numeric)))
);


--
-- Name: funcionario; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE funcionario (
    matricula numeric(8,0) NOT NULL,
    idins integer NOT NULL,
    idset integer,
    idpes numeric(8,0) NOT NULL,
    situacao character(1) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_funcionario_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_funcionario_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_funcionario_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: historico_cartao; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE historico_cartao (
    idpes_cidadao numeric(8,0) NOT NULL,
    data_hora timestamp without time zone NOT NULL,
    idpes_emitiu numeric(8,0) NOT NULL,
    tipo character(1) NOT NULL,
    CONSTRAINT ck_historico_cartao_tipo CHECK (((tipo = 'P'::bpchar) OR (tipo = 'D'::bpchar)))
);


--
-- Name: juridica; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE juridica (
    idpes numeric(8,0) NOT NULL,
    cnpj numeric(14,0) NOT NULL,
    insc_estadual numeric(20,0),
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    fantasia character varying(255),
    capital_social character varying(255),
    CONSTRAINT ck_juridica_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_juridica_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: ocupacao; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE ocupacao (
    idocup numeric(6,0) NOT NULL,
    descricao character varying(250) NOT NULL
);


--
-- Name: orgao_emissor_rg_idorg_rg_seq; Type: SEQUENCE; Schema: cadastro; Owner: -
--

CREATE SEQUENCE orgao_emissor_rg_idorg_rg_seq
    START WITH 30
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: orgao_emissor_rg_idorg_rg_seq; Type: SEQUENCE SET; Schema: cadastro; Owner: -
--

SELECT pg_catalog.setval('orgao_emissor_rg_idorg_rg_seq', 30, false);


--
-- Name: orgao_emissor_rg; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE orgao_emissor_rg (
    idorg_rg integer DEFAULT nextval('orgao_emissor_rg_idorg_rg_seq'::regclass) NOT NULL,
    sigla character varying(20) NOT NULL,
    descricao character varying(60) NOT NULL,
    situacao character(1) NOT NULL,
    CONSTRAINT ck_orgao_emissor_rg_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: pessoa; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE pessoa (
    idpes numeric(8,0) DEFAULT nextval(('cadastro.seq_pessoa'::text)::regclass) NOT NULL,
    nome character varying(150) NOT NULL,
    idpes_cad numeric(8,0),
    data_cad timestamp without time zone NOT NULL,
    url character varying(60),
    tipo character(1) NOT NULL,
    idpes_rev numeric(8,0),
    data_rev timestamp without time zone,
    email character varying(50),
    situacao character(1) NOT NULL,
    origem_gravacao character(1) NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_pessoa_situacao CHECK ((((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)) OR (situacao = 'P'::bpchar))),
    CONSTRAINT ck_pessoa_tipo CHECK (((tipo = 'F'::bpchar) OR (tipo = 'J'::bpchar)))
);


--
-- Name: pessoa_fonetico; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE pessoa_fonetico (
    idpes numeric(8,0) NOT NULL,
    fonema character varying(30) NOT NULL
);


--
-- Name: raca_cod_raca_seq; Type: SEQUENCE; Schema: cadastro; Owner: -
--

CREATE SEQUENCE raca_cod_raca_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: raca_cod_raca_seq; Type: SEQUENCE SET; Schema: cadastro; Owner: -
--

SELECT pg_catalog.setval('raca_cod_raca_seq', 1, false);


--
-- Name: raca; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE raca (
    cod_raca integer DEFAULT nextval('raca_cod_raca_seq'::regclass) NOT NULL,
    idpes_exc integer,
    idpes_cad integer NOT NULL,
    nm_raca character varying(50) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo boolean DEFAULT false
);


--
-- Name: religiao_cod_religiao_seq; Type: SEQUENCE; Schema: cadastro; Owner: -
--

CREATE SEQUENCE religiao_cod_religiao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: religiao_cod_religiao_seq; Type: SEQUENCE SET; Schema: cadastro; Owner: -
--

SELECT pg_catalog.setval('religiao_cod_religiao_seq', 1, false);


SET default_with_oids = false;

--
-- Name: religiao; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE religiao (
    cod_religiao integer DEFAULT nextval('religiao_cod_religiao_seq'::regclass) NOT NULL,
    idpes_exc integer,
    idpes_cad integer NOT NULL,
    nm_religiao character varying(50) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo boolean DEFAULT false
);


--
-- Name: seq_pessoa; Type: SEQUENCE; Schema: cadastro; Owner: -
--

CREATE SEQUENCE seq_pessoa
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: seq_pessoa; Type: SEQUENCE SET; Schema: cadastro; Owner: -
--

SELECT pg_catalog.setval('seq_pessoa', 1, true);


SET default_with_oids = true;

--
-- Name: socio; Type: TABLE; Schema: cadastro; Owner: -; Tablespace:
--

CREATE TABLE socio (
    idpes_juridica numeric(8,0) NOT NULL,
    idpes_fisica numeric(8,0) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_socio_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_socio_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


SET search_path = public, pg_catalog;

--
-- Name: bairro; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE bairro (
    idmun numeric(6,0) NOT NULL,
    geom character varying,
    idbai numeric(6,0) DEFAULT nextval(('public.seq_bairro'::text)::regclass) NOT NULL,
    nome character varying(80) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    zona_localizacao integer DEFAULT 1,
    CONSTRAINT ck_bairro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_bairro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: logradouro; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE logradouro (
    idlog numeric(6,0) DEFAULT nextval(('public.seq_logradouro'::text)::regclass) NOT NULL,
    idtlog character varying(5) NOT NULL,
    nome character varying(150) NOT NULL,
    idmun numeric(6,0) NOT NULL,
    geom character varying,
    ident_oficial character(1),
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_logradouro_ident_oficial CHECK (((ident_oficial = 'S'::bpchar) OR (ident_oficial = 'N'::bpchar))),
    CONSTRAINT ck_logradouro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_logradouro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: municipio; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE municipio (
    idmun numeric(6,0) DEFAULT nextval(('public.seq_municipio'::text)::regclass) NOT NULL,
    nome character varying(60) NOT NULL,
    sigla_uf character(2) NOT NULL,
    area_km2 numeric(6,0),
    idmreg numeric(2,0),
    idasmun numeric(2,0),
    cod_ibge numeric(6,0),
    geom character varying,
    tipo character(1) NOT NULL,
    idmun_pai numeric(6,0),
    idpes_rev numeric,
    idpes_cad numeric,
    data_rev timestamp without time zone,
    data_cad timestamp without time zone NOT NULL,
    origem_gravacao character(1) NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_municipio_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_municipio_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_municipio_tipo CHECK (((((tipo = 'D'::bpchar) OR (tipo = 'M'::bpchar)) OR (tipo = 'P'::bpchar)) OR (tipo = 'R'::bpchar)))
);


SET search_path = cadastro, pg_catalog;

--
-- Name: v_endereco; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_endereco AS
    SELECT e.idpes, e.cep, e.idlog, e.numero, e.letra, e.complemento, e.idbai, e.bloco, e.andar, e.apartamento, l.nome AS logradouro, l.idtlog, b.nome AS bairro, m.nome AS cidade, m.sigla_uf, b.zona_localizacao FROM endereco_pessoa e, public.logradouro l, public.bairro b, public.municipio m WHERE ((((e.idlog = l.idlog) AND (e.idbai = b.idbai)) AND (b.idmun = m.idmun)) AND (e.tipo = (1)::numeric)) UNION SELECT e.idpes, e.cep, NULL::"unknown" AS idlog, e.numero, e.letra, e.complemento, NULL::"unknown" AS idbai, e.bloco, e.andar, e.apartamento, e.logradouro, e.idtlog, e.bairro, e.cidade, e.sigla_uf, e.zona_localizacao FROM endereco_externo e WHERE (e.tipo = (1)::numeric);


--
-- Name: v_fone_pessoa; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_fone_pessoa AS
    SELECT DISTINCT t.idpes, (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (1)::numeric) AND (t.idpes = t1.idpes))) AS ddd_1, (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (1)::numeric) AND (t.idpes = t1.idpes))) AS fone_1, (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (2)::numeric) AND (t.idpes = t1.idpes))) AS ddd_2, (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (2)::numeric) AND (t.idpes = t1.idpes))) AS fone_2, (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (3)::numeric) AND (t.idpes = t1.idpes))) AS ddd_mov, (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (3)::numeric) AND (t.idpes = t1.idpes))) AS fone_mov, (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (4)::numeric) AND (t.idpes = t1.idpes))) AS ddd_fax, (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (4)::numeric) AND (t.idpes = t1.idpes))) AS fone_fax FROM fone_pessoa t ORDER BY t.idpes, (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (1)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (1)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (2)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (2)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (3)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (3)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.ddd FROM fone_pessoa t1 WHERE ((t1.tipo = (4)::numeric) AND (t.idpes = t1.idpes))), (SELECT t1.fone FROM fone_pessoa t1 WHERE ((t1.tipo = (4)::numeric) AND (t.idpes = t1.idpes)));


--
-- Name: v_pessoa_fisica; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_pessoa_fisica AS
    SELECT p.idpes, p.nome, p.url, p.email, p.situacao, f.data_nasc, f.sexo, f.cpf, f.ref_cod_sistema, f.idesco FROM pessoa p, fisica f WHERE (p.idpes = f.idpes);


--
-- Name: v_pessoa_fisica_simples; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_pessoa_fisica_simples AS
    SELECT p.idpes, (SELECT fisica_cpf.cpf FROM fisica_cpf WHERE (fisica_cpf.idpes = p.idpes)) AS cpf, f.ref_cod_sistema, f.idesco FROM pessoa p, fisica f WHERE (p.idpes = f.idpes);


--
-- Name: v_pessoa_fj; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_pessoa_fj AS
    SELECT p.idpes, p.nome, (SELECT fisica.ref_cod_sistema FROM fisica WHERE (fisica.idpes = p.idpes)) AS ref_cod_sistema, (SELECT juridica.fantasia FROM juridica WHERE (juridica.idpes = p.idpes)) AS fantasia, p.tipo, COALESCE((SELECT fisica.cpf FROM fisica WHERE (fisica.idpes = p.idpes)), (SELECT juridica.cnpj FROM juridica WHERE (juridica.idpes = p.idpes))) AS id_federal FROM pessoa p;


--
-- Name: v_pessoa_juridica; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_pessoa_juridica AS
    SELECT j.idpes, j.fantasia, j.cnpj, j.insc_estadual, j.capital_social, (SELECT pessoa.nome FROM pessoa WHERE (pessoa.idpes = j.idpes)) AS nome FROM juridica j;


--
-- Name: v_pessoafj_count; Type: VIEW; Schema: cadastro; Owner: -
--

CREATE VIEW v_pessoafj_count AS
    SELECT fisica.ref_cod_sistema, fisica.cpf AS id_federal FROM fisica UNION ALL SELECT NULL::"unknown" AS ref_cod_sistema, juridica.cnpj AS id_federal FROM juridica;


SET search_path = consistenciacao, pg_catalog;

--
-- Name: campo_consistenciacao; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE campo_consistenciacao (
    idcam numeric(3,0) NOT NULL,
    campo character varying(50) NOT NULL,
    permite_regra_cadastrada character(1) NOT NULL,
    tamanho_maximo numeric(4,0),
    CONSTRAINT ck_campo_consistenciacao_permite_regra CHECK (((permite_regra_cadastrada = 'S'::bpchar) OR (permite_regra_cadastrada = 'N'::bpchar)))
);


--
-- Name: campo_metadado_id_campo_met_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE campo_metadado_id_campo_met_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: campo_metadado_id_campo_met_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('campo_metadado_id_campo_met_seq', 1, false);


--
-- Name: campo_metadado; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE campo_metadado (
    id_campo_met integer DEFAULT nextval('campo_metadado_id_campo_met_seq'::regclass) NOT NULL,
    idmet integer NOT NULL,
    idreg integer,
    idcam numeric(3,0),
    posicao_inicial numeric(5,0),
    posicao_final numeric(5,0),
    posicao_coluna numeric(5,0),
    credibilidade numeric(1,0) NOT NULL,
    data_atualizacao character(1) NOT NULL,
    CONSTRAINT ck_cam_met_campo_cred CHECK (((credibilidade >= (2)::numeric) AND (credibilidade <= (4)::numeric))),
    CONSTRAINT ck_cam_met_data_atualizacao CHECK (((data_atualizacao = 'S'::bpchar) OR (data_atualizacao = 'N'::bpchar)))
);


--
-- Name: confrontacao_idcon_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE confrontacao_idcon_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: confrontacao_idcon_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('confrontacao_idcon_seq', 1, false);


--
-- Name: confrontacao; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE confrontacao (
    idcon integer DEFAULT nextval('confrontacao_idcon_seq'::regclass) NOT NULL,
    idins integer NOT NULL,
    idpes integer NOT NULL,
    idmet integer NOT NULL,
    arquivo_fonte_dados character varying(250) NOT NULL,
    ignorar_reg_fonte date,
    desconsiderar_reg_cred_maxima date,
    data_hora timestamp without time zone NOT NULL
);


--
-- Name: fonte_idfon_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE fonte_idfon_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: fonte_idfon_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('fonte_idfon_seq', 1, false);


--
-- Name: fonte; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE fonte (
    idfon integer DEFAULT nextval('fonte_idfon_seq'::regclass) NOT NULL,
    nome character varying(60) NOT NULL,
    situacao character(1) NOT NULL,
    CONSTRAINT ck_fonte_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: historico_campo; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE historico_campo (
    idpes numeric(8,0) NOT NULL,
    idcam numeric(3,0) NOT NULL,
    credibilidade numeric(1,0) NOT NULL,
    data_hora timestamp without time zone NOT NULL,
    CONSTRAINT ck_historico_campo_cred CHECK (((credibilidade >= (1)::numeric) AND (credibilidade <= (5)::numeric)))
);


--
-- Name: incoerencia_idinc_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE incoerencia_idinc_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: incoerencia_idinc_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('incoerencia_idinc_seq', 1, false);


--
-- Name: incoerencia; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE incoerencia (
    idinc integer DEFAULT nextval('incoerencia_idinc_seq'::regclass) NOT NULL,
    idcon integer NOT NULL,
    data_gravacao date NOT NULL,
    ultima_etapa numeric(1,0) NOT NULL,
    cpf_cnpj numeric(14,0),
    nome character varying(150),
    email character varying(100),
    url character varying(60),
    data_nasc character varying(20),
    fantasia character varying(50),
    insc_estadual numeric(10,0),
    sexo character varying(10),
    nome_mae character varying(150),
    nome_pai character varying(150),
    nome_responsavel character varying(150),
    nome_conjuge character varying(150),
    ultima_empresa character varying(150),
    ocupacao character varying(250),
    escolaridade character varying(60),
    estado_civil character varying(15),
    pais_estrangeiro character varying(60),
    data_chegada_brasil character varying(20),
    data_obito character varying(20),
    data_uniao character varying(20)
);


--
-- Name: incoerencia_documento_id_inc_doc_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE incoerencia_documento_id_inc_doc_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: incoerencia_documento_id_inc_doc_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('incoerencia_documento_id_inc_doc_seq', 1, false);


--
-- Name: incoerencia_documento; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE incoerencia_documento (
    id_inc_doc integer DEFAULT nextval('incoerencia_documento_id_inc_doc_seq'::regclass) NOT NULL,
    idinc integer NOT NULL,
    rg numeric(10,0),
    orgao_exp_rg character varying(20),
    data_exp_rg character varying(20),
    sigla_uf_rg_exp character varying(30),
    tipo_cert_civil numeric(2,0),
    num_termo numeric(8,0),
    num_livro numeric(8,0),
    num_folha numeric(4,0),
    data_emissao_cert_civil character varying(20),
    cartorio_cert_civil character varying(150),
    sigla_uf_cert_civil character varying(30),
    num_cart_trabalho numeric(7,0),
    serie_cart_trabalho numeric(5,0),
    data_emissao_cart_trabalho character varying(20),
    sigla_uf_cart_trabalho character varying(30),
    num_tit_eleitor numeric(13,0),
    zona_tit_eleitor numeric(4,0),
    secao_tit_eleitor numeric(4,0)
);


--
-- Name: incoerencia_endereco_id_inc_end_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE incoerencia_endereco_id_inc_end_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: incoerencia_endereco_id_inc_end_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('incoerencia_endereco_id_inc_end_seq', 1, false);


--
-- Name: incoerencia_endereco; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE incoerencia_endereco (
    id_inc_end integer DEFAULT nextval('incoerencia_endereco_id_inc_end_seq'::regclass) NOT NULL,
    idinc integer NOT NULL,
    tipo character varying(60) NOT NULL,
    tipo_logradouro character varying(15),
    logradouro character varying(150),
    numero numeric(6,0),
    letra character(1),
    complemento character varying(20),
    bairro character varying(40),
    cep numeric(8,0),
    cidade character varying(60),
    uf character varying(30),
    CONSTRAINT ck_incoerencia_endereco_tipo CHECK ((((tipo)::text >= ((1)::numeric)::text) AND ((tipo)::text <= ((3)::numeric)::text)))
);


--
-- Name: incoerencia_fone_id_inc_fone_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE incoerencia_fone_id_inc_fone_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: incoerencia_fone_id_inc_fone_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('incoerencia_fone_id_inc_fone_seq', 1, false);


--
-- Name: incoerencia_fone; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE incoerencia_fone (
    id_inc_fone integer DEFAULT nextval('incoerencia_fone_id_inc_fone_seq'::regclass) NOT NULL,
    idinc integer NOT NULL,
    tipo character varying(60) NOT NULL,
    ddd numeric(3,0),
    fone numeric(8,0),
    CONSTRAINT ck_incoerencia_fone_tipo CHECK ((((tipo)::text >= ((1)::numeric)::text) AND ((tipo)::text <= ((4)::numeric)::text)))
);


--
-- Name: incoerencia_pessoa_possivel; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE incoerencia_pessoa_possivel (
    idinc integer NOT NULL,
    idpes numeric(8,0) NOT NULL
);


--
-- Name: incoerencia_tipo_incoerencia; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE incoerencia_tipo_incoerencia (
    id_tipo_inc numeric(3,0) NOT NULL,
    idinc integer NOT NULL
);


--
-- Name: metadado_idmet_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE metadado_idmet_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: metadado_idmet_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('metadado_idmet_seq', 1, false);


--
-- Name: metadado; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE metadado (
    idmet integer DEFAULT nextval('metadado_idmet_seq'::regclass) NOT NULL,
    idfon integer NOT NULL,
    nome character varying(60) NOT NULL,
    situacao character(1) NOT NULL,
    separador character(1),
    CONSTRAINT ck_metadado_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: ocorrencia_regra_campo; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE ocorrencia_regra_campo (
    idreg integer NOT NULL,
    conteudo_padrao character varying(60) NOT NULL,
    ocorrencias text NOT NULL
);


--
-- Name: regra_campo_idreg_seq; Type: SEQUENCE; Schema: consistenciacao; Owner: -
--

CREATE SEQUENCE regra_campo_idreg_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: regra_campo_idreg_seq; Type: SEQUENCE SET; Schema: consistenciacao; Owner: -
--

SELECT pg_catalog.setval('regra_campo_idreg_seq', 1, false);


--
-- Name: regra_campo; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE regra_campo (
    idreg integer DEFAULT nextval('regra_campo_idreg_seq'::regclass) NOT NULL,
    nome character varying(60) NOT NULL,
    tipo character(1) NOT NULL,
    CONSTRAINT ck_regra_campo_tipo CHECK (((tipo = 'S'::bpchar) OR (tipo = 'N'::bpchar)))
);


--
-- Name: temp_cadastro_unificacao_cmf; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE temp_cadastro_unificacao_cmf (
    idpes numeric(8,0) NOT NULL,
    nome character varying(150) NOT NULL,
    cpf_cnpj character varying(14),
    rg character varying(10),
    uf_rg character varying(50),
    data_nascimento character varying(10),
    logradouro character varying(150),
    cep character varying(10),
    bairro character varying(40),
    numero character varying(6),
    cidade_end character varying(60),
    uf_end character varying(2),
    complemento character varying(20),
    fone character varying(14),
    nome_mae character varying(150),
    nome_mae_idpes character varying(150),
    data_cadastro character varying(10),
    data_atualizacao character varying(10),
    situacao character varying(15),
    tipo_pess character varying(1),
    nome_fantasia character varying(50),
    inscr_estadual character varying(10)
);


--
-- Name: temp_cadastro_unificacao_siam; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE temp_cadastro_unificacao_siam (
    idpes numeric(8,0) NOT NULL,
    nome character varying(40) NOT NULL,
    cpf_cnpj character varying(14),
    rg character varying(15),
    logradouro character varying(40),
    cep character varying(10),
    bairro character varying(20),
    cidade_end character varying(20),
    uf_end character varying(2),
    fone character varying(14),
    data_cadastro character varying(10)
);


--
-- Name: tipo_incoerencia; Type: TABLE; Schema: consistenciacao; Owner: -; Tablespace:
--

CREATE TABLE tipo_incoerencia (
    id_tipo_inc numeric(3,0) NOT NULL,
    idcam numeric(3,0) NOT NULL,
    descricao character varying(250) NOT NULL
);


SET search_path = historico, pg_catalog;

--
-- Name: bairro; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE bairro (
    idbai numeric(6,0) NOT NULL,
    idmun numeric(6,0) NOT NULL,
    nome character varying(80) NOT NULL,
    geom character varying,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_bairro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_bairro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: cep_logradouro; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE cep_logradouro (
    cep numeric(8,0) NOT NULL,
    idlog numeric(6,0) NOT NULL,
    nroini numeric(6,0),
    nrofin numeric(6,0),
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_cep_logradouro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_cep_logradouro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: cep_logradouro_bairro; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE cep_logradouro_bairro (
    idbai numeric(6,0) NOT NULL,
    idlog numeric(6,0) NOT NULL,
    cep numeric(8,0) NOT NULL,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_cep_logradouro_bairro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_cep_logradouro_bairro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: documento; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE documento (
    idpes numeric(8,0) NOT NULL,
    rg numeric(20,0),
    data_exp_rg date,
    sigla_uf_exp_rg character(2),
    tipo_cert_civil numeric(2,0),
    num_termo numeric(8,0),
    num_livro character varying(8),
    num_folha numeric(4,0),
    data_emissao_cert_civil date,
    sigla_uf_cert_civil character(2),
    cartorio_cert_civil character varying(150),
    num_cart_trabalho numeric(7,0),
    serie_cart_trabalho numeric(5,0),
    data_emissao_cart_trabalho date,
    sigla_uf_cart_trabalho character(2),
    num_tit_eleitor numeric(13,0),
    zona_tit_eleitor numeric(4,0),
    secao_tit_eleitor numeric(4,0),
    idorg_exp_rg integer,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_documento_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_documento_tipo_cert CHECK (((tipo_cert_civil >= (91)::numeric) AND (tipo_cert_civil <= (92)::numeric))),
    CONSTRAINT ck_fone_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: endereco_externo; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE endereco_externo (
    idpes numeric(8,0) NOT NULL,
    tipo numeric(1,0) NOT NULL,
    idtlog character varying(5) NOT NULL,
    logradouro character varying(150) NOT NULL,
    numero numeric(6,0),
    letra character(1),
    complemento character varying(20),
    bairro character varying(40),
    cep numeric(8,0),
    cidade character varying(60) NOT NULL,
    sigla_uf character(2) NOT NULL,
    reside_desde date,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_endereco_externo_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_endereco_externo_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_endereco_externo_tipo CHECK (((tipo >= (1)::numeric) AND (tipo <= (3)::numeric)))
);


--
-- Name: endereco_pessoa; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE endereco_pessoa (
    idpes numeric(8,0) NOT NULL,
    tipo numeric(1,0) NOT NULL,
    cep numeric(8,0) NOT NULL,
    idlog numeric(6,0) NOT NULL,
    idbai numeric(6,0) NOT NULL,
    numero numeric(6,0),
    letra character(1),
    complemento character varying(20),
    reside_desde date,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_endereco_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_endereco_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_endereco_pessoa_tipo CHECK (((tipo >= (1)::numeric) AND (tipo <= (3)::numeric)))
);


--
-- Name: fisica; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE fisica (
    idpes numeric(8,0) NOT NULL,
    data_nasc date,
    sexo character(1),
    idpes_mae numeric(8,0),
    idpes_pai numeric(8,0),
    idpes_responsavel numeric(8,0),
    idesco numeric(2,0),
    ideciv numeric(1,0),
    idpes_con numeric(8,0),
    data_uniao date,
    data_obito date,
    nacionalidade numeric(1,0),
    idpais_estrangeiro numeric(3,0),
    data_chegada_brasil date,
    idmun_nascimento numeric(6,0),
    ultima_empresa character varying(150),
    idocup numeric(6,0),
    nome_mae character varying(150),
    nome_pai character varying(150),
    nome_conjuge character varying(150),
    nome_responsavel character varying(150),
    justificativa_provisorio character varying(150),
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_fisica_nacionalidade CHECK (((nacionalidade >= (1)::numeric) AND (nacionalidade <= (3)::numeric))),
    CONSTRAINT ck_fisica_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_fisica_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_fisica_sexo CHECK (((sexo = 'M'::bpchar) OR (sexo = 'F'::bpchar)))
);


--
-- Name: fisica_cpf; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE fisica_cpf (
    idpes numeric(8,0) NOT NULL,
    cpf numeric(11,0) NOT NULL,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_fisica_cpf_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_fone_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar)))
);


--
-- Name: fone_pessoa; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE fone_pessoa (
    idpes numeric(8,0) NOT NULL,
    tipo numeric(1,0) NOT NULL,
    ddd numeric(3,0) NOT NULL,
    fone numeric(11,0) NOT NULL,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_fone_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_fone_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_fone_pessoa_tipo CHECK (((tipo >= (1)::numeric) AND (tipo <= (4)::numeric)))
);


--
-- Name: funcionario; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE funcionario (
    matricula numeric(8,0) NOT NULL,
    idins integer NOT NULL,
    idset integer,
    idpes numeric(8,0) NOT NULL,
    situacao character(1) NOT NULL,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_fone_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_funcionario_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_funcionario_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: juridica; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE juridica (
    idpes numeric(8,0) NOT NULL,
    cnpj numeric(14,0) NOT NULL,
    insc_estadual numeric(20,0),
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    fantasia character varying(255),
    CONSTRAINT ck_juridica_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_juridica_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: logradouro; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE logradouro (
    idlog numeric(6,0) NOT NULL,
    idtlog character varying(5) NOT NULL,
    nome character varying(150) NOT NULL,
    idmun numeric(6,0) NOT NULL,
    geom character varying,
    ident_oficial character(1) NOT NULL,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_logradouro_ident_oficial CHECK (((ident_oficial = 'S'::bpchar) OR (ident_oficial = 'N'::bpchar))),
    CONSTRAINT ck_logradouro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_logradouro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


--
-- Name: municipio; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE municipio (
    idmun numeric(6,0) NOT NULL,
    nome character varying(60) NOT NULL,
    sigla_uf character(2) NOT NULL,
    area_km2 numeric(6,0),
    idmreg numeric(2,0),
    idasmun numeric(2,0),
    cod_ibge numeric(6,0),
    geom character varying,
    tipo character(1) NOT NULL,
    idmun_pai numeric(6,0),
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_municipio_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_municipio_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_municipio_tipo CHECK (((((tipo = 'D'::bpchar) OR (tipo = 'M'::bpchar)) OR (tipo = 'P'::bpchar)) OR (tipo = 'R'::bpchar)))
);


--
-- Name: pessoa; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE pessoa (
    idpes numeric(8,0) NOT NULL,
    nome character varying(150) NOT NULL,
    idpes_cad numeric(8,0),
    data_cad timestamp without time zone NOT NULL,
    url character varying(60),
    tipo character(1) NOT NULL,
    idpes_rev numeric(8,0),
    data_rev timestamp without time zone,
    email character varying(50),
    situacao character(1) NOT NULL,
    origem_gravacao character(1) NOT NULL,
    idsis_rev numeric,
    idsis_cad numeric NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_pessoa_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_pessoa_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_pessoa_situacao CHECK ((((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)) OR (situacao = 'P'::bpchar))),
    CONSTRAINT ck_pessoa_tipo CHECK (((tipo = 'F'::bpchar) OR (tipo = 'J'::bpchar)))
);


--
-- Name: socio; Type: TABLE; Schema: historico; Owner: -; Tablespace:
--

CREATE TABLE socio (
    idpes_juridica numeric(8,0) NOT NULL,
    idpes_fisica numeric(8,0) NOT NULL,
    idpes_rev numeric,
    idsis_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    idsis_cad numeric NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    CONSTRAINT ck_socio_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar))),
    CONSTRAINT ck_socio_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar)))
);


SET search_path = modules, pg_catalog;

SET default_with_oids = false;

--
-- Name: area_conhecimento; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE area_conhecimento (
    id integer NOT NULL,
    instituicao_id integer NOT NULL,
    nome character varying(40) NOT NULL
);


--
-- Name: area_conhecimento_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE area_conhecimento_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: area_conhecimento_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE area_conhecimento_id_seq OWNED BY area_conhecimento.id;


--
-- Name: area_conhecimento_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('area_conhecimento_id_seq', 1, false);


--
-- Name: calendario_turma; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE calendario_turma (
    calendario_ano_letivo_id integer NOT NULL,
    ano integer NOT NULL,
    mes integer NOT NULL,
    dia integer NOT NULL,
    turma_id integer NOT NULL
);


--
-- Name: componente_curricular; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE componente_curricular (
    id integer NOT NULL,
    instituicao_id integer NOT NULL,
    area_conhecimento_id integer NOT NULL,
    nome character varying(100) NOT NULL,
    abreviatura character varying(15) NOT NULL,
    tipo_base smallint NOT NULL
);


--
-- Name: componente_curricular_ano_escolar; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE componente_curricular_ano_escolar (
    componente_curricular_id integer NOT NULL,
    ano_escolar_id integer NOT NULL,
    carga_horaria numeric(6,3)
);


--
-- Name: componente_curricular_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE componente_curricular_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: componente_curricular_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE componente_curricular_id_seq OWNED BY componente_curricular.id;


--
-- Name: componente_curricular_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('componente_curricular_id_seq', 1, false);


--
-- Name: componente_curricular_turma; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE componente_curricular_turma (
    componente_curricular_id integer NOT NULL,
    ano_escolar_id integer NOT NULL,
    escola_id integer NOT NULL,
    turma_id integer NOT NULL,
    carga_horaria numeric(6,3)
);


--
-- Name: docente_licenciatura; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE docente_licenciatura (
    id integer NOT NULL,
    servidor_id integer NOT NULL,
    licenciatura integer NOT NULL,
    curso_id integer,
    ano_conclusao integer NOT NULL,
    ies_id integer,
    user_id integer NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone
);


--
-- Name: docente_licenciatura_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE docente_licenciatura_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: docente_licenciatura_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE docente_licenciatura_id_seq OWNED BY docente_licenciatura.id;


--
-- Name: docente_licenciatura_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('docente_licenciatura_id_seq', 1, false);


--
-- Name: educacenso_cod_aluno; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE educacenso_cod_aluno (
    cod_aluno integer NOT NULL,
    cod_aluno_inep bigint NOT NULL,
    nome_inep character varying(255),
    fonte character varying(255),
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone
);


--
-- Name: educacenso_cod_docente; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE educacenso_cod_docente (
    cod_servidor integer NOT NULL,
    cod_docente_inep bigint NOT NULL,
    nome_inep character varying(255),
    fonte character varying(255),
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone
);


--
-- Name: educacenso_curso_superior; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE educacenso_curso_superior (
    id integer NOT NULL,
    curso_id character varying(100) NOT NULL,
    nome character varying(255) NOT NULL,
    classe_id integer NOT NULL,
    user_id integer NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone
);


--
-- Name: educacenso_curso_superior_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE educacenso_curso_superior_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: educacenso_curso_superior_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE educacenso_curso_superior_id_seq OWNED BY educacenso_curso_superior.id;


--
-- Name: educacenso_curso_superior_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('educacenso_curso_superior_id_seq', 124, true);


--
-- Name: educacenso_ies; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE educacenso_ies (
    id integer NOT NULL,
    ies_id integer NOT NULL,
    nome character varying(255) NOT NULL,
    dependencia_administrativa_id integer NOT NULL,
    tipo_instituicao_id integer NOT NULL,
    uf character(2),
    user_id integer NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone
);


--
-- Name: educacenso_ies_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE educacenso_ies_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: educacenso_ies_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE educacenso_ies_id_seq OWNED BY educacenso_ies.id;


--
-- Name: educacenso_ies_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('educacenso_ies_id_seq', 5301, true);


--
-- Name: falta_aluno; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE falta_aluno (
    id integer NOT NULL,
    matricula_id integer NOT NULL,
    tipo_falta smallint NOT NULL
);


--
-- Name: falta_aluno_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE falta_aluno_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: falta_aluno_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE falta_aluno_id_seq OWNED BY falta_aluno.id;


--
-- Name: falta_aluno_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('falta_aluno_id_seq', 1, false);


--
-- Name: falta_componente_curricular; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE falta_componente_curricular (
    id integer NOT NULL,
    falta_aluno_id integer NOT NULL,
    componente_curricular_id integer NOT NULL,
    quantidade integer DEFAULT 0,
    etapa character varying(2) NOT NULL
);


--
-- Name: falta_componente_curricular_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE falta_componente_curricular_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: falta_componente_curricular_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE falta_componente_curricular_id_seq OWNED BY falta_componente_curricular.id;


--
-- Name: falta_componente_curricular_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('falta_componente_curricular_id_seq', 1, false);


--
-- Name: falta_geral; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE falta_geral (
    id integer NOT NULL,
    falta_aluno_id integer NOT NULL,
    quantidade integer DEFAULT 0,
    etapa character varying(2) NOT NULL
);


--
-- Name: falta_geral_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE falta_geral_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: falta_geral_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE falta_geral_id_seq OWNED BY falta_geral.id;


--
-- Name: falta_geral_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('falta_geral_id_seq', 1, false);


--
-- Name: formula_media; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE formula_media (
    id integer NOT NULL,
    instituicao_id integer NOT NULL,
    nome character varying(50) NOT NULL,
    formula_media character varying(50) NOT NULL,
    tipo_formula smallint DEFAULT 1
);


--
-- Name: formula_media_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE formula_media_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: formula_media_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE formula_media_id_seq OWNED BY formula_media.id;


--
-- Name: formula_media_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('formula_media_id_seq', 1, false);


--
-- Name: nota_aluno; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE nota_aluno (
    id integer NOT NULL,
    matricula_id integer NOT NULL
);


--
-- Name: nota_aluno_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE nota_aluno_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: nota_aluno_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE nota_aluno_id_seq OWNED BY nota_aluno.id;


--
-- Name: nota_aluno_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('nota_aluno_id_seq', 1, false);


--
-- Name: nota_componente_curricular; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE nota_componente_curricular (
    id integer NOT NULL,
    nota_aluno_id integer NOT NULL,
    componente_curricular_id integer NOT NULL,
    nota numeric(5,3) DEFAULT 0,
    nota_arredondada character varying(5) DEFAULT 0,
    etapa character varying(2) NOT NULL
);


--
-- Name: nota_componente_curricular_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE nota_componente_curricular_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: nota_componente_curricular_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE nota_componente_curricular_id_seq OWNED BY nota_componente_curricular.id;


--
-- Name: nota_componente_curricular_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('nota_componente_curricular_id_seq', 1, false);


--
-- Name: nota_componente_curricular_media; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE nota_componente_curricular_media (
    nota_aluno_id integer NOT NULL,
    componente_curricular_id integer NOT NULL,
    media numeric(5,3) DEFAULT 0,
    media_arredondada character varying(5) DEFAULT 0,
    etapa character varying(2) NOT NULL
);


--
-- Name: parecer_aluno; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE parecer_aluno (
    id integer NOT NULL,
    matricula_id integer NOT NULL,
    parecer_descritivo smallint NOT NULL
);


--
-- Name: parecer_aluno_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE parecer_aluno_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: parecer_aluno_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE parecer_aluno_id_seq OWNED BY parecer_aluno.id;


--
-- Name: parecer_aluno_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('parecer_aluno_id_seq', 1, false);


--
-- Name: parecer_componente_curricular; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE parecer_componente_curricular (
    id integer NOT NULL,
    parecer_aluno_id integer NOT NULL,
    componente_curricular_id integer NOT NULL,
    parecer text,
    etapa character varying(2) NOT NULL
);


--
-- Name: parecer_componente_curricular_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE parecer_componente_curricular_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: parecer_componente_curricular_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE parecer_componente_curricular_id_seq OWNED BY parecer_componente_curricular.id;


--
-- Name: parecer_componente_curricular_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('parecer_componente_curricular_id_seq', 1, false);


--
-- Name: parecer_geral; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE parecer_geral (
    id integer NOT NULL,
    parecer_aluno_id integer NOT NULL,
    parecer text,
    etapa character varying(2) NOT NULL
);


--
-- Name: parecer_geral_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE parecer_geral_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: parecer_geral_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE parecer_geral_id_seq OWNED BY parecer_geral.id;


--
-- Name: parecer_geral_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('parecer_geral_id_seq', 1, false);


--
-- Name: regra_avaliacao; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE regra_avaliacao (
    id integer NOT NULL,
    instituicao_id integer NOT NULL,
    formula_media_id integer,
    formula_recuperacao_id integer DEFAULT 0,
    tabela_arredondamento_id integer,
    nome character varying(50) NOT NULL,
    tipo_nota smallint NOT NULL,
    tipo_progressao smallint NOT NULL,
    media numeric(5,3) DEFAULT 0.000,
    porcentagem_presenca numeric(6,3) DEFAULT 0.000,
    parecer_descritivo smallint DEFAULT 0,
    tipo_presenca smallint NOT NULL
);


--
-- Name: regra_avaliacao_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE regra_avaliacao_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: regra_avaliacao_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE regra_avaliacao_id_seq OWNED BY regra_avaliacao.id;


--
-- Name: regra_avaliacao_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('regra_avaliacao_id_seq', 1, false);


--
-- Name: tabela_arredondamento; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE tabela_arredondamento (
    id integer NOT NULL,
    instituicao_id integer NOT NULL,
    nome character varying(50) NOT NULL,
    tipo_nota smallint DEFAULT 1 NOT NULL
);


--
-- Name: tabela_arredondamento_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE tabela_arredondamento_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: tabela_arredondamento_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE tabela_arredondamento_id_seq OWNED BY tabela_arredondamento.id;


--
-- Name: tabela_arredondamento_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('tabela_arredondamento_id_seq', 1, false);


--
-- Name: tabela_arredondamento_valor; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE tabela_arredondamento_valor (
    id integer NOT NULL,
    tabela_arredondamento_id integer NOT NULL,
    nome character varying(5) NOT NULL,
    descricao character varying(25),
    valor_minimo numeric(5,3) NOT NULL,
    valor_maximo numeric(5,3) NOT NULL
);


--
-- Name: tabela_arredondamento_valor_id_seq; Type: SEQUENCE; Schema: modules; Owner: -
--

CREATE SEQUENCE tabela_arredondamento_valor_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: tabela_arredondamento_valor_id_seq; Type: SEQUENCE OWNED BY; Schema: modules; Owner: -
--

ALTER SEQUENCE tabela_arredondamento_valor_id_seq OWNED BY tabela_arredondamento_valor.id;


--
-- Name: tabela_arredondamento_valor_id_seq; Type: SEQUENCE SET; Schema: modules; Owner: -
--

SELECT pg_catalog.setval('tabela_arredondamento_valor_id_seq', 1, false);


--
-- Name: transporte_aluno; Type: TABLE; Schema: modules; Owner: -; Tablespace:
--

CREATE TABLE transporte_aluno (
    aluno_id integer NOT NULL,
    responsavel integer NOT NULL,
    user_id integer NOT NULL,
    created_at timestamp without time zone NOT NULL,
    updated_at timestamp without time zone
);


SET search_path = pmiacoes, pg_catalog;

--
-- Name: acao_governo_cod_acao_governo_seq; Type: SEQUENCE; Schema: pmiacoes; Owner: -
--

CREATE SEQUENCE acao_governo_cod_acao_governo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acao_governo_cod_acao_governo_seq; Type: SEQUENCE SET; Schema: pmiacoes; Owner: -
--

SELECT pg_catalog.setval('acao_governo_cod_acao_governo_seq', 1, false);


SET default_with_oids = true;

--
-- Name: acao_governo; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo (
    cod_acao_governo integer DEFAULT nextval('acao_governo_cod_acao_governo_seq'::regclass) NOT NULL,
    ref_funcionario_exc integer,
    ref_funcionario_cad integer NOT NULL,
    nm_acao character varying(255) NOT NULL,
    descricao text,
    data_inauguracao timestamp without time zone,
    valor double precision,
    destaque smallint DEFAULT (0)::smallint NOT NULL,
    status_acao smallint DEFAULT (0)::smallint NOT NULL,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    numero_acao smallint DEFAULT 0,
    categoria smallint,
    idbai bigint
);


--
-- Name: acao_governo_arquivo_cod_acao_governo_arquivo_seq; Type: SEQUENCE; Schema: pmiacoes; Owner: -
--

CREATE SEQUENCE acao_governo_arquivo_cod_acao_governo_arquivo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acao_governo_arquivo_cod_acao_governo_arquivo_seq; Type: SEQUENCE SET; Schema: pmiacoes; Owner: -
--

SELECT pg_catalog.setval('acao_governo_arquivo_cod_acao_governo_arquivo_seq', 1, false);


--
-- Name: acao_governo_arquivo; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo_arquivo (
    cod_acao_governo_arquivo integer DEFAULT nextval('acao_governo_arquivo_cod_acao_governo_arquivo_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_cod_acao_governo integer NOT NULL,
    nm_arquivo character varying(255) NOT NULL,
    caminho_arquivo character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: acao_governo_categoria; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo_categoria (
    ref_cod_categoria integer NOT NULL,
    ref_cod_acao_governo integer NOT NULL
);


--
-- Name: acao_governo_foto_cod_acao_governo_foto_seq; Type: SEQUENCE; Schema: pmiacoes; Owner: -
--

CREATE SEQUENCE acao_governo_foto_cod_acao_governo_foto_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acao_governo_foto_cod_acao_governo_foto_seq; Type: SEQUENCE SET; Schema: pmiacoes; Owner: -
--

SELECT pg_catalog.setval('acao_governo_foto_cod_acao_governo_foto_seq', 1, false);


--
-- Name: acao_governo_foto; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo_foto (
    cod_acao_governo_foto integer DEFAULT nextval('acao_governo_foto_cod_acao_governo_foto_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_cod_acao_governo integer NOT NULL,
    nm_foto character varying(255) NOT NULL,
    caminho character varying(255) NOT NULL,
    data_foto timestamp without time zone,
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: acao_governo_foto_portal; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo_foto_portal (
    ref_cod_acao_governo integer NOT NULL,
    ref_cod_foto_portal integer NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: acao_governo_noticia; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo_noticia (
    ref_cod_acao_governo integer NOT NULL,
    ref_cod_not_portal integer NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: acao_governo_setor; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE acao_governo_setor (
    ref_cod_acao_governo integer NOT NULL,
    ref_cod_setor integer NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: categoria_cod_categoria_seq; Type: SEQUENCE; Schema: pmiacoes; Owner: -
--

CREATE SEQUENCE categoria_cod_categoria_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: categoria_cod_categoria_seq; Type: SEQUENCE SET; Schema: pmiacoes; Owner: -
--

SELECT pg_catalog.setval('categoria_cod_categoria_seq', 1, false);


--
-- Name: categoria; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE categoria (
    cod_categoria integer DEFAULT nextval('categoria_cod_categoria_seq'::regclass) NOT NULL,
    ref_funcionario_exc integer,
    ref_funcionario_cad integer NOT NULL,
    nm_categoria character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: secretaria_responsavel; Type: TABLE; Schema: pmiacoes; Owner: -; Tablespace:
--

CREATE TABLE secretaria_responsavel (
    ref_cod_setor integer NOT NULL,
    ref_cod_funcionario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL
);


SET search_path = pmicontrolesis, pg_catalog;

--
-- Name: acontecimento_cod_acontecimento_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE acontecimento_cod_acontecimento_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acontecimento_cod_acontecimento_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('acontecimento_cod_acontecimento_seq', 1, false);


--
-- Name: acontecimento; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE acontecimento (
    cod_acontecimento integer DEFAULT nextval('acontecimento_cod_acontecimento_seq'::regclass) NOT NULL,
    ref_cod_tipo_acontecimento integer NOT NULL,
    ref_cod_funcionario_cad integer NOT NULL,
    ref_cod_funcionario_exc integer,
    titulo character varying(255),
    descricao text,
    dt_inicio timestamp without time zone,
    dt_fim timestamp without time zone,
    hr_inicio time without time zone,
    hr_fim time without time zone,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint,
    local character varying,
    contato character varying,
    link character varying
);


--
-- Name: artigo_cod_artigo_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE artigo_cod_artigo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: artigo_cod_artigo_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('artigo_cod_artigo_seq', 1, false);


--
-- Name: artigo; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE artigo (
    cod_artigo integer DEFAULT nextval('artigo_cod_artigo_seq'::regclass) NOT NULL,
    texto text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint
);


--
-- Name: foto_evento_cod_foto_evento_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE foto_evento_cod_foto_evento_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: foto_evento_cod_foto_evento_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('foto_evento_cod_foto_evento_seq', 1, false);


--
-- Name: foto_evento; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE foto_evento (
    cod_foto_evento integer DEFAULT nextval('foto_evento_cod_foto_evento_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    data_foto timestamp without time zone,
    titulo character varying(255),
    descricao text,
    caminho character varying(255),
    altura integer,
    largura integer,
    nm_credito character varying(255)
);


--
-- Name: foto_vinc_cod_foto_vinc_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE foto_vinc_cod_foto_vinc_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: foto_vinc_cod_foto_vinc_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('foto_vinc_cod_foto_vinc_seq', 1, false);


SET default_with_oids = false;

--
-- Name: foto_vinc; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE foto_vinc (
    cod_foto_vinc integer DEFAULT nextval('foto_vinc_cod_foto_vinc_seq'::regclass) NOT NULL,
    ref_cod_acontecimento integer NOT NULL,
    ref_cod_foto_evento integer NOT NULL
);


--
-- Name: itinerario_cod_itinerario_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE itinerario_cod_itinerario_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: itinerario_cod_itinerario_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('itinerario_cod_itinerario_seq', 1, false);


SET default_with_oids = true;

--
-- Name: itinerario; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE itinerario (
    cod_itinerario integer DEFAULT nextval('itinerario_cod_itinerario_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    numero integer,
    itinerario text,
    retorno text,
    horarios text,
    descricao_horario text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    nome character varying(255) NOT NULL
);


--
-- Name: menu_cod_menu_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE menu_cod_menu_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: menu_cod_menu_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('menu_cod_menu_seq', 20709, true);


--
-- Name: menu; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE menu (
    cod_menu integer DEFAULT nextval('menu_cod_menu_seq'::regclass) NOT NULL,
    ref_cod_menu_submenu integer,
    ref_cod_menu_pai integer,
    tt_menu character varying(40) NOT NULL,
    ord_menu integer NOT NULL,
    caminho character varying(255),
    alvo character varying(20),
    suprime_menu smallint DEFAULT 1,
    ref_cod_tutormenu integer,
    ref_cod_ico integer
);


--
-- Name: menu_portal_cod_menu_portal_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE menu_portal_cod_menu_portal_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: menu_portal_cod_menu_portal_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('menu_portal_cod_menu_portal_seq', 1, false);


--
-- Name: menu_portal; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE menu_portal (
    cod_menu_portal integer DEFAULT nextval('menu_portal_cod_menu_portal_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    nm_menu character varying(15) DEFAULT ''::character varying NOT NULL,
    title character varying(255),
    caminho character varying(255),
    cor character varying(255),
    posicao character(1) DEFAULT 'E'::bpchar NOT NULL,
    ordem double precision DEFAULT (0)::double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    expansivel smallint
);


--
-- Name: portais_cod_portais_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE portais_cod_portais_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: portais_cod_portais_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('portais_cod_portais_seq', 1, false);


--
-- Name: portais; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE portais (
    cod_portais integer DEFAULT nextval('portais_cod_portais_seq'::regclass) NOT NULL,
    ref_cod_funcionario_cad integer NOT NULL,
    ref_cod_funcionario_exc integer,
    url character varying(255),
    caminho character varying(255),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint,
    title character varying(255),
    descricao text
);


--
-- Name: servicos_cod_servicos_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE servicos_cod_servicos_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: servicos_cod_servicos_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('servicos_cod_servicos_seq', 1, false);


--
-- Name: servicos; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE servicos (
    cod_servicos integer DEFAULT nextval('servicos_cod_servicos_seq'::regclass) NOT NULL,
    ref_cod_funcionario_cad integer NOT NULL,
    ref_cod_funcionario_exc integer,
    url character varying(255),
    caminho character varying(255),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint,
    title character varying(255),
    descricao text
);


--
-- Name: sistema_cod_sistema_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE sistema_cod_sistema_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: sistema_cod_sistema_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('sistema_cod_sistema_seq', 1, false);


--
-- Name: sistema; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE sistema (
    cod_sistema integer DEFAULT nextval('sistema_cod_sistema_seq'::regclass) NOT NULL,
    nm_sistema character varying(255) NOT NULL,
    ref_cod_funcionario_cad integer NOT NULL,
    ref_cod_funcionario_exc integer,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint
);


--
-- Name: submenu_portal_cod_submenu_portal_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE submenu_portal_cod_submenu_portal_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: submenu_portal_cod_submenu_portal_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('submenu_portal_cod_submenu_portal_seq', 1, false);


--
-- Name: submenu_portal; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE submenu_portal (
    cod_submenu_portal integer DEFAULT nextval('submenu_portal_cod_submenu_portal_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    ref_cod_menu_portal integer DEFAULT 0 NOT NULL,
    nm_submenu character varying(255) DEFAULT ''::character varying NOT NULL,
    arquivo character varying(255) DEFAULT ''::character varying NOT NULL,
    target character(1) DEFAULT 'S'::bpchar NOT NULL,
    title text,
    ordem double precision DEFAULT (0)::double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: telefones_cod_telefones_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE telefones_cod_telefones_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: telefones_cod_telefones_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('telefones_cod_telefones_seq', 1, false);


--
-- Name: telefones; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE telefones (
    cod_telefones integer DEFAULT nextval('telefones_cod_telefones_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    nome character varying(255) NOT NULL,
    numero character varying,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: tipo_acontecimento_cod_tipo_acontecimento_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE tipo_acontecimento_cod_tipo_acontecimento_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_acontecimento_cod_tipo_acontecimento_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('tipo_acontecimento_cod_tipo_acontecimento_seq', 1, false);


--
-- Name: tipo_acontecimento; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE tipo_acontecimento (
    cod_tipo_acontecimento integer DEFAULT nextval('tipo_acontecimento_cod_tipo_acontecimento_seq'::regclass) NOT NULL,
    ref_cod_funcionario_cad integer NOT NULL,
    ref_cod_funcionario_exc integer,
    nm_tipo character varying(255),
    caminho character varying(255),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint
);


--
-- Name: topo_portal_cod_topo_portal_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE topo_portal_cod_topo_portal_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: topo_portal_cod_topo_portal_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('topo_portal_cod_topo_portal_seq', 1, false);


--
-- Name: topo_portal; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE topo_portal (
    cod_topo_portal integer DEFAULT nextval('topo_portal_cod_topo_portal_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    ref_cod_menu_portal integer DEFAULT 0,
    caminho1 character varying(255),
    caminho2 character varying(255),
    caminho3 character varying(255),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: tutormenu_cod_tutormenu_seq; Type: SEQUENCE; Schema: pmicontrolesis; Owner: -
--

CREATE SEQUENCE tutormenu_cod_tutormenu_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tutormenu_cod_tutormenu_seq; Type: SEQUENCE SET; Schema: pmicontrolesis; Owner: -
--

SELECT pg_catalog.setval('tutormenu_cod_tutormenu_seq', 16, true);


--
-- Name: tutormenu; Type: TABLE; Schema: pmicontrolesis; Owner: -; Tablespace:
--

CREATE TABLE tutormenu (
    cod_tutormenu integer DEFAULT nextval('tutormenu_cod_tutormenu_seq'::regclass) NOT NULL,
    nm_tutormenu character varying(200) NOT NULL
);


SET search_path = pmidrh, pg_catalog;

--
-- Name: diaria_cod_diaria_seq; Type: SEQUENCE; Schema: pmidrh; Owner: -
--

CREATE SEQUENCE diaria_cod_diaria_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: diaria_cod_diaria_seq; Type: SEQUENCE SET; Schema: pmidrh; Owner: -
--

SELECT pg_catalog.setval('diaria_cod_diaria_seq', 1, false);


--
-- Name: diaria; Type: TABLE; Schema: pmidrh; Owner: -; Tablespace:
--

CREATE TABLE diaria (
    cod_diaria integer DEFAULT nextval('diaria_cod_diaria_seq'::regclass) NOT NULL,
    ref_funcionario_cadastro integer NOT NULL,
    ref_cod_diaria_grupo integer NOT NULL,
    ref_funcionario integer NOT NULL,
    conta_corrente integer,
    agencia integer,
    banco integer,
    dotacao_orcamentaria character varying(50),
    objetivo text,
    data_partida timestamp without time zone,
    data_chegada timestamp without time zone,
    estadual smallint,
    destino character varying(100),
    data_pedido timestamp without time zone,
    vl100 double precision,
    vl75 double precision,
    vl50 double precision,
    vl25 double precision,
    roteiro integer,
    ativo boolean DEFAULT true,
    ref_cod_setor integer,
    num_diaria numeric(6,0)
);


--
-- Name: diaria_grupo_cod_diaria_grupo_seq; Type: SEQUENCE; Schema: pmidrh; Owner: -
--

CREATE SEQUENCE diaria_grupo_cod_diaria_grupo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: diaria_grupo_cod_diaria_grupo_seq; Type: SEQUENCE SET; Schema: pmidrh; Owner: -
--

SELECT pg_catalog.setval('diaria_grupo_cod_diaria_grupo_seq', 1, false);


--
-- Name: diaria_grupo; Type: TABLE; Schema: pmidrh; Owner: -; Tablespace:
--

CREATE TABLE diaria_grupo (
    cod_diaria_grupo integer DEFAULT nextval('diaria_grupo_cod_diaria_grupo_seq'::regclass) NOT NULL,
    desc_grupo character varying(255) NOT NULL
);


--
-- Name: diaria_valores_cod_diaria_valores_seq; Type: SEQUENCE; Schema: pmidrh; Owner: -
--

CREATE SEQUENCE diaria_valores_cod_diaria_valores_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: diaria_valores_cod_diaria_valores_seq; Type: SEQUENCE SET; Schema: pmidrh; Owner: -
--

SELECT pg_catalog.setval('diaria_valores_cod_diaria_valores_seq', 1, false);


--
-- Name: diaria_valores; Type: TABLE; Schema: pmidrh; Owner: -; Tablespace:
--

CREATE TABLE diaria_valores (
    cod_diaria_valores integer DEFAULT nextval('diaria_valores_cod_diaria_valores_seq'::regclass) NOT NULL,
    ref_funcionario_cadastro integer NOT NULL,
    ref_cod_diaria_grupo integer NOT NULL,
    estadual smallint NOT NULL,
    p100 double precision,
    p75 double precision,
    p50 double precision,
    p25 double precision,
    data_vigencia timestamp without time zone NOT NULL
);


--
-- Name: setor_cod_setor_seq; Type: SEQUENCE; Schema: pmidrh; Owner: -
--

CREATE SEQUENCE setor_cod_setor_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: setor_cod_setor_seq; Type: SEQUENCE SET; Schema: pmidrh; Owner: -
--

SELECT pg_catalog.setval('setor_cod_setor_seq', 1, false);


--
-- Name: setor; Type: TABLE; Schema: pmidrh; Owner: -; Tablespace:
--

CREATE TABLE setor (
    cod_setor integer DEFAULT nextval('setor_cod_setor_seq'::regclass) NOT NULL,
    ref_cod_pessoa_exc integer,
    ref_cod_pessoa_cad integer NOT NULL,
    ref_cod_setor integer,
    nm_setor character varying(255) NOT NULL,
    sgl_setor character varying(15) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    nivel smallint DEFAULT (1)::smallint NOT NULL,
    no_paco smallint DEFAULT 1,
    endereco text,
    tipo character(1),
    ref_idpes_resp integer
);


SET search_path = pmieducar, pg_catalog;

--
-- Name: acervo_cod_acervo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE acervo_cod_acervo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acervo_cod_acervo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('acervo_cod_acervo_seq', 1, false);


--
-- Name: acervo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo (
    cod_acervo integer DEFAULT nextval('acervo_cod_acervo_seq'::regclass) NOT NULL,
    ref_cod_exemplar_tipo integer NOT NULL,
    ref_cod_acervo integer,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_acervo_colecao integer,
    ref_cod_acervo_idioma integer NOT NULL,
    ref_cod_acervo_editora integer NOT NULL,
    titulo character varying(255) NOT NULL,
    sub_titulo character varying(255),
    cdu character varying(20),
    cutter character varying(20),
    volume integer NOT NULL,
    num_edicao integer NOT NULL,
    ano numeric(4,0) NOT NULL,
    num_paginas integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer NOT NULL,
    isbn numeric(13,0)
);


--
-- Name: acervo_acervo_assunto; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_acervo_assunto (
    ref_cod_acervo integer NOT NULL,
    ref_cod_acervo_assunto integer NOT NULL
);


--
-- Name: acervo_acervo_autor; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_acervo_autor (
    ref_cod_acervo_autor integer NOT NULL,
    ref_cod_acervo integer NOT NULL,
    principal smallint DEFAULT (0)::smallint NOT NULL
);


--
-- Name: acervo_assunto_cod_acervo_assunto_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE acervo_assunto_cod_acervo_assunto_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acervo_assunto_cod_acervo_assunto_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('acervo_assunto_cod_acervo_assunto_seq', 1, false);


--
-- Name: acervo_assunto; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_assunto (
    cod_acervo_assunto integer DEFAULT nextval('acervo_assunto_cod_acervo_assunto_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_assunto character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: acervo_autor_cod_acervo_autor_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE acervo_autor_cod_acervo_autor_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acervo_autor_cod_acervo_autor_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('acervo_autor_cod_acervo_autor_seq', 1, false);


--
-- Name: acervo_autor; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_autor (
    cod_acervo_autor integer DEFAULT nextval('acervo_autor_cod_acervo_autor_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_autor character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer NOT NULL
);


--
-- Name: acervo_colecao_cod_acervo_colecao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE acervo_colecao_cod_acervo_colecao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acervo_colecao_cod_acervo_colecao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('acervo_colecao_cod_acervo_colecao_seq', 1, false);


--
-- Name: acervo_colecao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_colecao (
    cod_acervo_colecao integer DEFAULT nextval('acervo_colecao_cod_acervo_colecao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_colecao character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: acervo_editora_cod_acervo_editora_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE acervo_editora_cod_acervo_editora_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acervo_editora_cod_acervo_editora_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('acervo_editora_cod_acervo_editora_seq', 1, false);


--
-- Name: acervo_editora; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_editora (
    cod_acervo_editora integer DEFAULT nextval('acervo_editora_cod_acervo_editora_seq'::regclass) NOT NULL,
    ref_usuario_cad integer NOT NULL,
    ref_usuario_exc integer,
    ref_idtlog character varying(20) NOT NULL,
    ref_sigla_uf character(2) NOT NULL,
    nm_editora character varying(255) NOT NULL,
    cep numeric(8,0) NOT NULL,
    cidade character varying(60) NOT NULL,
    bairro character varying(60) NOT NULL,
    logradouro character varying(255) NOT NULL,
    numero numeric(6,0),
    telefone integer,
    ddd_telefone numeric(3,0),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: acervo_idioma_cod_acervo_idioma_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE acervo_idioma_cod_acervo_idioma_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acervo_idioma_cod_acervo_idioma_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('acervo_idioma_cod_acervo_idioma_seq', 1, false);


--
-- Name: acervo_idioma; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE acervo_idioma (
    cod_acervo_idioma integer DEFAULT nextval('acervo_idioma_cod_acervo_idioma_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_idioma character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: aluno_cod_aluno_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE aluno_cod_aluno_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: aluno_cod_aluno_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('aluno_cod_aluno_seq', 1, false);


--
-- Name: aluno; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE aluno (
    cod_aluno integer DEFAULT nextval('aluno_cod_aluno_seq'::regclass) NOT NULL,
    ref_cod_aluno_beneficio integer,
    ref_cod_religiao integer,
    ref_usuario_exc integer,
    ref_usuario_cad integer,
    ref_idpes integer,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    caminho_foto character varying(255),
    analfabeto smallint DEFAULT (0)::smallint,
    nm_pai character varying(255),
    nm_mae character varying(255),
    tipo_responsavel character(1)
);


--
-- Name: aluno_beneficio_cod_aluno_beneficio_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE aluno_beneficio_cod_aluno_beneficio_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: aluno_beneficio_cod_aluno_beneficio_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('aluno_beneficio_cod_aluno_beneficio_seq', 1, false);


--
-- Name: aluno_beneficio; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE aluno_beneficio (
    cod_aluno_beneficio integer DEFAULT nextval('aluno_beneficio_cod_aluno_beneficio_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_beneficio character varying(255) NOT NULL,
    desc_beneficio text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: ano_letivo_modulo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE ano_letivo_modulo (
    ref_ano integer NOT NULL,
    ref_ref_cod_escola integer NOT NULL,
    sequencial integer NOT NULL,
    ref_cod_modulo integer NOT NULL,
    data_inicio date NOT NULL,
    data_fim date NOT NULL
);


--
-- Name: avaliacao_desempenho; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE avaliacao_desempenho (
    sequencial integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    descricao text NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    titulo_avaliacao character varying(255) NOT NULL
);


--
-- Name: biblioteca_cod_biblioteca_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE biblioteca_cod_biblioteca_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: biblioteca_cod_biblioteca_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('biblioteca_cod_biblioteca_seq', 1, false);


--
-- Name: biblioteca; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE biblioteca (
    cod_biblioteca integer DEFAULT nextval('biblioteca_cod_biblioteca_seq'::regclass) NOT NULL,
    ref_cod_instituicao integer,
    ref_cod_escola integer,
    nm_biblioteca character varying(255) NOT NULL,
    valor_multa double precision,
    max_emprestimo integer,
    valor_maximo_multa double precision,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    requisita_senha smallint DEFAULT (0)::smallint NOT NULL,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    dias_espera numeric(2,0),
    tombo_automatico boolean DEFAULT true
);


--
-- Name: biblioteca_dia; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE biblioteca_dia (
    ref_cod_biblioteca integer NOT NULL,
    dia numeric(1,0) NOT NULL
);


--
-- Name: biblioteca_feriados_cod_feriado_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE biblioteca_feriados_cod_feriado_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: biblioteca_feriados_cod_feriado_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('biblioteca_feriados_cod_feriado_seq', 1, false);


--
-- Name: biblioteca_feriados; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE biblioteca_feriados (
    cod_feriado integer DEFAULT nextval('biblioteca_feriados_cod_feriado_seq'::regclass) NOT NULL,
    ref_cod_biblioteca integer NOT NULL,
    nm_feriado character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    data_feriado date NOT NULL
);


--
-- Name: biblioteca_usuario; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE biblioteca_usuario (
    ref_cod_biblioteca integer NOT NULL,
    ref_cod_usuario integer NOT NULL
);


--
-- Name: calendario_ano_letivo_cod_calendario_ano_letivo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE calendario_ano_letivo_cod_calendario_ano_letivo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: calendario_ano_letivo_cod_calendario_ano_letivo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('calendario_ano_letivo_cod_calendario_ano_letivo_seq', 1, false);


--
-- Name: calendario_ano_letivo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE calendario_ano_letivo (
    cod_calendario_ano_letivo integer DEFAULT nextval('calendario_ano_letivo_cod_calendario_ano_letivo_seq'::regclass) NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ano integer NOT NULL,
    data_cadastra timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: calendario_anotacao_cod_calendario_anotacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE calendario_anotacao_cod_calendario_anotacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: calendario_anotacao_cod_calendario_anotacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('calendario_anotacao_cod_calendario_anotacao_seq', 1, false);


--
-- Name: calendario_anotacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE calendario_anotacao (
    cod_calendario_anotacao integer DEFAULT nextval('calendario_anotacao_cod_calendario_anotacao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_anotacao character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint NOT NULL
);


--
-- Name: calendario_dia; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE calendario_dia (
    ref_cod_calendario_ano_letivo integer NOT NULL,
    mes integer NOT NULL,
    dia integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_calendario_dia_motivo integer,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    descricao text
);


--
-- Name: calendario_dia_anotacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE calendario_dia_anotacao (
    ref_dia integer NOT NULL,
    ref_mes integer NOT NULL,
    ref_ref_cod_calendario_ano_letivo integer NOT NULL,
    ref_cod_calendario_anotacao integer NOT NULL
);


--
-- Name: calendario_dia_motivo_cod_calendario_dia_motivo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE calendario_dia_motivo_cod_calendario_dia_motivo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: calendario_dia_motivo_cod_calendario_dia_motivo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('calendario_dia_motivo_cod_calendario_dia_motivo_seq', 1, false);


--
-- Name: calendario_dia_motivo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE calendario_dia_motivo (
    cod_calendario_dia_motivo integer DEFAULT nextval('calendario_dia_motivo_cod_calendario_dia_motivo_seq'::regclass) NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    sigla character varying(15) NOT NULL,
    descricao text,
    tipo character(1) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    nm_motivo character varying(255) NOT NULL
);


--
-- Name: categoria_nivel_cod_categoria_nivel_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE categoria_nivel_cod_categoria_nivel_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: categoria_nivel_cod_categoria_nivel_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('categoria_nivel_cod_categoria_nivel_seq', 1, false);


SET default_with_oids = false;

--
-- Name: categoria_nivel; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE categoria_nivel (
    cod_categoria_nivel integer DEFAULT nextval('categoria_nivel_cod_categoria_nivel_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_categoria_nivel character varying(100) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo boolean DEFAULT true NOT NULL
);


--
-- Name: cliente_cod_cliente_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE cliente_cod_cliente_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: cliente_cod_cliente_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('cliente_cod_cliente_seq', 1, false);


SET default_with_oids = true;

--
-- Name: cliente; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE cliente (
    cod_cliente integer DEFAULT nextval('cliente_cod_cliente_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_idpes integer NOT NULL,
    login integer,
    senha character varying(255),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: cliente_suspensao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE cliente_suspensao (
    sequencial integer NOT NULL,
    ref_cod_cliente integer NOT NULL,
    ref_cod_motivo_suspensao integer NOT NULL,
    ref_usuario_libera integer,
    ref_usuario_suspende integer NOT NULL,
    dias integer NOT NULL,
    data_suspensao timestamp without time zone NOT NULL,
    data_liberacao timestamp without time zone
);


--
-- Name: cliente_tipo_cod_cliente_tipo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE cliente_tipo_cod_cliente_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: cliente_tipo_cod_cliente_tipo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('cliente_tipo_cod_cliente_tipo_seq', 1, false);


--
-- Name: cliente_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE cliente_tipo (
    cod_cliente_tipo integer DEFAULT nextval('cliente_tipo_cod_cliente_tipo_seq'::regclass) NOT NULL,
    ref_cod_biblioteca integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


SET default_with_oids = false;

--
-- Name: cliente_tipo_cliente; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE cliente_tipo_cliente (
    ref_cod_cliente_tipo integer NOT NULL,
    ref_cod_cliente integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ref_usuario_cad integer NOT NULL,
    ref_usuario_exc integer,
    ativo smallint DEFAULT (1)::smallint,
    ref_cod_biblioteca integer
);


SET default_with_oids = true;

--
-- Name: cliente_tipo_exemplar_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE cliente_tipo_exemplar_tipo (
    ref_cod_cliente_tipo integer NOT NULL,
    ref_cod_exemplar_tipo integer NOT NULL,
    dias_emprestimo numeric(3,0)
);


--
-- Name: coffebreak_tipo_cod_coffebreak_tipo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE coffebreak_tipo_cod_coffebreak_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: coffebreak_tipo_cod_coffebreak_tipo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('coffebreak_tipo_cod_coffebreak_tipo_seq', 1, false);


--
-- Name: coffebreak_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE coffebreak_tipo (
    cod_coffebreak_tipo integer DEFAULT nextval('coffebreak_tipo_cod_coffebreak_tipo_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    desc_tipo text,
    custo_unitario double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: curso_cod_curso_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE curso_cod_curso_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: curso_cod_curso_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('curso_cod_curso_seq', 1, false);


--
-- Name: curso; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE curso (
    cod_curso integer DEFAULT nextval('curso_cod_curso_seq'::regclass) NOT NULL,
    ref_usuario_cad integer NOT NULL,
    ref_cod_tipo_regime integer,
    ref_cod_nivel_ensino integer NOT NULL,
    ref_cod_tipo_ensino integer NOT NULL,
    nm_curso character varying(255) NOT NULL,
    sgl_curso character varying(15) NOT NULL,
    qtd_etapas smallint NOT NULL,
    carga_horaria double precision NOT NULL,
    ato_poder_publico character varying(255),
    objetivo_curso text,
    publico_alvo text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_usuario_exc integer,
    ref_cod_instituicao integer NOT NULL,
    padrao_ano_escolar smallint DEFAULT (0)::smallint NOT NULL,
    hora_falta double precision DEFAULT 0.00 NOT NULL
);


--
-- Name: disciplina_cod_disciplina_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE disciplina_cod_disciplina_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: disciplina_cod_disciplina_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('disciplina_cod_disciplina_seq', 1, false);


--
-- Name: disciplina; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE disciplina (
    cod_disciplina integer DEFAULT nextval('disciplina_cod_disciplina_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    desc_disciplina text,
    desc_resumida text,
    abreviatura character varying(15) NOT NULL,
    carga_horaria integer NOT NULL,
    apura_falta smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    nm_disciplina character varying(255) NOT NULL,
    ref_cod_curso integer
);


SET default_with_oids = false;

--
-- Name: disciplina_serie; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE disciplina_serie (
    ref_cod_disciplina integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: disciplina_topico_cod_disciplina_topico_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE disciplina_topico_cod_disciplina_topico_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: disciplina_topico_cod_disciplina_topico_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('disciplina_topico_cod_disciplina_topico_seq', 1, false);


SET default_with_oids = true;

--
-- Name: disciplina_topico; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE disciplina_topico (
    cod_disciplina_topico integer DEFAULT nextval('disciplina_topico_cod_disciplina_topico_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_topico character varying(255) NOT NULL,
    desc_topico text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


SET default_with_oids = false;

--
-- Name: dispensa_disciplina; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE dispensa_disciplina (
    ref_cod_matricula integer NOT NULL,
    ref_cod_disciplina integer NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_tipo_dispensa integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    observacao text,
    cod_dispensa integer NOT NULL
);


--
-- Name: escola_cod_escola_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE escola_cod_escola_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: escola_cod_escola_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('escola_cod_escola_seq', 1, false);


SET default_with_oids = true;

--
-- Name: escola; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola (
    cod_escola integer DEFAULT nextval('escola_cod_escola_seq'::regclass) NOT NULL,
    ref_usuario_cad integer NOT NULL,
    ref_usuario_exc integer,
    ref_cod_instituicao integer NOT NULL,
    ref_cod_escola_localizacao integer NOT NULL,
    ref_cod_escola_rede_ensino integer NOT NULL,
    ref_idpes integer,
    sigla character varying(20) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: escola_ano_letivo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_ano_letivo (
    ref_cod_escola integer NOT NULL,
    ano integer NOT NULL,
    ref_usuario_cad integer NOT NULL,
    ref_usuario_exc integer,
    andamento smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: escola_complemento; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_complemento (
    ref_cod_escola integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    cep numeric(8,0) NOT NULL,
    numero numeric(6,0),
    complemento character varying(50),
    email character varying(50),
    nm_escola character varying(255) NOT NULL,
    municipio character varying(60) NOT NULL,
    bairro character varying(40) NOT NULL,
    logradouro character varying(150) NOT NULL,
    ddd_telefone numeric(2,0),
    telefone numeric(11,0),
    ddd_fax numeric(2,0),
    fax numeric(11,0),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: escola_curso; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_curso (
    ref_cod_escola integer NOT NULL,
    ref_cod_curso integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: escola_localizacao_cod_escola_localizacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE escola_localizacao_cod_escola_localizacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: escola_localizacao_cod_escola_localizacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('escola_localizacao_cod_escola_localizacao_seq', 1, false);


--
-- Name: escola_localizacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_localizacao (
    cod_escola_localizacao integer DEFAULT nextval('escola_localizacao_cod_escola_localizacao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_localizacao character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: escola_rede_ensino_cod_escola_rede_ensino_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE escola_rede_ensino_cod_escola_rede_ensino_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: escola_rede_ensino_cod_escola_rede_ensino_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('escola_rede_ensino_cod_escola_rede_ensino_seq', 1, false);


--
-- Name: escola_rede_ensino; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_rede_ensino (
    cod_escola_rede_ensino integer DEFAULT nextval('escola_rede_ensino_cod_escola_rede_ensino_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_rede character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: escola_serie; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_serie (
    ref_cod_escola integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    hora_inicial time without time zone NOT NULL,
    hora_final time without time zone NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    hora_inicio_intervalo time without time zone NOT NULL,
    hora_fim_intervalo time without time zone NOT NULL
);


SET default_with_oids = false;

--
-- Name: escola_serie_disciplina; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE escola_serie_disciplina (
    ref_ref_cod_serie integer NOT NULL,
    ref_ref_cod_escola integer NOT NULL,
    ref_cod_disciplina integer NOT NULL,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    carga_horaria numeric(6,3)
);


--
-- Name: exemplar_cod_exemplar_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE exemplar_cod_exemplar_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: exemplar_cod_exemplar_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('exemplar_cod_exemplar_seq', 1, false);


SET default_with_oids = true;

--
-- Name: exemplar; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE exemplar (
    cod_exemplar integer DEFAULT nextval('exemplar_cod_exemplar_seq'::regclass) NOT NULL,
    ref_cod_fonte integer NOT NULL,
    ref_cod_motivo_baixa integer,
    ref_cod_acervo integer NOT NULL,
    ref_cod_situacao integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    permite_emprestimo smallint DEFAULT (1)::smallint NOT NULL,
    preco double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    data_aquisicao timestamp without time zone,
    tombo integer
);


--
-- Name: exemplar_emprestimo_cod_emprestimo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE exemplar_emprestimo_cod_emprestimo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: exemplar_emprestimo_cod_emprestimo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('exemplar_emprestimo_cod_emprestimo_seq', 1, false);


--
-- Name: exemplar_emprestimo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE exemplar_emprestimo (
    cod_emprestimo integer DEFAULT nextval('exemplar_emprestimo_cod_emprestimo_seq'::regclass) NOT NULL,
    ref_usuario_devolucao integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_cliente integer NOT NULL,
    ref_cod_exemplar integer NOT NULL,
    data_retirada timestamp without time zone NOT NULL,
    data_devolucao timestamp without time zone,
    valor_multa double precision
);


--
-- Name: exemplar_tipo_cod_exemplar_tipo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE exemplar_tipo_cod_exemplar_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: exemplar_tipo_cod_exemplar_tipo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('exemplar_tipo_cod_exemplar_tipo_seq', 1, false);


--
-- Name: exemplar_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE exemplar_tipo (
    cod_exemplar_tipo integer DEFAULT nextval('exemplar_tipo_cod_exemplar_tipo_seq'::regclass) NOT NULL,
    ref_cod_biblioteca integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: falta_aluno_cod_falta_aluno_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE falta_aluno_cod_falta_aluno_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: falta_aluno_cod_falta_aluno_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('falta_aluno_cod_falta_aluno_seq', 1, false);


SET default_with_oids = false;

--
-- Name: falta_aluno; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE falta_aluno (
    cod_falta_aluno integer DEFAULT nextval('falta_aluno_cod_falta_aluno_seq'::regclass) NOT NULL,
    ref_cod_disciplina integer,
    ref_cod_escola integer,
    ref_cod_serie integer,
    ref_cod_matricula integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    faltas integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    modulo smallint NOT NULL,
    ref_cod_curso_disciplina integer
);


--
-- Name: falta_atraso_cod_falta_atraso_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE falta_atraso_cod_falta_atraso_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: falta_atraso_cod_falta_atraso_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('falta_atraso_cod_falta_atraso_seq', 1, false);


SET default_with_oids = true;

--
-- Name: falta_atraso; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE falta_atraso (
    cod_falta_atraso integer DEFAULT nextval('falta_atraso_cod_falta_atraso_seq'::regclass) NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    tipo smallint NOT NULL,
    data_falta_atraso timestamp without time zone NOT NULL,
    qtd_horas integer,
    qtd_min integer,
    justificada smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: falta_atraso_compensado_cod_compensado_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE falta_atraso_compensado_cod_compensado_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: falta_atraso_compensado_cod_compensado_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('falta_atraso_compensado_cod_compensado_seq', 1, false);


--
-- Name: falta_atraso_compensado; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE falta_atraso_compensado (
    cod_compensado integer DEFAULT nextval('falta_atraso_compensado_cod_compensado_seq'::regclass) NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    data_inicio timestamp without time zone NOT NULL,
    data_fim timestamp without time zone NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: faltas_sequencial_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE faltas_sequencial_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: faltas_sequencial_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('faltas_sequencial_seq', 1, false);


SET default_with_oids = false;

--
-- Name: faltas; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE faltas (
    ref_cod_matricula integer NOT NULL,
    sequencial integer DEFAULT nextval('faltas_sequencial_seq'::regclass) NOT NULL,
    ref_usuario_cad integer NOT NULL,
    falta integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: fonte_cod_fonte_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE fonte_cod_fonte_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: fonte_cod_fonte_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('fonte_cod_fonte_seq', 1, false);


SET default_with_oids = true;

--
-- Name: fonte; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE fonte (
    cod_fonte integer DEFAULT nextval('fonte_cod_fonte_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_fonte character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: funcao_cod_funcao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE funcao_cod_funcao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: funcao_cod_funcao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('funcao_cod_funcao_seq', 1, false);


--
-- Name: funcao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE funcao (
    cod_funcao integer DEFAULT nextval('funcao_cod_funcao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_funcao character varying(255) NOT NULL,
    abreviatura character varying(30) NOT NULL,
    professor smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: habilitacao_cod_habilitacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE habilitacao_cod_habilitacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: habilitacao_cod_habilitacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('habilitacao_cod_habilitacao_seq', 1, false);


--
-- Name: habilitacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE habilitacao (
    cod_habilitacao integer DEFAULT nextval('habilitacao_cod_habilitacao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: habilitacao_curso; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE habilitacao_curso (
    ref_cod_habilitacao integer NOT NULL,
    ref_cod_curso integer NOT NULL
);


--
-- Name: historico_disciplinas; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE historico_disciplinas (
    sequencial integer NOT NULL,
    ref_ref_cod_aluno integer NOT NULL,
    ref_sequencial integer NOT NULL,
    nm_disciplina character varying(255) NOT NULL,
    nota character varying(255) NOT NULL,
    faltas integer,
    import numeric(1,0)
);


SET default_with_oids = false;

--
-- Name: historico_educar; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE historico_educar (
    tabela character varying(50),
    alteracao text,
    data timestamp without time zone,
    insercao smallint DEFAULT 0
);


SET default_with_oids = true;

--
-- Name: historico_escolar; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE historico_escolar (
    ref_cod_aluno integer NOT NULL,
    sequencial integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ano integer NOT NULL,
    carga_horaria double precision NOT NULL,
    dias_letivos integer,
    escola character varying(255) NOT NULL,
    escola_cidade character varying(255) NOT NULL,
    escola_uf character(2),
    observacao text,
    aprovado smallint DEFAULT (1)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    faltas_globalizadas integer,
    nm_serie character varying(255),
    origem smallint DEFAULT (1)::smallint,
    extra_curricular smallint DEFAULT (0)::smallint,
    ref_cod_matricula integer,
    ref_cod_instituicao integer,
    import numeric(1,0)
);


--
-- Name: infra_comodo_funcao_cod_infra_comodo_funcao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE infra_comodo_funcao_cod_infra_comodo_funcao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: infra_comodo_funcao_cod_infra_comodo_funcao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('infra_comodo_funcao_cod_infra_comodo_funcao_seq', 1, false);


--
-- Name: infra_comodo_funcao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE infra_comodo_funcao (
    cod_infra_comodo_funcao integer DEFAULT nextval('infra_comodo_funcao_cod_infra_comodo_funcao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_funcao character varying(255) NOT NULL,
    desc_funcao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_escola integer
);


--
-- Name: infra_predio_cod_infra_predio_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE infra_predio_cod_infra_predio_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: infra_predio_cod_infra_predio_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('infra_predio_cod_infra_predio_seq', 1, false);


--
-- Name: infra_predio; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE infra_predio (
    cod_infra_predio integer DEFAULT nextval('infra_predio_cod_infra_predio_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_escola integer NOT NULL,
    nm_predio character varying(255) NOT NULL,
    desc_predio text,
    endereco text NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: infra_predio_comodo_cod_infra_predio_comodo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE infra_predio_comodo_cod_infra_predio_comodo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: infra_predio_comodo_cod_infra_predio_comodo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('infra_predio_comodo_cod_infra_predio_comodo_seq', 1, false);


--
-- Name: infra_predio_comodo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE infra_predio_comodo (
    cod_infra_predio_comodo integer DEFAULT nextval('infra_predio_comodo_cod_infra_predio_comodo_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_infra_comodo_funcao integer NOT NULL,
    ref_cod_infra_predio integer NOT NULL,
    nm_comodo character varying(255) NOT NULL,
    desc_comodo text,
    area double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: instituicao_cod_instituicao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE instituicao_cod_instituicao_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: instituicao_cod_instituicao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('instituicao_cod_instituicao_seq', 1, true);


--
-- Name: instituicao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE instituicao (
    cod_instituicao integer DEFAULT nextval('instituicao_cod_instituicao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_idtlog character varying(20) NOT NULL,
    ref_sigla_uf character(2) NOT NULL,
    cep numeric(8,0) NOT NULL,
    cidade character varying(60) NOT NULL,
    bairro character varying(40) NOT NULL,
    logradouro character varying(255) NOT NULL,
    numero numeric(6,0),
    complemento character varying(50),
    nm_responsavel character varying(255) NOT NULL,
    ddd_telefone numeric(2,0),
    telefone numeric(11,0),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    nm_instituicao character varying(255) NOT NULL
);


--
-- Name: material_didatico_cod_material_didatico_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE material_didatico_cod_material_didatico_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: material_didatico_cod_material_didatico_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('material_didatico_cod_material_didatico_seq', 1, false);


--
-- Name: material_didatico; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE material_didatico (
    cod_material_didatico integer DEFAULT nextval('material_didatico_cod_material_didatico_seq'::regclass) NOT NULL,
    ref_cod_instituicao integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_material_tipo integer NOT NULL,
    nm_material character varying(255) NOT NULL,
    desc_material text,
    custo_unitario double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: material_tipo_cod_material_tipo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE material_tipo_cod_material_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: material_tipo_cod_material_tipo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('material_tipo_cod_material_tipo_seq', 1, false);


--
-- Name: material_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE material_tipo (
    cod_material_tipo integer DEFAULT nextval('material_tipo_cod_material_tipo_seq'::regclass) NOT NULL,
    ref_usuario_cad integer NOT NULL,
    ref_usuario_exc integer,
    nm_tipo character varying(255) NOT NULL,
    desc_tipo text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: matricula_cod_matricula_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE matricula_cod_matricula_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: matricula_cod_matricula_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('matricula_cod_matricula_seq', 1, false);


--
-- Name: matricula; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE matricula (
    cod_matricula integer DEFAULT nextval('matricula_cod_matricula_seq'::regclass) NOT NULL,
    ref_cod_reserva_vaga integer,
    ref_ref_cod_escola integer,
    ref_ref_cod_serie integer,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_aluno integer NOT NULL,
    aprovado smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ano integer NOT NULL,
    ultima_matricula smallint DEFAULT (0)::smallint NOT NULL,
    modulo smallint DEFAULT 1 NOT NULL,
    descricao_reclassificacao text,
    formando smallint DEFAULT (0)::smallint NOT NULL,
    matricula_reclassificacao smallint DEFAULT (0)::smallint,
    ref_cod_curso integer,
    matricula_transferencia boolean DEFAULT false NOT NULL,
    semestre smallint
);


--
-- Name: matricula_excessao_cod_aluno_excessao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE matricula_excessao_cod_aluno_excessao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: matricula_excessao_cod_aluno_excessao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('matricula_excessao_cod_aluno_excessao_seq', 1, false);


SET default_with_oids = false;

--
-- Name: matricula_excessao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE matricula_excessao (
    cod_aluno_excessao integer DEFAULT nextval('matricula_excessao_cod_aluno_excessao_seq'::regclass) NOT NULL,
    ref_cod_matricula integer NOT NULL,
    ref_cod_turma integer NOT NULL,
    ref_sequencial integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_cod_disciplina integer NOT NULL,
    reprovado_faltas boolean NOT NULL,
    precisa_exame boolean NOT NULL,
    permite_exame boolean
);


SET default_with_oids = true;

--
-- Name: matricula_ocorrencia_disciplinar; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE matricula_ocorrencia_disciplinar (
    ref_cod_matricula integer NOT NULL,
    ref_cod_tipo_ocorrencia_disciplinar integer NOT NULL,
    sequencial integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    observacao text NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: matricula_turma; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE matricula_turma (
    ref_cod_matricula integer NOT NULL,
    ref_cod_turma integer NOT NULL,
    sequencial integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: menu_tipo_usuario; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE menu_tipo_usuario (
    ref_cod_tipo_usuario integer NOT NULL,
    ref_cod_menu_submenu integer NOT NULL,
    cadastra smallint DEFAULT 0 NOT NULL,
    visualiza smallint DEFAULT 0 NOT NULL,
    exclui smallint DEFAULT 0 NOT NULL
);


--
-- Name: modulo_cod_modulo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE modulo_cod_modulo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: modulo_cod_modulo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('modulo_cod_modulo_seq', 1, false);


--
-- Name: modulo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE modulo (
    cod_modulo integer DEFAULT nextval('modulo_cod_modulo_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    num_meses numeric(2,0) NOT NULL,
    num_semanas integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: motivo_afastamento_cod_motivo_afastamento_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE motivo_afastamento_cod_motivo_afastamento_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: motivo_afastamento_cod_motivo_afastamento_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('motivo_afastamento_cod_motivo_afastamento_seq', 1, false);


--
-- Name: motivo_afastamento; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE motivo_afastamento (
    cod_motivo_afastamento integer DEFAULT nextval('motivo_afastamento_cod_motivo_afastamento_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_motivo character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: motivo_baixa_cod_motivo_baixa_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE motivo_baixa_cod_motivo_baixa_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: motivo_baixa_cod_motivo_baixa_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('motivo_baixa_cod_motivo_baixa_seq', 1, false);


--
-- Name: motivo_baixa; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE motivo_baixa (
    cod_motivo_baixa integer DEFAULT nextval('motivo_baixa_cod_motivo_baixa_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_motivo_baixa character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: motivo_suspensao_cod_motivo_suspensao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE motivo_suspensao_cod_motivo_suspensao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: motivo_suspensao_cod_motivo_suspensao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('motivo_suspensao_cod_motivo_suspensao_seq', 1, false);


--
-- Name: motivo_suspensao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE motivo_suspensao (
    cod_motivo_suspensao integer DEFAULT nextval('motivo_suspensao_cod_motivo_suspensao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_motivo character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer
);


--
-- Name: nivel_cod_nivel_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE nivel_cod_nivel_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: nivel_cod_nivel_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('nivel_cod_nivel_seq', 1, false);


SET default_with_oids = false;

--
-- Name: nivel; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE nivel (
    cod_nivel integer DEFAULT nextval('nivel_cod_nivel_seq'::regclass) NOT NULL,
    ref_cod_categoria_nivel integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_nivel_anterior integer,
    nm_nivel character varying(100) NOT NULL,
    salario_base double precision,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo boolean DEFAULT true NOT NULL
);


--
-- Name: nivel_ensino_cod_nivel_ensino_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE nivel_ensino_cod_nivel_ensino_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: nivel_ensino_cod_nivel_ensino_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('nivel_ensino_cod_nivel_ensino_seq', 1, false);


SET default_with_oids = true;

--
-- Name: nivel_ensino; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE nivel_ensino (
    cod_nivel_ensino integer DEFAULT nextval('nivel_ensino_cod_nivel_ensino_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_nivel character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: nota_aluno_cod_nota_aluno_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE nota_aluno_cod_nota_aluno_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: nota_aluno_cod_nota_aluno_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('nota_aluno_cod_nota_aluno_seq', 1, false);


SET default_with_oids = false;

--
-- Name: nota_aluno; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE nota_aluno (
    cod_nota_aluno integer DEFAULT nextval('nota_aluno_cod_nota_aluno_seq'::regclass) NOT NULL,
    ref_cod_disciplina integer,
    ref_cod_escola integer,
    ref_cod_serie integer,
    ref_cod_matricula integer NOT NULL,
    ref_sequencial integer,
    ref_ref_cod_tipo_avaliacao integer,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    modulo smallint NOT NULL,
    ref_cod_curso_disciplina integer,
    nota double precision
);


--
-- Name: operador_cod_operador_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE operador_cod_operador_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: operador_cod_operador_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('operador_cod_operador_seq', 1, false);


SET default_with_oids = true;

--
-- Name: operador; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE operador (
    cod_operador integer DEFAULT nextval('operador_cod_operador_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nome character varying(50) NOT NULL,
    valor text NOT NULL,
    fim_sentenca smallint DEFAULT (1)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: pagamento_multa_cod_pagamento_multa_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE pagamento_multa_cod_pagamento_multa_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: pagamento_multa_cod_pagamento_multa_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('pagamento_multa_cod_pagamento_multa_seq', 1, false);


--
-- Name: pagamento_multa; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE pagamento_multa (
    cod_pagamento_multa integer DEFAULT nextval('pagamento_multa_cod_pagamento_multa_seq'::regclass) NOT NULL,
    ref_usuario_cad integer NOT NULL,
    ref_cod_cliente integer NOT NULL,
    valor_pago double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    ref_cod_biblioteca integer NOT NULL
);


--
-- Name: pre_requisito_cod_pre_requisito_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE pre_requisito_cod_pre_requisito_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: pre_requisito_cod_pre_requisito_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('pre_requisito_cod_pre_requisito_seq', 1, false);


--
-- Name: pre_requisito; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE pre_requisito (
    cod_pre_requisito integer DEFAULT nextval('pre_requisito_cod_pre_requisito_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    schema_ character varying(50) NOT NULL,
    tabela character varying(50) NOT NULL,
    nome character varying(50) NOT NULL,
    sql text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: quadro_horario_cod_quadro_horario_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE quadro_horario_cod_quadro_horario_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: quadro_horario_cod_quadro_horario_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('quadro_horario_cod_quadro_horario_seq', 1, false);


--
-- Name: quadro_horario; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE quadro_horario (
    cod_quadro_horario integer DEFAULT nextval('quadro_horario_cod_quadro_horario_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_turma integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


SET default_with_oids = false;

--
-- Name: quadro_horario_horarios; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE quadro_horario_horarios (
    ref_cod_quadro_horario integer NOT NULL,
    sequencial integer NOT NULL,
    ref_cod_disciplina integer NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    ref_cod_instituicao_substituto integer,
    ref_cod_instituicao_servidor integer NOT NULL,
    ref_servidor_substituto integer,
    ref_servidor integer NOT NULL,
    dia_semana integer NOT NULL,
    hora_inicial time without time zone NOT NULL,
    hora_final time without time zone NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: quadro_horario_horarios_aux; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE quadro_horario_horarios_aux (
    ref_cod_quadro_horario integer NOT NULL,
    sequencial integer NOT NULL,
    ref_cod_disciplina integer NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    ref_cod_instituicao_servidor integer NOT NULL,
    ref_servidor integer NOT NULL,
    dia_semana integer NOT NULL,
    hora_inicial time without time zone NOT NULL,
    hora_final time without time zone NOT NULL,
    identificador character varying(30),
    data_cadastro timestamp without time zone NOT NULL
);


--
-- Name: religiao_cod_religiao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE religiao_cod_religiao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: religiao_cod_religiao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('religiao_cod_religiao_seq', 1, false);


SET default_with_oids = true;

--
-- Name: religiao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE religiao (
    cod_religiao integer DEFAULT nextval('religiao_cod_religiao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_religiao character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: reserva_vaga_cod_reserva_vaga_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE reserva_vaga_cod_reserva_vaga_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: reserva_vaga_cod_reserva_vaga_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('reserva_vaga_cod_reserva_vaga_seq', 1, false);


--
-- Name: reserva_vaga; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE reserva_vaga (
    cod_reserva_vaga integer DEFAULT nextval('reserva_vaga_cod_reserva_vaga_seq'::regclass) NOT NULL,
    ref_ref_cod_escola integer NOT NULL,
    ref_ref_cod_serie integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_aluno integer,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    nm_aluno character varying(255),
    cpf_responsavel numeric(11,0)
);


--
-- Name: reservas_cod_reserva_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE reservas_cod_reserva_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: reservas_cod_reserva_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('reservas_cod_reserva_seq', 1, false);


--
-- Name: reservas; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE reservas (
    cod_reserva integer DEFAULT nextval('reservas_cod_reserva_seq'::regclass) NOT NULL,
    ref_usuario_libera integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_cliente integer NOT NULL,
    data_reserva timestamp without time zone,
    data_prevista_disponivel timestamp without time zone,
    data_retirada timestamp without time zone,
    ref_cod_exemplar integer NOT NULL,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: sequencia_serie; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE sequencia_serie (
    ref_serie_origem integer NOT NULL,
    ref_serie_destino integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: serie_cod_serie_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE serie_cod_serie_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: serie_cod_serie_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('serie_cod_serie_seq', 1, false);


--
-- Name: serie; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE serie (
    cod_serie integer DEFAULT nextval('serie_cod_serie_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_curso integer NOT NULL,
    nm_serie character varying(255) NOT NULL,
    etapa_curso integer NOT NULL,
    concluinte smallint DEFAULT (0)::smallint NOT NULL,
    carga_horaria double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    intervalo integer NOT NULL,
    idade_inicial numeric(3,0),
    idade_final numeric(3,0),
    regra_avaliacao_id integer
);


--
-- Name: serie_pre_requisito; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE serie_pre_requisito (
    ref_cod_pre_requisito integer NOT NULL,
    ref_cod_operador integer NOT NULL,
    ref_cod_serie integer NOT NULL,
    valor character varying
);


--
-- Name: servidor; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor (
    cod_servidor integer NOT NULL,
    ref_cod_instituicao integer NOT NULL,
    ref_cod_deficiencia integer,
    ref_idesco numeric(2,0),
    carga_horaria double precision NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_subnivel integer
);


--
-- Name: servidor_afastamento; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_afastamento (
    ref_cod_servidor integer NOT NULL,
    sequencial integer NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_cod_motivo_afastamento integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    data_retorno timestamp without time zone,
    data_saida timestamp without time zone NOT NULL,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: servidor_alocacao_cod_servidor_alocacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE servidor_alocacao_cod_servidor_alocacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: servidor_alocacao_cod_servidor_alocacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('servidor_alocacao_cod_servidor_alocacao_seq', 1, false);


--
-- Name: servidor_alocacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_alocacao (
    cod_servidor_alocacao integer DEFAULT nextval('servidor_alocacao_cod_servidor_alocacao_seq'::regclass) NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_escola integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    carga_horaria character varying(8),
    periodo smallint DEFAULT (1)::smallint,
    hora_final time without time zone,
    hora_inicial time without time zone,
    dia_semana integer
);


--
-- Name: servidor_curso_cod_servidor_curso_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE servidor_curso_cod_servidor_curso_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: servidor_curso_cod_servidor_curso_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('servidor_curso_cod_servidor_curso_seq', 1, false);


--
-- Name: servidor_curso; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_curso (
    cod_servidor_curso integer DEFAULT nextval('servidor_curso_cod_servidor_curso_seq'::regclass) NOT NULL,
    ref_cod_formacao integer NOT NULL,
    data_conclusao timestamp without time zone NOT NULL,
    data_registro timestamp without time zone,
    diplomas_registros text
);


SET default_with_oids = false;

--
-- Name: servidor_curso_ministra; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_curso_ministra (
    ref_cod_curso integer NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_cod_servidor integer NOT NULL
);


--
-- Name: servidor_disciplina; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_disciplina (
    ref_cod_disciplina integer NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    ref_cod_curso integer NOT NULL
);


--
-- Name: servidor_formacao_cod_formacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE servidor_formacao_cod_formacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: servidor_formacao_cod_formacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('servidor_formacao_cod_formacao_seq', 1, false);


SET default_with_oids = true;

--
-- Name: servidor_formacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_formacao (
    cod_formacao integer DEFAULT nextval('servidor_formacao_cod_formacao_seq'::regclass) NOT NULL,
    ref_ref_cod_instituicao integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    nm_formacao character varying(255) NOT NULL,
    tipo character(1) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


SET default_with_oids = false;

--
-- Name: servidor_funcao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_funcao (
    ref_ref_cod_instituicao integer NOT NULL,
    ref_cod_servidor integer NOT NULL,
    ref_cod_funcao integer NOT NULL
);


--
-- Name: servidor_titulo_concurso_cod_servidor_titulo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE servidor_titulo_concurso_cod_servidor_titulo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: servidor_titulo_concurso_cod_servidor_titulo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('servidor_titulo_concurso_cod_servidor_titulo_seq', 1, false);


SET default_with_oids = true;

--
-- Name: servidor_titulo_concurso; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE servidor_titulo_concurso (
    cod_servidor_titulo integer DEFAULT nextval('servidor_titulo_concurso_cod_servidor_titulo_seq'::regclass) NOT NULL,
    ref_cod_formacao integer NOT NULL,
    data_vigencia_homolog timestamp without time zone NOT NULL,
    data_publicacao timestamp without time zone NOT NULL
);


--
-- Name: situacao_cod_situacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE situacao_cod_situacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: situacao_cod_situacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('situacao_cod_situacao_seq', 1, false);


--
-- Name: situacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE situacao (
    cod_situacao integer DEFAULT nextval('situacao_cod_situacao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_situacao character varying(255) NOT NULL,
    permite_emprestimo smallint DEFAULT (1)::smallint NOT NULL,
    descricao text,
    situacao_padrao smallint DEFAULT (0)::smallint NOT NULL,
    situacao_emprestada smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_biblioteca integer NOT NULL
);


--
-- Name: subnivel_cod_subnivel_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE subnivel_cod_subnivel_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: subnivel_cod_subnivel_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('subnivel_cod_subnivel_seq', 1, false);


SET default_with_oids = false;

--
-- Name: subnivel; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE subnivel (
    cod_subnivel integer DEFAULT nextval('subnivel_cod_subnivel_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_subnivel_anterior integer,
    ref_cod_nivel integer NOT NULL,
    nm_subnivel character varying(100),
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo boolean DEFAULT true NOT NULL,
    salario double precision NOT NULL
);


--
-- Name: tipo_avaliacao_cod_tipo_avaliacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE tipo_avaliacao_cod_tipo_avaliacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_avaliacao_cod_tipo_avaliacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('tipo_avaliacao_cod_tipo_avaliacao_seq', 1, false);


SET default_with_oids = true;

--
-- Name: tipo_avaliacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_avaliacao (
    cod_tipo_avaliacao integer DEFAULT nextval('tipo_avaliacao_cod_tipo_avaliacao_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    conceitual smallint DEFAULT 1,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: tipo_avaliacao_valores; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_avaliacao_valores (
    ref_cod_tipo_avaliacao integer NOT NULL,
    sequencial integer NOT NULL,
    nome character varying(255) NOT NULL,
    valor double precision NOT NULL,
    valor_min double precision NOT NULL,
    valor_max double precision NOT NULL,
    ativo boolean DEFAULT true
);


--
-- Name: tipo_dispensa_cod_tipo_dispensa_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE tipo_dispensa_cod_tipo_dispensa_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_dispensa_cod_tipo_dispensa_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('tipo_dispensa_cod_tipo_dispensa_seq', 1, false);


--
-- Name: tipo_dispensa; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_dispensa (
    cod_tipo_dispensa integer DEFAULT nextval('tipo_dispensa_cod_tipo_dispensa_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: tipo_ensino_cod_tipo_ensino_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE tipo_ensino_cod_tipo_ensino_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_ensino_cod_tipo_ensino_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('tipo_ensino_cod_tipo_ensino_seq', 1, false);


--
-- Name: tipo_ensino; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_ensino (
    cod_tipo_ensino integer DEFAULT nextval('tipo_ensino_cod_tipo_ensino_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: tipo_ocorrencia_disciplinar_cod_tipo_ocorrencia_disciplinar_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE tipo_ocorrencia_disciplinar_cod_tipo_ocorrencia_disciplinar_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_ocorrencia_disciplinar_cod_tipo_ocorrencia_disciplinar_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('tipo_ocorrencia_disciplinar_cod_tipo_ocorrencia_disciplinar_seq', 1, false);


--
-- Name: tipo_ocorrencia_disciplinar; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_ocorrencia_disciplinar (
    cod_tipo_ocorrencia_disciplinar integer DEFAULT nextval('tipo_ocorrencia_disciplinar_cod_tipo_ocorrencia_disciplinar_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    max_ocorrencias integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: tipo_regime_cod_tipo_regime_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE tipo_regime_cod_tipo_regime_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_regime_cod_tipo_regime_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('tipo_regime_cod_tipo_regime_seq', 1, false);


--
-- Name: tipo_regime; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_regime (
    cod_tipo_regime integer DEFAULT nextval('tipo_regime_cod_tipo_regime_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint NOT NULL,
    ref_cod_instituicao integer NOT NULL
);


--
-- Name: tipo_usuario_cod_tipo_usuario_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE tipo_usuario_cod_tipo_usuario_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: tipo_usuario_cod_tipo_usuario_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('tipo_usuario_cod_tipo_usuario_seq', 3, true);


--
-- Name: tipo_usuario; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE tipo_usuario (
    cod_tipo_usuario integer DEFAULT nextval('tipo_usuario_cod_tipo_usuario_seq'::regclass) NOT NULL,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    nm_tipo character varying(255) NOT NULL,
    descricao text,
    nivel integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: transferencia_solicitacao_cod_transferencia_solicitacao_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE transferencia_solicitacao_cod_transferencia_solicitacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: transferencia_solicitacao_cod_transferencia_solicitacao_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('transferencia_solicitacao_cod_transferencia_solicitacao_seq', 1, false);


--
-- Name: transferencia_solicitacao; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE transferencia_solicitacao (
    cod_transferencia_solicitacao integer DEFAULT nextval('transferencia_solicitacao_cod_transferencia_solicitacao_seq'::regclass) NOT NULL,
    ref_cod_transferencia_tipo integer NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_cod_matricula_entrada integer,
    ref_cod_matricula_saida integer NOT NULL,
    observacao text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    data_transferencia timestamp without time zone
);


--
-- Name: transferencia_tipo_cod_transferencia_tipo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE transferencia_tipo_cod_transferencia_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: transferencia_tipo_cod_transferencia_tipo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('transferencia_tipo_cod_transferencia_tipo_seq', 1, false);


--
-- Name: transferencia_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE transferencia_tipo (
    cod_transferencia_tipo integer DEFAULT nextval('transferencia_tipo_cod_transferencia_tipo_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    desc_tipo text,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_escola integer NOT NULL
);


--
-- Name: turma_cod_turma_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE turma_cod_turma_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: turma_cod_turma_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('turma_cod_turma_seq', 1, false);


--
-- Name: turma; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE turma (
    cod_turma integer DEFAULT nextval('turma_cod_turma_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    ref_ref_cod_serie integer,
    ref_ref_cod_escola integer,
    ref_cod_infra_predio_comodo integer,
    nm_turma character varying(255) NOT NULL,
    sgl_turma character varying(15),
    max_aluno integer NOT NULL,
    multiseriada smallint DEFAULT (0)::smallint NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_turma_tipo integer NOT NULL,
    hora_inicial time without time zone,
    hora_final time without time zone,
    hora_inicio_intervalo time without time zone,
    hora_fim_intervalo time without time zone,
    ref_cod_regente integer,
    ref_cod_instituicao_regente integer,
    ref_cod_instituicao integer,
    ref_cod_curso integer,
    ref_ref_cod_serie_mult integer,
    ref_ref_cod_escola_mult integer,
    visivel boolean
);


--
-- Name: turma_dia_semana; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE turma_dia_semana (
    dia_semana numeric(1,0) NOT NULL,
    ref_cod_turma integer NOT NULL,
    hora_inicial time without time zone,
    hora_final time without time zone
);


--
-- Name: turma_modulo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE turma_modulo (
    ref_cod_turma integer NOT NULL,
    ref_cod_modulo integer NOT NULL,
    sequencial integer NOT NULL,
    data_inicio date NOT NULL,
    data_fim date NOT NULL
);


--
-- Name: turma_tipo_cod_turma_tipo_seq; Type: SEQUENCE; Schema: pmieducar; Owner: -
--

CREATE SEQUENCE turma_tipo_cod_turma_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: turma_tipo_cod_turma_tipo_seq; Type: SEQUENCE SET; Schema: pmieducar; Owner: -
--

SELECT pg_catalog.setval('turma_tipo_cod_turma_tipo_seq', 1, false);


--
-- Name: turma_tipo; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE turma_tipo (
    cod_turma_tipo integer DEFAULT nextval('turma_tipo_cod_turma_tipo_seq'::regclass) NOT NULL,
    ref_usuario_exc integer,
    ref_usuario_cad integer NOT NULL,
    nm_tipo character varying(255) NOT NULL,
    sgl_tipo character varying(15) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_instituicao integer
);


--
-- Name: usuario; Type: TABLE; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE TABLE usuario (
    cod_usuario integer NOT NULL,
    ref_cod_escola integer,
    ref_cod_instituicao integer,
    ref_funcionario_cad integer NOT NULL,
    ref_funcionario_exc integer,
    ref_cod_tipo_usuario integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: v_matricula_matricula_turma; Type: VIEW; Schema: pmieducar; Owner: -
--

CREATE VIEW v_matricula_matricula_turma AS
    SELECT ma.cod_matricula, ma.ref_ref_cod_escola AS ref_cod_escola, ma.ref_ref_cod_serie AS ref_cod_serie, ma.ref_cod_aluno, ma.ref_cod_curso, mt.ref_cod_turma, ma.ano, ma.aprovado, ma.ultima_matricula, ma.modulo, mt.sequencial, ma.ativo, (SELECT count(0) AS count FROM dispensa_disciplina dd WHERE (dd.ref_cod_matricula = ma.cod_matricula)) AS qtd_dispensa_disciplina, (SELECT COALESCE((max(n.modulo))::integer, 0) AS "coalesce" FROM nota_aluno n WHERE ((n.ref_cod_matricula = ma.cod_matricula) AND (n.ativo = 1))) AS maior_modulo_com_nota FROM matricula ma, matricula_turma mt WHERE ((mt.ref_cod_matricula = ma.cod_matricula) AND (mt.ativo = ma.ativo));


SET search_path = pmiotopic, pg_catalog;

--
-- Name: funcionario_su; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE funcionario_su (
    ref_ref_cod_pessoa_fj integer NOT NULL
);


--
-- Name: grupomoderador; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE grupomoderador (
    ref_ref_cod_pessoa_fj integer NOT NULL,
    ref_cod_grupos integer NOT NULL,
    ref_pessoa_exc integer,
    ref_pessoa_cad integer NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: grupopessoa; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE grupopessoa (
    ref_idpes integer NOT NULL,
    ref_cod_grupos integer NOT NULL,
    ref_grupos_exc integer,
    ref_pessoa_exc integer,
    ref_grupos_cad integer,
    ref_pessoa_cad integer,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_cod_auxiliar_cad integer,
    ref_ref_cod_atendimento_cad integer
);


--
-- Name: grupos_cod_grupos_seq; Type: SEQUENCE; Schema: pmiotopic; Owner: -
--

CREATE SEQUENCE grupos_cod_grupos_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: grupos_cod_grupos_seq; Type: SEQUENCE SET; Schema: pmiotopic; Owner: -
--

SELECT pg_catalog.setval('grupos_cod_grupos_seq', 1, false);


--
-- Name: grupos; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE grupos (
    cod_grupos integer DEFAULT nextval('grupos_cod_grupos_seq'::regclass) NOT NULL,
    ref_pessoa_exc integer,
    ref_pessoa_cad integer NOT NULL,
    nm_grupo character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    atendimento smallint DEFAULT 0 NOT NULL
);


--
-- Name: notas; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE notas (
    sequencial integer NOT NULL,
    ref_idpes integer NOT NULL,
    ref_pessoa_exc integer,
    ref_pessoa_cad integer NOT NULL,
    nota text NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: participante; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE participante (
    sequencial integer NOT NULL,
    ref_ref_cod_grupos integer NOT NULL,
    ref_ref_idpes integer NOT NULL,
    ref_cod_reuniao integer NOT NULL,
    data_chegada timestamp without time zone NOT NULL,
    data_saida timestamp without time zone
);


--
-- Name: reuniao_cod_reuniao_seq; Type: SEQUENCE; Schema: pmiotopic; Owner: -
--

CREATE SEQUENCE reuniao_cod_reuniao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: reuniao_cod_reuniao_seq; Type: SEQUENCE SET; Schema: pmiotopic; Owner: -
--

SELECT pg_catalog.setval('reuniao_cod_reuniao_seq', 1, false);


--
-- Name: reuniao; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE reuniao (
    cod_reuniao integer DEFAULT nextval('reuniao_cod_reuniao_seq'::regclass) NOT NULL,
    ref_grupos_moderador integer NOT NULL,
    ref_moderador integer NOT NULL,
    data_inicio_marcado timestamp without time zone NOT NULL,
    data_fim_marcado timestamp without time zone NOT NULL,
    data_inicio_real timestamp without time zone,
    data_fim_real timestamp without time zone,
    descricao text NOT NULL,
    email_enviado timestamp without time zone,
    publica smallint DEFAULT 0 NOT NULL
);


--
-- Name: topico_cod_topico_seq; Type: SEQUENCE; Schema: pmiotopic; Owner: -
--

CREATE SEQUENCE topico_cod_topico_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: topico_cod_topico_seq; Type: SEQUENCE SET; Schema: pmiotopic; Owner: -
--

SELECT pg_catalog.setval('topico_cod_topico_seq', 1, false);


--
-- Name: topico; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE topico (
    cod_topico integer DEFAULT nextval('topico_cod_topico_seq'::regclass) NOT NULL,
    ref_idpes_cad integer NOT NULL,
    ref_cod_grupos_cad integer NOT NULL,
    assunto character varying(255) NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_exclusao timestamp without time zone,
    ativo smallint DEFAULT (1)::smallint NOT NULL,
    ref_idpes_exc integer,
    ref_cod_grupos_exc integer
);


--
-- Name: topicoreuniao; Type: TABLE; Schema: pmiotopic; Owner: -; Tablespace:
--

CREATE TABLE topicoreuniao (
    ref_cod_topico integer NOT NULL,
    ref_cod_reuniao integer NOT NULL,
    parecer text,
    finalizado smallint,
    data_parecer timestamp without time zone
);


SET search_path = portal, pg_catalog;

--
-- Name: acesso_cod_acesso_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE acesso_cod_acesso_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: acesso_cod_acesso_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('acesso_cod_acesso_seq', 1, false);


--
-- Name: acesso; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE acesso (
    cod_acesso integer DEFAULT nextval('acesso_cod_acesso_seq'::regclass) NOT NULL,
    data_hora timestamp without time zone NOT NULL,
    ip_externo character varying(15) DEFAULT ''::character varying NOT NULL,
    ip_interno character varying(255) DEFAULT ''::character varying NOT NULL,
    cod_pessoa integer DEFAULT 0 NOT NULL,
    obs text,
    sucesso boolean DEFAULT true NOT NULL
);


--
-- Name: agenda_cod_agenda_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE agenda_cod_agenda_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: agenda_cod_agenda_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('agenda_cod_agenda_seq', 1, false);


--
-- Name: agenda; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE agenda (
    cod_agenda integer DEFAULT nextval('agenda_cod_agenda_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_exc integer,
    ref_ref_cod_pessoa_cad integer NOT NULL,
    nm_agenda character varying NOT NULL,
    publica smallint DEFAULT 0 NOT NULL,
    envia_alerta smallint DEFAULT 0 NOT NULL,
    data_cad timestamp without time zone NOT NULL,
    data_edicao timestamp without time zone,
    ref_ref_cod_pessoa_own integer
);


--
-- Name: agenda_compromisso; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE agenda_compromisso (
    cod_agenda_compromisso integer NOT NULL,
    versao integer NOT NULL,
    ref_cod_agenda integer NOT NULL,
    ref_ref_cod_pessoa_cad integer NOT NULL,
    ativo smallint DEFAULT 1,
    data_inicio timestamp without time zone,
    titulo character varying,
    descricao text,
    importante smallint DEFAULT 0 NOT NULL,
    publico smallint DEFAULT 0 NOT NULL,
    data_cadastro timestamp without time zone NOT NULL,
    data_fim timestamp without time zone
);


--
-- Name: agenda_pref_cod_comp_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE agenda_pref_cod_comp_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: agenda_pref_cod_comp_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('agenda_pref_cod_comp_seq', 1, false);


--
-- Name: agenda_pref; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE agenda_pref (
    cod_comp integer DEFAULT nextval('agenda_pref_cod_comp_seq'::regclass) NOT NULL,
    data_comp date NOT NULL,
    hora_comp time without time zone NOT NULL,
    hora_f_comp time without time zone NOT NULL,
    comp_comp text NOT NULL,
    local_comp character(1) DEFAULT 'I'::bpchar NOT NULL,
    publico_comp character(1) DEFAULT 'S'::bpchar NOT NULL,
    agenda_de character(1) DEFAULT 'P'::bpchar,
    ref_cad integer,
    versao integer DEFAULT 1 NOT NULL,
    ref_auto_cod integer
);


--
-- Name: agenda_responsavel; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE agenda_responsavel (
    ref_cod_agenda integer NOT NULL,
    ref_ref_cod_pessoa_fj integer NOT NULL,
    principal smallint
);


--
-- Name: compras_editais_editais_cod_compras_editais_editais_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_editais_editais_cod_compras_editais_editais_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_editais_editais_cod_compras_editais_editais_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_editais_editais_cod_compras_editais_editais_seq', 1, false);


--
-- Name: compras_editais_editais; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_editais_editais (
    cod_compras_editais_editais integer DEFAULT nextval('compras_editais_editais_cod_compras_editais_editais_seq'::regclass) NOT NULL,
    ref_cod_compras_licitacoes integer DEFAULT 0 NOT NULL,
    versao integer DEFAULT 0 NOT NULL,
    data_hora timestamp without time zone NOT NULL,
    arquivo character varying(255) DEFAULT ''::character varying NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    motivo_alteracao text,
    visivel smallint DEFAULT 1 NOT NULL
);


--
-- Name: compras_editais_editais_empresas; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_editais_editais_empresas (
    ref_cod_compras_editais_editais integer DEFAULT 0 NOT NULL,
    ref_cod_compras_editais_empresa integer DEFAULT 0 NOT NULL,
    data_hora timestamp without time zone NOT NULL
);


--
-- Name: compras_editais_empresa_cod_compras_editais_empresa_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_editais_empresa_cod_compras_editais_empresa_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_editais_empresa_cod_compras_editais_empresa_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_editais_empresa_cod_compras_editais_empresa_seq', 1, false);


--
-- Name: compras_editais_empresa; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_editais_empresa (
    cod_compras_editais_empresa integer DEFAULT nextval('compras_editais_empresa_cod_compras_editais_empresa_seq'::regclass) NOT NULL,
    cnpj character varying(20) DEFAULT ''::character varying NOT NULL,
    nm_empresa character varying(255) DEFAULT ''::character varying NOT NULL,
    email character varying(255) DEFAULT ''::character varying NOT NULL,
    data_hora timestamp without time zone NOT NULL,
    endereco text,
    ref_sigla_uf character(2),
    cidade character varying(255),
    bairro character varying(255),
    telefone bigint,
    fax bigint,
    cep bigint,
    nome_contato character varying(255),
    senha character varying(32) DEFAULT ''::character varying NOT NULL
);


--
-- Name: compras_final_pregao_cod_compras_final_pregao_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_final_pregao_cod_compras_final_pregao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_final_pregao_cod_compras_final_pregao_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_final_pregao_cod_compras_final_pregao_seq', 1, false);


--
-- Name: compras_final_pregao; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_final_pregao (
    cod_compras_final_pregao integer DEFAULT nextval('compras_final_pregao_cod_compras_final_pregao_seq'::regclass) NOT NULL,
    nm_final character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: compras_funcionarios; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_funcionarios (
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL
);


--
-- Name: compras_licitacoes_cod_compras_licitacoes_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_licitacoes_cod_compras_licitacoes_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_licitacoes_cod_compras_licitacoes_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_licitacoes_cod_compras_licitacoes_seq', 1, false);


--
-- Name: compras_licitacoes; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_licitacoes (
    cod_compras_licitacoes integer DEFAULT nextval('compras_licitacoes_cod_compras_licitacoes_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    ref_cod_compras_modalidade integer DEFAULT 0 NOT NULL,
    numero character varying(30) DEFAULT ''::character varying NOT NULL,
    objeto text NOT NULL,
    data_hora timestamp without time zone NOT NULL,
    cod_licitacao_semasa integer,
    oculto boolean DEFAULT false
);


--
-- Name: compras_modalidade_cod_compras_modalidade_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_modalidade_cod_compras_modalidade_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_modalidade_cod_compras_modalidade_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_modalidade_cod_compras_modalidade_seq', 1, false);


--
-- Name: compras_modalidade; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_modalidade (
    cod_compras_modalidade integer DEFAULT nextval('compras_modalidade_cod_compras_modalidade_seq'::regclass) NOT NULL,
    nm_modalidade character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: compras_pregao_execucao_cod_compras_pregao_execucao_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_pregao_execucao_cod_compras_pregao_execucao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_pregao_execucao_cod_compras_pregao_execucao_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_pregao_execucao_cod_compras_pregao_execucao_seq', 1, false);


--
-- Name: compras_pregao_execucao; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_pregao_execucao (
    cod_compras_pregao_execucao integer DEFAULT nextval('compras_pregao_execucao_cod_compras_pregao_execucao_seq'::regclass) NOT NULL,
    ref_cod_compras_licitacoes integer DEFAULT 0 NOT NULL,
    ref_pregoeiro integer DEFAULT 0 NOT NULL,
    ref_equipe1 integer DEFAULT 0 NOT NULL,
    ref_equipe2 integer DEFAULT 0 NOT NULL,
    ref_equipe3 integer DEFAULT 0 NOT NULL,
    ano_processo integer,
    mes_processo integer,
    seq_processo integer,
    seq_portaria integer,
    ano_portaria integer,
    valor_referencia double precision,
    valor_real double precision,
    ref_cod_compras_final_pregao integer
);


--
-- Name: compras_prestacao_contas_cod_compras_prestacao_contas_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE compras_prestacao_contas_cod_compras_prestacao_contas_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: compras_prestacao_contas_cod_compras_prestacao_contas_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('compras_prestacao_contas_cod_compras_prestacao_contas_seq', 1, false);


--
-- Name: compras_prestacao_contas; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE compras_prestacao_contas (
    cod_compras_prestacao_contas integer DEFAULT nextval('compras_prestacao_contas_cod_compras_prestacao_contas_seq'::regclass) NOT NULL,
    caminho character varying(255) DEFAULT ''::character varying NOT NULL,
    mes integer DEFAULT 0 NOT NULL,
    ano integer DEFAULT 0 NOT NULL
);


--
-- Name: foto_portal_cod_foto_portal_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE foto_portal_cod_foto_portal_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: foto_portal_cod_foto_portal_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('foto_portal_cod_foto_portal_seq', 1, false);


--
-- Name: foto_portal; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE foto_portal (
    cod_foto_portal integer DEFAULT nextval('foto_portal_cod_foto_portal_seq'::regclass) NOT NULL,
    ref_cod_foto_secao integer,
    ref_cod_credito integer,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    data_foto timestamp without time zone,
    titulo character varying(255),
    descricao text,
    caminho character varying(255),
    altura integer,
    largura integer,
    nm_credito character varying(255),
    bkp_ref_secao bigint
);


--
-- Name: foto_secao_cod_foto_secao_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE foto_secao_cod_foto_secao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: foto_secao_cod_foto_secao_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('foto_secao_cod_foto_secao_seq', 1, false);


--
-- Name: foto_secao; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE foto_secao (
    cod_foto_secao integer DEFAULT nextval('foto_secao_cod_foto_secao_seq'::regclass) NOT NULL,
    nm_secao character varying(255)
);


--
-- Name: funcionario; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE funcionario (
    ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    matricula character varying(12),
    senha character varying(32),
    ativo smallint,
    ref_sec integer,
    ramal character varying(10),
    sequencial character(3),
    opcao_menu text,
    ref_cod_setor integer,
    ref_cod_funcionario_vinculo integer,
    tempo_expira_senha integer,
    tempo_expira_conta integer,
    data_troca_senha date,
    data_reativa_conta date,
    ref_ref_cod_pessoa_fj integer,
    proibido integer DEFAULT 0 NOT NULL,
    ref_cod_setor_new integer,
    matricula_new bigint,
    matricula_permanente smallint DEFAULT 0,
    tipo_menu smallint DEFAULT 0 NOT NULL,
    ip_logado character varying(15),
    data_login timestamp without time zone
);


--
-- Name: funcionario_vinculo_cod_funcionario_vinculo_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE funcionario_vinculo_cod_funcionario_vinculo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: funcionario_vinculo_cod_funcionario_vinculo_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('funcionario_vinculo_cod_funcionario_vinculo_seq', 1, false);


--
-- Name: funcionario_vinculo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE funcionario_vinculo (
    cod_funcionario_vinculo integer DEFAULT nextval('funcionario_vinculo_cod_funcionario_vinculo_seq'::regclass) NOT NULL,
    nm_vinculo character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: imagem_cod_imagem_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE imagem_cod_imagem_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: imagem_cod_imagem_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('imagem_cod_imagem_seq', 186, true);


--
-- Name: imagem; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE imagem (
    cod_imagem integer DEFAULT nextval('imagem_cod_imagem_seq'::regclass) NOT NULL,
    ref_cod_imagem_tipo integer NOT NULL,
    caminho character varying(255) NOT NULL,
    nm_imagem character varying(100),
    extensao character(3) NOT NULL,
    altura integer,
    largura integer,
    data_cadastro timestamp without time zone NOT NULL,
    ref_cod_pessoa_cad integer NOT NULL,
    data_exclusao timestamp without time zone,
    ref_cod_pessoa_exc integer
);


--
-- Name: imagem_tipo_cod_imagem_tipo_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE imagem_tipo_cod_imagem_tipo_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: imagem_tipo_cod_imagem_tipo_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('imagem_tipo_cod_imagem_tipo_seq', 6, true);


--
-- Name: imagem_tipo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE imagem_tipo (
    cod_imagem_tipo integer DEFAULT nextval('imagem_tipo_cod_imagem_tipo_seq'::regclass) NOT NULL,
    nm_tipo character varying(100) NOT NULL
);


--
-- Name: intranet_segur_permissao_nega_cod_intranet_segur_permissao__seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE intranet_segur_permissao_nega_cod_intranet_segur_permissao__seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: intranet_segur_permissao_nega_cod_intranet_segur_permissao__seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('intranet_segur_permissao_nega_cod_intranet_segur_permissao__seq', 1, false);


--
-- Name: intranet_segur_permissao_negada; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE intranet_segur_permissao_negada (
    cod_intranet_segur_permissao_negada integer DEFAULT nextval('intranet_segur_permissao_nega_cod_intranet_segur_permissao__seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer,
    ip_externo character varying(15) DEFAULT ''::character varying NOT NULL,
    ip_interno character varying(255),
    data_hora timestamp without time zone NOT NULL,
    pagina character varying(255),
    variaveis text
);


--
-- Name: jor_arquivo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE jor_arquivo (
    ref_cod_jor_edicao integer DEFAULT 0 NOT NULL,
    jor_arquivo smallint DEFAULT (0)::smallint NOT NULL,
    jor_caminho character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: jor_edicao_cod_jor_edicao_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE jor_edicao_cod_jor_edicao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: jor_edicao_cod_jor_edicao_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('jor_edicao_cod_jor_edicao_seq', 1, false);


--
-- Name: jor_edicao; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE jor_edicao (
    cod_jor_edicao integer DEFAULT nextval('jor_edicao_cod_jor_edicao_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    jor_ano_edicao character varying(5) DEFAULT ''::character varying NOT NULL,
    jor_edicao integer DEFAULT 0 NOT NULL,
    jor_dt_inicial date NOT NULL,
    jor_dt_final date,
    jor_extra smallint DEFAULT (0)::smallint
);


--
-- Name: mailling_email_cod_mailling_email_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE mailling_email_cod_mailling_email_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: mailling_email_cod_mailling_email_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('mailling_email_cod_mailling_email_seq', 1, false);


--
-- Name: mailling_email; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE mailling_email (
    cod_mailling_email integer DEFAULT nextval('mailling_email_cod_mailling_email_seq'::regclass) NOT NULL,
    nm_pessoa character varying(255) DEFAULT ''::character varying NOT NULL,
    email character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: mailling_email_conteudo_cod_mailling_email_conteudo_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE mailling_email_conteudo_cod_mailling_email_conteudo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: mailling_email_conteudo_cod_mailling_email_conteudo_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('mailling_email_conteudo_cod_mailling_email_conteudo_seq', 1, false);


--
-- Name: mailling_email_conteudo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE mailling_email_conteudo (
    cod_mailling_email_conteudo integer DEFAULT nextval('mailling_email_conteudo_cod_mailling_email_conteudo_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    conteudo text NOT NULL,
    nm_remetente character varying(255),
    email_remetente character varying(255),
    assunto character varying(255)
);


--
-- Name: mailling_fila_envio_cod_mailling_fila_envio_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE mailling_fila_envio_cod_mailling_fila_envio_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: mailling_fila_envio_cod_mailling_fila_envio_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('mailling_fila_envio_cod_mailling_fila_envio_seq', 1, false);


--
-- Name: mailling_fila_envio; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE mailling_fila_envio (
    cod_mailling_fila_envio integer DEFAULT nextval('mailling_fila_envio_cod_mailling_fila_envio_seq'::regclass) NOT NULL,
    ref_cod_mailling_email_conteudo integer DEFAULT 0 NOT NULL,
    ref_cod_mailling_email integer,
    ref_ref_cod_pessoa_fj integer,
    data_cadastro timestamp without time zone NOT NULL,
    data_envio timestamp without time zone
);


--
-- Name: mailling_grupo_cod_mailling_grupo_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE mailling_grupo_cod_mailling_grupo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: mailling_grupo_cod_mailling_grupo_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('mailling_grupo_cod_mailling_grupo_seq', 1, false);


--
-- Name: mailling_grupo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE mailling_grupo (
    cod_mailling_grupo integer DEFAULT nextval('mailling_grupo_cod_mailling_grupo_seq'::regclass) NOT NULL,
    nm_grupo character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: mailling_grupo_email; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE mailling_grupo_email (
    ref_cod_mailling_email integer DEFAULT 0 NOT NULL,
    ref_cod_mailling_grupo integer DEFAULT 0 NOT NULL
);


--
-- Name: mailling_historico_cod_mailling_historico_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE mailling_historico_cod_mailling_historico_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: mailling_historico_cod_mailling_historico_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('mailling_historico_cod_mailling_historico_seq', 1, false);


--
-- Name: mailling_historico; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE mailling_historico (
    cod_mailling_historico integer DEFAULT nextval('mailling_historico_cod_mailling_historico_seq'::regclass) NOT NULL,
    ref_cod_not_portal integer DEFAULT 0 NOT NULL,
    ref_cod_mailling_grupo integer DEFAULT 0 NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    data_hora timestamp without time zone NOT NULL
);


--
-- Name: menu_funcionario; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE menu_funcionario (
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    cadastra smallint DEFAULT (0)::smallint NOT NULL,
    exclui smallint DEFAULT (0)::smallint NOT NULL,
    ref_cod_menu_submenu integer DEFAULT 0 NOT NULL
);


--
-- Name: menu_menu_cod_menu_menu_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE menu_menu_cod_menu_menu_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: menu_menu_cod_menu_menu_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('menu_menu_cod_menu_menu_seq', 68, true);


--
-- Name: menu_menu; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE menu_menu (
    cod_menu_menu integer DEFAULT nextval('menu_menu_cod_menu_menu_seq'::regclass) NOT NULL,
    nm_menu character varying(255) DEFAULT ''::character varying NOT NULL,
    title character varying(255),
    ref_cod_menu_pai integer
);


--
-- Name: menu_submenu_cod_menu_submenu_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE menu_submenu_cod_menu_submenu_seq
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: menu_submenu_cod_menu_submenu_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('menu_submenu_cod_menu_submenu_seq', 944, true);


--
-- Name: menu_submenu; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE menu_submenu (
    cod_menu_submenu integer DEFAULT nextval('menu_submenu_cod_menu_submenu_seq'::regclass) NOT NULL,
    ref_cod_menu_menu integer,
    cod_sistema integer,
    nm_submenu character varying(100) DEFAULT ''::character varying NOT NULL,
    arquivo character varying(255) DEFAULT ''::character varying NOT NULL,
    title text,
    nivel smallint DEFAULT (3)::smallint NOT NULL
);


--
-- Name: not_portal_cod_not_portal_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE not_portal_cod_not_portal_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: not_portal_cod_not_portal_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('not_portal_cod_not_portal_seq', 1, false);


--
-- Name: not_portal; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE not_portal (
    cod_not_portal integer DEFAULT nextval('not_portal_cod_not_portal_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    titulo character varying(255),
    descricao text,
    data_noticia timestamp without time zone NOT NULL
);


--
-- Name: not_portal_tipo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE not_portal_tipo (
    ref_cod_not_portal integer DEFAULT 0 NOT NULL,
    ref_cod_not_tipo integer DEFAULT 0 NOT NULL
);


--
-- Name: not_tipo_cod_not_tipo_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE not_tipo_cod_not_tipo_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: not_tipo_cod_not_tipo_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('not_tipo_cod_not_tipo_seq', 1, false);


--
-- Name: not_tipo; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE not_tipo (
    cod_not_tipo integer DEFAULT nextval('not_tipo_cod_not_tipo_seq'::regclass) NOT NULL,
    nm_tipo character varying(255) DEFAULT ''::character varying NOT NULL
);


--
-- Name: not_vinc_portal; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE not_vinc_portal (
    ref_cod_not_portal integer DEFAULT 0 NOT NULL,
    vic_num integer DEFAULT 0 NOT NULL,
    tipo character(1) DEFAULT 'F'::bpchar NOT NULL,
    cod_vinc integer,
    caminho character varying(255),
    nome_arquivo character varying(255)
);


--
-- Name: notificacao_cod_notificacao_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE notificacao_cod_notificacao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: notificacao_cod_notificacao_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('notificacao_cod_notificacao_seq', 1, false);


--
-- Name: notificacao; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE notificacao (
    cod_notificacao integer DEFAULT nextval('notificacao_cod_notificacao_seq'::regclass) NOT NULL,
    ref_cod_funcionario integer NOT NULL,
    titulo character varying,
    conteudo text,
    data_hora_ativa timestamp without time zone,
    url character varying,
    visualizacoes smallint DEFAULT 0 NOT NULL
);


--
-- Name: pessoa_atividade_cod_pessoa_atividade_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE pessoa_atividade_cod_pessoa_atividade_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: pessoa_atividade_cod_pessoa_atividade_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('pessoa_atividade_cod_pessoa_atividade_seq', 1, false);


--
-- Name: pessoa_atividade; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE pessoa_atividade (
    cod_pessoa_atividade integer DEFAULT nextval('pessoa_atividade_cod_pessoa_atividade_seq'::regclass) NOT NULL,
    ref_cod_ramo_atividade integer DEFAULT 0 NOT NULL,
    nm_atividade character varying(255)
);


--
-- Name: pessoa_fj_cod_pessoa_fj_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE pessoa_fj_cod_pessoa_fj_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: pessoa_fj_cod_pessoa_fj_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('pessoa_fj_cod_pessoa_fj_seq', 1, false);


--
-- Name: pessoa_fj; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE pessoa_fj (
    cod_pessoa_fj integer DEFAULT nextval('pessoa_fj_cod_pessoa_fj_seq'::regclass) NOT NULL,
    nm_pessoa character varying(255) DEFAULT ''::character varying NOT NULL,
    id_federal character varying(30),
    endereco text,
    cep character varying(9),
    ref_bairro integer,
    ddd_telefone_1 integer,
    telefone_1 character varying(15),
    ddd_telefone_2 integer,
    telefone_2 character varying(15),
    ddd_telefone_mov integer,
    telefone_mov character varying(15),
    ddd_telefone_fax integer,
    telefone_fax character varying(15),
    email character varying(255),
    http character varying(255),
    tipo_pessoa character(1) DEFAULT 'F'::bpchar NOT NULL,
    sexo smallint,
    razao_social character varying(255),
    ins_est character varying(30),
    ins_mun character varying(30),
    rg character varying(30),
    ref_cod_pessoa_pai integer,
    ref_cod_pessoa_mae integer,
    data_nasc date,
    ref_ref_cod_pessoa_fj integer
);


--
-- Name: pessoa_fj_pessoa_atividade; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE pessoa_fj_pessoa_atividade (
    ref_cod_pessoa_atividade integer DEFAULT 0 NOT NULL,
    ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL
);


--
-- Name: pessoa_ramo_atividade_cod_ramo_atividade_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE pessoa_ramo_atividade_cod_ramo_atividade_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: pessoa_ramo_atividade_cod_ramo_atividade_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('pessoa_ramo_atividade_cod_ramo_atividade_seq', 1, false);


--
-- Name: pessoa_ramo_atividade; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE pessoa_ramo_atividade (
    cod_ramo_atividade integer DEFAULT nextval('pessoa_ramo_atividade_cod_ramo_atividade_seq'::regclass) NOT NULL,
    nm_ramo_atividade character varying(255)
);


--
-- Name: portal_banner_cod_portal_banner_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE portal_banner_cod_portal_banner_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: portal_banner_cod_portal_banner_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('portal_banner_cod_portal_banner_seq', 1, false);


--
-- Name: portal_banner; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE portal_banner (
    cod_portal_banner integer DEFAULT nextval('portal_banner_cod_portal_banner_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    caminho character varying(255) DEFAULT ''::character varying NOT NULL,
    title character varying(255),
    prioridade integer DEFAULT 0 NOT NULL,
    link character varying(255) DEFAULT ''::character varying NOT NULL,
    lateral smallint DEFAULT (1)::smallint NOT NULL
);


--
-- Name: portal_concurso_cod_portal_concurso_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE portal_concurso_cod_portal_concurso_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: portal_concurso_cod_portal_concurso_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('portal_concurso_cod_portal_concurso_seq', 1, false);


--
-- Name: portal_concurso; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE portal_concurso (
    cod_portal_concurso integer DEFAULT nextval('portal_concurso_cod_portal_concurso_seq'::regclass) NOT NULL,
    ref_ref_cod_pessoa_fj integer DEFAULT 0 NOT NULL,
    nm_concurso character varying(255) DEFAULT ''::character varying NOT NULL,
    descricao text,
    caminho character varying(255) DEFAULT ''::character varying NOT NULL,
    tipo_arquivo character(3) DEFAULT ''::bpchar NOT NULL,
    data_hora timestamp without time zone
);


--
-- Name: sistema_cod_sistema_seq; Type: SEQUENCE; Schema: portal; Owner: -
--

CREATE SEQUENCE sistema_cod_sistema_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: sistema_cod_sistema_seq; Type: SEQUENCE SET; Schema: portal; Owner: -
--

SELECT pg_catalog.setval('sistema_cod_sistema_seq', 1, false);


--
-- Name: sistema; Type: TABLE; Schema: portal; Owner: -; Tablespace:
--

CREATE TABLE sistema (
    cod_sistema integer DEFAULT nextval('sistema_cod_sistema_seq'::regclass) NOT NULL,
    nome character varying(255),
    versao smallint NOT NULL,
    release smallint NOT NULL,
    patch smallint NOT NULL,
    tipo character varying(255)
);


--
-- Name: v_funcionario; Type: VIEW; Schema: portal; Owner: -
--

CREATE VIEW v_funcionario AS
    SELECT f.ref_cod_pessoa_fj, f.matricula, f.senha, f.ativo, f.ramal, f.sequencial, f.opcao_menu, f.ref_cod_setor, f.ref_cod_funcionario_vinculo, f.tempo_expira_senha, f.tempo_expira_conta, f.data_troca_senha, f.data_reativa_conta, f.ref_ref_cod_pessoa_fj, f.proibido, f.ref_cod_setor_new, (SELECT pessoa.nome FROM cadastro.pessoa WHERE (pessoa.idpes = (f.ref_cod_pessoa_fj)::numeric)) AS nome FROM funcionario f;


SET search_path = public, pg_catalog;

--
-- Name: bairro_regiao; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE bairro_regiao (
    ref_cod_regiao integer NOT NULL,
    ref_idbai integer NOT NULL
);


SET default_with_oids = false;

--
-- Name: changelog; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE changelog (
    change_number bigint NOT NULL,
    delta_set character varying(10) NOT NULL,
    start_dt timestamp without time zone NOT NULL,
    complete_dt timestamp without time zone,
    applied_by character varying(100) NOT NULL,
    description character varying(500) NOT NULL
);


SET default_with_oids = true;

--
-- Name: logradouro_fonetico; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE logradouro_fonetico (
    fonema character varying(30) NOT NULL,
    idlog numeric(8,0) NOT NULL
);


--
-- Name: pais; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE pais (
    idpais numeric(3,0) NOT NULL,
    nome character varying(60) NOT NULL,
    geom character varying
);


--
-- Name: regiao_cod_regiao_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE regiao_cod_regiao_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: regiao_cod_regiao_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('regiao_cod_regiao_seq', 1, false);


--
-- Name: regiao; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE regiao (
    cod_regiao integer DEFAULT nextval('regiao_cod_regiao_seq'::regclass) NOT NULL,
    nm_regiao character varying(100)
);


--
-- Name: seq_bairro; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE seq_bairro
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: seq_bairro; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('seq_bairro', 1, false);


--
-- Name: seq_logradouro; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE seq_logradouro
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: seq_logradouro; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('seq_logradouro', 1, false);


--
-- Name: seq_municipio; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE seq_municipio
    START WITH 5565
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: seq_municipio; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('seq_municipio', 5565, false);


--
-- Name: setor_idset_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE setor_idset_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 0
    CACHE 1;


--
-- Name: setor_idset_seq; Type: SEQUENCE SET; Schema: public; Owner: -
--

SELECT pg_catalog.setval('setor_idset_seq', 1, false);


--
-- Name: setor; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE setor (
    idset integer DEFAULT nextval('setor_idset_seq'::regclass) NOT NULL,
    nivel numeric(1,0) NOT NULL,
    nome character varying(100) NOT NULL,
    sigla character varying(25),
    idsetsub integer,
    idsetredir integer,
    situacao character(1) NOT NULL,
    localizacao character(1) NOT NULL,
    CONSTRAINT ck_setor_localizacao CHECK (((localizacao = 'E'::bpchar) OR (localizacao = 'I'::bpchar))),
    CONSTRAINT ck_setor_situacao CHECK (((situacao = 'A'::bpchar) OR (situacao = 'I'::bpchar)))
);


--
-- Name: uf; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE uf (
    sigla_uf character(2) NOT NULL,
    nome character varying(30) NOT NULL,
    geom character varying,
    idpais numeric(3,0)
);


--
-- Name: vila; Type: TABLE; Schema: public; Owner: -; Tablespace:
--

CREATE TABLE vila (
    idvil numeric(4,0) NOT NULL,
    idmun numeric(6,0) NOT NULL,
    nome character varying(50) NOT NULL,
    geom character varying
);


SET search_path = urbano, pg_catalog;

--
-- Name: cep_logradouro; Type: TABLE; Schema: urbano; Owner: -; Tablespace:
--

CREATE TABLE cep_logradouro (
    cep numeric(8,0) NOT NULL,
    idlog numeric(6,0) NOT NULL,
    nroini numeric(6,0),
    nrofin numeric(6,0),
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_cep_logradouro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_logradouro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar)))
);


--
-- Name: cep_logradouro_bairro; Type: TABLE; Schema: urbano; Owner: -; Tablespace:
--

CREATE TABLE cep_logradouro_bairro (
    idlog numeric(6,0) NOT NULL,
    cep numeric(8,0) NOT NULL,
    idbai numeric(6,0) NOT NULL,
    idpes_rev numeric,
    data_rev timestamp without time zone,
    origem_gravacao character(1) NOT NULL,
    idpes_cad numeric,
    data_cad timestamp without time zone NOT NULL,
    operacao character(1) NOT NULL,
    idsis_rev integer,
    idsis_cad integer NOT NULL,
    CONSTRAINT ck_cep_logradouro_bairro_origem_gravacao CHECK (((((origem_gravacao = 'M'::bpchar) OR (origem_gravacao = 'U'::bpchar)) OR (origem_gravacao = 'C'::bpchar)) OR (origem_gravacao = 'O'::bpchar))),
    CONSTRAINT ck_logradouro_operacao CHECK ((((operacao = 'I'::bpchar) OR (operacao = 'A'::bpchar)) OR (operacao = 'E'::bpchar)))
);


--
-- Name: tipo_logradouro; Type: TABLE; Schema: urbano; Owner: -; Tablespace:
--

CREATE TABLE tipo_logradouro (
    idtlog character varying(5) NOT NULL,
    descricao character varying(40) NOT NULL
);


SET search_path = modules, pg_catalog;

--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE area_conhecimento ALTER COLUMN id SET DEFAULT nextval('area_conhecimento_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE componente_curricular ALTER COLUMN id SET DEFAULT nextval('componente_curricular_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE docente_licenciatura ALTER COLUMN id SET DEFAULT nextval('docente_licenciatura_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE educacenso_curso_superior ALTER COLUMN id SET DEFAULT nextval('educacenso_curso_superior_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE educacenso_ies ALTER COLUMN id SET DEFAULT nextval('educacenso_ies_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE falta_aluno ALTER COLUMN id SET DEFAULT nextval('falta_aluno_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE falta_componente_curricular ALTER COLUMN id SET DEFAULT nextval('falta_componente_curricular_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE falta_geral ALTER COLUMN id SET DEFAULT nextval('falta_geral_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE formula_media ALTER COLUMN id SET DEFAULT nextval('formula_media_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE nota_aluno ALTER COLUMN id SET DEFAULT nextval('nota_aluno_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE nota_componente_curricular ALTER COLUMN id SET DEFAULT nextval('nota_componente_curricular_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE parecer_aluno ALTER COLUMN id SET DEFAULT nextval('parecer_aluno_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE parecer_componente_curricular ALTER COLUMN id SET DEFAULT nextval('parecer_componente_curricular_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE parecer_geral ALTER COLUMN id SET DEFAULT nextval('parecer_geral_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE regra_avaliacao ALTER COLUMN id SET DEFAULT nextval('regra_avaliacao_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE tabela_arredondamento ALTER COLUMN id SET DEFAULT nextval('tabela_arredondamento_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: modules; Owner: -
--

ALTER TABLE tabela_arredondamento_valor ALTER COLUMN id SET DEFAULT nextval('tabela_arredondamento_valor_id_seq'::regclass);


SET search_path = acesso, pg_catalog;

--
-- Data for Name: funcao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: grupo; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: grupo_funcao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: grupo_menu; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: grupo_operacao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: grupo_sistema; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: historico_senha; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: instituicao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: log_acesso; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: log_erro; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: menu; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: operacao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: operacao_funcao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: pessoa_instituicao; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: sistema; Type: TABLE DATA; Schema: acesso; Owner: -
--

INSERT INTO sistema VALUES (1, 'i-Educar', 'Sistema bsico', '', 'A');
INSERT INTO sistema VALUES (9, 'Cdigo legado', 'Sistema bsico', '', 'A');
INSERT INTO sistema VALUES (17, 'Cdigo legado', 'Sistema bsico', '', 'A');


--
-- Data for Name: usuario; Type: TABLE DATA; Schema: acesso; Owner: -
--



--
-- Data for Name: usuario_grupo; Type: TABLE DATA; Schema: acesso; Owner: -
--



SET search_path = alimentos, pg_catalog;

--
-- Data for Name: baixa_guia_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: baixa_guia_remessa; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: calendario; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: cardapio; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: cardapio_faixa_unidade; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: cardapio_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: cardapio_receita; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: cliente; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: composto_quimico; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: contrato; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: contrato_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: evento; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: faixa_composto_quimico; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: faixa_etaria; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: fornecedor; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: fornecedor_unidade_atendida; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: grupo_quimico; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: guia_produto_diario; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: guia_remessa; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: guia_remessa_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: log_guia_remessa; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: medidas_caseiras; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: pessoa; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: produto_composto_quimico; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: produto_fornecedor; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: produto_medida_caseira; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: receita; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: receita_composto_quimico; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: receita_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: tipo_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: tipo_refeicao; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: tipo_unidade; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: unidade_atendida; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: unidade_faixa_etaria; Type: TABLE DATA; Schema: alimentos; Owner: -
--



--
-- Data for Name: unidade_produto; Type: TABLE DATA; Schema: alimentos; Owner: -
--



SET search_path = cadastro, pg_catalog;

--
-- Data for Name: aviso_nome; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: deficiencia; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: documento; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: endereco_externo; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: endereco_pessoa; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: escolaridade; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: estado_civil; Type: TABLE DATA; Schema: cadastro; Owner: -
--

INSERT INTO estado_civil VALUES (1, 'Solteiro(a)');
INSERT INTO estado_civil VALUES (2, 'Casado(a)');
INSERT INTO estado_civil VALUES (3, 'Divorciado(a)');
INSERT INTO estado_civil VALUES (4, 'Separado(a)');
INSERT INTO estado_civil VALUES (5, 'Vivo(a)');
INSERT INTO estado_civil VALUES (6, 'Companheiro(a)');


--
-- Data for Name: fisica; Type: TABLE DATA; Schema: cadastro; Owner: -
--

INSERT INTO fisica VALUES (1, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 'M', NULL, 'NOW()', 'I', NULL, 1, NULL, NULL, NULL);


--
-- Data for Name: fisica_cpf; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: fisica_deficiencia; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: fisica_foto; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: fisica_raca; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: fisica_sangue; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: fone_pessoa; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: funcionario; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: historico_cartao; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: juridica; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: ocupacao; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: orgao_emissor_rg; Type: TABLE DATA; Schema: cadastro; Owner: -
--

INSERT INTO orgao_emissor_rg VALUES (1, 'SSP', 'SSP', 'I');
INSERT INTO orgao_emissor_rg VALUES (2, 'MMil', 'Ministrios Militares', 'I');
INSERT INTO orgao_emissor_rg VALUES (3, 'MAer', 'Ministrio da Aeronutica', 'I');
INSERT INTO orgao_emissor_rg VALUES (4, 'MExe', 'Ministrio do Exrcito', 'I');
INSERT INTO orgao_emissor_rg VALUES (5, 'MMar', 'Ministrio da Marinha', 'I');
INSERT INTO orgao_emissor_rg VALUES (6, 'PF', 'Polcia Federal', 'I');
INSERT INTO orgao_emissor_rg VALUES (7, 'CIC', 'Carteira de Identidade Classista', 'I');
INSERT INTO orgao_emissor_rg VALUES (8, 'CRA', 'Conselho Regional de Administrao', 'I');
INSERT INTO orgao_emissor_rg VALUES (10, 'CRB', 'Conselho Regional de Biblioteconomia', 'I');
INSERT INTO orgao_emissor_rg VALUES (11, 'CRC', 'Conselho Regional de Contabilidade', 'I');
INSERT INTO orgao_emissor_rg VALUES (12, 'CRECI', 'Conselho Regional de Corretores Imveis', 'I');
INSERT INTO orgao_emissor_rg VALUES (14, 'CREA', 'Conselho Regional de Engenharia, Arquitetura e Agronomia', 'I');
INSERT INTO orgao_emissor_rg VALUES (18, 'CRM', 'Conselho Regional de Medicina', 'I');
INSERT INTO orgao_emissor_rg VALUES (19, 'CRMV', 'Conselho Regional de Medicina Veterinria', 'I');
INSERT INTO orgao_emissor_rg VALUES (20, 'OMB', 'Ordem dos Msicos do Brasil', 'I');
INSERT INTO orgao_emissor_rg VALUES (22, 'CRO', 'Conselho Regional de Odontologia', 'I');
INSERT INTO orgao_emissor_rg VALUES (24, 'CRP', 'Conselho Regional de Psicologia', 'I');
INSERT INTO orgao_emissor_rg VALUES (25, 'CRQ', 'Conselho Regional de Qumica', 'I');
INSERT INTO orgao_emissor_rg VALUES (27, 'OAB', 'Ordem dos Advogados do Brasil', 'I');
INSERT INTO orgao_emissor_rg VALUES (28, 'OEmi', 'Outros Emissores', 'I');
INSERT INTO orgao_emissor_rg VALUES (29, 'DExt', 'Documento Estrangeiro', 'I');
INSERT INTO orgao_emissor_rg VALUES (9, 'CRESS', 'Conselho Regional de Assist. Social', 'I');
INSERT INTO orgao_emissor_rg VALUES (13, 'COREN', 'Conselho Regional de Enfermagem', 'I');
INSERT INTO orgao_emissor_rg VALUES (15, 'CONRE', 'Conselho Regional de Estatstica', 'I');
INSERT INTO orgao_emissor_rg VALUES (16, 'CRF', 'Conselho Regional de Farmcia', 'I');
INSERT INTO orgao_emissor_rg VALUES (17, 'CREFITO', 'Conselho Regional de Fisioterapia e Terapia Ocupacional', 'I');
INSERT INTO orgao_emissor_rg VALUES (21, 'CRN', 'Conselho Regional de Nutrio', 'I');
INSERT INTO orgao_emissor_rg VALUES (23, 'CONRERP', 'Conselho Regional de Profissionais de Relaes Pblicas', 'I');
INSERT INTO orgao_emissor_rg VALUES (26, 'CORE', 'Conselho Regional de Representantes Comerciais', 'I');


--
-- Data for Name: pessoa; Type: TABLE DATA; Schema: cadastro; Owner: -
--

INSERT INTO pessoa VALUES (1, 'Administrador', NULL, 'NOW()', NULL, 'F', NULL, NULL, '', 'A', 'M', 'I', NULL, 1);


--
-- Data for Name: pessoa_fonetico; Type: TABLE DATA; Schema: cadastro; Owner: -
--

INSERT INTO pessoa_fonetico VALUES (1, 'ADIMINISTIHADOH');


--
-- Data for Name: raca; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: religiao; Type: TABLE DATA; Schema: cadastro; Owner: -
--



--
-- Data for Name: socio; Type: TABLE DATA; Schema: cadastro; Owner: -
--



SET search_path = consistenciacao, pg_catalog;

--
-- Data for Name: campo_consistenciacao; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--

INSERT INTO campo_consistenciacao VALUES (1, 'CPF ou CNPJ', 'N', 14);
INSERT INTO campo_consistenciacao VALUES (2, 'Nome pessoa fsica ou jurdica', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (3, 'E-mail', 'N', 50);
INSERT INTO campo_consistenciacao VALUES (4, 'URL', 'N', 60);
INSERT INTO campo_consistenciacao VALUES (5, 'Data nascimento', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (6, 'Nmero RG', 'N', 10);
INSERT INTO campo_consistenciacao VALUES (7, 'rgo expedidor RG', 'S', 20);
INSERT INTO campo_consistenciacao VALUES (8, 'Data expedio RG', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (9, 'UF expedio RG', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (10, 'Tipo certido civil', 'S', 2);
INSERT INTO campo_consistenciacao VALUES (11, 'Nmero termo certido civil', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (12, 'Nmero livro certido civil', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (13, 'Nmero folha certido civil', 'N', 4);
INSERT INTO campo_consistenciacao VALUES (14, 'Data emisso certido civil', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (15, 'Cartrio emissor certido civil', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (16, 'UF emisso certido civil', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (17, 'Nmero carteira trabalho', 'N', 7);
INSERT INTO campo_consistenciacao VALUES (18, 'Nmero srie carteira trabalho', 'N', 5);
INSERT INTO campo_consistenciacao VALUES (19, 'Data emisso carteira trabalho', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (20, 'UF emisso carteira trabalho', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (21, 'Nmero ttulo de eleitor', 'N', 13);
INSERT INTO campo_consistenciacao VALUES (22, 'Nmero zona ttulo eleitor', 'N', 4);
INSERT INTO campo_consistenciacao VALUES (23, 'Nmero seo ttulo eleitor', 'N', 4);
INSERT INTO campo_consistenciacao VALUES (24, 'Nome fantasia pessoa jurdica', 'N', 50);
INSERT INTO campo_consistenciacao VALUES (25, 'Nmero inscrio estadual', 'N', 10);
INSERT INTO campo_consistenciacao VALUES (26, 'Sexo pessoa fsica', 'S', 10);
INSERT INTO campo_consistenciacao VALUES (27, 'Nome me', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (28, 'Nome pai', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (29, 'Nome cnjuge', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (30, 'Nome responsvel', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (31, 'Nome ltima empresa', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (32, 'Ocupao profissional', 'S', 250);
INSERT INTO campo_consistenciacao VALUES (33, 'Escolaridade', 'S', 60);
INSERT INTO campo_consistenciacao VALUES (34, 'Estado civil', 'S', 15);
INSERT INTO campo_consistenciacao VALUES (35, 'Pas de origem(se estrangeiro)', 'S', 60);
INSERT INTO campo_consistenciacao VALUES (36, 'Data chegada ao Brasil', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (37, 'Data bito', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (38, 'Data unio com cnjuge', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (39, 'DDD telefone residencial', 'N', 3);
INSERT INTO campo_consistenciacao VALUES (40, 'Telefone residencial', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (41, 'DDD telefone comercial', 'N', 3);
INSERT INTO campo_consistenciacao VALUES (42, 'Telefone comercial', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (43, 'DDD telefone celular', 'N', 3);
INSERT INTO campo_consistenciacao VALUES (44, 'Telefone celular', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (45, 'DDD fax', 'N', 3);
INSERT INTO campo_consistenciacao VALUES (46, 'Fax', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (47, 'Logradouro (end. correspondncia)', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (48, 'Tipo logradouro (end. correspondncia)', 'S', 15);
INSERT INTO campo_consistenciacao VALUES (49, 'Nmero (end. correspondncia)', 'N', 6);
INSERT INTO campo_consistenciacao VALUES (50, 'Letra (end. correspondncia)', 'N', 1);
INSERT INTO campo_consistenciacao VALUES (51, 'Complemento (end. correspondncia)', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (52, 'Bairro (end. correspondncia)', 'N', 40);
INSERT INTO campo_consistenciacao VALUES (53, 'CEP (end. correspondncia)', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (54, 'Cidade (end. correspondncia)', 'N', 60);
INSERT INTO campo_consistenciacao VALUES (55, 'UF (end. correspondncia)', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (56, 'Logradouro (end. residencial)', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (57, 'Tipo logradouro (end. residencial)', 'S', 15);
INSERT INTO campo_consistenciacao VALUES (58, 'Nmero (end. residencial)', 'N', 6);
INSERT INTO campo_consistenciacao VALUES (59, 'Letra (end. residencial)', 'N', 1);
INSERT INTO campo_consistenciacao VALUES (60, 'Complemento (end. residencial)', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (61, 'Bairro (end. residencial)', 'N', 40);
INSERT INTO campo_consistenciacao VALUES (62, 'CEP (end. residencial)', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (63, 'Cidade (end. residencial)', 'N', 60);
INSERT INTO campo_consistenciacao VALUES (64, 'UF (end. residencial)', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (65, 'Logradouro (end. comercial)', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (66, 'Tipo logradouro (end. comercial)', 'S', 15);
INSERT INTO campo_consistenciacao VALUES (67, 'Nmero (end. comercial)', 'N', 6);
INSERT INTO campo_consistenciacao VALUES (68, 'Letra (end. comercial)', 'N', 1);
INSERT INTO campo_consistenciacao VALUES (69, 'Complemento (end. comercial)', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (70, 'Bairro (end. comercial)', 'N', 40);
INSERT INTO campo_consistenciacao VALUES (71, 'CEP (end. comercial)', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (72, 'Cidade (end. comercial)', 'N', 60);
INSERT INTO campo_consistenciacao VALUES (73, 'UF (end. comercial)', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (74, 'Logradouro (correspondncia ou residencial)', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (75, 'Tipo logradouro (correspondncia ou residencial)', 'S', 15);
INSERT INTO campo_consistenciacao VALUES (76, 'Nmero (correspondncia ou residencial)', 'N', 6);
INSERT INTO campo_consistenciacao VALUES (77, 'Letra (correspondncia ou residencial)', 'N', 1);
INSERT INTO campo_consistenciacao VALUES (78, 'Complemento (correspondncia ou residencial)', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (79, 'Bairro (correspondncia ou residencial)', 'N', 40);
INSERT INTO campo_consistenciacao VALUES (80, 'CEP (correspondncia ou residencial)', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (81, 'Cidade (correspondncia ou residencial)', 'N', 60);
INSERT INTO campo_consistenciacao VALUES (82, 'UF (correspondncia ou residencial)', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (83, 'Logradouro (correspondncia ou comercial)', 'N', 150);
INSERT INTO campo_consistenciacao VALUES (84, 'Tipo logradouro (correspondncia ou comercial)', 'S', 15);
INSERT INTO campo_consistenciacao VALUES (85, 'Nmero (correspondncia ou comercial)', 'N', 6);
INSERT INTO campo_consistenciacao VALUES (86, 'Letra (correspondncia ou comercial)', 'N', 1);
INSERT INTO campo_consistenciacao VALUES (87, 'Complemento (correspondncia ou comercial)', 'N', 20);
INSERT INTO campo_consistenciacao VALUES (88, 'Bairro (correspondncia ou comercial)', 'N', 40);
INSERT INTO campo_consistenciacao VALUES (89, 'CEP (correspondncia ou comercial)', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (90, 'Cidade (correspondncia ou comercial)', 'N', 60);
INSERT INTO campo_consistenciacao VALUES (91, 'UF (correspondncia ou comercial)', 'S', 30);
INSERT INTO campo_consistenciacao VALUES (92, 'DDD telefone comercial ou residencial', 'N', 3);
INSERT INTO campo_consistenciacao VALUES (93, 'Telefone comercial ou residencial', 'N', 8);
INSERT INTO campo_consistenciacao VALUES (94, 'CPF', 'N', 14);
INSERT INTO campo_consistenciacao VALUES (95, 'CNPJ', 'N', 14);


--
-- Data for Name: campo_metadado; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: confrontacao; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: fonte; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: historico_campo; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--

INSERT INTO historico_campo VALUES (1, 2, 2, 'NOW()');
INSERT INTO historico_campo VALUES (1, 3, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 4, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 5, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 38, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 37, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 36, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 27, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 28, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 29, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 30, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 31, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 26, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 32, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 33, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 34, 5, 'NOW()');
INSERT INTO historico_campo VALUES (1, 35, 5, 'NOW()');


--
-- Data for Name: incoerencia; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: incoerencia_documento; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: incoerencia_endereco; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: incoerencia_fone; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: incoerencia_pessoa_possivel; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: incoerencia_tipo_incoerencia; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: metadado; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: ocorrencia_regra_campo; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: regra_campo; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: temp_cadastro_unificacao_cmf; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: temp_cadastro_unificacao_siam; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



--
-- Data for Name: tipo_incoerencia; Type: TABLE DATA; Schema: consistenciacao; Owner: -
--



SET search_path = historico, pg_catalog;

--
-- Data for Name: bairro; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: cep_logradouro; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: cep_logradouro_bairro; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: documento; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: endereco_externo; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: endereco_pessoa; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: fisica; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: fisica_cpf; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: fone_pessoa; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: funcionario; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: juridica; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: logradouro; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: municipio; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: pessoa; Type: TABLE DATA; Schema: historico; Owner: -
--



--
-- Data for Name: socio; Type: TABLE DATA; Schema: historico; Owner: -
--



SET search_path = modules, pg_catalog;

--
-- Data for Name: area_conhecimento; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: calendario_turma; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: componente_curricular; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: componente_curricular_ano_escolar; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: componente_curricular_turma; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: docente_licenciatura; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: educacenso_cod_aluno; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: educacenso_cod_docente; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: educacenso_curso_superior; Type: TABLE DATA; Schema: modules; Owner: -
--

INSERT INTO educacenso_curso_superior VALUES (1, '142C01', 'Cincias da educao', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (2, '143F01', 'Formao de professor de creche', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (3, '143F02', 'Formao de professor de educao infantil', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (4, '143F03', 'Formao de professor de pr-escola', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (5, '144F01', 'Formao de professor das sries finais do ensino fundamental', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (6, '144F02', 'Formao de professor das sries iniciais do ensino fundamental', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (7, '144F03', 'Formao de professor de alfabetizao', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (8, '144F04', 'Formao de professor de educao especial', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (9, '144F05', 'Formao de professor de educao infantil e sries iniciais do ensino fundamental', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (10, '144F06', 'Formao de professor do ensino fundamental', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (11, '144F07', 'Formao de professor do ensino mdio', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (12, '144F08', 'Formao de professor de jovens e adultos', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (13, '144F11', 'Formao de professor para a educao bsica', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (14, '144N01', 'Normal superior', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (15, '145F01', 'Formao de professor de biologia', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (16, '145F02', 'Formao de professor de cincias', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (17, '145F05', 'Formao de professor de educao religiosa', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (18, '145F07', 'Formao de professor de estudos sociais', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (19, '145F08', 'Formao de professor de filosofia', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (20, '145F09', 'Formao de professor de fsica', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (21, '145F10', 'Formao de professor de geografia', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (22, '145F11', 'Formao de professor de histria', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (23, '145F13', 'Formao de professor de lngua estrangeira', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (24, '145F15', 'Formao de professor de portugus', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (25, '145F16', 'Formao de professor de portugus e lngua estrangeira', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (26, '145F18', 'Formao de professor de matemtica', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (27, '145F21', 'Formao de professor de qumica', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (28, '145F24', 'Formao de professor de sociologia', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (29, '145F25', 'Formao de professor em cincias sociais', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (30, '146A01', 'Formao de professor de disciplinas profissionais - AMBIENTE, SADE E SEGURANA', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (31, '146E01', 'Formao de professor de disciplinas profissionais - APOIO EDUCACIONAL', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (32, '146P01', 'Formao de professor de disciplinas profissionais - CONTROLE E PROCESSOS INDUSTRIAIS', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (33, '146G01', 'Formao de professor de disciplinas profissionais - GESTO E NEGCIOS', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (34, '146C01', 'Formao de professor de disciplinas profissionais - INFORMAO E COMUNICAO', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (35, '146I01', 'Formao de professor de disciplinas profissionais - INFRA-ESTRUTURA', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (36, '146D01', 'Formao de professor de disciplinas profissionais - PRODUO CULTURAL E DESIGN', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (37, '146N01', 'Formao de professor de disciplinas profissionais - RECURSOS NATURAIS', 1, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (38, '210A01', 'Artes', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (39, '212999', 'Msica e artes cnicas', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (40, '213999', 'Tcnicas audiovisuais e produo de mdia', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (41, '214999', 'Design e estilismo', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (42, '215999', 'Artesanato', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (43, '220H01', 'Humanidades', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (44, '220L01', 'Letras', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (45, '220P01', 'Letras portugus', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (46, '220E01', 'Letras lnguas estrangeiras', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (47, '220L03', 'Letras portugus e lnguas estrangeiras', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (48, '221999', 'Religio e teologia', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (49, '225A01', 'Arqueologia', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (50, '225H01', 'Histria', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (51, '225M01', 'Museologia', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (52, '226999', 'Filosofia e tica', 2, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (53, '312999', 'Sociologia e estudos culturais', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (54, '311P02', 'Psicologia', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (55, '313999', 'Cincia poltica e educao cvica', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (56, '314E02', 'Economia', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (57, '321999', 'Jornalismo e reportagem', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (58, '322999', 'Biblioteconomia, informao, arquivos', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (59, '341999', 'Vendas em atacado e varejo', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (60, '342999', 'Marketing e publicidade', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (61, '343F01', 'Finanas, bancos, seguros', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (62, '344999', 'Contabilidade e tributao', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (63, '345999', 'Gerenciamento e administrao', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (64, '346999', 'Secretariado e trabalhos de escritrio', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (65, '347V01', 'Vida profissional', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (66, '380D01', 'Direito', 3, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (67, '420C01', 'Cincias', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (68, '421B01', 'Biofsica', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (69, '421B02', 'Biologia', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (70, '421B07', 'Biomedicina', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (71, '421B09', 'Bioqumica', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (72, '441F01', 'Fsica', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (73, '442Q01', 'Qumica', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (74, '443999', 'Cincias da terra', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (75, '443G04', 'Geografia', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (76, '443M01', 'Meteorologia', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (77, '443P01', 'Paleontologia', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (78, '461M01', 'Matemtica', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (79, '462E01', 'Estatstica', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (80, '481C01', 'Cincia da computao', 4, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (81, '520999', 'Engenharia e profisses de engenharia', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (82, '520D01', 'Desenho industrial', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (83, '521999', 'Engenharia mecnica e metalurgia (trabalhos com metais)', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (84, '522999', 'Eletricidade e energia', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (85, '523999', 'Eletrnica e automao', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (86, '523E04', 'Engenharia de computao', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (87, '523E04', 'Engenharia de redes de comunicao', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (88, '524999', 'Qumica e engenharia de processos', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (89, '524E02', 'Engenharia de biotecnologia', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (90, '524E06', 'Engenharia nuclear', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (91, '525999', 'Veculos a motor, construo naval e aeronutica', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (92, '540F01', 'Fabricao e processamento', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (93, '541P04', 'Processamento de alimentos e bebidas', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (94, '542999', 'Txteis, roupas, calados, couros', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (95, '543999', 'Materiais (madeira, papel, plstico, vidro)', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (96, '544M02', 'Minerao e extrao', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (97, '581A05', 'Arquitetura e urbanismo', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (98, '582999', 'Engenharia civil e de construo', 5, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (99, '621999', 'Produo agrcola e pecuria', 6, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (100, '623E01', 'Engenharia florestal - silvicultura', 6, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (101, '624999', 'Recursos pesqueiros', 6, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (102, '641999', 'Veterinria', 6, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (103, '720S01', 'Sade', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (104, '813E01', 'Educao fsica', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (105, '721M01', 'Medicina', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (106, '723999', 'Enfermagem e ateno primria (assistncia bsica)', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (107, '724O01', 'Odontologia', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (108, '726999', 'Terapia e reabilitao', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (109, '726F01', 'Fisioterapia', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (110, '726F03', 'Fonoaudiologia', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (111, '726N02', 'Nutrio', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (112, '727F01', 'Farmcia', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (113, '762999', 'Servio social e orientao', 7, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (114, '811999', 'Hotelaria, restaurantes  e servios de alimentao', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (115, '812999', 'Viagens, turismo e lazer', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (116, '813E01', 'Esportes', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (117, '814C01', 'Cincias domsticas', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (118, '815999', 'Servios de beleza', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (119, '840999', 'Transportes e servios', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (120, '850999', 'Proteo ambiental e saneamento', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (121, '861999', 'Proteo de pessoas e de propriedades', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (122, '862999', 'Sade e segurana do trabalho', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (123, '863999', 'Setor militar e de defesa', 8, 1, 'NOW()', NULL);
INSERT INTO educacenso_curso_superior VALUES (124, '999999', 'Outro curso de formao superior', 9, 1, 'NOW()', NULL);


--
-- Data for Name: educacenso_ies; Type: TABLE DATA; Schema: modules; Owner: -
--

INSERT INTO educacenso_ies VALUES (1, 1, 'UNIVERSIDADE FEDERAL DE MATO GROSSO', 1, 1, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2, 2, 'UNIVERSIDADE DE BRASLIA', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3, 3, 'UNIVERSIDADE FEDERAL DE SERGIPE', 1, 1, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4, 4, 'UNIVERSIDADE FEDERAL DO AMAZONAS', 1, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5, 5, 'UNIVERSIDADE FEDERAL DO PIAU', 1, 1, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (6, 6, 'UNIVERSIDADE FEDERAL DE OURO PRETO', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (7, 7, 'UNIVERSIDADE FEDERAL DE SO CARLOS', 1, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (8, 8, 'FUNDAO UNIVERSIDADE FEDERAL DE VIOSA', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (9, 9, 'UNIVERSIDADE ESTADUAL DE LONDRINA', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (10, 10, 'PONTIFCIA UNIVERSIDADE CATLICA DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (11, 11, 'UNIVERSIDADE CATLICA DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (12, 12, 'FUNDAO UNIVERSIDADE FEDERAL DO RIO GRANDE', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (13, 13, 'UNIVERSIDADE DE CAXIAS DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (14, 14, 'UNIVERSIDADE DO VALE DO RIO DOS SINOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (15, 15, 'UNIVERSIDADE CATLICA DE PETRPOLIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (16, 16, 'UNIVERSIDADE GAMA FILHO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (17, 17, 'UNIVERSIDADE FEDERAL DE UBERLNDIA', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (18, 18, 'UNIVERSIDADE CATLICA DE PELOTAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (19, 19, 'PONTIFCIA UNIVERSIDADE CATLICA DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (20, 20, 'UNIVERSIDADE DE PASSO FUNDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (21, 21, 'PONTIFCIA UNIVERSIDADE CATLICA DO RIO GRANDE DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (22, 22, 'UNIVERSIDADE PRESBITERIANA MACKENZIE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (23, 23, 'CENTRO UNIVERSITRIO FEEVALE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (24, 24, 'UNIVERSIDADE ESTADUAL DE SANTA CRUZ', 2, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (25, 26, 'ESCOLA NACIONAL DE CINCIAS ESTATSTICAS', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (26, 27, 'UNIVERSIDADE VALE DO RIO VERDE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (27, 29, 'UNIVERSIDADE ESTADUAL DO CEAR', 2, 1, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (28, 30, 'UNIVERSIDADE JOS DO ROSRIO VELLANO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (29, 31, 'CENTRO DE EDUCAO TCNICA DA UTRAMIG', 2, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (30, 32, 'UNIVERSIDADE ESTADUAL DE CINCIAS DA SADE DE ALAGOAS - UNCISAL', 2, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (31, 33, 'FACULDADE DE TECNOLOGIA DE SOROCABA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (32, 34, 'FACULDADE DE TECNOLOGIA DE SO PAULO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (33, 35, 'FACULDADE DE TECNOLOGIA DE AMERICANA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (34, 36, 'FACULDADE DE TECNOLOGIA DA BAIXADA SANTISTA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (35, 37, 'FACULDADE DE TECNOLOGIA DE JAHU', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (36, 38, 'UNIVERSIDADE DO ESTADO DO PAR', 2, 1, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (37, 39, 'INSTITUTO DE TECNOLOGIA DA AMAZNIA', 2, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (38, 40, 'UNIVERSIDADE DO ESTADO DA BAHIA', 2, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (39, 41, 'FACULDADE ESTADUAL DE EDUCAO FSICA DE JACAREZINHO', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (40, 42, 'FACULDADE ESTADUAL DE DIREITO DO NORTE PIONEIRO', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (41, 43, 'FUNDAO UNIVERSIDADE DO ESTADO DE SANTA CATARINA', 2, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (42, 44, 'FACULDADE DE ENGENHARIA CIVIL DE ITAJUB', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (43, 45, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE ITAJUB', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (44, 46, 'FACULDADE ESTADUAL DE CINCIAS ECONMICAS DE APUCARANA', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (45, 47, 'UNIVERSIDADE ESTADUAL DE GOIS', 2, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (46, 54, 'UNIVERSIDADE ESTADUAL DE CAMPINAS', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (47, 55, 'UNIVERSIDADE DE SO PAULO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (48, 56, 'UNIVERSIDADE ESTADUAL PAULISTA JLIO DE MESQUITA FILHO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (49, 57, 'UNIVERSIDADE ESTADUAL DE MARING', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (50, 58, 'FACULDADE DE DIREITO DE SO BERNARDO DO CAMPO', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (51, 59, 'FACULDADE DE DIREITO DE FRANCA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (52, 60, 'CENTRO UNIVERSITRIO DE FRANCA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (53, 61, 'ESCOLA SUPERIOR DE EDUCAO FSICA DE AVAR', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (54, 62, 'FACULDADE DE CINCIAS E LETRAS DE AVAR', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (55, 63, 'FACULDADE DE CINCIAS DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (56, 64, 'FACULDADE DE ENGENHARIA DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (57, 65, 'FACULDADE DE ODONTOLOGIA DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (58, 66, 'FACULDADE DE DIREITO E ADMINISTRAO DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (59, 67, 'ESCOLA DE ENGENHARIA DE PIRACICABA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (60, 68, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE PENPOLIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (61, 71, 'UNIVERSIDADE DO ESTADO DO RIO GRANDE DO NORTE', 2, 1, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (62, 72, 'FACULDADE DE FORMAO DE PROFESSORES DA MATA SUL', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (63, 73, 'CENTRO DE ENSINO SUPERIOR DE ARCOVERDE', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (64, 74, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE SO JOS DO RIO PARDO', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (65, 75, 'FACULDADE ESTADUAL DE EDUCAO CINCIAS E LETRAS DE PARANAVA', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (66, 76, 'UNIVERSIDADE REGIONAL DE BLUMENAU', 3, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (67, 77, 'FACULDADE DA REGIO DOS LAGOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (68, 78, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE ALEGRE', 3, 1, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (69, 79, 'INSTITUTO MUNICIPAL DE ENSINO SUPERIOR DE CATANDUVA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (70, 80, 'CENTRO UNIVERSITRIO PARA O DESENVOLVIMENTO DO ALTO VALE DO ITAJA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (71, 81, 'UNIVERSIDADE DA REGIO DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (72, 82, 'UNIVERSIDADE DO OESTE DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (73, 83, 'UNIVERSIDADE DO VALE DO ITAJA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (74, 84, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE MACA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (75, 85, 'FACULDADE DE FILOSOFIA DE ITAPERUNA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (76, 86, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS PADRE HUMBERTO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (77, 87, 'CENTRO UNIVERSITRIO DE BRUSQUE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (78, 88, 'FACULDADE DE FORMAO DE PROFESSORES DE PENEDO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (79, 89, 'FUNDAO FACULDADES LUIZ MENEGHEL', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (80, 90, 'ESCOLA SUPERIOR DE EDUCAO CINCIAS E LETRAS DE RIO VERDE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (81, 91, 'ESCOLA SUPERIOR DE CINCIAS AGRRIAS DE RIO VERDE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (82, 92, 'ESCOLA SUPERIOR CINCIAS HUMANAS E EXATAS DE RIO VERDE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (83, 93, 'FACULDADE DE MEDICINA DE JUNDIA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (84, 94, 'ESCOLA SUPERIOR DE EDUCAO FSICA DE JUNDIA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (85, 95, 'UNIVERSIDADE ESTADUAL DO VALE DO ACARA', 2, 1, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (86, 96, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS PROFESSOR JOS AUGUSTO VIEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (87, 97, 'ESCOLA SUPERIOR DE AGRICULTURA E CINCIAS DE MACHADO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (88, 98, 'FACULDADE DE ADMINISTRAO E FINANAS DE MACHADO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (89, 99, 'FACULDADE DE DIREITO DE CONSELHEIRO LAFAIETE', 3, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (90, 100, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE ARAX', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (91, 101, 'FACULDADE DE CINCIAS GERENCIAIS DO ALTO PARANABA', 3, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (92, 102, 'FACULDADE DE DIREITO DO ALTO PARANABA', 3, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (93, 103, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE ARAGUARI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (94, 104, 'FACULDADE DE CINCIAS ECONMICAS DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (95, 105, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (96, 106, 'FACULDADE DE EDUCAO DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (97, 107, 'UNIVERSIDADE FEDERAL DE SO JOO DEL REI', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (98, 108, 'ESCOLA SUPERIOR DE EDUCAO FSICA E TCNICAS DESPORTIVAS DE ANDRADINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (99, 109, 'FACULDADES INTEGRADAS RUI BARBOSA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (100, 110, 'FACULDADE DE DIREITO DE SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (101, 111, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (102, 122, 'FACULDADE DE ENGENHARIA DE AGRIMENSURA DE ARARAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (103, 123, 'FACULDADE DE ENGENHARIA CIVIL DE ARARAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (104, 124, 'CENTRO UNIVERSITRIO DE ARARAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (105, 125, 'CENTRO UNIVERSITRIO DE ARARAS - "DR. EDMUNDO ULSON" - UNAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (106, 126, 'INSTITUTO NACIONAL DE TELECOMUNICAES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (107, 127, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE ATIBAIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (108, 128, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (109, 129, 'FACULDADE DE DIREITO DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (110, 130, 'FACULDADE DE ADMINISTRAO DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (111, 131, 'FACULDADE DE EDUCAO FSICA DE BARRA BONITA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (112, 132, 'FACULDADE DE EDUCAO ANTNIO AUGUSTO REIS NEVES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (113, 133, 'FACULDADE DE CINCIAS CONTBEIS DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (114, 134, 'FACULDADE DE TECNOLOGIA E PROCESSAMENTO DE DADOS - BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (115, 135, 'CENTRO UNIVERSITRIO CLARETIANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (116, 136, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE BEBEDOURO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (117, 137, 'UNIVERSIDADE DO SAGRADO CORAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (118, 138, 'CENTRO UNIVERSITRIO BARO DE MAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (119, 139, 'FACULDADE DE CINCIAS ECONMICAS DO TRINGULO MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (120, 140, 'UNIVERSIDADE SEVERINO SOMBRA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (121, 141, 'FACULDADE DE DIREITO DE VARGINHA - FADIVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (122, 142, 'CENTRO UNIVERSITRIO DO TRINGULO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (123, 143, 'UNIVERSIDADE DE UBERABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (124, 144, 'FACULDADE DE CINCIAS HUMANAS DE OLINDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (125, 145, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (126, 146, 'CENTRO UNIVERSITRIO DE RIO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (127, 149, 'FACULDADE DE MEDICINA DE SO JOS DO RIO PRETO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (128, 150, 'UNIVERSIDADE DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (129, 151, 'ESCOLA DE ADMINISTRAO DE EMPRESAS DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (130, 152, 'FACULDADE DE CINCIAS CONTBEIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (131, 158, 'CONSERVATRIO BRASILEIRO DE MSICA - CENTRO UNIVERSITRIO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (132, 159, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE CARUARU', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (133, 160, 'FACULDADE FRASSINETTI DO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (134, 161, 'CENTRO UNIVERSITRIO ASSUNO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (135, 162, 'CENTRO UNIVERSITRIO BELAS ARTES DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (136, 163, 'UNIVERSIDADE ESTCIO DE S', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (137, 165, 'UNIVERSIDADE VEIGA DE ALMEIDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (138, 166, 'FACULDADE DE ADMINISTRAO E INFORMTICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (139, 167, 'UNIVERSIDADE METODISTA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (140, 170, 'FACULDADES DA FUNDAO DE ENSINO DE MOCOCA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (141, 171, 'FACULDADE DE DIREITO DO SUL DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (142, 172, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS EUGNIO PACELLI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (143, 173, 'FACULDADE DE CINCIAS MDICAS DR. JOS ANTONIO GARCIA COUTINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (144, 174, 'INSTITUTO SUPERIOR DE ESTUDOS SOCIAIS CLVIS BEVILACQUA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (145, 176, 'UNIVERSIDADE CASTELO BRANCO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (146, 177, 'FACULDADE DE CINCIAS ECONMICAS E DE ADMINISTRAO DE EMPRESAS DE SO JOS DO RIO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (147, 178, 'FACULDADE DE ENGENHARIA DE SO JOS DO RIO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (148, 179, 'FACULDADE DE ARQUITETURA E URBANISMO DE SO JOS DO RIO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (149, 180, 'FACULDADE DE DIREITO DE CARUARU', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (150, 181, 'FACULDADE DE ODONTOLOGIA DE CARUARU', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (151, 190, 'ESCOLA DE MEDICINA SOUZA MARQUES DA FUNDAO TCNICO-EDUCACIONAL SOUZA MARQUES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (152, 191, 'FACULDADE DE CINCIAS CONTBEIS E DE ADMINISTRAO DE EMPRESAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (153, 192, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS SOUZA MARQUES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (154, 193, 'FACULDADE DE ENGENHARIA SOUZA MARQUES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (155, 194, 'ESCOLA DE ENFERMAGEM DA FUNDAO TCNICO EDUCACIONAL SOUZA MARQUES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (156, 195, 'FACULDADE MORAES JNIOR - MACKENZIE RIO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (157, 197, 'INSTITUTO DE CINCIAS SOCIAIS DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (158, 198, 'CENTRO UNIVERSITRIO DA CIDADE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (159, 199, 'FACULDADE SO LUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (160, 200, 'FACULDADE CSPER LBERO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (161, 201, 'FACULDADE BRASILEIRA DE CINCIAS JURDICAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (162, 202, 'FACULDADE DE ECONOMIA E FINANAS DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (163, 203, 'UNIVERSIDADE SO JUDAS TADEU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (164, 206, 'CENTRO UNIVERSITRIO TALO-BRASILEIRO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (165, 207, 'CENTRO UNIVERSITRIO MOURA LACERDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (166, 208, 'UNIVERSIDADE DE RIBEIRO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (167, 210, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (168, 211, 'FACULDADE DE CINCIAS ECONMICAS E ADMINISTRATIVAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (169, 212, 'FACULDADE DE FILOSOFIA DE CAMPO GRANDE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (170, 213, 'CENTRO UNIVERSITRIO LVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (171, 214, 'FACULDADE DE ENGENHARIA DE MINAS GERAIS - FEAMIG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (172, 215, 'UNIVERSIDADE DO GRANDE ABC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (173, 216, 'CENTRO UNIVERSITRIO METODISTA IZABELA HENDRIX', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (174, 217, 'CENTRO UNIVERSITRIO DAS FACULDADES ASSOCIADAS DE ENSINO - FAE', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (175, 218, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE DUQUE DE CAXIAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (176, 219, 'FACULDADE DE EDUCAO CINCIAS E ARTES DOM BOSCO DE MONTE APRAZVEL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (177, 220, 'FACULDADE SO JUDAS TADEU', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (178, 221, 'UNIVERSIDADE CRUZEIRO DO SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (179, 222, 'CENTRO UNIVERSITRIO DE VOTUPORANGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (180, 223, 'FACULDADE DE EDUCAO FSICA DE SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (181, 224, 'FACULDADE DE MEDICINA DO ABC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (182, 225, 'INSTITUTO DE ENSINO SUPERIOR SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (183, 226, 'CENTRO UNIVERSITRIO LUSADA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (184, 227, 'UNIVERSIDADE CATLICA DE SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (185, 228, 'FACULDADE DE ENGENHARIA INDUSTRIAL DE SO BERNARDO DO CAMPO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (186, 229, 'ESCOLA SUPERIOR DE ADMINISTRAO DE NEGCIOS DE SO BERNARDO DO CAMPO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (187, 230, 'ESCOLA SUPERIOR DE ADMINISTRAO DE NEGCIOS DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (188, 231, 'FACULDADE DE SO BERNARDO DO CAMPO - FASB', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (189, 234, 'FACULDADES OSWALDO CRUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (190, 236, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE SO JOO DA BOA VISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (191, 237, 'FACULDADE DE DIREITO DE SO JOO DA BOA VISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (192, 238, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE SO JOO DA BOA VISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (193, 239, 'FACULDADE DE MEDICINA VETERINRIA OCTVIO BASTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (194, 240, 'UNIVERSIDADE SANTA RSULA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (195, 241, 'FACULDADE DE ENGENHARIA QUMICA DE LORENA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (196, 242, 'CENTRO UNIVERSITRIO DE SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (197, 243, 'FACULDADE DO CLUBE NUTICO MOGIANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (198, 244, 'CENTRO UNIVERSITRIO FIEO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (199, 245, 'FACULDADE DE ADMINISTRAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (200, 254, 'CENTRO UNIVERSITRIO CAPITAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (201, 255, 'FACULDADE DE ENGENHARIA E AGRIMENSURA DE PIRASSUNUNGA - FEAP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (202, 256, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE PIRAJU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (203, 257, 'FACULDADE DE ADMINISTRAO E CINCIAS CONTBEIS LUZWELL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (204, 263, 'FACULDADES INTEGRADAS CAMPOS SALLES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (205, 264, 'UNIVERSIDADE SO MARCOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (206, 265, 'FACULDADES INTEGRADAS DE OURINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (207, 266, 'UNIVERSIDADE METODISTA DE PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (208, 267, 'FACULDADES INTEGRADAS ESPRITO SANTENSES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (209, 268, 'FACULDADE DE DIREITO DA ALTA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (210, 269, 'FACULDADE FACCAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (211, 270, 'FACULDADES ESEFAP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (212, 271, 'UNIVERSIDADE DO OESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (213, 273, 'ESCOLA DE ENGENHARIA INDUSTRIAL DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (214, 274, 'FACULDADE DE CINCIAS APLICADAS DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (215, 275, 'UNIVERSIDADE DO VALE DO PARABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (216, 276, 'FACULDADE DE DIREITO DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (217, 277, 'CENTRO UNIVERSITRIO AUGUSTO MOTTA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (218, 278, 'FACULDADES INTEGRADAS SIMONSEN', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (219, 279, 'FACULDADES INTEGRADAS HLIO ALONSO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (220, 280, 'FACULDADE DE DIREITO DO OESTE DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (221, 281, 'CONSERVATRIO DRAMTICO MUSICAL SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (222, 282, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E CONTBEIS COSTA BRAGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (223, 283, 'FACULDADE DE EDUCAO COSTA BRAGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (224, 284, 'FACULDADE DE MEDICINA DE ITAJUB', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (225, 288, 'FACULDADE DE DIREITO DO VALE DO RIO DOCE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (226, 289, 'FACULDADE DE EDUCAO DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (227, 290, 'FACULDADE DE DIREITO DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (228, 291, 'FACULDADE DE PSICOLOGIA DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (229, 292, 'FACULDADE DE CINCIAS DA SADE DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (230, 293, 'FACULDADE DE CINCIAS E TECNOLOGIA DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (231, 294, 'FACULDADE TRS DE MAIO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (232, 295, 'UNIVERSIDADE DE SANTA CRUZ DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (233, 296, 'UNIVERSIDADE DA REGIO DA CAMPANHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (234, 297, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS SO JUDAS TADEU', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (235, 298, 'UNIVERSIDADE NORTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (236, 299, 'FACULDADE DE JANDAIA DO SUL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (237, 300, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE FORMIGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (238, 301, 'ESCOLA DE BIBLIOTECONOMIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (239, 302, 'FACULDADE DE CINCIAS ECONMICAS, ADMINISTRATIVAS E CONTBEIS DE DIVINPOLIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (240, 303, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (241, 304, 'FACULDADE DE CINCIAS CONTBEIS DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (242, 307, 'FACULDADE DE MEDICINA DE BARBACENA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (243, 308, 'UNIVERSIDADE PRESIDENTE ANTNIO CARLOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (244, 316, 'UNIVERSIDADE NOVE DE JULHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (245, 317, 'FACULDADES INTEGRADAS ALCNTARA MACHADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (246, 318, 'FACULDADE DE ARTES ALCNTARA MACHADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (247, 319, 'UNIVERSIDADE CAMILO CASTELO BRANCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (248, 320, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE TATU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (249, 321, 'FACULDADE DE DESENHO DE TATU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (250, 322, 'UNIVERSIDADE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (251, 328, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS VIANNA JNIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (252, 329, 'ESCOLA SUPERIOR DE AGRONOMIA DE PARAGUA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (253, 330, 'UNIVERSIDADE IGUAU', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (254, 332, 'ESCOLA DE ENGENHARIA DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (255, 333, 'FACULDADE DE INFORMTICA DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (256, 334, 'FACULDADE DE SERVIO SOCIAL DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (257, 335, 'FACULDADE DE EDUCAO E CINCIAS PINHEIRENSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (258, 336, 'FACULDADE MACHADO SOBRINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (259, 337, 'CENTRO DE ENSINO SUPERIOR DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (260, 338, 'PONTIFCIA UNIVERSIDADE CATLICA DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (261, 339, 'FACULDADE SANTA EMLIA DE RODAT', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (262, 343, 'CENTRO UNIVERSITRIO NEWTON PAIVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (263, 344, 'CENTRO UNIVERSITRIO UNA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (264, 345, 'ESCOLA DE ENGENHARIA KENNEDY', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (265, 346, 'INSTITUTO SUPERIOR DE CINCIAS APLICADAS - ISCA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (266, 349, 'CENTRO UNIVERSITRIO DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (267, 351, 'FACULDADE DE CINCIAS MDICAS DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (268, 352, 'CENTRO UNIVERSITRIO DE JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (269, 353, 'FACULDADE EVANGLICA DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (270, 355, 'UNIVERSIDADE TUIUTI DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (271, 359, 'ESCOLA SUPERIOR DE EDUCAO FSICA DE MUZAMBINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (272, 360, 'CENTRO UNIVERSITRIO PAULISTANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (273, 361, 'FACULDADE PAULISTA DE SERVIO SOCIAL DE SO CAETANO DO SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (274, 362, 'FACULDADE PAULISTA DE SERVIO SOCIAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (275, 363, 'FACULDADE MOZARTEUM DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (276, 364, 'FACULDADE DE CINCIAS CONTBEIS DE PONTE NOVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (277, 367, 'UNIVERSIDADE ESTADUAL DE MONTES CLAROS', 2, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (278, 368, 'FACULDADE DE EDUCAO FSICA DA ASSOCIAO CRIST DE MOOS DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (279, 369, 'FACULDADE AUXILIUM DE FILOSOFIA CINCIAS E LETRAS DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (280, 370, 'FACULDADE PAULISTA DE ARTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (281, 371, 'FACULDADE DE MSICA CARLOS GOMES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (282, 372, 'FACULDADE DE BIBLIOTECONOMIA E CINCIA DA INFORMAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (283, 373, 'ESCOLA DE SOCIOLOGIA E POLTICA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (284, 374, 'CENTRO UNIVERSITRIO DAS FACULDADES METROPOLITANAS UNIDAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (285, 375, 'UNIVERSIDADE DE SANTO AMARO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (286, 376, 'CENTRO UNIVERSITRIO IBERO-AMERICANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (287, 377, 'FACULDADE DE EDUCAO PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (288, 378, 'FACULDADE DE DIREITO PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (289, 379, 'FACULDADE DE CINCIAS E LETRAS PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (290, 380, 'FACULDADE DE CINCIAS ECONMICAS, CONTBEIS E DE ADM DE EMPRESAS PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (291, 381, 'FACULDADE DE PSICOLOGIA PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (292, 382, 'FACULDADE DE TECNOLOGIA PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (293, 383, 'UNIVERSIDADE DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (294, 384, 'CENTRO UNIVERSITRIO DE ANPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (295, 385, 'UNIVERSIDADE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (296, 386, 'CENTRO UNIVERSITRIO DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (297, 387, 'UNIVERSIDADE CATLICA DOM BOSCO', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (298, 388, 'FACULDADE DE CINCIAS CONTBEIS E ATUARIAIS DA ALTA NOROESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (299, 389, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E CONTBEIS DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (300, 390, 'FACULDADE DE EDUCAO FSICA DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (301, 391, 'FACULDADE DE TECNOLOGIA DA ALTA NOROESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (302, 395, 'FACULDADE DE CINCIAS CONTBEIS DE LUCLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (303, 396, 'FACULDADE RUY BARBOSA DE ADMINISTRAO E DE DIREITO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (304, 397, 'FACULDADE RUY BARBOSA DE TECNOLOGIA EM PROCESSAMENTO DE DADOS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (305, 398, 'UNIVERSIDADE TIRADENTES', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (306, 399, 'ESCOLA DE ENGENHARIA DE AGRIMENSURA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (307, 400, 'FACULDADE DE DIREITO DE OLINDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (308, 401, 'CENTRO DE ESTUDOS SUPERIORES BARROS MELO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (309, 402, 'CENTRO UNIVERSITRIO DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (310, 403, 'UNIVERSIDADE CATLICA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (311, 404, 'FACULDADES INTEGRADAS DA UPIS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (312, 405, 'FOCCA - FACULDADE DE OLINDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (313, 408, 'ESCOLA SUPERIOR DE RELAES PBLICAS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (314, 409, 'UNIVERSIDADE DE PERNAMBUCO', 2, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (315, 410, 'FACULDADE DE CINCIAS HUMANAS ESUDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (316, 413, 'FACULDADE PORTO-ALEGRENSE DE EDUCAO, CINCIAS E LETRAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (317, 414, 'FACULDADE PORTO-ALEGRENSE DE CINCIAS CONTBEIS E ADMINISTRATIVAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (318, 415, 'FACULDADE DE CINCIAS MDICAS DA SANTA CASA SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (319, 416, 'FACULDADES INTEGRADAS TERESA MARTIN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (320, 417, 'UNIVERSIDADE CIDADE DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (321, 420, 'UNIVERSIDADE DE MARLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (322, 421, 'FACULDADE ESTADUAL DE CINCIAS E LETRAS DE CAMPO MOURO', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (323, 423, 'UNIVERSIDADE REGIONAL INTEGRADA DO ALTO URUGUAI E DAS MISSES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (324, 424, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE SANTA ROSA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (325, 425, 'FACULDADE DE EDUCAO ARTSTICA DE SANTA ROSA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (326, 426, 'CENTRO UNIVERSITRIO FRANCISCANO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (327, 428, 'FACULDADE DE CINCIAS CONTBEIS E DE ADMINISTRAO DE MARLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (328, 429, 'FACULDADE DE DIREITO DE MARLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (329, 430, 'CENTRO UNIVERSITRIO FILADLFIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (330, 431, 'FACULDADE DE MEDICINA DE MARLIA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (331, 432, 'FACULDADE PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (332, 433, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE PALMAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (333, 434, 'FACULDADE SANTA MARCELINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (334, 435, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS SANTA MARCELINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (335, 436, 'FACULDADE DE CINCIAS DA SADE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (336, 437, 'UNIVERSIDADE PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (337, 438, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE ITUVERAVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (338, 439, 'FACULDADE DOUTOR FRANCISCO MAEDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (339, 440, 'FACULDADE DE DIREITO DE IT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (340, 441, 'UNIVERSIDADE DO CONTESTADO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (341, 442, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE ITARAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (342, 443, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE ITARAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (343, 446, 'UNIVERSIDADE DE CRUZ ALTA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (344, 447, 'FACULDADE CATLICA DE CINCIAS ECONMICAS DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (345, 448, 'CENTRO UNIVERSITRIO RITTER DOS REIS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (346, 449, 'UNIVERSIDADE LUTERANA DO BRASIL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (347, 450, 'CENTRO UNIVERSITRIO LUTERANO DE JI-PARAN', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (348, 451, 'CENTRO UNIVERSITRIO LUTERANO DE SANTARM', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (349, 452, 'CENTRO UNIVERSITRIO LUTERANO DE MANAUS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (350, 453, 'CENTRO UNIVERSITRIO LUTERANO DE PALMAS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (351, 454, 'FACULDADE CAMAQENSE DE CINCIAS CONTBEIS E ADMINISTRATIVAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (352, 455, 'FACULDADE DE FORMAO DE PROFESSORES E ESPECIALISTAS DE EDUCAO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (353, 456, 'CENTRO UNIVERSITRIO SANTANNA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (354, 457, 'UNIVERSIDADE BANDEIRANTE DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (355, 458, 'UNIVERSIDADE IBIRAPUERA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (356, 459, 'FACULDADE RENASCENA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (357, 460, 'FACULDADE DE ARTES PLSTICAS DA FUNDAO ARMANDO ALVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (358, 461, 'FACULDADE DE ADMINISTRAO DA FUNDAO ARMANDO ALVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (359, 462, 'FACULDADE DE ENGENHARIA DA FUNDAO ARMANDO ALVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (360, 463, 'FACULDADE DE COMUNICAO E MARKETING DA - FUNDAO ARMANDO LVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (361, 464, 'FACULDADE DE ECONOMIA DA FUNDAO ARMANDO ALVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (362, 465, 'FACULDADE DE COMPUTAO E INFORMTICA DA FUNDAO ALVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (363, 466, 'UNIVERSIDADE ANHEMBI MORUMBI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (364, 467, 'FACULDADE DE CINCIAS CONTBEIS DE ITAPETININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (365, 468, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE ITAPETININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (366, 469, 'ABEU FACULDADES INTEGRADAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (367, 470, 'FACULDADE DE ODONTOLOGIA DE NOVA FRIBURGO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (368, 472, 'UNIVERSIDADE DO GRANDE RIO PROFESSOR JOS DE SOUZA HERDY', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (369, 473, 'FACULDADE DE CINCIAS ECONMICAS, ADMINISTRATIVAS E DA COMPUTAO DOM BOSCO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (370, 474, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DOM BOSCO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (371, 475, 'FACULDADE DE MEDICINA DE PETRPOLIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (372, 480, 'CENTRO UNIVERSITRIO SERRA DOS RGOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (373, 481, 'UNIVERSIDADE GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (374, 482, 'UNIVERSIDADE DO EXTREMO SUL CATARINENSE', 3, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (375, 483, 'CENTRO UNIVERSITRIO METROPOLITANO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (376, 486, 'FACULDADE DE FORMAO DE PROFESSORES DE ARAPIRACA', 2, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (377, 487, 'FACULDADE DE ADMINISTRAO, CINCIAS CONTBEIS, JURDICAS E SOCIAIS DO ESTADO DE ALAGOAS', 2, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (378, 488, 'ESCOLA SUPERIOR DE CINCIAS HUMANAS FSICAS E BIOLGICAS DO SERTO', 2, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (379, 489, 'CENTRO UNIVERSITRIO DE VOLTA REDONDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (380, 490, 'CENTRO DE ENSINO SUPERIOR DE VALENA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (381, 491, 'FACULDADE DE EDUCAO CINCIAS E LETRAS DON DOMNICO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (382, 492, 'FACULDADE DE ADMINISTRAO CINCIAS ECON E CONTBEIS DE GUARATINGUET', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (383, 493, 'FACULDADE DE EDUCAO DE GUARATINGUET', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (384, 494, 'UNIVERSIDADE DO SUL DE SANTA CATARINA', 3, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (385, 495, 'FACULDADE DE CINCIAS E LETRAS DE BRAGANA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (386, 496, 'UNIVERSIDADE DE FRANCA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (387, 498, 'ESCOLA SUPERIOR DE EDUCAO FSICA E DESPORTOS DE CATANDUVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (388, 499, 'FACULDADE DE DIREITO E ADMINISTRAO CATANDUVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (389, 500, 'FACULDADE DE MEDICINA DE CATANDUVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (390, 501, 'ESCOLA SUPERIOR DE CINCIAS DA SANTA CASA DE MISERICRDIA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (391, 502, 'CENTRO UNIVERSITRIO MONTE SERRAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (392, 503, 'FACULDADE DE DIREITO DE CAMPOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (393, 504, 'FACULDADE DE ODONTOLOGIA DE CAMPOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (394, 505, 'FACULDADE DE FILOSOFIA DE CAMPOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (395, 506, 'FACULDADE DE MEDICINA DE CAMPOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (396, 507, 'FACULDADE TRINGULO MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (397, 508, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DO SUL DE MINAS - FACESM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (398, 509, 'FACULDADES INTEGRADAS DE CRUZEIRO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (399, 510, 'ESCOLA DE ENFERMAGEM WENCESLAU BRAZ', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (400, 511, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE GUAXUP', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (401, 512, 'FACULDADE DE CINCIAS ADMINISTRATIVAS ECONMICAS E CONTBEIS DE GUAXUP', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (402, 513, 'UNIVERSIDADE VALE DO RIO DOCE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (403, 514, 'CENTRO UNIVERSITRIO DE BARRA MANSA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (404, 515, 'CENTRO UNIVERSITRIO PLNIO LEITE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (405, 516, 'CENTRO UNIVERSITRIO MOACYR SREDER BASTOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (406, 517, 'FACULDADE DE EDUCAO SO LUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (407, 518, 'CENTRO UNIVERSITRIO DO DISTRITO FEDERAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (408, 519, 'UNIVERSIDADE CATLICA DO SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (409, 521, 'UNIVERSIDADE DE MOGI DAS CRUZES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (410, 522, 'CENTRO UNIVERSITRIO CELSO LISBOA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (411, 524, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE OURO FINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (412, 526, 'UNIVERSIDADE BRAZ CUBAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (413, 527, 'UNIVERSIDADE CATLICA DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (414, 528, 'PONTIFCIA UNIVERSIDADE CATLICA DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (415, 530, 'FACULDADE DE MSICA DO ESPRITO SANTO', 2, 1, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (416, 531, 'FACULDADE DE FORMAO DE PROFESSORES DE VITRIA DE SANTO ANTO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (417, 532, 'UNIVERSIDADE REGIONAL DO NOROESTE DO ESTADO DO RIO GRANDE DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (418, 533, 'FACULDADES INTEGRADAS DE ITAPETININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (419, 534, 'ESCOLA BAHIANA DE MEDICINA E SADE PBLICA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (420, 535, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE MANDAGUARI', 3, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (421, 543, 'FACULDADE DE CINCIAS DA FUNDAO INSTITUTO TECNOLGICO DE OSASCO', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (422, 544, 'FACULDADE DE CINCIAS HUMANAS DE ITABIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (423, 545, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E CONTBEIS DE ITABIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (424, 546, 'PONTIFCIA UNIVERSIDADE CATLICA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (425, 547, 'UNIVERSIDADE DO ESTADO DO RIO DE JANEIRO', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (426, 548, 'UNIVERSIDADE FEDERAL DO MARANHO', 1, 1, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (427, 549, 'UNIVERSIDADE FEDERAL DO ACRE', 1, 1, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (428, 550, 'UNIVERSIDADE ESTADUAL DA PARABA', 2, 1, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (429, 552, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (430, 553, 'FACULDADE DE CINCIAS ADMINISTRATIVAS DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (431, 554, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE BOA ESPERANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (432, 555, 'UNIVERSIDADE DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (433, 556, 'CONSERVATRIO DE MSICA DE NITERI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (434, 557, 'ESCOLA DE MSICA E BELAS ARTES DO PARAN', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (435, 558, 'FACULDADE DE ARTES DO PARAN', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (436, 568, 'UNIVERSIDADE ESTADUAL DO MARANHO', 2, 1, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (437, 569, 'UNIVERSIDADE FEDERAL DO PAR', 1, 1, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (438, 570, 'UNIVERSIDADE FEDERAL DO RIO GRANDE DO NORTE', 1, 1, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (439, 571, 'UNIVERSIDADE FEDERAL DO PARAN', 1, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (440, 572, 'UNIVERSIDADE FEDERAL FLUMINENSE', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (441, 573, 'UNIVERSIDADE FEDERAL DO ESPRITO SANTO', 1, 1, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (442, 574, 'UNIVERSIDADE FEDERAL RURAL DO RIO DE JANEIRO', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (443, 575, 'UNIVERSIDADE FEDERAL DE MINAS GERAIS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (444, 576, 'UNIVERSIDADE FEDERAL DE JUIZ DE FORA', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (445, 577, 'UNIVERSIDADE FEDERAL DE ALAGOAS', 1, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (446, 578, 'UNIVERSIDADE FEDERAL DA BAHIA', 1, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (447, 579, 'UNIVERSIDADE FEDERAL DA PARABA', 1, 1, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (448, 580, 'UNIVERSIDADE FEDERAL DE PERNAMBUCO', 1, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (449, 581, 'UNIVERSIDADE FEDERAL DO RIO GRANDE DO SUL', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (450, 582, 'UNIVERSIDADE FEDERAL DE SANTA MARIA', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (451, 583, 'UNIVERSIDADE FEDERAL DO CEAR', 1, 1, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (452, 584, 'UNIVERSIDADE FEDERAL DE GOIS', 1, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (453, 585, 'UNIVERSIDADE FEDERAL DE SANTA CATARINA', 1, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (454, 586, 'UNIVERSIDADE FEDERAL DO RIO DE JANEIRO', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (455, 587, 'UNIVERSIDADE FEDERAL RURAL DE PERNAMBUCO', 1, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (456, 588, 'UNIVERSIDADE TECNOLGICA FEDERAL DO PARAN', 1, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (457, 589, 'UNIVERSIDADE FEDERAL RURAL DO SEMI-RIDO', 1, 1, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (458, 590, 'UNIVERSIDADE FEDERAL RURAL DA AMAZNIA', 1, 1, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (459, 591, 'UNIVERSIDADE FEDERAL DE SO PAULO', 1, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (460, 592, 'UNIVERSIDADE FEDERAL DE LAVRAS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (461, 593, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA CELSO SUCKOW DA FONSECA', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (462, 594, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA DE MINAS GERAIS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (463, 595, 'UNIVERSIDADE FEDERAL DE ALFENAS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (464, 596, 'UNIVERSIDADE FEDERAL DOS VALES DO JEQUITINHONHA E MUCURI', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (465, 597, 'UNIVERSIDADE FEDERAL DO TRINGULO MINEIRO', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (466, 598, 'UNIVERSIDADE FEDERAL DE ITAJUB - UNIFEI', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (467, 599, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DA BAHIA', 1, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (468, 600, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO MARANHO', 1, 1, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (469, 601, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO RIO GRANDE DO SUL', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (470, 602, 'INSTITUTO TECNOLGICO DE AERONUTICA', 1, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (471, 605, 'UNIVERSIDADE MUNICIPAL DE SO CAETANO DO SUL', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (472, 606, 'FACULDADE DE DIREITO DE CACHOEIRO DO ITAPEMIRIM', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (473, 607, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE CACHOEIRO DO ITAPEMIRIM', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (474, 608, 'ESCOLA SUPERIOR DE CRUZEIRO "PREFEITO HAMILTON VIEIRA MENDES"', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (475, 609, 'UNIVERSIDADE ESTADUAL DO OESTE DO PARAN', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (476, 610, 'CENTRO UNIVERSITRIO METODISTA BENNETT', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (477, 614, 'FACULDADE DE FILOSOFIA SANTA DOROTIA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (478, 615, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS PROFESSORA NAIR FORTES ABU-MERHY', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (479, 616, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (480, 617, 'FACULDADE DE ENFERMAGEM E OBSTETRCIA DE GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (481, 618, 'FACULDADE DE FISIOTERAPIA DE GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (482, 621, 'CENTRO DE ESTUDOS SUPERIORES DE MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (483, 622, 'FACULDADES INTEGRADAS SILVA E SOUZA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (484, 623, 'FACULDADES DE FILOSOFIA CINCIAS E LETRAS DE SANTA CRUZ DO RIO PARDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (485, 624, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS CARLOS QUEIROZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (486, 625, 'FACULDADE CENECISTA DE CAPIVARI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (487, 626, 'FACULDADE CENECISTA DE OSRIO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (488, 627, 'FACULDADE CENECISTA DE VARGINHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (489, 629, 'FACULDADE CENECISTA DE VITRIA - FACEV', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (490, 630, 'FACULDADE CENECISTA DE CINCIAS ADMINISTRATIVAS DE OSRIO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (491, 631, 'CENTRO DE ENSINO SUPERIOR CENECISTA DE FARROUPILHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (492, 632, 'FACULDADE DE CINCIAS ECONMICAS, ADMINISTRATIVAS E CONTBEIS DE SO SEBASTIO DO PARASO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (493, 633, 'INSTITUTO MILITAR DE ENGENHARIA', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (494, 634, 'UNIVERSIDADE FEDERAL DE PELOTAS', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (495, 636, 'ESCOLA SUPERIOR DE PROPAGANDA E MARKETING', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (496, 637, 'FACULDADE DE ENGENHARIA SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (497, 638, 'FACULDADE DE DIREITO MILTON CAMPOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (498, 639, 'FACULDADE DE DESENHO INDUSTRIAL DE MAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (499, 640, 'FACULDADES INTEGRADAS MARIA THEREZA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (500, 641, 'CENTRO UNIVERSITRIO LA SALLE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (501, 645, 'CENTRO UNIVERSITRIO DE JARAGU DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (502, 646, 'FACULDADES ASSOCIADAS DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (503, 647, 'FACULDADE DE ENFERMAGEM LUIZA DE MARILLAC', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (504, 648, 'FACULDADES ASSOCIADAS DE UBERABA - FAZU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (505, 649, 'CENTRO UNIVERSITRIO DE UNIO DA VITRIA', 3, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (506, 650, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE AVAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (507, 651, 'FACULDADE DE ADMINISTRAO DE GOVERNADOR VALADARES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (508, 652, 'FACULDADE SANTA CECLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (509, 657, 'FACULDADE DE FORMAO DE PROFESSORES DE SERRA TALHADA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (510, 659, 'FACULDADE DE CINCIAS HUMANAS DO SUL PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (511, 660, 'FACULDADE METODISTA DO SUL PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (512, 661, 'FACULDADE PIO DCIMO', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (513, 662, 'FACULDADES INTEGRADAS CORAO DE JESUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (514, 663, 'UNIVERSIDADE SALGADO DE OLIVEIRA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (515, 664, 'CENTRO UNIVERSITRIO VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (516, 665, 'UNIVERSIDADE DE TAUBAT', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (517, 666, 'UNIVERSIDADE ESTADUAL DE FEIRA DE SANTANA', 2, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (518, 667, 'FACULDADES INTEGRADAS DE JACAREPAGU', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (519, 668, 'CENTRO UNIVERSITRIO DE ENSINO SUPERIOR DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (520, 669, 'CENTRO UNIVERSITRIO NILTON LINS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (521, 670, 'UNIVERSIDADE SO FRANCISCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (522, 671, 'UNIVERSIDADE ANHANGUERA - UNIDERP', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (523, 672, 'FACULDADE DE ENGENHARIA DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (524, 673, 'CENTRO UNIVERSITRIO DA GRANDE DOURADOS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (525, 674, 'CENTRO DE ENSINO SUPERIOR DO VALE SO FRANCISCO', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (526, 677, 'FACULDADE DE REABILITAO DA ASCE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (527, 678, 'FACULDADE DE ODONTOLOGIA DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (528, 679, 'FACULDADE DE REABILITAO DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (529, 680, 'FACULDADES INTEGRADAS DE PEDRO LEOPOLDO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (530, 681, 'FACULDADES INTEGRADAS ANGLO-AMERICANO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (531, 682, 'FACULDADE DO NOROESTE DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (532, 683, 'FACULDADE DE CINCIAS ECONMICAS DE COLATINA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (533, 684, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE COLATINA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (534, 685, 'FACULDADE DE FORMAO DE PROFESSORES DE BELO JARDIM', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (535, 686, 'FACULDADE DE CINCIAS DA ADMINISTRAO DO LIMOEIRO', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (536, 687, 'FACULDADE ESTADUAL DE FILOSOFIA CINCIAS LETRAS UNIO DA VITRIA', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (537, 688, 'UNIVERSIDADE ESTADUAL DO SUDOESTE DA BAHIA', 2, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (538, 689, 'FACULDADE DE NUTRIO E FONOAUDIOLOGIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (539, 690, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (540, 691, 'FACULDADE DE EDUCAO DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (541, 692, 'FACULDADE DE CINCIAS APLICADAS E SOCIAIS DE PETROLINA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (542, 693, 'UNIVERSIDADE FEDERAL DO ESTADO DO RIO DE JANEIRO', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (543, 694, 'UNIVERSIDADE FEDERAL DE MATO GROSSO DO SUL', 1, 1, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (544, 695, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE ITUMBIARA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (545, 696, 'FACULDADE DE CINCIAS AGRRIAS DE ITUMBIARA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (546, 697, 'FACULDADE DE ADMINISTRAO DE EMPRESAS DE ITUMBIARA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (547, 699, 'FUNDAO UNIVERSIDADE FEDERAL DE RONDNIA', 1, 1, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (548, 700, 'FACULDADE DE EDUCAO DE ITABORA', 3, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (549, 701, 'FACULDADE DE CINCIAS DA ADMINISTRAO DE GARANHUNS', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (550, 702, 'FACULDADE DE FORMAO DE PROFESSORES DE GOIANA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (551, 703, 'FACULDADE DE FORMAO DE PROFESSORES DE ARARIPINA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (552, 704, 'FACULDADE DE CINCIAS AGRRIAS DE ARARIPINA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (553, 705, 'FACULDADE SO JOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (554, 706, 'FACULDADE DE ARTES DULCINA DE MORAES', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (555, 707, 'CENTRO UNIVERSITRIO CENTRAL PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (556, 708, 'INSTITUTO DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (557, 709, 'INSTITUTO DE CINCIAS EXATAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (558, 710, 'FACULDADE BTHENCOURT DA SILVA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (559, 711, 'FACULDADES REUNIDAS DE ADMINISTRAO CINCIAS CONTBEIS E ECONMICAS DE PALMAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (560, 712, 'CENTRO UNIVERSITRIO HERMNIO DA SILVEIRA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (561, 713, 'ESCOLA SUPERIOR DE ESTATSTICA DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (562, 714, 'FACULDADE ESTADUAL DE FILOSOFIA CINCIAS E LETRAS DE CORNLIO PROCPIO', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (563, 715, 'CENTRO UNIVERSITRIO FRANCISCANO DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (564, 716, 'FACULDADES INTEGRADAS DE BOTUCATU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (565, 717, 'FUNDAO UNIVERSIDADE FEDERAL DE CINCIAS DA SADE DE PORTO ALEGRE', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (566, 718, 'UNIVERSIDADE POTIGUAR', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (567, 719, 'UNIVERSIDADE DO ESTADO DE MATO GROSSO', 2, 1, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (568, 720, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS APLICADAS DO CABO DE SANTO AGOSTINHO', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (569, 721, 'FACULDADE DE EDUCAO DE ASSIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (570, 722, 'ESCOLA DE EDUCAO FSICA DE ASSIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (571, 723, 'FACULDADE DE ADMINISTRAO DE ASSIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (572, 724, 'FACULDADE DE CINCIAS, CULTURA E EXTENSO DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (573, 725, 'FACULDADE ESTADUAL DE FILOSOFIA, CINCIAS E LETRAS DE PARANAGU', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (574, 726, 'FACULDADE ESTADUAL DE FILOSOFIA CINCIAS E LETRAS DE JACAREZINHO', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (575, 727, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DO ALTO SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (576, 728, 'FACULDADE DE CINCIAS HUMANAS DE CURVELO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (577, 729, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE PRESIDENTE VENCESLAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (578, 730, 'UNIVERSIDADE ESTADUAL DE PONTA GROSSA', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (579, 734, 'FACULDADES INTEGRADAS MARIA IMACULADA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (580, 736, 'FACULDADE DE CINCIAS APLICADAS SAGRADO CORAO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (581, 737, 'CENTRO UNIVERSITRIO SO CAMILO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (582, 738, 'FACULDADES INTEGRADAS TERESA DVILA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (583, 739, 'CENTRO UNIVERSITRIO SO CAMILO - ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (584, 740, 'FACULDADE DE CINCIAS HUMANAS DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (585, 741, 'ESCOLA SUPERIOR DE SECRETARIADO DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (586, 742, 'FACULDADE DE ARQUITETURA E URBANISMO DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (587, 743, 'ESCOLA SUPERIOR DE ENSINO HELENA ANTIPOFF', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (588, 744, 'FACULDADE DE EDUCAO E CINCIAS HUMANAS DE ANICUNS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (589, 745, 'INSTITUTO MUNICIPAL DE ENSINO SUPERIOR DE SO MANUEL', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (590, 746, 'UNIVERSIDADE REGIONAL DO CARIRI', 2, 1, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (591, 747, 'FACULDADE DE CINCIAS HUMANAS DE IVAIPOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (592, 749, 'CENTRO DE ENSINO SUPERIOR DO VALE DO PARNABA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (593, 750, 'CENTRO UNIVERSITRIO UNIRG', 3, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (594, 751, 'FACULDADE DA ILHA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (595, 752, 'FACULDADE DE PAR DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (596, 753, 'FACULDADE DE CINCIAS HUMANAS DO SERTO CENTRAL', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (597, 754, 'CENTRO DE ENSINO SUPERIOR DE CATALO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (598, 755, 'CENTRO DE ENSINO SUPERIOR DE JATA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (599, 756, 'UNIVERSIDADE ESTADUAL DO PIAU', 2, 1, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (600, 757, 'FACULDADES INTEGRADAS DO TAPAJS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (601, 758, 'FACULDADE BRASILEIRA DE RECURSOS HUMANOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (602, 759, 'FACULDADE DE CINCIAS MDICAS E PARAMDICAS FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (603, 760, 'FACULDADE DE CINCIAS HUMANAS DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (604, 761, 'FACULDADES SPEI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (605, 762, 'FACULDADES INTEGRADAS DE FTIMA DO SUL', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (606, 763, 'INSTITUTO UNIFICADO DE ENSINO SUPERIOR OBJETIVO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (607, 764, 'FACULDADE PINHEIRO GUIMARES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (608, 767, 'INSTITUTO SUPERIOR DE INFORMTICA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (609, 768, 'FACULDADE DE ADMINISTRAO DE NOVA ANDRADINA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (610, 769, 'FACULDADES INTEGRADAS DE NAVIRA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (611, 770, 'FACULDADE DE EDUCAO DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (612, 771, 'FACULDADE MARIA AUGUSTA RIBEIRO DAHER', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (613, 773, 'FACULDADE DE ADMINISTRAO E CINCIAS CONTBEIS DE LINHARES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (614, 774, 'FACULDADE ALVORADA DE INFORMTICA E PROCESSAMENTO DE DADOS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (615, 775, 'FACULDADE ALVORADA DE EDUCAO FSICA E DESPORTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (616, 776, 'FACULDADE DE CINCIAS CONTBEIS DE JEQUI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (617, 777, 'FACULDADE DE EDUCAO THEREZA PORTO MARQUES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (618, 778, 'FACULDADE DE TECNOLOGIA THEREZA PORTO MARQUES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (619, 779, 'FACULDADE DE PONTA POR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (620, 780, 'UNIVERSIDADE DE CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (621, 781, 'FACULDADE DO SUL DE MATO GROSSO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (622, 782, 'FACULDADE CUIABANA DE EDUCAO E LETRAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (623, 783, 'FACULDADE MATOGROSSENSE DE CINCIAS CONTBEIS E ADMINISTRATIVA DE CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (624, 785, 'FACULDADE DE EDUCAO DE TANGAR DA SERRA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (625, 788, 'FACULDADE DE CINCIAS HUMANAS, EXATAS E LETRAS DE RONDNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (626, 789, 'UNIVERSIDADE FEDERAL DE RORAIMA', 1, 1, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (627, 790, 'FACULDADE DE ENFERMAGEM DO HOSPITAL ISRAELITA ALBERT EINSTEIN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (628, 791, 'FACULDADE SENAC DE TURISMO E HOTELARIA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (629, 792, 'CENTRO UNIVERSITRIO DO ESTADO DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (630, 793, 'CENTRO UNIVERSITRIO RADIAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (631, 794, 'CENTRO UNIVERSITRIO DE VRZEA GRANDE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (632, 795, 'FACULDADES INTEGRADAS DE DIAMANTINO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (633, 796, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DO VALE DO SO LOURENO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (634, 797, 'FACULDADE DE EDUCAO E CINCIAS ADMINISTRATIVAS DE VILHENA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (635, 798, 'FACULDADE DE ARACRUZ', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (636, 799, 'FACULDADE LEONARDO DA VINCI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (637, 800, 'FACULDADE DE EDUCAO FSICA MONTENEGRO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (638, 801, 'FACULDADE DE EDUCAO MONTENEGRO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (639, 802, 'CENTRO UNIVERSITRIO CARIOCA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (640, 803, 'FACULDADE INTERAO AMERICANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (641, 809, 'FACULDADE DE DIREITO E CINCIAS SOCIAIS DO LESTE DE MINAS - FADILESTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (642, 810, 'CENTRO TCNICO-EDUCACIONAL SUPERIOR DO OESTE PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (643, 811, 'FACULDADES INTEGRADAS DE CASSILNDIA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (644, 817, 'FACULDADE BATISTA CARIOCA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (645, 818, 'FACULDADE DE ADMINISTRAO DE MINEIROS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (646, 819, 'FACULDADE DE AGRONOMIA DE MINEIROS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (647, 820, 'FACULDADE DE CINCIAS DA COMPUTAO DE MINEIROS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (648, 821, 'FACULDADE DE PEDAGOGIA DE MINEIROS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (649, 822, 'ESCOLA SUPERIOR DE ESTUDOS EMPRESARIAIS E INFORMTICA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (650, 823, 'CENTRO UNIVERSITRIO DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (651, 824, 'FACULDADE DE FILOSOFIA E CINCIAS HUMANAS DE GOIATUBA', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (652, 825, 'FACULDADE PRESBITERIANA GAMMON', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (653, 826, 'CENTRO UNIVERSITRIO DE DESENVOLVIMENTO DO CENTRO-OESTE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (654, 827, 'FACULDADES UNIDAS DO VALE DO ARAGUAIA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (655, 828, 'FACULDADE DE SO LOURENO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (656, 829, 'UNIVERSIDADE DO TOCANTINS', 2, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (657, 830, 'UNIVERSIDADE FEDERAL DO AMAP', 1, 1, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (658, 831, 'FACULDADE DE CINCIAS HUMANAS DO VALE DO RIO GRANDE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (659, 832, 'FACULDADE DE CINCIAS ADMINISTRATIVAS DE CURVELO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (660, 833, 'FACULDADES INTEGRADAS DE ARIQUEMES', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (661, 834, 'FACULDADE VARZEAGRANDENSE DE CINCIAS HUMANAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (662, 835, 'FACULDADE VARZEAGRANDENSE DE COMUNICAO SOCIAL', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (663, 837, 'FACULDADE DE CINCIAS HUMANAS DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (664, 838, 'FACULDADE DE CINCIAS TECNOLGICAS DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (665, 839, 'FACULDADE VITORIANA DE CINCIAS CONTBEIS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (666, 840, 'FACULDADE VITORIANA DE TECNOLOGIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (667, 841, 'FACULDADE DE TURISMO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (668, 843, 'FACULDADE DE CINCIAS E TECNOLOGIA DE BIRIGUI', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (669, 844, 'FACULDADE DE AGRONOMIA E ENGENHARIA FLORESTAL DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (670, 845, 'FACULDADE DA FUNDAO EDUCACIONAL ARAATUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (671, 846, 'FACULDADE DE ADMINISTRAO DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (672, 847, 'FACULDADE PIAUIENSE DE PROCESSAMENTO DE DADOS', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (673, 848, 'FACULDADE PARAIBANA DE PROCESSAMENTO DE DADOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (674, 849, 'FACULDADE JESUTA DE FILOSOFIA E TEOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (675, 850, 'FACULDADE PRIMUS DE INFORMTICA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (676, 851, 'FACULDADE DE TECNOLOGIA EM PROCESSAMENTO DE DADOS DO LITORAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (677, 852, 'FACULDADE DE INFORMTICA E ADMINISTRAO PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (678, 859, 'FACULDADE REGIONAL SANTA CRUZ DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (679, 860, 'FACULDADE DE CINCIAS SOCIAIS DE ARAUCRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (680, 861, 'CENTRO DE ENSINO SUPERIOR DO AMAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (681, 862, 'FACULDADE ESTCIO DE S DE CAMPO GRANDE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (682, 864, 'FACULDADE DE DIREITO DE VRZEA GRANDE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (683, 866, 'FACULDADE DE INFORMTICA DE CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (684, 867, 'FACULDADE DE EDUCAO CINCIAS E LETRAS DE PONTA POR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (685, 873, 'FACULDADES UNIFICADAS DE FOZ DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (686, 874, 'FACULDADE DE ESTUDOS SOCIAIS APLICADOS DE VIANA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (687, 875, 'FACULDADE DE ESTUDOS SOCIAIS DE BARRA DO GARAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (688, 876, 'FACULDADE DE TECNOLOGIA DO VALE DO ARAGUAIA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (689, 877, 'FACULDADE DE ECONOMIA E PROCESSAMENTO DE DADOS DE FOZ DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (690, 878, 'CENTRO UNIVERSITRIO DO LESTE DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (691, 880, 'FACULDADE DE CINCIAS E TECNOLOGIA DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (692, 881, 'INSTITUTO MUNICIPAL DE ENSINO SUPERIOR DE ASSIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (693, 882, 'INSTITUTO MUNICIPAL DE ENSINO SUPERIOR DE BEBEDOURO VICTRIO CARDASSI', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (694, 886, 'FACULDADE PAULISTA DE ADMINISTRAO E CINCIAS CONTBEIS DE HORTOLNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (695, 887, 'FACULDADE CHRISTUS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (696, 888, 'FACULDADE BANDEIRANTES DE CINCIAS EXATAS E HUMANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (697, 890, 'ESCOLA DE GOVERNO PROFESSOR PAULO NEVES DE CARVALHO', 2, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (698, 891, 'FACULDADE BRASILEIRA DE CINCIAS EXATAS, HUMANAS E SOCIAIS - FABRAI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (699, 896, 'ESCOLA SUPERIOR DE TURISMO E HOTELARIA - FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (700, 897, 'UNIO DAS ESCOLAS SUPERIORES DO VALE DO IVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (701, 898, 'FACULDADE DE CINCIAS ECONMICAS E ADMINISTRATIVAS SANTA RITA DE CSSIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (702, 899, 'FACULDADES INTEGRADAS DE RIO VERDE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (703, 900, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E DE TECNOLOGIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (704, 903, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (705, 904, 'FACULDADE DE PROCESSAMENTO DE DADOS DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (706, 905, 'FACULDADE ESCOLA PAULISTA DE DIREITO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (707, 906, 'FACULDADE DE EDUCAO E CINCIAS GERENCIAIS DE SUMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (708, 907, 'FACULDADE DE EDUCAO E CINCIAS GERENCIAIS DE INDAIATUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (709, 908, 'FACULDADE ALAGOANA DE ADMINISTRAO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (710, 909, 'FACULDADE DE BELFORD ROXO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (711, 910, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E CONTBEIS SANTA LCIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (712, 911, 'FACULDADES UNIDAS DE ITUMBIARA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (713, 912, 'FACULDADE COMPACTO DE CINCIAS GERENCIAIS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (714, 913, 'FACULDADES INTEGRADAS DE PARANABA - FIPAR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (715, 914, 'FACULDADE PARANAENSE DE ADMINISTRAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (716, 915, 'FACULDADE DE ADMINISTRAO DE FTIMA DO SUL', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (717, 916, 'FACULDADE DE CINCIAS HUMANAS E JURDICAS DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (718, 917, 'FACULDADE DE CINCIAS CONTBEIS DE NAVIRAI', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (719, 918, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE CASCAVEL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (720, 922, 'FACULDADE DE CINCIAS GERENCIAIS DE DRACENA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (721, 923, 'FACULDADE DE ADMINISTRAO E CINCIAS CONTBEIS DE SO ROQUE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (722, 924, 'FACULDADE INTERLAGOS DE EDUCAO E CULTURA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (723, 925, 'FACULDADE DE CINCIAS GERNCIAIS DE VILHENA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (724, 926, 'CENTRO UNIVERSITRIO DE CAMPO GRANDE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (725, 927, 'CENTRO DE ESTUDOS SUPERIORES DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (726, 929, 'INSTITUTO MANAUARA DE ENSINO SUPERIOR', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (727, 930, 'FACULDADE DE EDUCAO DE ALTA FLORESTA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (728, 934, 'FACULDADE SO CARLOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (729, 935, 'FACULDADE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (730, 939, 'FACULDADE CATLICA DE ANPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (731, 940, 'FACULDADE DE CINCIAS CONTBEIS DE PONTA POR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (732, 941, 'FACULDADE DE EDUCAO DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (733, 944, 'ESCOLA SUPERIOR DE PROPAGANDA E MARKETING DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (734, 945, 'INSTITUTO DE CINCIAS SOCIAIS E HUMANAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (735, 950, 'UNIO DE ENSINO SUPERIOR DE NOVA MUTUM', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (736, 951, 'FACULDADES INTEGRADAS DE COXIM', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (737, 952, 'UNIVERSIDADE SANTA CECLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (738, 953, 'UNIVERSIDADE METROPOLITANA DE SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (739, 954, 'FACULDADE DE CINCIAS AGRO- AMBIENTAIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (740, 955, 'FACULDADE DE FILOSOFIA NOSSA SENHORA IMACULADA CONCEIO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (741, 959, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (742, 960, 'FACULDADE DE CINCIAS CONTBEIS DE ALTA FLORESTA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (743, 961, 'FACULDADE GARCIA SILVEIRA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (744, 966, 'FACULDADE DE TURISMO - SANTOS DUMONT', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (745, 967, 'FACULDADE DE TECNOLOGIA DE TAQUARITINGA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (746, 968, 'FACULDADE DE TECNOLOGIA EM PROCESSAMENTO DE DADOS DE ANDRADINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (747, 969, 'FACULDADE DE TURISMO - FORMIGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (748, 970, 'FACULDADE DE SERVIO SOCIAL DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (749, 971, 'FACULDADE DE TECNOLOGIA E CINCIAS DE CONSELHEIRO LAFAIETE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (750, 973, 'FACULDADE DE CINCIAS GERENCIAIS E ORAMENTOS E CONTBEIS LUCLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (751, 975, 'FACULDADE DE FISIOTERAPIA DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (752, 976, 'FACULDADES INTEGRADAS DE PONTA POR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (753, 977, 'FACULDADE SO CAMILO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (754, 980, 'FACULDADE DE ADMINISTRAO DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (755, 983, 'FACULDADE PRUDENTE DE MORAES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (756, 984, 'FACULDADE DE EDUCAO DE IVAIPOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (757, 985, 'FACULDADE DE CINCIAS E EDUCAO DE RUBIATABA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (758, 986, 'FACULDADE DE EDUCAO E CULTURA MONTESSORI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (759, 988, 'FACULDADE DE COMUNICAO SOCIAL TAMOIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (760, 989, 'ESCOLA SUPERIOR DE BIOLOGIA E MEIO AMBIENTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (761, 990, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DE CONGONHAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (762, 991, 'FACULDADE SENAI-CETIQT', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (763, 994, 'FACULDADE FLAMA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (764, 997, 'FACULDADE DE CINCIAS ECONMICAS DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (765, 998, 'FACULDADE DE SERVIO SOCIAL DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (766, 999, 'FACULDADE DE DIREITO DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (767, 1001, 'FACULDADE DE CINCIAS ECONMICAS E ADMINISTRATIVAS DE PRESIDENTE PRUDENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (768, 1002, 'FACULDADE DE DIREITO DE PRESIDENTE PRUDENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (769, 1003, 'FACULDADE DE SERVIO SOCIAL PRESIDENTE PRUDENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (770, 1012, 'FACULDADE DE FORMAO DE PROFESSORES DE AFOGADOS DA INGAZEIRA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (771, 1013, 'FACULDADE SUDOESTE PAULISTANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (772, 1014, 'INSTITUTO SUPERIOR E CENTRO EDUCACIONAL LUTERANO - BOM JESUS - IELUSC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (773, 1015, 'FACULDADE DE CINCIAS CONTBEIS MACHADO DE ASSIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (774, 1016, 'FACULDADE DE CINCIAS HUMANAS DO VALE DO PIRANGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (775, 1017, 'FACULDADE STEFANINI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (776, 1019, 'FACULDADE DE CINCIAS E TECNOLOGIA DE UNA - FACTU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (777, 1021, 'FACULDADE DE CINCIAS DE TIMBABA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (778, 1027, 'UNIVERSIDADE ESTADUAL DO NORTE FLUMINENSE DARCY RIBEIRO', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (779, 1028, 'UNIVERSIDADE ESTADUAL DE MATO GROSSO DO SUL', 2, 1, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (780, 1030, 'FACULDADE DE ECONOMIA E FINANAS IBMEC', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (781, 1031, 'FACULDADE ANDR DA SILVEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (782, 1032, 'CENTRO UNIVERSITRIO SALESIANO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (783, 1034, 'UNIO DE ESCOLAS SUPERIORES DA FUNESO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (784, 1036, 'UNIVERSIDADE DO ESTADO DE MINAS GERAIS', 2, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (785, 1038, 'FACULDADES INTEGRADAS DE TRS LAGOAS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (786, 1040, 'FACULDADES DE VALINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (787, 1041, 'CENTRO UNIVERSITRIO UNIVATES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (788, 1042, 'UNIVERSIDADE POSITIVO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (789, 1043, 'CENTRO UNIVERSITRIO HERMINIO OMETTO DE ARARAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (790, 1045, 'CENTRO UNIVERSITRIO ANHANGERA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (791, 1046, 'UNIO DAS FACULDADES DOS GRANDES LAGOS - UNILAGO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (792, 1047, 'FACULDADE TECNOLGICA PORTO-ALEGRENSE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (793, 1048, 'FACULDADE CAMPOS ELSEOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (794, 1049, 'INSTITUTO DE ENSINO SUPERIOR FUCAPI', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (795, 1051, 'FACULDADE DE CINCIAS CONTBEIS - MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (796, 1052, 'FACULDADE ADVENTISTA DE ADMINISTRAO DO NORDESTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (797, 1053, 'FACULDADE DE TECNOLOGIA E CINCIAS DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (798, 1054, 'ESCOLA BAHIANA DE ADMINISTRAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (799, 1055, 'FACULDADE MAURCIO DE NASSAU DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (800, 1058, 'CENTRO UNIVERSITRIO DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (801, 1059, 'FACULDADE LOURENO FILHO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (802, 1060, 'INSTITUTO DE EDUCAO SUPERIOR DE BRASLIA - IESB', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (803, 1061, 'FACULDADE PLANALTO DE ADMINISTRAO E CINCIAS ECONMICAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (804, 1063, 'FACULDADE BATISTA DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (805, 1064, 'FACULDADE BATISTA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (806, 1065, 'FACULDADE DE CINCIAS ECONMICAS E ADMINISTRATIVAS DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (807, 1066, 'INSTITUTO DE ENSINO E PESQUISA OBJETIVO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (808, 1067, 'FACULDADE DE JUSSARA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (809, 1068, 'FACULDADE ALDETE MARIA ALVES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (810, 1069, 'CENTRO REGIONAL DE ESTUDOS EM CINCIAS HUMANAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (811, 1070, 'INSTITUTO DE ENSINO SUPERIOR CENECISTA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (812, 1071, 'INSTITUTO DE ENSINO SUPERIOR DA FUNLEC', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (813, 1072, 'FACULDADE AFIRMATIVO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (814, 1075, 'INSTITUTO DE EDUCAO SUPERIOR DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (815, 1076, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE CAJAZEIRAS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (816, 1077, 'FACULDADE INTEGRADA DO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (817, 1078, 'FACULDADE INTEGRADO DE CAMPO MOURO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (818, 1079, 'FACULDADE MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (819, 1080, 'FACULDADE ARTHUR S EARP NETO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (820, 1081, 'FACULDADE DE ENGENHARIA DE RESENDE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (821, 1082, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO RIO GRANDE DO NORTE', 1, 1, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (822, 1084, 'FACULDADE CENECISTA DE BENTO GONALVES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (823, 1085, 'FACULDADE METODISTA DE SANTA MARIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (824, 1086, 'FACULDADE DE EDUCAO DE COLORADO DO OESTE', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (825, 1087, 'FACULDADES INTEGRADAS APARCIO CARVALHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (826, 1088, 'FACULDADE DE DESENHO INDUSTRIAL DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (827, 1089, 'CENTRO DE EDUCAO SUPERIOR - NICA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (828, 1090, 'FACULDADE SO LUS DE FRANA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (829, 1091, 'FACULDADE DE PEDAGOGIA DE VARGEM GRANDE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (830, 1092, 'FACULDADES INTEGRADAS DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (831, 1093, 'FACULDADE DE CINCIAS JURDICAS E GERENCIAIS DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (832, 1095, 'FACULDADE DE ADMINISTRAO E NEGCIOS DE MONTE ALTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (833, 1096, 'FACULDADE DE PRESIDENTE EPITCIO - FAPE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (834, 1097, 'FACULDADES INTEGRADAS DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (835, 1099, 'FACULDADE DE EDUCAO E CINCIAS GERENCIAIS DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (836, 1100, 'FACULDADE CARLOS DRUMMOND DE ANDRADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (837, 1101, 'FACULDADE ADVENTISTA DE CINCIAS DA SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (838, 1102, 'FACULDADE DE INFORMTICA - MARLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (839, 1103, 'FACULDADE DE ADMINISTRAO SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (840, 1104, 'FACULDADE DE COMUNICAO SOCIAL DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (841, 1107, 'FACULDADE INTEGRADA DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (842, 1108, 'FACULDADE DE CINCIAS GERENCIAIS - COTIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (843, 1111, 'FACULDADE DE TURISMO DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (844, 1112, 'FACULDADE DE RELAES INTERNACIONAIS DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (845, 1113, 'CENTRO UNIVERSITRIO EURO-AMERICANO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (846, 1114, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS APLICADAS RIO BRANCO - FIRB', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (847, 1115, 'FACULDADE SANTA TEREZINHA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (848, 1116, 'FACULDADE CAIARAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (849, 1117, 'FACULDADE SENAC DE COMUNICAO E ARTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (850, 1118, 'UNIO DAS ESCOLAS DO GRUPO FAIMI DE EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (851, 1119, 'FACULDADE PLANALTO DE FILOSOFIA, CINCIAS E LETRAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (852, 1120, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA FLUMINENSE', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (853, 1122, 'FACULDADE DE EDUCAO DE OSVALDO CRUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (854, 1123, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (855, 1124, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA DE BARREIRAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (856, 1125, 'FACULDADES INTEGRADAS ESPRITA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (857, 1126, 'UNIVERSIDADE ESTADUAL DO CENTRO OESTE', 2, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (858, 1128, 'UNIVERSIDADE DE ITANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (859, 1129, 'CENTRO UNIVERSITRIO DO NORTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (860, 1130, 'FACULDADE METODISTA DE CINCIAS HUMANAS E EXATAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (861, 1131, 'FACULDADE SANTO AGOSTINHO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (862, 1132, 'FACULDADE DE CINCIA DA COMPUTAO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (863, 1134, 'FACULDADE EVANGLICA LUTERANA MARTINUS*', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (864, 1135, 'FACULDADE PLANALTO DE CINCIA DA COMPUTAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (865, 1136, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS DE IGARASSU', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (866, 1137, 'CENTRO DE ENSINO SUPERIOR HERIS DE JENIPAPO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (867, 1138, 'CENTRO DE EDUCAO SUPERIOR DE BLUMENAU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (868, 1139, 'FACULDADE DE ESTUDOS ADMINISTRATIVOS DE MINAS GERAIS - FEAD-MG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (869, 1140, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (870, 1141, 'FACULDADE GAMA E SOUZA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (871, 1142, 'FACULDADES INTEGRADAS DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (872, 1143, 'FACULDADE DE EDUCAO SUPERIOR DE TIMBABA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (873, 1144, 'FACULDADE DE CINCIAS HUMANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (874, 1145, 'FACULDADE DE CINCIAS AGRRIAS DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (875, 1147, 'FACULDADE RUY BARBOSA DE PSICOLOGIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (876, 1149, 'CENTRO UNIVERSITRIO NOSSA SENHORA DO PATROCNIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (877, 1150, 'FACULDADE SENAI DE TECNOLOGIA GRFICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (878, 1151, 'FACULDADE DE ADMINISTRAO E NEGCIOS DE SERGIPE', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (879, 1152, 'INSTITUTO SUL BRASIL DE ENSINO SUPERIOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (880, 1153, 'UNIVERSIDADE CNDIDO MENDES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (881, 1155, 'FACULDADE SALESIANA DE SANTA TERESA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (882, 1156, 'FACULDADE CENECISTA DE ITABORA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (883, 1157, 'ESCOLA SUPERIOR SO FRANCISCO DE ASSIS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (884, 1158, 'FACULDADE RUY BARBOSA DE CINCIA DA COMPUTAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (885, 1159, 'CENTRO DE ENSINO SUPERIOR DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (886, 1160, 'FACULDADE CAMBURY', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (887, 1161, 'FACULDADE IBMEC SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (888, 1162, 'FACULDADE DE ADMINISTRAO DE ALTA FLORESTA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (889, 1163, 'FACULDADE DE CINCIA DA COMPUTAO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (890, 1164, 'FACULDADE VARZEAGRANDENSE DE ADMINISTRAO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (891, 1166, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA  DA PARABA', 1, 1, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (892, 1168, 'FACULDADE DE ANLISE DE SISTEMAS DE INFORMAO DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (893, 1169, 'FACULDADE SENAC DE MODA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (894, 1170, 'FACULDADE METROPOLITANA DE CAMAARI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (895, 1171, 'FACULDADE DE LETRAS - MARLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (896, 1172, 'FACULDADE AD 1', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (897, 1173, 'FACULDADE JUSCELINO KUBITSCHEK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (898, 1174, 'FACULDADE DE SABAR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (899, 1175, 'FACULDADES RIOGRANDENSES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (900, 1176, 'FACULDADE DE ANGRA DOS REIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (901, 1177, 'ESCOLA SUPERIOR DE CINCIAS AGRRIAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (902, 1179, 'ESCOLA SUPERIOR DE CINCIAS DA SADE DE RIO VERDE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (903, 1180, 'FACULDADE BARDDAL DE LETRAS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (904, 1181, 'FACULDADE BRASLIA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (905, 1182, 'INSTITUTO DE ENSINO SUPERIOR DE AMERICANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (906, 1183, 'INSTITUTO SO JOS DOS CAMPOS DE ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (907, 1184, 'CENTRO DE CINCIAS DA SADE DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (908, 1185, 'CENTRO UNIVERSITRIO JORGE AMADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (909, 1186, 'FACULDADE DE TURISMO E HOTELARIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (910, 1187, 'CENTRO UNIVERSITRIO MDULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (911, 1189, 'UNIVERSIDADE DO PLANALTO CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (912, 1190, 'FACULDADE INTEGRAL CANTAREIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (913, 1191, 'INSTITUTO DE ENSINO SUPERIOR DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (914, 1192, 'FACULDADE DE CINCIAS DA SADE DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (915, 1194, 'FACULDADE DE TECNOLOGIA DE PONTA POR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (916, 1195, 'FACULDADE SENAI DE TECNOLOGIA MECATRNICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (917, 1196, 'CENTRO UNIVERSITRIO DE MARING - CEUMAR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (918, 1197, 'FACULDADE DE TURISMO - AEMA/RJ', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (919, 1198, 'FACULDADE DE CINCIAS SOCIAIS E APLICADAS DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (920, 1200, 'INSTITUTO BRASILIENSE DE TECNOLOGIA E CINCIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (921, 1201, 'FACULDADE DE ADMINISTRAO MILTON CAMPOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (922, 1202, 'FACULDADE SANTA RITA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (923, 1203, 'FACULDADE DE ECONOMIA DE OURO FINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (924, 1204, 'FACULDADE DE AMAMBAI', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (925, 1205, 'FACULDADE EDUCACIONAL DA LAPA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (926, 1206, 'FACULDADE ADVENTISTA DE FISIOTERAPIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (927, 1207, 'FACULDADE DE ADMINISTRAO GERAL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (928, 1208, 'FACULDADE DE NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (929, 1209, 'FACULDADE DE ADMINISTRAO DE EMPRESAS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (930, 1210, 'FACULDADE DE CINCIAS DA COMUNICAO DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (931, 1211, 'INSTITUTO DE ENSINO SUPERIOR DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (932, 1212, 'FACULDADE DE CINCIAS CONTBEIS DE ASSIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (933, 1213, 'FACULDADE FLEMING', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (934, 1215, 'FACULDADE ANGLO LATINO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (935, 1217, 'CENTRO DE TECNOLOGIA EM AUTOMAO DE INFORMTICA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (936, 1219, 'INSTITUTO ITAPETININGANO DE ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (937, 1220, 'INSTITUTO MATO DE ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (938, 1221, 'FACULDADE DE CINCIAS GERENCIAIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (939, 1222, 'FACULDADE SENAC DE CINCIAS EXATAS E TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (940, 1223, 'FACULDADE RADIAL JABAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (941, 1224, 'CENTRO UNIVERSITRIO DE JALES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (942, 1225, 'CENTRO UNIVERSITRIO AMPARENSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (943, 1226, 'INSTITUTO DE ENSINO SUPERIOR DO ACRE', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (944, 1227, 'FACULDADE SO FRANCISCO DE BARREIRAS - FASB', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (945, 1228, 'FACULDADE DE TECNOLOGIA EMPRESARIAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (946, 1229, 'INSTITUTO DE ENSINO SUPERIOR DO NORDESTE', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (947, 1230, 'CENTRO REGIONAL UNIVERSITRIO DE ESPRITO SANTO DO PINHAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (948, 1231, 'INSTITUTO CENECISTA DE ENSINO SUPERIOR DE SANTO NGELO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (949, 1232, 'CENTRO UNIVERSITRIO CAMPOS DE ANDRADE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (950, 1233, 'CENTRO UNIVERSITRIO CNDIDO RONDON', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (951, 1234, 'FACULDADE METROPOLITANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (952, 1235, 'ESCOLA DE EDUCAO SUPERIOR IPITANGA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (953, 1236, 'ESCOLA POLITCNICA IPITANGA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (954, 1237, 'FACULDADE NOSSA SENHORA APARECIDA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (955, 1238, 'FACULDADE DE CINCIAS CONTBEIS DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (956, 1239, 'FACULDADE PADRO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (957, 1240, 'FACULDADE DE ESTUDOS SOCIAIS DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (958, 1241, 'FACULDADE ESPRITO SANTENSE DE ENSINO TECNOLGICO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (959, 1243, 'FACULDADE DE ADMINISTRAO DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (960, 1244, 'FACULDADE BRASILEIRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (961, 1245, 'FACULDADE DE CASTELO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (962, 1246, 'FACULDADE CANDIDO MENDES DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (963, 1247, 'FACULDADE DE CINCIAS CONTBEIS DE NOVA ANDRADINA - FACINAN', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (964, 1248, 'FACULDADE DE PEDAGOGIA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (965, 1249, 'FACULDADE DE EDUCAO DE COSTA RICA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (966, 1252, 'FACULDADE PROMOVE DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (967, 1253, 'FACULDADE METODISTA GRANBERY', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (968, 1254, 'FACULDADE DE ADMINISTRAO DE ITABIRITO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (969, 1255, 'FACULDADE BOA VIAGEM', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (970, 1256, 'FACULDADE DOM BOSCO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (971, 1257, 'FACULDADE DE ADMINISTRAO, CINCIAS, EDUCAO E LETRAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (972, 1258, 'FACULDADE DE CINCIAS APLICADAS DE CASCAVEL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (973, 1259, 'FACULDADE NOBEL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (974, 1260, 'FACULDADE OPET', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (975, 1261, 'FACULDADE ADVENTISTA DE CINCIAS CONTBEIS DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (976, 1262, 'FACULDADE ADVENTISTA DE EDUCAO DO NORDESTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (977, 1263, 'FACULDADE BEZERRA DE ARAJO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (978, 1264, 'FACULDADE NATALENSE PARA O DESENVOLVIMENTO DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (979, 1265, 'FACULDADE DE PROCESSAMENTO DE DADOS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (980, 1266, 'FACULDADES INTEGRADAS URUBUPUNG', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (981, 1267, 'INSTITUTO DE ENSINO SUPERIOR DA GRANDE FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (982, 1270, 'INSTITUTO DE ENSINO SUPERIOR COC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (983, 1271, 'FACULDADE DE COMUNICAO SOCIAL DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (984, 1272, 'FACULDADE SANTANNA DE SALTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (985, 1273, 'FACULDADE CAMPO LIMPO PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (986, 1274, 'FACULDADE DE EDUCAO DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (987, 1275, 'FACULDADE DO GUARUJ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (988, 1276, 'INSTITUTO TAUBAT DE ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (989, 1277, 'FACULDADE DE INFORMTICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (990, 1278, 'FACULDADE ADVENTISTA DE EDUCAO E CINCIAS HUMANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (991, 1279, 'FACULDADE DE ITPOLIS - FACITA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (992, 1280, 'INSTITUTO DE ENSINO SUPERIOR DE COTIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (993, 1281, 'FACULDADE DE CINCIAS SOCIAIS E AGRRIAS DE ITAPEVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (994, 1283, 'INSTITUTO MANCHESTER PAULISTA DE ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (995, 1285, 'INSTITUTO DE ENSINO SUPERIOR DE MAIRIPOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (996, 1286, 'FACULDADE SENAI DE TECNOLOGIA AMBIENTAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (997, 1287, 'FACULDADE DE CINCIAS GERENCIAIS DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (998, 1288, 'FACULDADE INDEPENDENTE BUTANT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (999, 1289, 'FACULDADE DE DIREITO DA FUNDAO ARMANDO ALVARES PENTEADO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1000, 1290, 'FACULDADE ALBERT EINSTEIN DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1001, 1291, 'FACULDADE DO NORTE PIONEIRO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1002, 1292, 'FACULDADES ADAMANTINENSES INTEGRADAS', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1003, 1293, 'INSTITUTO DE ENSINO SUPERIOR DO OESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1004, 1294, 'FACULDADE DAS AMRICAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1005, 1295, 'FACULDADE MORUMBI SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1006, 1296, 'ESCOLA SUPERIOR DE MARKETING', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1007, 1297, 'FACULDADE DOURADOS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1008, 1298, 'FACULDADE DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1009, 1299, 'FACULDADES INTEGRADAS DE FERNANDPOLIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1010, 1300, 'INSTITUTO TAQUARITINGUENSE DE ENSINO SUPERIOR DR. ARISTIDES DE CARVALHO SCHLOBACH', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1011, 1301, 'INSTITUTO MATONENSE MUNICIPAL DE ENSINO SUPERIOR', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1012, 1302, 'FACULDADE BATISTA BRASILEIRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1013, 1303, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA GOIANO', 1, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1014, 1304, 'FACULDADES INTEGRADAS DE RIBEIRO PIRES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1015, 1305, 'FACULDADE DE CINCIAS JURDICAS, GERENCIAIS E EDUCAO DE SINOP', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1016, 1306, 'FACULDADE DE CINCIAS CONTBEIS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1017, 1307, 'FACULDADE DE CINCIAS HUMANAS DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1018, 1308, 'FACULDADE NOVO MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1019, 1309, 'FACULDADE JESUS MARIA JOS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1020, 1310, 'FACULDADE DE AMERICANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1021, 1311, 'TREVISAN ESCOLA SUPERIOR DE NEGCIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1022, 1312, 'FACULDADES INTEGRADAS DE RONDONPOLIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1023, 1313, 'FACULDADE BANDEIRANTES DE EDUCAO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1024, 1314, 'INSTITUTO DE ENSINO SUPERIOR JOO ALFREDO DE ANDRADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1025, 1315, 'FACULDADE DE EDUCAO DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1026, 1316, 'FACULDADE DA TERRA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1027, 1317, 'FACULDADE SUDOESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1028, 1318, 'FACULDADE JUVNCIO TERRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1029, 1319, 'FACULDADE TECSOMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1030, 1320, 'FACULDADE DE EDUCAO FSICA DO LITORAL NORTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1031, 1321, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E CONTBEIS DA ZONA NORTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1032, 1322, 'FACULDADE EDUVALE DE AVAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1033, 1323, 'FACULDADE TANCREDO NEVES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1034, 1324, 'FACULDADE DE INFORMTICA LEMOS DE CASTRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1035, 1325, 'FACULDADE DE APUCARANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1036, 1326, 'ESCOLA SUPERIOR DE ENSINO ANSIO TEIXEIRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1037, 1327, 'ESCOLA SUPERIOR DE PROPAGANDA E MARKETING DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1038, 1328, 'FACULDADE DE COMUNICAO E TURISMO DE OLINDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1039, 1329, 'FACULDADE DE ADMINISTRAO DE DIADEMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1040, 1330, 'FACULDADE COTEMIG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1041, 1331, 'FACULDADE HOYLER DE COMUNICAO SOCIAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1042, 1332, 'FACULDADE DE COMUNICAO SOCIAL DE HORTOLNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1043, 1333, 'FACULDADE CECAP DO LAGO NORTE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1044, 1334, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS - FACISA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1045, 1335, 'INSTITUTO DE ENSINO SUPERIOR THATHI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1046, 1336, 'FACULDADE ASSIS GURGACZ', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1047, 1337, 'FACULDADE MATER DEI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1048, 1338, 'FACULDADE DE EDUCAO DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1049, 1339, 'FACULDADE DE CINCIAS ADMINISTRATIVAS DE PARANABA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1050, 1340, 'FACULDADE DE CINCIAS GERENCIAIS DE SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1051, 1341, 'FACULDADE DE ARQUITETURA E URBANISMO DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1052, 1342, 'FACULDADE DE PEDAGOGIA DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1053, 1343, 'FACULDADE DE EDUCAO DA TERRA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1054, 1344, 'FACULDADE DO SABER', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1055, 1345, 'FACULDADE INTEGRAO - ZONA OESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1056, 1346, 'FACULDADE BATISTA DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1057, 1347, 'FACULDADE ADVENTISTA DE ADMINISTRAO DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1058, 1348, 'FACULDADE BATISTA DE ADMINISTRAO E INFORMTICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1059, 1349, 'FACULDADE DE CINCIAS ECONMICAS DE AFONSO CLUDIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1060, 1350, 'FACULDADE DO VALE DO JAGUARIBE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1061, 1351, 'INSTITUTO SUPERIOR TUPY - IST', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1062, 1352, 'FACULDADE ADVENTISTA DE EDUCAO FSICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1063, 1353, 'INSTITUTO DE ENSINO SUPERIOR DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1064, 1354, 'FACULDADE DE CINCIA DE ADMINISTRAO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1065, 1355, 'FACULDADES INTEGRADAS DE JAHU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1066, 1356, 'FACULDADES INTEGRADAS DE SANTA F DO SUL', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1067, 1357, 'INSTITUTO SUPERIOR DE EDUCAO DO RIO DE JANEIRO', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1068, 1358, 'FACULDADE DE PEDAGOGIA DE AFONSO CLUDIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1069, 1359, 'FACULDADE CAPIXABA DE NOVA VENCIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1070, 1360, 'FACULDADE SO GABRIEL', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1071, 1361, 'FACULDADE DOM ORIONE', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1072, 1362, 'FACULDADE UBAENSE OZANAM COELHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1073, 1363, 'FACULDADE VISCONDE DE CAIR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1074, 1364, 'FACULDADE DE TECNOLOGIA E CINCIAS DE VITRIA DA CONQUISTA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1075, 1365, 'CENTRO UNIVERSITRIO ADVENTISTA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1076, 1366, 'FACULDADE BARDDAL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1077, 1367, 'FACULDADE DE CINCIA DA COMPUTAO DE FORMIGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1078, 1368, 'FACULDADE DE CINCIAS CONTBEIS DE BOM DESPACHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1079, 1369, 'FACULDADE SENAC DE TURISMO E HOTELARIA DE GUAS DE SO PEDRO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1080, 1370, 'FACULDADE SENAC DE TURISMO E HOTELARIA DE CAMPOS DO JORDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1081, 1371, 'FACULDADE DE MIRANDPOLIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1082, 1372, 'FACULDADE SENAC DE EDUCAO AMBIENTAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1083, 1373, 'FACULDADE CASA BRANCA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1084, 1374, 'FACULDADE DE CINCIAS SOCIAIS DE GUARANT DO NORTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1085, 1375, 'FACULDADE CATLICA RAINHA DA PAZ DE ARAPUTANGA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1086, 1376, 'FACULDADE COMPACTO DE EDUCAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1087, 1377, 'FACULDADE DE CINCIA E TECNOLOGIA DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1088, 1378, 'FACULDADE ASSEMBLEIANA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1089, 1379, 'FACULDADES INTEGRADAS SO PEDRO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1090, 1380, 'FACULDADE DE DOMINGOS MARTINS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1091, 1381, 'FACULDADE MARTHA FALCO', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1092, 1382, 'ESCOLA SUPERIOR DE TEOLOGIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1093, 1383, 'FACULDADE JOS LACERDA FILHO DE CINCIAS APLICADAS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1094, 1384, 'FACULDADE SANTA HELENA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1095, 1385, 'FACULDADES INTEGRADAS IPEP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1096, 1386, 'ESCOLA SUPERIOR DE ADMINISTRAO E MARKETING E COMUNICAO DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1097, 1387, 'FACULDADE OCTGONO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1098, 1388, 'FACULDADE SUMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1099, 1389, 'FACULDADE TORRICELLI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1100, 1390, 'FACULDADE ELITE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1101, 1394, 'FACULDADE CASTRO ALVES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1102, 1395, 'FACULDADE DE CALDAS NOVAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1103, 1396, 'FACULDADE DINMICA DAS CATARATAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1104, 1397, 'FACULDADE OESTE MINEIRA DE INFORMTICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1105, 1398, 'FACULDADE DE DIREITO DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1106, 1399, 'FACULDADE UNISSA DE SARANDI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1107, 1400, 'FACULDADE METROPOLITANA DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1108, 1401, 'FACULDADE ADELMAR ROSADO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1109, 1402, 'INSTITUTO DE CINCIAS SOCIAIS E HUMANAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1110, 1403, 'FACULDADE DE PIMENTA BUENO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1111, 1404, 'FACULDADE DE PIRACANJUBA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1112, 1405, 'FACULDADE DE CINCIAS CONTBEIS DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1113, 1406, 'FACULDADE DE ENGENHARIA E TECNOLOGIA DO INSTITUTO MAIRIPOR DE ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1114, 1407, 'FACULDADE DE ECONOMIA E ADMINISTRAO DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1115, 1408, 'FACULDADE DE SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1116, 1409, 'FACULDADE DO CENTRO LESTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1117, 1410, 'REA1 - FACULDADE DE CINCIA E TECNOLOGIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1118, 1411, 'FACULDADE DOIS DE JULHO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1119, 1412, 'FACULDADE ANHAGUERA DE JUNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1120, 1413, 'FACULDADE DE PRIMAVERA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1121, 1414, 'FACULDADE SO LUCAS', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1122, 1415, 'FACULDADE MAGISTER', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1123, 1416, 'FACULDADE GRANDE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1124, 1417, 'FACULDADE CENECISTA DE CAMPO LARGO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1125, 1418, 'CENTRO UNIVERSITRIO TOLEDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1126, 1419, 'CENTRO UNIVERSITRIO CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1127, 1420, 'FACULDADES DE DRACENA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1128, 1422, 'CENTRO UNIVERSITRIO DO NORTE', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1129, 1423, 'FACULDADE IDEAL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1130, 1424, 'FACULDADE PIO XII', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1131, 1425, 'FACULDADE EVOLUTIVO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1132, 1426, 'INSTITUTO LUTERANO DE ENSINO SUPERIOR DE ITUMBIARA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1133, 1427, 'FACULDADE DA SERRA GACHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1134, 1428, 'INSTITUTO DE ENSINO SUPERIOR PLANALTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1135, 1429, 'FACULDADE BATISTA DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1136, 1430, 'FACULDADE ING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1137, 1431, 'FACULDADE PIRATININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1138, 1432, 'FACULDADE BERTIOGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1139, 1433, 'FACULDADE ORGENES LESSA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1140, 1434, 'INSTITUTO DE ENSINO SUPERIOR DE ITAPIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1141, 1435, 'FACULDADE DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1142, 1436, 'ESCOLA SUPERIOR BATISTA DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1143, 1437, 'ESCOLA SUPERIOR DE TECNOLOGIA E EDUCAO DE RIO CLARO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1144, 1438, 'FACULDADE DE ADMINISTRAO DE EMPRESAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1145, 1439, 'FACULDADE DE CINCIAS ECONMICAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1146, 1440, 'INSTITUTO APHONSIANO DE ENSINO SUPERIOR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1147, 1441, 'FACULDADE DE TECNOLOGIA DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1148, 1442, 'FACULDADE DE DIREITO DE TANGAR DA SERRA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1149, 1443, 'FACULDADE NACIONAL', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1150, 1444, 'FACULDADES INTEGRADAS EINSTEIN DE LIMEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1151, 1445, 'CENTRO UNIVERSITRIO DO INSTITUTO MAU DE TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1152, 1446, 'CENTRO UNIVERSITRIO PLANALTO DO DISTRITO FEDERAL - UNIPLAN', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1153, 1447, 'FACULDADE DE DIREITO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1154, 1448, 'FACULDADE DO CENTRO EDUCACIONAL DE NITERI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1155, 1449, 'FACULDADE DE EDUCAO, ADMINISTRAO E TECNOLOGIA DE IBAITI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1156, 1450, 'CENTRO UNIVERSITRIO DO CERRADO-PATROCNIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1157, 1451, 'FACULDADES INTEGRADAS DE SO CARLOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1158, 1452, 'FACULDADE ATENAS MARANHENSE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1159, 1453, 'FACULDADE SARANDI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1160, 1454, 'FACULDADE SALESIANA DO NORDESTE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1161, 1455, 'FACULDADE SANTSSIMO SACRAMENTO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1162, 1456, 'FACULDADE DE SERTOZINHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1163, 1457, 'FACULDADE EURO-PANAMERICANA DE HUMANIDADES E TECNOLOGIAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1164, 1458, 'FACULDADE DO NORTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1165, 1459, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1166, 1460, 'FACULDADE DE CINCIAS SOCIAIS E APLICADAS DO COMPLEXO DE ENSINO SUPERIOR DO BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1167, 1461, 'FACULDADE DE TECNOLOGIA E CINCIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1168, 1462, 'INSTITUTO PARAIBANO DE ENSINO RENOVADO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1169, 1463, 'FACULDADE ALFACASTELO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1170, 1464, 'FACULDADE MDULO PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1171, 1465, 'FACULDADE BANDEIRANTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1172, 1466, 'INSTITUTO DE ENSINO SUPERIOR DE ALPHAVILLE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1173, 1467, 'FACULDADE CENECISTA DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1174, 1468, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1175, 1469, 'FACULDADE DE CINCIAS SOCIAIS DE FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1176, 1470, 'FACULDADE CAMES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1177, 1471, 'INSTITUTO DE ENSINO SUPERIOR DE LINHARES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1178, 1472, 'CENTRO UNIVERSITRIO LEONARDO DA VINCI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1179, 1473, 'FACULDADE DE ADMINISTRAO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1180, 1474, 'FACULDADE DE EDUCAO DE ATIBAIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1181, 1475, 'FACULDADE DE CINCIAS ECONMICAS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1182, 1476, 'FACULDADES INTEGRADAS INTERAMERICANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1183, 1477, 'FACULDADE MICHELANGELO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1184, 1478, 'FACULDADE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1185, 1479, 'FACULDADES INTEGRADAS TIBIRI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1186, 1480, 'FACULDADE DE LETRAS DOMUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1187, 1481, 'FACULDADE DE ADMINISTRAO DE FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1188, 1482, 'FACULDADE DE CINCIAS ECONMICAS VIANNA JNIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1189, 1483, 'FACULDADE DE CINCIAS ECONMICAS DE EUNPOLIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1190, 1484, 'FACULDADE IBMEC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1191, 1485, 'FACULDADE DE ENFERMAGEM DE CATANDUVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1192, 1486, 'FACULDADE ESTCIO DE S DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1193, 1487, 'FACULDADE DOM BOSCO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1194, 1488, 'FACULDADE PARASO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1195, 1489, 'FACULDADE DE SELVRIA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1196, 1490, 'FACULDADE DE JAGUARINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1197, 1491, 'FACULDADE INTERNACIONAL DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1198, 1492, 'FACULDADE DE CINCIAS APLICADAS DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1199, 1493, 'FACULDADE ALVES FARIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1200, 1494, 'FACULDADE SALESIANA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1201, 1495, 'FACULDADE UNIO DE QUEDAS DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1202, 1496, 'FACULDADE ESTCIO DE S DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1203, 1497, 'FACULDADE DE ADMINISTRAO DE SANTA CRUZ DO RIO PARDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1204, 1498, 'INSTITUTO DE ENSINO SUPERIOR E FORMAO AVANADA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1205, 1499, 'FACULDADE TABOO DA SERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1206, 1500, 'FACULDADE DE ENSINO SUPERIOR DE SO MIGUEL DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1207, 1501, 'FACULDADE LATINO AMERICANA DE EDUCAO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1208, 1502, 'FACULDADE JAPI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1209, 1503, 'FACULDADE DOUTOR LEOCDIO JOS CORREIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1210, 1504, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE MACEI - ESAMC', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1211, 1505, 'FACULDADE DE MEDICINA VETERINRIA E ZOOTECNIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1212, 1506, 'INSTITUTO CENECISTA FAYAL DE ENSINO SUPERIOR', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1213, 1507, 'FACULDADE CENECISTA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1214, 1508, 'FACULDADE METROPOLITANA DE MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1215, 1509, 'FACULDADE ESTCIO DE S DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1216, 1510, 'FACULDADE ESTCIO DE S DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1217, 1511, 'FACULDADE DE ENFERMAGEM DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1218, 1512, 'FACULDADE DE CINCIA DA EDUCAO DE LAGES', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1219, 1513, 'INSTITUTO DE CINCIAS JURDICAS E SOCIAIS PROFESSOR CAMILLO FILHO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1220, 1514, 'FACULDADE VALE DO CRICAR', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1221, 1515, 'FACULDADE DE EDUCAO DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1222, 1516, 'FACULDADE DE VINHEDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1223, 1517, 'FACULDADE NOVO ATENEU DE GUARAPUAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1224, 1518, 'FACULDADE SANTA TEREZINHA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1225, 1519, 'FACULDADE DE PATO BRANCO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1226, 1520, 'FACULDADE MRIO DE ANDRADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1227, 1521, 'INSTITUTO DE ESTUDOS SUPERIORES DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1228, 1522, 'FACULDADE DE EDUCAO DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1229, 1523, 'FACULDADE DE DIREITO FRANCISCO BELTRO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1230, 1524, 'FACULDADES INTEGRADAS OLGA METTIG', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1231, 1525, 'FACULDADE DE CINCIAS CONTBEIS DE AFONSO CLUDIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1232, 1526, 'FACULDADE SENAI DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1233, 1527, 'FACULDADE TAMOIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1234, 1528, 'FACULDADE DE CINCIAS HUMANAS SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1235, 1529, 'FACULDADE DE EDUCAO FSICA DE AMERICANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1236, 1530, 'FACULDADE CENECISTA DE JOINVILLE - FACE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1237, 1531, 'FACULDADE DE ENFERMAGEM DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1238, 1532, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS NOSSA SENHORA APARECIDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1239, 1533, 'FACULDADE DE COMUNICAO SOCIAL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1240, 1534, 'FACULDADE DE CINCIAS, LETRAS E EDUCAO DO NOROESTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1241, 1535, 'FACULDADE SO JUDAS TADEU DE PINHAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1242, 1536, 'FACULDADE DE TELMACO BORBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1243, 1537, 'FACULDADE EXPONENCIAL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1244, 1538, 'FACULDADE PROJEO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1245, 1539, 'FACULDADE TALO BRASILEIRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1246, 1540, 'INSTITUTO DE ENSINO SUPERIOR DE RONDNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1247, 1541, 'FACULDADE FLAMINGO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1248, 1542, 'CENTRO UNIVERSITRIO GERALDO DI BIASE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1249, 1543, 'FACULDADE DE DIREITO DE LINHARES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1250, 1544, 'FACULDADE DECISO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1251, 1545, 'INSTITUTO DE EDUCAO SUPERIOR DE BOITUVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1252, 1546, 'FACULDADE DE EDUCAO SO FRANCISCO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1253, 1547, 'FACULDADE DE MEDICINA DE JUAZEIRO DO NORTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1254, 1548, 'FACULDADE CAMPO GRANDE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1255, 1549, 'FACULDADE PALAS ATENA DE CHOPINZINHO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1256, 1550, 'FACULDADE DE EDUCAO FSICA DE FOZ DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1257, 1551, 'FACULDADE FRANCO-BRASILEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1258, 1552, 'FACULDADE PIAUIENSE', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1259, 1553, 'FACULDADE DE ODONTOLOGIA DE SO JOS DO RIO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1260, 1554, 'FACULDADES INTEGRADAS DO VALE DO RIBEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1261, 1555, 'FACULDADE DE DIREITO DE GUARAPUAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1262, 1556, 'FACULDADE SETE DE SETEMBRO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1263, 1557, 'UNIVERSIDADE FUMEC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1264, 1558, 'INSTITUTO DE ENSINO SUPERIOR DE JOO MONLEVADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1265, 1559, 'CENTRO UNIVERSITRIO DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1266, 1561, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1267, 1562, 'FACULDADE DE ILHA SOLTEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1268, 1563, 'FACULDADE DE SADE, CINCIAS HUMANAS E TECNOLGICAS DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1269, 1564, 'FACULDADE SUL FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1270, 1565, 'FACULDADE UNIME DE CINCIAS JURDICAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1271, 1566, 'FACULDADE DE CINCIAS GERENCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1272, 1567, 'FACULDADE VIZINHANA VALE DO IGUAU', 3, 1, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1273, 1568, 'FACULDADES INTEGRADAS DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1274, 1569, 'FACULDADES INTEGRADAS DE ARARAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1275, 1570, 'FACULDADE DE CINCIAS CONTBEIS DE POMBAL', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1276, 1571, 'FACULDADE UNIME DE CINCIAS SOCIAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1277, 1572, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1278, 1573, 'FACULDADE ALFREDO NASSER', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1279, 1574, 'FACULDADE EDUCACIONAL DE MEDIANEIRA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1280, 1575, 'CENTRO UNIVERSITRIO DO PLANALTO DE ARAX', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1281, 1576, 'INSTITUTO DE ENSINO SUPERIOR BLAURO CARDOSO DE MATTOS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1282, 1577, 'FACULDADE INTERMUNICIPAL DO NOROESTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1283, 1578, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA SUL-RIO-GRANDENSE', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1284, 1579, 'FACULDADE DE CINCIAS SOCIAIS DE IBIRAU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1285, 1580, 'FACULDADE DE CINCIAS E EDUCAO SENA AIRES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1286, 1581, 'FACULDADE DA CIDADE DE SANTA LUZIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1287, 1582, 'FACULDADE DECISO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1288, 1583, 'FACULDADE DE TECNOLOGIA DE INDAIATUBA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1289, 1584, 'FACULDADE DE TECNOLOGIA DE GUARATINGUET', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1290, 1585, 'FACULDADE ENERGIA DE ADMINISTRAO E NEGCIOS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1291, 1586, 'UNIVERSIDADE DO VALE DO SAPUCA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1292, 1587, 'FACULDADES INTEGRADAS DE TANGAR DA SERRA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1293, 1588, 'FACULDADE PITGORAS DE TURISMO E HOTELARIA DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1294, 1589, 'FACULDADE DE CASCAVEL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1295, 1590, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS APLICADAS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1296, 1591, 'FACULDADE SEAMA', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1297, 1592, 'FACULDADE DE ODONTOLOGIA DE MANAUS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1298, 1593, 'FACULDADE DE PEDAGOGIA DE EUNPOLIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1299, 1594, 'FACULDADE DE LETRAS DA INSTITUIO DE ENSINO SUPERIOR DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1300, 1595, 'FACULDADE DE LETRAS UNIO DE ENSINO SUPERIOR CERTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1301, 1596, 'FACULDADE DE CINCIAS E EDUCAO DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1302, 1597, 'FACULDADE SANTA F', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1303, 1598, 'FACULDADE POLITCNICA DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1304, 1599, 'INSTITUTO DE ENSINO SUPERIOR PRESIDENTE TANCREDO DE ALMEIDA NEVES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1305, 1600, 'INSTITUTO DE CINCIAS DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1306, 1601, 'FACULDADE CIDADE DE COROMANDEL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1307, 1602, 'FACULDADE DE CINCIAS GERENCIAIS DO OESTE DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1308, 1603, 'FACULDADE DE COMUNICAO SOCIAL DO OESTE DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1309, 1604, 'FACULDADE MAGNUN DE EDUCAO FSICA E ESPORTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1310, 1605, 'FACULDADE DE ADMINISTRAO DE NOVA ANDRADINA - FANOVA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1311, 1606, 'FACULDADE VALE DO APOR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1312, 1607, 'FACULDADE LUTERANA SO MARCOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1313, 1608, 'FACULDADE DE INFORMTICA DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1314, 1609, 'FACULDADE DOS CERRADOS PIAUIENSES', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1315, 1610, 'FACULDADE DAS ATIVIDADES EMPRESARIAIS DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1316, 1611, 'FACULDADE EDUCACIONAL DE ARAPOTI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1317, 1612, 'FACULDADE SUL BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1318, 1613, 'FACULDADE ADVENTISTA PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1319, 1614, 'FACULDADE PALAS ATENA DE ASTORGA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1320, 1615, 'INSTITUTO SUPERIOR DO LITORAL DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1321, 1616, 'FACULDADE ANGEL VIANNA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1322, 1617, 'FACULDADE DE CINCIAS ECONMICAS DA REGIO CARBONFERA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1323, 1618, 'INSTITUTO BLUMENAUENSE DE ENSINO SUPERIOR', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1324, 1619, 'FACULDADE ERNESTO RISCALI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1325, 1620, 'FACULDADE SANTA RITA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1326, 1621, 'FACULDADE NETWORK', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1327, 1622, 'FACULDADE DO INTERIOR PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1328, 1623, 'FACULDADE DE CINCIAS AGRRIAS DE ANDRADINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1329, 1624, 'FACULDADE MARECHAL RONDON', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1330, 1625, 'FACULDADE REUNIDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1331, 1626, 'CENTRO DE TECNOLOGIA E CINCIA - CETEC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1332, 1627, 'FACULDADE DE EDUCAO DE MONTE ALTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1333, 1628, 'FACULDADE DE CINCIAS HUMANAS DE AGUA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1334, 1629, 'INSTITUTO DE ENSINO SUPERIOR AQUIDAUANENSE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1335, 1630, 'FACULDADE ITABIRANA DE DESENVOLVIMENTO DAS CINCIAS E TECNOLOGIAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1336, 1631, 'FACULDADE PITGORAS DE FISIOTERAPIA DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1337, 1632, 'FACULDADE METROPOLITANA LONDRINENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1338, 1633, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA DE CUIAB', 1, 1, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1339, 1634, 'FACULDADE BATISTA DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1340, 1635, 'FACULDADE DE CINCIAS DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1341, 1636, 'CASTELLI ESCOLA SUPERIOR DE HOTELARIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1342, 1637, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS DE MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1343, 1638, 'FACULDADE DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1344, 1639, 'FACULDADE HLIO ROCHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1345, 1640, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1346, 1641, 'FACULDADE SOCIAL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1347, 1642, 'FACULDADE DE TECNOLOGIA E CINCIAS DE ITABUNA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1348, 1643, 'FACULDADE ANSIO TEIXEIRA DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1349, 1644, 'FACULDADE UNIME DE EDUCAO E COMUNICAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1350, 1645, 'FACULDADE DE TECNOLOGIA E CINCIAS DE JEQUI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1351, 1646, 'FACULDADE DE EDUCAO CINCIAS E LETRAS DE PARASO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1352, 1647, 'INSTITUTO DE ENSINO SUPERIOR DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1353, 1648, 'FACULDADE COMPACTO DE COMUNICAO SOCIAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1354, 1649, 'FACULDADE DE COMUNICAO DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1355, 1650, 'FACULDADE J. SIMES ENSINO SUPERIOR', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1356, 1651, 'FACULDADE DE EDUCAO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1357, 1652, 'FACULDADE DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1358, 1653, 'FACULDADE DE GUAU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1359, 1654, 'INSTITUTO DE ENSINO SUPERIOR DE NOVA VENCIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1360, 1655, 'FACULDADE CENECISTA DE SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1361, 1656, 'INSTITUTO DE ENSINO SUPERIOR DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1362, 1657, 'FACULDADE EDUCACIONAL DE DOIS VIZINHOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1363, 1658, 'FACULDADE INTEGRADA DA GRANDE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1364, 1659, 'FACULDADE ESTCIO DE S DE OURINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1365, 1660, 'FACULDADE DE CINCIAS SOCIAIS E TECNOLGICAS FACITEC', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1366, 1661, 'FACULDADE DE ADMINISTRAO ESCOLA SUPERIOR PROFESSOR PAULO MARTINS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1367, 1662, 'FACULDADE CAPIXABA DE ADMINISTRAO E EDUCAO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1368, 1663, 'FACULDADE ARAGUAIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1369, 1664, 'FACULDADE DA SADE E ECOLOGIA HUMANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1370, 1665, 'FACULDADE KENNEDY', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1371, 1666, 'FACULDADE NOVOS HORIZONTES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1372, 1667, 'FACULDADE INTERNACIONAL DE CINCIAS EMPRESARIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1373, 1668, 'FACULDADE METROPOLITANA DE BELO HORIZONTE -METROPOLITANA -BH', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1374, 1669, 'INSTITUTO METROPOLITANO DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1375, 1670, 'FACULDADE ASA DE BRUMADINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1376, 1671, 'FACULDADE DE ENSINO UNIFICADO DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1377, 1672, 'INSTITUTO ESPERANA DE ENSINO SUPERIOR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1378, 1673, 'FACULDADE DE AGRONOMIA DE POMBAL', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1379, 1674, 'INSTITUTO SALESIANO DE FILOSOFIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1380, 1675, 'FACULDADE METROPOLITANA DA GRANDE RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1381, 1676, 'FACULDADE SO MIGUEL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1382, 1677, 'FACULDADE DE ENSINO SUPERIOR DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1383, 1678, 'FACULDADE DE CINCIAS DE WENCESLAU BRAZ', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1384, 1679, 'FACULDADE DE CAMPINA GRANDE DO SUL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1385, 1680, 'FACULDADE DE CINCIAS E TECNOLOGIA MATER CHRISTI', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1386, 1681, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS IBMEC', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1387, 1682, 'FACULDADE SALESIANA MARIA AUXILIADORA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1388, 1683, 'FACULDADE PIAUIENSE', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1389, 1684, 'FACULDADE DE RORAIMA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1390, 1685, 'FACULDADE BARDDAL DE CINCIAS CONTBEIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1391, 1686, 'FACULDADE BARDDAL DE ARTES APLICADAS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1392, 1687, 'FACULDADE IDEPE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1393, 1688, 'FACULDADE SENAC DE EDUCAO EM SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1394, 1689, 'FACULDADE SANTA IZILDINHA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1395, 1690, 'FACULDADE DE SO VICENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1396, 1691, 'FACULDADE PRXIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1397, 1692, 'ESCOLA SUPERIOR DE TECNOLOGIA E EDUCAO DE PORTO FERREIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1398, 1693, 'FACULDADE ITANHAM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1399, 1694, 'ESCOLA SUPERIOR DE CRICIMA - ESUCRI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1400, 1695, 'FACULDADE DE EDUCAO DE BOM DESPACHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1401, 1696, 'FACULDADE CENTRAL DE CRISTALINA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1402, 1697, 'FACULDADE ESCRITOR OSMAN DA COSTA LINS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1403, 1698, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DO CESCARELI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1404, 1699, 'FACULDADE DE EDUCAO DE JARU', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1405, 1700, 'FACULDADE SETE LAGOAS DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1406, 1701, 'FACULDADE MACHADO DE ASSIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1407, 1702, 'FACULDADE ATUAL DA AMAZNIA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1408, 1703, 'INSTITUTO DE ENSINO SUPERIOR DE RIO VERDE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1409, 1704, 'FACULDADE DE ADMINISTRAO DO ALTO VALE DO RIO TUBARO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1410, 1705, 'FACULDADE DE EDUCAO DO ALTO VALE DO RIO TUBARO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1411, 1706, 'FACULDADE DE INFORMTICA DO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1412, 1707, 'FACULDADE DE CINCIAS CONTBEIS DE RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1413, 1708, 'INSTITUTO PERNAMBUCANO DE ENSINO SUPERIOR', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1414, 1709, 'FACULDADE INTEGRADA DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1415, 1710, 'FACULDADE DE NEGCIOS E TECNOLOGIAS DA INFORMAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1416, 1711, 'FACULDADE DE PRESIDENTE PRUDENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1417, 1712, 'FACULDADE DE CINCIAS E TECNOLOGIA DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1418, 1713, 'FACULDADE DE TECNOLOGIA DE SO VICENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1419, 1714, 'ESCOLA SUPERIOR DE GESTO COMERCIAL E MARKETING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1420, 1715, 'FACULDADE DO SUL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1421, 1716, 'FACULDADE UNIO DAS AMRICAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1422, 1717, 'FACULDADE DE IMPERATRIZ', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1423, 1718, 'FACULDADE NOBRE DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1424, 1719, 'FACULDADE DE ADMINISTRAO DE CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1425, 1720, 'FACULDADE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1426, 1721, 'FACULDADE DE VIOSA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1427, 1722, 'FACULDADE DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1428, 1723, 'ESCOLA SUPERIOR DE ADMINISTRAO E GESTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1429, 1724, 'FACULDADE DE GUARARAPES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1430, 1725, 'FACULDADE XV DE AGOSTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1431, 1726, 'FACULDADE SANTA AMLIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1432, 1727, 'FACULDADE DE CARIACICA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1433, 1728, 'FACULDADE INTERAMERICANA DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1434, 1729, 'FACULDADE DE ODONTOLOGIA DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1435, 1730, 'FACULDADE NORTE CAPIXABA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1436, 1731, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1437, 1732, 'FACULDADE GENNARI E PEARTREE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1438, 1733, 'FACULDADE DE ENSINO SUPERIOR DE MARECHAL CNDIDO RONDON', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1439, 1734, 'FACULDADE INTEGRAL DIFERENCIAL', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1440, 1735, 'FACULDADE DE CINCIAS APLICADAS DOUTOR LEO SAMPAIO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1441, 1736, 'INSTITUTO SUPERIOR DE EDUCAO LA SALLE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1442, 1737, 'FACULDADE DE INFORMTICA DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1443, 1738, 'INSTITUTO LUTERANO DE ENSINO SUPERIOR DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1444, 1739, 'INSTITUTO DE ENSINO SUPERIOR DO CENTRO OESTE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1445, 1740, 'FACULDADE DE TECNOLOGIA RADIAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1446, 1741, 'FACULDADE DE TECNOLOGIA INTERAMRICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1447, 1742, 'FACULDADE DE TECNOLOGIA CARLOS DRUMMOND DE ANDRADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1448, 1743, 'FACULDADE DE TECNOLOGIA OSWALDO CRUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1449, 1744, 'FACULDADE DE TECNOLOGIA OPET', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1450, 1745, 'FACULDADE DE TECNOLOGIA ALBERT EINSTEIN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1451, 1746, 'FACULDADE DE TECNOLOGIA DAS AMRICAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1452, 1747, 'FACULDADE DE TECNOLOGIA FLAMINGO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1453, 1748, 'FACULDADE DE TECNOLOGIA MRIO DE ANDRADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1454, 1749, 'FACULDADE LUSO-BRASILEIRA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1455, 1750, 'UNIDADE DE ENSINO SUPERIOR EXPOENTE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1456, 1751, 'FACULDADE DE DESIGN GRFICO', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1457, 1752, 'FACULDADE AUTNOMA DE DIREITO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1458, 1753, 'FACULDADE DE ENFERMAGEM NOVA ESPERANA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1459, 1754, 'INSTITUTO SUPERIOR DE EDUCAO DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1460, 1755, 'FACULDADE EQUIPE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1461, 1756, 'FACULDADE ASSOCIADA BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1462, 1757, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE GOINIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1463, 1758, 'FACULDADE INDEPENDENTE DO NORDESTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1464, 1759, 'FACULDADE PADRE JOO BAGOZZI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1465, 1760, 'FACULDADE DE PEDAGOGIA DE VITRIA DE SANTO ANTO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1466, 1761, 'FACULDADE BRASILEIRA DE GESTO E NEGCIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1467, 1762, 'FACULDADE DE TECNOLOGIA JOO XXIII', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1468, 1763, 'FACULDADE DE TECNOLOGIA SENAI CHAPEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1469, 1764, 'FACULDADE DE EDUCAO REGIONAL SERRANA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1470, 1765, 'FACULDADE DE JOS BONIFCIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1471, 1766, 'FACULDADE ESPRITO SANTENSE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1472, 1767, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1473, 1768, 'FACULDADE REGIONAL SERRANA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1474, 1769, 'FACULDADE DE TURISMO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1475, 1770, 'FACULDADE DE CINCIAS EXATAS E TECNOLGICAS DA INSTITUIO DE ENSINO SUPERIOR DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1476, 1771, 'FACULDADE SANTA MARIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1477, 1772, 'FACULDADE NORDESTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1478, 1773, 'FACULDADE DE ITAPIRANGA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1479, 1774, 'FACULDADE EDUCACIONAL DE PONTA GROSSA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1480, 1775, 'FACULDADE DO VALE DO IPOJUCA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1481, 1776, 'FACULDADE DE TECNOLOGIA PROF. LUIZ ROSA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1482, 1777, 'FACULDADE METROPOLITANA DE GUARAMIRIM', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1483, 1778, 'FACULDADE DE EDUCAO, CINCIAS E LETRAS SO JUDAS TADEU', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1484, 1779, 'FACULDADE AIEC', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1485, 1780, 'FACULDADE HORIZONTINA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1486, 1781, 'FACULDADE INTESP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1487, 1782, 'FACULDADE DE ARTES, CINCIAS E TECNOLOGIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1488, 1783, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1489, 1784, 'FACULDADE EDITORA NACIONAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1490, 1785, 'FACULDADE DE COLIDER', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1491, 1786, 'INSTITUTO MACHADENSE DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1492, 1787, 'INSTITUTO DE ENSINO SUPERIOR DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1493, 1788, 'FACULDADE DE ADMINISTRAO E ARTES DE LIMEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1494, 1789, 'INSTITUTO CATUA DE ENSINO SUPERIOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1495, 1790, 'INSTITUTO SUPERIOR DE EDUCAO DO SUL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1496, 1791, 'FACULDADE DE CINCIAS EXATAS E TECNOLGICAS DA UNIO DE ENSINO SUPERIOR CERTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1497, 1792, 'FACULDADE DE EDUCAO DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1498, 1793, 'FACULDADE DE CINCIAS EXATAS E TECNOLGICAS DE UNIO DA VITRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1499, 1794, 'FACULDADE DE CINCIAS HUMANAS DE UNIO DA VITRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1500, 1795, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE UNIO DA VITRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1501, 1796, 'FACULDADE DE MATEMTICA DA INSTITUIO DE ENSINO SUPERIOR DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1502, 1797, 'FACULDADE NORTE PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1503, 1798, 'FACULDADE EDUCACIONAL DE CORNLIO PROCPIO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1504, 1799, 'FACULDADE BRASLIA DE TECNOLOGIA, CINCIAS E EDUCAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1505, 1800, 'FACULDADE DE REALEZA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1506, 1801, 'FACULDADE DOM BOSCO DE UBIRAT', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1507, 1802, 'FACULDADE DE CINCIAS DA SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1508, 1803, 'FACULDADE DE INFORMTICA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1509, 1804, 'FACULDADE DE MAU - FAMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1510, 1805, 'FACULDADE DOS GUARARAPES', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1511, 1806, 'FACULDADE FERNO DIAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1512, 1807, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO CEAR', 1, 1, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1513, 1808, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO ESPRITO SANTO', 1, 1, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1514, 1809, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE PERNAMBUCO', 1, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1515, 1810, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE SO PAULO', 1, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1516, 1811, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE GOIS', 1, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1517, 1812, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA  DO AMAZONAS', 1, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1518, 1813, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO PAR', 1, 1, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1519, 1814, 'FACULDADE DE CINCIAS SOCIAIS E APLICADAS DE DIAMANTINO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1520, 1815, 'FACULDADE DO INSTITUTO BRASIL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1521, 1816, 'INSTITUTO DE ENSINO SUPERIOR DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1522, 1817, 'FACULDADE RADIAL CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1523, 1818, 'FACULDADE PITGORAS DE ADMINISTRAO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1524, 1819, 'FACULDADE FARIAS BRITO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1525, 1820, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO PIAU', 1, 1, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1526, 1821, 'FACULDADE LIONS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1527, 1822, 'FACULDADE SUL-AMERICANA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1528, 1823, 'INSTITUTO DE ENSINO SUPERIOR DO AMAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1529, 1824, 'FACULDADE DE CINCIAS CONTBEIS DE ITABIRITO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1530, 1825, 'FACULDADE DE ESTUDOS SUPERIORES DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1531, 1826, 'FACULDADE DE DIREITO DE LEOPOLDINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1532, 1827, 'FACULDADE LUTERANA DE TEOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1533, 1828, 'ESCOLA DA CIDADE - FACULDADE DE ARQUITETURA E URBANISMO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1534, 1829, 'FACULDADE DE TECNOLOGIA UIRAPURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1535, 1830, 'FACULDADE ANHANGUERA DE PASSO FUNDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1536, 1831, 'FACULDADE CENTRO PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1537, 1832, 'FACULDADE DE JABOTICABAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1538, 1833, 'FACULDADE DE ARUJ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1539, 1834, 'FACULDADE DE AGUDOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1540, 1835, 'FACULDADE INTEGRADA DE ENSINO SUPERIOR DE COLINAS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1541, 1836, 'CENTRO UNIVERSITRIO FUNDAO DE ENSINO OCTVIO BASTOS - FEOB', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1542, 1837, 'ESCOLA DE ESTUDOS SUPERIORES DE VIOSA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1543, 1838, 'FACULDADES INTEGRADAS RIO BRANCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1544, 1839, 'FACULDADES INTEGRADAS DE VRZEA GRANDE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1545, 1840, 'FACULDADE CASTELO BRANCO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1546, 1841, 'FACULDADES INTEGRADAS ANTNIO EUFRSIO DE TOLEDO DE PRESIDENTE PRUDENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1547, 1842, 'FACULDADES INTEGRADAS MACHADO DE ASSIS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1548, 1843, 'FACULDADE DE ADMINISTRAO DE CAMPO BELO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1549, 1844, 'FACULDADES INTEGRADAS STELLA MARIS DE ANDRADINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1550, 1845, 'FEFISA - FACULDADES INTEGRADAS DE SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1551, 1846, 'CENTRO UNIVERSITRIO DE LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1552, 1847, 'FACULDADE ATENAS MARANHENSE DE IMPERATRIZ', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1553, 1848, 'FACULDADE CENECISTA DE SANTA TEREZINHA DE ITAIPU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1554, 1849, 'FACULDADE DE DIREITO DE MOGI MIRIM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1555, 1850, 'FACULDADE ALVORADA DE TECNOLOGIA E EDUCAO DE MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1556, 1851, 'ESCOLA BRASILEIRA DE ADMINISTRAO PBLICA E DE EMPRESAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1557, 1852, 'FACULDADE SABERES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1558, 1853, 'FACULDADE DE TECNOLOGIA IBTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1559, 1854, 'FACULDADES INTEGRADAS CLARETIANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1560, 1855, 'CENTRO UNIVERSITRIO CATLICO DO SUDOESTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1561, 1856, 'FACULDADE DO NORTE NOVO DE APUCARANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1562, 1857, 'FACULDADE DEHONIANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1563, 1858, 'FACULDADE DE ADMINISTRAO DE JEQUI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1564, 1859, 'INSTITUTO SUPERIOR DE EDUCAO UIRAPURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1565, 1860, 'FACULDADE DE CINCIAS GERENCIAIS PADRE ARNALDO JANSSEN', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1566, 1861, 'FACULDADE DE DIREITO DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1567, 1862, 'FACULDADE DA REGIO SERRANA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1568, 1863, 'FACULDADE DE MANTENA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1569, 1864, 'FACULDADE DE ADMINISTRAO DA ASSOCIAO BRASILIENSE DE EDUCAO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1570, 1865, 'INSTITUTO SUPERIOR DE EDUCAO ALVORADA PLUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1571, 1866, 'FACULDADE DE CINCIAS EMPRESARIAIS E ESTUDOS COSTEIROS DE NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1572, 1867, 'FACULDADE DE CINCIAS GERENCIAIS DE SANTOS DUMONT', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1573, 1868, 'INSTITUTO SUPERIOR DE EDUCAO DE SANTOS DUMONT', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1574, 1869, 'CENTRO UNIVERSITRIO DE ITAJUB', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1575, 1870, 'FACULDADES INTEGRADAS REGIONAIS DE AVAR', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1576, 1871, 'FACULDADE DE MEDICINA DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1577, 1872, 'FACULDADES INTEGRADAS SANTA CRUZ DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1578, 1873, 'FACULDADE MERCRIO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1579, 1874, 'FACULDADE DE TECNOLOGIA DE OURINHOS', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1580, 1875, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE IBITINGA', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1581, 1876, 'FACULDADE MUNICIPAL PROFESSOR FRANCO MONTORO DE MOGI GUAU', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1582, 1877, 'FACULDADE ATUAL', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1583, 1878, 'CENTRO UNIVERSITRIO DA FUNDAO EDUCACIONAL INACIANA PE SABIA DE MEDEIROS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1584, 1879, 'FACULDADE EDUCACIONAL DE ARAUCRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1585, 1880, 'FACULDADE CASA DO ESTUDANTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1586, 1881, 'FACULDADE DA ESCADA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1587, 1882, 'FACULDADE DE ASSUNTOS ACADMICOS E CIENTFICOS MONSENHOR EDISSON VIEIRA LCIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1588, 1883, 'FACULDADE DE CINCIAS AGRRIAS E DA SADE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1589, 1884, 'FACULDADE RANCHARIENSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1590, 1885, 'UNIDADE DE ENSINO SUPERIOR DOM BOSCO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1591, 1886, 'FACULDADE DE MONTE ALTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1592, 1887, 'FACULDADE DE DIREITO DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1593, 1888, 'FACULDADE DE SADE TECSOMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1594, 1889, 'FACULDADE HOYLER DE LETRAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1595, 1890, 'FACULDADE VILA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1596, 1891, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1597, 1892, 'FACULDADE ENIAC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1598, 1893, 'ESCOLA DE NEGCIOS DO ESTADO DA BAHIA - ENEB', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1599, 1894, 'FACULDADE EUROPIA DE ADMINISTRAO E MARKETING', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1600, 1895, 'FACULDADE CHRISTUS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1601, 1896, 'FACULDADE DE TURISMO DE NOVA ANDRADINA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1602, 1897, 'FACULDADE DE LETRAS DE NOVA ANDRADINA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1603, 1898, 'FACULDADE DO LITORAL SUL PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1604, 1899, 'FACULDADE DO NOROESTE PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1605, 1900, 'FACULDADE ANCHIETA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1606, 1901, 'FACULDADE CATLICA DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1607, 1902, 'FACULDADE SANTO AGOSTINHO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1608, 1903, 'FACULDADE CRISTO REI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1609, 1904, 'FACULDADE SO CAMILO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1610, 1905, 'FACULDADE SANTA ADAME', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1611, 1906, 'INSTITUTO DE ENSINO SUPERIOR MATERDEI', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1612, 1907, 'FACULDADE EDUCACIONAL DE COLOMBO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1613, 1908, 'ESCOLA DE ENSINO SUPERIOR ALTERNATIVO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1614, 1909, 'FACULDADE PEREIRA DE FREITAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1615, 1910, 'FACULDADE BAIANA DE CINCIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1616, 1911, 'FACULDADE CENTRO-OESTE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1617, 1912, 'INSTITUTO SUPERIOR DE EDUCAO JAPI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1618, 1913, 'FACULDADE DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1619, 1914, 'FACULDADE PITGORAS DE ENFERMAGEM DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1620, 1915, 'FACULDADE DE EDUCAO SILVA SERPA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1621, 1916, 'INSTITUTO SUPERIOR DE EDUCAO PITGORAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1622, 1917, 'FACULDADE DE CINCIAS BIOMDICAS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1623, 1918, 'FACULDADE CAPIVARI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1624, 1919, 'FACULDADE DE CINCIAS EXATAS E TECNOLGICAS SANTO AGOSTINHO - FACET', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1625, 1920, 'FACULDADE DE FARMCIA DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1626, 1921, 'FACULDADE DE SO BENTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1627, 1922, 'ESCOLA SUPERIOR DE FISIOTERAPIA - ESUFI', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1628, 1923, 'FACULDADE DE DIREITO PADRE ARNALDO JANSSEN', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1629, 1924, 'FACULDADE DO NORTE PIONEIRO DE JACAREZINHO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1630, 1925, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS SANTO AGOSTINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1631, 1926, 'FACULDADE DE ADMINISTRAO DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1632, 1927, 'FACULDADE DE CINCIAS BIOLGICAS E DA SADE DE UNIO DA VITRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1633, 1928, 'FACULDADE CENECISTA ILHA DO GOVERNADOR', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1634, 1929, 'FACULDADE DE CINCIAS CONTBEIS - FACICONT', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1635, 1930, 'INSTITUTO SUPERIOR DE EDUCAO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1636, 1931, 'INSTITUTO DE ENSINO SUPERIOR FRANCISCANO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1637, 1932, 'FACULDADE PALOTINA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1638, 1933, 'FACULDADE BIRIGUI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1639, 1934, 'FACULDADE SINOP', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1640, 1935, 'FACULDADE SO PAULO DE SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1641, 1936, 'FACULDADE DE LUCAS DO RIO VERDE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1642, 1937, 'FACULDADE EVANGLICA DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1643, 1938, 'FACULDADE GLOBAL DE UMUARAMA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1644, 1939, 'INSTITUTO DE ENSINO SUPERIOR DE LONDRINA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1645, 1940, 'FACULDADE DE NOVA SERRANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1646, 1941, 'FACULDADE CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1647, 1942, 'FACULDADE DE COMUNICAO SOCIAL DE FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1648, 1943, 'FACULDADE SERRANA DE ENSINO SUPERIOR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1649, 1944, 'FACULDADE DE TECNOLOGIA IBRATEC', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1650, 1945, 'FACULDADE DE SORRISO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1651, 1946, 'FACULDADE MATER', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1652, 1947, 'FACULDADE DE ENGENHARIA DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1653, 1948, 'FACULDADE DE ENSINO SUPERIOR DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1654, 1949, 'FACULDADE DE PAULNIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1655, 1950, 'FACULDADE DE ORLNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1656, 1951, 'INSTITUTO DE EDUCAO E ENSINO SUPERIOR DE SAMAMBAIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1657, 1952, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS APLICADAS DO ARAGUAIA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1658, 1953, 'FACULDADE DE CINCIAS BIOLGICAS E DA SADE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1659, 1954, 'FACULDADE DE ECONOMIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1660, 1955, 'FACULDADE DE CINCIAS HUMANAS DE LAGES', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1661, 1956, 'INSTITUTO BATISTA DE ENSINO SUPERIOR DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1662, 1957, 'FACULDADE DE TECNOLOGIA SENAI JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1663, 1958, 'FACULDADE DE TECNOLOGIA SENAI BLUMENAU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1664, 1959, 'FACULDADE DE CINCIAS JURDICAS DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1665, 1960, 'FACULDADE DE MEDICINA DO PLANALTO CENTRAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1666, 1961, 'INSTITUTO TECNOLGICO E DAS CINCIAS SOCIAIS APLICADAS E DA SADE DO CENTRO EDUC. N. SR AUXILIADORA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1667, 1962, 'INSTITUTO SUPERIOR DE EDUCAO DO CENTRO EDUCACIONAL NOSSA SENHORA AUXILIADORA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1668, 1963, 'FACULDADE DE FISIOTERAPIA DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1669, 1964, 'FACULDADE DE TECNOLOGIA EM HOTELARIA, GASTRONOMIA E TURISMO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1670, 1965, 'FACULDADE DE TECNOLOGIA DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1671, 1966, 'FACULDADE ALBERT EINSTEIN', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1672, 1967, 'FACULDADE DO VALE DO ITAPECUR', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1673, 1968, 'FACULDADE ATLNTICO', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1674, 1969, 'FACULDADE DOS IMIGRANTES - FAI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1675, 1970, 'FACULDADE DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1676, 1971, 'FACULDADE DE TECNOLOGIA MORUMBI SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1677, 1972, 'FACULDADE TIJUCUSSU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1678, 1973, 'INSTITUTO SUPERIOR DE EDUCAO ORGENES LESSA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1679, 1974, 'INSTITUTO SUPERIOR DE EDUCAO DE CAXIAS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1680, 1975, 'FACULDADE DE CINCIAS AGRRIAS DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1681, 1976, 'INSTITUTO SUPERIOR DE EDUCAO DE LINHARES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1682, 1977, 'CENTRO DE ESTUDOS SUPERIORES APRENDIZ', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1683, 1978, 'FACULDADE METROPOLITANA DA GRANDE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1684, 1979, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1685, 1980, 'FACULDADE SINERGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1686, 1981, 'FACULDADE MILLENNIUM', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1687, 1982, 'FACULDADE DE COMUNICAO SOCIAL DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1688, 1983, 'FACULDADE DE CINCIAS E DA SADE DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1689, 1984, 'FACULDADE DE CINCIAS GERENCIAIS DE MANHUAU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1690, 1985, 'FACULDADE COMUNITRIA DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1691, 1986, 'FACULDADE ESTCIO DE S DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1692, 1987, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS DEPUTADO JOS LAVIOLA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1693, 1988, 'FACULDADE AVANTIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1694, 1989, 'INSTITUTO SUPERIOR DE EDUCAO AVANTIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1695, 1990, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1696, 1991, 'FACULDADE JOO PAULO PRIMEIRO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1697, 1992, 'INSTITUTO DE ENSINO SUPERIOR DE PIEDADE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1698, 1993, 'FACULDADE DE CINCIAS HUMANAS E DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1699, 1994, 'INSTITUTO DE ENSINO SUPERIOR DE DOURADOS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1700, 1995, 'FACULDADE DE MEDICINA NOVA ESPERANA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1701, 1996, 'INSTITUTO CUIAB DE ENSINO E CULTURA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1702, 1997, 'INSTITUTO DE ENSINO SUPERIOR DE ARIPUAN', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1703, 1998, 'INSTITUTO SUPERIOR DE EDUCAO ELVIRA DAYRELL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1704, 1999, 'JOHN KNOX', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1705, 2000, 'FACULDADE TATUAP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1706, 2001, 'SEMINRIO ADVENTISTA LATINO-AMERICANO DE TEOLOGIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1707, 2002, 'INSTITUTO SUPERIOR DE EDUCAO DIVINO SALVADOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1708, 2003, 'FACULADE TEOLGICA SINAL', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1709, 2004, 'FACULDADE DE DIREITO PRESIDENTE CASTELO BRANCO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1710, 2005, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE SINOP', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1711, 2006, 'INSTITUTO SUPERIOR DE EDUCAO DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1712, 2007, 'FACULDADE DE CINCIAS EXATAS E TECNOLGICAS DO BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1713, 2008, 'INSTITUTO SUPERIOR DE EDUCAO DE SO GOTARDO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1714, 2009, 'FACULDADE DE TAQUARITINGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1715, 2010, 'FACULDADE DE CINCIAS E LETRAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1716, 2011, 'INSTITUTO NACIONAL DE EDUCAO DE PORTO SEGURO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1717, 2012, 'INSTITUTO SUPERIOR DE EDUCAO DE PESQUEIRA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1718, 2013, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1719, 2014, 'FACULDADE SETELAGOANA DE CINCIAS GERENCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1720, 2015, 'INSTITUTO SANTO TOMS DE AQUINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1721, 2016, 'LOGOS FACULDADE', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1722, 2017, 'FACULDADES DE ALTA FLORESTA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1723, 2018, 'FACULDADE EVOLUO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1724, 2019, 'INSTITUTO SUPERIOR DE EDUCAO DE SALGUEIRO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1725, 2020, 'INSTITUTO DE EDUCAO SUPERIOR EUROBRAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1726, 2021, 'FACULDADES EUROBRAS - ESTUDOS UNIVERSITRIOS REGIONAIS BRASILEIROS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1727, 2022, 'FACULDADE DE ROLIM DE MOURA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1728, 2023, 'FACULDADE DE GUANAMBI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1729, 2024, 'INSTITUTO SUPERIOR DE EDUCAO DAS CLARETIANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1730, 2025, 'INSTITUTO SUPERIOR DE EDUCAO CARLI GIOVANNI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1731, 2026, 'INSTITUTO SUPERIOR DE EDUCAO BRASIL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1732, 2027, 'FACULDADE DE CAMPINA GRANDE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1733, 2028, 'FALCUDADE DE CINCIAS ECONMICAS, CONTBEIS E ADMINISTRATIVAS DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1734, 2029, 'INSTITUTO SUPERIOR DE EDUCAO DE MARECHAL CNDIDO RONDON', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1735, 2030, 'INSTITUTO SUPERIOR DE EDUCAO CAMPO LIMPO PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1736, 2031, 'FACULDADE DE EDUCAO DE PRESIDENTE MDICI', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1737, 2032, 'FACULDADE DE PEDAGOGIA E FORMAO DE NORMALISTAS DE ARAGUANA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1738, 2033, 'INSTITUTO SUPERIOR DE EDUCAO DE FLORESTA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1739, 2034, 'FACULDADE DE CINCIAS CONTBEIS DE ARAGUANA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1740, 2035, 'FACULDADE JAGUARIAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1741, 2036, 'FACULDADE DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1742, 2037, 'FACULDADE UNIME DE CINCIAS EXATAS E TECNOLGICAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1743, 2038, 'INSTITUTO DE EDUCAO SUPERIOR VALE DO MEARIM', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1744, 2039, 'FACULDADE CIDADE LUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1745, 2040, 'FACULDADE DO FUTURO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1746, 2041, 'FACULDADE SUDAMRICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1747, 2042, 'FACULDADE FIGUEIREDO COSTA - FIC', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1748, 2043, 'FACULDADE VALE DO SALGADO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1749, 2044, 'INSTITUTO SUPERIOR DE EDUCAO VALE DO SALGADO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1750, 2045, 'FACULDADE AMADEUS', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1751, 2046, 'INSTITUTO SUPERIOR DE EDUCAO AMADEUS', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1752, 2047, 'FACULDADE DE PSICOLOGIA DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1753, 2048, 'FACULDADE DE CINCIAS E TECNOLOGIA DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1754, 2049, 'INSTITUTO DE ESTUDOS SUPERIORES DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1755, 2050, 'ESCOLA SUPERIOR DE GESTO E CINCIAS DA SADE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1756, 2051, 'FACULDADE INTERFUTURA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1757, 2052, 'FACULDADE DE DIREITO DE LAGES', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1758, 2053, 'FACULDADE PARA O DESENVOLVIMENTO DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1759, 2054, 'FACULDADE DE EDUCAO DE NOVA ANDRADINA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1760, 2055, 'FACULDADE ARAUCRIA DO SUDOESTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1761, 2056, 'FACULDADE DA ALTA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1762, 2057, 'FACULDADE DE MOGI GUAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1763, 2058, 'FACULDADE METROPOLITANA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1764, 2059, 'FACULDADE DE EDUCAO E CINCIAS DE TAGUATINGA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1765, 2060, 'FACULDADE PADRE SEVERIANO LTDA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1766, 2061, 'FACULDADE DE FORMAO DE PROFESSORES DE ARAPIRACA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1767, 2062, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE VARGINHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1768, 2063, 'INSTITUTO NOSSA SENHORA APARECIDA DE EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1769, 2064, 'FACULDADE METROPOLITANA DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1770, 2065, 'FACULDADE DE FILOSOFIA E TEOLOGIA DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1771, 2066, 'FACULDADE LEONARDO DA VINCI', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1772, 2067, 'FACULDADE DE CINCIAS EMPRESARIAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1773, 2068, 'FACULDADE DE CASTILHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1774, 2069, 'FACULDADE DE TEOLOGIA E CINCIAS HUMANAS', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1775, 2070, 'FACULDADE DE CINCIAS SOCIAIS AMORIL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1776, 2071, 'FACULDADE TEOLGICA BATISTA DO BRASIL', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1777, 2072, 'FACULDADE DE DESENVOLVIMENTO SUSTENTVEL DE CRUZEIRO DO SUL', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1778, 2073, 'INSTITUTO SUPERIOR DE EDUCAO PADRO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1779, 2074, 'INSTITUTO EDUCACIONAL SO JOO DA ESCIA DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1780, 2075, 'INSTITUTO DE ENSINO SUPERIOR DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1781, 2076, 'FACULDADE REGIONAL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1782, 2077, 'FACULDADE ANGLO-AMERICANO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1783, 2078, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1784, 2079, 'FACULDADE DE ITAITUBA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1785, 2080, 'FACULDADE UNIVRSICA DE FILOSOFIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1786, 2081, 'FACULDADE DE CINCIAS GERENCIAS DE PEDREIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1787, 2082, 'FACULDADE DE CINCIAS MDICAS DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1788, 2083, 'FACULDADE DE IBITINGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1789, 2084, 'FACULDADE CENECISTA DE NOVA PETRPOLIS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1790, 2085, 'INSTITUTO BAIANO DE ENSINO SUPERIOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1791, 2086, 'INSTITUTO DE ENSINO SUPERIOR DE FOZ DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1792, 2087, 'FACULDADE MARECHAL RONDON', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1793, 2088, 'INSTITUTO SUPERIOR DE EDUCAO SANTO AGOSTINHO DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1794, 2089, 'FACULDADE CENECISTA DE MOSSOR', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1795, 2090, 'FACULDADE DE TURISMO DE CONGONHAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1796, 2091, 'INSTITUTO SUPERIOR DE EDUCAO PR-SABER', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1797, 2092, 'FACULDADE DOM PEDRO II', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1798, 2093, 'FACULDADE GREGOR MENDEL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1799, 2094, 'FACULDADE DE ENSINO A', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1800, 2095, 'INSITUTO SUPERIOR DE JACARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1801, 2096, 'FACULDADE DE CINCIAS JURDICAS E GERENCIAIS DE MANHUAU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1802, 2097, 'ESCOLA SUPERIOR DE GUARARAPES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1803, 2098, 'FACULDADE KURIOS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1804, 2099, 'INSTITUTO DE ENSINO, PESQUISA E EXTENSO DO BRASIL OESTE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1805, 2100, 'FACULDADE DE LETRAS DO SERRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1806, 2101, 'FACULDADE ASCENSO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1807, 2102, 'FACULDADE DE AURIFLAMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1808, 2103, 'FACULDADE E CENTRO DE ENSINO DE LONDRINA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1809, 2104, 'FACULDADE DE TEOLOGIA DE HOKEMH', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1810, 2105, 'FACULDADE ADVENTISTA DE INFORMTICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1811, 2106, 'FACULDADE DE VIOSA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1812, 2107, 'FACULDADE IBTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1813, 2108, 'FACULDADE DE DIREITO DA SEDES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1814, 2109, 'FACULDADE JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1815, 2110, 'FACULDADE DOM CSAR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1816, 2111, 'INSTITUTO SUPERIOR DE TEOLOGIA APLICADA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1817, 2112, 'FACULDADE METODISTA DE BIRIGI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1818, 2113, 'FACULDADE DOM BOSCO DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1819, 2114, 'FACULDADE DE TEOLOGIA FILADLFIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1820, 2115, 'FACULDADE JOO PAULO II', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1821, 2116, 'FACULDADE IGUAPENSE SANTO AUGUSTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1822, 2117, 'FACULDADE DE CINCIA E TECNOLOGIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1823, 2118, 'INSTITUTO DE EDUCAO SUPERIOR GUAIANS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1824, 2119, 'FACULDADE GUARARAPES', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1825, 2120, 'INSTITUTO DE FILOSOFIA E DE TEOLOGIA PAULO VI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1826, 2121, 'FACULDADE DE INFORMTICA DE PEDREIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1827, 2122, 'FACULDADE DE SERGIPE', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1828, 2123, 'FACULDADE MAX PLANCK', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1829, 2124, 'INSTITUTO SUPERIOR DE EDUCAO DE PARASO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1830, 2125, 'FACULDADE DE EDUCAO DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1831, 2126, 'ESCOLA DE DIREITO DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1832, 2127, 'FACULDADE BATISTA DE EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1833, 2128, 'ESCOLA DE DIREITO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1834, 2129, 'ESCOLA DE ECONOMIA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1835, 2130, 'FACULDADE EVANGLICA DE TEOLOGIA DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1836, 2131, 'FACULDADE FAPAN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1837, 2132, 'FACULDADE BARO DO RIO BRANCO', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1838, 2133, 'FACULDADE DE CINCIAS, EDUCAO E TEOLOGIA DO NORTE DO BRASIL', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1839, 2134, 'FACULDADE DA IGREJA MINISTRIO FAMA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1840, 2135, 'INSTITUTO FILOSFICO TEOLGICO NOSSA SENHORA IMACULADA RAINHA DO SERTO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1841, 2136, 'INSTITUTO DE JUNA DE ENSINO SUPERIOR', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1842, 2137, 'FACULDADE SO MARCOS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1843, 2138, 'INSTITUTO SUPERIOR DE EDUCAO DIVINO SALVADOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1844, 2139, 'FACULDADE DE INFORMTICA DE PARANABA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1845, 2140, 'INSTITUTO SUPERIOR DE EDUCAO NOSSA SENHORA DE LOURDES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1846, 2141, 'FACULDADE TEOLGICA BATISTA DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1847, 2142, 'FACULDADE MULTIEDUCATIVA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1848, 2143, 'INSTITUTO SUPERIOR DE EDUCAO DE DUQUE DE CAXIAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1849, 2144, 'FACULDADE ECOPORANGA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1850, 2145, 'FACULDADE INFRIUM DE TECNOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1851, 2146, 'FACULDADE DO ACRE', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1852, 2147, 'FACULDADE METROPOLITANA DE MANAUS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1853, 2148, 'INSTITUTO PALMAS DE ENSINO SUPERIOR', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1854, 2149, 'INSTITUTO MATO GROSSO DO SUL DE EDUCAO E CULTURA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1855, 2150, 'INSTITUTO TERESINA DE ENSINO E CULTURA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1856, 2151, 'INSTITUTO SUPERIOR DE EDUCAO BERLAAR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1857, 2152, 'FACULDADE DE ADMINISTRAO E TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1858, 2153, 'FACULDADE DE DIREITO DE MACA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1859, 2154, 'INSTITUTO SUPERIOR DE CINCIAS HUMANAS E SOCIAIS ANSIO TEIXEIRA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1860, 2155, 'FACULDADE DE CINCIAS SOCIAIS E DA SADE DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1861, 2156, 'FACULDADE SO TOMS DE AQUINO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1862, 2157, 'INSTITUTO SUPERIOR DE EDUCAO SANTANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1863, 2158, 'FACULDADE SANTA MARIA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1864, 2160, 'FACULDADE SANT''ANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1865, 2161, 'FACULDADE TEOLGICA DE VALENA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1866, 2162, 'INSTITUTO METROPOLITANO DE ENSINO SUPERIOR DE VESPASIANO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1867, 2163, 'INSTITUTO SUPERIOR DE EDUCAO EUGNIO GOMES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1868, 2164, 'FACULDADE PANAMERICANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1869, 2165, 'INSTITUTO FOZ DO IGUAU DE ENSINO E CULTURA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1870, 2166, 'INSTITUTO DE EDUCAO E CULTURA INTEGRADA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1871, 2167, 'FACULDADE DE EDUCAO E CINCIAS GERENCIAIS DE SALTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1872, 2168, 'INSTITUTO CAMPO GRANDE DE ENSINO SUPERIOR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1873, 2169, 'FACULDADE CENECISTA DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1874, 2170, 'FACULDADE DE MATEMTICA JESUS MARIA JOS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1875, 2171, 'INSTITUTO BELO HORIZONTE DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1876, 2172, 'FACULDADE CENECISTA DE TRS RIOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1877, 2173, 'FACULDADE GUAIANS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1878, 2174, 'INSTITUTO SANTA CATARINA DE EDUCAO E CULTURA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1879, 2175, 'FACULDADE METROPOLITANA DE BLUMENAU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1880, 2176, 'FACULDADE DE CINCIAS BIOMDICAS DE COLATINA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1881, 2177, 'INSTITUTO PAULISTA DE ENSINO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1882, 2178, 'FACULDADE CENECISTA DE MACA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1883, 2179, 'FACULDADE DE TURISMO DO ESTADO DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1884, 2180, 'RATIO - FACULDADE TEOLGICA E FILOSFICA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1885, 2181, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE CAPIVARI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1886, 2182, 'FACULDADE BATISTA IDA NELSON', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1887, 2183, 'CENTRO UNIVERSITRIO FUNDAO SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1888, 2184, 'FACULDADE CENECISTA NOSSA SENHORA DOS ANJOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1889, 2185, 'FACULDADE DE NUTRIO DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1890, 2186, 'FACULDADE SALESIANA DOM BOSCO', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1891, 2187, 'FACULDADE DE CINCIAS GERENCIAIS DE VALINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1892, 2188, 'INSTITUTO SUPERIOR DE EDUCAO NOSSA SENHORA DE SION', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1893, 2189, 'FACULDADE DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1894, 2190, 'FACULDADE JLIO CAMPOS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1895, 2191, 'FACULDADE ATLNTICO SUL DE PELOTAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1896, 2192, 'INSTITUTO SUPERIOR DE EDUCAO IVOTI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1897, 2193, 'INSTITUTO SUPERIOR DE EDUCAO IDEAL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1898, 2194, 'FACULDADE ATLNTICO SUL DO RIO GRANDE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1899, 2195, 'FACULDADE DE CINCIAS GERENCIAIS DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1900, 2196, 'FACULDADE DE INFORMTICA E TECNOLOGIAS VARZEA-GRANDENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1901, 2197, 'INSTITUTO SUPERIOR DE EDUCAO SANTA MARINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1902, 2198, 'FACULDADE MONTEIRO LOBATO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1903, 2199, 'FACULDADE DE EDUCAO E TEOLOGIA DO BRASIL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1904, 2200, 'FACULDADE VALE DO GORUTUBA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1905, 2201, 'FACULDADE ALPHA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1906, 2202, 'INSTITUTO SUPERIOR DE EDUCAO DE GUARATUBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1907, 2203, 'FACULDADE DE EDUCAO ANCHIETA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1908, 2204, 'FACULDADE CATARINENSE DOM BOSCO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1909, 2205, 'FACULDADE DE TUPI PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1910, 2206, 'FACULDADE PILARES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1911, 2207, 'FACULDADE SO JUDAS TADEU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1912, 2208, 'FACULDADE LUS DE CAMES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1913, 2209, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1914, 2210, 'FACULDADE GARAVELO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1915, 2211, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE FARROUPILHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1916, 2212, 'FACULDADE UNIDADE DAS AMRICAS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1917, 2213, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1918, 2214, 'FACULDADE DE ENGENHARIA DA TERRA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1919, 2215, 'FACULDADE DE TEOLOGIA DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1920, 2216, 'INSTITUTO SUPERIOR DE EDUCAO DE INA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1921, 2217, 'FACULDADE DE CINCIAS GERENCIAIS DE INA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1922, 2218, 'FACULDADE DE CINCIAS EXATAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1923, 2219, 'FACULDADE MUNDO MODERNO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1924, 2220, 'FACULDADE DE CINCIAS JURDICAS DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1925, 2221, 'FACULDADE MADEIRA MAMOR', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1926, 2222, 'FACULDADE SETE DE SETEMBRO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1927, 2223, 'FACULDADE DE CINCIAS SOCIAIS E DA SADE DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1928, 2224, 'FACULDADES INTEGRADAS DE IMPERATRIZ', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1929, 2225, 'INSTITUTO SUPERIOR DE FORMAO DE PROFESSORES', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1930, 2226, 'FACULDADE DE CINCIAS DA SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1931, 2227, 'FACULDADE DE CINCIAS BIOLGICAS JK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1932, 2228, 'FACULDADE DA SADE HUMANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1933, 2229, 'FACULDADE VICTOR HUGO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1934, 2230, 'FACULDADE DE ADMINISTRAO JK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1935, 2231, 'FACULDADE DE COMUNICAO SOCIAL JK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1936, 2232, 'FACULDADE DE ADMINISTRAO E MARKETING DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1937, 2233, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1938, 2234, 'FACULDADE DE PEDAGOGIA DE BETIM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1939, 2235, 'FACULDADE TEOLGICA DAS ASSEMBLIAS DE DEUS EM SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1940, 2236, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE SO BERNARDO DO CAMPO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1941, 2237, 'FACULDADE TEOLGICA BATISTA EQUATORIAL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1942, 2238, 'FACULDADE DE PSICOLOGIA DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1943, 2239, 'FACULDADE DE DIREITO DE VILHA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1944, 2240, 'INSTITUTO CEAR DE ENSINO E CULTURA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1945, 2241, 'INSTITUTO SUDESTE MINEIRO DE EDUCAO E CULTURA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1946, 2242, 'INSTITUTO PERNAMBUCO DE ENSINO E CULTURA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1947, 2243, 'INSTITUTO PARABA DE EDUCAO E CULTURA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1948, 2244, 'INSTITUTO MACEI DE ENSINO E CULTURA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1949, 2245, 'INSTITUTO VITRIA DE ENSINO E CULTURA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1950, 2246, 'FACULDADE UNIO AMERICANA', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1951, 2247, 'FACULDADE CHAFIC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1952, 2248, 'INSTITUTO SERGIPE DE ENSINO SUPERIOR', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1953, 2249, 'FACULDADE LOGOS EM ARACAJU', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1954, 2250, 'INSTITUTO SANTO TOMS DE AQUINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1955, 2251, 'FACULDADE DE POMPIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1956, 2252, 'FACULDADE DE CINCIA DA EDUCAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1957, 2253, 'FACULDADE DE DIREITO DE PRIMAVERA DO LESTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1958, 2254, 'FACULDADE DE CINCIAS GERENCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1959, 2255, 'INSTITUTO DE ENSINO SUPERIOR DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1960, 2256, 'FACULDADE CATLICA STELLA MARIS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1961, 2257, 'FACULDADE DIADEMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1962, 2258, 'FACULDADE DE PRIMEIRO MUNDO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1963, 2259, 'INSTITUTO SUPERIOR EDUCAO LEONARDO DA VINCI', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1964, 2260, 'FACULDADE DE TECNOLOGIA IESVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1965, 2261, 'FACULDADE DE CINCIAS HUMANAS E DA SADE DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1966, 2262, 'INSTITUTO DE EDUCAO NORMAL SUPERIOR DE PIMENTA BUENO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1967, 2263, 'FACULDADE SANTO ANTNIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1968, 2264, 'FACULDADE DE COMUNICAO SOCIAL DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1969, 2265, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS DE IBIRAMA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1970, 2266, 'FACULDADE CAMBURY DE FORMOSA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1971, 2267, 'FACULDADE DE SADE DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1972, 2268, 'FACULDADE DE TEOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1973, 2269, 'INSTITUTO SUPERIOR DE EDUCAO LMINI', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1974, 2270, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE EXTREMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1975, 2271, 'FACULDADE PITGORAS DE IPATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1976, 2272, 'FACULDADE DE PSICOLOGIA DE LAGES', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1977, 2273, 'INSTITUTO SUPERIOR DE EDUCAO DO ALTO SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1978, 2274, 'FACULDADE CIDADANIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1979, 2275, 'FACULDADE DE DIREITO SANTO AGOSTINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1980, 2276, 'FACULDADE DE DIREITO DE VALINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1981, 2277, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE UNA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1982, 2278, 'FACULDADE IGARATA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1983, 2279, 'FACULDADE INTEGRADA METROPOLITANA DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1984, 2280, 'FACULDADE SUPERIOR DE RIBAS DO RIO PARDO', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1985, 2281, 'FACULDADE DE FILOSOFIA SO BOAVENTURA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1986, 2282, 'FACULDADE DE CAFELNDIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1987, 2283, 'FACULDADE SO JOS DA COSTA VERDE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1988, 2284, 'FACULDADE DE FILOSOFIA, CNCIAS E LETRAS DE FAXINAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1989, 2285, 'FACULDADE DE PEDAGOGIA DE DORES DO INDAI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1990, 2286, 'FACULDADE DE BOM JESUS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1991, 2287, 'ESCOLA SUPERIOR DE TEOLOGIA E ESPIRITUALIDADE FRANCISCANA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1992, 2288, 'FACULDADE ALMEIDA RODRIGUES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1993, 2289, 'FACULDADES INTEGRADAS DE ANGELES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1994, 2290, 'FACULDADE DE EDUCAO, CINCIAS E HUMANIDADES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1995, 2291, 'INSTITUTO SUPERIOR DE EDUCAO DE BEBEDOURO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1996, 2292, 'INSTITUTO DE EDUCAO NORMAL SUPERIOR', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1997, 2293, 'FACULDADE DE CINCIAS HUMANAS, BIOLGICAS E DA SADE', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1998, 2294, 'FACULDADE DE CINCIAS APLICADAS DA REGIO CENTRO SUL DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (1999, 2295, 'INSTITUTO MEDICENSE DE EDUCAO SUPERIOR', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2000, 2296, 'FACULDADE DE LETRAS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2001, 2297, 'FACULDADE DE DIREITO DE SANTA MARIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2002, 2298, 'ESCOLA SUPERIOR DE DIREITO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2003, 2299, 'FACULDADE ESPECIAL DE EDUCAO E CULTURA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2004, 2300, 'INSTITUTO DE EDUCAO SUPERIOR DE ROLIM DE MOURA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2005, 2301, 'FACULDADE FIDELIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2006, 2302, 'FACULDADE PORTO SEGURO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2007, 2303, 'FACULDADE CIMO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2008, 2304, 'INSTITUTO DE ENSINO SUPERIOR DA FUNLEC DE BONITO', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2009, 2305, 'FACULDADE DE DIREITO DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2010, 2306, 'FACULDADE RIO BRANCO DO SUL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2011, 2307, 'INSTITUTO SUPERIOR DE EDUCAO DE ITABIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2012, 2308, 'FACULDADE GEREMRIO DANTAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2013, 2309, 'FACULDADE NOVO MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2014, 2310, 'FACULDADE DA REGIO DOS VALES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2015, 2311, 'FACULDADE DE CINCIAS JURDICAS PROFESSOR ALBERTO DEODATO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2016, 2312, 'FACULDADE LUTERANA RUI BARBOSA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2017, 2313, 'FACULDADE DE TEOLOGIA DE BOA VISTA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2018, 2314, 'INSTITUTO DE ENSINO SUPERIOR TECNOLGICO DE LONDRINA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2019, 2315, 'FACULDADE CEARENSE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2020, 2316, 'INSTITUTO NACIONAL DE DESENVOLVIMENTO DO ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2021, 2317, 'FACULDADE DO DESCOBRIMENTO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2022, 2318, 'FACULDADE DOM BOSCO DE NOVA AURORA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2023, 2319, 'ESCOLA SUPERIOR DE EDUCAO CORPORATIVA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2024, 2320, 'INSTITUTO SUPERIOR DE EDUCAO ATENEU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2025, 2321, 'FACULDADE INESP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2026, 2322, 'FACULDADE ATENEU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2027, 2323, 'FACULDADE DA AMAZNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2028, 2324, 'FACULDADE ANITA GARIBALDI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2029, 2325, 'INSTITUTO SUPERIOR DE EDUCAO DA AMAZNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2030, 2326, 'FACULDADE NOVO MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2031, 2327, 'FACULDADE SERRANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2032, 2328, 'FACULDADE ITEANA DE BOTUCATU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2033, 2329, 'FACULDADE BATISTA DO NORDESTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2034, 2330, 'FACULDADE PARA O DESENVOLVIMENTO DO ESTADO E DO PANTANAL MATO-GROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2035, 2331, 'ESCOLA SUPERIOR DE TEOLOGIA E FILOSOFIA DO BRASIL', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2036, 2332, 'FACULDADE CORPORATIVA CESPI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2037, 2333, 'FACULDADE DE TEOLOGIA MOKITI OKADA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2038, 2334, 'INSTITUTO SUPERIOR MENDES DE ALMEIDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2039, 2335, 'FACULDADE JOAQUIM MACHADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2040, 2336, 'FACULDADE MONTES BELOS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2041, 2337, 'INSTITUTO SUPERIOR DE EDUCAO DE MONTES BELOS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2042, 2338, 'INSTITUTO BRASILEIRO DE ESTUDOS TRIBUTRIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2043, 2339, 'FACULDADE INTERNACIONAL DE HOTELARIA E TURISMO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2044, 2340, 'FACULDE ALPHAVILLE DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2045, 2341, 'FACULDADE DA ALDEIA DE CARAPICUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2046, 2342, 'INSTITUTO SUPERIOR DE EDUCAO CERES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2047, 2343, 'FACULDADE DA AMAZNIA OCIDENTAL - FAAO', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2048, 2344, 'FACULDADE ITEANA DE IBITINGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2049, 2345, 'FACULDADE SETE DE SETEMBRO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2050, 2346, 'INSTITUTO SUPERIOR DE FILOSOFIA BERTHIER', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2051, 2347, 'FACULDADE GENNARI & PEARTREE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2052, 2348, 'INSTITUIO DE ENSINO SO FRANCISCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2053, 2349, 'DINMICO FACULDADE DE FLORIANO LTDA.', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2054, 2350, 'ESCOLA SUPERIOR MADRE CELESTE', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2055, 2351, 'FACULDADE ESPRITO SANTENSE DE CINCIAS JURDICAS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2056, 2352, 'INSTITUTO DE EDUCAO SUPERIOR PIO XII', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2057, 2353, 'FACULDADE DE ADMINISTRAO DO CENTRO EDUCACIONAL DE SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2058, 2354, 'INSTITUTO SUPERIOR DE EDUCAO DO BRASIL', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2059, 2355, 'FACULDADE ANHANGUERA DE MATO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2060, 2356, 'FACULDADE PAN AMERICANA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2061, 2357, 'FACULDADE DE CINCIAS BIOLGICAS E DA SADE DO CESCARELLI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2062, 2358, 'INSTITUTO SUPERIOR DE EDUCAO DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2063, 2359, 'INSTITUTO SUPERIOR DE EDUCAO NOVO MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2064, 2360, 'INSTITUTO SUPERIOR DE EDUCAO MESTRE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2065, 2361, 'FACULDADE MESTRA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2066, 2362, 'FACULDADE DE CINCIAS MDICAS DE CAMPINA GRANDE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2067, 2363, 'FACULDADE NOVO MILNIO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2068, 2364, 'FACULDADE DE ADMINISTRAO DE EMPRESAS DE PARASO DO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2069, 2365, 'FACULDADE CATLICA DO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2070, 2366, 'FACULDADE DE SISTEMAS DE INFORMAO DE PARASO DO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2071, 2367, 'INSTITUTO SUPERIOR DE EDUCAO NOVO MILNIO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2072, 2368, 'FACULDADE DE ODONTOLOGIA SO LEOPOLDO MANDIC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2073, 2369, 'INSTITUTO DE ENSINO SUPERIOR SANTO ANTNIO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2074, 2370, 'INSTITUTO SUPERIOR DE EDUCAO NOVO MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2075, 2371, 'INSTITUTO DE EDUCAO SUPERIOR SO FRANCISCO DE ASSIS DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2076, 2372, 'INSTITUTO DE CINCIAS GERENCIAIS DE BRASLIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2077, 2373, 'FACULDADE TELES PIRES', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2078, 2374, 'FACULDADES INTEGRADAS CONTEMPORNEAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2079, 2375, 'FACULDADE VIZCAYA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2080, 2376, 'INSTITUTO SUPERIOR DE EDUCAO DE BRASLIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2081, 2377, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA PRESIDENTE KENNEDY', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2082, 2378, 'FACULDADE DE ADMINISTRAO DE CAMPINA VERDE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2083, 2379, 'INSTITUTO SUPERIOR DE EDUCAO DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2084, 2380, 'FACULDADE DO AMAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2085, 2381, 'INSTITUTO DE ENSINO E CULTURA DE DOURADOS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2086, 2382, 'INSTITUTO SUPERIOR DE EDUCAO DE TEOFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2087, 2383, 'FACULDADE MERIDIONAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2088, 2384, 'FACULDADE INTEGRADA DO BRASIL', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2089, 2385, 'INSTITUTO SUPERIOR DE EDUCAO NOVO MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2090, 2386, 'FACULDADE SUL RIOGRANDENSE', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2091, 2387, 'FACULDADES INTEGRADAS POSIVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2092, 2388, 'FACULDADE NOVO MILNIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2093, 2389, 'FACULDADE DE FILOSOFIA E TEOLOGIA PAULO VI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2094, 2390, 'INSTITUTO BRUSQUENSE DE ENSINO SUPERIOR', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2095, 2391, 'INSTITUTO SUPERIOR DE EDUCAO NOVO MILNIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2096, 2392, 'INSTITUTO SUPERIOR DE EDUCAO DE SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2097, 2393, 'FACULDADE MARISTA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2098, 2394, 'FACULDADE DE INFORMTICA DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2099, 2395, 'FACULDADE DE CAMPO VERDE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2100, 2396, 'INSTITUTO DE EDUCAO SUPEIOR DO SERTO EM PO DE ACAR', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2101, 2397, 'FACULDADE FUCAPE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2102, 2398, 'INSTITUTO SUPERIOR DE EDUCAO DE CAMPO VERDE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2103, 2399, 'FACULDADE DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2104, 2400, 'FACULDADE DOM BOSCO DE MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2105, 2401, 'INSTITUTO SUPERIOR DE EDUCAO DE CAMPO BELO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2106, 2402, 'FACULDADE CIDADE DO SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2107, 2403, 'FACULDADE SALESIANA DE PINDAMONHANGABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2108, 2404, 'FACULDADES INTEGRADAS DE FILOSOFIA E CINCIAS DO PLANALTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2109, 2405, 'INSTITUTO SUPERIOR DE EDUCAO DE SANTA LUZIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2110, 2406, 'FACULDADE DE GESTO E EMPREENDEDORISMO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2111, 2407, 'FACULDADES INTEGRADAS FACVEST', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2112, 2408, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2113, 2409, 'FACULDADE DO AGRESTE DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2114, 2410, 'FACULDADE CEARENSE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2115, 2411, 'FACULDADE DA CIDADE DO NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2116, 2412, 'FACULDADE ECONOROESTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2117, 2413, 'FACULDADE DE ENSINO SUPERIOR DE FLORIANO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2118, 2414, 'INSTITUTO SUPERIOR DE EDUCAO IESDE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2119, 2415, 'FACULDADE BOM JESUS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2120, 2416, 'INSTITUTO SUPERIOR DE EDUCAO DE BOTUCATU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2121, 2417, 'INSTITUTO SUPERIOR DE EDUCAO METROPOLITANO DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2122, 2418, 'FACULDADES INTEGRADAS DO RECNCAVO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2123, 2419, 'FACITALGO - FACULDADE DE CIENCIAS E TECNOLOGIA DE AGUAS LINDAS DE GOIAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2124, 2420, 'INSTITUTO DE ENSINO E CULTURA DO PARAN - IECP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2125, 2421, 'INSTITUTO DE ENSINO SUPERIOR', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2126, 2422, 'FACULDADE DE FILOSOFIA LETRAS E ARTES DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2127, 2423, 'FACULDADE DE CINCIA E TECNOLOGIA DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2128, 2424, 'FACULDADE DE SISTEMA DE INFORMAO DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2129, 2425, 'FACULDADE DE JIPARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2130, 2426, 'FACULDADE INTEGRADA BRASIL AMAZONIA - FIBRA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2131, 2427, 'FACULDADE DE CINCIAS GERENCIAIS DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2132, 2428, 'FACULDADE DE CINCIAS DA SADE DE CAMPOS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2133, 2429, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2134, 2430, 'FACULDADE CATLICA ORTODOXA SEDES SAPIENTIAM', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2135, 2431, 'FACULDADE MONTESSORI DE IBINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2136, 2432, 'FACULDADE ALPHAVILLE DE JUNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2137, 2433, 'INSTITUTO SUPERIOR DE EDUCAO PORTO SEGURO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2138, 2434, 'FACULDADE AMRICA LATINA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2139, 2435, 'FACULDADE DE CINCIAS CONTBEIS LUIZ MENDES', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2140, 2436, 'FACULDADE THIRIH', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2141, 2437, 'FACULDADE DE TEIXEIRA DE FREITAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2142, 2438, 'INSTITUTO TEOLGICO PASTORAL DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2143, 2439, 'FACULDADE OCIDEMNTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2144, 2440, 'FACULDADE CIDADE DE JOO PINHEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2145, 2441, 'INSTITUTO SUPERIOR DE EDUCAO AUXILIUM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2146, 2442, 'FACULDADE DE CINCIAS BIOMDICAS DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2147, 2443, 'INSTITUTO SUPERIOR DE EDUCAO DE JANABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2148, 2444, 'FACULDADE JOO CALVINO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2149, 2445, 'FACULDADE PADRO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2150, 2446, 'INSTITUTO SUPERIOR DE EDUCAO PADRO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2151, 2447, 'INSTITUTO EDUCACIONAL DE CASTRO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2152, 2448, 'INSTITUTO SUPERIOR DE EDUCAO DE JANURIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2153, 2449, 'INSTITUTO MATO GROSSO DE EDUCAO E CULTURA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2154, 2450, 'FACULDADE DE DIREITO PROMOVE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2155, 2451, 'FACULDADE IPIRANGA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2156, 2452, 'INSTITUTO BRASILEIRO DE ENSINO SUPERIOR PROFISSIONALIZANTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2157, 2453, 'FACULDADE BRASIL CENTRAL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2158, 2454, 'FACULDADE MAFRENSE DE TURISMO E CINCIAS HUMANAS DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2159, 2455, 'FACULDADE CASTRO ALVES', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2160, 2456, 'FACULDADE DE ADMINISTRAO DE CHAPADO DO SUL', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2161, 2457, 'FACULDADE ISAAC NEWTON', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2162, 2458, 'CENTRO UNIFICADO DE ENSINO SUPERIOR DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2163, 2459, 'FACULDADE DE ESTUDOS AVANADOS DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2164, 2460, 'FACULDADE CMARA CASCUDO', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2165, 2461, 'FACULDADE CBES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2166, 2462, 'FACULDADE SO LUS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2167, 2463, 'FACULDADE JOANA D''ARC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2168, 2464, 'INSTITUTO SUPERIOR DE EDUCAO ATENEU', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2169, 2465, 'FACULDADE DE DIREITO PROFESSOR DAMSIO DE JESUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2170, 2466, 'FACULDADE ATENEU', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2171, 2467, 'FACULDADE NACIONAL DO NORTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2172, 2468, 'INSTITUTO TEOLGICO FRANCISCANO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2173, 2469, 'INSTITUTO MACAPAENSE DE ENSINO SUPERIOR', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2174, 2470, 'INSTITUTO SALVADOR DE ENSINO E CULTURA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2175, 2471, 'INTITUTO SUPERIOR DE EDUCAO DE BLUMENAU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2176, 2472, 'FACULDADE DE EDUCAO, CINCIA E TECNOLOGIA 2001', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2177, 2473, 'INSTITUTO BRASILEIRO DE ADMINISTRAO - BIM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2178, 2474, 'FACULDADE MARIA MILZA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2179, 2475, 'FACULDADE ARQUIDIOCESANA DE MARIANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2180, 2476, 'FACULDADE FNIX DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2181, 2477, 'FACULDADE DE ADMINISTRAO DE MARIANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2182, 2478, 'FACULDADE DE INTEGRAO DO ENSINO SUPERIOR DO CONE SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2183, 2479, 'FACULDADE LATINO AMERICANA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2184, 2480, 'INSTITUTO SUPERIOR DE EDUCAO DE ITPOLIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2185, 2481, 'ESCOLA BRASILEIRA DE FORMAO EMPRESARIAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2186, 2482, 'FACULDADE DE CINCIAS HUMANAS E APLICADAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2187, 2483, 'INSTITUTO SUPERIOR DE EDUCAO INTEGRAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2188, 2484, 'FACULDADE PROCESSUS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2189, 2485, 'INSTITUTO DE CINCIAS RELIGIOSAS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2190, 2486, 'FACULDADE LEONARDO DA VINCI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2191, 2487, 'FACULDADE EXMIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2192, 2488, 'FACULDADE ANGLICANA DE ERECHIM', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2193, 2489, 'INSTITUTO SUPERIOR DE EDUCAOBRASIL CENTRAL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2194, 2490, 'INSTITUTO SUPERIOR DE ENSINO DO LITORAL CATARINENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2195, 2491, 'INSTITUTO SUPERIOR DE EDUCAO DO CECAP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2196, 2492, 'INSTITUTO SUPERIOR DE EDUCAO A DISTNCIA DE BRASLIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2197, 2493, 'INSTITUTO SUPERIOR DE EDUCAO  DISTNCIA DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2198, 2494, 'FACULDADE DE PINDAMONHANGABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2199, 2495, 'INSTITUTO DE PESQUISA GRANDE VITRIA - IPGV FACULDADE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2200, 2496, 'INSTITUTO MACHADO DE ASSIS DE EDUCAO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2201, 2497, 'FACULDADE ATENEU', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2202, 2498, 'FACULDADE DE MLTIPLAS CINCIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2203, 2499, 'FACULDADE APOIO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2204, 2500, 'FACULDADE EINSTEIN', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2205, 2501, 'FACULDADE ESTCIO DE S DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2206, 2502, 'FACULDADE ARINOS DE ADMINISTRAO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2207, 2503, 'FACULDADE DE INTEGRAO NACIONAL LEDA JESUNO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2208, 2504, 'FACULDADE DIVINPOLIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2209, 2505, 'FACULDADE DE ARTE E DESIGN', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2210, 2506, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2211, 2507, 'FACULDADE DE TECNOLOGIA E CINCIAS DE SANTO ANTNIO DE JESUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2212, 2508, 'INSTITUTO SUPERIOR DE EDUCAO SANTO AGOSTINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2213, 2509, 'FACULDADE DE MEDICINA TRADICIONAL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2214, 2510, 'FACULDADES MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2215, 2511, 'FACULDADE DE DIREITO DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2216, 2512, 'INSTITUTO SUPERIOR DE EDUCAO CUIABANO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2217, 2513, 'FACULDADE ARTHUR THOMAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2218, 2514, 'INSTITUTO NATALENSE DE EDUCAO SUPERIOR', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2219, 2515, 'FACULDADE ERUDITA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2220, 2516, 'INSTITUTO SUPERIOR DE EDUCAO SANTA RITA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2221, 2517, 'FACULDADE DE ESTUDOS SOCIAIS E GESTO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2222, 2518, 'FACULDADE INTENSIVA DE PEDAGOGIA E FILOSOFIA DE ILHUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2223, 2519, 'FACULDADE DE INFORMTICA E TECNOLOGIA TERRANOVENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2224, 2520, 'INSTITUTO DE EDUCAO SUPERIOR TERRANOVENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2225, 2521, 'INSTITUTO SUPERIOR DE EDUCAO DO SUL DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2226, 2522, 'INSTITUTO MARISTA SUPERIOR DE EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2227, 2523, 'FACULDADE BONDESPACHENSE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2228, 2524, 'FACULDADE DE ENGENHARIA DA ITAJUB TECNPOLIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2229, 2525, 'FACULDADE PAULISTA DE CINCIAS APLICADAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2230, 2526, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2231, 2527, 'INSTITUTO SUPERIOR DE EDUCAO DE JACARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2232, 2528, 'INSTITUTO SUPERIOR DE EDUCAO ME DE DEUS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2233, 2529, 'INSTITUTO SUPERIOR DE EDUCAO DE IGUAPE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2234, 2530, 'FACULDADE DE CINCIAS DE GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2235, 2531, 'INSTITUTO SUPERIOR DE EDUCAO PROFESSORA LCIA DANTAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2236, 2532, 'FACULDADE DO SUDESTE GOIANO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2237, 2533, 'FACULDADE REGIONAL BARUERI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2238, 2534, 'FACULDADE METROPOLITANA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2239, 2535, 'FACULDADE DOS INCONFIDENTES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2240, 2536, 'FACULDADE RORAIMENSE DE ENSINO SUPERIOR', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2241, 2537, 'FACULDADE SO GERALDO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2242, 2538, 'FACULDADE VENDA NOVA DO IMIGRANTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2243, 2539, 'FACULDADE EVANGLICA DO MEIO NORTE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2244, 2540, 'INSTITUTO DE ENSINO SUPERIOR DE CAMAPU', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2245, 2541, 'ESCOLA SUPERIOR DE TECNOLOGIA E EDUCAO DE ARARAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2246, 2542, 'FACULDADE CENECISTA DE QUEIMADOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2247, 2543, 'FACULDADE DA CHAPADA DOS GUIMARES', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2248, 2544, 'INSTITUTO SUPERIOR DE EDUCAO PROFESSORA VILMA MONTEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2249, 2545, 'FACULDADE CENECISTA DE PARACAMBI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2250, 2546, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS DE VIANNA JNIOR DE MURIAR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2251, 2547, 'FACULDADE MAGNUM DE FORMAO DE PROFESSORES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2252, 2548, 'FACULDADE DE TEOLOGIA, FILOSOFIA E CINCIAS HUMANAS GAMALIEL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2253, 2549, 'FACULDADE PAULISTA DE EDUCAO E COMUNICAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2254, 2550, 'FACULDADE SEPAM', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2255, 2551, 'FACULDADE CENECISTA ITAGUA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2256, 2552, 'INSTITUTO SUPERIOR DE CINCIAS DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2257, 2553, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA LUIS EDUARDO MAGALHES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2258, 2554, 'FACULDADE DE EDUCAO SANTA TEREZINHA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2259, 2555, 'FACULDADE DE EDUCAO DE UBERABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2260, 2556, 'FIAM-FAAM - CENTRO UNIVERSITRIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2261, 2557, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE PENEDO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2262, 2558, 'FACULDADE DE DIREITO DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2263, 2559, 'INSTITUTO SUPERIOR DE EDUCAO DAS FACULDADES DE VALINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2264, 2560, 'FACULDADE DE ENSINO SUPERIOR DA CIDADE DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2265, 2561, 'FACULDADE DE CINCIAS ADMINISTRATIVAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2266, 2562, 'FACULDADE JOS LACERDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2267, 2563, 'INSTITUTO SUPERIOR DE EDUCAO PADRE JOO BAGOZZI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2268, 2564, 'UNIVERSIDADE FEDERAL DE CAMPINA GRANDE', 1, 1, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2269, 2565, 'ABEU - CENTRO UNIVERSITRIO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2270, 2566, 'INSTITUTO DE ENSINO SUPERIOR DE OLINDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2271, 2567, 'ESCOLA SUPERIOR DE CINCIAS HUMANAS E ECONMICAS', 2, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2272, 2568, 'FACULDADE ZACARIAS DE GES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2273, 2569, 'FACULDADE DE TECNOLOGIA FAESA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2274, 2570, 'FACULDADE DE CINCIAS EDUCACIONAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2275, 2571, 'FACULDADE REDENTOR', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2276, 2572, 'FACULDADE REGIONAL DE FILOSOFIA, CINCIAS E LETRAS DE CANDEIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2277, 2573, 'FACULDADE TEOLGICA SUL AMERICANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2278, 2574, 'INSTITUTO SUPERIOR DE EDUCAO DAS FACULDADES INTEGRADAS MDULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2279, 2575, 'FACULDADES INTEGRADAS SOARES DE OLIVEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2280, 2576, 'FACULDADES INTEGRADAS PADRE ANCHIETA DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2281, 2577, 'INSTITUTO SUPERIOR DE MSICA DE SO LEOPOLDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2282, 2578, 'FACULDADE DE TEOLOGIA CARDEAL EUGNIO SALES', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2283, 2579, 'FACULDADE ATENAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2284, 2580, 'INSTITUTO SUPERIOR DE EDUCAO PAULO MARTINS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2285, 2581, 'FACULDADE SO SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2286, 2582, 'FACULDADE DE ENSINO SUPERIOR DO CENTRO DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2287, 2583, 'FACULDADE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2288, 2584, 'FACULDADE DO SUDOESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2289, 2585, 'FACULDADE DE FISIOTERAPIA DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2290, 2586, 'INSTITUTO SUPERIOR DE EDUCAO TALO-BRASILEIRO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2291, 2587, 'FACULDADE CNDIDO MENDES DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2292, 2588, 'FALCULDADE MERCATOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2293, 2589, 'FACULDADE DE BROTAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2294, 2590, 'FACULDADE DE CINCIA DA COMPUTAO E INFORMTICA SILVA SERPA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2295, 2591, 'ESCOLA BRASILEIRA DE ECONOMIA E FINANAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2296, 2592, 'FACULDADE PRESBITERIANA 15 DE NOVEMBRO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2297, 2593, 'FACULDADE DE JUAZEIRO DO NORTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2298, 2594, 'FACULDADE TEOLGICA BATISTA ANA WOLLERMAN', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2299, 2595, 'FACULDADES COSTA BRAGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2300, 2596, 'INSTITUTO SUPERIOR DE EDUCAO DON DOMNICO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2301, 2597, 'FACULDADE DE CINCIAS DA SADE DA VITRIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2302, 2598, 'FACULDADE PENSAR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2303, 2599, 'FACULDADE DE EDUCAO E CINCIAS HUMANAS NOVA FLRIDA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2304, 2600, 'FACULDADE DE CIENCIAS DA ADMINISTRACAO DA VITORIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2305, 2601, 'FACULDADE DE CINCIAS, FILOSOFIA E LETRAS HAKAY', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2306, 2602, 'INSTITUTO SUPERIOR DE EDUCAO BALO VERMELHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2307, 2603, 'FACULDADE DE ODONTOLOGIA DOS GUARARAPES', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2308, 2604, 'FACULDADE TEOLGICA BATISTA DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2309, 2605, 'FACULDADE DE CINCIAS ECONMICAS DE TUBARO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2310, 2606, 'FACULDADE AFIRMATIVO DE CAMPO MOURO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2311, 2607, 'FACULDADE PADRE ANTONIO VIEIRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2312, 2608, 'INSTITUTO SUPERIOR DE EDUCAO DE LAVRAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2313, 2609, 'INSTITUTO SUPERIOR DE EDUCAO DE TUPI PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2314, 2610, 'FACULDADE DE JACARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2315, 2611, 'FACULDADE DE ENSINO SUPERIOR DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2316, 2612, 'INSTITUTO SUPERIOR DE EDUCAO MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2317, 2613, 'FACULDADE RIO BRANCO', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2318, 2614, 'FACULDADE EMMANUEL KANT', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2319, 2615, 'FACULDADE INCONFIDNCIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2320, 2616, 'FACULDADE SANTO ANTNIO DE PDUA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2321, 2617, 'FACULDADE SO BENTO DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2322, 2618, 'FACULDADE DOMINGOS SVIO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2323, 2619, 'FACULDADE DE DIREITO DE ITABIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2324, 2620, 'FACULDADE AMPRE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2325, 2621, 'INSTITUTO SUPERIOR DE EDUCAO DOMINGOS SVIO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2326, 2622, 'INSTITUTO SUPERIOR DE EDUCAO DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2327, 2623, 'FACULDADE DO TAPAJS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2328, 2624, 'FACULDADE DO LITORAL PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2329, 2625, 'FACULDADE DE TECNOLOGIA IBTA - SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2330, 2626, 'FACULDADE DE COMUNICAO SOCIAL SANTA EFIGNIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2331, 2627, 'FACULDADE JOO XXIII', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2332, 2628, 'INSTITUTO DE EDUCAO SUPERIOR ACADEMUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2333, 2629, 'FACULDADE DE CINCIAS CONTBEIS E DE ADMINISTRAO DO VALE DO JURUENA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2334, 2630, 'INSTITUTO SUPERIOR DE EDUCAO DO VALE DO JURUENA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2335, 2631, 'INSTITUTO DE EDUCAO SUPERIOR', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2336, 2632, 'FACULDADES INTEGRADAS DO EXTREMO SUL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2337, 2633, 'INSTITUTO DE EDUCAO E TECNOLOGIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2338, 2634, 'INSTITUTO EDUCACIONAL DALTON CUNHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2339, 2635, 'INSTITUTO SUPERIOR DE EDUCAO JK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2340, 2636, 'FACULDADE DINMICA DO VALE DO PIRANGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2341, 2637, 'FACULDADE BOM JESUS DE SIQUEIRA CAMPOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2342, 2638, 'INSTITUTO DE CINCIAS E EDUCAO DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2343, 2639, 'FACULDADE DE TECNOLOGIA SENAI CONCRDIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2344, 2640, 'FACULDADE DE CINCIAS DA SADE DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2345, 2641, 'FACULDADE TEOLGICA BATISTA FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2346, 2642, 'FACULDADE SO VICENTE', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2347, 2643, 'INSTITUTO SUPERIOR DE CINCIAS E TECNOLOGIA DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2348, 2644, 'INSTITUTO DE EDUCAO SUPERIOR DO TAPAJS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2349, 2645, 'INSTITUTO DE ENSINO SUPERIOR DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2350, 2646, 'FACULDADE REGIONAL IPIRANGA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2351, 2647, 'FACULDADE SANTA CLARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2352, 2648, 'FACULDADE SUL MINEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2353, 2649, 'FACULDADE DE CINCIAS JURDICAS DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2354, 2650, 'INSTITUTO DE ENSINO SUPERIOR DE ARIQUEMES', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2355, 2651, 'FACULDADE TAMANDAR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2356, 2652, 'FACULDADE UNIDA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2357, 2653, 'FACULDADE DE EDUCAO SUPERIOR DO PIEMONTE DA CHAPADA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2358, 2654, 'FACULDADE DE ENSINO TECNOLGICO DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2359, 2655, 'FACULDADE DE CINCIAS ADMINISTRATIVAS, SOCIAIS E DA SADE DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2360, 2656, 'FACULDADE SANTA CATARINA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2361, 2657, 'FACULDADE ADOLFO CAMINHA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2362, 2658, 'FACULDADE AMEC TRABUCO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2363, 2659, 'FACULDADE DO VALE DO PARNABA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2364, 2660, 'FACULDADE PHNIX DE CINCIAS HUMANAS E SOCIAIS DO BRASIL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2365, 2661, 'INSTITUTO DE ENSINO SUPERIOR DE BARREIRAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2366, 2662, 'INSTITUTO SUPERIOR DE ENSINO PAULA CNDIDO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2367, 2663, 'FACULDADE DE ODONTOLOGIA ARGEMIRO FIGUEIREDO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2368, 2664, 'INSTITUTO DE ENSINO SUPERIOR DO VALE DO PARNABA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2369, 2665, 'FACULDADE DE TECNOLOGIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2370, 2666, 'FACULDADE INTEGRADA DE BRUSQUE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2371, 2667, 'INSTITUTO SUPERIOR PAULO FREIRE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2372, 2668, 'FACULDADE TEOLGICA DO SUDOESTE GOIANO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2373, 2669, 'FACULDADE CNDIDO MENDES DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2374, 2670, 'INSTITUTO DE EDUCAO SUPERIOR ACREANO EUCLIDES DA CUNHA', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2375, 2671, 'FACULDADE DE CINCIAS HUMANAS E PSICOLGICAS EUCLIDES DA CUNHA', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2376, 2672, 'FACULDADE HUGO DE SO VITOR', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2377, 2673, 'INSTITUTO SUPERIOR DE EDUCAO VILA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2378, 2674, 'INSTITUTO DE ENSINO SUPERIOR UNILLUZ', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2379, 2675, 'INSTITUTO SUPERIOR DE EDUCAO LATINO AMERICANO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2380, 2676, 'FACULDADE LA SALLE', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2381, 2677, 'INSTITUTO SUPERIOR DE EDUCAO SO JUDAS TADEU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2382, 2678, 'FACULDADE DE COMPUTAO E INFORMTICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2383, 2679, 'FACULDADE DE CINCIA DA COMPUTAO DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2384, 2680, 'FACULDADE DE COMPUTAO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2385, 2681, 'FACULDADE DE ENGENHARIA CIVIL DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2386, 2682, 'FACULDADE DE CINCIAS DA SADE DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2387, 2683, 'FACULDADE DE CINCIAS DA COMPUTAO DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2388, 2684, 'FACULDADE DE CINCIAS GERENCIAIS DE LEOPOLDINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2389, 2685, 'FACULDADE DELTA DE ENSINO SUPERIOR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2390, 2686, 'FACULDADE SANTA IZILDINHA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2391, 2687, 'FACULDADE DOM ALBERTO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2392, 2688, 'FACULDADE INESP - INSTITUTO NACIONAL DE ENSINO E PESQUISA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2393, 2689, 'FACULDADE CARLOS LOBO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2394, 2690, 'FACULDADES INTEGRADAS LOGOS - FILOGOS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2395, 2691, 'FACULDADE ASSOCIADA DA BAHIA - FAB', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2396, 2692, 'FACULDADE VILA DE CINCIAS HUMANAS E EXATAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2397, 2693, 'FACULDADES INTEGRADAS DO TRINGULO MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2398, 2694, 'FACULDADE VALE DO ARAGUAIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2399, 2695, 'FACULDADE DO CENTRO-OESTE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2400, 2696, 'INSUBA - INSTITUTO SUPERIOR DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2401, 2697, 'INSTITUTO SUPERIOR DE EDUCAO DE AMLIA RODRIGUES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2402, 2698, 'INSTITUTO SUPERIOR DE EDUCAO VALE DO ARAGUAIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2403, 2699, 'INSTITUTO DE ENSINO SUPERIOR AVANT GARDE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2404, 2700, 'FACULDADE UNIPOLI', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2405, 2701, 'FACULDADE DE ORLNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2406, 2702, 'FACULDADE DE CINCIAS JURDICAS DE PARASO DO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2407, 2703, 'FACULDADE SENAC MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2408, 2704, 'INSTITUTO SUPERIOR DE EDUCAO DO INSTITUTO METODISTA UNIO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2409, 2705, 'FACULDADE DE EDUCAO E CINCIAS HUMANAS NOVA FLRIDA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2410, 2706, 'FACULDADE EUROPEU', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2411, 2707, 'INSTITUTO SUPERIOR DE EDUCAO PROFESSOR EDGARDO BEZERRA DA ROCHA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2412, 2708, 'INSTITUTO SUPERIOR DE CINCIAS HUMANAS E FILOSOFIA LA SALLE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2413, 2709, 'INSTITUTO DE ENSINO SUPERIOR NOVO HORIZONTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2414, 2710, 'INSTITUTO SUPERIOR DE EDUCAO RANCHARIENSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2415, 2711, 'FACULDADE DE ADMINISTRAO DE EMPRESAS DE CAJAMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2416, 2712, 'FACULDADE DE SISTEMAS DE INFORMAO DE CAJAMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2417, 2713, 'INSTITUTO NORMAL SUPERIOR DE CAJAMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2418, 2714, 'FACULDADE DE ENFERMAGEM E OBSTETRCIA DE CAJAMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2419, 2715, 'FACULDADE DE CINCIAS HUMANAS DE PONTE NOVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2420, 2716, 'FACULDADE COTEMGE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2421, 2717, 'FACULDADE DE CINCIAS JURDICAS VISCONDE DE CAIRU', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2422, 2718, 'INSTITUTO SUPERIOR DE EDUCAO DO NORTE', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2423, 2719, 'FACULDADE DE DIREITO OSWALDO CRUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2424, 2720, 'FACULDADE AUGUSTO MAIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2425, 2721, 'FACULDADE IMPACTA DE TECNOLOGIA DA INFORMAO - FITI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2426, 2722, 'FACULDADE CATLICA DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2427, 2723, 'FACULDADE MARISTA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2428, 2724, 'INSTITUTO DE ENSINO SUPERIOR DO SUL DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2429, 2725, 'INSTITUTO SUPERIOR DE EDUCAO DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2430, 2726, 'ESCOLA SUPERIOR DE ADMINISTRAO DE EMPRESAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2431, 2727, 'FACULDADE DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2432, 2728, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS MAX', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2433, 2729, 'FACULDADE DE PSICOLOGIA MAX', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2434, 2730, 'UNIPB - FACULDADE UNIDA DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2435, 2731, 'ESCOLA SUPERIOR DE COMPUTAO E TECNOLOGIAS DA INFORMAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2436, 2732, 'FACULDADE UNIDAS DO ENTORNO GOIS/DF - UNIENTORNO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2437, 2733, 'FACULDADE TEOLOGICA ALPHA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2438, 2734, 'FACULDADE DE CINCIAS DA SADE DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2439, 2735, 'FACULDADE DE CINCIAS DA COMPUTAO DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2440, 2736, 'INSTITUTO TECNOLGICO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2441, 2737, 'FACULDADE DE CINCIAS DA INFORMAO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2442, 2738, 'INSTITUTO SUPERIOR DE EDUCAO DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2443, 2739, 'FACULDADE CASTILHENSE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2444, 2740, 'FACULDADE DO VALE DO JAGUARIBE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2445, 2741, 'FACULDADE CATLICA DOM ORIONE', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2446, 2742, 'FACULDADE DE EDUCAO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2447, 2743, 'FACULDADE DE CINCIA DA COMPUTAO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2448, 2744, 'INSTITUTO SUPERIOR DE EDUCAO VERA CRUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2449, 2745, 'ESCOLA SUPERIOR DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2450, 2746, 'INSTITUTO DE ENSINO SUPERIOR ETAPA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2451, 2747, 'FACULDADE EUROPEU DO CEARA - FEC', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2452, 2748, 'FACULDADE LATO SENSU DO BRASIL', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2453, 2749, 'INSTITUTO DE CINCIAS JURDICAS PITGORAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2454, 2750, 'FACULDADE DE TECNOLOGIA SENAI LUZERNA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2455, 2751, 'INSTITUTO SUPERIOR DE EDUCAO DE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2456, 2752, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS PROFESSORA VILMA MONTEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2457, 2753, 'FACULDADE DE CINCIAS AGRRIAS, BIOLGICAS, EXATAS E SOCIAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2458, 2754, 'FACULDADE SO PAULO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2459, 2755, 'FACULDADE DO VALE DO ITAJA MIRIM', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2460, 2756, 'FACULDADE JK VALPARASO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2461, 2757, 'FACULDADE JK ADMINISTRAO DE VALPARASO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2462, 2758, 'INSTITUTO SUPERIOR DE EDUCAO PROFESSOR PAULO ALBINO PIMENTEL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2463, 2759, 'FACULDADE PROFESSOR PAULO ALBINO PIMENTEL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2464, 2760, 'ESCOLA SUPERIOR DE TECNOLOGIA DA INFORMAO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2465, 2761, 'FACULDADE DO SERTO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2466, 2762, 'INSTITUTO ISAAC NEWTON', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2467, 2763, 'FACULDADE DE CINCIAS GERENCIAS DE SO GOTARDO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2468, 2764, 'INSTITUTO DE ENSINO SUPERIOR DE OURO FINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2469, 2765, 'INSTITUTO DE CINCIAS RELIGIOSAS LUMEN CHRISTI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2470, 2766, 'FACULDADE EMPRESARIAL DE CHAPEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2471, 2767, 'INSTITUTO SUPERIOR DE EDUCAO DA AIMORS - M.G.', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2472, 2768, 'INSTITUTO SUPERIOR DE EDUCAO - FACREFI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2473, 2769, 'FAVE - FACULDADE DE ADMINISTRAO DE VESPASIANO LTDA.', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2474, 2770, 'FACULDADE UNIDA DE CAMPINAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2475, 2771, 'CENTRO DE ENSINO SUPERIOR DE ILHUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2476, 2772, 'INSTITUTO SUPERIOR DE EDUCAO MANHUMIRIM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2477, 2773, 'FACULDADE DE MACAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2478, 2774, 'FACULDADES INTEGRADAS FAFIBE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2479, 2775, 'FACULDADES INTEGRADAS DESEMBARGADOR SVIO BRANDO - FAUSB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2480, 2776, 'FACULDADES INTEGRADAS NORTE CAPIXABA - FANORTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2481, 2777, 'INSTITUTO DE ENSINO SUPERIOR SANTA MARIA DE ASSIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2482, 2778, 'INSTITUTO SUPERIOR ESPECIAL DE EDUCAO DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2483, 2779, 'FACULDADE DE INFORMTICA DE OURO PRETO DO OESTE', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2484, 2780, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DA ALTA NOROESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2485, 2781, 'FACULDADE DE CINCIAS DA SADE DA ALTA NOROESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2486, 2782, 'INSTITUTO SUPERIOR DE EDUCAO AFONSO LINARES PRADO', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2487, 2783, 'FACULDADE DE TECNOLOGIA DO NORDESTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2488, 2784, 'FACULDADE FRANCISCANA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2489, 2785, 'ESCOLA SUPERIOR DE EDUCAO CONTINUADA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2490, 2786, 'INSTITUTO DE EDUCAO SUPERIOR E TECNOLOGICA PROF. FRANBRAN', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2491, 2787, 'FACULDADES PEQUENO PRNCIPE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2492, 2788, 'FACULDADE DE CINCIAS AMBIENTAIS DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2493, 2789, 'FACULDADE JOINVILLE DE ENSINO E CULTURA - FAJEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2494, 2790, 'FACULDADE FRANCISCANA DE ENSINO SUPERIOR - FAFRES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2495, 2791, 'FACULDADE DE CINCIAS AGRRIAS E EXATAS DE PRIMAVERA DO LESTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2496, 2792, 'FACULDADE DE CINCIAS SOCIAIS E EDUCAO - FACE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2497, 2793, 'FACULDADE UNIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2498, 2794, 'FACULDADE DE CINCIAS SOCIAIS E HUMANAS SOBRAL PINTO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2499, 2795, 'FACULDADE SO FRANCISCO DE ASSIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2500, 2796, 'FACULDADE DE IPOR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2501, 2797, 'FACULDADE DO LITORAL SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2502, 2798, 'FAESP - FACULDADE DE ENSINO SUPERIOR DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2503, 2799, 'FANEESP - FACULDADE NACIONAL DE EDUCAO E ENSINO SUPERIOR DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2504, 2800, 'FACULDADE DE ITABUNA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2505, 2801, 'INSTITUTO GUARANT DE EDUCAO SUPERIOR', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2506, 2802, 'FACULDADE DE CINCIAS AGRRIAS DE GUARANT DO NORTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2507, 2803, 'FACULDADE CETESI', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2508, 2804, 'FACULDADE CCAA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2509, 2805, 'FACULDADE MODELO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2510, 2806, 'FACULDADE EVANGLICA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2511, 2807, 'FACULDADE CENTRO CAPE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2512, 2808, 'FACULDADES ITECNE DE CASCAVEL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2513, 2809, 'INSTITUTO DE ENSINO SUPERIOR DE UBERLNDIA - IESU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2514, 2810, 'INSTITUTO SUPERIOR DE EDUCAO ANSIO TEIXEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2515, 2811, 'FACULDADE PAULUS DE TECNOLOGIA E COMUNICAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2516, 2812, 'FACULDADE DO SUDESTE BRASILEIRO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2517, 2813, 'INSTITUTO SUPERIOR DE EDUCAO CAPIMGROSSENSE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2518, 2814, 'FACULDADE SO SEBASTIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2519, 2815, 'FACULDADE VALE DO RIBEIRA - FVR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2520, 2816, 'FACULDADE SANTO AGOSTINHO - FSA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2521, 2817, 'FACULDADE DE IUNA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2522, 2818, 'FACULDADE DE VETERINRIA DE VESPASIANO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2523, 2819, 'FACULDADE ANGELO VASCONCELOS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2524, 2820, 'FACULDADE PITGORAS DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2525, 2821, 'FACULDADE IDC', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2526, 2822, 'FACULDADE DE ENGENHARIA FLORESTAL DE IBIRAU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2527, 2823, 'FACULDADE DE ENGENHARIA AGRCOLA DE IBIRAU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2528, 2824, 'FACULDADE DE ENGENHARIA AGRONMICA DE IBIRAU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2529, 2825, 'FACULDADE DO SISTEMA FINDES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2530, 2826, 'FACULDADE NOSSA SENHORA DE FTIMA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2531, 2827, 'FACULDADE EVANGLICA DO PIAUI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2532, 2828, 'FACULDADE ITABIRANA DE SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2533, 2829, 'INSTITUTO SUPERIOR DE EDUCAO E FILOSOFIA DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2534, 2830, 'FACULDADE ADVENTISTA DE CINCIAS DA SADE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2535, 2831, 'FACULDADE DOM BOSCO DE GOIOER', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2536, 2832, 'INSTITUTO SUPERIOR DE EDUCAO PROGRAMUS', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2537, 2833, 'FACULDADE ON LINE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2538, 2834, 'INSTITUTO SUPERIOR DE PSICOLOGIA DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2539, 2835, 'FACULDADE MAURCIO DE NASSAU', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2540, 2836, 'FACULDADE SUL DA AMRICA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2541, 2837, 'INSTITUTO SUPERIOR DE EDUCAO SUL DA AMRICA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2542, 2838, 'FASES - FACULDADE SOTEROPOLITANA DE ENSINO SUPERIOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2543, 2839, 'INSTITUTO METODISTA SUPERIOR DE EDUCAO DA AMAZNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2544, 2840, 'FACULDADE METODISTA DE TEOLOGIA E CINCIAS HUMANAS DA AMAZNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2545, 2841, 'FACULDADE PARANAPANEMA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2546, 2842, 'FACULDADE DE MEDICINA MACHADO DE ASSIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2547, 2843, 'FACULDADE DE CINCIAS MDICAS E DA SADE DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2548, 2844, 'FACULDADE SALESIANA DOM BOSCO DE PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2549, 2845, 'FACULDADE REINALDO RAMOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2550, 2846, 'FACULDADE DE TEOLOGIA UMBANDISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2551, 2847, 'INSTITUTO SUPERIOR DE EDUCAO DO PARANAPANEMA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2552, 2848, 'FACULDADE DE NOVO PROGRESSO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2553, 2849, 'ESCOLA SUPERIOR DOM HELDER CMARA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2554, 2850, 'FACULDADE DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2555, 2851, 'FACULDADE SANTA PAULINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2556, 2852, 'FACULDADE DE GESTO E NEGCIOS DO ALTO URUGUAI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2557, 2853, 'FACULDADE DE TECNOLOGIA INED - UNIDADE DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2558, 2854, 'FACULDADE CONHECER', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2559, 2855, 'FACULDADE SO FRANCISCO DE ASSIS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2560, 2856, 'FACULDADE DE TECNOLOGIA QUASAR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2561, 2857, 'INSTITUTO CIENTFICO DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2562, 2858, 'INSTITUTO DE SADE ALEXANDER FLEMING', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2563, 2859, 'FACULDADE DE BELM', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2564, 2860, 'FACULDADE CEAQ', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2565, 2861, 'FACULDADE DE ITAPETININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2566, 2862, 'FACULDADE DE DIREITO PRESIDENTE TANCREDO NEVES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2567, 2863, 'FACULDADE DE ADMINISTRAO DE PETROLINA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2568, 2864, 'FACULDADE DE DIREITO DE PETROLINA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2569, 2865, 'FACULDADE DE CINCIAS SOCIAIS DE RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2570, 2866, 'FACULDADE DE CINCIAS JURDICAS DE RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2571, 2867, 'FACULDADE DE DIREITO DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2572, 2868, 'FACULDADE DE ADMINISTRAO DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2573, 2869, 'FACULDADE DE CINCIAS SOCIAIS ADO DE ALMEIDA E SILVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2574, 2870, 'FACULDADE SERRA DO CARMO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2575, 2871, 'INSTITUTO SUPERIOR DE EDUCAO SERRA DO CARMO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2576, 2872, 'FACULDADE DE CINCIAS AGRRIAS RENATO AZEREDO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2577, 2873, 'FACULDADE DE SISTEMAS DE INFORMAO DE SO SEBASTIO DO PARASO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2578, 2874, 'FACULDADE DE DIREITO DE SO SEBASTIO DO PARASO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2579, 2875, 'FACULDADE INDGENA DA AMAZNIA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2580, 2876, 'FACULDADE UNISSA DE SARANDI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2581, 2877, 'FACULDADE UNISSA DE EDUCAO TECNOLGICA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2582, 2878, 'INSTITUTO SUPERIOR DE EDUCAO DE CAIEIRAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2583, 2879, 'FACULDADE METROPOLITANA DE CAIEIRAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2584, 2880, 'INSTITUTO SUPERIOR DE EDUCAO DE TIJUCUSSU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2585, 2881, 'INSTITUTO SUPERIOR DE EDUCAO DO SENAC-SP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2586, 2882, 'FACULDADE SO JOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2587, 2883, 'FACULDADE EDUVALE DE PARANAPANEMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2588, 2884, 'INSTITUTO SUPERIOR DE EDUCAO BOM JESUS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2589, 2885, 'FACULDADE SO CAMILO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2590, 2886, 'FACULDADE ZUMBI DOS PALMARES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2591, 2887, 'FACULDADE EDUVALE DE PIRAJU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2592, 2888, 'UNIVERSIDADE DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2593, 2889, 'UNIVERSIDADE PARA O DESENVOLVIMENTO DA AMAZNIA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2594, 2890, 'UNIVERSIDADE DE INTEGRAO BRASILEIRA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2595, 2891, 'FACULDADE BARO DE PIRATININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2596, 2892, 'FACULDADE DE PALMARES - FAPAL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2597, 2893, 'INSTITUTO SUPERIOR DE EDUCAO DOS PALMARES', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2598, 2894, 'FACULDADE INTEGRAL DE SANTA HELENA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2599, 2895, 'FACULDADE DE EDUCAO E TECNOLOGIA DA REGIO MISSIONEIRA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2600, 2896, 'FACULDADE SATC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2601, 2897, 'FACULDADE JK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2602, 2899, 'FACULDADES CAIARAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2603, 2900, 'INSTITUTO TECNOLGICO REGIONAL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2604, 2901, 'INSTITUTO SUPERIOR DE EDUCAO DO MDIO NORTE MATO-GROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2605, 2902, 'FACULDADE DO MDIO NORTE MATO-GROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2606, 2903, 'FACULDADE CONCRDIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2607, 2904, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2608, 2905, 'FACULDADE BRASILEIRA DE PEDAGOGIA / UNIBRA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2609, 2906, 'FACULDADE NACIONAL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2610, 2907, 'FACULDADE BRASILEIRA DE NANUQUE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2611, 2908, 'INSTITUTO NATALENSE DE ENSINO E CULTURA', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2612, 2909, 'INSTITUTO DE ENSINO SUPERIOR DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2613, 2910, 'INSTITUTO SUPERIOR DE EDUCAO VERDE NORTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2614, 2911, 'INSTITUTO DE ENSINO SUPERIOR DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2615, 2912, 'FACULDADE DE CINCIA E TECNOLOGIA ALBERT EINSTEIN', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2616, 2913, 'FACULDADE DO VALE DO PARAGUAU', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2617, 2914, 'INSTITUTO ABUCHAIM', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2618, 2915, 'FACULDADE CIDADE DE PATOS DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2619, 2916, 'INSTITUTO DE EDUCAO SUPERIOR DE SERGIPE - ISES', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2620, 2917, 'INSTITUTO DE ENSINO E CULTURA DO AMAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2621, 2918, 'FACULDADE DE ENSINO SUPERIOR DA AMAZNIA REUNIDA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2622, 2919, 'FACULDADE SO JOS DE RIBAMAR', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2623, 2920, 'INSTITUTO DE ENSINO SUPERIOR CATTA PRETA E DIAMANTINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2624, 2921, 'INSTITUTO SUPERIOR DE EDUCAO DA AMAZNIA REUNIDA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2625, 2922, 'INSTITUTO SUPERIOR DE EDUCAO DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2626, 2923, 'FACULDADE TABIRENSE DE ENFERMAGEM', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2627, 2924, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE BUTI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2628, 2925, 'INSTITUTO SUPERIOR DE EDUCAO DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2629, 2926, 'FACULDADE BRASIL CENTRAL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2630, 2927, 'FACULDADE DEL REY', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2631, 2928, 'FACULDADE HOLSTICA DE CINCIAS HUMANAS - FHCH', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2632, 2929, 'FACULDADE DE MILAGRES CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2633, 2930, 'FACULDADE DE SARANDI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2634, 2931, 'FACULDADE RIO GRANDE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2635, 2932, 'FACULDADE PELOTENSE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2636, 2933, 'FACULDADE TIMONENSE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2637, 2934, 'FACULDADE DE ADMINISTRAO E ECONMIA DE JOINVILLE - FAEJ', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2638, 2935, 'FACULDADE DE TALENTOS HUMANOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2639, 2936, 'INSTITUTO DE EDUCAO SUPERIOR MARISTA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2640, 2937, 'FACULDADE METROPOLITANA DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2641, 2938, 'FACULDADE DE EDUCAO DE BARCARENA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2642, 2939, 'INSTITUTO SUPERIOR DE EDUCAO TORRICELLI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2643, 2940, 'FACULDADE CENECISTA DE CACHOEIRAS DE MACACU', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2644, 2941, 'INSTITUTO SUPERIOR DE TEOLOGIA E PASTORAL DE BONFIM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2645, 2942, 'INSTITUTO SUPERIOR DE EDUCAO FRANCISCANO NOSSA SENHORA DE FTIMA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2646, 2943, 'INSTITUTO SUPERIOR DE EDUCAO PIRMIDE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2647, 2944, 'FACULDADE DO SUL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2648, 2945, 'FACULDADE DA ASSOCIAO BRASILIENSE DE EDUCAO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2649, 2946, 'FACULDADE DE ARQUITETURA E URBANISMO DO DISTRITO FEDERAL (FAU-DF)', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2650, 2947, 'FACULDADE DAS AMRICAS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2651, 2948, 'REAL FACULDADE DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2652, 2949, 'FACULDADE DO BAIXO PARNABA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2653, 2950, 'ESCOLA SUPERIOR DE ADMINISTRAO, DIREITO E ECONOMIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2654, 2951, 'INSTITUTO SUPERIOR DE EDUCAO DE MORRO AGUDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2655, 2952, 'FACULDADES DO SUL - FASUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2656, 2953, 'INSTITUTO SUPERIOR DE EDUCAO DA CIDADE DO NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2657, 2954, 'FACULDADE SUL CAPIXABA - UNISUL', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2658, 2955, 'FACULDADE DE CIENCIAS HUMANAS, BIOLGICAS E DA SAUDE DO LESTE MATOGROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2659, 2956, 'FACULDADE DE CINCIAS JURIDICAS, SOCIAIS E TECNOLGICA-FACITE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2660, 2957, 'FACULDADE DE CINCIAS EXATAS E AGRRIAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2661, 2958, 'FACULDADE DE CINCIAS BIOLGICAS, HUMANAS E SADE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2662, 2959, 'FACULDADE DE CIENCIAS AGRARIAS E EXATAS DO LESTE MATOGROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2663, 2960, 'FACULDADE SANTOS DUMONT', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2664, 2961, 'FACULDADE DO PANTANAL MATOGROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2665, 2962, 'FACULDADE DE CINCIAS SOCIAIS DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2666, 2963, 'FACULDADE DE CINCIAS SOCIAIS DE PETROLINA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2667, 2964, 'FACULDADE DE CINCIAS DE BRASLIA - FACIBRA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2668, 2965, 'FACULDADE MOKITI OKADA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2669, 2966, 'INSTITUTO SUPERIOR DE EDUCAO HELIO MAGDALENA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2670, 2967, 'INSTITUTO SUPERIOR DE EDUCAAO PEDRO II', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2671, 2968, 'FACULDADE NOVA ESPERANA DE NUTRIO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2672, 2969, 'INSTITUTO SUPERIOR DE EDUCAO OCIDENTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2673, 2970, 'FACULDADE INTEGRAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2674, 2971, 'FACULDADE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2675, 2972, 'FACULDADE CHAPADINHENSE DE EDUCAO, CINCIAS E TECNOLOGIA - FACHAP', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2676, 2973, 'FACULDADE DE CINCIAS HUMANAS E BIOLGICAS E DA SADE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2677, 2974, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS APLICADAS DE PRIMAVERA DO LESTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2678, 2975, 'FACULDADE JK GUAR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2679, 2976, 'INSTITUTO SUPERIOR DE EDUCAO DA FATEB', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2680, 2977, 'FACULDADES INTEGRADAS DE CIENCIAS E BIOTECNOLOGIA - FICEB', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2681, 2978, 'FACUDADE DE ODONTOLOGIA MARIO DE OLIVEIRA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2682, 2979, 'FACULDADE SUL ESPRITO SANTENSE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2683, 2980, 'FACULDADE SENAC-CEAR DE ADMINISTRAO E COMRCIO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2684, 2981, 'FACULDADE SO VICENTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2685, 2982, 'FACULDADE VICTOR HUGO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2686, 2983, 'FACULDADE SO VICENTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2687, 2984, 'FACULDADE ABRAHAN LINCON', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2688, 2985, 'FACULDADE MERIDIONAL DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2689, 2986, 'INSTITUTO DE FILOSOFIA E CIENCIAS HUMANAS DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2690, 2987, 'FACULDADE DE ODONTOLOGIA ALFEU GOMES DE OLIVEIRA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2691, 2988, 'INSTITUTO SUPERIOR DE EDUCAO FNIX DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2692, 2989, 'IBESS - INSTITUTO BRASILEIRO DE EDUCAO SUED E SUSEJ LTDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2693, 2990, 'FACULDADES DE ADMINISTRAO DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2694, 2991, 'FACULDADE DE SAPEAU', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2695, 2992, 'FACULDADE DO SUDESTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2696, 2993, 'FACULDADE DE CINCIAS BIOLGICAS E DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2697, 2994, 'INSTITUTO SUPERIOR UNIFELI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2698, 2995, 'INSTITUTO SUPERIOR DE EDUCAO LUTERANO DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2699, 2996, 'FACULDADE SOTEROPOLITANA DE DIREITO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2700, 2997, 'FACULDADE DE COMUNICAO E INFORMAO DO CENTRO OESTE - COMUNICO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2701, 2998, 'FACULDADES SESI AMAZONAS - FASESI', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2702, 2999, 'INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE RONDONPOLIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2703, 3000, 'FACULDADE CENECISTA DE SINOP', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2704, 3001, 'FACULDADE CENECISTA DE RONDONPOLIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2705, 3002, 'FACULDADE DE CINCIAS DA SADE NOSSA SENHORA DAS MERCS DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2706, 3003, 'INSTITUTO SUPERIOR DE EDUCAO JUSCELINO KUBITSCHEK', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2707, 3004, 'INSTITUTO DE ENSINO SUPERIOR SANTA CECLIA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2708, 3005, 'FACULDADE SEM FRONTEIRAS - FASEF', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2709, 3006, 'INSTITUTO SUPERIOR DE EDUCAO MILTON CAMPOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2710, 3007, 'FACULDADE DO INSTITUTO NACIONAL DE PS-GRADUAO DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2711, 3008, 'FACULDADE DO INSTITUTO NACIONAL DE PS-GRADUAO DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2712, 3009, 'INSTITUTO DE ENSINO SUPERIOR APIA - ISA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2713, 3010, 'FACULDADE DE CINCIAS HUMANAS DE RONDNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2714, 3011, 'FACULDADE PROF. MIGUEL NGELO DA SILVA SANTOS', 3, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2715, 3012, 'FAJOPA - FACULDADE JOO PAULO II', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2716, 3013, 'INSTITUTO SUPERIOR DE EDUCAO CIDADANIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2717, 3014, 'FACULDADE DE FARMCIA DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2718, 3015, 'INSTITUTO DE EDUCAO SUPERIOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2719, 3016, 'FACULDADE DE ENFERMAGEM JOS RODRIGUES DA SILVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2720, 3017, 'INSTITUTO SUPERIOR DE EDUCAO UNIPLAM', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2721, 3018, 'FACULDADE METROPOLITANA DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2722, 3019, 'INSTITUTO DE EDUCAO SUPERIOR ANAMLIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2723, 3020, 'FACULDADE SANTA RITA DE CSSIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2724, 3021, 'INSTITUTO SUPERIOR DE EDUCAO SANTA RITA DE CSSIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2725, 3022, 'FACULDADE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2726, 3023, 'FACULDADES INTEGRADAS ATLNTICA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2727, 3024, 'FACULDADES INTEGRADAS DE ENSINO SUPERIOR DO ESTADO DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2728, 3025, 'INSTITUTO PRESBITERIANO DE EDUCAO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2729, 3026, 'FACULDADE DO PLANALTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2730, 3027, 'FACULDADE DE CINCIAS EDUCACIONAIS E SISTEMAS INTEGRADOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2731, 3028, 'FACULDADE REUNIDAS DO LITORAL DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2732, 3029, 'FACULDADE DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2733, 3030, 'INSTITUTO SUPERIOR DE EDUCAO EDUVALE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2734, 3031, 'FACULDADE ALPHA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2735, 3032, 'FACULDADE SANTA MARIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2736, 3033, 'FACULDADE DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2737, 3034, 'FACULDADE DELTA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2738, 3035, 'FACULDADE BARBOSA CASTILHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2739, 3036, 'LIBERTAS - INSTITUTO SUPERIOR DE CINCIAS JURIDICAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2740, 3037, 'FACULDADE DE IBINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2741, 3038, 'INSTITUTO SUPERIOR DE EDUCAO LIONS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2742, 3039, 'FACULDADE VASCONCELLOS & SOUZA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2743, 3040, 'FACULDADE VALE DO JEQUITINHONHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2744, 3041, 'FACULDADE FILADELFIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2745, 3042, 'CHRISTUS FACULDADE DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2746, 3043, 'ABEU FACULDADE DE SO JOO DE MERITI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2747, 3044, 'ABEU FACULDADE VILAR DOS TELES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2748, 3045, 'ABEU FACULDADE DE QUEIMADOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2749, 3046, 'INSTITUTO SUPERIOR DE EDUCAO - ISE', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2750, 3047, 'FACULDADE POLIVALENTE - FACIP', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2751, 3048, 'INSTITUTO SUPERIOR DE EDUCAO DO INSTITUTO PORTO ALEGRE DA IGREJA METODISTA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2752, 3049, 'INSTITUTO ODONTOLGICO DO PARAN - ODONS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2753, 3050, 'FACULDADE DE TEOLOGIA DA ASSEMBLIA DE DEUS - FATEAD', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2754, 3051, 'FACULDADE TEOLGICA DE CINCIAS HUMANAS E SOCIAIS LOGOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2755, 3052, 'FACULDADE EDUCARE', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2756, 3053, 'FACULDADE DE ODONTOLOGIA PR-PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2757, 3054, 'FACULDADE DE CINCIAS E TECNOLOGIA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2758, 3055, 'INSTITUTO SUPERIOR DE EDUCAO DE RONDNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2759, 3056, 'ESCOLA SUPERIOR OSWALDO CRUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2760, 3057, 'INSTITUTO DE ENSINO SUPERIOR SAGRADA FAMLIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2761, 3058, 'INSTITUTO SUPERIOR DE EDUCAO SAGRADA FAMLIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2762, 3059, 'INSTITUTO DE ENSINO SUPERIOR DO PARANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2763, 3060, 'INSTITUTO DE ENSINO SUPERIOR DE ARAUCRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2764, 3061, 'INSTITUTO SUPERIOR DE EDUCAO DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2765, 3062, 'FACULDADE COMERCIAL DE SERRINHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2766, 3063, 'FAECO - FACULDADE EDUCACIONAL COSTA OESTE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2767, 3064, 'FACULDADE DE CINCIAS EXATAS E AGRARIAS DE RORAIMA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2768, 3065, 'INSTITUTO SUPERIOR DE EDUCAO DE ARAUCRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2769, 3066, 'FACULDADE INTERAMERICANA DE BELM', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2770, 3067, 'FACULDADE ROTHSCHILD DE EDUCAO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2771, 3068, 'FACULDADE DE TEOLOGIA E CINCIAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2772, 3069, 'INSTITUTO DE EDUCAO SUPERIOR DE ALAGOAS NO LITORAL NORTE', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2773, 3070, 'FACULDADE DE BELA VISTA DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2774, 3071, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA DE SERRINHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2775, 3072, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA DE JACOBINA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2776, 3073, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA DE CRUZ DAS ALMAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2777, 3074, 'FACULDADE DO EXTREMO SUL DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2778, 3075, 'INSTITUTO DE EDUCAO SUPERIOR UNYAHNA DE ALAGOINHAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2779, 3076, 'FACULDADE DE SADE IPITANGA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2780, 3077, 'FACULDADE METROPOLITANA DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2781, 3078, 'INSTITUTO SUPERIOR DE EDUCAO FRUTALENSE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2782, 3079, 'INSTITUTO DE ENSINO SUPERIOR ALTERNATIVO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2783, 3080, 'FACULDADE DE ARTE E DESIGN BERLIM - BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2784, 3081, 'FACULDADE ITALIANA DE TECNOLOGIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2785, 3082, 'FACULDADE 2 DE JULHO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2786, 3083, 'FACULDADE INTEGRADA UNIVERACRUZ DE GUANAMBI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2787, 3084, 'FACULDADE MARQUS DE POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2788, 3085, 'FACULDADE EVANGLICA LUZ E VIDA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2789, 3086, 'FACULDADE DA MODA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2790, 3087, 'CHRISTUS INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2791, 3088, 'FACULDADE UNIDAS SUL BAHIA - FUSBA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2792, 3089, 'FACULDADE CAIARAS DE TAGUATINGA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2793, 3090, 'INSTITUTO SUPERIOR NORMAL AURELINA PALMEIRA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2794, 3091, 'CENTRO DE ENSINO LEIDE TORRES', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2795, 3092, 'FACULDADE DE BEM ESTAR SOCIAL E SADE DE ITAPECERICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2796, 3093, 'FACULDADE DE ENFERMAGEM JOO FRANCISCO DA SILVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2797, 3094, 'FACULDADE DE DIREITO DESEMBARGADOR AFFONSO TEIXEIRA LAGES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2798, 3095, 'FACULDADE DE CINCIAS AGRRIAS DOM ORIONE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2799, 3096, 'FACULDADE DE AGRONOMIA MANOEL ANTNIO DA SILVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2800, 3097, 'FACULDADE DE DIREITO AFONSO PENA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2801, 3098, 'INSTITUTO SUPERIOR DE EDUCAO SO JUDAS TADEU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2802, 3099, 'FACULDADE POTIGUAR DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2803, 3100, 'FACULDADE INTEGRADA UNIVERACRUZ DE BOM JESUS DA LAPA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2804, 3101, 'FACULDADE INTEGRADA UNIVERACRUZ DE ALAGOINHAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2805, 3102, 'FACULDADE INTEGRADA UNIVERACRUZ DE CAMPO FORMOSO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2806, 3103, 'FACULDADE INTEGRADA UNIVERACRUZ DE BRUMADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2807, 3104, 'FACULDADE INTEGRADA UNIVERACRUZ DE JEQUI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2808, 3105, 'FACULDADE INTEGRADA UNIVERACRUZ DE JACOBINA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2809, 3106, 'FACULDADE INTEGRADA UNIVERACRUZ DE ITAMARAJ', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2810, 3107, 'FACULDADE BENEMRITA EDUCADORA UNIVERACRUZ DE PAULO AFONSO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2811, 3108, 'FACULDADE INTEGRADA UNIVERACRUZ DE ITABUNA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2812, 3109, 'FACULDADE ITEGRADA UNIVERACRUZ DE VALENA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2813, 3110, 'FACULDADE ITEGRADA UNIVERACRUZ DE CAMAARI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2814, 3111, 'FACULDADE INTEGRADA UNIVERACRUZ DE BARREIRAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2815, 3112, 'FACULDADE INTEGRADA UNIVERACRUZ DE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2816, 3113, 'FACULDADE INTEGRADA UNIVERACRUZ DE PILO ARCADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2817, 3114, 'FACULDADE INTEGRADA UNIVERACRUZ DE VITRIA DA CONQUISTA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2818, 3115, 'INSTITUTO SUPERIOR DE EDUCAO SANTO AGOSTINHO DE PIRAPORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2819, 3116, 'FACULDADE SANTO AGOSTINHO DE PIRAPORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2820, 3117, 'FACULDADE DE SADE E DESENVOLVIMENTO HUMANO SANTO AGOSTINHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2821, 3118, 'FALCUDADE INTEGRADA UNIVERACRUZ DE BARRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2822, 3119, 'FACULDADE INTEGRADA UNIVERACRUZ DE DIAS D''AVILA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2823, 3120, 'FACULDADE INTEGRADA UNIVERACRUZ DE CANDEIAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2824, 3121, 'FACULDADE INTEGRADA UNIVERACRUZ DE SERRINHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2825, 3122, 'FACULDADE INTEGRADA UNIVERACRUZ SIMES FILHO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2826, 3123, 'FACULDADE INTEGRADA UNIVERACRUZ DE SANTA MARIA DA VITRIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2827, 3124, 'FACULDADE DO DISTRITO FEDERAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2828, 3125, 'FACULDADE INTEGRADA UNIVERACRUZ DE PORTO SEGURO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2829, 3126, 'FACULDADE INTEGRADA UNIVERACRUZ DE SENHOR DO BONFIM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2830, 3127, 'FACULDADE INTEGRADA UNIVERACRUZ DE IRECE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2831, 3128, 'FACULDADE INTEGRADA UNIVERACRUZ DE LAURO DE FREITAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2832, 3129, 'FACULDADE INTEGRADA UNIVERACRUZ DE ILHUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2833, 3130, 'FACULDADE INTEGRADA UNIVERACRUZ DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2834, 3131, 'FACULDADE INTEGRADA UNIVERACRUZ DE EUNPOLIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2835, 3132, 'FACULDADE INEGRADA UNIVERACRUZ DE EUCLIDES DA CUNHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2836, 3133, 'FACULDADE INTEGRADA UNIVERACRUZ DE JUAZEIRO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2837, 3134, 'FACULDADE INTEGRADA UNIVERACRUZ DE ITABERABA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2838, 3135, 'FACULDADE INTEGRADA UNIVERACRUZ DE TEIXEIRA DE FREITAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2839, 3136, 'FACULDADES INTEGRADA UNIVERACRUZ DE IPIR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2840, 3137, 'FACULDADE INTEGRADA UNIVERACRUZ DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2841, 3138, 'FACULDADE DINMICA DE ITAIPULNDIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2842, 3139, 'INSTITUTO DE SUPERIOR DE EDUCAO FRANCISCO AGUIAR', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2843, 3140, 'FACULDADE ZEUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2844, 3141, 'FACVALE - FACULDADE DO VALE DO JACUPE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2845, 3142, 'INSTITUTO SUPERIOR DE EDUCAO EDEM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2846, 3143, 'FACULDADE DO VAREJO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2847, 3144, 'FACULDADE GUAS LINDAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2848, 3145, 'FACULDADE DE DIREITO SO JUDAS TADEU DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2849, 3146, 'FACULDADE DE ODONTOLOGIA DO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2850, 3147, 'FACULDADE DE DIREITO DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2851, 3148, 'FACULDADE ANCHIETA DO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2852, 3149, 'FACULDADES INTEGRADAS DO OESTE DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2853, 3150, 'INSTITUTO SUPERIOR DE EDUCAO DA FACULDADE DE CINCIAS E LETRAS DE ARARAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2854, 3151, 'UNIVERSIDADE COMUNITRIA REGIONAL DE CHAPEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2855, 3152, 'INSTITUTO SUPERIOR DE EDUCAO SANTA RITA DO SAPUCA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2856, 3153, 'INSTITUTO SUPERIOR DE EDUCAO CANTAREIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2857, 3154, 'FACULDADE DE TECNOLOGIA SENAI RIO DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2858, 3155, 'FACULDADE DE TECNOLOGIA SENAI JARAGU DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2859, 3156, 'FACULDADE DE TECNOLOGIA SENAI BRUSQUE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2860, 3157, 'INSTITUTO SUPERIOR DE EDUCAO ALMEIDA RODRIGUES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2861, 3158, 'INSTITUTO SUPERIOR DE EDUCAO ANGLO-AMERICANO DE FOZ DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2862, 3159, 'FACULDADE DE TECNOLOGIA SENAI FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2863, 3160, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE ALAGOAS', 1, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2864, 3161, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO SERTO PERNAMBUCANO', 1, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2865, 3162, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA  DE SANTA CATARINA', 1, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2866, 3163, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO RIO DE JANEIRO', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2867, 3164, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE MATO GROSSO', 1, 1, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2868, 3165, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA  DO TRINGULO MINEIRO', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2869, 3166, 'INSTITUTO SUPERIOR DE EDUCAO DE IBINA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2870, 3167, 'FACULDADE DE TECNOLOGIA ESTCIO DE S DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2871, 3168, 'FACULDADE DE TECNOLOGIA ESTCIO DE S DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2872, 3169, 'FACULDADE DE TECNOLOGIA IBTA CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2873, 3170, 'FACULDADES INTEGRADAS ASSOCIAO DE ENSINO DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2874, 3171, 'FACULDADE EQUIPE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2875, 3172, 'UNIVERSIDADE DO ESTADO DO AMAZONAS', 2, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2876, 3173, 'FACULDADE DE TECNOLOGIA DO COMRCIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2877, 3174, 'INSTITUTO SUPERIOR DE EDUCAO DE VINHEDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2878, 3175, 'INSTITUTO SUPERIOR DE EDUCAO DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2879, 3176, 'INSTITUTO SUPERIOR DE EDUCAO EQUIPE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2880, 3177, 'FACULDADE DE ADMINISTRAO DA FESPSP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2881, 3178, 'INSTITUTO DE EDUCAO SUPERIOR DE BARUERI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2882, 3179, 'FACULDADE DA CIDADE DE SO GONALO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2883, 3180, 'INSTITUTO SUPERIOR DE EDUCAO UNIO DAS AMRICAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2884, 3181, 'FACULDADE DE DIREITO DE BOA VISTA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2885, 3182, 'CENTRO DE ENSINO SUPERIOR ARCANJO MIKAEL DE ARAPIRACA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2886, 3183, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE SERGIPE', 1, 1, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2887, 3184, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE RORAIMA', 1, 1, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2888, 3185, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA DE SO VICENTE DO SUL', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2889, 3186, 'FACULDADE EVOLUO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2890, 3187, 'INSTITUTO SUPERIOR DE EDUCAO CORAO DE JESUS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2891, 3188, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO NORTE DE MINAS GERAIS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2892, 3189, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE MINAS GERAIS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2893, 3190, 'INSTITUTO SUPERIOR DE EDUCAO DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2894, 3191, 'INSTITUTO SUPERIOR DE EDUCAO DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2895, 3192, 'INSTITUTO SUPERIOR DE EDUCAO DE CAJAZEIRAS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2896, 3193, 'FACULDADES INTEGRADAS DOS CAMPOS GERAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2897, 3194, 'FACULDADE DE MINAS BH', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2898, 3195, 'FACULDADE VILA RICA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2899, 3196, 'FACULDADE PONTES E LACERDA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2900, 3197, 'FACULDADES DO PARECIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2901, 3198, 'FACULDADE DE CINCIAS HUMANAS DE IVAIPOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2902, 3199, 'FANESF - FACULDADE NACIONAL DE ENSINO SUPERIOR DA FAZENDA RIO GRANDE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2903, 3200, 'FANPINHAIS - FACULDADE NACIONAL DE PINHAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2904, 3201, 'FACULDADE DE EDUCAO E ENSINO SUPERIOR DO ABRANCHES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2905, 3202, 'FACULDADES AUGUSTUS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2906, 3203, 'INSTITUTO SUPERIOR DE EDUCAO ALBERT EINSTEIN', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2907, 3204, 'FACULDADE DE QUATRO MARCOS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2908, 3205, 'FACULDADE DE CINCIAS BIOLGICAS E DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2909, 3206, 'INSTITUTO DO SABER', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2910, 3207, 'FACULDADES DE CAFELNDIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2911, 3208, 'FACULDADE AMERICA DO SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2912, 3209, 'FACULDADE DE ARAATUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2913, 3210, 'INSTITUTO EDUCACIONAL EVANGELICO DE ARAPONGAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2914, 3211, 'FACDIPAR-FACULDADE DE DIREITO DO PARANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2915, 3212, 'INSTITUTO SUPERIOR DE FILOSOFIA BERTHIER', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2916, 3213, 'AFIMA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2917, 3214, 'FACULDADE EUROPEU DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2918, 3215, 'FACULDADE RELIGARE TEOLGICA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2919, 3216, 'FACULDADE DE CINCIAS DA BAHIA - FACIBA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2920, 3217, 'FACULDADE DE ENSINO SUPERIOR DO BRASIL', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2921, 3218, 'FACULDADE DE CINCIAS DA SADE DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2922, 3219, 'FACULDADE EVANGLICA DE INDAIATUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2923, 3220, 'INSTITUTO NACIONAL DE EDUCAO SUPERIOR DE EDUCAO E CULTURA - INESC', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2924, 3221, 'FACULDADE DE ODONTOLOGIA DE MACAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2925, 3222, 'FACULDADE DE ODONTOLOGIA DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2926, 3223, 'ESCOLA SUPERIOR DE CINCIAS DA SADE', 2, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2927, 3224, 'FACULDADE DE FARMCIA E BIOQUMICA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2928, 3225, 'FACULDADE DE ENFERMAGEM', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2929, 3226, 'FACULDADE DE MEDICINA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2930, 3227, 'FACULDADE DE SISTEMA DE INFORMAO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2931, 3228, 'FACULDADE DE ADMINISTRAO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2932, 3229, 'FACULDADE DE TURISMO DE DUQUE DE CAXIAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2933, 3230, 'FACULDADE LUIZ EDUARDO MAGALHES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2934, 3231, 'INSTITUTO SUPERIOR DE EDUCAO PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2935, 3232, 'FACULDADE IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2936, 3233, 'FACULDADE AMRICA DO SUL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2937, 3234, 'URN. FACULDADE DE ALAGOINHAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2938, 3235, 'INSTITUTO SUPERIOR DE EDUCAO JOO PAULO PRIMEIRO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2939, 3236, 'INSITUTO DE TEOLOGIA DA DIOCESE DE SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2940, 3237, 'FACULDADE SENAC DE ADMINISTRAO E NEGCIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2941, 3238, 'FACULDADE DE ODONTOLOGIA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2942, 3239, 'INSTITUTO DE DESENVOLVIMENTO EDUCACIONAL SUL CATARINENSE S/C LTDA.', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2943, 3240, 'INSTITUTO SUPERIOR DE EDUCAO DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2944, 3241, 'FACULDADE NACIONAL DE EDUCAO E ENSINO SUPERIOR DO BRASIL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2945, 3242, 'FACULDADE ALFA BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2946, 3243, 'FACULDADE NACIONAL CAMPO LARGO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2947, 3244, 'FACULDADE DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2948, 3245, 'FACULDADE DE SENADOR CANEDO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2949, 3246, 'FACULDADE DE LINHARES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2950, 3247, 'FACULDADE VITRIA DO ESPRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2951, 3248, 'FACULDADE DE DOMINGOS MARTINS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2952, 3249, 'FACULDADE PADRAO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2953, 3250, 'FACULDADE DE MORRINHOS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2954, 3251, 'FACULDADE DE VILA VELHA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2955, 3252, 'FACULDADE QUIRINPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2956, 3253, 'FACULDADE COSTA ESMERALDA ITAPEMA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2957, 3254, 'FACULDADE DE EDUCAO E TEOLOGIA VIVA - FATEV', 1, 1, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2958, 3255, 'FACULDADE INPG', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2959, 3256, 'INSTITUTO SUPERIOR DE ECOLOGIA HUMANA E DESENVOLVIMENTO SUSTENTVEL- ISEDS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2960, 3257, 'FAESSF - FACULDADE DE ENSINO SUPERIOR SANTA FELICIDADE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2961, 3258, 'FACULDADE DE DIREITO DE RESENDE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2962, 3259, 'FACULDADES DO ICESP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2963, 3260, 'FACULDADE PRINCESA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2964, 3261, 'FACULDADE DE EDUCAO E ENSINO SUPERIOR INTEGRAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2965, 3262, 'FACULDADE DOM BOSCO DE GUARANIAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2966, 3263, 'FACULDADE YVERDUN BRASILIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2967, 3265, 'FACULDADE BATISTA IDA NELSON', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2968, 3266, 'UNIDADE VOCACIONAL CIDADES - UVC', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2969, 3267, 'FACULDADE TADEU ARGOLO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2970, 3268, 'FACULDADE MADRE THAIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2971, 3269, 'FACULDADE DE JUAZEIRO DO NORTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2972, 3270, 'FACULDADE SO BENTO DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2973, 3271, 'FACULDADE QUASAR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2974, 3272, 'FACULDADE DE DIREITO DE PETROLINA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2975, 3273, 'FACULDADE DO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2976, 3274, 'FACULDADE PSICOLOGIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2977, 3275, 'FACULDADE JURIDICA DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2978, 3276, 'FACULDADE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2979, 3277, 'FACULDADE DE TECNOLOGIA PREVE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2980, 3278, 'FACULDADE DE TECNOLOGIA MARTINUS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2981, 3279, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO SUDESTE DE MINAS GERAIS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2982, 3280, 'FACULDADE SO JOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2983, 3281, 'FACULDADE DO DESENVOLVIMENTO DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2984, 3282, 'FACULDADE PONTAL DO PARANAPANEMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2985, 3283, 'INSTITUTO BRASILEIRO DE ZOOTECNOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2986, 3284, 'ESCOLA SUPERIOR DE ADMINISTRAO E NEGCIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2987, 3285, 'FACULDADE SANTO ANTONIO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2988, 3286, 'FACULDADE DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2989, 3287, 'FACULDADE DA ASSEDIPA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2990, 3288, 'FACULDADE GENESIS - FAGEN', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2991, 3289, 'FACULDADE DE CINCIAS ADMINISTRATIVAS E CONTBEIS COSTA BRAGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2992, 3290, 'INSTITUTO SUPERIOR DE EDUCAO DOS INCONFIDENTES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2993, 3291, 'FACULDADE DUARTE COELHO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2994, 3292, 'INSTITUTO SUPERIOR DE EDUCAO DE CAET', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2995, 3293, 'FACULDADE DE TECNOLOGIA DIAMANTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2996, 3294, 'FACULDADE DE TECNOLOGIA ENIAC-FAPI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2997, 3295, 'FACULDADE DE TECNOLOGIA SENAC FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2998, 3296, 'FACULDADE DE TECNOLOGIA SENAC RIO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (2999, 3297, 'CENTRO DE EDUCAO TECNOLGICA CHRISTUS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3000, 3298, 'FACULDADE DE TECNOLOGIA CAMES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3001, 3299, 'FACULDADE DE TECNOLOGIA INFORMTICA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3002, 3300, 'FACULDADE DE TECNOLOGIA MICHEL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3003, 3301, 'FACULDADE DE TECNOLOGIA MAGISTER', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3004, 3302, 'INSTITUTO SUPERIOR DE EDUCAO DE JUNQUEIRPOLIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3005, 3303, 'FACULDADES INTEGRADAS MATO-GROSSENSES DE CINCIAS SOCIAIS E HUMANAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3006, 3304, 'FACULDADES INTEGRADAS DE PATOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3007, 3305, 'FACULDADE PITGORAS DE PSICOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3008, 3306, 'FACULDADE DE TECNOLOGIA IPUC', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3009, 3307, 'FACULDADE DE TECNOLOGIA DE RIO CLARO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3010, 3308, 'FACULDADE DE TECNOLOGIA TERMOMECNICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3011, 3309, 'FACULDADE DE TECNOLOGIA INTERNACIONAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3012, 3310, 'FACULDADE DE TECNOLOGIA DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3013, 3311, 'FACULDADE DE TECNOLOGIA SO FRANCISCO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3014, 3312, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA DE RIO VERDE', 1, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3015, 3313, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA DE BAMBU', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3016, 3314, 'FACULDADE DE CINCIAS GERENCIAIS DA FUNCEC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3017, 3315, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3018, 3316, 'FACULDADE SUDOESTE PAULISTANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3019, 3317, 'FACULDADE MASTER', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3020, 3318, 'FACULDADE MASTER', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3021, 3319, 'FACULDADE DE ENSINO SUPERIOR DE CATALO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3022, 3320, 'INSTITUTO DE TELEMARKETING E SERVIO AO CLIENTE LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3023, 3321, 'FACIVALE -FACULDADE INTERNACIONAL DO VALE DO IVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3024, 3322, 'FACULDADE POLITCNICA DE ATIBAIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3025, 3323, 'INSTITUTO DE EDUCAO E ENSINO SUPERIOR DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3026, 3324, 'FACULDADE TALENTI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3027, 3325, 'FACULDADE COMUNITRIA DE CONTAGEM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3028, 3326, 'CENTRO SUPERIOR DE ESTUDOS JURDICOS CARLOS DRUMMOND DE ANDRADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3029, 3327, 'UNIPONTE - FACULDADE DA MEIA PONTE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3030, 3328, 'CENTRO DE ESTUDOS SUPERIORES EM CINCIAS DA SADE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3031, 3329, 'FACULDADE DE DIREITO DE PAR DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3032, 3330, 'INSTITUTO DE LNGUA PORTUGUESA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3033, 3331, 'FAMOL - FACULDADE MONTEIRO LOBATO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3034, 3332, 'FACULDADE DE TECNOLOGIA SENAC RIO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3035, 3333, 'FACULDADE DE TECNOLOGIA TECBRASIL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3036, 3334, 'FACULDADE DE TECNOLOGIA SO GONALO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3037, 3335, 'FACULDADE DE CINCIAS E LETRAS DO NOROESTE DO PARAN FACIP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3038, 3336, 'UNIVERSIDADE ESTADUAL DO RIO GRANDE DO SUL', 2, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3039, 3337, 'FACULDADE DE TECNOLOGIA DE TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3040, 3338, 'INSTITUTO SUPERIOR DE EDUCAO GETLIO VARGAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3041, 3339, 'FACULDADE DE GETLIO VARGAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3042, 3340, 'FACULDADE DE PAULISTA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3043, 3341, 'FACULDADES DE ALTO ARAGUAIA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3044, 3342, 'FACULDADE EVANGLICA DE CINCIA FILOSOFICA E TEOLGICA DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3045, 3343, 'FACULDADES DO PARECIS - FACCIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3046, 3344, 'FACULDADE REGIONAL DE IREC', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3047, 3345, 'FACULDADE YVERDON DE GOINIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3048, 3346, 'FACULDADE YVERDON', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3049, 3347, 'FACULDADE REGIONAL DE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3050, 3348, 'FACULDADE REGIONAL DE SERRINHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3051, 3349, 'FACULDADE REGIONAL METROPOLITANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3052, 3350, 'FACULDADE REGIONAL DE ALAGOINHAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3053, 3351, 'INSTITUTO SUPERIOR DO NOROESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3054, 3352, 'FACULDADE REGIONAL DE SANTO AMARO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3055, 3353, 'FACULDADE RECON.BAHIA EMPRESARIAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3056, 3354, 'FACULDADE REGIONAL DE ILHUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3057, 3355, 'FACULDADE REGIONAL DE SO FLIX', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3058, 3356, 'INSTITUTO TEOLGICO DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3059, 3357, 'FACULDADE DO SERTO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3060, 3358, 'FACULDADE CIENTFICA DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3061, 3359, 'FESC - FACULDADE DO EXTREMO SUL CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3062, 3360, 'INSTITUTO DE EDUCAO SUPERIOR PRESIDENTE KENNEDY - CENTRO DE FORMAO DE PROFISSIONAIS DE EDUCAO', 2, 1, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3063, 3361, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3064, 3362, 'INSTITUTO DE CINCIAS HUMANAS E SOCIAIS APLICADAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3065, 3363, 'FACULDADE GUARA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3066, 3364, 'FACULDADE DE ENFERMAGEM DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3067, 3365, 'FACULDADE SO FRANCISCO DE JUAZEIRO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3068, 3366, 'FACULDADE DE CINCIAS GERENCIAIS DE OLIVEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3069, 3367, 'INSTITUTO SUPERIOR DE EDUCAO DE OLIVEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3070, 3368, 'CENTRO UNIVERSITRIO DO SUL DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3071, 3369, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS GEREMRIO DANTAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3072, 3370, 'ESCOLA DOMINICANA DE TEOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3073, 3371, 'CENTRO UNIVERSITRIO DE PATOS DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3074, 3372, 'CENTRO UNIVERSITRIO DE LAVRAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3075, 3373, 'INSTITUTO SUPERIOR DE EDUCAO DE FORMIGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3076, 3374, 'FACULDADE PAUL SABATIER DE EDUCAO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3077, 3375, 'INSTITUTO DE ENSINO SUPERIOR MLTIPLO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3078, 3376, 'FACULDADE DE TEOLOGIA INTEGRADA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3079, 3377, 'FACULDADE MONTESSORIANO DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3080, 3378, 'FACULDADE CENECISTA DE RECIFE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3081, 3379, 'INSTITUTO DE EDUCAO SUPERIOR SERRANO - IESSER', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3082, 3380, 'FACULDADE MISSIONEIRA DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3083, 3381, 'UNITEC FACULDADE TECNOLGICA E CIENTFICA DE UNA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3084, 3382, 'ESCOLA SUPERIOR DE ENFERMAGEM E PEDAGOGIA MARIA PASTOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3085, 3383, 'INSTITUTO SUPERIOR DE EDUCAO SAGRA - ISAGRA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3086, 3384, 'FACULDADE SAGRA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3087, 3385, 'FACULDADE DE DIREITO PROFESSOR PEDRO ALEIXO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3088, 3386, 'FACULDADE DE ADMINISTRAO DE EMPRESAS - UNITECH', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3089, 3387, 'FACULDADE NACIONAL DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3090, 3388, 'FACULDADE PARASO DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3091, 3389, 'FACULDADE RAZES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3092, 3390, 'FACULDADE DE DIREITO DO ICESP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3093, 3391, 'INSTITUTO SUPERIOR DE EDUCAO NOVA VIDA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3094, 3392, 'FACULDADE DA ACADEMIA BRASILEIRA DE EDUCAO E CULTURA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3095, 3393, 'FACULDADE DE ENSINO SUPERIOR DOM BOSCO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3096, 3394, 'FACULDADE VALE DO IPIRANGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3097, 3395, 'FACULDADE EDUCIONAL SUL PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3098, 3396, 'FACULDADE LS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3099, 3397, 'FACULDADE BOAS NOVAS DE CINCIAS TEOLGICAS, SOCIAIS E BIOTECNOLGICAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3100, 3398, 'FACULDADE DE CINCIAS DO ESTADO DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3101, 3399, 'FACULDADE DE TUBARO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3102, 3400, 'FACULDADE DE SADE DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3103, 3401, 'FACULDADE DE ITAPECERICA DA SERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3104, 3402, 'FACULDADE ANGLICANA DE NEGCIOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3105, 3403, 'INSTITUTO SUPERIOR DE EDUCAO RUI BARBOSA - ISURB', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3106, 3404, 'FACULDADE DE FILOSOFIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3107, 3405, 'FACULDADE RUI BARBOSA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3108, 3406, 'FACULDADE CNDIDO RONDON', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3109, 3407, 'IESEMA - INSTITUTO SUPERIOR DE EDUCAO DA MATA ALAGOANA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3110, 3408, 'FACULDADE AMAZNIA DE RORAIMA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3111, 3409, 'FACULDADE DE CINCIAS DA SADE CRISTO REI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3112, 3410, 'INSTITUTO DE EDUCAO SUPERIOR FATEV', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3113, 3411, 'FACULDADE EA DE QUEIROS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3114, 3412, 'FACULDADE DO INTERIOR PAULISTA DE TANABI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3115, 3413, 'ISEMP - INSTITUTO SUPERIOR DE EDUCAO DA MATA PERNAMBUCANA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3116, 3414, 'FACULDADE DO ENTORNO DE BRASLIA - FAEB', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3117, 3415, 'FACULDADE DE TECNOLOGIA DE CAXIAS DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3118, 3416, 'FACULDADE JK DO GAMA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3119, 3417, 'FACULDADE DO CENTRO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3120, 3418, 'FACULDADE VALE DO CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3121, 3419, 'FACULDADE CHROMOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3122, 3420, 'INSTITUTO SUPERIOR DE EDUCAO DE CAMPOS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3123, 3421, 'FACULDADE DE MANDAGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3124, 3422, 'FACULDADE TEOLGICA DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3125, 3423, 'FACULDADE DE ADMINISTRAO DE NOVO PROGRESSO', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3126, 3424, 'FACULDADE DE LAGES', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3127, 3425, 'FACETE - FACULDADE DE ARTES, CINCIAS E TECNOLOGIAS DE EUNPOLIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3128, 3426, 'ESCOLA SUPERIOR DE CINCIAS TRADICIONAIS E AMBIENTAIS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3129, 3427, 'FACULDADE JOS AUGUSTO VIEIRA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3130, 3428, 'FACULDADE ALFA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3131, 3429, 'FACULDADE RIO DAS CONTAS - FAC-RIO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3132, 3430, 'FACULDADE DE MARKETING E NEGCIOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3133, 3431, 'FACULDADE DO SERID', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3134, 3432, 'FACULDADES INTEGRADAS DE CINCIAS HUMANAS, SADE E EDUCAO DE GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3135, 3433, 'INSTITUTO SUPERIOR DE EDUCAO INTERLAGOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3136, 3434, 'FACULDADE DE SADE IBITURUNA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3137, 3435, 'INSTITUTO SUPERIOR DE EDUCAO NORTE MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3138, 3436, 'FACULDADE SAVONITTI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3139, 3437, 'FACULDADE DO LITORAL CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3140, 3438, 'FACULDADE MASTER DE PALMAS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3141, 3439, 'FACULDADE MASTER DE PARAUAPEBAS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3142, 3440, 'FACULDADE MASTER DE SANTARM', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3143, 3441, 'FACULDADE MASTER DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3144, 3442, 'INSTITUTO DE ENSINO ALVARES DE AZEVEDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3145, 3443, 'FACULDADE INEDI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3146, 3444, 'FACULDADE DE CINCIAS BIOLGICAS E SOCIAIS APLICADAS JEFFERSON JORGE SALOMO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3147, 3445, 'FACULDADE JOO DO VALE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3148, 3446, 'INSTITUTO SUPERIOR DE EDUCAO JOO DO VALE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3149, 3447, 'FACULDADES SANTAMARIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3150, 3448, 'INSTITUTO SUPERIOR DE EDUCAO IBITURUNA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3151, 3449, 'TRECSON FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3152, 3450, 'FACULDADES MASTER', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3153, 3451, 'INSTITUTO SUPERIOR DE EDUCAO DO LITORAL CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3154, 3453, 'FACULDADE JUSCELINO KUBTSCHEK GUAR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3155, 3454, 'FACULDADES LEONEL AGUIAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3156, 3455, 'FACULDADE DE EDUCAO EXPOVEST', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3157, 3456, 'FACULDADE POLITCNICA DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3158, 3457, 'FESC- FACULDADE DO EXTREMO SUL CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3159, 3458, 'ACADEMIA TEOLGICA DA GRAA DE DEUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3160, 3459, 'INSTITUTO DE ENSINO SUPERIOR E PESQUISA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3161, 3460, 'INSTITUTO SUPERIOR DE ENSINO E PESQUISA DE ITUIUTABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3162, 3461, 'FACULDADE DE FILOSOFIA DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3163, 3462, 'FACULDADE DE ENFERMAGEM DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3164, 3463, 'FACULDADE DE DIREITO DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3165, 3464, 'FACULDADE DE ENGENHARIA DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3166, 3465, 'FACULDADE DE INFORMTICA DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3167, 3466, 'FACULDADE DE SERVIO SOCIAL DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3168, 3467, 'FACULDADE DE NUTRIO DA FUNDAO DE ENSINO SUPERIOR DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3169, 3468, 'FACULDADE DE ENGENHARIA DO TRINGULO MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3170, 3469, 'FACULDADE DE ADMINISTRAO DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3171, 3470, 'FACULDADE EDUCAO FSICA DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3172, 3471, 'FACULDADE DE MODA DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3173, 3472, 'INSTITUTO SUPERIOR DE EDUCAO DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3174, 3473, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DE CARANGOLA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3175, 3474, 'FACULDADE DE CINCIAS EXATAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3176, 3475, 'INSTITUTO SUPERIOR DE EDUCAO DE CARANGOLA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3177, 3476, 'FACULDADE DE FILOSOFIA E LETRAS DE DIAMANTINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3178, 3477, 'FACULDADE DE FILOSOFIA CINCIAS E LETRAS NOSSA SENHORA DE SION', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3179, 3478, 'FACULDADE DE CINCIAS EXATAS E HUMANAS DA CAMPANHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3180, 3479, 'INSTITUTO SUPERIOR DE EDUCAO NOSSA SENHORA DE SION', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3181, 3480, 'FACULDADE DE CINCIAS JURDICAS DE DIAMANTINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3182, 3481, 'INSTITUTO SUPERIOR DE EDUCAO DE DIAMANTINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3183, 3482, 'INSTITUTO SUPERIOR DE EDUCAO DE DIVINPOLIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3184, 3483, 'INSTITUTO SUPERIOR DE EDUCAO DE CLUDIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3185, 3484, 'FACULDADE DE CINCIAS GERENCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3186, 3485, 'INSTITUTO SUPERIOR DE EDUCAO DO ALTO SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3187, 3486, 'INSTITUTO SUPERIOR DE EDUCAO DE ITUIUTABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3188, 3487, 'FACULDADE SENAC/RS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3189, 3488, 'FACULDADE DE CINCIAS ECONMICAS DE CONSELHEIRO LAFAIETE', 3, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3190, 3489, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE CONSELHEIRO LAFAIETE', 3, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3191, 3490, 'FACULDADE DE ENGENHARIA ELTRICA DE CONSELHEIRO LAFAIETE', 3, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3192, 3491, 'FACULDADE EVANGLICA DO BRASIL (FAEB)', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3193, 3492, 'FACULDADE DE DIREITO DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3194, 3493, 'FACULDADE PROJETO DIDTICO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3195, 3494, 'FACULDADE GUAJARAENSE DE EDUCAO E CULTURA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3196, 3495, 'FACULDADE PINHALZINHO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3197, 3496, 'INSTITUTO NACIONAL DE DESENVOLVIMENTO HUMANO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3198, 3497, 'FACULDADE BRASILIENSE DE DIREITO - FADIR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3199, 3498, 'FACULDADE DE ARTES, TECNOLOGIA E NEGCIOS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3200, 3499, 'INSTITUTO BOLSA DE VALORES', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3201, 3500, 'FACULDADE DO SERTO BAIANO - FASB', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3202, 3501, 'FACULDADE DE ADMINISTRAO DE SANTO ANTNIO DO MONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3203, 3502, 'FACULDADE DA FRONTEIRA - FAF', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3204, 3503, 'FACUDADE DA CINCIA DA SAUDE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3205, 3504, 'ESCOLA DE ENSINO SUPERIOR DE BRASILNDIA DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3206, 3505, 'INSTITUTO SUPERIOR DE EDUCAO DE JACUTINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3207, 3506, 'FACULDADE DE CINCIAS EDUCACIONAIS DE SERGIPE', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3208, 3507, 'FACULDADE DE ADMINISTRAO DE EMPRESAS IBRAFEM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3209, 3508, 'FACULDADE SANTO ANTONIO DE QUEIMADAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3210, 3509, 'FACULDADE VICENTINA - FAVI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3211, 3510, 'UNASP.FACULDADE DO SERVIDOR PBLICO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3212, 3511, 'FACULDADES COMUNITRIA UNIDAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3213, 3512, 'ILES - INSTITUTO LOGOS DE EDUCAO E ENSINO SUPERIOR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3214, 3513, 'FACULDADE DE PRESIDENTE VENCESLAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3215, 3514, 'FACULDADE DE CINCIAS GERENCIAIS ALVES FORTES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3216, 3515, 'FACULDADES INTEGRADAS DA VITRIA DE SANTO ANTO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3217, 3516, 'FACULDADE DE CINCIAS DA SADE ARCHIMEDES THEODORO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3218, 3517, 'FACULDADE IEGS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3219, 3518, 'FACULDADE JANGADA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3220, 3519, 'FACULDADE DE CINCAS GERENCIAIS DO ALTO PARAOPEBA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3221, 3520, 'INSTITUTO DE EDUCAO SUPERIOR SANTA MARIA (IESSAM)', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3222, 3521, 'FACULDADE EXPOENTE DE EDUCAO E PESQUISA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3223, 3522, 'FACULDADE CATLICA DE POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3224, 3523, 'ESCOLA SUPERIOR DO MINISTRIO PBLICO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3225, 3524, 'UNIBUSINESS - FACULDADE INTERNACIONAL DE EDUCAO E GERNCIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3226, 3525, 'FACULDADE EVANGLICA DE TECNOLOGIA, CINCIAS E BIOTECNOLOGIA DA CGADB', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3227, 3526, 'INSTITUTO DE ENSINO SUPERIOR MIGUEL DE CERVANTES', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3228, 3527, 'FACULDADE CATLICA DOM AQUINO DE CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3229, 3528, 'FACULDADES CATLICAS DE FRANCA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3230, 3529, 'CENTRO UNIVERSITRIO EURPEDES DE MARLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3231, 3530, 'FACULDADE DE NANUQUE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3232, 3531, 'FACULDADE CACHOEIRO DE ITAPEMIRIM', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3233, 3532, 'FACULDADE DO OESTE CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3234, 3533, 'FACULDADE CERES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3235, 3534, 'FACULDADE BRASILEIRA DE ENSINO SUPERIOR E TECNOLOGIA - FABEST', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3236, 3535, 'FACULDADE PAULISTA DE CINCIAS FARMACUTICAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3237, 3536, 'FACULDADE DE TEOLOGIA EVANGLICA EM CURITIBA - FATEV', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3238, 3537, 'INSTITUTO BRASILEIRO DE MODA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3239, 3538, 'FACULDADES INTEGRADAS SVIGN', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3240, 3539, 'FACULDADE SANTA LCIA - MOGI GUAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3241, 3540, 'UNIO DE FACULDADES DE ADMINISTRAO PBLICA - UNIFAP', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3242, 3541, 'FACULDADE NOTRE DAME', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3243, 3542, 'INSTITUTO DE ENSINO SUPERIOR DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3244, 3543, 'FACULDADE NOVO HAMBURGO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3245, 3544, 'FACULDADE DE FISIOTERAPIA MACHADO DE ASSIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3246, 3546, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE BARRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3247, 3547, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE ITABUNA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3248, 3548, 'FACULDADE SUA DE HOTELARIA DE JOINVILLE S.A.', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3249, 3549, 'FACULDADE NACIONAL SNIOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3250, 3550, 'FACULDADE SAO FRANCISCO DE CAJAZEIRAS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3251, 3551, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE BARREIRAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3252, 3552, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE TEIXEIRA DE FREITAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3253, 3553, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE VALENA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3254, 3554, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE JACOBINA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3255, 3555, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE ALAGOINHAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3256, 3556, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3257, 3557, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE IRECE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3258, 3558, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE IPIR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3259, 3559, 'FACULDADE DE PORANGATU', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3260, 3560, 'FACULDADE DE CINCIAS DA SADE DE CAMPINA GRANDE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3261, 3561, 'FACULDADE SANTA RITA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3262, 3562, 'FACULDADE DE ODONTOLOGIA RORAIMA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3263, 3563, 'FACULDADE DE ODONTOLOGIA RIO BRANCO', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3264, 3564, 'FACULDADE MORENENSE DE EDUCAO-FAM', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3265, 3565, 'FACULDADE MORENENSE DE EDUCAO-FAM', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3266, 3566, 'FACULDADE PAULO FREIRE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3267, 3567, 'FMV - FACULDADE METROPOLITANA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3268, 3568, 'FACULDADE DE DESENVOLVIMENTO E INTEGRAO REGIONAL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3269, 3569, 'INSTITUTO SUPERIOR DE EDUCAO DE MANHUAU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3270, 3570, 'FACULDADE DE CINCIAS E EDUCAO DE SO SIMO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3271, 3571, 'FACULDADE DOS INCONFIDENTES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3272, 3572, 'FACULDADE DE DIREITO DA ACADEMIA BRASILEIRA DE CINCIAS JURDICAS E SOCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3273, 3573, 'FACULDADE TEOLGICA BATISTA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3274, 3574, 'UNIO DE FACULDADES DE PARASO - UNIFAP', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3275, 3575, 'FACULDADE ATIVA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3276, 3576, 'FACULDADE DE ENSINO SUPERIOR DO LITORAL SUL - FASUL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3277, 3577, 'FACULDADE DE ALTOS ESTUDOS DA BAHIA - FAE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3278, 3578, 'FACULDADE DE CIENCIAS FARMACEUTICAS DO PANTANAL', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3279, 3579, 'INSTITUTO LUTERANO DE EDUCAO SUPERIOR DO PARECIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3280, 3580, 'FACULDADE DE EDUCAO, CINCIAS SOCIAIS E DA SADE - SESI/TO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3281, 3581, 'FACULDADE DE ENSINO TECNOLGICO FILADLFIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3282, 3582, 'FACULDADE PORTO CALVO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3283, 3583, 'ESCOLAS SUPERIORES DE ADMINISTRAO, GEOGRAFIA E CURSO NORMAL SUPERIOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3284, 3584, 'INSTITUTO SUPERIOR DE EDUCAO SANTA CRUZ', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3285, 3585, 'FACULDADE SANTA CRUZ', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3286, 3586, 'FACULDADE OXFORD DO BRASIL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3287, 3587, 'FACULDADE DIOCESANA SO JOS', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3288, 3588, 'FACULDADE DOM PEDRO II', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3289, 3589, 'FACULDADE TEOREMA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3290, 3590, 'FACULDADE DE ENFERMAGEM SO VICENTE DE PAULA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3291, 3591, 'INSTITUTO SUPERIOR DE EDUCAO DE MACAPARANA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3292, 3593, 'FACULDADE DE CHAPEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3293, 3594, 'FACULDADE DE CINCIA, TECNOLOGIA E EDUCAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3294, 3595, 'INSTITUTO SUPERIOR DE EDUCAO DE SANTA MARIA DA VITRIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3295, 3596, 'FACULDADE MONTSERRAT', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3296, 3597, 'FACULDADE DE ENSINO SUPERIOR DE SANTA CRUZ DO CAPIBARIBE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3297, 3598, 'FACULDADE DE IRATI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3298, 3599, 'FACULDADE DE ITAPEMA - FAI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3299, 3600, 'FACULDADE DO LITORAL PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3300, 3601, 'FACULDADE DE TAPEJARA - FATAP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3301, 3602, 'FACULDADES INTEGRADAS DO BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3302, 3603, 'FACULDADES INTEGRADAS TORRICELLI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3303, 3604, 'INSTITUTO SUPERIOR DE EDUCAO DE CARIACICA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3304, 3605, 'FACULDADE DE CINCIAS, COMUNICAES E TECNOLOGIAS DE PESQUEIRA - FACCIP', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3305, 3606, 'INSTITUTO SUPERIOR DE EDUCAO PRESBITERIANO VALE DO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3306, 3607, 'ESCOLA SUPERIOR ASSOCIADA DE GOINIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3307, 3608, 'INSTITUTO SUPERIOR DE EDUCAO DE LONDRINA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3308, 3609, 'FACULDADES INTEGRADAS IPITANGA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3309, 3610, 'FACULDADES INTEGRADAS UNICESP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3310, 3611, 'FACULDADES INTEGRADAS DA UNIO DE ENSINO SUPERIOR CERTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3311, 3612, 'FACULDADE ANHANGUERA DE VALINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3312, 3613, 'FACULDADES INTEGRADAS IESGO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3313, 3614, 'ESCOLA SUPERIOR DE CINCIAS SOCIAIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3314, 3615, 'FACULDADE BOA VIAGEM - INSTITUTO MATERNO INFANTIL DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3315, 3616, 'FACULDADES PHD', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3316, 3617, 'FACULDADE DE CINCIAS HUMANAS E EXATAS DO SERTO DO SO FRANCISCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3317, 3618, 'FACULDADE MARIO SCHENBERG', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3318, 3619, 'UNESC - UNIO DAS ESCOLAS SUPERIORES DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3319, 3620, 'ESCOLA INTERNACIONAL DE ADMINISTRAO E NEGCIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3320, 3621, 'INSTITUTO DE ENSINO SUPERIOR DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3321, 3622, 'INSTITUTO DE ENSINO SUPERIOR DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3322, 3623, 'FACULDADE DE CINCIAS DA BAHIA - FACIBA (NOME ALTERADO POR SOLICITAO DO MEC)', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3323, 3624, 'PUC MINAS ARCOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3324, 3625, 'FACULDADE CATLICA NOSSA SENHORA DAS VITRIAS', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3325, 3626, 'FACULDADE BRASILEIRA DE EDUCAO E NEGCIOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3326, 3627, 'FACULDADE DE TECNOLOGIA BARO DO RIO BRANCO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3327, 3628, 'FACULDADADE TEOLGICA REFORMADA - FTR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3328, 3629, 'FACULDADADE TEOLGICA REFORMADA - FTR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3329, 3630, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE PAULO AFONSO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3330, 3631, 'FACULDADE DAMAS DA INSTRUO CRIST', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3331, 3632, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE ITABERABA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3332, 3633, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE CAMPO FORMOSO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3333, 3634, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE BOM JESUS DA LAPA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3334, 3635, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE SANTA MARIA DA VITRIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3335, 3636, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE SENHOR DO BONFIM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3336, 3637, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3337, 3638, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE ILHUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3338, 3639, 'FACULDADE DE DIREITO DO INSTITUTO DE ENSINO SUPERIOR DO CIRCUITO DO OURO - FDINESO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3339, 3640, 'FACULDADE DE CINCIAS ECONMICAS DE JEQUI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3340, 3641, 'CENTRO UNIVERSITRIO DA FUNDAO EDUCACIONAL DE BARRETOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3341, 3642, 'FACULDADE INTEGRADA DE PERNAMBUCO - SURUBIM (FACIPE - SURUBIM)', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3342, 3643, 'FACULDADE CATLICA NOSSA SENHORA DAS NEVES', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3343, 3644, 'FACULDADE CATLICA SANTA TERESINHA', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3344, 3645, 'FACULDADE CATLICA SANTA TERESINHA (CADASTRADA INDEVIDAMENTE)', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3345, 3646, 'FACULDADE DE TECNOLOGIA DE OURO FINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3346, 3647, 'FACULDADE DE TECNOLOGIA MARISTA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3347, 3648, 'FACULDADE CENTRO AMRICA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3348, 3649, 'FACULDADE CIDADE VERDE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3349, 3650, 'INSTITUTO SUPERIOR DE EDUCAAO GREGORIANO - ISEG', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3350, 3651, 'FACULDADE DAS GUAS QUENTES - FAQ', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3351, 3652, 'FACULDADE PRESBITERIANA SAMUEL GRAHAM', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3352, 3653, 'FACULDADE PRESBITERIANA SAMUEL GRAHAM', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3353, 3654, 'FACULDADE DE CINCIAS COMUNICAES E TECNOLOGIAS DE FLORESTA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3354, 3655, 'FACULDADE DE JANABA - FAJAN', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3355, 3656, 'FACULDADE NOVA TRINDADE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3356, 3657, 'FACULDADE DE COMPUTAO DE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3357, 3658, 'FACULDADE UIRAPURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3358, 3659, 'FACULDADE DE CINCIAS COMUNICAO E TECNOLOGIA DE SALGUEIRO-FACTS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3359, 3660, 'INSTITUTO SUPERIOR DE EDUCAO DE SALGUEIRO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3360, 3662, 'FACULDADE EVANGLICA LOGOS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3361, 3663, 'FACULDADE EVANGLICA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3362, 3664, 'INSTITUTO SUPERIOR DE EDUCAO DE VARGEM GRANDE DO SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3363, 3665, 'FACULDADE DE JOAABA - FJ', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3364, 3666, 'FACULDADE DE ADMINISTRAO DA AERONUTCA', 1, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3365, 3667, 'FACULDADE FILADLFIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3366, 3668, 'FACULDADE DE ENFERMAGEM E ADMINISTRAO DE JATA - FEAJ', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3367, 3669, 'FACULDADE REGIONAL DE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3368, 3670, 'FACULDADE DE CRICIUMA - FACRI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3369, 3671, 'FACULDADE INTERDISCIPLINAR DE BLUMENAU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3370, 3672, 'FACULDADE DE CAMPOS NOVOS - FACAN', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3371, 3673, 'FACULDADE PREMIER', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3372, 3674, 'INSTITUTO NORMAL SUPERIOR - FBV', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3373, 3675, 'SINAL - FACULDADE DE TEOLOGIA E FILOSOFIA', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3374, 3676, 'FACULDADE DE TECNOLOGIA DO VALE DO RIBEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3375, 3677, 'FACULDADE DE TECNOLOGIA DE ALTA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3376, 3678, 'FACULDADE DE TECNOLOGIA DO VALE DO ITAJA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3377, 3679, 'INSTITUTO DE ENSINO SUPERIOR DE GARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3378, 3680, 'FACULDADE DE TECNOLOGIA DARCY RIBEIRO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3379, 3681, 'FACULDADES DE GARANHUNS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3380, 3682, 'INSTITUTO SUPERIOR DE EDUCAO DO MUNICPIO DE ITAPERUNA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3381, 3683, 'FACULDADE UNILAGOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3382, 3684, 'FUNDAO DE ENSINO SUPERIOR DE CLEVELNDIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3383, 3685, 'FACULDADE DA POLI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3384, 3686, 'FADITU - FACULDADE DE DIREITO DE IT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3385, 3687, 'FACULDADES INTEGRADAS DE JACARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3386, 3688, 'FACULDADES INTEGRADAS DO VALE DO IVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3387, 3689, 'FACULDADE PEDRO JULIO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3388, 3690, 'FACULDADE PEDRO II', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3389, 3691, 'FACULDADE DE TECNOLOGIA TUPY DE SO BENTO DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3390, 3692, 'FACULDADE DE TECNOLOGIA IPEP DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3391, 3693, 'FACULDADE DE TECNOLOGIA IPEP DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3392, 3694, 'FACULDADE DE TECNOLOGIA ESTCIO DE S DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3393, 3695, 'FACULDADE DE TECNOLOGIA ESTCIO DE S DE OURINHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3394, 3696, 'FACULDADE DE TECNOLOGIA DE JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3395, 3697, 'FACULDADE DE TECNOLOGIA SAINT PASTOUS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3396, 3698, 'PORTAL FACULDADES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3397, 3699, 'FACULDADE PORTAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3398, 3700, 'FACULDADE DE CINCIAS JURDICAS DE PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3399, 3701, 'FACULDADE DE CINCIAS GERENCIAIS DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3400, 3702, 'FACULDADES ELO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3401, 3703, 'INSTITUTO SUPERIOR DE ADMINISTRAO E NEGOCIOS DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3402, 3704, 'FACULDADE DO ALTO PARNABA - FAP', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3403, 3705, 'FACULDADE KHALIL LTDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3404, 3706, 'FACULDADE REGIONAL DE POJUCA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3405, 3707, 'FACULDADE INTEGRADAS DO MORENO - FIAM', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3406, 3708, 'FACULDADE INTEGRAL DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3407, 3709, 'INSTUTO DE ENSINO SUPERIOR SO LUCAS', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3408, 3710, 'ESCOLA SUPERIOR NACIONAL DE SEGUROS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3409, 3711, 'FACULDADE CENTRAL DE CRISTALINA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3410, 3712, 'FACULDADE VITRIA', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3411, 3713, 'INSTITUTO DE TEOLOGIA E PASTORAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3412, 3714, 'INSTITUTO TATUIANO DE ENSINO SUPERIOR - ITES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3413, 3715, 'FACULDADE METROPOLITANA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3414, 3716, 'FACULDADE CINCIAS DA VIDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3415, 3717, 'FACULDADE CULTURAL ANJAGUN', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3416, 3718, 'FACULDADE DE CINCIAS HUMANAS JLIO GOMES', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3417, 3719, 'FACULDADE DE CORAO DE JESUS-FADECOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3418, 3720, 'FACULDADE PORTO REAL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3419, 3721, 'FACULDADE OLIVEIRA TELLES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3420, 3722, 'FACULDADE DE CINCIAS MDICAS INTEGRADAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3421, 3723, 'FACULDADE NACIONAL POLICURSOS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3422, 3724, 'FACULDADE SO JOS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3423, 3725, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE ITAMARAJU', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3424, 3726, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE JEQUI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3425, 3727, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE EUCLIDES DA CUNHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3426, 3728, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3427, 3729, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE GUANAMBI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3428, 3730, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE CAMAARI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3429, 3731, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE BRUMADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3430, 3732, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE VITRIA DA CONQUISTA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3431, 3733, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE JUAZEIRO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3432, 3734, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE SERRINHA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3433, 3735, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE EUNPOLIS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3434, 3736, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE PILO ARCADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3435, 3737, 'FACULDADE INTEGRADA DE ADMINISTRAO PBLICA DE PORTO SEGURO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3436, 3738, 'FACULDADE DE EDUCAO, CINCIAS E LETRAS DO NOROESTE DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3437, 3739, 'FACULDADE ASSOCIAO HOSPITALAR DE ILHA SOLTEIRA - FACISA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3438, 3740, 'FACULDADE DE MEDICINA DE BOTUCATU', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3439, 3741, 'FACULDADE DE EDUCAO DE BACABAL - FEBAC', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3440, 3742, 'FACULDADES DE CINCIAS E TECNOLOGIA DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3441, 3743, 'FACULDADE DE TECNOLOGIA SO FRANCISCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3442, 3744, 'FACULDADE CETEP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3443, 3745, 'INSTITUTO SUPERIOR SEJA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3444, 3746, 'FACULDADE PAULISTA DE PESQUISA E ENSINO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3445, 3747, 'FACULDADE METROPOLITANA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3446, 3748, 'FACULDADE DE TECNOLOGIA THEREZA PORTO MARQUES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3447, 3749, 'FACULDADE DE TECNOLOGIA BRASLIA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3448, 3750, 'FACULDADE DE TECNOLOGIA DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3449, 3751, 'FACULDADE DE TECNOLOGIA DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3450, 3752, 'FACULDADE DE TECNOLOGIA CONTEC', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3451, 3753, 'FACULDADES INTEGRADAS DOM PEDRO II', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3452, 3754, 'FACULDADES INTEGRADAS ADVENTISTAS DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3453, 3755, 'FACULDADE EM AUTOCONHECIMENTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3454, 3756, 'FACULDADE DE ALMENARA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3455, 3757, 'FACULDADE SERRA DA MESA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3456, 3758, 'INSTITUTO SUPERIOR TUPY DE FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3457, 3759, 'INSTITUTO SUPERIOR TUPY DE SO BENTO DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3458, 3760, 'FACULDADE OMNI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3459, 3761, 'FACULDADE MONSENHOR JOS SINFRNIO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3460, 3762, 'FACULDADE DE TECNOLOGIA MILNIO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3461, 3763, 'FACULDADE DO INTERIOR PAULISTA DE TANABI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3462, 3764, 'CENTRO DE EDUCAO SUPERIOR DE POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3463, 3765, 'FACULDADE SANTO AGOSTINHO DE SANTA VITRIA - FASAV', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3464, 3766, 'INSTITUTO SUPERIOR DE EDUCAO SANTO AGOSTINHO DE SANTA VITRIA - ISAV', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3465, 3767, 'FACULDADE DE EDUCAAO CIENCIAS E TECNOLOGIA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3466, 3768, 'FACULDADE IBGEN - INSTITUTO BRASILEIRO DE GESTO DE NEGCIOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3467, 3769, 'FACULDADE MADRE TEREZA', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3468, 3770, 'FACULDADE TEOLGICA BATISTA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3469, 3771, 'FACULDADE DE TURISMO E ADMINISTRAO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3470, 3772, 'FACULDADE DE TECNOLOGIA PASTOR DOHMS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3471, 3773, 'FACULDADE PAULIPOLITANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3472, 3774, 'FACULDADE SO TOMAZ DE AQUINO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3473, 3775, 'INSTITUTO UVB.BR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3474, 3776, 'INSTITUTO DE ENSINO SUPERIOR DE MATO GROSSO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3475, 3777, 'INSTITUTO DE ENSINO SUPERIOR DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3476, 3778, 'INSTITUTO ARACAJU DE ENSINO E CULTURA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3477, 3779, 'FACULDADE DE TECNOLOGIA RADIAL SANTO ANDR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3478, 3780, 'FACULDADE DE TECNOLOGIA IMPACTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3479, 3781, 'FACULDADE DE TECNOLOGIA DE JARAGU DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3480, 3782, 'FACULDADE DE TECNOLOGIA CACHOEIRO DE ITAPEMIRIM', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3481, 3783, 'INSTITUTO PARAENSE DE ENSINO E CULTURA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3482, 3784, 'INSTITUTO DE ENSINO SUPERIOR DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3483, 3785, 'INSTITUTO MARANHENSE DE ENSINO E CULTURA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3484, 3786, 'INSTITUTO MINAS GERAIS DE ENSINO E CULTURA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3485, 3787, 'INSTITUTO MACAP DE ENSINO SUPERIOR', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3486, 3788, 'INSTITUTO JUIZ DE FORA DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3487, 3789, 'FACULDADE BETEL DE GOIANSIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3488, 3790, 'FACULDADE DE TECNOLOGIA ENSITEC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3489, 3791, 'FACULDADE DE MEDICINA DO VALE DO AO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3490, 3792, 'FACULDADE DE TECNOLOGIA DO ITAJA-MIRIM', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3491, 3793, 'FACULDADE FRUTAL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3492, 3794, 'FACULDADE DE TECNOLOGIA DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3493, 3795, 'CENTRO DE EDUCAO SUPERIOR TECNOLGICA DA TERRA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3494, 3796, 'CENTRO DE ENSINO SUPERIOR DO ESTADO DO PIAU', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3495, 3797, 'FACULDADE GUAIRAC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3496, 3798, 'FACULDADE DE TEOLOGIA DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3497, 3799, 'INSTITUTO SUPERIOR DE EDUCAO E CINCIAS DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3498, 3800, 'INSTITUTO DE EDUCAO E CULTURA VALE DO IPANEMA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3499, 3801, 'CENTRO UNIVERSIT[ARIO ZEN', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3500, 3802, 'CENTRO DE EDUCAO TECNOLGICA DO INTERIOR PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3501, 3803, 'FACULDADE JAUENSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3502, 3804, 'FACULDADE DE TECNOLOGIA SENAC DO RIO GRANDE DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3503, 3805, 'FACULDADE DE TECNOLOGIA DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3504, 3806, 'FACULDADE DE TECNOLOGIA DE RORAIMA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3505, 3807, 'FACULDADE DE TECNOLOGIA IAPEC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3506, 3808, 'FACULDADE DE ENSINO FACTUAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3507, 3809, 'INSTITUTO SUPERIOR DE EDUCAO TEOLOGICA DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3508, 3810, 'NCLEO DE CINCIAS FIORAVANTI DI PIERO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3509, 3811, 'CENTRO DE ESTUDOS EM NEGCIOS INTERNACIONAIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3510, 3812, 'FACULDADES PARA O DESENVOLVIMENTO SUSTENTVEL DO SUL DA BAHIA - FADESB', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3511, 3813, 'FATECE - FACULDADES DE TECNOLOGIA CINCIAS E EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3512, 3814, 'FACULDADE PLANALTO DO BRASIL - FPB', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3513, 3815, 'FACULDADE DE DIREITO DE ALTA FLORESTA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3514, 3816, 'FACULDADE SANTA MARIA GORETTI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3515, 3817, 'FACULDADE METROPOLITANA DA PARAIBA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3516, 3818, 'FACULDADES EQUIPE LTDA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3517, 3819, 'FACULDADE DE ADMINISTRAO DE EMPRESAS E GESTO PBLICA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3518, 3820, 'FACULDADE PETRNIO PORTELA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3519, 3821, 'FACULDADE SO BOAVENTURA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3520, 3822, 'INSTITUTO DE EDUCAO SUPERIOR DE POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3521, 3823, 'INSTITUTO DE EDUCAO SUPERIOR DE POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3522, 3824, 'FACULDADE JOS CNDIDO DE MELO CARVALHO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3523, 3825, 'FACULDADE CATLICA RIO-PRETENSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3524, 3826, 'FACULDADE VASCO DA GAMA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3525, 3827, 'ESCOLA SUPERIOR DE CIENCIAS COMERCIAIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3526, 3828, 'INSTITUTO SUPERIOR DE EDUCAO DE TUP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3527, 3829, 'CENTRO DE EDUCAO SUPERIOR TECNOLGICA DA TERRA PAULISTA - CETERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3528, 3830, 'INSTITUTO CENTRO DE ENSINO TECNOLGICO - SOBRAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3529, 3831, 'FACULDADE DE CINCIAS CONTBEIS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3530, 3832, 'FACULDADE DE ZOOTECNIA', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3531, 3833, 'FACULDADE DE ENGENHARIA FLORESTAL', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3532, 3834, 'FACULDADE SANTA LCIA - PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3533, 3835, 'UNIVERSIDADE VIRTUAL DO ESTADO DO MARANHO', 2, 1, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3534, 3836, 'UNIVERSIDADE PAULISTA-UNIP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3535, 3837, 'FACULDADE DE TECNOLOGIA MDULO PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3536, 3838, 'FACULDADE DE TECNOLOGIA DO UNIUOL', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3537, 3839, 'FACULDADE DE TECNOLOGIA NOVO RUMO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3538, 3840, 'FACULDADE DE TECNOLOGIA E PROFISSIONAL CATARINENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3539, 3841, 'FACULDADE PITGORAS DE ADMINISTRAO SUPERIOR DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3540, 3842, 'FACULDADE METROPOLITANA DE VITRIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3541, 3843, 'FACULDADE CANDIDO MENDES DE ARARUAMA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3542, 3844, 'ESCOLA SUPERIOR DE CASTANHAL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3543, 3845, 'FACULDADE TAQUARITUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3544, 3846, 'FACULDADE DE CINCIAS E CULTURA DA LIBERDADE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3545, 3847, 'FACULDADE BAIANA DE DIREITO E GESTO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3546, 3848, 'FACULDADE GABRIELA GONALVES', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3547, 3849, 'FUNDAO UNIVERSIDADE FEDERAL DO TOCANTINS', 1, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3548, 3850, 'FACULDADE CONTEMPORNEA - UNC-SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3549, 3851, 'FACULDADE DO RECNCAVO - UNR-CRUZ DAS ALMAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3550, 3852, 'FACULDADE DO RECNCAVO - UNR-CONCEIO DO ALMEIDA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3551, 3853, 'FACULDADE MAURCIO DE NASSAU DE NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3552, 3854, 'FACULDADE DAS GUAS EMENDADAS - FAE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3553, 3855, 'FACULDADES INTERNACIONAIS SAN MARTIN DO BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3554, 3856, 'FACULDADE DE SANTO ANASTCIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3555, 3857, 'FACULDADE GAUDIUM', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3556, 3858, 'FACULDADE TOBIAS BARRETO DE MENEZES-FATOB', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3557, 3859, 'INSTITUTO SUPERIOR DE EDUCAO TECNOLGICA DO NORTE FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3558, 3860, 'FACULDADE TRIUNFO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3559, 3861, 'FACULDADE DE TECNOLOGIA E NEGCIOS CONTEMPORNEO - FATEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3560, 3862, 'FACULDADE LUCIANO FEIJO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3561, 3863, 'FACULDADE COOPERATIVA DE GOINIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3562, 3864, 'FACULDADE REGIONAL DE ALAGOINHAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3563, 3865, 'FACULDADES INTEGRADAS PADRE HUMBERTO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3564, 3866, 'FACULDADE BRASILEIRA DE EDUCAO E CULTURA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3565, 3867, 'FACULDADE MAU DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3566, 3868, 'INSTITUTO DE EDUCAO SUPERIOR DO ALTO PARANABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3567, 3869, 'INSTITUTO FLORENCE DE ENSINO SUPERIOR', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3568, 3870, 'INSTITUTO DE ESTUDOS DE NEGCIOS INTERNACIONAIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3569, 3871, 'ESCOLA DE ADMINISTRAO FAZENDRIA', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3570, 3872, 'FACULDADE DE TECNOLOGIA SO CARLOS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3571, 3873, 'FACULDADE DE TECNOLOGIA DE BRUMADO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3572, 3874, 'ESCOLA SUPERIOR PROFESSOR PAULO MARTINS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3573, 3875, 'CENTRO UNIVERSITRIO DA FUNDAO EDUCACIONAL GUAXUP', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3574, 3876, 'INSTITUTO A VEZ DO MESTRE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3575, 3877, 'FACULDADE BRASIL CENTRAL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3576, 3878, 'FACULDADE DECISION DE NEGCIOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3577, 3879, 'FACULDADE MAURCIO DE NASSAU DE CAMPINA GRANDE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3578, 3880, 'INSTITUTO SOCIAL DE EDUCAO', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3579, 3881, 'FACULDADE DE INTEGRAO DO SERTO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3580, 3882, 'INSTITUTO DE ENSINO SUPERIOR DE FORMAO PROFISSIONAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3581, 3883, 'FACULDADE EVANGLICA LOGOS - FAEL', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3582, 3884, 'CESB - CENTRO DE EDUCAO SUPERIOR DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3583, 3885, 'FACULDADES INTEGRADAS DA TERRA PAULISTA - FATERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3584, 3886, 'FACULDADE GUA BOA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3585, 3887, 'FACULDADE DE CINCIAS JURDICAS E GERENCIAIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3586, 3888, 'ESCOLA SUPERIOR DE ADMINISTRAO E NEGCIOS DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3587, 3889, 'FACULDADE DE IGARAPAVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3588, 3890, 'FACULDADE MARQUES GUIMARES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3589, 3891, 'SOCIEDADE EDUCACIONAL ALBERTI TOLENTINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3590, 3892, 'FACULDADE DE CINCIAS SOCIAIS J.G. MELO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3591, 3894, 'UNIESP UNIO DAS INSTITUIES EDUCACIONAIS DO ESTADO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3592, 3895, 'INSTITUTO DE EDUCAO SUPERIOR ALBERTI TOLENTINO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3593, 3896, 'FACULDADE MUNDIAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3594, 3897, 'FACULDADE BUSCA VIDA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3595, 3898, 'SLCSA FACULDADE SANTA INS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3596, 3899, 'FACULDADE DE RIBEIRAO DAS NEVES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3597, 3901, 'FACULDADES PROEDUC', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3598, 3902, 'FUNDAO ESCOLA DO MINISTRIO PBLICO DO ESTADO DO PARAN - FEMPAR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3599, 3903, 'INFAMILIA - INSTITUTO BRASILEIRO DE SADE DA FAMLIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3600, 3904, 'FACULDADE DE CINCIAS GERENCIAIS DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3601, 3905, 'FACULDADE DE DIREITO DOCTUM DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3602, 3906, 'FACULDADE DE CINCIAS AGRRIAS E DA TERRA DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3603, 3907, 'FACULDADE DE SAPIRANGA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3604, 3908, 'FACOM - FACULDADE COMUNITRIA DO BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3605, 3909, 'ISTITUTO DE ENSINO SUPERIOR PARAENSE', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3606, 3910, 'INSTITUTO SUPERIOR DE EDUCAO DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3607, 3911, 'FACULDADE DE TECNOLOGIA OXFORD DO BRASIL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3608, 3912, 'FACULDADE SUDOESTE PAULISTA - BOTUCATU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3609, 3913, 'FACULDADE SUDOESTE PAULISTA - TAQUARITUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3610, 3914, 'INSTITUTO SUPERIOR DE EDUCAO DO VALE DO CAPIBARIBE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3611, 3915, 'INSTITUTO SUPERIOR DE EDUCAO CARLOS JOEL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3612, 3916, 'FACULDADE DE CINCIAS JURDICAS E SOCIAIS DO PIAUI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3613, 3917, 'FACULDADE DE MERCADO AMPLO - FAMA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3614, 3918, 'FACULDADE INTEGRADA EM CINCIAS HUMANAS', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3615, 3919, 'FACULDADE NOSSA SENHORA DO BOM CONSELHO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3616, 3920, 'FACULDADE DE TECNOLOGIA SENAI ROBERTO MANGE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3617, 3921, 'FACULDADE ARNALDO HORCIO FERREIRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3618, 3922, 'FACULDADE ARNALDO HORCIO FERREIRA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3619, 3923, 'FACULDADE SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3620, 3924, 'FACULDADE DE IBIRACU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3621, 3925, 'FACULDADE DE TEOLOGIA DO BRASIL - FATEB', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3622, 3926, 'FACULDADE DE FILOSOFIA, CINCIAS E LETRAS DO ALTO SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3623, 3927, 'FACULDADE DE CCERES', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3624, 3928, 'FAESB - FACULDADE DE ENSINO SUPERIOR DO BRASIL', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3625, 3929, 'FACULDADES UNIDAS DE SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3626, 3930, 'INSTITUTO DE EDUCAO SUPERIOR RAIMUNDO S', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3627, 3931, 'FACULDADE PRESBITERIANA ARMSTRONG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3628, 3932, 'FACULDADE DE EDUCAO E TECNOLOGIA DE ITANHM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3629, 3933, 'FACULDADE MTODO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3630, 3934, 'FACULDADE DA SADE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3631, 3935, 'FACULDADE DE EDUCAO TEOLOGICA DO DISTRITO FEDERAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3632, 3936, 'FACULDADE ANHANGUERA DE LIMEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3633, 3937, 'FACULDADE ANHAGUERA DE INDAIATUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3634, 3938, 'FACULDADE DO NORTE DO MATO GROSSO DO SUL FANOR', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3635, 3939, 'FACULDADE ALIANA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3636, 3940, 'FACULDADE DE ENSINO SUPERIOR SANTA BARBARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3637, 3941, 'FACULDADE COMUNITRIA DE SANTA BRBARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3638, 3942, 'FACULDADE COMUNITRIA DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3639, 3943, 'FASC-TB FACULDADES ASSOCIADAS DE SC, DE TUBARO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3640, 3944, 'FACULDADE DE CINCIAS DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3641, 3945, 'INSTITUTO DE EDUCAO EXECUTIVA CATHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3642, 3946, 'FACULDADE DE TECNOLOGIA SENAC TUBARO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3643, 3947, 'FACULDADE DE TECNOLOGIA SENAC CHAPEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3644, 3948, 'FACULDADE DE TECNOLOGIA SENAC BLUMENAU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3645, 3949, 'CENTRO DE EDUCAO SUPERIOR TECNOLGICA DA TERRA PAULISTA - CETERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3646, 3950, 'CENTRO DE CINCIAS ETECNOLOGIA DETERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3647, 3951, 'FACULDADE DE TECNOLOGIA LUIZ ADELAR SCHEUER', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3648, 3952, 'FACULDADE VILAVELHENSE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3649, 3953, 'FACULDADE DE ODONTOLOGIA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3650, 3954, 'FACULDADE DE CINCIAS JURDICAS E GERENCIAIS DE OLIVEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3651, 3955, 'FACULDADES INTEGRADAS DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3652, 3956, 'ASSOCIAO EDUCACIONAL CENTRAL DE CRISTALINA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3653, 3957, 'INSTITUTO SUPERIOR DE EDUCAO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3654, 3958, 'FACULDADE TECNOLOGICA DE MARILIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3655, 3959, 'FACULDADE TECNOLOGICA AMRICA DO SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3656, 3960, 'FACULDADE TECNOLOGICA DE BIRIGUI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3657, 3961, 'FACULDADE DE TECNOLOGIA SANTA RITA DE CSSIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3658, 3962, 'FACULDADE DE TECNOLOGIA SENAI CIMATEC', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3659, 3963, 'INSTITUTO SUPERIOR DE EDUCAO CARLOS CHAGAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3660, 3964, 'FACULDADE DE TECNOLOGIA PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3661, 3965, 'FACULDADE DE TECNOLOGIA ENG. MARCELO SILVA GUIMARES', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3662, 3966, 'CENTRO UNIVERSITRIO DE CARATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3663, 3967, 'INSTITUTO SUPERIOR DE EDUCAO DE RORAIMA', 2, 1, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3664, 3968, 'INSTITUTO SUPERIOR DE EDUCAO DE RORAINPOLIS', 2, 1, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3665, 3969, 'INSTITUTO SUPERIOR DE SEGURANA E CIDADANIA DE RORAIMA', 2, 1, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3666, 3970, 'INSTITUTO SUPERIOR DE EDUCAO DE PARAISPOLIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3667, 3971, 'UNIO EDUCACIONAL DE PORTO NACIONAL - UNIPORTO', 3, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3668, 3972, 'CENTRO SUPERIOR DE ENSINO E PESQUISA DE MACHADO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3669, 3973, 'FACULDADE DE TECNOLOGIA BANDEIRANTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3670, 3974, 'UNIVERSIDADE DE RIO VERDE', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3671, 3975, 'FACULDADE SO FRANCISCO DE PIUMHI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3672, 3976, 'INSTITUTO SUPERIOR DE CINCIAS HUMANAS E SOCIAIS APLICADAS DE ABAET', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3673, 3977, 'FACULDADE DE TECNOLOGIA DO AMAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3674, 3978, 'FACULDADE DE TECNOLOGIA PENTGONO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3675, 3979, 'FACULDADE DE TECNOLOGIA LICEU NOROESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3676, 3980, 'FACULDADE DE TECNOLOGIA CENACAP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3677, 3981, 'FACULDADE DE TECNOLOGIA INTEGRAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3678, 3982, 'FACULDADE DE TECNOLOGIA FAMEG - FAMETEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3679, 3983, 'CENTRO UNIVERSITRIO DE FORMIGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3680, 3984, 'FUNDAO UNIVERSIDADE FEDERAL DO VALE DO SO FRANCISCO', 1, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3681, 3985, 'CENTRO UNIVERSITRIO SENAC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3682, 3986, 'INSTITUTO SUPERIOR DE EDUCAO DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3683, 3987, 'FACULDADE UNIO DE GOYAZES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3684, 3988, 'FACULDADE UNIO DE GOYAZES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3685, 3989, 'FACULDADE DE TECNOLOGIA DE JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3686, 3990, 'FACULDADE DE TECNOLOGIA ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3687, 3991, 'FACULDADE DE TECNOLOGIA SO MATEUS', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3688, 3992, 'FACULDADE ILAPE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3689, 3993, 'FACULDADE DE TECNOLOGIA CETEP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3690, 3994, 'FACULDADE DE TECNOLOGIA LATINO AMERICANO DE ANPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3691, 3995, 'FACULDADE EVANGLICA CRISTO REI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3692, 3996, 'FACULDADE SENAC PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3693, 3997, 'FACULDADES DE JI-PARAN', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3694, 3998, 'FACULDADE DE TECNOLOGIA INFNET RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3695, 3999, 'CENTRO DE ENSINO SUPERIOR TECNOLGICO DO INTERIOR PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3696, 4000, 'FACULDADE DE ENSINO SUPERIOR DO INTERIOR PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3697, 4001, 'FACULDADE DE ENSINO SUPERIOR DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3698, 4002, 'INSTITUTO DE ENSINO SUPERIOR DA PARAIBA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3699, 4003, 'FACULDADE CENTRO OESTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3700, 4004, 'FACULDADE CENECISTA DE ITUBER', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3701, 4005, 'FACULDADE CENECISTA DE SENHOR DO BONFIM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3702, 4006, 'FACULDADE DE TECNOLOGIA SENAC PELOTAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3703, 4007, 'FATECE - FACULDADE DE TECNOLOGIA, CINCIAS E EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3704, 4008, 'FACULDADE DE TECNOLOGIA SENAC PASSO FUNDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3705, 4009, 'FACULDADE DE TECNOLOGIA RADIAL CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3706, 4010, 'CENTRO UNIVERSITRIO METODISTA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3707, 4011, 'UNI- DORINHA FACULDADE DE CINCIAS E TECNOLOGIA DE DORINHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3708, 4012, 'UNI-METROPOLITANA; FACULDADES DE CINCIAS HUMANAS E DE BIOTECNOLOGIA DA SADE', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3709, 4013, 'FACULDADE ANHANGUERA DE RIO CLARO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3710, 4014, 'FACULDADE ASSOCIADA PAULISTA - FAP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3711, 4015, 'FACULDADE DE TECNOLOGIA IBTA - RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3712, 4016, 'INSTITUTO NACIONAL DE EDUCAO DE SURDOS', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3713, 4017, 'CENTRO UNIVERSITRIO PADRE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3714, 4018, 'FACULDADE GAT DE GESTAO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3715, 4019, 'CENTRO DE EDUCAO TECNOLOGIA RUI BARBOSA - CETERB', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3716, 4020, 'FACULDADE DE TECNOLOGIA DE BOTUCATU', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3717, 4021, 'FACULDADE DE TECNOLOGIA DE GARA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3718, 4022, 'FACULDADE DE TECNOLOGIA DE JUNDIA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3719, 4023, 'FACULDADE DE TECNOLOGIA DE MAU', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3720, 4024, 'FACULDADE DE TECNOLOGIA DE MOCOCA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3721, 4025, 'FACULDADE DE TECNOLOGIA DE SO JOS DO RIO PRETO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3722, 4026, 'FACULDADE DE TECNOLOGIA ZONA LESTE', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3723, 4027, 'INSTITUTO SUPERIOR DE ENSINO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3724, 4028, 'FACULDADE DE ITAPECERICA DA SERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3725, 4029, 'FACULDADE POLITEC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3726, 4030, 'CENTRO UNIVERSITRIO FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3727, 4031, 'INSTITUTO DE ENSINO SUPERIOR PROFESSORA ALMEIDA (CESPMA)', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3728, 4032, 'FACULDADE DE CINCIAS E TECNOLOGIA DE PAULISTA- FACIT', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3729, 4033, 'FACULDADE DE TECNOLOGIA FABRAI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3730, 4034, 'ASSOCIAO EDUCACIONAL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3731, 4035, 'FACULDADE MAU - FAMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3732, 4036, 'FACULDADE DE TECNOLOGIA DE CATALO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3733, 4037, 'FACULDADE DE TECNOLOGIA INTENSIVA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3734, 4038, 'FACULDADE DE TECNOLOGIA CONSULTIME', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3735, 4039, 'FACULDADE UNILIVRE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3736, 4040, 'ESCOLA AGROTCNICA FEDERAL DE UBERLNDIA', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3737, 4041, 'FACULDADE DE IRAR - FAIR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3738, 4042, 'FACULDADE DO MEIO AMBIENTE E DE TECNOLOGIA DE NEGOCIOS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3739, 4043, 'FACULDADE TECNOLOGIA EDUVALE - AVAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3740, 4044, 'FACULDADE TECNOLOGIA EXPERT', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3741, 4045, 'FACULDADE TECNOLOGIA TUPY CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3742, 4046, 'ESCOLA FIEMG DE LDERES EMPRESARIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3743, 4047, 'FACULDADE TECNOLOGIA IBTA BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3744, 4048, 'FCC - FACULDADE CATLICA DO CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3745, 4049, 'FACULDADE TEOLGICA DA ASSEMBLIA DE DEUS EM CURITIBA-FATADC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3746, 4050, 'FAESP -  FACULDADE DE ENSINO SUPERIOR E PROFISSIONALIZANTE', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3747, 4051, 'FACULDADE SILVIO ROMERO', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3748, 4052, 'INSTITUTO BONDESPACHENSE PRESIDENTE ANTONIO CARLOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3749, 4053, 'CENTRO DE EDUCAO SUPERIOR BARNABITA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3750, 4054, 'INSTITUTO UNIFICADO DE ENSINO SUPERIOR OBJETIVO - IUESO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3751, 4055, 'INSTITUTO BBLICO DAS ASSEMBLIAS DE DEUS DO ESTADO DO PARAN- IBADEP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3752, 4057, 'FACISA - FACULDADE DE CINCIA DA SADE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3753, 4058, 'FACULDADE IP', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3754, 4059, 'FACULDADE DE CINCIAS GERENCIAIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3755, 4060, 'CENTRO DE EDUCAO TECNOLGICA CENECISTA DE RIO NEGRINHO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3756, 4061, 'FACULDADE DE TECNOLOGIA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3757, 4062, 'FACULDADE ATHENIENSSE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3758, 4064, 'FACULDADE DE TECNOLOGIA FAMA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3759, 4065, 'FACULDADE DE TECNOLOGIA DO INSTITUTO BANDEIRANTE DE EDUCAO E CULTURA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3760, 4066, 'FACULDADE DE TECNOLOGIA NORTE FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3761, 4067, 'FACIT - FACULDADE INDEPENDENTE DE ITAPETINGA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3762, 4068, 'INSTITUTO SUPERIOR DE EDUCAO DO ICESP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3763, 4069, 'FACULDADE HORIZONTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3764, 4070, 'ESCOLA AGROTCNICA FEDERAL DE SALINAS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3765, 4071, 'ESCOLA AGROTCNICA FEDERAL DE INCONFIDENTES', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3766, 4072, 'ESCOLA AGROTCNICA FEDERAL DE IGUATU', 1, 1, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3767, 4073, 'FACULDADE DE TECNOLOGIA IPIRANGA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3768, 4074, 'FACULDADE DELTA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3769, 4076, 'FACULDADE DE TECNOLOGIA PREVE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3770, 4077, 'FACULDADE DE TECNOLOGIA DE GRAVATA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3771, 4078, 'FACULDADE METROPOLITANA DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3772, 4079, 'FACULDADE TECNOLGICA INAP', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3773, 4080, 'FACULDADE DE PRUDENTPOLIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3774, 4081, 'FATEG-FACULDADE DE TECNOLOGIA GALVO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3775, 4082, 'IBEC - INSTITUTO BRASILEIRO DE EDUCAO SUPERIOR CONTINUADA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3776, 4083, 'MATEC-CENTRO DE ENSINO TECNOLOGICO DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3777, 4084, 'INSTITUTO DE ENSINO SUPERIOR DOS MILITARES', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3778, 4085, 'FACULDADE ANCHIETA DE EDUCAO A DISTNCIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3779, 4086, 'FACULDADE DE TECNOLOGIA JARDIM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3780, 4087, 'FACULDADE DE EDUCAO TECNOLGICA - FET DA CDL DE SOBRAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3781, 4088, 'FACULDADE DE TECNOLOGIA DO INSTITUTO EDUCACIONAL PROFISSIONALIZANTE DE JALES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3782, 4089, 'FACULDADE DE SIDROLNDIA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3783, 4090, 'FACULDADE DE ROSEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3784, 4091, 'FACULDADE DE TECNOLOGIA AD1', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3785, 4092, 'FACULDADE DE TECNOLOGIA PEDRO ROGRIO GARCIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3786, 4093, 'FACULDADE DE TECNOLOGIA DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3787, 4094, 'FACULDADE DE TECNOLOGIA IBRATEC DE JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3788, 4095, 'FACULDADE DE TECNOLOGIA EQUIPE DARWIN', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3789, 4096, 'FACULDADE DE TECNOLOGIA TECBRASIL - UNIDADE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3790, 4097, 'FACULDADE DE TECNOLOGIA TECBRASIL - UNIDADE BENTO GONALVES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3791, 4098, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE FARROUPILHA', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3792, 4099, 'ESCOLA AGROTCNICA FEDERAL DE BARBACENA', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3793, 4100, 'FACULDADE DE TECNOLOGIA SENAI TALO BOLOGNA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3794, 4101, 'FACULDADE DE TECNOLOGIA SENAI DE DESENVOLVIMENTO GERENCIAL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3795, 4102, 'FACULDADE DE TECNOLOGIA IBRATEC DE MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3796, 4103, 'FACULDADE METROPOLITANA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3797, 4104, 'ESCOLA SUPERIOR DE ENGENHARIA E GESTO DE SO PAULO - ESEG', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3798, 4105, 'FACULDADES COSTA BRAGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3799, 4106, 'FAQ - FACULDADE QUINZE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3800, 4107, 'FACULDADE DE TECNOLOGIA SENAI PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3801, 4108, 'INSTITUTO CATLICO DE ESTUDOS SUPERIORES DO PIAU - ICESPI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3802, 4109, 'FACULDADE DE SADE E MEDICINA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3803, 4110, 'FACULDADE DE CINCIAS GERENCIAIS DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3804, 4111, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3805, 4112, 'FACULDADE TECNOLGICA FILADLFIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3806, 4113, 'FACULDADE DE CERES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3807, 4114, 'FACULDADE TEOLGICA CRIST DO BRASIL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3808, 4116, 'FACULDADE MAURICIO DE NASSAU JABOATO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3809, 4117, 'FACULDADE MAURCIO DE NASSAU MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3810, 4118, 'FACULDADE JOAQUIM NABUCO - PAULISTA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3811, 4119, 'FACULDADE MAURCIO DE NASSAU GARANHUNS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3812, 4120, 'FACULDADE MAURCIO DE NASSAU CARUARU', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3813, 4121, 'FACULDADE TOBIAS BARRETO', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3814, 4122, 'CENTRO DE EDUCAO TECNOCNOLGICA RUI BARBOSA - CETRB', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3815, 4123, 'FACULDADE JATAIENSE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3816, 4124, 'FACULDADE DOM HEITOR SALES', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3817, 4125, 'FACULDADE BATISTA IDA NELSON', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3818, 4126, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS E DE TECNOLOGIAS DE AGUA BOA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3819, 4127, 'ISEF - INSTITUTO SUPERIOR DE EDUCAO CENECISTA DE FARROUPILHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3820, 4128, 'INSTITUTO DE EDUCAO SUPERIOR DO PIAUI - IESPI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3821, 4129, 'FACULDADE JOS DE ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3822, 4130, 'FACULDADE DE TECNOLOGIA DE CINCIAS HOLSTICAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3823, 4131, 'FACULDADE DO DELTA DO PARNABA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3824, 4132, 'FACULDADE DE COLORADO - FAC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3825, 4133, 'FACULDADE DE EDUCAO TECNOLGICA SANTA GEMA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3826, 4134, 'INEJ - INSTITUTO NACIONAL DE ESTUDOS JURDICOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3827, 4135, 'FACULDADE SANTA DOROTIA DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3828, 4136, 'FACULDADE CNDIDO RONDON DE RONDONPOLIS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3829, 4137, 'INSTITUTO EVANGLICO SUPERIOR DE EDUCAO CRISTO REI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3830, 4138, 'FACULDADE ANHANGUERA DE SANTA BRBARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3831, 4139, 'FACULDADE COMUNITRIA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3832, 4140, 'FACULDADE COMUNITRIA DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3833, 4141, 'FACULDADE ANHANGUERA DE TAUBAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3834, 4142, 'FACULDADE UNIO DE EDUCADORES E ESTUDANTES DO ENSINO SUPERIOR DE BRASLIA - UNESB', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3835, 4143, 'FACULDADE EVANGLICA SHALOM', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3836, 4144, 'FACULDADE INTEGRADA GAMALIEL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3837, 4145, 'CENTRO NACIONAL DE EDUCAO A DISTNCIA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3838, 4146, 'FACULDADE DE CINCIAS JURDICAS DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3839, 4147, 'FACULDADE TEOLGICA DE UBERLANDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3840, 4148, 'FACULDADE DE TECNOLOGIA SENAI SO JOS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3841, 4149, 'FACULDADE DE TECNOLOGIA SENAI ITAJA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3842, 4150, 'FACULDADE DE TECNOLOGIA DE PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3843, 4151, 'FACULDADE BOAVENTURA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3844, 4152, 'FACULDADE MAURCIO DE NASSAU JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3845, 4153, 'FACULDADE JOAQUIM NABUCO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3846, 4154, 'FACULDADE MAURCIO DE NASSAU CAMPINA GRANDE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3847, 4155, 'FACULDADE JOAQUIM NABUCO CAMARAGIBE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3848, 4156, 'FACULDADE TECNOLGICA DE SOORETAMA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3849, 4157, 'FACULDADE METROPOLITANA DE RIO DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3850, 4158, 'FACULDADE EVANGLICA DO MEARIM', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3851, 4159, 'FACULDADE PIRATININGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3852, 4160, 'FACULDADE DE TECNOLOGIA DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3853, 4161, 'INSTITUTO DE ENSINO SUPERIOR CAETANO VELOSO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3854, 4162, 'FACULDADE DE TECNOLOGIA SENAC GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3855, 4163, 'CENTRO UNIVERSITRIO BARRIGA VERDE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3856, 4164, 'FACULDADE XODUS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3857, 4165, 'FACULDADE POTNCIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3858, 4166, 'CENTRO DE ENSINO SUPERIOR DE UBERABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3859, 4167, 'INSTITUTO DE ENSINO SUPERIOR DA FUPESP', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3860, 4168, 'INSTITUTO DE ENSINO SUPERIOR DO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3861, 4169, 'FACULDADE NOSSA CIDADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3862, 4170, 'FACULDADE INTEGRADA SO PEDRO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3863, 4171, 'ISAG - INSTITUTO SUPERIOR DE ADMINISTRAO E GESTO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3864, 4172, 'CENTRO DE ENSINO SUPERIOR DO SERVIDOR PUBLICO MARANHENSE CESPMA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3865, 4173, 'FACULDADE DE TECNOLOGIA PAULO FREIRE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3866, 4174, 'FACULDADE DE ADMINISTRAO DE EMPRESAS DE ENERGIA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3867, 4175, 'INSTITUTO UNIFICADO EUROPEU DO BRASIL IUNE-BRASIL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3868, 4176, 'FACULDADE DE MARILIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3869, 4177, 'FACULDADE DE TECNOLOGIA DE SANTA MARIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3870, 4178, 'FACULDADE EVILSIO FORMIGA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3871, 4179, 'FACULDADE NAZARENA DO BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3872, 4180, 'FACULDADE HERRERO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3873, 4182, 'FACULDADE PRESBITERIANA SIMONTON - IPES', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3874, 4183, 'FACULDADE PRESBITERIANA SIMONTON DE SAPEZAL - IPES', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3875, 4184, 'INSTITUTO SUPERIOR DE EDUCAO DE SERRANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3876, 4185, 'FACULDADE PERUIBE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3877, 4187, 'ISEFAJIPA - INSTITUTO SUPERIOR DE EDUCAO DE JI-PARAN', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3878, 4188, 'FACULDADE IAPEDEM/ANAMACO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3879, 4189, 'INSTITUTO DE EDUCAO SUPERIOR SO JUDAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3880, 4190, 'UNIFOS - UNIVERSO DE FORMAO TCNICA E SUPERIOR', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3881, 4191, 'INSTITUTO SUPERIOR METROPOLITANA - ISUME', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3882, 4192, 'FACULDADE METROPOLITANA DE TERESINA - FAMET', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3883, 4193, 'INSTITUTO SUPERIOR DE EDUCAO DO BRASIL - ISEB', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3884, 4194, 'FACULDADE DO ALTO TIETE - FAAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3885, 4195, 'FACULDADE PROFETA ABRAO - FAPA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3886, 4196, 'INSTITUTO BRASILEIRO ACADMICO SOCIAL DE PESQUISAS E ENSINO/IBRASPE', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3887, 4197, 'FACULDADE CERTO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3888, 4198, 'FACULDADE MINEIRENSE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3889, 4199, 'FACULDADE DE TECNOLOGIA ROGACIONISTA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3890, 4200, 'FACULDADE DE COMUNICAO SOCIAL DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3891, 4201, 'FACULDADE DE ENSINO SUPERIOR DO NORDESTE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3892, 4202, 'FACULDADE ALFA - FAAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3893, 4203, 'FACULDADE ABERTA DO BRASIL - UNIABRA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3894, 4204, 'FACULDADE DE TECNOLOGIA INED - UNIDADE IPATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3895, 4205, 'ESCOLA SUPERIOR DO MINISTRIO PBLICO DA UNIO (ESMPU)', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3896, 4206, 'INSTITUTO DE ENSINO SUPERIOR DE TAGUATINGA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3897, 4207, 'NAP ODONTO SC LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3898, 4208, 'FACULDADE DE TECNOLOGIA MONTEIRO LOBATO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3899, 4209, 'FACULDADE DE TECNOLOGIA CSAR LATTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3900, 4210, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DO MORUMBI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3901, 4211, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE PERDIZES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3902, 4212, 'FACULDADE DE TECNOLOGIA DE POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3903, 4213, 'FACULDADE DE CAAPAVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3904, 4214, 'INSTITUTO SUPERIOR PAULO FREIRE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3905, 4215, 'FACULDADE DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3906, 4216, 'INSTITUTO MOINHOS DE VENTO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3907, 4217, 'FACULDADE DE TECNOLOGIA INSTITUTO POLITCNICO DO PARAN - UNIDADE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3908, 4218, 'FACULDADE DE DIREITO DE PEDRO LEOPOLDO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3909, 4219, 'INSTITUTO SUPERIOR DE EDUCAO PROFESSORA NAIR FORTES ABU-MERHY', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3910, 4220, 'FACULDADE DE CINCIAS GERENCIAIS DE BICAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3911, 4221, 'INSTITUTO SUPERIOR DE EDUCAO DE BICAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3912, 4222, 'INSTITUTO SUPERIOR DE EDUCAO DE MATIAS BARBOSA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3913, 4223, 'FACULDADE SANTOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3914, 4224, 'FACULDADE DE ITAPECIRICA - FACITA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3915, 4225, 'FACULDADE DE TECNOLOGIA LUIZ ADELAR SCHEUER', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3916, 4227, 'FACULDADE DE TECNOLOGIA SENAC PASSO FUNDO - PASSO FUNDO/RS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3917, 4228, 'INSTITUTO DE EDUCAO SUPERIOR MISSES', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3918, 4229, 'FACULDADE PARANTINGA - FAP', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3919, 4230, 'FACULDADE TECNOLGICA UNIVERSAL DE FORMAO TCNICA E SUPERIOR', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3920, 4233, 'INSTITUTO METROPOLITANO DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3921, 4234, 'SOCIEDADE EVOLUO DE TECNOLOGIA E EDUCAO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3922, 4235, 'FACULDADE TRINGULO MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3923, 4236, 'FACULDADES INTEGRADAS DE SERRANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3924, 4237, 'CENTRO FEDERAL DE EDUCAO TECNOLGICA DO AMAZONAS', 1, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3925, 4239, 'FACULDADE DE CINCIAS E TECNOLOGIA DO VALE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3926, 4241, 'FACULDADE DE CINCIAS HUMANAS, SOCIAIS E TECNOLGICAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3927, 4243, 'FACULDADE DE CINCIAS GERENCIAIS DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3928, 4244, 'FACULDADE DE TECNOLOGIA ENSITEC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3929, 4245, 'FACULDADE DE TECNOLOGIA DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3930, 4246, 'PROGRAMA DE PS GRADUAO LATO SENSU DA FUNDAO UNIMED', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3931, 4247, 'FACULDADE DE CIENCIAS HUMANAS E SOCIAIS SO PAULO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3932, 4248, 'INSTITUTO DE ENSINO, PESQUISA E PS-GRADUAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3933, 4249, 'FACULDADE DE CINCIAS DE ALAGOAS - FACAL', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3934, 4250, 'FACULDADE DE CINCIAS GERENCIAIS ALVES FORTES (JUIZ DE FORA)', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3935, 4251, 'FACULDADE DE TECNOLOGIA INED - UNIDADE LAGOA DA PAMPULHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3936, 4252, 'FACULDADE DE TECNOLOGIA EXPOENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3937, 4253, 'ESCOLA AGROTCNICA FEDERAL DE CONCRDIA', 1, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3938, 4254, 'FACULDADES INTEGRADAS DA TERRA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3939, 4255, 'FACULDADES INTEGRADAS DE CACOAL', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3940, 4256, 'FACULDADES INTEGRADAS PITGORAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3941, 4257, 'INSTITUTO DE FILOSOFIA E TEOLOGIA DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3942, 4258, 'FACULDADE PURIFICAO LTDA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3943, 4259, 'FACULDADE DE INHUMAS - FAC-MAIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3944, 4260, 'FACULDADE DE TECNOLOGIA DE SO LEOPOLDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3945, 4261, 'FACULDADE DE TECNOLOGIA DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3946, 4262, 'FACULDADE DE TECNOLOGIA IDESC', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3947, 4263, 'FACULDADE ESPRITA RAMATIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3948, 4264, 'INSTITUTO FIGUEIREDO DE EDUCAO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3949, 4265, 'UNEPPSI - UNIDADE DE ENSINO E PESQUISA EM PSICOLOGIA E PSICANLISE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3950, 4266, 'INSTITUTO GUERRA BIAGI - INGB', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3951, 4267, 'INSTITUTO SUPERIOR DE EDUCAO DA AMRICA LATINA - ISAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3952, 4268, 'FACULDADE ISAG', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3953, 4269, 'SOCIEDADE BRASILEIRA DE EDUCAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3954, 4270, 'FACULDADE LIBERTAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3955, 4271, 'INSTITUTO SUPERIOR EM CINCIAS DA HUMANIDADE', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3956, 4272, 'INSTITUTO MODAL LTDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3957, 4273, 'INSTITUTO SUPERIOR DE EDUCAO SEESB', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3958, 4274, 'INSTITUTO TEOLGICO DA ASSEMBLIA DE DEUS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3959, 4275, 'INSTITUTO DE ENSINO E PESQUISA EM ADMINISTRAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3960, 4276, 'IETEC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3961, 4277, 'FACULDADE LITERATUS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3962, 4278, 'FACULDADE INTEGRADA SANTA RITA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3963, 4279, 'FACULDADE DE TECNOLOGIA INSTITUTO POLITCNICO DO PARAN - UNIDADE GUARAPUAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3964, 4280, 'FACULDADE INTEGRADA CIDADE DE PATOS DE MINAS LTDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3965, 4281, 'FACULDADE AMBIENTAL DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3966, 4282, 'FACULDADE PARA O DESENVOLVIMENTO DO ESTADO DE SO PAULO - FADESP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3967, 4283, 'FACULDADE IASOCIAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3968, 4284, 'FACULDADE SO FRANCISCO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3969, 4285, 'FUNDAO INSTITUTO DE ADMINISTRAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3970, 4286, 'FACULDADE PRATA - FAP', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3971, 4287, 'FACULDADE DE CINCIAS HUMANAS E SOCIAIS DO XINGU E AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3972, 4288, 'UNIED TECNOLOGIA EDUCACIONAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3973, 4289, 'FACULDADE DO VALE ELVIRA DAYRELL - FAVED', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3974, 4290, 'FACULDADE DE ARTES INTEGRADAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3975, 4291, 'IEMS INSTITUTO EDUCACIONAL MORADA DA SERRA DO MUNICPIO DE CUIAB/MT.', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3976, 4292, 'ITECSS - INSTITUTO TIRADENTES DE EDUCAO EM CINCIAS SOCIAIS E DA SADE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3977, 4293, 'FACULDADE INTEGRAO TIETE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3978, 4294, 'FACULDADE DE TECNOLOGIA SENAC JARAGU DO SUL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3979, 4295, 'FACULDADE DE TECNOLOGIA SENAI CETIND', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3980, 4296, 'INSTITUTO BRASILEIRO DE CIENCIAS CRIMINAIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3981, 4297, 'CENTRO DE ESTUDOS PROF JOS BELMIRO DE CASTRO MOREIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3982, 4298, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3983, 4299, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA/RS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3984, 4300, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SECO PARAIBA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3985, 4301, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL ABO-DF', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3986, 4302, 'ASSOCIAO CATARINENSE DE ODONTOLOGIA ESPECIALIZADA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3987, 4303, 'SOCIEDADE DOS CIRURGIES DENTISTAS DE PERNAMBUCO - ABO/PE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3988, 4304, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA ABO-RO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3989, 4305, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA-ES/RVV', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3990, 4306, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA ASSOCIAO BRASILEIRA DE ODONTOLOGIA REGIONAL POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3991, 4307, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - SEO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3992, 4308, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SECO AMAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3993, 4309, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - SEO TOCANTINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3994, 4310, 'UNIS-MG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3995, 4311, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL - ABO/AL', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3996, 4312, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO - REGIONAL CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3997, 4313, 'ASSOCIACAO BRASILEIRA DE ODONTOLOGIA SAOPAULO REGIONAL BIRIGUI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3998, 4314, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL LINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (3999, 4315, 'FACULDADE DE TEOLOGIA E CIENCIAS DA RELIGIAO', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4000, 4316, 'FACULDADE DE TECNOLOGIA SANTOS DUMONT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4001, 4317, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4002, 4318, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4003, 4319, 'CAP/ABORJ - CENTRO DE APERFEIOAMENTO PROFISSIONAL DA ABO-RJ', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4004, 4320, 'ESCOLA DE PS GRADUAO /ASSOCIAO BRASILEIRA DE ODONTOLOGIA REGIONAL DE CASCAVEL - PR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4005, 4321, 'IETEC INSTITUTO DE CINCIA E TECNOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4006, 4322, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4007, 4323, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA REGIONAL CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4008, 4324, 'FACULDADE GONZAGA DE PELOTAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4009, 4325, 'EAP ABO-MURIAE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4010, 4326, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL - ASSOCIAO BRASILEIRA DE ODONTOLOGIA REGIONAL JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4011, 4327, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4012, 4328, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4013, 4329, 'FACULDADE DO CENTRO EDUCACIONAL MINEIRO - FACEM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4014, 4330, 'FACULDADE DE TECNOLOGIA AMRICA DO SUL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4015, 4331, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO - REGIONAL ARAATUBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4016, 4332, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4017, 4333, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL ARARAQUARA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4018, 4334, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL SERTOZINHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4019, 4335, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO - REGIONAL SO JOSE DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4020, 4336, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL RIBEIRO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4021, 4337, 'ASSOCIACAO BRASILEIRA DE ODONTOLOGIA SAO PAULO REGIONAL PRESIDENTE PRUDENTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4022, 4338, 'ASSOCIACAO BRASILEIRA DE ODONTOLOGIA SAO PAULO REGIONAL CATANDUVA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4023, 4339, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO REGIONAL VALE DO PARAIBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4024, 4340, 'FACULDADE DE ENSINO SUPERIOR DE VALENA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4025, 4341, 'ASSOCIAO MARINGAENSE DE ODONTOLOGIA ( AMO ) - ABO REGIONAL MARINGA - PR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4026, 4342, 'FACULDADE STAR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4027, 4343, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA ABO-MS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4028, 4344, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA ABO/PI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4029, 4345, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA ABO-BA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4030, 4346, 'FACULDADE DE ENSINO SUPERIOR DE MIGUEL ALVES', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4031, 4347, 'FACULDADE SERRA DOS PIRINEUS DE EDUCAO DE CINCIAS APLICADAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4032, 4348, 'FACULDADE DE COMUNICAO, TECNOLOGIA E CIENCIA - FACTEC', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4033, 4349, 'ABO - REGIONAL SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4034, 4350, 'E.A.B. ASSOCIAO BRASILEIRA DE ODOTOLOGIA RJ - D.DE CAXIAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4035, 4351, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - SEO SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4036, 4352, 'ASSOCIACAO BRASILEIRA DE ODONTOLOGIA REGIONAL OSASCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4037, 4353, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SO PAULO - REGIONAL ABC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4038, 4354, 'AFISVEC', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4039, 4355, 'FACULDADE DE TECNOLOGIA DE PALMAS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4040, 4356, 'FACULDADE DE CINCIAS SOCIAIS DOS PALMARES', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4041, 4357, 'FACULDADE DE TECNOLOGIA LVARES DE AZEVEDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4042, 4358, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO SUL DE MINAS GERAIS', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4043, 4359, 'ESCOLA AGROTCNICA FEDERAL DE SATUBA', 1, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4044, 4360, 'ESCOLA AGROTCNICA FEDERAL DE MACHADO', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4045, 4361, 'FACULDADE DE CINCIAS GERENCIAIS DE SANTA LUZIA MG - FAGESA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4046, 4362, 'FACULDADE PITGORAS DE BETIM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4047, 4363, 'INSTITUTO SANTA MNICA DE ESTUDOS E PESQUISAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4048, 4364, 'FACULDADE QUASAR', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4049, 4365, 'FABS - FACULDADE DE BOM SUCESSO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4050, 4366, 'EAP - ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4051, 4367, 'FACULDADE TERRA NORDESTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4052, 4368, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4053, 4369, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL - EAP - ABO/RN', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4054, 4370, 'SANTA CASA DE MISERICRDIA DE PONTA GROSSA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4055, 4371, 'FACULDADE PADRE ANCHIETA DE CAJAMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4056, 4372, 'U.E.P. UNIDADE DE ENSINO PROFISSIONAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4057, 4373, 'FACULDADE DE EDUCAO CINCIA E TECNOLOGIA DE REDENO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4058, 4374, 'CURSO DE TEOLOGIA E PASTORAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4059, 4375, 'UNC-SALVADOR UNIDADE AVANADA DE PS-GRADUAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4060, 4376, 'IBEG - PS-GRADUAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4061, 4377, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL /EAP', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4062, 4378, 'NCLEO DE ESTUDOS ORTODNTICOS MBT BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4063, 4379, 'ASSOCIAO DE CIRURGIES DENTISTAS DO ESTADO DE SO PAULO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4064, 4380, 'CENTRO DE QUALIFICAO ODONTOLGICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4065, 4381, 'ACADEMIA NORTE RIO-GRANDENSE DE ODONTOLOGIA', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4066, 4382, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - REGIONAL PETRPOLIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4067, 4383, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - SECO DE GOIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4068, 4384, 'CESVIA FACULDADE DE VIAMO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4069, 4386, 'ESAB - ESCOLA SUPERIOR ABERTA DO BRASIL', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4070, 4387, 'NETT ODONTO - NCLEO DE ESPECIALIZAO, TREINAMENTO E TRATAMENTO ODONTOLGICO S/C LTDA.', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4071, 4388, 'IPEC - INSTITUTO PAULISTA DE EDUCAO CONTINUADA LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4072, 4389, 'FACULDADE INTEGRADA CARAJS DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4073, 4390, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA ASSOCIAO BRASILEIRA DE ODONTOLOGIA SECO SERGIPE', 2, 1, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4074, 4391, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4075, 4392, 'FATEC DE CONTAGEM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4076, 4393, 'INSTITUTO SUPERIOR DE CINCIAS DO COMPORTAMENTO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4077, 4394, 'FACULDADE DE EDUCAO FSICA DE ARAGUANA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4078, 4395, 'FACULDADE DE DIREITO DE ARAGUANA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4079, 4396, 'FACULDADES INTEGRADAS DE MINEIROS', 3, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4080, 4397, 'ESCOLA SUPERIOR DE ENSINO DA SEICHO-NO-IE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4081, 4398, 'INSTITUTO SUPERIOR DE EDUCAO DE GOIANA', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4082, 4399, 'FACULDADE DE BALSAS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4083, 4400, 'INSTITUTO DE ESTUDOS DA SADE - PS GRADUAO EM CINCIAS DA SADE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4084, 4401, 'FACULDADE DE APIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4085, 4403, 'FACULDADE DA ACADEMIA MEMORIAL DE ENSINO SUPERIOR DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4086, 4404, 'INSTITUTO SEDES SAPIENTIAE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4087, 4405, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA REGIONAL DE FOZ DO IGUAU', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4088, 4406, 'ASSOCIACAO BRAS DE ODONTO SEC DE GOIAS SUB SEC DE JATAI', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4089, 4407, 'FACULDADE DE ENSINO SUPERIOR DA TERRA PAULISTA - FATERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4090, 4408, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL ABO/RN REGIONAL MOSSOR', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4091, 4409, 'ASSOCIAO INSTITUTO DE FOMENTO EDUCACIONAL FENIX', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4092, 4410, 'CRDA PS GRADUAES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4093, 4411, 'FACULDADE PANAMERICANA DE JI-PARAN', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4094, 4412, 'FACULDADE VALE DO ARAGUAIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4095, 4413, 'ASSOCIAO VITA CARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4096, 4414, 'INSTITUTO SUPERIOR DE EDUCAO VALE DO ARAGUAIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4097, 4415, 'FSD - FACULDADE SO DOMINGOS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4098, 4416, 'FACULDADE CIMAN', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4099, 4417, 'FACULDADE DE ENFERMAGEM DE BELO JARDIM', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4100, 4418, 'FACULDADE SO BERNARDO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4101, 4419, 'INSTITUTO DE PESQUISA E ENSINO EM SADE ANIMAL - IPESA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4102, 4420, 'FACULDADES INTEGRADAS BARROS MELO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4103, 4421, 'FACULDADE DE TECNOLOGIA SENAI BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4104, 4422, 'FACULDADE DE TECNOLOGIA DO ABC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4105, 4423, 'FACULDADE ADVENTISTA DE ENFERMAGEM DO NORDESTE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4106, 4424, 'LIBERTAS CLNICA ESCOLA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4107, 4425, 'FACULDADE CORPO & MENTE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4108, 4426, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA-REGIONAL ANPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4109, 4427, 'FACULDADE DAS CINCIAS MULTIPLAS E DO MEIOAMBIENTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4110, 4428, 'FACULDADE PROFESSOR MIGUEL NGELO DA SILVA SANTOS', 3, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4111, 4429, 'FACULDADE UNIGRAN CAPITAL', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4112, 4430, 'ITAL - INSTITUTO DE TECNOLOGIA DE ALIMENTOS', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4113, 4431, 'FACULDADE DE ENFERMAGEM NOVA ESPERANA DE MOSSOR - FACENE/RN', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4114, 4432, 'FACULDADE GERAO POSITIVA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4115, 4433, 'FAWEST- FACULDADE WESTPHALIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4116, 4434, 'INSTITUTO DE DIREITO ROMEU FELIPE BACELLAR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4117, 4435, 'FACULDADE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4118, 4436, 'FACULDADE BIG MASTER', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4119, 4437, 'CENTRO ODONTOLGICO DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4120, 4438, 'ACADEMIA BRASILEIRA DE DIREITO CONSTITUCIONAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4121, 4439, 'COOENSINO - COOPERATIVA DE ENSINO UNIVERSAL DOIS MIL', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4122, 4440, 'FACULDADE DE MEDICINA NOVA ESPERANA DE NATAL - FAMENE/RN', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4123, 4441, 'SISTEMA EDUCACIONAL SUPERIOR UNIVERSITRIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4124, 4442, 'ESCOLA SUPERIOR PAULISTA DE ADMINISTRAO - ESPA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4125, 4443, 'FACULDADE RIO CLARO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4126, 4444, 'CBPEX PS-GRADUAO', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4127, 4445, 'FACULDADE MEDIANEIRA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4128, 4446, 'FACULDADE CIDADE DE GUANHES - FACIG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4129, 4447, 'SOCIEDADE BRASILEIRA DE DIREITO PUBLICO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4130, 4448, 'SOCIEDADE BRASILEIRA DE DIREITO PUBLICO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4131, 4449, 'FACULDADE DE TECNOLOGIA DO NORTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4132, 4450, 'FACULDADE METROPOLITANA DA AMAZNIA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4133, 4451, 'FACULDADE TEOLOGICA E EDUCACIONAL AGAPE COMUNITARIA - FATEAC', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4134, 4452, 'FACULDADE METROPOLITANA DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4135, 4454, 'FACULDADE PITGORAS DE NOVA LIMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4136, 4455, 'GESTOS - GRUPO DE ESTUDOS ORTODNTICOS E SERVIOS S/C LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4137, 4456, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4138, 4457, 'INSTITUTO SUPERIOR DE EDUCAO DE ITAPECIRICA DA SERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4139, 4458, 'INSTITUTO DE ENSINO SUPERIOR DE MACEI - IESMA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4140, 4459, 'INEES - INSTITUTO DE EDUCAO E ESPECIALIZAO SUPERIOR', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4141, 4460, 'FACULDADE DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4142, 4461, 'FACULDADE DE TECNOLOGIA SENAI CSAR RODRIGUES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4143, 4462, 'ESCOLA DE GESTO E CONTROLE FRANCISCO JURUENA', 2, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4144, 4463, 'GRUPO DE ESTUDO ODONTOLGICO E SERVIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4145, 4464, 'FACULDADE LAGEANA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4146, 4465, 'CENTRO ODONTOLOGICO DE ESTUDOS E PESQUISAS - COESP', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4147, 4466, 'CENTRO DE ESTUDOS PROF DIRCEU VIEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4148, 4467, 'IACE - INSTITUTO AVANADO DE CINCIAS E EDUCAO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4149, 4468, 'INSTITUTO PICE DE ENSINO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4150, 4469, 'FACULDADE HEXGONO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4151, 4470, 'TVALLE ASSESSORIA E TREINAMENTO EDUCACIONAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4152, 4471, 'FACULDADE DE TECNOLOGIA POLITEC', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4153, 4472, 'CESDE BRASIL INTELIGENCIA EDUCACIONAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4154, 4473, 'FACULDADE DE TECNOLOGIA ACCTIVA DE CINCIA E TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4155, 4474, 'ESCOLA PAULISTANA DE ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4156, 4475, 'ESCOLA CAPIXABA DE ODONTOLOGIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4157, 4476, 'ESCOLA PAULISTA DE ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4158, 4477, 'ENSINO  DISTNCIA - SLMANDIC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4159, 4478, 'ESCOLA BRASILIENSE DE ODONTOLOGIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4160, 4479, 'ESCOLA CHAPECOENSE DE ODONTOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4161, 4480, 'ESCOLA CAMBORI DE ODONTOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4162, 4481, 'ESCOLA FLUMINENSE DE ODONTOLOGIA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4163, 4482, 'ESCOLA DO NORDESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4164, 4483, 'ESCOLA CEARENSE DE ODONTOLOGIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4165, 4484, 'CENTRO DE PESQUISAS ODONTOLGICAS SO LEOPOLDO MANDIC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4166, 4485, 'ESCOLA FORENSE DE ODONTOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4167, 4486, 'ESCOLA MINEIRA DE ODONTOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4168, 4487, 'SOBRACURSOS - CURSOS DE PS-GRADUAO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4169, 4488, 'ESCOLA METROPOLITANA DE ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4170, 4489, 'ESCOLA SANTISTA DE ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4171, 4490, 'FACULDADE SENADOR CANEDO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4172, 4491, 'FACULDADE TECNOLOGICA ITAI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4173, 4492, 'FACULDADE PASCHOAL DANTAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4174, 4493, 'FACULDADE DE TECNOLOGIA CERTUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4175, 4494, 'COORDENADORIA DE ENSINO  DISTNCIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4176, 4495, 'FACULDADE DE TECNOLOGIA JARAGUAENSE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4177, 4496, 'FACULDADE DE TECNOLOGIA DO VALE DO IVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4178, 4497, 'WP - CENTRO DE PSICOTERAPIA COGNITIVO-COMPORTAMENTAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4179, 4498, 'CENTRO DE ENSINO TEORICO TECNICO DE TRANSITO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4180, 4499, 'CENTRO DE ENSINO TEORICO TECNICO DE TRANSITO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4181, 4500, 'FACULDADE DE TECNOLOGIA EM SADE DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4182, 4501, 'FACULDADE DE TECNOLOGIA ALTO TIET', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4183, 4502, 'FACULDADES INTEGRADAS CAMPO-GRANDENSES', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4184, 4503, 'UNIVERSIDADE FEDERAL DO RECNCAVO DA BAHIA', 1, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4185, 4504, 'FUNDAO UNIVERSIDADE FEDERAL DA GRANDE DOURADOS', 1, 1, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4186, 4505, 'FACULDADE FACISA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4187, 4506, 'CEFE - CENTRO DE ESTUDOS DE FISIOLOGIA DO EXERCCIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4188, 4507, 'INSTITUTO SUPERIOR XAVIER CORDEIRO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4189, 4508, 'FACULDADE DE ENSINO SUPERIOR DE FLORIANO - PI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4190, 4509, 'INSTITUTO SUL BRASILEIRO DE ENSINO SUPERIOR - INSBES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4191, 4510, 'FASOMA-FACULDADE SOLUO MARANHENSE LTDA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4192, 4511, 'IPPEO INSTITUTO PARANAENSE DE PESQUISA E ENSINO EM ODONTOLOGIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4193, 4512, 'FACULDADE GUIAS DO DIREITO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4194, 4513, 'FATEB', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4195, 4514, 'ESCOLA SUPERIOR DA MAGISTRATURA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4196, 4515, 'ABO-REGIONAL DE TAGUATINGA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4197, 4516, 'INSTITUTO CHIAVENATO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4198, 4517, 'FACULDADE DE EDUCAO DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4199, 4518, 'FACULDADE REGIONAL PALMITOS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4200, 4519, 'OPEG - FACULDADE DE TECNOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4201, 4520, 'FACULDADE CENACAP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4202, 4521, 'FACULDADE DE CINCIAS APLICADAS DE ITAPEVA - FACITA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4203, 4522, 'CENTRO UNIVERSITRIO CATLICO SALESIANO AUXILIUM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4204, 4523, 'FACULDADE DO INSTITUTO EDUCACIONAL PROFISSIONALIZANTE DE JALES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4205, 4524, 'INSTITUTO TECNOLOGICO DE AVALIAO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4206, 4525, 'FACULDADE TECNOLOGICA DE EXCELENCIA EDUCACIONAL DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4207, 4526, 'FACULDADES INTEGRADAS LAVOISIER', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4208, 4527, 'FACULDADE DE URUAU', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4209, 4528, 'IBEXP - INSTITUTO BRASILEIRO DE EXECUTIVOS PROFISSIONAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4210, 4530, 'FACULDADE INTEGRADA TIRADENTES', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4211, 4531, 'FACULDADE ADVENTISTA DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4212, 4532, 'FACULDADE DE TECNOLOGIA SENAI CAMPO GRANDE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4213, 4533, 'FACULDADE SO FRANCISCO DA PARABA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4214, 4534, 'FACULDADE DE TECNOLOGIA HERRERO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4215, 4535, 'FACULDADE DA AMAZNIA - FAMA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4216, 4536, 'IPEO-MT', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4217, 4537, 'INSTITUTO SUPERIOR DE EDUCAO DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4218, 4538, 'FACULDADE PARQUE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4219, 4539, 'FACULDADE DE TEOLOGIA E CINCIAS HUMANAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4220, 4540, 'FACULDADES CONE SUL', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4221, 4541, 'CENTRO BAIANO DE ESTUDOS ODONTOLGICOS - CEBEO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4222, 4542, 'INSTITUTO BIOSFERA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4223, 4543, 'FACULDADE DE EDUCAO E ESTUDOS SUPERIORES DE AREIA - FAESA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4224, 4544, 'FACULDADE LDER', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4225, 4545, 'FACULDADE PARANATINGA - FAP', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4226, 4546, 'FACULDADE METODISTA UNIO URUGUAIANA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4227, 4547, 'CENTRO DE ESTUDOS E APERFEICOAMENTO PROFISSIONAL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4228, 4548, 'CAP- CENTRO DE APERFEIOAMENTO PROFISSIONAL', 3, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4229, 4550, 'INSTITUTO DE ENSINO PEDAGOGOS ASSOCIADOS - INESPA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4230, 4551, 'FACULDADES ASSOC. DE MNACO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4231, 4552, 'FACULDADE ECUMENICA LAYRA DE ALTO PARAISO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4232, 4553, 'FACULDADE MERIDIONAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4233, 4554, 'FUNDAO CARDIOVASCULAR SO FRANCISCO DE ASSIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4234, 4555, 'INSTITUTO DE EDUCAO A DISTNCIA NORTE FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4235, 4556, 'FACULDADE EVOLUO DE CINCIAS E TECNOLOGIA - FECET', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4236, 4557, 'CLINICA INTEGRADA DE ODONTOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4237, 4558, 'FACULDADE CENECISTA DE ALTA FLORESTA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4238, 4559, 'INSTITUTO SUPERIOR LAGEANO DE EDUCAO A DISTNCIA - ISLED', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4239, 4560, 'FACULDADE DE SENHOR DO BOMFIM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4240, 4561, 'FACULDADE SANTO ANTNIO DE QUEIMADAS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4241, 4562, 'FACULDADE LUZ DE GUARABIRA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4242, 4563, 'FACULDADE DE TECNOLOGIA DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4243, 4564, 'FACULDADE SANTA RITA DE CASSIA FASARC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4244, 4565, 'INSTITUTO EDUCACIONAL SEMINRIO PAULOPOLITANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4245, 4566, 'FACULDADE DE EXCELNCIA EDUCACIONAL DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4246, 4567, 'FACULDADE CENTRO MATO-GROSSENSE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4247, 4568, 'FACULDADE EA DE QUEIROZ - SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4248, 4569, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL DA APCD REGIONAL TAUBAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4249, 4570, 'FACULDADE DE TEOLOGIA DE SELVRIA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4250, 4571, 'INSTITUTO IPESP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4251, 4572, 'FACULDADE DE TEOLOGIA E EDUCAO DE MINAS GERAIS-FATEMIG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4252, 4573, 'IBTEC - INSTITUTO BRASILEIRO DE TECNOLOGIA E CINCIAS', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4253, 4574, 'FACULDADE JACQUES COUSTEAU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4254, 4575, 'INSTITUTO DE CINCIAS DO TRABALHO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4255, 4576, 'CENTRO DE ENSINO SUPERIOR LEOPOLDINENSE ALBERT EINSTEIN', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4256, 4577, 'ISAE FGV PARAN - INSTITUTO SUPERIOR DE ADMINISTRAO E ECONOMIA DO PARAN - FUNDAO GETLIO VARGAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4257, 4578, 'FACULDADE CARLOS CHAGAS FILHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4258, 4579, 'FACULDADE DO SUDESTE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4259, 4580, 'FACULDADE REUNIDA DE ILHA SOLTEIRA - MESMO QUE 1625', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4260, 4581, 'CENTRO LUMN DE EXTENSO E PS-GRAFUAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4261, 4582, 'FACULDADE CENTRO SUL DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4262, 4583, 'FACULDADE PITGORAS DE ITABUNA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4263, 4584, 'FACULDADE DE TECNOLOGIA MACHADO DE ASSIS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4264, 4585, 'ESCOLA SUPERIOR DA MAGISTRATURA DE PERNAMBUCO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4265, 4586, 'FACULDADE DO NORTE GOIANO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4266, 4587, 'FACULDADE UNIVERSAL DE FORMAO TECNOLGICA E SUPERIOR - UNIFOS', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4267, 4588, 'FACULDADE APOGEU', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4268, 4589, 'FACULDADE DE DIREITO DO DISTRITO FEDERAL - FDDF', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4269, 4590, 'FACULDADE DE TECNOLOGIA FATEP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4270, 4592, 'FACULDADE DE OURO PRETO DO OESTE', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4271, 4593, 'CENTRO EURO-AMERICANO DE GESTO E HOSPITALIDADE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4272, 4594, 'FACULDADE CATLICA DE RONDONIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4273, 4595, 'DEPARTAMENTO DE APERFEIOAMENTO PROFISSIONAL - DAP', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4274, 4596, 'FACULDADE DE ADMINISTRAO E NEGCIOS EM RIBEIRO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4275, 4597, 'FACULDADE DE CINCIAS GERENCIAIS EM VOTUPORANGA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4276, 4598, 'FACULDADE DE CINCIAS GERENCIAIS BARO DE JUNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4277, 4599, 'INSTITUTO EL SHADDAY DE TEOLOGIA E MISSES', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4278, 4600, 'FACULDADE DE TECNOLOGIA DE MACAP - FATEMA', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4279, 4601, 'FACULDADE TECNOLOGICA CRISTO REI', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4280, 4602, 'FUNDAO ESCOLA SUPERIOR DO MPDFT', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4281, 4603, 'INSTITUTO DE TECNOLOGIA EM SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4282, 4604, 'FWM - FACULDADE TEOLGICA E APOLOGTICA DR WALTER MARTIN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4283, 4605, 'FACULDADE DE ENSINO SUPERIOR KM 125', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4284, 4606, 'FACULDADE DE TECNOLOGIA CONTEC - UNIDADE DE CARAPINA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4285, 4607, 'ESCOLA SUPERIOR KLYMUS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4286, 4608, 'FACULDADE 2001', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4287, 4609, 'FATESP - FACULDADE TEOLGICA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4288, 4610, 'FACULDADE DE TECNOLOGIA ASSESSORITEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4289, 4611, 'FACULDADE PLANALTO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4290, 4612, 'ISEBES - INSTITUTO BETH SHALOM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4291, 4613, 'FACULDADE DE EDUCAO E MEIO AMBIENTE', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4292, 4614, 'FACULDADE APROVAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4293, 4615, 'ACADEMIA DE GESTO FISCAL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4294, 4616, 'FACULDADE KANTUN', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4295, 4617, 'ATAME- PS-GRADUAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4296, 4618, 'E.A.P. ESCOLA DE APERFEIOAMENTO PROFISSIONAL ABO-RJ REGIONAL DUQUE DE CAXIAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4297, 4619, 'SINDICATO DOS ODONTOLOGISTAS DO ESTADO DE SAO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4298, 4620, 'FACULDADE EVANGLICA - CEILNDIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4299, 4621, 'FACULDADE EVANGLICA - GUAS CLARAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4300, 4622, 'FACULDADE EVANGLICA - TAGUATINGA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4301, 4623, 'FACULDADE LATINO-AMERICANA DE TEOLOGIA INTEGRAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4302, 4624, 'FACULDADE DE ODONTOLOGIA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4303, 4625, 'A&C CONSULTING LTDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4304, 4626, 'FACIDER II', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4305, 4627, 'SO JUDAS - FACULDADES INTEGRADAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4306, 4628, 'ESCOLA SUPERIOR VERBO JURIDICO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4307, 4629, 'FACULDADE DE TECNOLOGIA FUNDETEC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4308, 4630, 'FACULDADE DE TECNOLOGIA TECMED', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4309, 4631, 'FACULDADE ANGLO-AMERICANO DE JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4310, 4632, 'FACULDADE ANGLO-AMERICANO DE CAXIAS DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4311, 4633, 'FACULDADE ANGLO-AMERICANO DE PASSO FUNDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4312, 4634, 'ESCOLA AGROTCNICA FEDERAL DE ALEGRE', 1, 1, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4313, 4635, 'FACULDADE INTEGRADO DA COMCAM', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4314, 4636, 'FACULDADES INTEGRADAS INTERAMERICANAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4315, 4637, 'FACULDADE DE RELAES INTERNACIONAIS SRGIO VIEIRA DE MELLO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4316, 4638, 'FACULDADE TEOLGICA BATISTA EQUATORIAL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4317, 4639, 'FACULDADE IJUI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4318, 4640, 'ESCOLA DE TERAPIA MANUAL E POSTURAL S/S LTDDA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4319, 4641, 'UNIDIDTICO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4320, 4642, 'FACULDADE UNIREAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4321, 4643, 'INSTITUTO SO PAULO DE ESTUDOS SUPERIORES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4322, 4644, 'UNIEXODOS - FACULDADE DE TECNOLOGIA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4323, 4645, 'INSTITUTO BRASILEIRO DE CHI KUNG E TERAPIAS AFINS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4324, 4646, 'IPG - INSTITUTO DE PS-GRADUAO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4325, 4647, 'FACULDADE DE REABILITAO DO PLANALTO CENTRAL - FARPLAC', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4326, 4648, 'DEPARTAMENTO DE ENSINO E CULTURA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4327, 4649, 'JRK ESCOLA DE PS GRADUAO EM ODONTOLOGIA LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4328, 4650, 'FACULDADE DA AMAZNIA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4329, 4651, 'FACULDADES ANDEF', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4330, 4652, 'FACULDADE ANHANGERA DE SO JOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4331, 4653, 'FACULDADE COMUNITRIA DE GUARATINGUET', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4332, 4654, 'FACULDADE COMUNITRIA DE BAUR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4333, 4655, 'FACULDADE ANHANGERA DE SOROCABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4334, 4656, 'FACULDADE ANHANGERA DE PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4335, 4657, 'FACULDADE DE EDUCAO E ENSINO SUPERIOR TECNOLGICO  FAEST', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4336, 4658, '3158 - INSTITUTO SUPERIOR DE EDUCAO ANGLO-AMERICANO DE FOZ DO IGUAU - ISEAAFI', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4337, 4659, 'FACULDADE DE TEOLOGIA, EDUCAO E CINCIAS - FATEC', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4338, 4660, 'UNIPESSOA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4339, 4661, 'FACULDADE DE AGRONEGCIO PARASO DO NORTE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4340, 4662, 'INSTITUTO LATINO AMERICANO DE PESQUISA E ENSINO ODONTOLGICO - ILAPEO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4341, 4663, 'NAP INSTITUTO DE ENSINO SUPERIOR LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4342, 4664, 'FAEPO - NCLEO DE PS-GRADUAO E PESQUISA EM ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4343, 4665, 'SOCIEDADE DE EDUCACAO E CULTURA CAICARAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4344, 4666, 'FACULDADE DE RONDNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4345, 4667, 'FACULDADE RIO SONO', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4346, 4668, 'FACULDADE COMUNITRIA DE SUMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4347, 4669, 'FACULDADE DELTA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4348, 4670, 'FACULDADE DE ENSINO SUPERIOR E TECNOLOGIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4349, 4671, 'IOCP - CEPS - CENTRO DE ESTUDOS E PS-GRADUAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4350, 4672, 'FACULDADE EMPREENDEDORA AURORA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4351, 4673, 'ESCOLA SUPERIOR DE TEOLOGIA E FILOSOFIA DO BRASIL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4352, 4674, 'FACULDADE PORTO DAS GUAS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4353, 4675, 'JUSPODIVM', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4354, 4676, 'SENAI DENDEZEIROS', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4355, 4677, 'UNIVERSIDADE JOS DO ROSRIO VELLANO - UNIFENAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4356, 4678, 'INSTITUTO DE DERMATOLOGIA, MEDICINA E CIRURGIA ESTETICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4357, 4679, 'FESPPAR - FACULDADE EDUCACIONAL DE PINHAIS DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4358, 4680, 'FEPPAR - FACULDADE EDUCACIONAL DE PIRAQUARA DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4359, 4681, 'FACULDADE CENTRO PAULISTANO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4360, 4682, 'FACULDADE INTEGRADA BRASILEIRA RENASCENA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4361, 4683, 'PROJECTO CENTRO CULTURAL E DE FORMAO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4362, 4684, 'BUSINESS SCHOOL CEDEPE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4363, 4685, 'FACULDADES INTEGRADAS PLURAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4364, 4686, 'FACULDADE SO BASILIO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4365, 4687, 'FACULDADE DE TEOLOGIA BRASILEIRA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4366, 4688, 'FACULDADE DE DIREITO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4367, 4689, 'INSTITUTO DE ENSINO SUPERIOR PASSIONISTA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4368, 4690, 'FACULDADE DE EDUCAO SUPERIOR E TEOLGICA MARANHENSE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4369, 4691, 'INSTITUTO PARANAENSE DE PESQUISA, DESENVOLVIMENTO E PS-GRADUAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4370, 4692, 'FACULDADE VITORIO BONACIN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4371, 4693, 'FACULDADE DE TECNOLOGIA DE NOVA ANDRADINA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4372, 4694, 'FACULDADE DE TEOLOGIA E CIENCIAS DA EDUCAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4373, 4695, 'UB FACULDADE TECNOLGICA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4374, 4696, 'FASE - FACULDADE DE TECNOLOGIA DO AMAZONAS', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4375, 4697, 'MOUSE COM CURSOS PROFISSIONALIZANTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4376, 4698, 'TECNOLOGIA RADIOLGICA E RADIOTERPICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4377, 4699, 'FACULDADE DE EDUCAAO, CIENCIA E TECNOLOGIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4378, 4700, 'FACULDADE NOVA ROMA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4379, 4701, 'FACULDADE CATLICA DE CAMPINA GRANDE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4380, 4702, 'FACULDADE DE TECNOLOGIA GESTO & MARKETING', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4381, 4703, 'INSTITUTO D ENSINO SUPERIOR CASTRO ALVES', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4382, 4704, 'FACULDADE PONTE SERRADA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4383, 4705, 'FACULDADE UNIECO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4384, 4706, 'FACULDADE WELLINGTON', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4385, 4707, 'NCLEO DE EDUCAO CONTINUADA - NEC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4386, 4708, 'FACULDADE SINOPENSE DE ENSINO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4387, 4709, 'IRMANDADE DA SANTA CASA DE MISERICRDIA DE PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4388, 4710, 'INSTITUTO CABRERA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4389, 4711, 'UNIDADE DE ENSINO SUPERIOR DE APUCARANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4390, 4712, 'FACULDADES CASA DO SABER', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4391, 4713, 'ASSOCIAO DE ENSINO SUPERIOR EDUCARE - EDUCARE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4392, 4714, 'FACULDADE ALTERNATIVA DE ENSINO SUPERIOR DO AGRESTE - FAESA', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4393, 4715, 'BRIGADA AMBIENTAL DO MUNICPIO DE SO PEDRO DA ALDEIA', 3, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4394, 4716, 'FACULDADE DE MEDICINA DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4395, 4717, 'INSTITUTO SUPERIOR DE EDUCAO SANTA MARIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4396, 4718, 'FACULDADE DE TECNOLOGIA SANTA MARIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4397, 4719, 'FACULDADE DE TECNOLOGIA IJUI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4398, 4720, 'IBREAS - INSTITUTO BRASILEIRO DE ESTUDOS AVANADOS EM SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4399, 4721, 'ESCOLA DOMINICANA DE TEOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4400, 4722, 'FACULDADES INTEGRADAS VIANNA JNIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4401, 4723, 'ACADEMIA NACIONAL DE POLCIA', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4402, 4724, 'FACULDADE DE CINCIAS HUMANAS DE CRUZEIRO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4403, 4725, 'FACULDADE DE TECNOLOGIA EGDIO JOS DA SILVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4404, 4726, 'FACULDADE EQUIPE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4405, 4727, 'FACULDADE DE EDUCAO, CINCIA E TECNOLOGIA EM CAMOCIM', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4406, 4728, 'FACULDADE CENECISTA DE QUISSAM', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4407, 4729, 'FACULDADE CENECISTA DE RIO BONITO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4408, 4730, 'FACULDADE LUSIADA DE JI-PARANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4409, 4731, 'FACULDADE MESSIANICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4410, 4732, 'FACULDADE DE TECNOLOGIA SENAC DF', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4411, 4733, 'VITA CARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4412, 4734, 'FACULDADES INTEGRADAS EDUCAR BRASIL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4413, 4735, 'FACULDADE ATENAS MARANHENSE - FAMA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4414, 4736, 'MORAES BASTOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4415, 4737, 'FAMEM - FACULDADE DO MDIO MEARIM', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4416, 4738, 'INSTITUTO BIODELTA DE ENSINO E PESQUISA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4417, 4739, 'FACULDADE PRESBITERIANA AUGUSTO GALVO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4418, 4740, 'FACULDADE DE PATROCINIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4419, 4741, 'FACULDADE CENECISTA DE MARIC-FACMAR', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4420, 4742, 'FACULDADE SANTA EMLIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4421, 4743, 'FACULDADE DE EDUCAO INTEGRADA E TECNOLOGIA - FACEI', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4422, 4744, 'FACULDADE LITERATUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4423, 4745, 'FACULDADE LITORNEA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4424, 4746, 'FACULDADE CATLICA DO CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4425, 4747, 'FACULDADE REGIONAL DE RIACHO DO JACUPE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4426, 4748, 'FACULDADE DE TECNOLOGIA DE SANTA CATARINA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4427, 4749, 'FACULDADE ACCTIVA DE CINCIA E TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4428, 4750, 'FACULDADE SER', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4429, 4751, 'FACULDADE DE TECNOLOGIA E CIENCIAS DO NORTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4430, 4752, 'FACULDADE NOTARIAL E REGISTRAL DO BRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4431, 4753, 'FACILETEC- FACULDADES DE CIENCIAS, LETRAS, E DE TECNOLOGIA DE TURMALINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4432, 4754, 'FACULDADES INTEGRADAS LUSO BRASILEIRA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4433, 4755, 'AMADO@ASSOCIADOS IRH S/C LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4434, 4756, 'CENTRO UNIVERSITRIO MUNICIPAL DE SO JOS', 3, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4435, 4757, 'MASTER FACE CLINICA ODONTOLOGICA LTDA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4436, 4758, 'FACULDADE DE BAIXO GUANDU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4437, 4759, 'FACULDADE DE ITAPEMIRIM', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4438, 4760, 'INSTITUTO DE ENSINO SUPERIOR BRASLIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4439, 4761, 'CETAO-CENTRO DE ESTUDOS TREINAMENTO E APERFEIOAMENTO EM ODONTOLOGIA S/C LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4440, 4762, 'FACULDADE IGUAUANO LEONARDO CARIELO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4441, 4763, 'FATERJ - FACULDADE TEOLOGICA EVANGLICA DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4442, 4764, 'ESCOLA DE SADE PBLICA DR JORGE DAVID NASSER', 2, 1, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4443, 4765, 'FACULDADE DE TECNOLOGIA ALFA DE UMUARAMA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4444, 4766, 'FACULDADE UNA DE CONTAGEM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4445, 4767, 'FACULDADE QUALITAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4446, 4768, 'INSTITUTO DE CINCIAS GERENCIAIS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4447, 4769, 'FACULDADES EMPREZA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4448, 4770, 'J.G.MELO CURSOS DE ESPECIALIZAO EDUCACIONAL', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4449, 4771, 'FACULDADE BI CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4450, 4772, 'FACULDADE BI SOCIAL QUARESMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4451, 4773, 'FACULDADE BI MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4452, 4774, 'INSTITUTO ELO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4453, 4775, 'ESCOLA DE APERFEICOAMENTO E FORMACAO PROFISSIONAL PROF. PAULO SERODIO MELLO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4454, 4776, 'FACULDADE COMUNITRIA DE UB', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4455, 4777, 'FACULDADE KENNEDY DE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4456, 4778, 'FACULDADE CENECISTA DE TEUTNIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4457, 4779, 'INSTITUTO NACIONAL DE PESQUISAS DA AMAZNIA - INPA', 1, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4458, 4780, 'FACISA-UNAI FACULDADE DE CINCIAS DA SADE DE UNA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4459, 4781, 'FACULDADE DE COMUNICAO TECNOLOGIA E TURISMO DE OLINDA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4460, 4782, 'FACULDADE DE TECNOLOGIA OXFORD', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4461, 4783, 'LIBERTAS CLNICA-ESCOLA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4462, 4784, 'FACULDADE DE TECNOLOGIA CDL DE FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4463, 4785, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DE RONDNIA', 1, 1, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4464, 4786, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA DO TOCANTINS', 1, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4465, 4787, 'ESCOLA AGROTCNICA FEDERAL DE SO JOO EVANGELISTA', 1, 1, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4466, 4788, 'INSTITUTO CENTRO DE ENSINO TECNOLGICO - CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4467, 4789, 'INSTITUTO CENTRO DE ENSINO TECNOLGICO - LIMOEIRO DO NORTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4468, 4790, 'FACULDADE DE TECNOLOGIA CENTEC - SOBRAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4469, 4791, 'FACULDADE DE TECNOLOGIA CENTEC - CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4470, 4792, 'FACULDADE DE TECNOLOGIA CENTEC - LIMOEIRO DO NORTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4471, 4793, 'FACULDADES INTEGRADAS DE ITARAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4472, 4794, 'ESCOLA SUPERIOR DE ENGENHARIA E GESTO DE VALINHOS - ESEG', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4473, 4795, 'FACREI - FACULDADE CRISTO REI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4474, 4796, 'FACULDADE DE CINCIAS APLICADAS DE CAMARAGIBE - FAC', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4475, 4797, 'GOVERNADOR PAULO MALUF', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4476, 4798, 'FCS - FACULDADE DE CINCIAS DA SADE', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4477, 4799, 'CENTRO ODONTOLOGICO PIO XII', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4478, 4800, 'FACULDADE DE TECNOLOGIA CENECISTA DE RIO NEGRINHO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4479, 4801, 'FACULDADE LITERATUS - JA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4480, 4802, 'FACULDADE LITERATUS - PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4481, 4803, 'FACULDADE SENAI DE TECNOLOGIA -', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4482, 4804, 'SECONE-SERVIOS DE CONSULTORIA EDUCACIONAL', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4483, 4805, 'FACULDADE AMBIENTALISTA COMUNITRIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4484, 4806, 'INSTITUTO DE EDUCAO COELHONETENSE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4485, 4807, 'INSTITUTO SUPERIOR DE ODONTOLOGIA - ISO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4486, 4808, 'FACULDADE ATHENEU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4487, 4809, 'INSTITUTO SUPERIOR DE EDUCAO JK VALPARASO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4488, 4810, 'FACULDADE ANTNIO MENEGHETTI', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4489, 4811, 'FACULDADE SENAI DE TECNOLOGIA ROBERTO SIMONSEN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4490, 4812, 'FACULDADE SENAI DE TECNOLOGIA ROBERTO MANGE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4491, 4813, 'FACULDADE SENAI DE TECNOLOGIA GASPAR RICARDO JUNIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4492, 4814, 'FACULDADE DE TECNOLOGIA SENAI FELIX GUISARD', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4493, 4815, 'FACULDADE DE TECNOLOGIA SENAI NADIR DIAS DE FIGUEIREDO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4494, 4816, 'FACULDADE SENAI DE TECNOLOGIA SUIO-BRASILEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4495, 4817, 'FACULDADE DE TECNOLOGIA SENAI ANCHIETA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4496, 4818, 'FACULDADE SENAI DE TECNOLOGIA HORCIO AUGUSTO DA SILVEIRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4497, 4819, 'FACULDADE DE TECNOLOGIA SENAI MARIANO FERRAZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4498, 4820, 'FACULDADE DE TECNOLOGIA SENAI ANTNIO ADOLPHO LOBBE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4499, 4821, 'FACULDADE DE CINCIAS GERENCIAIS E EMPREENDEDORISMO - FACIGE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4500, 4822, 'FACULDADE DE TEOLOGIA BATISTA BETEL - FTBB', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4501, 4823, 'FACULDADE DE TECNOLOGIA EVOLUO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4502, 4824, 'INSTITUTO LABORATIVA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4503, 4825, 'FACULDADE DE CINCIAS E TECNOLOGIA DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4504, 4826, 'FACULDADE ANHANGERA DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4505, 4827, 'CENTRO AVANADO DE ENSINO,EDUCAO E CULTURA-CAEEC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4506, 4828, 'FACULDADE DE TECNOLOGIA CAMPIUOL', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4507, 4829, 'ESCOLA SUPERIOR DE ARTES CLIA HELENA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4508, 4830, 'SEMINARIO MAIOR IMACULADO CORAO DE MARIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4509, 4831, 'FACULDADE DE TECNOLOGIA DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4510, 4832, 'FACULDADE POLITCNICA DE TAUBAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4511, 4833, 'FACULDADE DE TECNOLOGIA DE VALENA - FACTIVA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4512, 4834, 'INSTITUTO DE EDUCAO MORUMBI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4513, 4836, 'FACULDADE DE EDUCAO  ACREANO EUCLIDES DA CUNHA', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4514, 4837, 'FACULDADE IESPLAC', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4515, 4838, 'INSTITUTO SUPERIOR DE EDUCAO MONTEIRO LOBATO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4516, 4839, 'FACULDADE DE CINCIAS, LETRAS E ARTES LUIZA DE MARILLAC', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4517, 4840, 'HD HEALTH DENTAL ENSINO ODONTOLGICOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4518, 4841, 'FACULDADE WITTENBERG', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4519, 4842, 'INSTITUTO RACINE EDITORA INDUSTRIAL E SERVIOS LTDA.', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4520, 4843, 'INSTITUTO DE TEOLOGIA, PASTORAL E ENSINO SUPERIOR DA AMAZNIA - ITEPES', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4521, 4844, 'FTC EAD-FACULDADE DE TECNOLOGIA E CINCIAS ENSINO A DISTNCIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4522, 4845, 'FACULDADE DA IGREJA MIISTERIO FAMA - FAIFA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4523, 4846, 'FACULDADE VRTICE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4524, 4847, 'FACULDADE DIOCESANA DE MOSSOR  FDM', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4525, 4848, 'FACULDADE DE TECNOLOGIA E NEGCIOS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4526, 4849, 'FACULDADE DE CINCIAS HUMANAS, ECONMICAS E DA SADE DE ARAGUANA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4527, 4850, 'MUSEU DE ASTRONOMIA E CINCIAS AFINS - MAST', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4528, 4851, 'FACULDADE E INSTITUTO SUPERIOR TECNOLGICO E EDUCACIONAL - FATEC PARACATU', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4529, 4852, 'FACULDADE E INSTITUTO SUPERIOR TECNOLGICO E EDUCACIONAL - FATEC PATOS DE MINAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4530, 4853, 'FACULDADE E INSTITUTO SUPERIOR TECNOLGICO E EDUCACIONAL - FATEC UBERABA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4531, 4854, 'FACULDADE RADIAL CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4532, 4855, 'FACULDADE DO CENTRO DO RIO GRANDE DO SUL - FACERGS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4533, 4856, 'FACULDADE DE ADMINISTRAO, CINCIAS HUMANAS E SOCIAIS - BOM JESUS  GO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4534, 4857, 'IEST- INSTITUTO DE EDUCAO SUPERIOR DE TAGUATINGA SOCIEDADE SIMPLES', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4535, 4858, 'FACULDADE DE ENFERMAGEM DE ARCOVERDE', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4536, 4859, 'FACULDADE DE TECNOLOGIA FAESA - UNIDADE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4537, 4860, 'CENTRO BRASILEIRO DE EXCELNCIA EM PS-GRADUAO NA SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4538, 4861, 'FACULDADE PITGORAS DE POUSO ALEGRE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4539, 4862, 'FACULDADE PITGORAS DE JUIZ DE FORA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4540, 4863, 'FACULDADE PITGORAS DE POOS DE CALDAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4541, 4864, 'FACULDADE PITGORAS DE GOVERNADOR VALADARES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4542, 4865, 'FACULDADE PITGORAS DE SO LUIZ', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4543, 4866, 'FACULDADE PITGORAS DE CONTAGEM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4544, 4867, 'FACULDADE PITGORAS DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4545, 4868, 'FACULDADE HSM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4546, 4869, 'FACULDADE TECNOLGICA DE LONDRINA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4547, 4870, 'FACULDADE DE SOBRAL - FASOL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4548, 4871, 'FACULDADE INTEGRADAS DO MEORENO EM CARUARU-FIAMCA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4549, 4872, 'FACULDADE ETNIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4550, 4873, 'FACULDADE DE TECNOLOGIA DE TAUBAT', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4551, 4874, 'INSTITUTO DE EDUCAO E CULTURA DO OESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4552, 4875, 'FACULDADES INTEGRADAS CIRCUITO DAS GUAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4553, 4876, 'FACULDADE TECNOLGICA GERAO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4554, 4877, 'FACULDADE MESSEJANA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4555, 4878, 'FACULDADE ANHANGUERA DE ITAPECERICA DA SERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4556, 4879, 'FUNDAO PARA O DESENVOLVIMENTO CIENTIFICO E TECNOLOGICO DA ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4557, 4880, 'FACULDADE DE DIREITO DE GUARANT DO NORTE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4558, 4881, 'FACULDADE VILAVELHENSE', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4559, 4882, 'FACULDADE DE IBIRAU', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4560, 4883, 'FACULDADE DE MEDICINA DE MINAS GERAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4561, 4884, 'FACULDADE ATENAS MARANHENSE DE IMPERATRIZ MA.', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4562, 4885, 'FACULDADE PORTO SEGURO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4563, 4886, 'INSTITUTO SUPERIOR DE EDUCAO PORTO SEGURO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4564, 4887, 'FACULDADE DO CIRCUITO DAS GUAS PAULISTA - FAC AP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4565, 4888, 'CURSO NACIONAL DE ATUALIZAO EM NUTROLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4566, 4889, 'FACULDADE DE CONCHAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4567, 4890, 'FACULDADE DO FUTURO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4568, 4891, 'FACULDADE DE TEOLOGIA INTEGRADA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4569, 4892, 'FUNDAO DE ADMINISTRAO E PESQUISA ECONMICO-SOCIAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4570, 4893, 'FACULDADE DE GESTO E TECNOLOGIA ASSUMPO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4571, 4894, 'INSTITUTO DE ENSINO E PESQUISA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4572, 4895, 'ESCOLA DE APERFEIOAMENTO PROFISSIONAL- ABO REGIONAL DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4573, 4896, 'INSTITUTO FLEURY', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4574, 4897, 'PROCEAD - PROMOTORA DE CURSOS LTDA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4575, 4898, 'FACULDADE DE EDUCAO E ARTES SAO GERALDO', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4576, 4899, 'FACULDADE CABRLIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4577, 4900, 'FACULDADE VALE DO OURO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4578, 4901, 'FACULDADE FASIPE', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4579, 4902, 'FACULDADE BATISTA PIONEIRA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4580, 4903, 'FACULDADE MONITOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4581, 4904, 'FACULDADE SENAI DE TECNOLOGIA " EUVALDO LODI "', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4582, 4905, 'FACULDADE SENAI DE TECNOLOGIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4583, 4906, 'FACULDADE SENAI DE TECNOLOGIA MARCELINO CORRADI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4584, 4907, 'FACULDADE SENAI DE TECNOLOGIA "JOSE FAGUNDES NETTO "', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4585, 4908, 'FACULDADE SENAI DE TECNOLOGIA DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4586, 4909, 'FACULDADE SENAI DE TECNOLOGIA " FIDELIS REIS "', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4587, 4910, 'FACULDADE DO TRABALHO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4588, 4911, 'CEPAEM - CENTRO PARANAENSE DE ESTUDOS MDICOS LTDA', 4, 2, 'AC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4589, 4912, 'FACULDADE EXPOENTE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4590, 4913, 'FACULDADE SO VICENTE DE LAVRAS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4591, 4914, 'SENSU CONSULTORIA ORGANIZACIONAL LTDA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4592, 4915, 'FACULDADE DE CASTANHAL', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4593, 4916, 'FACULDADE CATLICA DO SUL DO MARANHO - CASUMA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4594, 4917, 'ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4595, 4918, 'FACULDADE CENECISTA DE JARDIM DO SERID', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4596, 4919, 'FACULDADE UNIPROCESSUS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4597, 4920, 'FACULDADE DO ENSINO SUPERIOR DO MUNICPIO DE LINHARES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4598, 4921, 'FACULDADE DE ALMIRANTE TAMANDAR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4599, 4922, 'FACULDADE INTEGRADA DAS CATARATAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4600, 4923, 'INSTITUTO SABER', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4601, 4924, 'CENTRO DE ESTUDOS AVANADOS DA CONSERVAO INTEGRADA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4602, 4925, 'FUNDAO UNIVERSIDADE FEDERAL DO ABC', 1, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4603, 4926, 'FACULDADE DE TECNOLOGIA DE PRAIA GRANDE', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4604, 4927, 'INSTITUTO SUPERIOR DE CINCIAS DA SADE VERDE NORTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4605, 4928, 'INSTITUTO SUPERIOR DE CINCIAS SOCIAIS APLICADAS VERDE NORTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4606, 4929, 'CESUC - CENTRO DE ENSINO SUPERIOR DE CUIABA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4607, 4930, 'FACULDADE DE SADE DE PAULISTA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4608, 4931, 'INSTITUTO SUPERIOR DE EDUCAO CECAP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4609, 4932, 'IES CEAJUFE INSTITUIO DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4610, 4933, 'FACULDADE POLITCNICA DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4611, 4934, 'FACULDADE DE ENSINO SUPERIOR DE CIENCIAS HUMANAS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4612, 4935, 'FACULDADE DO TRABALHO DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4613, 4936, 'INSTITUTO AGORA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4614, 4937, 'INSTITUTO LOGOS DE EDUCAO SUPERIOR', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4615, 4938, 'FACULDADE CLARETIANA DE TEOLOGIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4616, 4939, 'FACULDADE 2 DE JULHO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4617, 4940, 'CENTRO DE ESTUDOS, PESQUISAS E PS-GRADUAO EM ODONTOLOGIA - CEPPO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4618, 4941, 'FACULDADES INTEGRADAS DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4619, 4942, 'FAEME - FACULDADE EVANGLICA DO MEIO NORTE DE MARITUBA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4620, 4943, 'ESCOLA SUPERIOR DE ADMINISTRAO E GESTO DA BAIXADA SANTISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4621, 4944, 'DENTAL PRESS - CENTRO DE PESQUISA E PS-GRADUAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4622, 4945, 'FACULDADE INTERNACIONAL DO DELTA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4623, 4946, 'FACULDADE DE AGRONOMIA E EDUCAO TECNOLGICA DE SOORETAMA - FAETES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4624, 4947, 'FACULDADE VITORIA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4625, 4948, 'FACULDADES DAS GUAS QUENTES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4626, 4949, 'FCA - FACULDADE CIRCUITO DAS GUAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4627, 4950, 'FACULDADE AUM', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4628, 4951, 'FACULDADE ABERTA DE CINCIAS MODERNAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4629, 4952, 'FACULDADE TEOLGICA SULBRASILEIRA - FATESUL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4630, 4953, 'CENTRO DE ESTUDOS E PESQUISAS EM ORTODONTIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4631, 4954, 'ESCOLA DE ENSINO SUPERIOR ALTERNATIVO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4632, 4956, 'ESCOLA PAULISTA DA MAGISTRATURA', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4633, 4957, 'FACULDADE BRASILEIRA DE CULTURA E EDUCAO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4634, 4958, 'FACULDADE CORPORATIVA CESPI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4635, 4959, 'FACULDADE PITGORAS DE FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4636, 4960, 'FACULDADE DE TECNOLOGIA LAGOA DA PAMPULHA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4637, 4961, 'FACULDADE INTEGRADA DE ARAGUATINS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4638, 4962, 'CENTRO UNIVERSITRIO DE SETE LAGOAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4639, 4963, 'FACULDADE PRESBITERIANA MACKENZIE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4640, 4964, 'FACULDADE DE CINCIAS E TECNOLOGIA DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4641, 4965, 'FACULDADE VEZ DO MESTRE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4642, 4967, 'ESCOLA SUPERIOR DE DIREITO MUNICIPAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4643, 4969, 'FACULDADE ITOP', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4644, 4970, 'FACULDADE FLORES DA CUNHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4645, 4971, 'CENTRO DE ESTUDOS E TCNICAS ODONTOLGICAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4646, 4972, 'FACULDADE ALFREDO NASSER DE REMANSO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4647, 4973, 'FACULDADE SENAI DE TECNOLOGIA DE POOS DE CALDAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4648, 4974, 'INSTITUTO CATARINENSE DE ESTUDOS AVANADOS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4649, 4975, 'FACULDADE OBO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4650, 4976, 'INSTITUTO DE ESTUDOS SUPERIORES DO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4651, 4977, 'FACULDADE DE TECNOLOGIA APOENA - FTA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4652, 4978, 'ALPHA SMILE CENTRO CURSOS PESQUISAS ODON COM LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4653, 4979, 'FACULDADE DE LAVRAS DA MANGABEIRA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4654, 4980, 'CLIVO POS GRADUAO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4655, 4981, 'FACULDADE UNICESP SADE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4656, 4982, 'FACULDADE METROPOLITANA DO PLANALTO NORTE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4657, 4983, 'FACULDADE DE SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4658, 4984, 'ASSOCIAO DE ENSINO SUPERIOR BARO DE JUNDIA (AESB)', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4659, 4985, 'FACULDADE PARA O DESENVOLVIMENTO SUSTENTVEL DO NORDESTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4660, 4986, 'CENTRO UNIVERSITRIO DA TERRA PAULISTA - UNITERRA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4661, 4987, 'ESCOLA SUPERIOR DE APERFEIOAMENTO PROFISSIONAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4662, 4988, 'FACULDADE DE TECNOLOGIA INED DE IPATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4663, 4989, 'CITE - CENTRO INTEGRADO DE TERAPIAS ENERGTICAS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4664, 4990, 'INSTITUTO DE DESENVOLVIMENTO TECNOLGICO DO CENTRO-OESTE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4665, 4991, 'FACULDADE DE TECNOLOGIA FAMEG - FAMETEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4666, 4992, 'FACULDADE DE TECNOLOGIA CENTEC - SERTO CENTRAL', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4667, 4993, 'FACULDADE DE TECNOLOGIA DO IBRATE - FAITEC *', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4668, 4994, 'FACULDADE LOURENO FILHO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4669, 4995, 'FACULDADE CRUZ AZUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4670, 4996, 'FACULDADE TECNOLGICA DO ESTADO DO ESPRITO SANTO - FATEES', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4671, 4997, 'FACULDADE DO AGRESTE SETENTRIONAL DE PERNAMBUCO - FASPE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4672, 4998, 'FACULDADE DE TECNOLOGIA INESUL DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4673, 4999, 'FACULDADE INTEGRADO DO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4674, 5000, 'FACULDADE ESTAO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4675, 5001, 'CENTRO UNIVERSITRIO AUTNOMO DO BRASIL - UNIBRASIL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4676, 5002, 'SINDICATO DOS PROFESSORES DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4677, 5003, 'FACULDADE DE CINCIAS MDICAS DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4678, 5004, 'ABO SEO CEAR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4679, 5005, 'CEUNI - COMPLEXO EDUCACIONAL UNICIDADE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4680, 5006, 'FACULDADE REDENTOR DE CAMPOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4681, 5007, 'FACULDADE ORTODOXA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4682, 5008, 'FACULDADE DO MDIO PARNABA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4683, 5009, 'FACULDADE DO AMAZONAS - FAMA', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4684, 5010, 'FACULDADE DENIZARD RIVAIL', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4685, 5011, 'FACULDADE BATISTA DO CARIRI', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4686, 5012, 'FACULDADE RELIGARE PIAUIENSE', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4687, 5013, 'CENTRO UNIVERSITRIO ESTADUAL DA ZONA OESTE', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4688, 5014, 'CENTRO TECNOLGICO UNIVERSITRIO DO ESTADO DO RIO DE JANEIRO', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4689, 5015, 'INSTITUTO SUPERIOR DE TECNOLOGIA EM HORTICULTURA', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4690, 5016, 'INSTITUTO SUPERIOR DE TECNOLOGIA EM CINCIAS DA COMPUTAO DO RIO DE JANEIRO', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4691, 5017, 'INSTITUTO SUPERIOR DE TECNOLOGIA EM CINCIA DA INFORMAO DE PETRPOLIS', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4692, 5018, 'INSTITUTO SUPERIOR DE EDUCAO PROFESSOR ALDO MUYLAERT', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4693, 5019, 'INSTITUTO SUPERIOR DE EDUCAO DA ZONA OESTE', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4694, 5020, 'INSTITUTO SUPERIOR DE EDUCAO DE SANTO ANTNIO DE PDUA', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4695, 5021, 'INSTITUTO SUPERIOR DE EDUCAO DE ITAPERUNA', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4696, 5022, 'FACULDADES INTEGRADAS PREVE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4697, 5023, 'FACULDADE SANTO AUGUSTO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4698, 5024, 'ESCOLA AGROTCNICA FEDERAL DE SERTO', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4699, 5025, 'FACULDADE DE EDUCAO SO BRAZ', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4700, 5026, 'UNIRATI - INSTITUIO DE ENSINO SUPERIOR IRATI S/C LTDA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4701, 5027, 'FAPETRO - FACULDADE DE PETRLEO E GS LTDA', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4702, 5028, 'IBEFIN - INSTITUTO BRASILEIRO DE ESTUDOS EM FINANAS E NEGCIOS', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4703, 5029, 'FAMA - FACULDADE DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4704, 5030, 'FACULDADE DE FILOSOFIA E CINCIAS DO PLANALTO - FAFICIP', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4705, 5031, 'ESCOLA SUPERIOR NACIONAL DE SEGUROS DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4706, 5032, 'ILAPE - PS-GRADUAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4707, 5033, 'FACULDADE PRISMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4708, 5034, 'EDUCC - EDUCAO PROF. CAP. E CONSULTORIA LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4709, 5035, 'FACULDADE DA FUNDAO UNIVERSITRIA DE CARDIOLOGIA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4710, 5036, 'INSTITUTO FEDERAL DE EDUCAO, CINCIA E TECNOLOGIA CATARINENSE', 1, 1, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4711, 5037, 'FUNDAO DE APOIO AO INSTITUTO DE PESQUISAS TECNOLGICAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4712, 5038, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS DE MARAB', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4713, 5039, 'FACULDADE DE PSICOLOGIA - FUNFARME', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4714, 5040, 'FACULDADE BAHIANA DE ENGENHARIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4715, 5041, 'ITEP - INSTITUTO DE TECNOLOGIA ENSINO E PESQUISA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4716, 5042, 'SEMINARIO NACIONAL ITEJ', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4717, 5043, 'INSTITUTO SUPERIOR DE EDUCAO DE CARAPICUIBA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4718, 5044, 'FACULDADE DE DIREITO DE GARANHUNS', 3, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4719, 5045, 'FACULDADE DE TECNOLOGIA E AGRONEGCIO DE LUPRCIO - FAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4720, 5046, 'FACULDADE POLIS DAS ARTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4721, 5047, 'FACULDADE DO INTERIOR PAULISTA - FIP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4722, 5048, 'FACULDADE DE TEOLOGIA DE SO PAULO DA IGREJA PRESBITERIANA INDEPENDENTE DO BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4723, 5049, 'INSTITUTO DE ENSINO SUPERIOR SAO FRANCISO DE ASSIS DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4724, 5050, 'FACULDADE ARAUTOS DO EVANGELHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4725, 5051, 'INSTITUTO SUPERIOR DE EDUCAO BOM JESUS DO ITABAPOANA', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4726, 5052, 'INSTITUTO SUPERIOR DE EDUCAO TRS RIOS', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4727, 5053, 'INSTITUTO SUPERIOR TECNOLGICO DE PARACAMBI', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4728, 5054, 'INSTITUTO SUPERIOR DE EDUCAO DO SERTO DO PAJE', 2, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4729, 5055, 'INSTITUTO LOBO PARA O DESENVOLVIMENTO DA EDUCAO, DA CINCIA E DA TECNOLOGIA (NO IES)', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4730, 5056, 'FACULDADE DE TECNOLOGIA UNIBRATEC MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4731, 5057, 'FACULDADE SO VICENTE DE PAULO - FASVIP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4732, 5058, 'FACULDADE DE TECNOLOGIA PEDRO ROGRIO GARCIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4733, 5059, 'INSTITUTO DO CORAO DO DISTRITO FEDERAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4734, 5060, 'FACULDADE DE TECNOLOGIA DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4735, 5061, 'FACULDADE NATAL - FN', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4736, 5062, 'FACULDADE DE PIRACICABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4737, 5063, 'FACULDADE DE TECNOLOGIA INED - UNIDADE TIMTEO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4738, 5064, 'CENTRO DE FORMAO, TREINAMENTO E APERFEIOAMENTO', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4739, 5065, 'FACULDADE CONSULTIME DO ESPIRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4740, 5066, 'FACULDADE DE TECNOLOGIA INED - UNIDADE CONTAGEM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4741, 5067, 'FACULDADE PITGORAS DE UBERLNDIA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4742, 5068, 'FACEAR - FACULDADE EDUCACIONAL ARAUCRIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4743, 5069, 'FACULDADE PADRAO DE PORANGATU', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4744, 5070, 'FACULDADE DE MEDICINA CARLOS CHAGAS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4745, 5071, 'FACULDADE NORTE CAPIXABA DE SO MATEUS - FAMAT', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4746, 5072, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - SEO MARANHO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4747, 5073, 'FACULDADE PITGORAS DE DIVINPOLIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4748, 5074, 'INSTITUTO EDUCACIONAL DE ENSINO SUPERIOR FACLESTE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4749, 5075, 'ESCOLA AGROTCNICA FEDERAL DE CCERES', 1, 1, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4750, 5076, 'BRAZIL GLOBAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4751, 5077, 'UNIVERSIDADE ESTADUAL DE RORAIMA', 2, 1, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4752, 5078, 'UNIPED', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4753, 5079, 'FACULDADE EDUCACIONAL DE MATELNDIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4754, 5080, 'CETECLAGOS - CENTRO DE CAPACITAO TECNOLGICA DA REGIO DOS LAGOS LTDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4755, 5081, 'DIFERENCIAL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4756, 5082, 'FUNETEC', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4757, 5083, '1344 - ESCOLA SUPERIOR DE HOTELARIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4758, 5084, 'FACIAC - FACULDADE DE CINCIAS APLICADAS DE CATANDUVAS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4759, 5085, 'FACULDADE REGIONAL DE TIMB', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4760, 5086, 'ESAMC - ESCOLA SUPERIOR DE ADMINISTRAO, MARKETING E COMUNICAO DE MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4761, 5087, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SEO DISTRITO FEDERAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4762, 5088, 'IBRAFA - CENTRO DE ESTUDOS EM SADE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4763, 5089, 'FACULDADE DE TECNOLOGIA APOENA - FTA', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4764, 5090, 'FACULDADE DO PROFESSOR', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4765, 5091, 'INSTITUTO DE ESTUDOS SUPERIORES E PS-GRADUAO - IESP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4766, 5092, 'UNIVALE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4767, 5093, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA - SECO DO ESTADO DO RIO GRANDE DO NORTE', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4768, 5094, 'FACULDADE EVOLUO DO ALTO OESTE POTIGUAR (FACEP)', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4769, 5095, 'FIESU - FACULDADE INTERNACIONAL DE ENSINO SUPERIOR', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4770, 5096, 'ESCOLA AGROTCNICA FEDERAL DE ARAGUATINS', 1, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4771, 5097, 'FACULDADE VALE DOS CARAJS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4772, 5098, 'FACULDADE CELEIRO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4773, 5099, 'FACULDADE DE TECNOLOGIA DE SO BERNARDO DO CAMPO', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4774, 5100, 'FACULDADE MACHADINHO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4775, 5101, 'FACULDADE CENTRO OESTE DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4776, 5102, 'INSTITUTO NACIONAL DE ENSINO AVANADO LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4777, 5103, 'IEP - INSTITUTO DE ESTUDOS PREVIDENCIARIOS', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4778, 5104, 'ESAD - ESCOLA DE ADMINISTRAO E NEGCIOS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4779, 5105, 'FACULDADE INTERNACIONAL SIGNORELLI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4780, 5106, 'INSTITUTO DE GESTO EDUCACIONAL SIGNORELLI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4781, 5107, 'FACULDADE SOGIPA DE EDUCAO FSICA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4782, 5108, 'ASSOCIAO INSTITUTO SAPIENTIAE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4783, 5109, 'ESCOLA SUPERIOR DE TERAPIA INTEGRAL', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4784, 5110, 'FAINP - FACULDADE DE INTEGRAO DO NOROESTE PARANAENSE', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4785, 5111, 'INSTITUTO DE ENSINO E PESQUISA E PS GRADUAO EM EDUCAO E SADE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4786, 5112, 'FACULDADE UNIO ARARUAMA DE ENSINO S/S LTDA.', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4787, 5113, 'FACULDADE UNIMAC', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4788, 5114, 'INSTITUTO DE PESQUISAS EDUCACIONAIS PRAXIS LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4789, 5115, 'FACULDADE EXPANSO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4790, 5116, 'FACULDADE EVOLUO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4791, 5117, 'INSTITUTO DE EDUCAO SUPERIOR DE SO LOURENO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4792, 5118, 'FACULDADE MEGA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4793, 5119, 'FACULDADES MATISSE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4794, 5120, 'FACULDADE DINMICA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4795, 5121, 'SEMINRIO BAIANO DE TEOLOGIA - SEBATE', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4796, 5122, 'INSTITUTO CIMAN DE EDUCAO E ENSINO SUPERIOR', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4797, 5123, 'ESCOLA DA MAGISTRATURA DO ESTADO DO RIO DE JANEIRO', 2, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4798, 5124, 'FACULDADE FACMIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4799, 5125, 'INSTITUTO DE ENSINO E PESQUISA EM ODONTOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4800, 5126, 'ESCOLA BRASILEIRA DE MEDICINA CHINESA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4801, 5127, 'FACULDADE SOCIAL DE SO PAULO - FASSP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4802, 5128, 'FACULDADE SANTO ANDR - FASA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4803, 5129, 'ESCOLA AGROTCNICA FEDERAL DE CERES', 1, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4804, 5130, 'CEDEFACE - CENTRO DE PESQUISA E PS-GRADUAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4805, 5131, 'FACULDADE DE TECNOLOGIA SENAC CAADOR', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4806, 5132, 'FACULDADE SENAC CONCRDIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4807, 5133, 'FACULDADE DE TECNOLOGIA SENAC SO MIGUEL DO OESTE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4808, 5134, 'FACULDADE SENAC CRICIMA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4809, 5135, 'CENTRO UNIVERSITRIO DE RIO PRETO - UNIRP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4810, 5136, 'FACULDADE SO BOAVENTURA DE EDUCAO CONTINUADA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4811, 5137, 'FATEC - FACULDADE DE TEOLOGIA E CINCIAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4812, 5138, 'INSTITUTO DELMIRENSE DE INSINO SUPERIOR', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4813, 5139, 'FACULDADE SANTA MARIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4814, 5140, 'IPEO/MT', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4815, 5141, 'ESCOLA AGROTCNICA FEDERAL ANTNIO JOS TEIXEIRA', 1, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4816, 5142, 'FACULDADES TECNOLOGICAS DA SERRA GAUCHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4817, 5143, 'FACULDADE LUTERANA DE TEOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4818, 5144, 'UNIO METROPOLITANA DE EDUCAO E CULTURA S/C LTDA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4819, 5145, 'SOCIEDADE DE ENSINO SUPERIOR CEDRO S/S LTDA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4820, 5146, 'FACULDADE PETROLINA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4821, 5147, 'ACADEMIA CEARENSE DE ODONTOLOGIA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4822, 5148, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SEO BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4823, 5149, 'ESCOLA SUPERIOR DE GESTO E CINCIAS DA SADE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4824, 5150, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4825, 5151, 'FACULDADE DE DIREITO (NOME FICTCIO APENAS PARA FINS DE CADASTRAMENTO E OBTENO DE SENHA)', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4826, 5152, 'ESCOLA DA ADVOCACIA-GERAL DA UNIO', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4827, 5153, 'CENTRO DE PS GRADUAO - ILADEM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4828, 5154, 'ASSSOCIAO BRASILEIRA DE ODONTOLOGIA SEO ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4829, 5155, 'ABO - REGIONAL UMUARAMA  - PR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4830, 5156, 'UNIVERSIDADE CAMILO CASTELO BRANCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4831, 5157, 'FACULDADES TECNOLGICAS DA SERRA GACHA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4832, 5158, 'FACULDADE MARECHAL CNDIDO RONDON - FAMAR', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4833, 5159, 'CENTRO AVANADO DE ORTODONTIA PAULO PICANO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4834, 5160, 'FACULDADE COMUNITRIA DE PINDAMONHANGABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4835, 5161, 'FACULDADE COMUNITRIA DE AMERICANA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4836, 5162, 'FACULDADE COMUNITRIA DO PLANALTO CENTRAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4837, 5163, 'FACULDADE COMUNITRIA DE RIBEIRO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4838, 5164, 'FACULDADE COMUNITRIA DE OSASCO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4839, 5165, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SEO RIO GRANDE DO SU8L', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4840, 5166, 'FACULDADE DE CINCIAS DA SADE HIPCRATES', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4841, 5167, 'FACULDADE DOMINGOS OLIMPIO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4842, 5168, 'ESCOLA AGROTECNICA FEDERAL DE CATU', 1, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4843, 5169, 'UNICETREX - PS-GRADUAO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4844, 5170, 'UNIVERCAPANEMA - FACULDADES DE LETRAS, CINCIAS, TECNOLOGIA E DE BIOTECNOLOGIA DA SADE DE CAPANEMA', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4845, 5171, 'GRUPO DO RIO DE JANEIRO DE ESTUDOS DE ORTODONTIA PELA TCNICA DO STRAIGHT-WIRE DE ANDREWS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4846, 5172, 'FACULDADE DE TECNOLOGIA CENTEC - SO GONALO DO AMARANTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4847, 5173, 'FACULDADE DE TECNOLOGIA DE PIRASSUNUNGA - FATEPI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4848, 5174, 'BRAZILIAN BUSINESS SCHOOL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4849, 5175, 'FACULDADE IDAAM', 4, 2, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4850, 5176, 'FACULDADE DE DIREITO VALE DO ASS', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4851, 5177, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SEO MS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4852, 5178, 'MIRANDA E CARVALHO INSTITUTO DE PESQUISA, PS-GRADUAO E SERVIOS EM ODONTOLOGIA LTDA.', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4853, 5179, 'FACULDADE SAINT PAUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4854, 5180, 'FACULDADE DO POVO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4855, 5181, 'FACULDADE MOGIANA DO ESTADO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4856, 5182, 'IRMANDADE DA SANTA CASA DE MISERICRDIA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4857, 5183, 'INSTITUTO TECNOLGICO DE ENSINO SUPERIOR BANDEIRANTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4858, 5184, 'FACULDADE DE CINCIAS E TECNOLOGIA DA BAHIA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4859, 5185, 'FACULDADE DO INSTITUTO NACIONAL DE PS-GRADUAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4860, 5186, 'FACULDADE DINMICA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4861, 5187, 'FAEJA - FACULDADE ALBERT EINSTEIN DE JACIARA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4862, 5188, 'EXPANSO GESTO EM EDUCAO E EVENTOS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4863, 5189, 'FACULDADE BAIANA DE DIREITO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4864, 5190, 'INSTITUTO CATUA DE ENSINO SUPERIOR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4865, 5191, 'FAMAC - FACULDADE DE MACHADINHO DO OESTE', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4866, 5192, 'CENTES - CENTRO TECNOLGICO DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4867, 5193, 'FEIT - FACULDADE DE ENGENHARIA E INOVAO TECNOLGICA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4868, 5194, 'ESCOLA SUPERIOR DE TEOLOGIA E ESPIRITUALIDADE FRANCISCANA - ESTEF', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4869, 5195, 'FACULDADE DO VALE DO PAJE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4870, 5196, 'INSTITUTO NOSSA SENHORA DO PILAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4871, 5197, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SECO PIAUI', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4872, 5198, 'UNIBRASILIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4873, 5199, 'CENTRO  AVANADO DE ENSINO,EDUCAO E CULTURA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4874, 5200, 'FACULDADE DE TECNOLOGIA INED - UNIDADE BETIM', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4875, 5201, 'FACULDADE DE TECNOLOGIA INED - UNIDADE BARREIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4876, 5202, 'FACULDADE INDEPENDNCIA - FAI', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4877, 5203, 'FACULDADE MONTESSORIANO DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4878, 5204, 'INSTITUTO SUPERIOR DE EDUCAO ANTONINO FREIRE - ISEAF', 2, 1, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4879, 5205, 'INSTITUTO DE EDUCAO SUPERIOR SUL BRASILEIRO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4880, 5206, 'IMEN - INSTITUTO DE METABOLISMO E NUTRIO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4881, 5207, 'ATUALIZA PS-GRADUAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4882, 5208, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA SEO ESPIRITO SANTO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4883, 5209, 'INSTITUTO DE PS-GRADUAO DO PETRLEO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4884, 5210, 'FATESMA - FACULDADE DE TECNOLOGIA SANTA MARIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4885, 5211, 'INSTITUTO SUPERIOR ISAG DE FORMAO CONTINUADA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4886, 5212, 'COMISSO CIENTFICA SPP', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4887, 5213, 'INSTITUTO FISIOMAR', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4888, 5214, 'CENTRO DE ESTUDOS ODONTOLGICOS DO IPSEMG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4889, 5215, 'FACULDADES INTEGRADAS PADRE ALBINO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4890, 5216, 'FACULDADE UIRAPURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4891, 5217, 'FACULDADES INTEGRADAS PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4892, 5218, 'FACULDADE DE TECNOLOGIA PROFESSOR WALDOMIRO MAY', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4893, 5219, 'INSTITUTO SUPERIOR DE ENSINO EM FONOAUDIOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4894, 5220, 'INSTITUTO DE PSGRADUAO E ATUALIZAO EM ODONTOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4895, 5221, 'FUNDAO ANTARES DE ENSINO SUPERIOR , PS-GRADUAO, PESQUISA E EXTENSO', 1, 1, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4896, 5222, 'ESCOLA AGROTCNICA FEDERAL DE SOUSA', 1, 1, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4897, 5223, 'COORDENAO GERAL DE EDUCAO DA ANDIMA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4898, 5224, 'CENTRO DE PS GRADUAO, ESTUDOS, PESQUISA E EXTENSO- INSPIRAR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4899, 5225, 'IMEO - INSTITUTO MINEIRO DE ESTUDOS ODONTOLGICOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4900, 5226, 'IMEO - INSTITUTO MINEIRO DE ESTUDOS ODONTOLGICOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4901, 5227, 'SEPAI -  SERVIO DE PSICANLISE EM ATENO A INFNCIA / COORDENAO DE ASSUNTOS EDUCACIONAIS - CAE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4902, 5228, 'FACULDADE RAIMUNDO MARINHO', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4903, 5229, 'IPDR - INSTITUTO DE PS-GRADUAO DAVID ROCHA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4904, 5230, 'INSTITUTO PRISMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4905, 5231, 'LFG ESCOLA NACIONAL DE ESPECIALIZAO (LFG-ENESP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4906, 5232, 'INSTITUTO TOCANTINENSE DE EDUCAO  DISTNCIA', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4907, 5233, 'FUNDAO DE APOIO  PESQUISA E ESTUDO NA REA DE SADE - FAPES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4908, 5234, 'FASS - SISTEMA DE ENSINO A DISTNCIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4909, 5235, 'FACULDADE DE TECNOLOGIA IBRATEC', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4910, 5236, 'CENTRO DE ESTUDOS SUPERIORES DA FUNDACOR', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4911, 5237, 'INSTITUTO SUPERIOR DE FORMAO CONTINUADA DOUGLAS ANDREANI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4912, 5238, 'INSTITUTO SAPIENTIA DE EDUCAO SUPERIOR - ISES', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4913, 5239, 'SBAC - CENTRO DE PS -GRADUAO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4914, 5240, 'ASSOCIAO BRASILEIRA DE ODONTOLOGIA-SECO DO PARAN-REGIONAL DE PONTA GROSSA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4915, 5241, 'MUSEU PARAENSE EMILIO GOELDI', 1, 1, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4916, 5242, 'UNIVERSIDADE ESTADUAL DE ALAGOAS - UNEAL', 2, 1, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4917, 5243, 'FACULDADE DE CINCIAS JURDICAS DE SANTOS DUMONT', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4918, 5244, 'ASSOCIAO DOS GESTORES DE CONHECIMENTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4919, 5245, 'SOCIEDADE DE HIPNOSE MDICA DO ESTADO DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4920, 5246, 'CENTRO DE ESTUDOS AVANCADOS EM SAUDE UNIMAGEM', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4921, 5247, 'INAPS - INSTITUTO NACIONAL DE ENSINO SUPERIOR E PS-GRADUAO PADRE GERVSIO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4922, 5248, 'INAPS - INSTITUTO NACIONAL DE ENSINO SUPERIOR E PS-GRADUAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4923, 5249, 'V. F.LIMA CURSOS AVANADOS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4924, 5250, 'FACULDADE EVOLUTIVO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4925, 5251, 'INAPS - INSTITUTO NACIONAL DE ENSINO SUPERIOR E PS-GRADUAO PADRE GERVSIO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4926, 5252, 'INAPS - INSTITUTO NACIONAL DE ENSINO SUPERIOR E PS-GRADUAO PADRE GERVSIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4927, 5253, 'INAPS - INSTITUTO NACIONAL DE ENSINO SUPERIOR E PS-GRADUAO PADRE GERVSIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4928, 5254, 'INSTITUTO SUPERIOR DE ODONTOLOGIA FACIAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4929, 5255, 'FACULDADE REGIONAL DE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4930, 5256, 'CENTRO DE ENSINO ESPECIAL SUPERIOR E PROFISSIONALIZANTE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4931, 5257, 'APROBATUM - CENTRO NACIONAL DE QUALIFICAO DE PESSOAL LTDA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4932, 5258, 'INSTITUTO SUPERIOR ORTHODONTIC INTERNACIONAL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4933, 5259, 'AGRICULTURA TROPICAL E SUBTROPICAL', 2, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4934, 5260, 'FACULDADE DE CRUZEIRO DO OESTE - FACO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4935, 5261, 'CENTRO DE EDUCAO DO HEMOCENTRO DE RIBEIRO PRETO - CEDUC-HRP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4936, 5262, 'INSTITUIO DE ENSINO UNIORKA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4937, 5263, 'CEU- CENTRO DE EXTENSO UNIVERSITARIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4938, 5264, 'INSTITUTO SUPERIOR DO MINISTRIO PBLICO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4939, 5265, 'FACULDADE METROPOLITANA SO CARLOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4940, 5266, 'INSTITUTO DE EDUCAO SCULO XXI LTDA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4941, 5267, 'INSTITUTO EFICAZ CURSOS PROFISSIONALIZANTES DE MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4942, 5268, 'FACULDADE DE SO JOAQUIM DA BARRA LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4943, 5269, 'FACULDADE CRIST', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4944, 5270, 'ESCOLA NACIONAL DE SADE PBLICA - ENSP', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4945, 5271, 'ESCOLA NACIONAL DE SADE PBLICA', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4946, 5272, 'INSTITUTO DE PS-GRADUAO PIO XII', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4947, 5273, 'FACULDADE SANTA CRUZ', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4948, 5274, 'INSTITUTO SUPERIOR CULTURAL BRASILEIRO - ISCB', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4949, 5275, 'FACULDADE DE CINCIAS GERENCIAIS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4950, 5276, 'FACULDADE DE DIREITO DE CARANGOLA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4951, 5277, 'FACULDADE FORTIUM', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4952, 5278, 'FACULDADE ATUAL', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4953, 5279, 'CRDA PS GRADUAES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4954, 5280, 'FACULDADE DE EDUCACAO SUPERIOR COELHONETENSE', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4955, 5281, 'FACULDADE DE EDUCAO, CINCIA E TECNOLOGIA DO VALE DO ACARA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4956, 5282, 'CENTRO DE ESTUDOS DE PESSOAL', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4957, 5283, 'HD ENSINOS ODONTOLGICOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4958, 5284, 'FACULDADE PARA O ENSINO DAS CINCIAS JURDICAS INDEPENDENTEMENTE DE DISTNCIA - FECJID', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4959, 5285, 'FACULDADES INTEGRADAS DE TAQUARA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4960, 5286, 'INSTITUTO DE CINCIAS SOCIAIS E ECONMICAS LTDA', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4961, 5287, 'NEO - NCLEO DE ESTUDOS ODONTOLGICOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4962, 5288, 'FACULDADE ANHANGERA DE RIBEIRO PRETO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4963, 5289, 'FACULDADE DE DIREITO DO ESTADO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4964, 5290, 'FACULDADE DE CINCIAS E TECNOLOGIA DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4965, 5291, 'FACULDADE ARAUTOS DO EVANGELHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4966, 5292, 'FACULDADE VIPACS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4967, 5293, 'FACULDADE ANHANGERA DE SUMAR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4968, 5294, 'FACULDADE DINMICA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4969, 5295, 'CEPEO - CENTRO DE ENSINO E PESQUISA EM ODONTOLOGIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4970, 5296, 'ESCOLA DA VILA - ESPECIALIZAO PEDAGGICA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4971, 5297, 'ESCOLA NACIONAL DE ACUPUNTURA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4972, 5298, 'FACULDADE METAFSICA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4973, 5299, 'FACULDADE NOVO ATENEU DE GUARAPUAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4974, 5300, 'FAKCEN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4975, 5301, 'FACULDADE RAZES', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4976, 5302, 'IDEAL PS GRADUAO S/S LTDA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4977, 5303, 'FACULDADE ANHANGERA DE DOURADOS', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4978, 5304, 'FACULDADE DE ENFERMAGEM DE SO SEBASTIO DO PARASO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4979, 5305, 'FACULDADE DE CINCIAS SOCIAIS APLICADAS E HUMANAS DE GARANHUNS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4980, 5306, 'INSTITUTO SUPERIOR DE CINCIAS AGRRIAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4981, 5307, 'FASES - FACULDADE SHALOM DE ENSINO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4982, 5308, 'IM&T - INSTUTO MUNDO & TRABALHO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4983, 5309, 'CEPRENO- CENTRO DE ESTUDOS PAULISTA DE REABILITAO NEUROCLUSAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4984, 5310, 'FACULDADE CIMAN', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4985, 5311, 'FACULDADE GABARITO', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4986, 5312, 'FACULDADES ATIBAIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4987, 5313, 'FACULDADES UNIFICADAS DOCTUM DE CATAGUASES', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4988, 5314, 'FACULDADES UNIFICADAS DOCTUM DE GUARAPARI', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4989, 5315, 'FACULDADES UNIFICADAS DOCTUM DE INA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4990, 5316, 'FACULDADES UNIFICADAS DOCTUM DE LEOPOLDINA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4991, 5317, 'FACULDADE PORTO-ALEGRENSE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4992, 5318, 'FACULDADE GUILHERME GUIMBALA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4993, 5319, 'INSTITUTO DE PS-GRADUAO AURLIO BUARQUE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4994, 5320, 'ESCOLA SUPERIOR DE ENSINO ANSIO TEIXEIRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4995, 5321, 'FESMIP', 2, 1, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4996, 5322, 'FUNDAO UNIVERSIDADE FEDERAL DO PAMPA - UNIPAMPA', 1, 1, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4997, 5323, 'INSTITUTO DE EDUCAO SUPERIOR OLIVEIRAS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4998, 5324, 'CIODONTO PS GRADUAO SANTOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (4999, 5325, 'CIODONTO PS GRADUAO SO JOS DOS CAMPOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5000, 5326, 'CIODONTO PS GRADUAO SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5001, 5327, 'CIODONTO PS GRADUAO SO PAULO I', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5002, 5328, 'CIODONTO PS GRADUAO SERTOZINHO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5003, 5329, 'CIODONTO PS-GRADUAO BELO HORIZONTE II', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5004, 5330, 'CIODONTO PS GRADUAO ITAJA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5005, 5331, 'CIODONTO PS GRADUAO RIO DE JANEIRO III', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5006, 5332, 'CIODONTO PS GRADUAO VITRIA II', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5007, 5333, 'CIODONTO PS GRADUAO VITRIA I', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5008, 5334, 'CIODONTO PS GRADUAO ERECHIM', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5009, 5335, 'CIODONTO PS GRADUAO FEIRA DE SANTANA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5010, 5336, 'CIODONTO PS GRADUAO PORTO VELHO', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5011, 5337, 'CIODONTO PS GRADUAO PORTO ALEGRE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5012, 5338, 'CIODONTO PS GRADUAO RECIFE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5013, 5339, 'CIODONTO PGRADUAO CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5014, 5340, 'CIODONTO PGRADUAO CAMPO GRANDE', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5015, 5341, 'CIODONTO PS GRADUAO MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5016, 5342, 'CIODONTO PS GRADUAO PASSO FUNDO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5017, 5343, 'CIODONTO PS GRADUAO FLORIANPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5018, 5344, 'CIODONTO PS GRADUAO GOINIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5019, 5345, 'CIODONTO PS GRADUAO GUARULHOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5020, 5346, 'CIODONTO PS GRADUAO JOINVILLE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5021, 5347, 'CIODONTO PS GRADUAO MACAP', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5022, 5348, 'CIODONTO PS GRADUAO RIO DE JANEIRO I', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5023, 5349, 'CIODONTO PS-GRADUAO BELO HORIZONTE I', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5024, 5350, 'CIODONTO PS GRADUAO CURITIBA I', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5025, 5351, 'FACULDADE DE TECNOLOGIA INSTITUTO POLITCNICO DO PARAN - UNIDADE GUARAPUAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5026, 5352, 'FACULDADE DE CINCIAS E TECNOLOGIA DE NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5027, 5353, 'FACULDADE DE TECNOLOGIA BANDEIRANTES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5028, 5354, 'FACULDADE CAL DE ARTES CNICAS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5029, 5355, 'FACULDADE DE CINCIAS E DESENVOLVIMENTO DO VALE DO CAPIBARIBE', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5030, 5356, 'FACULDADE FLUMINENSE DE ENGENHARIA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5031, 5357, 'FACULDADE DE CINCIAS APLICADAS E TECNOLGICAS DO LITORAL SUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5032, 5358, 'FACULDADE INTEGRADO DE IMPERATRIZ', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5033, 5359, 'FACULDADE PAULO SETUBAL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5034, 5360, 'FACULDADE PADRE ANCHIETA DE VRZEA PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5035, 5361, 'FACULDADE PITAGRAS DE VOTORANTIM', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5036, 5362, 'FACULDADE SERIGY', 4, 2, 'SE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5037, 5363, 'INPET - INSTITUTO NACIONAL DE PESQUISAS E ESTUDOS TRIBUTRIOS S/S LTDA', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5038, 5364, 'FACULDADE ESPECIALIZADA NA REA DE SADE DO RIO GRANDE DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5039, 5365, 'INSTITUTO TAQUARITINGUENSE DE ENSINO SUPERIOR DOUTOR ARISTIDES DE CARVALHO SCHLOBACH', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5040, 5366, 'CEOSP - CENTRO DE ESTUDOS ODONTOLGICOS SMILEPREV', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5041, 5367, 'CIODONTO PS GRADUAO JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5042, 5368, 'CIODONTO PS GRADUAO RIO DE JANEIRO II', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5043, 5369, 'FACULDADES INTEGRADAS ASMEC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5044, 5370, 'FACULDADES UNIFICADAS DOCTUM DE TEFILO OTONI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5045, 5371, 'CEFAC - POS-GRADUAO EM SADE E EDUCAO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5046, 5372, 'FACULDADE DE RONDNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5047, 5373, 'INSTITUTO NACIONAL DE ENSINO SUPERIOR E PS-GRADUAO PADRE GERVSIO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5048, 5374, 'FACULDADE BRASILEIRA DE ESTUDOS AVANADOS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5049, 5375, 'ESCOLA SUPERIOR DE AVIAO CIVIL', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5050, 5376, 'FACULDADE UNIESP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5051, 5377, 'INSTITUTO INTEGRAL DE PESQUISA E ENSINO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5052, 5378, 'INSTITUTO DE MEDICINA TROPICAL', 2, 1, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5053, 5379, 'INSTITUTO PAULISTA DE ESTUDOS SISTMICOS - IPES', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5054, 5380, 'INSTITUTO DE CRIMINOLOGIA E POLTICA CRIMINAL - ICPC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5055, 5381, 'NCD- NUTROCLINICA CURSOS A DISTNCIA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5056, 5382, 'FACULDADE DE PINDAMONHANGABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5057, 5383, 'AFIP - ASSOCIAO FUNDO DE INCENTIVO  PSICOFARMACOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5058, 5384, 'IEN - INSTITUTO SUPERIOR DE ENSINO DE NEUROPSICOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5059, 5385, 'FACULDADE DO INSTITUTO DE EDUCAO EM NEGCIOS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5060, 5386, 'FACULDADES IBDI CAMBORI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5061, 5387, 'FACULDADE IMPACTA DE TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5062, 5388, 'FACULDADES UNIVERSITAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5063, 5389, 'INSTITUTO HELIPOLIS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5064, 5390, 'FACULDADE METROPOLITANA SO CARLOS BJI', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5065, 5391, 'THUM INSTITUTO DE PS-GRADUAO E BIOPESQUISAS LTDA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5066, 5392, 'INSTITUTO DE PESQUISA E ENSINO ODONTOLOGICO LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5067, 5393, 'INSTITUTO SOBRAIMO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5068, 5394, 'INSTITUTO DE ENSINO SUPERIOR INTEGRADO-IESI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5069, 5395, 'INSTITUTO DE ENSINO SUPERIOR MICHELNGELO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5070, 5396, 'INSTITUTO SUPERIOR DA ASSOCIAO FLUMINENSE DE AMPARO AOS CEGOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5071, 5397, 'FACULDADE DE EDUCAO E CULTURA DE VILHENA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5072, 5398, 'FACULDADE SUPERIOR DE ENSINO QUALIFICADO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5073, 5399, 'FACULDADE METROPOLITANA DE EDUCAO E CULTURA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5074, 5400, 'FACULDADE FAMA DE SO LUS', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5075, 5401, 'INSTITUTO SUPERIOR DE ENSINO E PESQUISA SO RAFAEL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5076, 5402, 'FUNDAO JOAQUIM NABUCO', 1, 1, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5077, 5403, 'FACULDADES OPET', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5078, 5404, 'INSTITUTO BRASILEIRO DE THERAPIAS E ENSINO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5079, 5405, 'ARCA-INSTITUTO DE PS-GRADUAO E SERVIOS EM ODONTOLOGIA LTDA.', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5080, 5406, 'INSTITUTO ORAL ESTHETIC DE PS-GRADUAO E APERFEIOAMENTO EM ODONTOLOGIA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5081, 5407, 'SISTEMA EDUCACIONAL CORPORATIVO', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5082, 5408, 'INSTITUTO BRASILEIRO DE PS GRADUAO - IBPG', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5083, 5409, 'ASSOCIAO BRASILEIRA DE CIRURGIES DENTISTAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5084, 5410, 'ASSOCIAO BRASILEIRA DE CIRURGIES DENTISTAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5085, 5411, 'ASSOCIAO PAULISTA DE CIRURGIES-DENTISTAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5086, 5412, 'CENTRO DE DESENVOLVIMENTO CIENTIFICO EM SADE', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5087, 5413, 'FACULDADE  DOTTORI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5088, 5414, 'ASSOCIAO PAULISTA DE CIRURGIES DENTISTAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5089, 5415, 'FACULDADES UNIDAS DE PESQUISA, CINCIAS E SADE LTDA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5090, 5416, 'FACULDADE METROPOLITANA DE LAGES', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5091, 5417, 'FACULDADE LA SALLE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5092, 5418, 'FACULDADE IBMEC DISTRITO FEDERAL', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5093, 5419, 'FACULDADE EVANGLICA DE TAGUATINGA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5094, 5420, 'FACULDADE DE TECNOLOGIA IBDI BLU', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5095, 5421, 'FACULDADE DE TECNOLOGIA IBDI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5096, 5422, 'FACULDADE DE TECNOLOGIA SENAI CUIAB', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5097, 5423, 'GANEP GRUPO DE NUTRIO HUMANA LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5098, 5424, 'INSTITUTO LEGISLATIVO BRASILEIRO', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5099, 5425, 'FACULDADE DE TECNOLOGIA EM SADE CIEPH', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5100, 5426, 'IMES - INSTITUTO MINEIRO DE EDUCAO SUPERIOR', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5101, 5427, 'FCETM - FACULDADE DE CINCIAS ECONMICAS DO TRINGULO MINEIRO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5102, 5428, 'FACULDADE ILAPE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5103, 5429, 'INSTITUTO SUPERIOR DE ENSINO DE ACREDITAO E QUALIDADE EM SADE - ISAQS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5104, 5430, 'FACULDADE DE TECNOLOGIA DE MINAS GERAIS - FATEMIG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5105, 5431, 'CEOSP - CENTRO DE ENSINO ODONTOLGICO SMILE PREV LTDA', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5106, 5432, 'SOCIEDADE EDUCACIONAL RIOGRANDENSE LTDA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5107, 5433, 'INSTITUTO BRASILEIRO DA FORMAO E DO EMPREGO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5108, 5434, 'CENTRO HERMES DE EDUCAO SUPERIOR', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5109, 5435, 'FACULDADE DE TECNOLOGIA DO TRANSPORTE', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5110, 5436, 'FACULDADE DO PROFESSOR', 4, 2, 'AP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5111, 5437, 'FACULDADE PASSIONISTA DE EDUCAO DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5112, 5438, 'FACULDADE DE TECNOLOGIA DE ENSINO SUPERIOR - FATEC', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5113, 5439, 'FACULDADES INTEGRADAS DA UNIO EDUCACIONAL DO PLANALTO CENTRAL - FACIPLAC', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5114, 5440, 'FACULDADE SEAC', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5115, 5441, 'PS GRADUAO  IREI', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5116, 5442, 'FACULADE DE CINCIAS JURDICAS DE NOVA LIMA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5117, 5443, 'FACULDADE ANDREAS PALLADIO ARTES CINCIAS', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5118, 5444, 'FACULDADE DE EDUCAO, TECNOLOGIA E ADMINISTRAO DE CAARAP', 4, 2, 'MS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5119, 5445, 'FACULDADE DE TECNOLOGIA SAINT PAUL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5120, 5446, 'FACULDADE NBREGA', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5121, 5447, 'FACULDADE PARA O DESENVOLVIMENTO DO SUDESTE TOCANTINENSE', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5122, 5448, 'FACULDADE DE TECNOLOGIA FOCUS', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5123, 5449, 'FACULDADE DE TECNOLOGIA DO NORTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5124, 5450, 'FAT FUNDAO DE APOIO A TECNOLOGIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5125, 5451, 'FACULDADE ANHANGERA DE BAURU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5126, 5452, 'EVOLUIR', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5127, 5453, 'INSTITUTO DE PS-GRADUAO EM CINCIAS DA SADE IZAMAR MILIDIU DA SILVA LTDA.', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5128, 5454, 'FACULDADES GESTO BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5129, 5455, 'FACULDADE PORTO-ALEGRENSE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5130, 5456, 'INSTITUTO DE ENSINO SUPERIOR DA AMAZNIA', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5131, 5457, 'INSTITUTO SUPERIOR DE EDUCAO DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5132, 5458, 'ESCOLA DO MINISTRIO PBLICO DE MARING', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5133, 5459, 'FACULDADE DE PEDAGOGIA DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5134, 5460, 'ESCOLA DO MINISTRIO PBLICO DE CURITIBA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5135, 5461, 'ESCOLA DO MINISTRIO PBLICO DE LONDRINA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5136, 5462, 'FACULDADE DE CINCIAS DA SADE DA SERRA', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5137, 5463, 'INSTITUTO SUPERIOR DA ASSOCIAO FLUMINENSE DE AMPARO AOS CEGOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5138, 5464, 'ETHOSFARMA INSTITUTO DE ENSINO', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5139, 5465, 'CONTEMPORNEO - INSTITUTO DE PSICANLISE E TRANSDISCIPLINARIDADE', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5140, 5466, 'FACULDADE DE CINCIAS JURDICAS DE AIMORS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5141, 5467, 'INSTITUTO BENJAMIN CONSTANT', 1, 1, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5142, 5468, 'ESCOLA EXISTENCIAL OTVIO DE OLIVEIRA BRASIL', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5143, 5469, 'INEPAD - ESCOLA DE NEGOCIOS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5144, 5470, 'INSTITUTO SIGNOS DE EDUCAO S/S LTDA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5145, 5471, 'FACULDADE RIBEIRA DO POMBAL', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5146, 5472, 'PROCESSUS FACULDADE DE DIREITO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5147, 5473, 'FACULDADE DE CINCIA, TECNOLOGIA E EDUCAO', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5148, 5474, 'ESCOLA DE ULTRA-SONOGRAFIA E RECICLAGEM MDICA RIBEIRO PRETO - EURP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5149, 5475, 'INSTITUTO ANSIO TEIXEIRA', 2, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5150, 5476, 'EXCELLENCE BUSSINESS SCHOOL', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5151, 5477, 'COLGIO BRASILEIRO DE CINCIAS MDICAS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5152, 5478, 'INSTITUTO SUPERIOR DE EDUCAO IRENE DA CUNHA COSTA', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5153, 5479, 'SOCIEDADE PAULISTA DE ORTODONTIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5154, 5480, 'SEITON CURSOS', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5155, 5481, 'CLNICA DE IMAGENOLOGIA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5156, 5482, 'FEMPERJ - FUNDAO ESCOLA SUPERIOR DO MINISTRIO PBLICO DO ESTADO DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5157, 5483, 'INSTITUTO DE PESQUISAS E EXPERIMENTOS EM ODONTOLOGIA DO PIAUI LTDA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5158, 5484, 'FACULDADE DE EDUCAO E CULTURA DE JI-PARAN', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5159, 5485, 'FACULDADE DE TECNOLOGIA DIAGRAMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5160, 5486, 'FACULDADE INSPIRAR', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5161, 5487, 'FACULDADE DE TECNOLOGIA ALTO MDIO SO FRANCISCO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5162, 5488, 'FACULDADE AURLIO DA ESTNCIA TURSTICA DE EMBU', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5163, 5489, 'FACULDADE FAMA DE IMPERATRIZ', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5164, 5490, 'FACULDADE DE MEDICINA DR PAULO PRATA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5165, 5491, 'FACULDADE DO INSTITUTO ATLAS DE CINCIA E TECNOLOGIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5166, 5492, 'FACULDADE DE MEDICINA E DE ENFERMAGEM NOVA ESPERANA DE NATAL', 4, 2, 'RN', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5167, 5493, 'FACULDADE DE TECNOLOGIA SENAI PAULO DE TARSO', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5168, 5494, 'INSTITUTO JOS ELIAS TAJRA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5169, 5495, 'FACULDADE MAURCIO DE NASSAU FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5170, 5496, 'FACULDADE MAURCIO DE NASSAU CABO', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5171, 5497, 'FACULDADE DE TECNOLOGIA DO ISTITUTO EUROPEO DI DESIGN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5172, 5498, 'FACULDADE ORCULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5173, 5499, 'FACULDADE EDUCACIONAL DE FRANCISCO BELTRO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5174, 5500, 'FACULDADE ANGLO-AMERICANO DE CHAPEC', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5175, 5501, 'FACULDADES COC DE GOINIA', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5176, 5502, 'FACULDADES COC DE MACEI', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5177, 5503, 'FACULDADES COC DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5178, 5504, 'FACULDADE UNIO DE CAMPO MOURO', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5179, 5505, 'INSTITUTO HAHNEMANNIANO DO BRASIL', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5180, 5506, 'TESTE MEC', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5181, 5507, 'CIODONTO PS-GRADUAO TERESINA', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5182, 5508, 'CIODONTO PS-GRADUAO FORTALEZA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5183, 5509, 'CENTRO DE ENSINO, REABILITAO E TRATAMENTO ODONTOLGICO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5184, 5510, 'FACULDADE DE TECNOLOGIA DE ALAGOAS', 4, 2, 'AL', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5185, 5511, 'FACULDADE CAMPO REAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5186, 5512, 'INSTITUTO NACIONAL DE PESQUISAS DA AMAZNIA', 1, 1, 'AM', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5187, 5513, 'CENTRO DE ESTUDOS E APERFEIOAMENTO ORTODNTICO PROFESSOR CARLOS TELLES (CEAO-CT)', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5188, 5514, 'CIODONTO PS-GRADUAO BELM', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5189, 5515, 'INSTITUTO SUPERIOR MENDES DE ALMEIDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5190, 5516, 'INSTITUTO DE PS GRADUAO MDICA DO RIO DE JANEIRO LTDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5191, 5517, 'INSTITUTO SOCRATES GUANAES - ISG', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5192, 5518, 'FACULDADE GUARAPUAVA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5193, 5519, 'IPE PS- GRADUAO LATO SENSU', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5194, 5520, 'FACULDADE CATHEDRAL', 4, 2, 'RR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5195, 5521, 'FACULDADE DE CINCIAS E TECNOLOGIA DO VALE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5196, 5522, 'FACULDADE ANHANGUERA DE PINDAMONHANGABA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5197, 5523, 'FACULDADE DE TECNOLOGIA TECBRASIL - UNIDADE NOVO HAMBURGO', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5198, 5524, 'FACULDADE DE EDUCAO E CULTURA DE BURITIS', 4, 2, 'RO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5199, 5525, 'FACULDADE DE TECNOLOGIA LOURENO FILHO', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5200, 5526, 'FACULDADE DE TECNOLOGIA E DESENVOLVIMENTO DE COMPETNCIAS', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5201, 5527, 'FACULDADE DE GOINIA - UNIDADE 1', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5202, 5528, 'FACULDADE METROPOLITANA DE ANPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5203, 5529, 'FACULDADE SO FRANCISCO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5204, 5530, 'ESCOLA DE DIREITO DE BRASLIA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5205, 5531, 'FACULDADE INTERATIVA DE TECNOLOGIA E CINCIAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5206, 5532, 'FACULDADE DE TECNOLOGIA FINACI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5207, 5533, 'FACULDADE DE CINCIAS CONTBEIS E ADMINISTRATIVAS DE CACHOEIRO DE ITAPEMIRIM', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5208, 5534, 'DINMICO FACULDADE DE FLORIANO', 4, 2, 'PI', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5209, 5535, 'INSTITUTO UNIVERSAL DE MARKETING EM AGRIBUSINESS - UMA', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5210, 5536, 'FACULDADE DE TECNOLOGIA INED - UNIDADE VENDA NOVA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5211, 5537, 'FACULDADES VALE DO CARANGOLA - FAVALE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5212, 5538, 'GANEP - NUTRIO HUMANA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5213, 5539, 'FACULDADE DO CENTRO DO PARAN', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5214, 5540, 'FACULDADE DE MSICA SOUZA LIMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5215, 5541, 'FACULDADE DE TECNOLOGIA SENAC MINAS - UNIDADE MONTES CLAROS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5216, 5542, 'GANEP - NUTRIO HUMANA', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5217, 5543, 'GANEP - NUTRIO HUMANA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5218, 5544, 'FACULDADE PRESIDENTE ANTNIO CARLOS', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5219, 5545, 'FACULDADE IPANEMA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5220, 5546, 'FACULDADE PRINCESA DO OESTE', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5221, 5547, 'FACULDADE DE MATO GROSSO', 4, 2, 'MT', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5222, 5548, 'FACULDADE VILLA LOBOS DO CONE LESTE PAULISTA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5223, 5549, 'IPOG - INSTITUTO  DE PS-GRADUAO E GRADUAO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5224, 5550, 'FACULDADE ANHANGUERA DE ANPOLIS', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5225, 5551, 'INSTITUTO MUNICIPAL DE ENSINO SUPERIOR DE ASSIS', 3, 1, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5226, 5552, 'FACULDADE DE TECNOLOGIA IBRATEC DE JOO PESSOA', 4, 2, 'PB', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5227, 5553, 'UNIVERSIDADE HOLISTICA INTERNACIONAL', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5228, 5554, 'FUNDAO DE APOIO CEFET-RJ', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5229, 5555, 'FACULDADE ANHANGUERA DE JACARE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5230, 5556, 'FACULDADE DO ENSINO DO ESTADO DO PAR', 4, 2, 'PA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5231, 5557, 'FACULDADE DE ENFERMAGEM DO HOSPITAL ISRAELITA ALBERT EINSTEIN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5232, 5558, 'INSTITUTO DE PS-GRADUAO NIVALDO NUERNBERG', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5233, 5559, 'FACULDADE DE ENGENHARIA DE MINAS GERAIS - FEAMIG', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5234, 5560, 'CENTRO DE APRENDIZAGEM E DESENVOLVIMENTO PROFISSIONAL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5235, 5561, 'ANGIOX', 4, 2, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5236, 5562, 'FUNDAO INSTITUTO DE PESQUISAS CONTBEIS, ATUARIAIS E FINANCEIRAS - FIPECAFI', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5237, 5563, 'SINDICATO DOS ODONTOLOGISTAS DO ESTADO DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5238, 5564, 'FACULDADE DOM BOSCO DE CAPITO LENIDAS MARQUES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5239, 5565, 'FACULDADE INTEGRADO DO RIO GRANDE DO SUL', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5240, 5566, 'FACULDADE DE TECNOLOGIA NORTE FLUMINENSE', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5241, 5567, 'INSTITUTO DE ENSINO SUPERIOR FRANCISCANO', 4, 2, 'MA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5242, 5568, 'PRAETORIUM INSTITUTO DE ENSINO, PESQUISA E ATIVIDADES DE EXTENSO EM DIREITO LTDA.', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5243, 5569, 'PR RIM INSTITUTO DE EDUCAO E PESQUISA EM SADE', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5244, 5570, 'FUNDAO GIRAMUNDO', 1, 1, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5245, 5571, 'ADVANCE - CURSOS DE PS-GRADUO', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5246, 5572, 'INSTITUTO DE ENSINO PESQUISA E EXTENSO', 4, 2, 'GO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5247, 5573, 'CENTRO DE PESQUISA E QUALIFICAO TECNOLGICA', 4, 2, 'CE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5248, 5574, 'FACULDADE DE TECNOLOGIA DE IPATINGA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5249, 5575, 'FACULDADE SO MARCOS-FASAMAR', 4, 2, 'TO', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5250, 5576, 'FUNDAO ESCOLA NACIONAL DE ADMINISTRAO PBLICA', 1, 1, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5251, 5577, 'FACULDADE MAX PLANCK', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5252, 5578, 'ASSOCIAO BRASILEIRA DE NUTROLOGIA - ABRAN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5253, 5579, 'ESCOLA RIOPRETENSE DE ACUPUNTURA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5254, 5580, 'FACULDADE DE MEDICINA DE GARANHUNS', 4, 2, 'PE', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5255, 5581, 'CENTRO DE ESTUDOS DE CINCIAS ODONTOLGICAS IPIRANGA - CECOI', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5256, 5582, 'CURSOS SUI JURIS', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5257, 5583, 'FACULDADE IGUAPENSE SANTO AUGUSTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5258, 5584, 'PANAMERICANA FACULDADE DE ARTE E DESIGN', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5259, 5585, 'FACULDADE DE TECNOLOGIA SENAC MINAS - UNIDADE BELO HORIZONTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5260, 5586, 'FACULDADE DE TECNOLOGIA SENAC MINAS - UNIDADE BARBACENA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5261, 5587, 'CENTRO DE PS-GRADUAO DA FUNDAO ESCOLA DE ADMINISTRAO DA UFBA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5262, 5588, 'CENTRO DE PS-GRADUAO DA FUNDAO ESCOLA DE ADMINISTRAO DA UFBA', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5263, 5589, 'INSTITUTO CAPITAL HUMANO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5264, 5590, 'ESCOLA DE SOCIOLOGIA E POLTICA DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5265, 5591, 'FACULDADES INTEGRADAS CAMES', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5266, 5592, 'FACULDADES INTEGRADAS DO NORTE DE MINAS - FUNORTE', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5267, 5593, 'FACULDADE ADVENTISTA DE HORTOLNDIA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5268, 5594, 'FEMPTEC - FUNDAO DE EMPREENDIMENTOS, PESQ. E DESENV. INST. CIENT. E TECNOL. DO RIO DE JANEIRO', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5269, 5595, 'FACULDADE DE FILOSOFIA  DE PASSOS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5270, 5596, 'FACULDADE EVANGLICA DE SALVADOR', 4, 2, 'BA', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5271, 5597, 'INSTITUTO CHIAVENATO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5272, 5598, 'INSTITUTO DENTALIS - ENSINO E PESQUISA LTDA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5273, 5599, 'LIBERTAS - FACULDADES INTEGRADAS', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5274, 5600, 'FACULDADES INTEGRADAS SO JUDAS TADEU', 4, 2, 'RS', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5275, 5601, 'FUNDAO GORCEIX', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5276, 5602, 'ESCOLA SUPERIOR NEREU RAMOS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5277, 5603, 'HERNANDEZ & ASSOCIADOS CONSULTORIA E TREINAMENTO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5278, 5604, 'PROFIS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5279, 5605, 'CENTRO DE ODONTOLOGIA E PS GRADUAO SO DOMINGOS LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5280, 5606, 'FACULDADE DE DIREITO DAMSIO DE JESUS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5281, 5607, 'CENTRO DE ESTUDOS MONTESSORIANOS MENINO JESUS', 4, 2, 'SC', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5282, 5608, 'CEU - CENTRO DE ESPECIALIZAO UNIED', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5283, 5609, 'FUNDAO DE ESTUDOS AGRRIOS LUIZ DE QUEIROZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5284, 5610, 'UNIVERSITAS PAULO FREIRE - UNIFREIRE', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5285, 5611, 'UNISOPAULO - UNIVERSIDADES IN COMPANY DE SO PAULO', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5286, 5612, 'ASSOCIAO  INSTITUTO ANCHIETA DE ENSINO E PESQUISA', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5287, 5613, 'ASSOCIAO DE ENSINO DE CAMB', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5288, 5614, 'CAD CENTRO DE IDIOMAS E DESENVOLVIMENTO PROFISSIONAL LTDA', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5289, 5615, 'AFARP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5290, 5616, 'ASSOCIAAO PAULISTA CIRURGIOES DENTISTAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5291, 5617, 'GAPO - GRUPO DE APOIO A PESQUISA ODONTOLGICA', 4, 2, 'MG', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5292, 5618, 'ASSOCIAO DE ASSISTNCIA  CRIANA DEFICIENTE (AACD)', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5293, 5619, 'ASSOCIAO ODONTOLGICA DE RIBEIRO PRETO - AORP', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5294, 5620, 'NAZAR UNIVERSIDADE DA LUZ', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5295, 5621, 'AES SOCIAIS EM SADE BUCAL E CIDADANIA B&B - INSTITUTO B&B', 1, 1, 'ES', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5296, 5622, 'ORDEM DOS ECONOMISTAS DO BRASIL', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5297, 5623, 'ASSOCIAO DOS CIRURGIES DENTISTAS DE CAMPINAS', 4, 2, 'SP', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5298, 5624, 'FACULDADE PARANAPANEMA', 4, 2, 'PR', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5299, 5625, 'SOCIEDADE BRASILEIRA DE OFTALMOLOGIA', 4, 2, 'RJ', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5300, 5626, 'UNIVERSA ESCOLA DE GESTO', 4, 2, 'DF', 1, 'NOW()', NULL);
INSERT INTO educacenso_ies VALUES (5301, 9999999, 'INSTITUIO NO CADASTRADA', 4, 2, NULL, 1, 'NOW()', NULL);


--
-- Data for Name: falta_aluno; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: falta_componente_curricular; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: falta_geral; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: formula_media; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: nota_aluno; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: nota_componente_curricular; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: nota_componente_curricular_media; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: parecer_aluno; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: parecer_componente_curricular; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: parecer_geral; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: regra_avaliacao; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: tabela_arredondamento; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: tabela_arredondamento_valor; Type: TABLE DATA; Schema: modules; Owner: -
--



--
-- Data for Name: transporte_aluno; Type: TABLE DATA; Schema: modules; Owner: -
--



SET search_path = pmiacoes, pg_catalog;

--
-- Data for Name: acao_governo; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: acao_governo_arquivo; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: acao_governo_categoria; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: acao_governo_foto; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: acao_governo_foto_portal; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: acao_governo_noticia; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: acao_governo_setor; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: categoria; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



--
-- Data for Name: secretaria_responsavel; Type: TABLE DATA; Schema: pmiacoes; Owner: -
--



SET search_path = pmicontrolesis, pg_catalog;

--
-- Data for Name: acontecimento; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: artigo; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: foto_evento; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: foto_vinc; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: itinerario; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: menu; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--

INSERT INTO menu VALUES (4624, 335, NULL, 'Super Usurio', 4, 'otopic_su_lst.php', '_self', 1, 7, 1);
INSERT INTO menu VALUES (15858, NULL, NULL, 'Cadastro', 1, NULL, '_self', 1, 16, 1);
INSERT INTO menu VALUES (15869, 609, 15858, 'Reservas', 6, 'educar_reservas_lst.php', '_self', 1, 16, 1);
INSERT INTO menu VALUES (15877, 592, 15858, 'Assunto', 14, 'educar_acervo_assunto_lst.php', '_self', 1, 16, 1);
INSERT INTO menu VALUES (15878, 598, 15858, 'Obras', 15, 'educar_acervo_lst.php', '_self', 1, 16, 1);
INSERT INTO menu VALUES (15879, 606, 15858, 'Exemplares', 16, 'educar_exemplar_lst.php', '_self', 1, 16, 1);
INSERT INTO menu VALUES (15859, NULL, NULL, 'Movimentao', 2, NULL, '_self', 1, 16, 1);
INSERT INTO menu VALUES (21122, NULL, NULL, 'Cadastros', 1, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21123, NULL, NULL, 'Servidores', 2, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21124, NULL, NULL, 'Movimentao', 3, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21125, NULL, NULL, 'Administrativo', 4, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21126, NULL, NULL, 'Relatrios', 5, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21127, NULL, NULL, 'Documentos', 6, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21128, 559, 21122, 'Instituio', 1, 'educar_instituicao_lst.php', '_self', 1, 15, 26);
INSERT INTO menu VALUES (21129, 772, 21126, 'Quadro Sinttico Alunos', 1, 'educar_relatorio_quadro_alunos_sintetico.php', '_self', 1, 15, 115);
INSERT INTO menu VALUES (21130, 635, 21123, 'Cadastrar', 1, 'educar_servidor_lst.php', '_self', 1, 15, 168);
INSERT INTO menu VALUES (21131, 554, 21125, 'Tipo de Usurio', 1, 'educar_tipo_usuario_lst.php', '_self', 1, 15, 111);
INSERT INTO menu VALUES (21132, 652, 21126, 'Relao de Alunos ANEEs', 1, 'educar_relatorio_relacao_aluno_pnee_escola.php', '_self', 1, 15, 142);
INSERT INTO menu VALUES (21133, 674, 21127, 'Boletim Escolar', 1, 'educar_relatorio_boletim.php', '_self', 1, 15, 110);
INSERT INTO menu VALUES (21134, 639, 21124, 'Reserva de Vaga', 1, 'educar_reserva_vaga_lst.php', '_self', 1, 15, 36);
INSERT INTO menu VALUES (21135, 900, 21126, 'Alunos ANEEs Instituio', 2, 'educar_relatorio_relacao_qtd_aluno_pnee_escola.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21136, 664, 21127, 'Dirio de Frequncia', 2, 'educar_relatorio_diario_classe.php', '_self', 1, 15, 122);
INSERT INTO menu VALUES (21137, 641, 21124, 'Quadro de Horrio', 2, 'educar_quadro_horario_lst.php', '_self', 1, 15, 105);
INSERT INTO menu VALUES (21138, 653, 21126, 'Demonstrativo Alunos Defasados', 2, 'educar_relatorio_demonstrativo_defasado.php', '_self', 1, 15, 82);
INSERT INTO menu VALUES (21139, 555, 21125, 'Usurios', 2, 'educar_usuario_lst.php', '_self', 1, 15, 141);
INSERT INTO menu VALUES (21140, NULL, 21122, 'Curso', 2, NULL, '_self', 1, 15, 20);
INSERT INTO menu VALUES (21141, 786, 21126, 'Demonst. Aluno Defasado Geral', 3, 'educar_relatorio_demonstrativo_defasado_geral.php', '_self', 1, 15, 84);
INSERT INTO menu VALUES (21142, 654, 21126, 'Controle Desempenho de Alunos', 3, 'educar_relatorio_controle_bim_desempenho.php', '_self', 1, 15, 111);
INSERT INTO menu VALUES (21144, 659, 21124, 'Enturmao', 3, 'educar_matriculas_turma_lst.php', '_self', 1, 15, 145);
INSERT INTO menu VALUES (21145, 670, 21127, 'Dirio de Avaliaes', 3, 'educar_relatorio_diario_classe_avaliacao.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21146, 632, 21123, 'Escolaridade', 3, 'educar_escolaridade_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21147, NULL, 21122, 'Componentes Curriculares', 3, NULL, '_self', 1, 15, 190);
INSERT INTO menu VALUES (21148, 746, 21127, 'Histrico Escolar', 4, 'educar_relatorio_historico_escolar.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21149, 661, 21126, 'Relatrio Mov, Mensal Alunos', 4, 'educar_relatorio_mov_mensal_aluno.php', '_self', 1, 15, 104);
INSERT INTO menu VALUES (21150, NULL, 21122, 'Srie', 4, NULL, '_self', 1, 15, 37);
INSERT INTO menu VALUES (21151, 633, 21123, 'Motivo de Afastamento', 4, 'educar_motivo_afastamento_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21152, 642, 21124, 'Faltas/Notas', 4, 'educar_falta_nota_aluno_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21153, 634, 21123, 'Funo', 5, 'educar_funcao_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21156, 829, 21123, 'Categoria Nveis', 5, 'educar_categoria_nivel_lst.php', '_self', 1, 15, 20);
INSERT INTO menu VALUES (21157, 587, 21122, 'Sequncia de Enturmao', 5, 'educar_sequencia_serie_lst.php', '_self', 1, 15, 145);
INSERT INTO menu VALUES (21158, NULL, 21123, 'Relatrios Servidores', 6, NULL, '_self', 1, 15, 176);
INSERT INTO menu VALUES (21159, 584, 21122, 'Mdulos', 6, 'educar_modulo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21160, 845, 21124, 'Rematrcula Automtica', 6, 'educar_avancar_mod_cad.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21161, NULL, 21122, 'Escola', 7, NULL, '_self', 1, 15, 91);
INSERT INTO menu VALUES (21162, NULL, 21122, 'Infra Estrutura', 8, NULL, '_self', 1, 15, 108);
INSERT INTO menu VALUES (21163, 684, 21126, 'Relao dos Alunos Enturmados', 8, 'educar_relatorio_alunos_enturmados.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21164, 685, 21126, 'Relao dos Alunos NEnturmados', 9, 'educar_relatorio_alunos_nao_enturmados.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21165, NULL, 21122, 'Turma', 9, NULL, '_self', 1, 15, 141);
INSERT INTO menu VALUES (21166, 696, 21126, 'Quadro Curricular', 9, 'educar_relatorio_quadro_curricular.php', '_self', 1, 15, 105);
INSERT INTO menu VALUES (21167, 807, 21126, 'Espelho de Notas', 10, 'educar_relatorio_alunos_nota_semestre.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21168, 811, 21126, 'Espelho de Notas Bimestral', 10, 'educar_relatorio_alunos_nota_semestre_disc.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21169, NULL, 21122, 'Calendrio Letivo', 10, NULL, '_self', 1, 15, 104);
INSERT INTO menu VALUES (21170, 631, 21122, 'Deficincias', 11, 'educar_deficiencia_lst.php', '_self', 1, 15, 142);
INSERT INTO menu VALUES (21171, NULL, 21122, 'Aluno', 12, NULL, '_self', 1, 15, 62);
INSERT INTO menu VALUES (21172, NULL, 21122, 'Material Didtico', 13, NULL, '_self', 1, 15, 126);
INSERT INTO menu VALUES (21173, 580, 21122, 'Tipos de Ocorrncias', 13, 'educar_tipo_ocorrencia_disciplinar_lst.php', '_self', 1, 15, 194);
INSERT INTO menu VALUES (21174, 575, 21122, 'Tipo de Transferncia', 14, 'educar_transferencia_tipo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21175, 707, 21126, 'Alunos Matriculados Sinttico', 15, 'educar_relatorio_alunos_matriculados_sintetico.php', '_self', 1, 15, 62);
INSERT INTO menu VALUES (21176, NULL, 21122, 'Tipos', 15, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21177, NULL, 21122, 'Pr-Requisito', 16, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21178, 711, 21126, 'Documentos Pendentes', 17, 'educar_relatorio_documentos_pendentes.php', '_self', 1, 15, 153);
INSERT INTO menu VALUES (21179, NULL, 21126, 'Relao de Alunos ANNEs - qtd', 22, NULL, '_self', 1, 15, 1);
INSERT INTO menu VALUES (21180, 911, 21126, 'Ficha de Rematrcula', 22, 'educar_relatorio_ficha_rematricula.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21181, 835, 21124, 'Relatrio Quadros de Horrio', 25, 'educar_relatorio_quadro_horario.php', '_self', 1, 15, 105);
INSERT INTO menu VALUES (21182, 916, 21126, 'Registro de Trans. Expedidas', 25, 'educar_relatorio_registro_transferencias.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21183, 774, 21126, 'Relatrio Alunos Idade x Sexo', 25, 'educar_relatorio_quadro_idade_sexo_serie.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21185, 918, 21126, 'Ata Resultado Final', 28, 'educar_relatorio_ata_resultado_final.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21186, 836, 21126, 'Relatrio de alunos por idade', 30, 'educar_relatorio_alunos_idade.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21187, 823, 21126, 'Resultado Final', 30, 'educar_relatorio_resultado_final.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21188, 930, 21126, 'Levantamento Alfab. e no Alf.', 30, 'educar_relatorio_alfabetizados.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21189, 830, 21126, 'Alunos Benefcios', 30, 'educar_relatorio_alunos_beneficios.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21190, 824, 21126, 'Acompanhamento Mensal', 31, 'educar_relatorio_acompanhamento_mensal.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21191, 933, 21126, 'Levantamento Turma Perodo', 31, 'educar_relatorio_levantamento_turma_periodo_aluno.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21192, 944, 21126, 'Demons. Alu. Defasados Nominal', 31, 'educar_alunos_defasados_nominal.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21193, 825, 21126, 'Ficha de Leitura Escrita', 33, 'educar_relatorio_ficha_leitura.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21194, 826, 21126, 'Acompanhamento Leitura', 35, 'educar_relatorio_acompanhamento_leitura_livros.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21195, 946, 21147, 'Listar Componentes', 0, 'module/ComponenteCurricular/index', '_self', 1, 15, 192);
INSERT INTO menu VALUES (21197, 586, 21165, 'Cadastrar', 1, 'educar_turma_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21198, 561, 21161, 'Cadastrar', 1, 'educar_escola_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21199, 620, 21169, 'Calendrios', 1, 'educar_calendario_ano_letivo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21200, 567, 21162, 'Prdios', 1, 'educar_infra_predio_lst.php', '_self', 1, 15, 147);
INSERT INTO menu VALUES (21201, 583, 21150, 'Cadastrar', 1, 'educar_serie_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21202, 566, 21140, 'Cadastrar', 1, 'educar_curso_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21203, 690, 21158, 'Relatrios Professores', 1, 'educar_relatorio_quadro_professores.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21204, 569, 21172, 'Materiais', 1, 'educar_material_didatico_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21205, 578, 21171, 'Alunos', 1, 'educar_aluno_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21206, 945, 21147, 'reas de Conhecimento', 1, 'module/AreaConhecimento/index', '_self', 1, 15, 189);
INSERT INTO menu VALUES (21207, 574, 21162, 'Comodo Prdio', 2, 'educar_infra_predio_comodo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21208, 573, 21140, 'Habilitao', 2, 'educar_habilitacao_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21209, 576, 21169, 'Motivos', 2, 'educar_calendario_dia_motivo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21210, 581, 21171, 'Benefcios', 2, 'educar_aluno_beneficio_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21211, 562, 21161, 'Localizao', 2, 'educar_escola_localizacao_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21212, 563, 21172, 'Tipo Material', 2, 'educar_material_tipo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21213, 577, 21147, 'Tipos de Dispensa', 2, 'educar_tipo_dispensa_lst.php', '_self', 1, 15, 193);
INSERT INTO menu VALUES (21214, 827, 21158, 'Professores por Disciplina', 2, 'educar_relatorio_professor_disciplina.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21215, 570, 21165, 'Tipo', 2, 'educar_turma_tipo_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21216, 585, 21150, 'Escola-Srie', 3, 'educar_escola_serie_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21217, 831, 21158, 'Servidores por Nvel', 3, 'educar_relatorio_servidor_nivel.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21218, 647, 21161, 'Rede de Ensino', 3, 'educar_escola_rede_ensino_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21219, 579, 21171, 'Religio', 3, 'educar_religiao_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21220, 568, 21140, 'Tipo de Regime', 3, 'educar_tipo_regime_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21221, 572, 21162, 'Funo Cmodo', 3, 'educar_infra_comodo_funcao_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21222, 558, 21140, 'Tipo de Ensino', 4, 'educar_tipo_ensino_lst.php', '_self', 1, 15, 124);
INSERT INTO menu VALUES (21223, 678, 21171, 'Raa', 4, 'educar_raca_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21224, 571, 21140, 'Nvel Ensino', 5, 'educar_nivel_ensino_lst.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21226, NULL, 21122, 'Regras de Avaliao', 2, NULL, '_self', 1, 15, 195);
INSERT INTO menu VALUES (21227, 947, 21226, 'Listar Regras', 0, 'module/RegraAvaliacao/index', '_self', 1, 15, 197);
INSERT INTO menu VALUES (21228, 948, 21226, 'Frmulas de Clculo de Mdia', 1, 'module/FormulaMedia/index', '_self', 1, 15, 198);
INSERT INTO menu VALUES (21229, 949, 21226, 'Tabelas de Arredondamento', 2, 'module/TabelaArredondamento/index', '_self', 1, 15, 199);
INSERT INTO menu VALUES (21230, 693, 21126, 'Registro de Matrculas', 30, 'educar_relatorio_registro_matriculas.php', '_self', 1, 15, 1);
INSERT INTO menu VALUES (21184, 917, 21126, 'Alunos em Exame', 26, 'educar_relatorio_alunos_quinta_avaliacao.php', '_self', 1, 15, 1);


--
-- Data for Name: menu_portal; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: portais; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: servicos; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: sistema; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: submenu_portal; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: telefones; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: tipo_acontecimento; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: topo_portal; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--



--
-- Data for Name: tutormenu; Type: TABLE DATA; Schema: pmicontrolesis; Owner: -
--

INSERT INTO tutormenu VALUES (7, 'i-Pauta');
INSERT INTO tutormenu VALUES (15, 'i-Educar');
INSERT INTO tutormenu VALUES (16, 'i-Educar Biblioteca');


SET search_path = pmidrh, pg_catalog;

--
-- Data for Name: diaria; Type: TABLE DATA; Schema: pmidrh; Owner: -
--



--
-- Data for Name: diaria_grupo; Type: TABLE DATA; Schema: pmidrh; Owner: -
--



--
-- Data for Name: diaria_valores; Type: TABLE DATA; Schema: pmidrh; Owner: -
--



--
-- Data for Name: setor; Type: TABLE DATA; Schema: pmidrh; Owner: -
--



SET search_path = pmieducar, pg_catalog;

--
-- Data for Name: acervo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_acervo_assunto; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_acervo_autor; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_assunto; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_autor; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_colecao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_editora; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: acervo_idioma; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: aluno; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: aluno_beneficio; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: ano_letivo_modulo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: avaliacao_desempenho; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: biblioteca; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: biblioteca_dia; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: biblioteca_feriados; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: biblioteca_usuario; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: calendario_ano_letivo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: calendario_anotacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: calendario_dia; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: calendario_dia_anotacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: calendario_dia_motivo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: categoria_nivel; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: cliente; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: cliente_suspensao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: cliente_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: cliente_tipo_cliente; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: cliente_tipo_exemplar_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: coffebreak_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: curso; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: disciplina; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: disciplina_serie; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: disciplina_topico; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: dispensa_disciplina; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_ano_letivo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_complemento; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_curso; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_localizacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_rede_ensino; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_serie; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: escola_serie_disciplina; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: exemplar; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: exemplar_emprestimo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: exemplar_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: falta_aluno; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: falta_atraso; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: falta_atraso_compensado; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: faltas; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: fonte; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: funcao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: habilitacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: habilitacao_curso; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: historico_disciplinas; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: historico_educar; Type: TABLE DATA; Schema: pmieducar; Owner: -
--

INSERT INTO historico_educar VALUES ('tipo_usuario', '(1,1,,Administrador,"",1,"NOW()",,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('tipo_usuario', '(2,1,,Escola,"",4,"NOW()",,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('tipo_usuario', '(3,1,,Biblioteca,"",8,"NOW()",,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,826,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,824,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,578,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,917,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,707,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,624,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,918,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,581,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,674,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,620,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,576,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,829,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,654,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,566,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,574,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,631,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,653,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,786,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,944,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,557,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,664,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,670,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,711,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,659,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,561,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,562,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,585,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,632,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,807,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,811,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,642,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,825,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,911,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,572,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,634,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,573,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,746,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,559,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,930,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,933,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,569,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,633,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,584,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,650,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,571,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,827,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,567,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,696,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,641,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,772,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,678,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,647,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,693,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,916,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,836,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,830,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,774,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,684,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,685,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,661,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,835,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,690,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,652,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,900,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,579,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,845,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,639,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,823,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,826,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,824,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,578,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,917,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,707,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,624,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,918,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,581,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,674,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,620,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,576,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,829,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,654,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,566,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,574,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,631,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,653,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,786,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,944,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,557,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,664,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,670,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,927,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,711,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,659,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,561,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,562,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,585,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,632,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,807,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,811,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,642,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,825,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,911,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,572,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,634,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,573,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,746,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,559,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,930,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,933,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,569,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,633,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,584,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,650,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,571,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,827,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,567,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,696,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,641,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,772,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,678,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,647,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,693,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,916,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,836,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,830,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,774,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,684,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,685,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,661,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,835,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,690,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,652,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,900,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,579,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,845,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,639,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,823,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,587,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,635,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,831,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,583,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,560,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,577,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,558,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,563,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,580,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,568,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,575,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,586,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,570,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('usuario', '(1,,,1,,1,"NOW()",,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('instituicao', '(1,,1,RUA,SP,9000000,"So Paulo",Centro,"Rua Nossa Senhora da S",1,"",i-Educar,,,"NOW()",,1,i-Educar)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,625,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,592,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,594,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,591,0,1,0)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,603,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,593,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,629,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,628,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,622,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,595,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,610,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,606,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,608,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,590,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,600,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,607,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,598,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,609,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,602,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,596,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(3,597,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,826,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,824,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,578,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,917,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,707,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,624,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,918,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,581,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,674,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,620,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,576,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,829,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,946,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,654,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,566,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,574,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,631,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,653,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,786,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,944,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,664,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,670,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,927,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,711,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,659,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,561,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,562,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,585,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,632,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,807,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,811,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,642,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,825,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,911,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,572,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,634,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,948,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,573,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,746,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,559,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,930,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,933,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,569,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,633,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,584,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,571,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,827,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,567,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,696,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,641,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,772,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,678,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,647,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,693,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,916,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,947,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,836,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,830,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,774,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,684,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,685,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,661,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,835,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,690,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,652,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,900,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,579,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,845,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,639,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,823,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,587,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,635,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,831,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,583,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,949,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,577,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,558,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,563,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,580,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,568,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,575,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,586,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,570,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,945,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,625,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,592,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,594,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,591,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,603,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,593,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,629,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,628,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,622,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,595,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,610,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,606,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,608,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,590,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,600,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,607,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,598,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,609,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,602,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,596,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(1,597,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,946,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,948,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,947,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,949,1,1,1)', 'NOW()', 1);
INSERT INTO historico_educar VALUES ('menu_tipo_usuario', '(2,945,1,1,1)', 'NOW()', 1);


--
-- Data for Name: historico_escolar; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: infra_comodo_funcao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: infra_predio; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: infra_predio_comodo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: instituicao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--

INSERT INTO instituicao VALUES (1, NULL, 1, 'RUA', 'SP', 9000000, 'So Paulo', 'Centro', 'Rua Nossa Senhora da S', 1, '', 'i-Educar', NULL, NULL, 'NOW()', NULL, 1, 'i-Educar');


--
-- Data for Name: material_didatico; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: material_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: matricula; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: matricula_excessao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: matricula_ocorrencia_disciplinar; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: matricula_turma; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: menu_tipo_usuario; Type: TABLE DATA; Schema: pmieducar; Owner: -
--

INSERT INTO menu_tipo_usuario VALUES (2, 826, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 824, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 578, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 917, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 707, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 624, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 918, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 581, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 674, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 620, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 576, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 829, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 654, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 566, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 574, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 631, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 653, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 786, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 944, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 664, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 670, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 711, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 659, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 561, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 562, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 585, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 632, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 807, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 811, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 642, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 825, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 911, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 572, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 634, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 573, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 746, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 559, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 930, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 933, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 569, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 633, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 584, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 571, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 827, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 567, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 696, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 641, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 772, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 678, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 647, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 693, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 916, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 836, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 830, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 774, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 684, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 685, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 661, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 835, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 690, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 652, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 900, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 579, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 845, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 639, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 823, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 587, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 635, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 831, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 583, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 577, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 558, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 563, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 580, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 568, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 575, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 586, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 570, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 625, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 592, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 594, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 591, 0, 1, 0);
INSERT INTO menu_tipo_usuario VALUES (3, 603, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 593, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 629, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 628, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 622, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 595, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 610, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 606, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 608, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 590, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 600, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 607, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 598, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 609, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 602, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 596, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (3, 597, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 826, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 824, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 578, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 917, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 707, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 624, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 918, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 581, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 674, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 620, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 576, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 829, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 946, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 654, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 566, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 574, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 631, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 653, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 786, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 944, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 664, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 670, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 711, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 659, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 561, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 562, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 585, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 632, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 807, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 811, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 642, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 825, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 911, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 572, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 634, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 948, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 573, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 746, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 559, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 930, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 933, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 569, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 633, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 584, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 571, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 827, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 567, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 696, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 641, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 772, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 678, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 647, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 693, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 916, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 947, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 836, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 830, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 774, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 684, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 685, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 661, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 835, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 690, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 652, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 900, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 579, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 845, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 639, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 823, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 587, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 635, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 831, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 583, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 949, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 577, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 558, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 563, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 580, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 568, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 575, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 586, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 570, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 945, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 625, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 592, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 594, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 591, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 603, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 593, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 629, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 628, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 622, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 595, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 610, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 606, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 608, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 590, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 600, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 607, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 598, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 609, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 602, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 596, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (1, 597, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 946, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 948, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 947, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 949, 1, 1, 1);
INSERT INTO menu_tipo_usuario VALUES (2, 945, 1, 1, 1);


--
-- Data for Name: modulo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: motivo_afastamento; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: motivo_baixa; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: motivo_suspensao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: nivel; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: nivel_ensino; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: nota_aluno; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: operador; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: pagamento_multa; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: pre_requisito; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: quadro_horario; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: quadro_horario_horarios; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: quadro_horario_horarios_aux; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: religiao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: reserva_vaga; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: reservas; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: sequencia_serie; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: serie; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: serie_pre_requisito; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_afastamento; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_alocacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_curso; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_curso_ministra; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_disciplina; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_formacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_funcao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: servidor_titulo_concurso; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: situacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: subnivel; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_avaliacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_avaliacao_valores; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_dispensa; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_ensino; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_ocorrencia_disciplinar; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_regime; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: tipo_usuario; Type: TABLE DATA; Schema: pmieducar; Owner: -
--

INSERT INTO tipo_usuario VALUES (1, 1, NULL, 'Administrador', '', 1, 'NOW()', NULL, 1);
INSERT INTO tipo_usuario VALUES (2, 1, NULL, 'Escola', '', 4, 'NOW()', NULL, 1);
INSERT INTO tipo_usuario VALUES (3, 1, NULL, 'Biblioteca', '', 8, 'NOW()', NULL, 1);


--
-- Data for Name: transferencia_solicitacao; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: transferencia_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: turma; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: turma_dia_semana; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: turma_modulo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: turma_tipo; Type: TABLE DATA; Schema: pmieducar; Owner: -
--



--
-- Data for Name: usuario; Type: TABLE DATA; Schema: pmieducar; Owner: -
--

INSERT INTO usuario VALUES (1, NULL, NULL, 1, NULL, 1, 'NOW()', NULL, 1);


SET search_path = pmiotopic, pg_catalog;

--
-- Data for Name: funcionario_su; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: grupomoderador; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: grupopessoa; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: grupos; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: notas; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: participante; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: reuniao; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: topico; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



--
-- Data for Name: topicoreuniao; Type: TABLE DATA; Schema: pmiotopic; Owner: -
--



SET search_path = portal, pg_catalog;

--
-- Data for Name: acesso; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: agenda; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: agenda_compromisso; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: agenda_pref; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: agenda_responsavel; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_editais_editais; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_editais_editais_empresas; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_editais_empresa; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_final_pregao; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_funcionarios; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_licitacoes; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_modalidade; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_pregao_execucao; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: compras_prestacao_contas; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: foto_portal; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: foto_secao; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: funcionario; Type: TABLE DATA; Schema: portal; Owner: -
--

INSERT INTO funcionario VALUES (1, 'admin', '21232f297a57a5a743894a0e4a801fc3', 1, NULL, NULL, '1  ', '1', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL, 0, 1, '127.0.0.1', 'NOW()');


--
-- Data for Name: funcionario_vinculo; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: imagem; Type: TABLE DATA; Schema: portal; Owner: -
--

INSERT INTO imagem VALUES (168, 6, 'logo_itajai.jpg', NULL, 'jpg', NULL, NULL, 'NOW()', 1, 'NOW()', 1);
INSERT INTO imagem VALUES (34, 1, 'letra_p.gif', 'letra_p', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (35, 1, 'letra_q.gif', 'letra_q', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (36, 1, 'letra_r.gif', 'letra_r', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (37, 1, 'letra_s.gif', 'letra_s', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (38, 1, 'letra_t.gif', 'letra_t', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (39, 1, 'letra_v.gif', 'letra_v', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (40, 1, 'letra_x.gif', 'letra_x', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (41, 1, 'letra_w.gif', 'letra_w', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (42, 1, 'letra_y.gif', 'letra_y', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (43, 1, 'letra_z.gif', 'letra_z', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (44, 1, 'letra_u.gif', 'letra_u', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (88, 1, 'letra_j.gif', 'letra_j', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (89, 1, 'letra_l.gif', 'letra_l', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (1, 1, 'transp.gif', 'Transp', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (48, 1, 'email.gif', 'contato', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (53, 1, 'informacoes.gif', 'informacoes', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (56, 1, 'visualizar.gif', 'visualizar', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (57, 1, 'novo.gif', 'novo', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (59, 1, 'lista.gif', 'lista', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (73, 1, 'baixa.gif', 'baixaa', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (75, 1, 'carrinho.gif', 'compras', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (83, 1, 'retorno.gif', 'retorno', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (85, 1, 'saida2.gif', 'saida', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (86, 1, 'entrada2.gif', 'entrada', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (87, 1, 'seta_cima.gif', 'seta up', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (66, 1, 'seta_left.gif', 'seta_left', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (67, 1, 'seta_right.gif', 'seta_right', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (68, 1, 'seta_up.gif', 'seta_up', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (78, 1, 'retorno.gif', 'devolucao', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (65, 1, 'seta_down.gif', 'seta_down', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (81, 1, 'excluirrr.gif', 'excluirrr', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (47, 1, 'check.gif', 'check', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (90, 1, 'incluir.gif', 'incluir', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (92, 1, 'interrogacao.gif', 'interrogacao', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (93, 1, 'exclamacao2.gif', 'exclamacao', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (102, 1, 'computador.gif', 'computador', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (103, 1, 'duvida.gif', 'duvida', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (144, 1, 'libera.gif', 'libera', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (145, 1, 'libera2.gif', 'libera', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (128, 1, 'lixeira.gif', 'lixeira', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (58, 1, 'opcoes.gif', 'opcoes', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (141, 1, 'candidatos.gif', 'candidatos', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (153, 1, 'calculadora.gif', 'calculadora', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (154, 1, 'cifrao.gif', 'cifrao', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (61, 1, 'pasta.gif', 'pasta', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (55, 1, 'cadeado.gif', 'cadeado', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (127, 1, 'vaievolta.gif', 'vaievolta', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (155, 1, 'familia.gif', 'familia', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (156, 1, 'back.png', 'back', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (104, 1, 'adenda.gif', 'agenda', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (105, 1, 'relogio.gif', 'relogio', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (106, 1, 'ark.gif', 'caixa', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (107, 1, 'download_manager.gif', 'download', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (108, 1, 'file-manager.gif', 'arquivo', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (109, 1, 'filetypes.gif', 'balao', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (110, 1, 'kate.gif', 'escrito', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (112, 1, 'kfm_home.gif', 'sainhaa', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (113, 1, 'ktip.gif', 'computerr', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (114, 1, 'mycomputer.gif', 'computer', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (115, 1, 'package_application.gif', 'lista', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (116, 1, 'package_settings.gif', 'ferramentas', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (117, 1, 'password.gif', 'senha', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (118, 1, 'personal.gif', 'pessoas', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (119, 1, 'window_list.gif', 'lista', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (120, 1, 'yast_partitioner.gif', 'grafico', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (121, 1, 'rede.gif', 'rede', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (122, 1, 'lista.gif', 'lista', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (123, 1, 'baixarr.gif', 'baixar', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (124, 1, 'escrever.gif', 'escrever', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (125, 1, 'identify.gif', 'identidade', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (126, 1, 'importante.gif', 'importante', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (111, 1, 'kdmconfig.gif', 'usuarios', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (137, 1, 'personal2.gif', 'pessoa', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (138, 1, 'email.gif', 'email', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (139, 1, 'email2.gif', 'email', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (140, 1, 'email3.gif', 'email', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (142, 1, 'deficientes.gif', 'deficientes', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (143, 1, 'areas.gif', 'areas', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (146, 1, 'encaminhamento.gif', 'encaminhamento', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (147, 1, 'empresa.gif', 'empresa', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (148, 1, 'supergrupo.gif', 'supergrupo', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (17, 1, 'letra_a.gif', 'letra_a', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (91, 1, 'endereco.gif', 'endereco', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (19, 1, 'letra_b.gif', 'letra_b', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (20, 1, 'letra_c.gif', 'letra_c', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (21, 1, 'letra_d.gif', 'letra_d', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (22, 1, 'letra_e.gif', 'letra_e', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (23, 1, 'letra_f.gif', 'letra_f', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (24, 1, 'letra_g.gif', 'letra_g', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (25, 1, 'letra_h.gif', 'letra_h', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (26, 1, 'letra_i.gif', 'letra_i', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (29, 1, 'letra_k.gif', 'letra_k', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (31, 1, 'letra_m.gif', 'letra_m', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (32, 1, 'letra_n.gif', 'letra_n', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (33, 1, 'letra_o.gif', 'letra_o', 'gif', NULL, NULL, 'NOW()', 1, 'NOW()', NULL);
INSERT INTO imagem VALUES (50, 1, 'grafico.gif', 'grafico', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (176, 1, 'escreve.gif', 'escrever', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (177, 1, 'livro.gif', 'livro', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (178, 1, 'sineta.gif', 'sineta', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (179, 1, 'check2.gif', 'check2', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (180, 1, 'geral.gif', 'Geral', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (62, 1, 'pessoa.gif', 'pessoa', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (63, 1, 'print.gif', 'impressao', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (70, 1, 'seta_baixo.gif', 'seta2_down', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (74, 1, 'check.gif', 'check', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (80, 1, 'entrada_rel.gif', 'entrada_rel', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (82, 1, 'grafico2.gif', 'grafico', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (84, 1, 'saida_rel.gif', 'saida_rel', '   ', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (181, 1, 'carp.jpg', 'carp', 'jpg', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (182, 1, 'money.gif', 'money', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (183, 1, 'reload.jpg', 'reload', 'jpg', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (185, 1, 'porca2.jpg', 'porca2', 'jpg', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (186, 1, 'agua.gif', 'agua', 'gif', NULL, NULL, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (187, 1, 'book.png', 'FAMFAMFAM iconset: book.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (188, 1, 'book_add.png', 'FAMFAMFAM iconset: book_add.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (189, 1, 'book_go.png', 'FAMFAMFAM iconset: book_go.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (190, 1, 'layout.png', 'FAMFAMFAM iconset: layout.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (191, 1, 'layout_add.png', 'FAMFAMFAM iconset: layout_add.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (192, 1, 'layout_content.png', 'FAMFAMFAM iconset: layout_content.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (193, 1, 'book_previous.png', 'FAMFAMFAM iconset: book_previous.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (194, 1, 'book_error.png', 'FAMFAMFAM iconset: book_error.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (195, 1, 'cog.png', 'FAMFAMFAM iconset: cog.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (196, 1, 'cog_add.png', 'FAMFAMFAM iconset: cog_add.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (197, 1, 'cog_go.png', 'FAMFAMFAM iconset: cog_go.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (198, 1, 'calculator.png', 'FAMFAMFAM iconset: calculator.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);
INSERT INTO imagem VALUES (199, 1, 'calendar_link.png', 'FAMFAMFAM iconset: calendar_link.png', 'png', 16, 16, 'NOW()', 1, NULL, NULL);


--
-- Data for Name: imagem_tipo; Type: TABLE DATA; Schema: portal; Owner: -
--

INSERT INTO imagem_tipo VALUES (1, 'Icone');
INSERT INTO imagem_tipo VALUES (6, 'Acervo');


--
-- Data for Name: intranet_segur_permissao_negada; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: jor_arquivo; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: jor_edicao; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: mailling_email; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: mailling_email_conteudo; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: mailling_fila_envio; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: mailling_grupo; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: mailling_grupo_email; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: mailling_historico; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: menu_funcionario; Type: TABLE DATA; Schema: portal; Owner: -
--

INSERT INTO menu_funcionario VALUES (1, 0, 0, 0);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 104);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 109);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 114);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 115);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 116);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 135);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 136);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 137);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 138);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 154);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 158);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 159);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 170);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 190);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 209);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 239);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 240);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 241);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 244);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 26);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 27);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 29);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 293);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 294);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 295);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 296);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 297);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 299);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 332);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 333);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 335);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 336);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 337);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 34);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 341);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 343);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 345);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 35);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 36);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 375);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 41);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 43);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 445);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 459);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 473);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 551);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 552);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 553);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 588);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 604);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 605);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 611);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 612);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 613);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 648);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 666);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 669);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 694);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 741);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 753);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 754);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 755);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 756);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 757);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 758);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 773);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 79);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 85);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 86);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 87);
INSERT INTO menu_funcionario VALUES (1, 0, 0, 89);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 554);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 555);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 826);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 824);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 578);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 917);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 707);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 624);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 918);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 581);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 674);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 620);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 576);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 829);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 946);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 654);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 566);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 574);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 631);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 653);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 786);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 944);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 664);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 670);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 711);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 659);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 561);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 562);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 585);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 632);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 807);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 811);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 642);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 825);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 911);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 572);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 634);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 948);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 573);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 746);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 559);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 930);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 933);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 569);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 633);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 584);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 571);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 827);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 567);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 696);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 641);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 772);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 678);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 647);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 693);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 916);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 947);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 836);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 830);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 774);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 684);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 685);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 661);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 835);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 690);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 652);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 900);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 579);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 845);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 639);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 823);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 587);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 635);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 831);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 583);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 949);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 577);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 558);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 563);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 580);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 568);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 575);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 586);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 570);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 945);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 625);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 592);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 594);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 591);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 603);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 593);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 629);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 628);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 622);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 595);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 610);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 606);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 608);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 590);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 600);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 607);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 598);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 609);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 602);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 596);
INSERT INTO menu_funcionario VALUES (1, 1, 1, 597);


--
-- Data for Name: menu_menu; Type: TABLE DATA; Schema: portal; Owner: -
--

INSERT INTO menu_menu VALUES (1, 'Todos', '', NULL);
INSERT INTO menu_menu VALUES (4, 'Menu', 'Menus', NULL);
INSERT INTO menu_menu VALUES (5, 'Agenda', 'Agendas', NULL);
INSERT INTO menu_menu VALUES (12, 'Administrao', 'Sistemas da Administrao', NULL);
INSERT INTO menu_menu VALUES (16, 'Controle Intranet', '', NULL);
INSERT INTO menu_menu VALUES (25, 'DRH', 'Sistemas do RH', NULL);
INSERT INTO menu_menu VALUES (7, 'Pessoa F/J', 'Pessoas Fsicas e Jurdicas', NULL);
INSERT INTO menu_menu VALUES (23, 'Principal', '', NULL);
INSERT INTO menu_menu VALUES (67, 'Controle Portal', '', NULL);
INSERT INTO menu_menu VALUES (6, 'Jornalistico', 'Menus de Jornalismo', NULL);
INSERT INTO menu_menu VALUES (54, 'Aes Governo', 'Aes Governo', NULL);
INSERT INTO menu_menu VALUES (19, 'Compras', '', NULL);
INSERT INTO menu_menu VALUES (68, 'Endereamento', '', NULL);
INSERT INTO menu_menu VALUES (38, 'i-Pauta', '', NULL);
INSERT INTO menu_menu VALUES (55, 'i-Educar - Escola', NULL, NULL);
INSERT INTO menu_menu VALUES (56, 'i-Educar - Controle', NULL, NULL);
INSERT INTO menu_menu VALUES (57, 'i-Educar - Biblioteca', NULL, NULL);


--
-- Data for Name: menu_submenu; Type: TABLE DATA; Schema: portal; Owner: -
--

INSERT INTO menu_submenu VALUES (0, 1, 0, 'Super Usurio', '', '', 3);
INSERT INTO menu_submenu VALUES (79, 4, 2, 'Cadastro de Menus', 'menu_menu_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (35, 4, 2, 'Cadastro de Submenus', 'menu_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (445, 4, 2, 'Menu Suspenso', 'menu_suspenso_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (345, 5, 2, 'Agenda Pessoal', 'agenda.php', 'Agenda pessoal', 2);
INSERT INTO menu_submenu VALUES (343, 5, 2, 'Agenda Admin', 'agenda_admin_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (341, 5, 2, 'Agendas', 'agenda_responsavel.php', '', 2);
INSERT INTO menu_submenu VALUES (375, 12, 2, 'Cadastro de Setores', 'oprot_setor_lst.php', 'Setores', 3);
INSERT INTO menu_submenu VALUES (170, 12, 2, 'Estrutura Secretarias', 'administracao_estrutura.php', '', 3);
INSERT INTO menu_submenu VALUES (473, 16, 2, 'Banco de Imagens', 'imagem_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (89, 67, 2, 'Banners', 'banner_lst.php', 'Banners do Portal', 3);
INSERT INTO menu_submenu VALUES (244, 16, 2, 'Acesso Indevido', 'acesso_indevido_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (158, 16, 2, 'Conexes Intranet', 'conexoes_todos_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (588, 16, 2, 'Sistemas', 'controlesis_sistema_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (666, 16, 2, 'Bloqueio de senha - intranet', 'portal_acesso_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (36, 25, 2, 'Cadastro de Funcionrios', 'funcionario_lst.php', 'Clique aqui para inserir, editar ou excluir um usurio.', 3);
INSERT INTO menu_submenu VALUES (190, 25, 2, 'Vnculos', 'funcionario_vinculo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (43, 7, 2, 'Pessoa Fsica', 'atendidos_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (41, 7, 2, 'Pessoa Jurdica', 'empresas_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (155, 23, 2, 'Minha Agenda', 'agenda.php', '', 2);
INSERT INTO menu_submenu VALUES (157, 23, 2, 'Minhas Conexes', 'conexoes_lst.php', 'Minhas Conexes', 2);
INSERT INTO menu_submenu VALUES (475, 23, 2, 'Preferncias', 'opcao_menu_lst.php', '', 2);
INSERT INTO menu_submenu VALUES (293, 25, 2, 'Dirias', 'diaria_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (297, 25, 2, 'Dirias Grupo', 'diaria_grupo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (295, 25, 2, 'Dirias Valores', 'diaria_valores_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (612, 67, 2, 'Menu Portal', 'controlesis_menu_portal_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (613, 67, 2, 'Submenu Portal', 'controlesis_submenu_portal_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (614, 67, 2, 'Portal - Itinerrio', 'controlesis_itinerario_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (615, 67, 2, 'Portal - Portais', 'controlesis_portais_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (616, 67, 2, 'Portal - Servios', 'controlesis_servicos_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (694, 67, 2, 'Topo portal', 'controlesis_topo_portal_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (611, 67, 2, 'Telefones teis', 'controlesis_telefones_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (605, 6, 2, 'Acontecimentos', 'controlesis_acontecimento_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (604, 6, 2, 'Tipo Acontecimento', 'controlesis_tipo_acontecimento_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (27, 6, 2, 'Cadastro de Fotos', 'fotos_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (104, 6, 2, 'Tipo de Notcia', 'not_tipos_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (26, 6, 2, 'Cadastro de Notcias', 'noticias_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (669, 6, 2, 'Galeria de Fotos', 'fotos_galeria_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (34, 6, 2, 'Cadastro de Jornal', 'jornal_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (551, 54, 2, 'Ao Governo', 'acoes_acao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (553, 54, 2, 'Secretaria Responsvel', 'acoes_setor_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (552, 54, 2, 'Categoria', 'acoes_categoria_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (137, 19, 2, 'Compras - Funcionrios', 'compras_funcionarios_lst.php', 'Cadastro de Funcionarios', 3);
INSERT INTO menu_submenu VALUES (29, 19, 2, 'Licitaes - Cadastro', 'licitacoes_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (135, 19, 2, 'Licitaes - Equipe', 'licitacoes_funcionarios_lst.php', 'Cadastro de equipe de Licitao', 3);
INSERT INTO menu_submenu VALUES (136, 19, 2, 'Licitaes - Finalizar', 'licitacoes_finalizar_lst.php', 'Finalizar uma licitao', 3);
INSERT INTO menu_submenu VALUES (159, 19, 2, 'Licitaes - Finalizadas', 'licitacoes_finalizadas_lst.php', 'Lista de Licitaoes Finalizadas', 3);
INSERT INTO menu_submenu VALUES (138, 19, 2, 'Licitaes - Status', 'licitacoes_statusfinal_lst.php', 'Cadastro de status final de licitacao', 3);
INSERT INTO menu_submenu VALUES (154, 19, 2, 'Relatrio de Licitaes', 'licitacoes_relatorio.php', 'Relatrio de Licitaes', 3);
INSERT INTO menu_submenu VALUES (239, 19, 2, 'Licitaes - Edital', 'licitacoes_edital_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (240, 19, 2, 'Licitaes - Edital - Empresas', 'licitacoes_edital_empresa_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (241, 19, 2, 'Licitaes - Edital - Relatorio', 'licitacoes_edital_relatorio.php', '', 3);
INSERT INTO menu_submenu VALUES (773, 19, 2, 'Licitaes - Modalidade', 'portal_compras_modalidade_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (753, 68, 2, 'Pais', 'public_pais_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (754, 68, 2, 'Estado', 'public_uf_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (755, 68, 2, 'Municpio', 'public_municipio_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (756, 68, 2, 'Bairro', 'public_bairro_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (757, 68, 2, 'Logradouro', 'public_logradouro_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (758, 68, 2, 'CEP', 'urbano_cep_logradouro_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (296, 38, 2, 'Cadastro de Grupos', 'otopic_grupos_lst.php', 'Cadastro de Grupos', 3);
INSERT INTO menu_submenu VALUES (459, 38, 2, 'Apresentao', 'topic_index.php', '', 2);
INSERT INTO menu_submenu VALUES (294, 38, 2, 'Meus Grupos', 'otopic_meus_grupos_lst.php', '', 2);
INSERT INTO menu_submenu VALUES (335, 38, 2, 'Super Usurio ', 'otopic_su_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (648, 25, 2, 'Dirias Relatrio Data Secretaria', 'diaria_rel_data_secretaria.php', '', 3);
INSERT INTO menu_submenu VALUES (336, 25, 2, 'Dirias Relatrio Geral', 'diaria_relatorio.php', 'Relatrio de Dirias', 3);
INSERT INTO menu_submenu VALUES (299, 25, 2, 'Dirias Relatrio Data', 'diaria_rel_data.php', '', 3);
INSERT INTO menu_submenu VALUES (332, 25, 2, 'Dirias Relatrio Funcionrio', 'diaria_rel_funcionario.php', '', 3);
INSERT INTO menu_submenu VALUES (333, 25, 2, 'Dirias Relatrio Valor', 'diaria_rel_valor.php', '', 3);
INSERT INTO menu_submenu VALUES (337, 25, 2, 'Dirias Relatrio Secretaria', 'diaria_rel_secretaria.php', '', 3);
INSERT INTO menu_submenu VALUES (116, 6, 2, 'Graf. Mailling - Dirio', 'mailling_grafico_enviados_dia.php', 'Grfico de mailling diario', 3);
INSERT INTO menu_submenu VALUES (114, 6, 2, 'Graf. Mailling - Mensal', 'mailling_grafico_enviados_mes.php', 'Grfico de Maillings enviados', 3);
INSERT INTO menu_submenu VALUES (115, 6, 2, 'Graf. Mailling - Semanal', 'mailling_grafico_enviados_semana.php', 'Grfico de e-mails enviados por semana', 3);
INSERT INTO menu_submenu VALUES (109, 6, 2, 'Graf. Noticias - Tipo', 'noticias_grafico_tipo.php', 'Grafico de notcias por tipo', 3);
INSERT INTO menu_submenu VALUES (86, 6, 2, 'Mailling - Emails', 'mailling_email_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (87, 6, 2, 'Mailling - Enviar', 'mailling_enviar_email.php', '', 3);
INSERT INTO menu_submenu VALUES (741, 6, 2, 'Mailling - Enviar Convites', 'mailling_enviar_convite.php', '', 3);
INSERT INTO menu_submenu VALUES (85, 6, 2, 'Mailling - Grupos', 'mailling_grupos_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (209, 67, 2, 'Publicaes', 'concursos_lst.php', 'Cadastro de concursos publicos', 3);
INSERT INTO menu_submenu VALUES (811, 55, 2, 'Espelho de Notas Bimestral', 'educar_relatorio_alunos_nota_semestre_disc.php', '', 3);
INSERT INTO menu_submenu VALUES (824, 55, 2, 'Acompanhamento Mensal', 'educar_relatorio_acompanhamento_mensal.php', '', 3);
INSERT INTO menu_submenu VALUES (829, 55, 2, 'Categorias  de Nveis', 'educar_categoria_nivel_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (831, 55, 2, 'Servidores por Nvel', 'educar_relatorio_servidor_nivel.php', 'Servidores por Nvel', 3);
INSERT INTO menu_submenu VALUES (746, 55, 2, 'Histrico Escolar', 'educar_relatorio_historico_escolar.php', '', 3);
INSERT INTO menu_submenu VALUES (568, 55, 2, 'Tipo Regime', 'educar_tipo_regime_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (569, 55, 2, 'Material Didtico', 'educar_material_didatico_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (561, 55, 2, 'Escola', 'educar_escola_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (570, 55, 2, 'Turma Tipo', 'educar_turma_tipo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (558, 55, 2, 'Tipo Ensino', 'educar_tipo_ensino_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (562, 55, 2, 'Escola Localizao', 'educar_escola_localizacao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (571, 55, 2, 'Nvel Ensino', 'educar_nivel_ensino_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (563, 55, 2, 'Tipo Material Didtico', 'educar_material_tipo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (574, 55, 2, 'Cmodo Prdio', 'educar_infra_predio_comodo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (566, 55, 2, 'Curso', 'educar_curso_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (567, 55, 2, 'Prdio Escola', 'educar_infra_predio_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (576, 55, 2, 'Calendrio Dia Motivo', 'educar_calendario_dia_motivo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (573, 55, 2, 'Habilitao', 'educar_habilitacao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (572, 55, 2, 'Funo Cmodo Prdio', 'educar_infra_comodo_funcao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (575, 55, 2, 'Transferncia Tipo', 'educar_transferencia_tipo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (577, 55, 2, 'Tipo Dispensa Disciplina', 'educar_tipo_dispensa_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (578, 55, 2, 'Aluno', 'educar_aluno_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (579, 55, 2, 'Religio', 'educar_religiao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (559, 55, 2, 'Instituio', 'educar_instituicao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (580, 55, 2, 'Tipo Ocorrncia Disciplinar', 'educar_tipo_ocorrencia_disciplinar_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (581, 55, 2, 'Benefcio Aluno', 'educar_aluno_beneficio_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (583, 55, 2, 'Srie', 'educar_serie_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (631, 55, 2, 'Deficincia', 'educar_deficiencia_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (585, 55, 2, 'Escola Srie', 'educar_escola_serie_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (586, 55, 2, 'Turma', 'educar_turma_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (587, 55, 2, 'Seqncia Enturmao', 'educar_sequencia_serie_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (620, 55, 2, 'Calendrio', 'educar_calendario_ano_letivo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (624, 55, 2, 'Apresentao', 'educar_index.php', '', 2);
INSERT INTO menu_submenu VALUES (584, 55, 2, 'Mdulo', 'educar_modulo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (632, 55, 2, 'Escolaridade', 'educar_escolaridade_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (633, 55, 2, 'Motivo de Afastamento', 'educar_motivo_afastamento_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (634, 55, 2, 'Funo Servidor', 'educar_funcao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (635, 55, 2, 'Servidor', 'educar_servidor_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (639, 55, 2, 'Reserva Vaga', 'educar_reserva_vaga_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (641, 55, 2, 'Quadro de Horrio', 'educar_quadro_horario_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (642, 55, 2, 'Faltas/Notas Aluno', 'educar_falta_nota_aluno_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (647, 55, 2, 'Rede Ensino', 'educar_escola_rede_ensino_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (654, 55, 2, 'Controle Desempenho de Alunos', 'educar_relatorio_controle_bim_desempenho.php', '', 3);
INSERT INTO menu_submenu VALUES (653, 55, 2, 'Demonstrativo Alunos Defasados', 'educar_relatorio_demonstrativo_defasado.php', '', 3);
INSERT INTO menu_submenu VALUES (659, 55, 2, 'Enturmao', 'educar_matriculas_turma_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (661, 55, 2, 'Relatrio Movimentao Mensal de Alunos', 'educar_relatorio_mov_mensal_aluno.php', '', 3);
INSERT INTO menu_submenu VALUES (670, 55, 2, 'Dirio de Classe - Avaliaes', 'educar_relatorio_diario_classe_avaliacao.php', '', 3);
INSERT INTO menu_submenu VALUES (664, 55, 2, 'Dirio de Classe', 'educar_relatorio_diario_classe.php', '', 3);
INSERT INTO menu_submenu VALUES (674, 55, 2, 'Boletim Escolar', 'educar_relatorio_boletim.php', '', 3);
INSERT INTO menu_submenu VALUES (678, 55, 2, 'Raa', 'educar_raca_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (684, 55, 2, 'Relatrio de Alunos Enturmados', 'educar_relatorio_alunos_enturmados.php', '', 3);
INSERT INTO menu_submenu VALUES (685, 55, 2, 'Relatrio de Alunos No Enturmados', 'educar_relatorio_alunos_nao_enturmados.php', '', 3);
INSERT INTO menu_submenu VALUES (690, 55, 2, 'Relatrio Relao Professores', 'educar_relatorio_quadro_professores.php', '', 3);
INSERT INTO menu_submenu VALUES (711, 55, 2, 'Documentos Pendentes', 'educar_relatorio_documentos_pendentes.php', '', 3);
INSERT INTO menu_submenu VALUES (707, 55, 2, 'Alunos Matriculados - Sinttico', 'educar_relatorio_alunos_matriculados_sintetico.php', '', 3);
INSERT INTO menu_submenu VALUES (772, 55, 2, 'Quadro Sinttico Aluno', 'educar_relatorio_quadro_alunos_sintetico.php', '', 3);
INSERT INTO menu_submenu VALUES (774, 55, 2, 'Relatrio Alunos Idade x Sexo', 'educar_relatorio_quadro_idade_sexo_serie.php', '', 3);
INSERT INTO menu_submenu VALUES (786, 55, 2, 'Demonstrativo Alunos Defasados Geral', 'educar_relatorio_demonstrativo_defasado_geral.php', '', 3);
INSERT INTO menu_submenu VALUES (807, 55, 2, 'Espelho de Notas', 'educar_relatorio_alunos_nota_semestre.php', '', 3);
INSERT INTO menu_submenu VALUES (830, 55, 2, 'Relatrio Alunos Benefcios', 'educar_relatorio_alunos_beneficios.php', '', 3);
INSERT INTO menu_submenu VALUES (823, 55, 2, 'Resultado Final', 'educar_relatorio_resultado_final.php', '', 3);
INSERT INTO menu_submenu VALUES (825, 55, 2, 'Ficha de Leitura, Escrita', 'educar_relatorio_ficha_leitura.php', '', 3);
INSERT INTO menu_submenu VALUES (826, 55, 2, 'Acompanhamento Leitura', 'educar_relatorio_acompanhamento_leitura_livros.php', '', 3);
INSERT INTO menu_submenu VALUES (827, 55, 2, 'Professores por Disciplina', 'educar_relatorio_professor_disciplina.php', '', 3);
INSERT INTO menu_submenu VALUES (835, 55, 2, 'Relatrio Quadros de Horrio', 'educar_relatorio_quadro_horario.php', '', 3);
INSERT INTO menu_submenu VALUES (836, 55, 2, 'Relatorio de alunos por idade', 'educar_relatorio_alunos_idade.php', '', 3);
INSERT INTO menu_submenu VALUES (652, 55, 2, 'Relao de Alunos ANEEs', 'educar_relatorio_relacao_aluno_pnee_escola.php', '', 3);
INSERT INTO menu_submenu VALUES (845, 55, 2, 'Rematrcula Automtica', 'educar_avancar_mod_cad.php', 'Rematrcula Automtica', 3);
INSERT INTO menu_submenu VALUES (944, 55, 2, 'Demonstrativo Alunos Defasados Nominal', 'educar_alunos_defasados_nominal.php', '', 3);
INSERT INTO menu_submenu VALUES (930, 55, 2, 'Levantamento Alfab. e no Alfab.', 'educar_relatorio_alfabetizados.php', '', 3);
INSERT INTO menu_submenu VALUES (918, 55, 2, 'Ata Resultado Final', 'educar_relatorio_ata_resultado_final.php', '', 3);
INSERT INTO menu_submenu VALUES (911, 55, 2, 'Ficha de Rematrcula', 'educar_relatorio_ficha_rematricula.php', '', 3);
INSERT INTO menu_submenu VALUES (916, 55, 2, 'Registro de Transferncias Expedidas', 'educar_relatorio_registro_transferencias.php', '', 3);
INSERT INTO menu_submenu VALUES (900, 55, 2, 'Relao de Alunos ANEEs Quantidade', 'educar_relatorio_relacao_qtd_aluno_pnee_escola.php', '', 3);
INSERT INTO menu_submenu VALUES (933, 55, 2, 'Levantamento Turma Perodo', 'educar_relatorio_levantamento_turma_periodo_aluno.php', '', 3);
INSERT INTO menu_submenu VALUES (555, 56, 2, 'Usurio', 'educar_usuario_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (554, 56, 2, 'Tipo Usurio', 'educar_tipo_usuario_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (592, 57, 2, 'Assunto', 'educar_acervo_assunto_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (594, 57, 2, 'Autor', 'educar_acervo_autor_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (603, 57, 2, 'Cliente', 'educar_cliente_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (590, 57, 2, 'Idioma', 'educar_acervo_idioma_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (591, 57, 2, 'Biblioteca', 'educar_biblioteca_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (593, 57, 2, 'Coleo', 'educar_acervo_colecao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (595, 57, 2, 'Editora', 'educar_acervo_editora_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (596, 57, 2, 'Tipo de Cliente', 'educar_cliente_tipo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (597, 57, 2, 'Tipo Exemplar', 'educar_exemplar_tipo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (606, 57, 2, 'Exemplares', 'educar_exemplar_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (600, 57, 2, 'Motivo Baixa', 'educar_motivo_baixa_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (602, 57, 2, 'Situao', 'educar_situacao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (598, 57, 2, 'Obras', 'educar_acervo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (607, 57, 2, 'Motivo Suspenso', 'educar_motivo_suspensao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (608, 57, 2, 'Fonte', 'educar_fonte_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (609, 57, 2, 'Reservas', 'educar_reservas_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (622, 57, 2, 'Dvidas', 'educar_pagamento_multa_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (610, 57, 2, 'Emprstimo', 'educar_exemplar_emprestimo_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (625, 57, 2, 'Apresentao', 'educar_biblioteca_index.php', '', 2);
INSERT INTO menu_submenu VALUES (628, 57, 2, 'Devoluo', 'educar_exemplar_devolucao_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (629, 57, 2, 'Dados Biblioteca', 'educar_biblioteca_dados_lst.php', '', 3);
INSERT INTO menu_submenu VALUES (696, 55, 2, 'Quadro Curricular', 'educar_relatorio_quadro_curricular.php', '', 3);
INSERT INTO menu_submenu VALUES (693, 55, 2, 'Registro de Matrculas', 'educar_relatorio_registro_matriculas.php', '', 3);
INSERT INTO menu_submenu VALUES (945, 55, 2, 'reas de Conhecimento', 'module/AreaConhecimento/index', '', 3);
INSERT INTO menu_submenu VALUES (946, 55, 2, 'Componentes Curriculares', 'module/ComponenteCurricular/index', '', 3);
INSERT INTO menu_submenu VALUES (947, 55, 2, 'Regras de Avaliao', 'module/RegraAvaliacao/index', '', 3);
INSERT INTO menu_submenu VALUES (948, 55, 2, 'Frmula de Clculo de Mdia', 'module/FormulaMedia/index', '', 3);
INSERT INTO menu_submenu VALUES (949, 55, 2, 'Tabelas de Arredondamento de Nota', 'module/TabelaArredondamento/index', '', 3);
INSERT INTO menu_submenu VALUES (917, 55, 2, 'Alunos em Exame', 'educar_relatorio_alunos_quinta_avaliacao.php', '', 3);


--
-- Data for Name: not_portal; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: not_portal_tipo; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: not_tipo; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: not_vinc_portal; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: notificacao; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: pessoa_atividade; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: pessoa_fj; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: pessoa_fj_pessoa_atividade; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: pessoa_ramo_atividade; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: portal_banner; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: portal_concurso; Type: TABLE DATA; Schema: portal; Owner: -
--



--
-- Data for Name: sistema; Type: TABLE DATA; Schema: portal; Owner: -
--



SET search_path = public, pg_catalog;

--
-- Data for Name: bairro; Type: TABLE DATA; Schema: public; Owner: -
--



--
-- Data for Name: bairro_regiao; Type: TABLE DATA; Schema: public; Owner: -
--



--
-- Data for Name: changelog; Type: TABLE DATA; Schema: public; Owner: -
--

INSERT INTO changelog VALUES (2, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '2_popula_dados_basicos.sql');
INSERT INTO changelog VALUES (1, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '1_cria_schema_inicial.sql');
INSERT INTO changelog VALUES (3, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '3_corrige_sequences.sql');
INSERT INTO changelog VALUES (4, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '4_permissoes_usuario_tipo_biblioteca.sql');
INSERT INTO changelog VALUES (5, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '5_corrige_triggers_constrainst.sql');
INSERT INTO changelog VALUES (6, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '6_cria_schema_modules.sql');
INSERT INTO changelog VALUES (7, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '7_cria_tabelas_modulo_regra_avaliacao.sql');
INSERT INTO changelog VALUES (8, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '8_remove_campos_pmieducar_serie.sql');
INSERT INTO changelog VALUES (9, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '9_atualiza_foreign_key_de_pmieducar_escola_serie_disciplina.sql');
INSERT INTO changelog VALUES (10, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '10_remove_campos_pmieducar_curso.sql');
INSERT INTO changelog VALUES (11, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '11_adiciona_foreign_key_entre_pmieducar_serie_e_modules_regra_avaliacao.sql');
INSERT INTO changelog VALUES (12, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '12_cria_tabelas_modulo_avaliacao.sql');
INSERT INTO changelog VALUES (13, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '13_adiciona_campo_carga_horaria_pmieducar_escola_serie_disciplina_modules_componente_curricular_ano_escolar');
INSERT INTO changelog VALUES (14, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '14_adiciona_campo_etapa_modules_nota_componente_curricular_media.sql');
INSERT INTO changelog VALUES (15, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '15_atualiza_foreign_key_modules_falta_geral.sql');
INSERT INTO changelog VALUES (16, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '16_permissoes_para_modules_e_correcao_menus.sql');
INSERT INTO changelog VALUES (17, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '17_adiciona_campo_curso_componente_ano_escolar_e_atualiza_constraint_servidor_disciplina.sql');
INSERT INTO changelog VALUES (18, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '18_cria_tabelas_de_parecer_descritivo.sql');
INSERT INTO changelog VALUES (19, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '19_atualiza_nome_de_relatorio_no_menu_e_permissoes.sql');
INSERT INTO changelog VALUES (20, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '20_remove_permissao_e_menu_diario_avaliacao_temp.sql');
INSERT INTO changelog VALUES (21, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '21_cria_tabelas_modulo_calendario.sql');
INSERT INTO changelog VALUES (22, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '22_cria_primary_key_em_modules_calendario_turma.sql');
INSERT INTO changelog VALUES (23, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '23_cria_tabela_modules_componente_curricular_turma.sql');
INSERT INTO changelog VALUES (24, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '24_altera_tipo_campo_carga_horaria_pmieducar_servidor_alocacao.sql');
INSERT INTO changelog VALUES (25, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '25_adiciona_campo_zona_localizacao_tabelas_enderecamento.sql');
INSERT INTO changelog VALUES (26, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '26_cria_tabela_modules_transporte_aluno.sql');
INSERT INTO changelog VALUES (27, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '27_cria_tabela_modules_educacenso_cod_aluno_e_docente.sql');
INSERT INTO changelog VALUES (28, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '28_cria_tabelas_modules_educacenso_ies_e_curso_superior.sql');
INSERT INTO changelog VALUES (29, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '29_cria_tabela_modules_docente_licenciatura.sql');
INSERT INTO changelog VALUES (30, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '30_cria_indices_otimizacao_queries_quadro_horarios.sql');
INSERT INTO changelog VALUES (31, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '31_remove_constraint_desnecessaria_pmieducar_quadro_horario_horarios.sql');
INSERT INTO changelog VALUES (32, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '32_corrige_comentario_em_indices.sql');
INSERT INTO changelog VALUES (33, 'Main', 'NOW()', 'NOW()', 'dbdeploy', '33_atualiza_regra_avaliacao_nota_nao_mais_obrigatoria.sql');


--
-- Data for Name: logradouro; Type: TABLE DATA; Schema: public; Owner: -
--



--
-- Data for Name: logradouro_fonetico; Type: TABLE DATA; Schema: public; Owner: -
--



--
-- Data for Name: municipio; Type: TABLE DATA; Schema: public; Owner: -
--

INSERT INTO municipio VALUES (1, 'Alta Floresta D''oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2, 'Alto Alegre dos Parecis', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3, 'Alto Paraiso', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4, 'Alvorada D''oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5, 'Ariquemes', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (6, 'Buritis', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (7, 'Cabixi', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (8, 'Cacaulandia', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (9, 'Cacoal', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (10, 'Campo Novo de Rondonia', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (11, 'Candeias do Jamari', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (12, 'Castanheiras', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (13, 'Cerejeiras', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (14, 'Chupinguaia', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (15, 'Colorado do Oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (16, 'Corumbiara', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (17, 'Costa Marques', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (18, 'Cujubim', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (19, 'Espigao D''oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (20, 'Governador Jorge Teixeira', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (21, 'Guajara-mirim', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (22, 'Itapua do Oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (23, 'Jaru', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (24, 'Ji-parana', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (25, 'Machadinho D''oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (26, 'Ministro Andreazza', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (27, 'Mirante da Serra', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (28, 'Monte Negro', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (29, 'Nova Brasilandia D''oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (30, 'Nova Mamore', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (31, 'Nova Uniao', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (32, 'Novo Horizonte do Oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (33, 'Ouro Preto do Oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (34, 'Parecis', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (35, 'Pimenta Bueno', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (36, 'Pimenteiras do Oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (37, 'Porto Velho', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (38, 'Presidente Medici', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (39, 'Primavera de Rondonia', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (40, 'Rio Crespo', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (41, 'Rolim de Moura', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (42, 'Santa Luzia D''oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (43, 'Sao Felipe D Oeste', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (44, 'Sao Francisco do Guapore', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (45, 'Sao Miguel do Guapore', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (46, 'Seringueiras', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (47, 'Teixeiropolis', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (48, 'Theobroma', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (49, 'Urupa', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (50, 'Vale do Anari', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (51, 'Vale do Paraiso', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (52, 'Vilhena', 'RO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (53, 'Acrelandia', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (54, 'Assis Brasil', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (55, 'Brasileia', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (56, 'Bujari', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (57, 'Capixaba', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (58, 'Cruzeiro do Sul', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (59, 'Epitaciolandia', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (60, 'Feijo', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (61, 'Jordao', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (62, 'Mancio Lima', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (63, 'Manoel Urbano', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (64, 'Marechal Thaumaturgo', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (65, 'Placido de Castro', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (66, 'Porto Acre', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (67, 'Porto Walter', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (68, 'Rio Branco', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (69, 'Rodrigues Alves', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (70, 'Santa Rosa do Purus', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (71, 'Sena Madureira', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (72, 'Senador Guiomard', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (73, 'Tarauaca', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (74, 'Xapuri', 'AC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (75, 'Alvaraes', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (76, 'Amatura', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (77, 'Anama', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (78, 'Anori', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (79, 'Apui', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (80, 'Atalaia do Norte', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (81, 'Autazes', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (82, 'Barcelos', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (83, 'Barreirinha', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (84, 'Benjamin Constant', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (85, 'Beruri', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (86, 'Boa Vista do Ramos', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (87, 'Boca do Acre', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (88, 'Borba', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (89, 'Caapiranga', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (90, 'Canutama', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (91, 'Carauari', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (92, 'Careiro', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (93, 'Careiro da Varzea', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (94, 'Coari', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (95, 'Codajas', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (96, 'Eirunepe', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (97, 'Envira', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (98, 'Fonte Boa', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (99, 'Guajara', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (100, 'Humaita', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (101, 'Ipixuna', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (102, 'Iranduba', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (103, 'Itamarati', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (104, 'Itapiranga', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (105, 'Japura', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (106, 'Jurua', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (107, 'Jutai', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (108, 'Labrea', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (109, 'Manacapuru', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (110, 'Manaquiri', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (111, 'Manaus', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (112, 'Manicore', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (113, 'Maraa', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (114, 'Maues', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (115, 'Nhamunda', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (116, 'Nova Olinda do Norte', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (117, 'Novo Airao', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (118, 'Novo Aripuana', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (119, 'Parintins', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (120, 'Pauini', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (121, 'Presidente Figueiredo', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (122, 'Rio Preto da Eva', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (123, 'Santa Isabel do Rio Negro', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (124, 'Santo Antonio do Ica', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (125, 'Sao Gabriel da Cachoeira', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (126, 'Sao Paulo de Olivenca', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (127, 'Sao Sebastiao do Uatuma', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (128, 'Silves', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (129, 'Tabatinga', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (130, 'Tapaua', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (131, 'Tefe', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (132, 'Tonantins', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (133, 'Uarini', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (134, 'Urucara', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (135, 'Urucurituba', 'AM', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (136, 'Alto Alegre', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (137, 'Amajari', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (138, 'Boa Vista', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (139, 'Bonfim', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (140, 'Canta', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (141, 'Caracarai', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (142, 'Caroebe', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (143, 'Iracema', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (144, 'Mucajai', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (145, 'Normandia', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (146, 'Pacaraima', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (147, 'Rorainopolis', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (148, 'Sao Joao da Baliza', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (149, 'Sao Luiz', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (150, 'Uiramuta', 'RR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (151, 'Abaetetuba', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (152, 'Abel Figueiredo', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (153, 'Acara', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (154, 'Afua', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (155, 'Agua Azul do Norte', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (156, 'Alenquer', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (157, 'Almeirim', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (158, 'Altamira', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (159, 'Anajas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (160, 'Ananindeua', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (161, 'Anapu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (162, 'Augusto Correa', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (163, 'Aurora do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (164, 'Aveiro', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (165, 'Bagre', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (166, 'Baiao', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (167, 'Bannach', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (168, 'Barcarena', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (169, 'Belem', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (170, 'Belterra', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (171, 'Benevides', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (172, 'Bom Jesus do Tocantins', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (173, 'Bonito', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (174, 'Braganca', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (175, 'Brasil Novo', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (176, 'Brejo Grande do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (177, 'Breu Branco', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (178, 'Breves', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (179, 'Bujaru', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (180, 'Cachoeira do Arari', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (181, 'Cachoeira do Piria', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (182, 'Cameta', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (183, 'Canaa dos Carajas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (184, 'Capanema', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (185, 'Capitao Poco', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (186, 'Castanhal', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (187, 'Chaves', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (188, 'Colares', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (189, 'Conceicao do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (190, 'Concordia do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (191, 'Cumaru do Norte', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (192, 'Curionopolis', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (193, 'Curralinho', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (194, 'Curua', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (195, 'Curuca', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (196, 'Dom Eliseu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (197, 'Eldorado dos Carajas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (198, 'Faro', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (199, 'Floresta do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (200, 'Garrafao do Norte', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (201, 'Goianesia do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (202, 'Gurupa', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (203, 'Igarape-acu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (204, 'Igarape-miri', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (205, 'Inhangapi', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (206, 'Ipixuna do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (207, 'Irituia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (208, 'Itaituba', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (209, 'Itupiranga', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (210, 'Jacareacanga', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (211, 'Jacunda', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (212, 'Juruti', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (213, 'Limoeiro do Ajuru', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (214, 'Mae do Rio', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (215, 'Magalhaes Barata', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (216, 'Maraba', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (217, 'Maracana', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (218, 'Marapanim', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (219, 'Marituba', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (220, 'Medicilandia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (221, 'Melgaco', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (222, 'Mocajuba', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (223, 'Moju', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (224, 'Monte Alegre', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (225, 'Muana', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (226, 'Nova Esperanca do Piria', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (227, 'Nova Ipixuna', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (228, 'Nova Timboteua', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (229, 'Novo Progresso', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (230, 'Novo Repartimento', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (231, 'Obidos', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (232, 'Oeiras do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (233, 'Oriximina', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (234, 'Ourem', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (235, 'Ourilandia do Norte', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (236, 'Pacaja', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (237, 'Palestina do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (238, 'Paragominas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (239, 'Parauapebas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (240, 'Pau D''arco', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (241, 'Peixe-boi', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (242, 'Picarra', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (243, 'Placas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (244, 'Ponta de Pedras', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (245, 'Portel', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (246, 'Porto de Moz', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (247, 'Prainha', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (248, 'Primavera', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (249, 'Quatipuru', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (250, 'Redencao', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (251, 'Rio Maria', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (252, 'Rondon do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (253, 'Ruropolis', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (254, 'Salinopolis', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (255, 'Salvaterra', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (256, 'Santa Barbara do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (257, 'Santa Cruz do Arari', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (258, 'Santa Isabel do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (259, 'Santa Luzia do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (260, 'Santa Maria das Barreiras', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (261, 'Santa Maria do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (262, 'Santana do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (263, 'Santarem', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (264, 'Santarem Novo', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (265, 'Santo Antonio do Taua', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (266, 'Sao Caetano de Odivelas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (267, 'Sao Domingos do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (268, 'Sao Domingos do Capim', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (269, 'Sao Felix do Xingu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (270, 'Sao Francisco do Para', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (271, 'Sao Geraldo do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (272, 'Sao Joao da Ponta', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (273, 'Sao Joao de Pirabas', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (274, 'Sao Joao do Araguaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (275, 'Sao Miguel do Guama', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (276, 'Sao Sebastiao da Boa Vista', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (277, 'Sapucaia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (278, 'Senador Jose Porfirio', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (279, 'Soure', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (280, 'Tailandia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (281, 'Terra Alta', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (282, 'Terra Santa', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (283, 'Tome-acu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (284, 'Tracuateua', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (285, 'Trairao', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (286, 'Tucuma', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (287, 'Tucurui', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (288, 'Ulianopolis', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (289, 'Uruara', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (290, 'Vigia', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (291, 'Viseu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (292, 'Vitoria do Xingu', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (293, 'Xinguara', 'PA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (294, 'Amapa', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (295, 'Calcoene', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (296, 'Cutias', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (297, 'Ferreira Gomes', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (298, 'Itaubal', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (299, 'Laranjal do Jari', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (300, 'Macapa', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (301, 'Mazagao', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (302, 'Oiapoque', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (303, 'Pedra Branca do Amapari', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (304, 'Porto Grande', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (305, 'Pracuuba', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (306, 'Santana', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (307, 'Serra do Navio', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (308, 'Tartarugalzinho', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (309, 'Vitoria do Jari', 'AP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (310, 'Abreulandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (311, 'Aguiarnopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (312, 'Alianca do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (313, 'Almas', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (314, 'Alvorada', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (315, 'Ananas', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (316, 'Angico', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (317, 'Aparecida do Rio Negro', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (318, 'Aragominas', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (319, 'Araguacema', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (320, 'Araguacu', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (321, 'Araguaina', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (322, 'Araguana', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (323, 'Araguatins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (324, 'Arapoema', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (325, 'Arraias', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (326, 'Augustinopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (327, 'Aurora do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (328, 'Axixa do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (329, 'Babaculandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (330, 'Bandeirantes do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (331, 'Barra do Ouro', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (332, 'Barrolandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (333, 'Bernardo Sayao', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (334, 'Bom Jesus do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (335, 'Brasilandia do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (336, 'Brejinho de Nazare', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (337, 'Buriti do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (338, 'Cachoeirinha', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (339, 'Campos Lindos', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (340, 'Cariri do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (341, 'Carmolandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (342, 'Carrasco Bonito', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (343, 'Caseara', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (344, 'Centenario', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (345, 'Chapada da Natividade', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (346, 'Chapada de Areia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (347, 'Colinas do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (348, 'Colmeia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (349, 'Combinado', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (350, 'Conceicao do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (351, 'Couto de Magalhaes', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (352, 'Cristalandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (353, 'Crixas do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (354, 'Darcinopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (355, 'Dianopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (356, 'Divinopolis do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (357, 'Dois Irmaos do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (358, 'Duere', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (359, 'Esperantina', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (360, 'Fatima', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (361, 'Figueiropolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (362, 'Filadelfia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (363, 'Formoso do Araguaia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (364, 'Fortaleza do Tabocao', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (365, 'Goianorte', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (366, 'Goiatins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (367, 'Guarai', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (368, 'Gurupi', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (369, 'Ipueiras', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (370, 'Itacaja', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (371, 'Itaguatins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (372, 'Itapiratins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (373, 'Itapora do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (374, 'Jau do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (375, 'Juarina', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (376, 'Lagoa da Confusao', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (377, 'Lagoa do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (378, 'Lajeado', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (379, 'Lavandeira', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (380, 'Lizarda', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (381, 'Luzinopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (382, 'Marianopolis do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (383, 'Mateiros', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (384, 'Maurilandia do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (385, 'Miracema do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (386, 'Miranorte', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (387, 'Monte do Carmo', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (388, 'Monte Santo do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (389, 'Muricilandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (390, 'Natividade', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (391, 'Nazare', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (392, 'Nova Olinda', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (393, 'Nova Rosalandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (394, 'Novo Acordo', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (395, 'Novo Alegre', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (396, 'Novo Jardim', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (397, 'Oliveira de Fatima', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (398, 'Palmas', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (399, 'Palmeirante', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (400, 'Palmeiras do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (401, 'Palmeiropolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (402, 'Paraiso do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (403, 'Parana', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (404, 'Pau D''arco', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (405, 'Pedro Afonso', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (406, 'Peixe', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (407, 'Pequizeiro', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (408, 'Pindorama do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (409, 'Piraque', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (410, 'Pium', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (411, 'Ponte Alta do Bom Jesus', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (412, 'Ponte Alta do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (413, 'Porto Alegre do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (414, 'Porto Nacional', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (415, 'Praia Norte', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (416, 'Presidente Kennedy', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (417, 'Pugmil', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (418, 'Recursolandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (419, 'Riachinho', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (420, 'Rio da Conceicao', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (421, 'Rio dos Bois', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (422, 'Rio Sono', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (423, 'Sampaio', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (424, 'Sandolandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (425, 'Santa Fe do Araguaia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (426, 'Santa Maria do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (427, 'Santa Rita do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (428, 'Santa Rosa do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (429, 'Santa Tereza do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (430, 'Santa Terezinha do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (431, 'Sao Bento do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (432, 'Sao Felix do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (433, 'Sao Miguel do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (434, 'Sao Salvador do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (435, 'Sao Sebastiao do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (436, 'Sao Valerio da Natividade', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (437, 'Silvanopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (438, 'Sitio Novo do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (439, 'Sucupira', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (440, 'Taguatinga', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (441, 'Taipas do Tocantins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (442, 'Talisma', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (443, 'Tocantinia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (444, 'Tocantinopolis', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (445, 'Tupirama', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (446, 'Tupiratins', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (447, 'Wanderlandia', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (448, 'Xambioa', 'TO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (449, 'Acailandia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (450, 'Afonso Cunha', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (451, 'Agua Doce do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (452, 'Alcantara', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (453, 'Aldeias Altas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (454, 'Altamira do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (455, 'Alto Alegre do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (456, 'Alto Alegre do Pindare', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (457, 'Alto Parnaiba', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (458, 'Amapa do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (459, 'Amarante do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (460, 'Anajatuba', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (461, 'Anapurus', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (462, 'Apicum-acu', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (463, 'Araguana', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (464, 'Araioses', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (465, 'Arame', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (466, 'Arari', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (467, 'Axixa', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (468, 'Bacabal', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (469, 'Bacabeira', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (470, 'Bacuri', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (471, 'Bacurituba', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (472, 'Balsas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (473, 'Barao de Grajau', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (474, 'Barra do Corda', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (475, 'Barreirinhas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (476, 'Bela Vista do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (477, 'Belagua', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (478, 'Benedito Leite', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (479, 'Bequimao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (480, 'Bernardo do Mearim', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (481, 'Boa Vista do Gurupi', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (482, 'Bom Jardim', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (483, 'Bom Jesus das Selvas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (484, 'Bom Lugar', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (485, 'Brejo', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (486, 'Brejo de Areia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (487, 'Buriti', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (488, 'Buriti Bravo', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (489, 'Buriticupu', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (490, 'Buritirana', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (491, 'Cachoeira Grande', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (492, 'Cajapio', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (493, 'Cajari', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (494, 'Campestre do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (495, 'Candido Mendes', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (496, 'Cantanhede', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (497, 'Capinzal do Norte', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (498, 'Carolina', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (499, 'Carutapera', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (500, 'Caxias', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (501, 'Cedral', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (502, 'Central do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (503, 'Centro do Guilherme', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (504, 'Centro Novo do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (505, 'Chapadinha', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (506, 'Cidelandia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (507, 'Codo', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (508, 'Coelho Neto', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (509, 'Colinas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (510, 'Conceicao do Lago-acu', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (511, 'Coroata', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (512, 'Cururupu', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (513, 'Davinopolis', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (514, 'Dom Pedro', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (515, 'Duque Bacelar', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (516, 'Esperantinopolis', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (517, 'Estreito', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (518, 'Feira Nova do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (519, 'Fernando Falcao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (520, 'Formosa da Serra Negra', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (521, 'Fortaleza dos Nogueiras', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (522, 'Fortuna', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (523, 'Godofredo Viana', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (524, 'Goncalves Dias', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (525, 'Governador Archer', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (526, 'Governador Edison Lobao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (527, 'Governador Eugenio Barros', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (528, 'Governador Luiz Rocha', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (529, 'Governador Newton Bello', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (530, 'Governador Nunes Freire', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (531, 'Graca Aranha', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (532, 'Grajau', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (533, 'Guimaraes', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (534, 'Humberto de Campos', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (535, 'Icatu', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (536, 'Igarape do Meio', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (537, 'Igarape Grande', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (538, 'Imperatriz', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (539, 'Itaipava do Grajau', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (540, 'Itapecuru Mirim', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (541, 'Itinga do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (542, 'Jatoba', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (543, 'Jenipapo dos Vieiras', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (544, 'Joao Lisboa', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (545, 'Joselandia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (546, 'Junco do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (547, 'Lago da Pedra', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (548, 'Lago do Junco', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (549, 'Lago dos Rodrigues', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (550, 'Lago Verde', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (551, 'Lagoa do Mato', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (552, 'Lagoa Grande do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (553, 'Lajeado Novo', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (554, 'Lima Campos', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (555, 'Loreto', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (556, 'Luis Domingues', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (557, 'Magalhaes de Almeida', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (558, 'Maracacume', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (559, 'Maraja do Sena', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (560, 'Maranhaozinho', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (561, 'Mata Roma', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (562, 'Matinha', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (563, 'Matoes', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (564, 'Matoes do Norte', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (565, 'Milagres do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (566, 'Mirador', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (567, 'Miranda do Norte', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (568, 'Mirinzal', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (569, 'Moncao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (570, 'Montes Altos', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (571, 'Morros', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (572, 'Nina Rodrigues', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (573, 'Nova Colinas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (574, 'Nova Iorque', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (575, 'Nova Olinda do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (576, 'Olho D''agua das Cunhas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (577, 'Olinda Nova do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (578, 'Paco do Lumiar', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (579, 'Palmeirandia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (580, 'Paraibano', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5512, 'Rialma', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (581, 'Parnarama', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (582, 'Passagem Franca', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (583, 'Pastos Bons', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (584, 'Paulino Neves', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (585, 'Paulo Ramos', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (586, 'Pedreiras', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (587, 'Pedro do Rosario', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (588, 'Penalva', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (589, 'Peri Mirim', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (590, 'Peritoro', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (591, 'Pindare Mirim', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (592, 'Pinheiro', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (593, 'Pio Xii', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (594, 'Pirapemas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (595, 'Pocao de Pedras', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (596, 'Porto Franco', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (597, 'Porto Rico do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (598, 'Presidente Dutra', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (599, 'Presidente Juscelino', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (600, 'Presidente Medici', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (601, 'Presidente Sarney', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (602, 'Presidente Vargas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (603, 'Primeira Cruz', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (604, 'Raposa', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (605, 'Riachao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (606, 'Ribamar Fiquene', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (607, 'Rosario', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (608, 'Sambaiba', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (609, 'Santa Filomena do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (610, 'Santa Helena', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (611, 'Santa Ines', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (612, 'Santa Luzia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (613, 'Santa Luzia do Parua', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (614, 'Santa Quiteria do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (615, 'Santa Rita', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (616, 'Santana do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (617, 'Santo Amaro do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (618, 'Santo Antonio dos Lopes', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (619, 'Sao Benedito do Rio Preto', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (620, 'Sao Bento', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (621, 'Sao Bernardo', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (622, 'Sao Domingos do Azeitao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (623, 'Sao Domingos do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (624, 'Sao Felix de Balsas', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (625, 'Sao Francisco do Brejao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (626, 'Sao Francisco do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (627, 'Sao Joao Batista', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (628, 'Sao Joao do Caru', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (629, 'Sao Joao do Paraiso', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (630, 'Sao Joao do Soter', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (631, 'Sao Joao dos Patos', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (632, 'Sao Jose de Ribamar', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (633, 'Sao Jose dos Basilios', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (634, 'Sao Luis', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (635, 'Sao Luis Gonzaga do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (636, 'Sao Mateus do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (637, 'Sao Pedro da Agua Branca', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (638, 'Sao Pedro dos Crentes', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (639, 'Sao Raimundo das Mangabeiras', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (640, 'Sao Raimundo do Doca Bezerra', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (641, 'Sao Roberto', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (642, 'Sao Vicente Ferrer', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (643, 'Satubinha', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (644, 'Senador Alexandre Costa', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (645, 'Senador La Rocque', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (646, 'Serrano do Maranhao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (647, 'Sitio Novo', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (648, 'Sucupira do Norte', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (649, 'Sucupira do Riachao', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (650, 'Tasso Fragoso', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (651, 'Timbiras', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (652, 'Timon', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (653, 'Trizidela do Vale', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (654, 'Tufilandia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (655, 'Tuntum', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (656, 'Turiacu', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (657, 'Turilandia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (658, 'Tutoia', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (659, 'Urbano Santos', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (660, 'Vargem Grande', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (661, 'Viana', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (662, 'Vila Nova dos Martirios', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (663, 'Vitoria do Mearim', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (664, 'Vitorino Freire', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (665, 'Ze Doca', 'MA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (666, 'Acaua', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (667, 'Agricolandia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (668, 'Agua Branca', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (669, 'Alagoinha do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (670, 'Alegrete do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (671, 'Alto Longa', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (672, 'Altos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (673, 'Alvorada do Gurgueia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (674, 'Amarante', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (675, 'Angical do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (676, 'Anisio de Abreu', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (677, 'Antonio Almeida', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (678, 'Aroazes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (679, 'Aroeira do Itaim', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (680, 'Arraial', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (681, 'Assuncao do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (682, 'Avelino Lopes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (683, 'Baixa Grande do Ribeiro', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (684, 'Barra D Alcantara', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (685, 'Barras', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (686, 'Barreiras do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (687, 'Barro Duro', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (688, 'Batalha', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (689, 'Bela Vista do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (690, 'Belem do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (691, 'Beneditinos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (692, 'Bertolinia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (693, 'Betania do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (694, 'Boa Hora', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (695, 'Bocaina', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (696, 'Bom Jesus', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (697, 'Bom Principio do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (698, 'Bonfim do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (699, 'Boqueirao do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (700, 'Brasileira', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (701, 'Brejo do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (702, 'Buriti dos Lopes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (703, 'Buriti dos Montes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (704, 'Cabeceiras do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (705, 'Cajazeiras do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (706, 'Cajueiro da Praia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (707, 'Caldeirao Grande do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (708, 'Campinas do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (709, 'Campo Alegre do Fidalgo', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (710, 'Campo Grande do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (711, 'Campo Largo do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (712, 'Campo Maior', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (713, 'Canavieira', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (714, 'Canto do Buriti', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (715, 'Capitao de Campos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (716, 'Capitao Gervasio Oliveira', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (717, 'Caracol', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (718, 'Caraubas do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (719, 'Caridade do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (720, 'Castelo do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (721, 'Caxingo', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (722, 'Cocal', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (723, 'Cocal de Telha', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (724, 'Cocal dos Alves', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (725, 'Coivaras', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (726, 'Colonia do Gurgueia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (727, 'Colonia do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (728, 'Conceicao do Caninde', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (729, 'Coronel Jose Dias', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (730, 'Corrente', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (731, 'Cristalandia do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (732, 'Cristino Castro', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (733, 'Curimata', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (734, 'Currais', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (735, 'Curral Novo do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (736, 'Curralinhos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (737, 'Demerval Lobao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (738, 'Dirceu Arcoverde', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (739, 'Dom Expedito Lopes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (740, 'Dom Inocencio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (741, 'Domingos Mourao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (742, 'Elesbao Veloso', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (743, 'Eliseu Martins', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (744, 'Esperantina', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (745, 'Fartura do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (746, 'Flores do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (747, 'Floresta do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (748, 'Floriano', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (749, 'Francinopolis', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (750, 'Francisco Ayres', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (751, 'Francisco Macedo', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (752, 'Francisco Santos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (753, 'Fronteiras', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (754, 'Geminiano', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (755, 'Gilbues', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (756, 'Guadalupe', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (757, 'Guaribas', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (758, 'Hugo Napoleao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (759, 'Ilha Grande', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (760, 'Inhuma', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (761, 'Ipiranga do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (762, 'Isaias Coelho', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (763, 'Itainopolis', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (764, 'Itaueira', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (765, 'Jacobina do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (766, 'Jaicos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (767, 'Jardim do Mulato', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (768, 'Jatoba do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (769, 'Jerumenha', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (770, 'Joao Costa', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (771, 'Joaquim Pires', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (772, 'Joca Marques', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (773, 'Jose de Freitas', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (774, 'Juazeiro do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (775, 'Julio Borges', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (776, 'Jurema', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (777, 'Lagoa Alegre', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (778, 'Lagoa de Sao Francisco', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (779, 'Lagoa do Barro do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (780, 'Lagoa do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (781, 'Lagoa do Sitio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (782, 'Lagoinha do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (783, 'Landri Sales', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (784, 'Luis Correia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (785, 'Luzilandia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (786, 'Madeiro', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (787, 'Manoel Emidio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (788, 'Marcolandia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (789, 'Marcos Parente', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (790, 'Massape do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (791, 'Matias Olimpio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (792, 'Miguel Alves', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (793, 'Miguel Leao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (794, 'Milton Brandao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (795, 'Monsenhor Gil', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (796, 'Monsenhor Hipolito', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (797, 'Monte Alegre do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (798, 'Morro Cabeca No Tempo', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (799, 'Morro do Chapeu do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (800, 'Murici dos Portelas', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (801, 'Nazare do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (802, 'Nazria', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (803, 'Nossa Senhora de Nazare', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (804, 'Nossa Senhora dos Remedios', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (805, 'Nova Santa Rita', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (806, 'Novo Oriente do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (807, 'Novo Santo Antonio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (808, 'Oeiras', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (809, 'Olho D Agua do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (810, 'Padre Marcos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (811, 'Paes Landim', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (812, 'Pajeu do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (813, 'Palmeira do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (814, 'Palmeirais', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (815, 'Paqueta', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (816, 'Parnagua', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (817, 'Parnaiba', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (818, 'Passagem Franca do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (819, 'Patos do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (820, 'Pau D''arco do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (821, 'Paulistana', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (822, 'Pavussu', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (823, 'Pedro Ii', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (824, 'Pedro Laurentino', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (825, 'Picos', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (826, 'Pimenteiras', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (827, 'Pio Ix', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (828, 'Piracuruca', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (829, 'Piripiri', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (830, 'Porto', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (831, 'Porto Alegre do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (832, 'Prata do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (833, 'Queimada Nova', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (834, 'Redencao do Gurgueia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (835, 'Regeneracao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (836, 'Riacho Frio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (837, 'Ribeira do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (838, 'Ribeiro Goncalves', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (839, 'Rio Grande do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (840, 'Santa Cruz do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (841, 'Santa Cruz dos Milagres', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (842, 'Santa Filomena', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (843, 'Santa Luz', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (844, 'Santa Rosa do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (845, 'Santana do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (846, 'Santo Antonio de Lisboa', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (847, 'Santo Antonio dos Milagres', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (848, 'Santo Inacio do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (849, 'Sao Braz do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (850, 'Sao Felix do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (851, 'Sao Francisco de Assis do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (852, 'Sao Francisco do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (853, 'Sao Goncalo do Gurgueia', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (854, 'Sao Goncalo do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (855, 'Sao Joao da Canabrava', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (856, 'Sao Joao da Fronteira', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (857, 'Sao Joao da Serra', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (858, 'Sao Joao da Varjota', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (859, 'Sao Joao do Arraial', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (860, 'Sao Joao do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (861, 'Sao Jose do Divino', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (862, 'Sao Jose do Peixe', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (863, 'Sao Jose do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (864, 'Sao Juliao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (865, 'Sao Lourenco do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (866, 'Sao Luis do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (867, 'Sao Miguel da Baixa Grande', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (868, 'Sao Miguel do Fidalgo', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (869, 'Sao Miguel do Tapuio', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (870, 'Sao Pedro do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (871, 'Sao Raimundo Nonato', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (872, 'Sebastiao Barros', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (873, 'Sebastiao Leal', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (874, 'Sigefredo Pacheco', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (875, 'Simoes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (876, 'Simplicio Mendes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (877, 'Socorro do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (878, 'Sussuapara', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (879, 'Tamboril do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (880, 'Tanque do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (881, 'Teresina', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (882, 'Uniao', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (883, 'Urucui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (884, 'Valenca do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (885, 'Varzea Branca', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (886, 'Varzea Grande', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (887, 'Vera Mendes', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (888, 'Vila Nova do Piaui', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (889, 'Wall Ferraz', 'PI', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (890, 'Abaiara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (891, 'Acarape', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (892, 'Acarau', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (893, 'Acopiara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (894, 'Aiuaba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (895, 'Alcantaras', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (896, 'Altaneira', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (897, 'Alto Santo', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (898, 'Amontada', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (899, 'Antonina do Norte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (900, 'Apuiares', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (901, 'Aquiraz', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (902, 'Aracati', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (903, 'Aracoiaba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (904, 'Ararenda', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (905, 'Araripe', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (906, 'Aratuba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (907, 'Arneiroz', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (908, 'Assare', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (909, 'Aurora', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (910, 'Baixio', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (911, 'Banabuiu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (912, 'Barbalha', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (913, 'Barreira', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (914, 'Barro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (915, 'Barroquinha', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (916, 'Baturite', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (917, 'Beberibe', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (918, 'Bela Cruz', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (919, 'Boa Viagem', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (920, 'Brejo Santo', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (921, 'Camocim', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (922, 'Campos Sales', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (923, 'Caninde', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (924, 'Capistrano', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (925, 'Caridade', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (926, 'Carire', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (927, 'Caririacu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (928, 'Carius', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (929, 'Carnaubal', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (930, 'Cascavel', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (931, 'Catarina', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (932, 'Catunda', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (933, 'Caucaia', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (934, 'Cedro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (935, 'Chaval', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (936, 'Choro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (937, 'Chorozinho', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (938, 'Coreau', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (939, 'Crateus', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (940, 'Crato', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (941, 'Croata', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (942, 'Cruz', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (943, 'Deputado Irapuan Pinheiro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (944, 'Erere', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (945, 'Eusebio', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (946, 'Farias Brito', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (947, 'Forquilha', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (948, 'Fortaleza', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (949, 'Fortim', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (950, 'Frecheirinha', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (951, 'General Sampaio', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (952, 'Graca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (953, 'Granja', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (954, 'Granjeiro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (955, 'Groairas', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (956, 'Guaiuba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (957, 'Guaraciaba do Norte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (958, 'Guaramiranga', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (959, 'Hidrolandia', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (960, 'Horizonte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (961, 'Ibaretama', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (962, 'Ibiapina', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (963, 'Ibicuitinga', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (964, 'Icapui', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (965, 'Ico', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (966, 'Iguatu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (967, 'Independencia', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (968, 'Ipaporanga', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (969, 'Ipaumirim', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (970, 'Ipu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (971, 'Ipueiras', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (972, 'Iracema', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (973, 'Iraucuba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (974, 'Itaicaba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (975, 'Itaitinga', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (976, 'Itapage', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (977, 'Itapipoca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (978, 'Itapiuna', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (979, 'Itarema', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (980, 'Itatira', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (981, 'Jaguaretama', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (982, 'Jaguaribara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (983, 'Jaguaribe', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (984, 'Jaguaruana', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (985, 'Jardim', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (986, 'Jati', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (987, 'Jijoca de Jericoacoara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (988, 'Juazeiro do Norte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (989, 'Jucas', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (990, 'Lavras da Mangabeira', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (991, 'Limoeiro do Norte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (992, 'Madalena', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (993, 'Maracanau', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (994, 'Maranguape', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (995, 'Marco', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (996, 'Martinopole', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (997, 'Massape', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (998, 'Mauriti', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (999, 'Meruoca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1000, 'Milagres', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1001, 'Milha', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1002, 'Miraima', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1003, 'Missao Velha', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1004, 'Mombaca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1005, 'Monsenhor Tabosa', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1006, 'Morada Nova', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1007, 'Moraujo', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1008, 'Morrinhos', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1009, 'Mucambo', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1010, 'Mulungu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1011, 'Nova Olinda', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1012, 'Nova Russas', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1013, 'Novo Oriente', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1014, 'Ocara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1015, 'Oros', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1016, 'Pacajus', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1017, 'Pacatuba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1018, 'Pacoti', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1019, 'Pacuja', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1020, 'Palhano', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1021, 'Palmacia', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1022, 'Paracuru', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1023, 'Paraipaba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1024, 'Parambu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1025, 'Paramoti', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1026, 'Pedra Branca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1027, 'Penaforte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1028, 'Pentecoste', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1029, 'Pereiro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1030, 'Pindoretama', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1031, 'Piquet Carneiro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1032, 'Pires Ferreira', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1033, 'Poranga', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1034, 'Porteiras', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1035, 'Potengi', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1036, 'Potiretama', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1037, 'Quiterianopolis', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1038, 'Quixada', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1039, 'Quixelo', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1040, 'Quixeramobim', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1041, 'Quixere', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1042, 'Redencao', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1043, 'Reriutaba', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1044, 'Russas', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1045, 'Saboeiro', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1046, 'Salitre', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1047, 'Santa Quiteria', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1048, 'Santana do Acarau', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1049, 'Santana do Cariri', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1050, 'Sao Benedito', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1051, 'Sao Goncalo do Amarante', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1052, 'Sao Joao do Jaguaribe', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1053, 'Sao Luis do Curu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1054, 'Senador Pompeu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1055, 'Senador Sa', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1056, 'Sobral', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1057, 'Solonopole', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1058, 'Tabuleiro do Norte', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1059, 'Tamboril', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1060, 'Tarrafas', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1061, 'Taua', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1062, 'Tejucuoca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1063, 'Tiangua', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1064, 'Trairi', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1065, 'Tururu', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1066, 'Ubajara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1067, 'Umari', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1068, 'Umirim', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1069, 'Uruburetama', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1070, 'Uruoca', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1071, 'Varjota', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1072, 'Varzea Alegre', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1073, 'Vicosa do Ceara', 'CE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1074, 'Acari', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1075, 'Acu', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1076, 'Afonso Bezerra', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1077, 'Agua Nova', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1078, 'Alexandria', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1079, 'Almino Afonso', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1080, 'Alto do Rodrigues', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1081, 'Angicos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1082, 'Antonio Martins', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1083, 'Apodi', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1084, 'Areia Branca', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1085, 'Ares', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1086, 'Augusto Severo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1087, 'Baia Formosa', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1088, 'Barauna', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1089, 'Barcelona', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1090, 'Bento Fernandes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1091, 'Bodo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1092, 'Bom Jesus', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1093, 'Brejinho', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1094, 'Caicara do Norte', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1095, 'Caicara do Rio do Vento', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1096, 'Caico', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1097, 'Campo Redondo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1098, 'Canguaretama', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1099, 'Caraubas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1100, 'Carnauba dos Dantas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1101, 'Carnaubais', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1102, 'Ceara-mirim', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1103, 'Cerro Cora', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1104, 'Coronel Ezequiel', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1105, 'Coronel Joao Pessoa', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1106, 'Cruzeta', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1107, 'Currais Novos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1108, 'Doutor Severiano', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1109, 'Encanto', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1110, 'Equador', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1111, 'Espirito Santo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1112, 'Extremoz', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1113, 'Felipe Guerra', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1114, 'Fernando Pedroza', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1115, 'Florania', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1116, 'Francisco Dantas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1117, 'Frutuoso Gomes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1118, 'Galinhos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1119, 'Goianinha', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1120, 'Governador Dix-sept Rosado', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1121, 'Grossos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1122, 'Guamare', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1123, 'Ielmo Marinho', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1124, 'Ipanguacu', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1125, 'Ipueira', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1126, 'Itaja', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1127, 'Itau', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1128, 'Jacana', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1129, 'Jandaira', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1130, 'Janduis', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1131, 'Januario Cicco', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1132, 'Japi', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1133, 'Jardim de Angicos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1134, 'Jardim de Piranhas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1135, 'Jardim do Serido', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1136, 'Joao Camara', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1137, 'Joao Dias', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1138, 'Jose da Penha', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1139, 'Jucurutu', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1140, 'Jundia', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1141, 'Lagoa D''anta', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1142, 'Lagoa de Pedras', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1143, 'Lagoa de Velhos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1144, 'Lagoa Nova', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1145, 'Lagoa Salgada', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1146, 'Lajes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1147, 'Lajes Pintadas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1148, 'Lucrecia', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1149, 'Luis Gomes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1150, 'Macaiba', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1151, 'Macau', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1152, 'Major Sales', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1153, 'Marcelino Vieira', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1154, 'Martins', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1155, 'Maxaranguape', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1156, 'Messias Targino', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1157, 'Montanhas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1158, 'Monte Alegre', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1159, 'Monte das Gameleiras', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1160, 'Mossoro', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1161, 'Natal', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1162, 'Nisia Floresta', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1163, 'Nova Cruz', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1164, 'Olho-d''agua do Borges', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1165, 'Ouro Branco', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1166, 'Parana', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1167, 'Parau', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1168, 'Parazinho', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1169, 'Parelhas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1170, 'Parnamirim', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1171, 'Passa e Fica', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1172, 'Passagem', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1173, 'Patu', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1174, 'Pau dos Ferros', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1175, 'Pedra Grande', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1176, 'Pedra Preta', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1177, 'Pedro Avelino', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1178, 'Pedro Velho', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1179, 'Pendencias', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1180, 'Piloes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1181, 'Poco Branco', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1182, 'Portalegre', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1183, 'Porto do Mangue', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1184, 'Presidente Juscelino', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1185, 'Pureza', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1186, 'Rafael Fernandes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1187, 'Rafael Godeiro', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1188, 'Riacho da Cruz', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1189, 'Riacho de Santana', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1190, 'Riachuelo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1191, 'Rio do Fogo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1192, 'Rodolfo Fernandes', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1193, 'Ruy Barbosa', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1194, 'Santa Cruz', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1195, 'Santa Maria', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1196, 'Santana do Matos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1197, 'Santana do Serido', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1198, 'Santo Antonio', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1199, 'Sao Bento do Norte', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1200, 'Sao Bento do Trairi', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1201, 'Sao Fernando', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1202, 'Sao Francisco do Oeste', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1203, 'Sao Goncalo do Amarante', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1204, 'Sao Joao do Sabugi', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1205, 'Sao Jose de Mipibu', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1206, 'Sao Jose do Campestre', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1207, 'Sao Jose do Serido', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1208, 'Sao Miguel', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1209, 'Sao Miguel do Gostoso', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1210, 'Sao Paulo do Potengi', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1211, 'Sao Pedro', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1212, 'Sao Rafael', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1213, 'Sao Tome', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1214, 'Sao Vicente', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1215, 'Senador Eloi de Souza', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1216, 'Senador Georgino Avelino', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1217, 'Serra de Sao Bento', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1218, 'Serra do Mel', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1219, 'Serra Negra do Norte', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1220, 'Serrinha', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1221, 'Serrinha dos Pintos', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1222, 'Severiano Melo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1223, 'Sitio Novo', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1224, 'Taboleiro Grande', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1225, 'Taipu', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1226, 'Tangara', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1227, 'Tenente Ananias', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1228, 'Tenente Laurentino Cruz', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1229, 'Tibau', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1230, 'Tibau do Sul', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1231, 'Timbauba dos Batistas', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1232, 'Touros', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1233, 'Triunfo Potiguar', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1234, 'Umarizal', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1235, 'Upanema', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1236, 'Varzea', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1237, 'Venha-ver', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1238, 'Vera Cruz', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1239, 'Vicosa', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1240, 'Vila Flor', 'RN', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1241, 'Agua Branca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1242, 'Aguiar', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1243, 'Alagoa Grande', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1244, 'Alagoa Nova', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1245, 'Alagoinha', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1246, 'Alcantil', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1247, 'Algodao de Jandaira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1248, 'Alhandra', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1249, 'Amparo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1250, 'Aparecida', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1251, 'Aracagi', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1252, 'Arara', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1253, 'Araruna', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1254, 'Areia', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1255, 'Areia de Baraunas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1256, 'Areial', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1257, 'Aroeiras', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1258, 'Assuncao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1259, 'Baia da Traicao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1260, 'Bananeiras', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1261, 'Barauna', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1262, 'Barra de Santa Rosa', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1263, 'Barra de Santana', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1264, 'Barra de Sao Miguel', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1265, 'Bayeux', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1266, 'Belem', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1267, 'Belem do Brejo do Cruz', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1268, 'Bernardino Batista', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1269, 'Boa Ventura', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1270, 'Boa Vista', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1271, 'Bom Jesus', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1272, 'Bom Sucesso', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1273, 'Bonito de Santa Fe', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1274, 'Boqueirao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1275, 'Borborema', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1276, 'Brejo do Cruz', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1277, 'Brejo dos Santos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1278, 'Caapora', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1279, 'Cabaceiras', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1280, 'Cabedelo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1281, 'Cachoeira dos Indios', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1282, 'Cacimba de Areia', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1283, 'Cacimba de Dentro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1284, 'Cacimbas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1285, 'Caicara', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1286, 'Cajazeiras', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1287, 'Cajazeirinhas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1288, 'Caldas Brandao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1289, 'Camalau', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1290, 'Campina Grande', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1291, 'Campo de Santana', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1292, 'Capim', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1293, 'Caraubas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1294, 'Carrapateira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1295, 'Casserengue', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1296, 'Catingueira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1297, 'Catole do Rocha', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1298, 'Caturite', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1299, 'Conceicao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1300, 'Condado', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1301, 'Conde', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1302, 'Congo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1303, 'Coremas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1304, 'Coxixola', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1305, 'Cruz do Espirito Santo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1306, 'Cubati', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1307, 'Cuite', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1308, 'Cuite de Mamanguape', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1309, 'Cuitegi', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1310, 'Curral de Cima', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1311, 'Curral Velho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1312, 'Damiao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1313, 'Desterro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1314, 'Diamante', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1315, 'Dona Ines', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1316, 'Duas Estradas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1317, 'Emas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1318, 'Esperanca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1319, 'Fagundes', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1320, 'Frei Martinho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1321, 'Gado Bravo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1322, 'Guarabira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1323, 'Gurinhem', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1324, 'Gurjao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1325, 'Ibiara', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1326, 'Igaracy', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1327, 'Imaculada', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1328, 'Inga', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1329, 'Itabaiana', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1330, 'Itaporanga', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1331, 'Itapororoca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1332, 'Itatuba', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1333, 'Jacarau', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1334, 'Jerico', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1335, 'Joao Pessoa', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1336, 'Juarez Tavora', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1337, 'Juazeirinho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1338, 'Junco do Serido', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1339, 'Juripiranga', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1340, 'Juru', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1341, 'Lagoa', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1342, 'Lagoa de Dentro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1343, 'Lagoa Seca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1344, 'Lastro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1345, 'Livramento', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1346, 'Logradouro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1347, 'Lucena', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1348, 'Mae D''agua', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1349, 'Malta', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1350, 'Mamanguape', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1351, 'Manaira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1352, 'Marcacao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1353, 'Mari', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1354, 'Marizopolis', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1355, 'Massaranduba', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1356, 'Mataraca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1357, 'Matinhas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1358, 'Mato Grosso', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1359, 'Matureia', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1360, 'Mogeiro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1361, 'Montadas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1362, 'Monte Horebe', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1363, 'Monteiro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1364, 'Mulungu', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1365, 'Natuba', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1366, 'Nazarezinho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1367, 'Nova Floresta', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1368, 'Nova Olinda', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1369, 'Nova Palmeira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1370, 'Olho D''agua', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1371, 'Olivedos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1372, 'Ouro Velho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1373, 'Parari', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1374, 'Passagem', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1375, 'Patos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1376, 'Paulista', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1377, 'Pedra Branca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1378, 'Pedra Lavrada', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1379, 'Pedras de Fogo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1380, 'Pedro Regis', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1381, 'Pianco', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1382, 'Picui', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1383, 'Pilar', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1384, 'Piloes', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1385, 'Piloezinhos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1386, 'Pirpirituba', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1387, 'Pitimbu', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1388, 'Pocinhos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1389, 'Poco Dantas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1390, 'Poco de Jose de Moura', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1391, 'Pombal', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1392, 'Prata', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1393, 'Princesa Isabel', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1394, 'Puxinana', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1395, 'Queimadas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1396, 'Quixaba', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1397, 'Remigio', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1398, 'Riachao', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1399, 'Riachao do Bacamarte', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1400, 'Riachao do Poco', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1401, 'Riacho de Santo Antonio', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1402, 'Riacho dos Cavalos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1403, 'Rio Tinto', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1404, 'Salgadinho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1405, 'Salgado de Sao Felix', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1406, 'Santa Cecilia', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1407, 'Santa Cruz', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1408, 'Santa Helena', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1409, 'Santa Ines', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1410, 'Santa Luzia', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1411, 'Santa Rita', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1412, 'Santa Teresinha', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1413, 'Santana de Mangueira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1414, 'Santana dos Garrotes', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1415, 'Santarem', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1416, 'Santo Andre', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1417, 'Sao Bentinho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1418, 'Sao Bento', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1419, 'Sao Domingos de Pombal', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1420, 'Sao Domingos do Cariri', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1421, 'Sao Francisco', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1422, 'Sao Joao do Cariri', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1423, 'Sao Joao do Rio do Peixe', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1424, 'Sao Joao do Tigre', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1425, 'Sao Jose da Lagoa Tapada', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1426, 'Sao Jose de Caiana', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1427, 'Sao Jose de Espinharas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1428, 'Sao Jose de Piranhas', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1429, 'Sao Jose de Princesa', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1430, 'Sao Jose do Bonfim', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1431, 'Sao Jose do Brejo do Cruz', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1432, 'Sao Jose do Sabugi', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1433, 'Sao Jose dos Cordeiros', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1434, 'Sao Jose dos Ramos', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1435, 'Sao Mamede', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1436, 'Sao Miguel de Taipu', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1437, 'Sao Sebastiao de Lagoa de Roca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1438, 'Sao Sebastiao do Umbuzeiro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1439, 'Sao Vicente do Serido', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1440, 'Sape', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1441, 'Serra Branca', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1442, 'Serra da Raiz', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1443, 'Serra Grande', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1444, 'Serra Redonda', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1445, 'Serraria', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1446, 'Sertaozinho', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1447, 'Sobrado', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1448, 'Solanea', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1449, 'Soledade', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1450, 'Sossego', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1451, 'Sousa', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1452, 'Sume', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1453, 'Taperoa', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1454, 'Tavares', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1455, 'Teixeira', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1456, 'Tenorio', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1457, 'Triunfo', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1458, 'Uirauna', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1459, 'Umbuzeiro', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1460, 'Varzea', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1461, 'Vieiropolis', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1462, 'Vista Serrana', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1463, 'Zabele', 'PB', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1464, 'Abreu e Lima', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1465, 'Afogados da Ingazeira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1466, 'Afranio', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1467, 'Agrestina', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1468, 'Agua Preta', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1469, 'Aguas Belas', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1470, 'Alagoinha', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1471, 'Alianca', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1472, 'Altinho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1473, 'Amaraji', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1474, 'Angelim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1475, 'Aracoiaba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1476, 'Araripina', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1477, 'Arcoverde', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1478, 'Barra de Guabiraba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1479, 'Barreiros', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1480, 'Belem de Maria', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1481, 'Belem de Sao Francisco', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1482, 'Belo Jardim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1483, 'Betania', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1484, 'Bezerros', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1485, 'Bodoco', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1486, 'Bom Conselho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1487, 'Bom Jardim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1488, 'Bonito', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1489, 'Brejao', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1490, 'Brejinho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1491, 'Brejo da Madre de Deus', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1492, 'Buenos Aires', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1493, 'Buique', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1494, 'Cabo de Santo Agostinho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1495, 'Cabrobo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1496, 'Cachoeirinha', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1497, 'Caetes', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1498, 'Calcado', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1499, 'Calumbi', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1500, 'Camaragibe', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1501, 'Camocim de Sao Felix', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1502, 'Camutanga', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1503, 'Canhotinho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1504, 'Capoeiras', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1505, 'Carnaiba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1506, 'Carnaubeira da Penha', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1507, 'Carpina', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1508, 'Caruaru', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1509, 'Casinhas', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1510, 'Catende', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1511, 'Cedro', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1512, 'Cha de Alegria', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1513, 'Cha Grande', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1514, 'Condado', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1515, 'Correntes', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1516, 'Cortes', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1517, 'Cumaru', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1518, 'Cupira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1519, 'Custodia', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1520, 'Dormentes', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1521, 'Escada', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1522, 'Exu', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1523, 'Feira Nova', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1524, 'Fernando de Noronha', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1525, 'Ferreiros', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1526, 'Flores', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1527, 'Floresta', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1528, 'Frei Miguelinho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1529, 'Gameleira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1530, 'Garanhuns', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1531, 'Gloria do Goita', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1532, 'Goiana', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1533, 'Granito', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1534, 'Gravata', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1535, 'Iati', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1536, 'Ibimirim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1537, 'Ibirajuba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1538, 'Igarassu', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1539, 'Iguaraci', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1540, 'Inaja', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1541, 'Ingazeira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1542, 'Ipojuca', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1543, 'Ipubi', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1544, 'Itacuruba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1545, 'Itaiba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1546, 'Itamaraca', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1547, 'Itambe', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1548, 'Itapetim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1549, 'Itapissuma', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1550, 'Itaquitinga', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1551, 'Jaboatao dos Guararapes', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1552, 'Jaqueira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1553, 'Jatauba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1554, 'Jatoba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1555, 'Joao Alfredo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1556, 'Joaquim Nabuco', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1557, 'Jucati', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1558, 'Jupi', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1559, 'Jurema', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1560, 'Lagoa do Carro', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1561, 'Lagoa do Itaenga', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1562, 'Lagoa do Ouro', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1563, 'Lagoa dos Gatos', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1564, 'Lagoa Grande', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1565, 'Lajedo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1566, 'Limoeiro', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1567, 'Macaparana', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1568, 'Machados', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1569, 'Manari', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1570, 'Maraial', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1571, 'Mirandiba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1572, 'Moreilandia', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1573, 'Moreno', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1574, 'Nazare da Mata', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1575, 'Olinda', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1576, 'Orobo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1577, 'Oroco', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1578, 'Ouricuri', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1579, 'Palmares', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1580, 'Palmeirina', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1581, 'Panelas', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1582, 'Paranatama', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1583, 'Parnamirim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1584, 'Passira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1585, 'Paudalho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1586, 'Paulista', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1587, 'Pedra', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1588, 'Pesqueira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1589, 'Petrolandia', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1590, 'Petrolina', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1591, 'Pocao', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1592, 'Pombos', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1593, 'Primavera', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1594, 'Quipapa', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1595, 'Quixaba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1596, 'Recife', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1597, 'Riacho das Almas', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1598, 'Ribeirao', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1599, 'Rio Formoso', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1600, 'Saire', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1601, 'Salgadinho', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1602, 'Salgueiro', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1603, 'Saloa', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1604, 'Sanharo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1605, 'Santa Cruz', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1606, 'Santa Cruz da Baixa Verde', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1607, 'Santa Cruz do Capibaribe', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1608, 'Santa Filomena', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1609, 'Santa Maria da Boa Vista', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1610, 'Santa Maria do Cambuca', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1611, 'Santa Terezinha', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1612, 'Sao Benedito do Sul', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1613, 'Sao Bento do Una', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1614, 'Sao Caitano', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1615, 'Sao Joao', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1616, 'Sao Joaquim do Monte', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1617, 'Sao Jose da Coroa Grande', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1618, 'Sao Jose do Belmonte', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1619, 'Sao Jose do Egito', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1620, 'Sao Lourenco da Mata', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1621, 'Sao Vicente Ferrer', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1622, 'Serra Talhada', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1623, 'Serrita', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1624, 'Sertania', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1625, 'Sirinhaem', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1626, 'Solidao', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1627, 'Surubim', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1628, 'Tabira', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1629, 'Tacaimbo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1630, 'Tacaratu', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1631, 'Tamandare', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1632, 'Taquaritinga do Norte', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1633, 'Terezinha', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1634, 'Terra Nova', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1635, 'Timbauba', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1636, 'Toritama', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1637, 'Tracunhaem', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1638, 'Trindade', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1639, 'Triunfo', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1640, 'Tupanatinga', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1641, 'Tuparetama', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1642, 'Venturosa', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1643, 'Verdejante', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1644, 'Vertente do Lerio', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1645, 'Vertentes', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1646, 'Vicencia', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1647, 'Vitoria de Santo Antao', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1648, 'Xexeu', 'PE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1649, 'Agua Branca', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1650, 'Anadia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1651, 'Arapiraca', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1652, 'Atalaia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1653, 'Barra de Santo Antonio', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1654, 'Barra de Sao Miguel', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1655, 'Batalha', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1656, 'Belem', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1657, 'Belo Monte', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1658, 'Boca da Mata', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1659, 'Branquinha', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1660, 'Cacimbinhas', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1661, 'Cajueiro', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1662, 'Campestre', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1663, 'Campo Alegre', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1664, 'Campo Grande', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1665, 'Canapi', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1666, 'Capela', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1667, 'Carneiros', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1668, 'Cha Preta', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1669, 'Coite do Noia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1670, 'Colonia Leopoldina', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1671, 'Coqueiro Seco', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1672, 'Coruripe', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1673, 'Craibas', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1674, 'Delmiro Gouveia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1675, 'Dois Riachos', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1676, 'Estrela de Alagoas', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1677, 'Feira Grande', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1678, 'Feliz Deserto', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1679, 'Flexeiras', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1680, 'Girau do Ponciano', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1681, 'Ibateguara', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1682, 'Igaci', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1683, 'Igreja Nova', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1684, 'Inhapi', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1685, 'Jacare dos Homens', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1686, 'Jacuipe', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1687, 'Japaratinga', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1688, 'Jaramataia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1689, 'Jequia da Praia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1690, 'Joaquim Gomes', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1691, 'Jundia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1692, 'Junqueiro', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1693, 'Lagoa da Canoa', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1694, 'Limoeiro de Anadia', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1695, 'Maceio', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1696, 'Major Isidoro', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1697, 'Mar Vermelho', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1698, 'Maragogi', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1699, 'Maravilha', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1700, 'Marechal Deodoro', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1701, 'Maribondo', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1702, 'Mata Grande', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1703, 'Matriz de Camaragibe', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1704, 'Messias', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1705, 'Minador do Negrao', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1706, 'Monteiropolis', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1707, 'Murici', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1708, 'Novo Lino', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1709, 'Olho D''agua das Flores', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1710, 'Olho D''agua do Casado', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1711, 'Olho D''agua Grande', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1712, 'Olivenca', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1713, 'Ouro Branco', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1714, 'Palestina', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1715, 'Palmeira dos Indios', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1716, 'Pao de Acucar', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1717, 'Pariconha', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1718, 'Paripueira', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1719, 'Passo de Camaragibe', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1720, 'Paulo Jacinto', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1721, 'Penedo', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1722, 'Piacabucu', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1723, 'Pilar', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1724, 'Pindoba', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1725, 'Piranhas', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1726, 'Poco das Trincheiras', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1727, 'Porto Calvo', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1728, 'Porto de Pedras', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1729, 'Porto Real do Colegio', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1730, 'Quebrangulo', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1731, 'Rio Largo', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1732, 'Roteiro', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1733, 'Santa Luzia do Norte', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1734, 'Santana do Ipanema', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1735, 'Santana do Mundau', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1736, 'Sao Bras', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1737, 'Sao Jose da Laje', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1738, 'Sao Jose da Tapera', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1739, 'Sao Luis do Quitunde', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1740, 'Sao Miguel dos Campos', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1741, 'Sao Miguel dos Milagres', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1742, 'Sao Sebastiao', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1743, 'Satuba', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1744, 'Senador Rui Palmeira', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1745, 'Tanque D''arca', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1746, 'Taquarana', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1747, 'Teotonio Vilela', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1748, 'Traipu', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1749, 'Uniao dos Palmares', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1750, 'Vicosa', 'AL', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1751, 'Amparo de Sao Francisco', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1752, 'Aquidaba', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1753, 'Aracaju', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1754, 'Araua', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1755, 'Areia Branca', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1756, 'Barra dos Coqueiros', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1757, 'Boquim', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1758, 'Brejo Grande', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1759, 'Campo do Brito', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1760, 'Canhoba', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1761, 'Caninde de Sao Francisco', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1762, 'Capela', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1763, 'Carira', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1764, 'Carmopolis', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1765, 'Cedro de Sao Joao', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1766, 'Cristinapolis', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1767, 'Cumbe', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1768, 'Divina Pastora', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1769, 'Estancia', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1770, 'Feira Nova', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1771, 'Frei Paulo', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1772, 'Gararu', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1773, 'General Maynard', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1774, 'Gracho Cardoso', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1775, 'Ilha das Flores', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1776, 'Indiaroba', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1777, 'Itabaiana', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1778, 'Itabaianinha', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1779, 'Itabi', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1780, 'Itaporanga D''ajuda', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1781, 'Japaratuba', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1782, 'Japoata', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1783, 'Lagarto', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1784, 'Laranjeiras', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1785, 'Macambira', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1786, 'Malhada dos Bois', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1787, 'Malhador', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1788, 'Maruim', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1789, 'Moita Bonita', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1790, 'Monte Alegre de Sergipe', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1791, 'Muribeca', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1792, 'Neopolis', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1793, 'Nossa Senhora Aparecida', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1794, 'Nossa Senhora da Gloria', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1795, 'Nossa Senhora das Dores', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1796, 'Nossa Senhora de Lourdes', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1797, 'Nossa Senhora do Socorro', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1798, 'Pacatuba', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1799, 'Pedra Mole', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1800, 'Pedrinhas', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1801, 'Pinhao', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1802, 'Pirambu', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1803, 'Poco Redondo', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1804, 'Poco Verde', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1805, 'Porto da Folha', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1806, 'Propria', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1807, 'Riachao do Dantas', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1808, 'Riachuelo', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1809, 'Ribeiropolis', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1810, 'Rosario do Catete', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1811, 'Salgado', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1812, 'Santa Luzia do Itanhy', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1813, 'Santa Rosa de Lima', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1814, 'Santana do Sao Francisco', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1815, 'Santo Amaro das Brotas', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1816, 'Sao Cristovao', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1817, 'Sao Domingos', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1818, 'Sao Francisco', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1819, 'Sao Miguel do Aleixo', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1820, 'Simao Dias', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1821, 'Siriri', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1822, 'Telha', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1823, 'Tobias Barreto', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1824, 'Tomar do Geru', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1825, 'Umbauba', 'SE', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1826, 'Abaira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1827, 'Abare', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1828, 'Acajutiba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1829, 'Adustina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1830, 'Agua Fria', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1831, 'Aiquara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1832, 'Alagoinhas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1833, 'Alcobaca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1834, 'Almadina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1835, 'Amargosa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1836, 'Amelia Rodrigues', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1837, 'America Dourada', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1838, 'Anage', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1839, 'Andarai', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1840, 'Andorinha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1841, 'Angical', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1842, 'Anguera', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1843, 'Antas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1844, 'Antonio Cardoso', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1845, 'Antonio Goncalves', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1846, 'Apora', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1847, 'Apuarema', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1848, 'Aracas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1849, 'Aracatu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1850, 'Araci', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1851, 'Aramari', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1852, 'Arataca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1853, 'Aratuipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1854, 'Aurelino Leal', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1855, 'Baianopolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1856, 'Baixa Grande', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1857, 'Banzae', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1858, 'Barra', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1859, 'Barra da Estiva', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1860, 'Barra do Choca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1861, 'Barra do Mendes', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1862, 'Barra do Rocha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1863, 'Barreiras', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1864, 'Barro Alto', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1865, 'Barrocas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1866, 'Belmonte', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1867, 'Belo Campo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1868, 'Biritinga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1869, 'Boa Nova', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1870, 'Boa Vista do Tupim', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1871, 'Bom Jesus da Lapa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1872, 'Bom Jesus da Serra', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1873, 'Boninal', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1874, 'Bonito', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1875, 'Boquira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1876, 'Botupora', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1877, 'Brejoes', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1878, 'Brejolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1879, 'Brotas de Macaubas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1880, 'Brumado', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1881, 'Buerarema', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1882, 'Buritirama', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1883, 'Caatiba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1884, 'Cabaceiras do Paraguacu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1885, 'Cachoeira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1886, 'Cacule', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1887, 'Caem', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1888, 'Caetanos', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1889, 'Caetite', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1890, 'Cafarnaum', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1891, 'Cairu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1892, 'Caldeirao Grande', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1893, 'Camacan', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1894, 'Camacari', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1895, 'Camamu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1896, 'Campo Alegre de Lourdes', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1897, 'Campo Formoso', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1898, 'Canapolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1899, 'Canarana', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1900, 'Canavieiras', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1901, 'Candeal', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1902, 'Candeias', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1903, 'Candiba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1904, 'Candido Sales', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1905, 'Cansancao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1906, 'Canudos', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1907, 'Capela do Alto Alegre', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1908, 'Capim Grosso', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1909, 'Caraibas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1910, 'Caravelas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1911, 'Cardeal da Silva', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1912, 'Carinhanha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1913, 'Casa Nova', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1914, 'Castro Alves', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1915, 'Catolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1916, 'Catu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1917, 'Caturama', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1918, 'Central', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1919, 'Chorrocho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1920, 'Cicero Dantas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1921, 'Cipo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1922, 'Coaraci', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1923, 'Cocos', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1924, 'Conceicao da Feira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1925, 'Conceicao do Almeida', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1926, 'Conceicao do Coite', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1927, 'Conceicao do Jacuipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1928, 'Conde', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1929, 'Condeuba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1930, 'Contendas do Sincora', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1931, 'Coracao de Maria', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1932, 'Cordeiros', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1933, 'Coribe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1934, 'Coronel Joao Sa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1935, 'Correntina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1936, 'Cotegipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1937, 'Cravolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1938, 'Crisopolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1939, 'Cristopolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1940, 'Cruz das Almas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1941, 'Curaca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1942, 'Dario Meira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1943, 'Dias D''avila', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1944, 'Dom Basilio', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1945, 'Dom Macedo Costa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1946, 'Elisio Medrado', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1947, 'Encruzilhada', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1948, 'Entre Rios', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1949, 'Erico Cardoso', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1950, 'Esplanada', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1951, 'Euclides da Cunha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1952, 'Eunapolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1953, 'Fatima', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1954, 'Feira da Mata', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1955, 'Feira de Santana', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1956, 'Filadelfia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1957, 'Firmino Alves', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1958, 'Floresta Azul', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1959, 'Formosa do Rio Preto', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1960, 'Gandu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1961, 'Gaviao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1962, 'Gentio do Ouro', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1963, 'Gloria', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1964, 'Gongogi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1965, 'Governador Lomanto Junior', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1966, 'Governador Mangabeira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1967, 'Guajeru', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1968, 'Guanambi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1969, 'Guaratinga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1970, 'Heliopolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1971, 'Iacu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1972, 'Ibiassuce', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1973, 'Ibicarai', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1974, 'Ibicoara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1975, 'Ibicui', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1976, 'Ibipeba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1977, 'Ibipitanga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1978, 'Ibiquera', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1979, 'Ibirapitanga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1980, 'Ibirapua', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1981, 'Ibirataia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1982, 'Ibitiara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1983, 'Ibitita', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1984, 'Ibotirama', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1985, 'Ichu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1986, 'Igapora', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1987, 'Igrapiuna', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1988, 'Iguai', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1989, 'Ilheus', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1990, 'Inhambupe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1991, 'Ipecaeta', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1992, 'Ipiau', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1993, 'Ipira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1994, 'Ipupiara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1995, 'Irajuba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1996, 'Iramaia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1997, 'Iraquara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1998, 'Irara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (1999, 'Irece', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2000, 'Itabela', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2001, 'Itaberaba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2002, 'Itabuna', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2003, 'Itacare', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2004, 'Itaete', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2005, 'Itagi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2006, 'Itagiba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2007, 'Itagimirim', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2008, 'Itaguacu da Bahia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2009, 'Itaju do Colonia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2010, 'Itajuipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2011, 'Itamaraju', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2012, 'Itamari', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2013, 'Itambe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2014, 'Itanagra', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2015, 'Itanhem', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2016, 'Itaparica', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2017, 'Itape', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2018, 'Itapebi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2019, 'Itapetinga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2020, 'Itapicuru', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2021, 'Itapitanga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2022, 'Itaquara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2023, 'Itarantim', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2024, 'Itatim', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2025, 'Itirucu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2026, 'Itiuba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2027, 'Itororo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2028, 'Ituacu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2029, 'Itubera', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2030, 'Iuiu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2031, 'Jaborandi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2032, 'Jacaraci', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2033, 'Jacobina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2034, 'Jaguaquara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2035, 'Jaguarari', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2036, 'Jaguaripe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2037, 'Jandaira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2038, 'Jequie', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2039, 'Jeremoabo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2040, 'Jiquirica', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2041, 'Jitauna', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2042, 'Joao Dourado', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2043, 'Juazeiro', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2044, 'Jucurucu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2045, 'Jussara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2046, 'Jussari', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2047, 'Jussiape', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2048, 'Lafaiete Coutinho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2049, 'Lagoa Real', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2050, 'Laje', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2051, 'Lajedao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2052, 'Lajedinho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2053, 'Lajedo do Tabocal', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2054, 'Lamarao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2055, 'Lapao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2056, 'Lauro de Freitas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2057, 'Lencois', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2058, 'Licinio de Almeida', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2059, 'Livramento de Nossa Senhora', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2060, 'Luis Eduardo Magalhaes', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2061, 'Macajuba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2062, 'Macarani', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2063, 'Macaubas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2064, 'Macurure', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2065, 'Madre de Deus', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2066, 'Maetinga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2067, 'Maiquinique', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2068, 'Mairi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2069, 'Malhada', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2070, 'Malhada de Pedras', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2071, 'Manoel Vitorino', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2072, 'Mansidao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2073, 'Maracas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2074, 'Maragogipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2075, 'Marau', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2076, 'Marcionilio Souza', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2077, 'Mascote', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2078, 'Mata de Sao Joao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2079, 'Matina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2080, 'Medeiros Neto', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2081, 'Miguel Calmon', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2082, 'Milagres', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2083, 'Mirangaba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2084, 'Mirante', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2085, 'Monte Santo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2086, 'Morpara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2087, 'Morro do Chapeu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2088, 'Mortugaba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2089, 'Mucuge', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2090, 'Mucuri', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2091, 'Mulungu do Morro', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2092, 'Mundo Novo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2093, 'Muniz Ferreira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2094, 'Muquem de Sao Francisco', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2095, 'Muritiba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2096, 'Mutuipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2097, 'Nazare', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2098, 'Nilo Pecanha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2099, 'Nordestina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2100, 'Nova Canaa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2101, 'Nova Fatima', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2102, 'Nova Ibia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2103, 'Nova Itarana', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2104, 'Nova Redencao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2105, 'Nova Soure', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2106, 'Nova Vicosa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2107, 'Novo Horizonte', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2108, 'Novo Triunfo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2109, 'Olindina', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2110, 'Oliveira dos Brejinhos', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2111, 'Ouricangas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2112, 'Ourolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2113, 'Palmas de Monte Alto', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2114, 'Palmeiras', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2115, 'Paramirim', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2116, 'Paratinga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2117, 'Paripiranga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2118, 'Pau Brasil', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2119, 'Paulo Afonso', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2120, 'Pe de Serra', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2121, 'Pedrao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2122, 'Pedro Alexandre', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2123, 'Piata', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2124, 'Pilao Arcado', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2125, 'Pindai', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2126, 'Pindobacu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2127, 'Pintadas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2128, 'Pirai do Norte', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2129, 'Piripa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2130, 'Piritiba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2131, 'Planaltino', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2132, 'Planalto', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2133, 'Pocoes', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2134, 'Pojuca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2135, 'Ponto Novo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2136, 'Porto Seguro', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2137, 'Potiragua', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2138, 'Prado', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2139, 'Presidente Dutra', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2140, 'Presidente Janio Quadros', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2141, 'Presidente Tancredo Neves', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2142, 'Queimadas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2143, 'Quijingue', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2144, 'Quixabeira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2145, 'Rafael Jambeiro', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2146, 'Remanso', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2147, 'Retirolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2148, 'Riachao das Neves', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2149, 'Riachao do Jacuipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2150, 'Riacho de Santana', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2151, 'Ribeira do Amparo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2152, 'Ribeira do Pombal', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2153, 'Ribeirao do Largo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2154, 'Rio de Contas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2155, 'Rio do Antonio', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2156, 'Rio do Pires', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2157, 'Rio Real', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2158, 'Rodelas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2159, 'Ruy Barbosa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2160, 'Salinas da Margarida', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2161, 'Salvador', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2162, 'Santa Barbara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2163, 'Santa Brigida', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2164, 'Santa Cruz Cabralia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2165, 'Santa Cruz da Vitoria', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2166, 'Santa Ines', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2167, 'Santa Luzia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2168, 'Santa Maria da Vitoria', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2169, 'Santa Rita de Cassia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2170, 'Santa Teresinha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2171, 'Santaluz', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2172, 'Santana', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2173, 'Santanopolis', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2174, 'Santo Amaro', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2175, 'Santo Antonio de Jesus', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2176, 'Santo Estevao', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2177, 'Sao Desiderio', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2178, 'Sao Domingos', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2179, 'Sao Felipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2180, 'Sao Felix', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2181, 'Sao Felix do Coribe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2182, 'Sao Francisco do Conde', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2183, 'Sao Gabriel', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2184, 'Sao Goncalo dos Campos', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2185, 'Sao Jose da Vitoria', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2186, 'Sao Jose do Jacuipe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2187, 'Sao Miguel das Matas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2188, 'Sao Sebastiao do Passe', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2189, 'Sapeacu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2190, 'Satiro Dias', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2191, 'Saubara', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2192, 'Saude', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2193, 'Seabra', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2194, 'Sebastiao Laranjeiras', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2195, 'Senhor do Bonfim', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2196, 'Sento Se', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2197, 'Serra do Ramalho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2198, 'Serra Dourada', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2199, 'Serra Preta', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2200, 'Serrinha', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2201, 'Serrolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2202, 'Simoes Filho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2203, 'Sitio do Mato', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2204, 'Sitio do Quinto', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2205, 'Sobradinho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2206, 'Souto Soares', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2207, 'Tabocas do Brejo Velho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2208, 'Tanhacu', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2209, 'Tanque Novo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2210, 'Tanquinho', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2211, 'Taperoa', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2212, 'Tapiramuta', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2213, 'Teixeira de Freitas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2214, 'Teodoro Sampaio', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2215, 'Teofilandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2216, 'Teolandia', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2217, 'Terra Nova', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2218, 'Tremedal', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2219, 'Tucano', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2220, 'Uaua', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2221, 'Ubaira', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2222, 'Ubaitaba', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2223, 'Ubata', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2224, 'Uibai', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2225, 'Umburanas', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2226, 'Una', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2227, 'Urandi', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2228, 'Urucuca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2229, 'Utinga', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2230, 'Valenca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2231, 'Valente', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2232, 'Varzea da Roca', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2233, 'Varzea do Poco', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2234, 'Varzea Nova', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2235, 'Varzedo', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2236, 'Vera Cruz', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2237, 'Vereda', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2238, 'Vitoria da Conquista', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2239, 'Wagner', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2240, 'Wanderley', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2241, 'Wenceslau Guimaraes', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2242, 'Xique-xique', 'BA', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2243, 'Abadia dos Dourados', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2244, 'Abaete', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2245, 'Abre Campo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2246, 'Acaiaca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2247, 'Acucena', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2248, 'Agua Boa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2249, 'Agua Comprida', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2250, 'Aguanil', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2251, 'Aguas Formosas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2252, 'Aguas Vermelhas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2253, 'Aimores', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2254, 'Aiuruoca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2255, 'Alagoa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2256, 'Albertina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2257, 'Alem Paraiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2258, 'Alfenas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2259, 'Alfredo Vasconcelos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2260, 'Almenara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2261, 'Alpercata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2262, 'Alpinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2263, 'Alterosa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2264, 'Alto Caparao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2265, 'Alto Jequitiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2266, 'Alto Rio Doce', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2267, 'Alvarenga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2268, 'Alvinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2269, 'Alvorada de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2270, 'Amparo do Serra', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2271, 'Andradas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2272, 'Andrelandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2273, 'Angelandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2274, 'Antonio Carlos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2275, 'Antonio Dias', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2276, 'Antonio Prado de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2277, 'Aracai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2278, 'Aracitaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2279, 'Aracuai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2280, 'Araguari', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2281, 'Arantina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2282, 'Araponga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2283, 'Arapora', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2284, 'Arapua', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2285, 'Araujos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2286, 'Araxa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2287, 'Arceburgo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2288, 'Arcos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2289, 'Areado', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2290, 'Argirita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2291, 'Aricanduva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2292, 'Arinos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2293, 'Astolfo Dutra', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2294, 'Ataleia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2295, 'Augusto de Lima', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2296, 'Baependi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2297, 'Baldim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2298, 'Bambui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2299, 'Bandeira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2300, 'Bandeira do Sul', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2301, 'Barao de Cocais', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2302, 'Barao de Monte Alto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2303, 'Barbacena', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2304, 'Barra Longa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2305, 'Barroso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2306, 'Bela Vista de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2307, 'Belmiro Braga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2308, 'Belo Horizonte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2309, 'Belo Oriente', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2310, 'Belo Vale', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2311, 'Berilo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2312, 'Berizal', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2313, 'Bertopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2314, 'Betim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2315, 'Bias Fortes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2316, 'Bicas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2317, 'Biquinhas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2318, 'Boa Esperanca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2319, 'Bocaina de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2320, 'Bocaiuva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2321, 'Bom Despacho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2322, 'Bom Jardim de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2323, 'Bom Jesus da Penha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2324, 'Bom Jesus do Amparo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2325, 'Bom Jesus do Galho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2326, 'Bom Repouso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2327, 'Bom Sucesso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2328, 'Bonfim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2329, 'Bonfinopolis de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2330, 'Bonito de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2331, 'Borda da Mata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2332, 'Botelhos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2333, 'Botumirim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2334, 'Bras Pires', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2335, 'Brasilandia de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2336, 'Brasilia de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2337, 'Brasopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2338, 'Braunas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2339, 'Brumadinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2340, 'Bueno Brandao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2341, 'Buenopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2342, 'Bugre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2343, 'Buritis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2344, 'Buritizeiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2345, 'Cabeceira Grande', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2346, 'Cabo Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2347, 'Cachoeira da Prata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2348, 'Cachoeira de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2349, 'Cachoeira de Pajeu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2350, 'Cachoeira Dourada', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2351, 'Caetanopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2352, 'Caete', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2353, 'Caiana', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2354, 'Cajuri', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2355, 'Caldas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2356, 'Camacho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2357, 'Camanducaia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2358, 'Cambui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2359, 'Cambuquira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2360, 'Campanario', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2361, 'Campanha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2362, 'Campestre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2363, 'Campina Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2364, 'Campo Azul', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2365, 'Campo Belo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2366, 'Campo do Meio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2367, 'Campo Florido', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2368, 'Campos Altos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2369, 'Campos Gerais', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2370, 'Cana Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2371, 'Canaa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2372, 'Canapolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2373, 'Candeias', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2374, 'Cantagalo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2375, 'Caparao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2376, 'Capela Nova', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2377, 'Capelinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2378, 'Capetinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2379, 'Capim Branco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2380, 'Capinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2381, 'Capitao Andrade', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2382, 'Capitao Eneas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2383, 'Capitolio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2384, 'Caputira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2385, 'Carai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2386, 'Caranaiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2387, 'Carandai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2388, 'Carangola', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2389, 'Caratinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2390, 'Carbonita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2391, 'Careacu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2392, 'Carlos Chagas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2393, 'Carmesia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2394, 'Carmo da Cachoeira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2395, 'Carmo da Mata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2396, 'Carmo de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2397, 'Carmo do Cajuru', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2398, 'Carmo do Paranaiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2399, 'Carmo do Rio Claro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2400, 'Carmopolis de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2401, 'Carneirinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2402, 'Carrancas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2403, 'Carvalhopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2404, 'Carvalhos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2405, 'Casa Grande', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2406, 'Cascalho Rico', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2407, 'Cassia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2408, 'Cataguases', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2409, 'Catas Altas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2410, 'Catas Altas da Noruega', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2411, 'Catuji', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2412, 'Catuti', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2413, 'Caxambu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2414, 'Cedro do Abaete', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2415, 'Central de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2416, 'Centralina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2417, 'Chacara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2418, 'Chale', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2419, 'Chapada do Norte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2420, 'Chapada Gaucha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2421, 'Chiador', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2422, 'Cipotanea', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2423, 'Claraval', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2424, 'Claro dos Pocoes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2425, 'Claudio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2426, 'Coimbra', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2427, 'Coluna', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2428, 'Comendador Gomes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2429, 'Comercinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2430, 'Conceicao da Aparecida', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2431, 'Conceicao da Barra de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2432, 'Conceicao das Alagoas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2433, 'Conceicao das Pedras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2434, 'Conceicao de Ipanema', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2435, 'Conceicao do Mato Dentro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2436, 'Conceicao do Para', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2437, 'Conceicao do Rio Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2438, 'Conceicao dos Ouros', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2439, 'Conego Marinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2440, 'Confins', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2441, 'Congonhal', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2442, 'Congonhas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2443, 'Congonhas do Norte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2444, 'Conquista', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2445, 'Conselheiro Lafaiete', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2446, 'Conselheiro Pena', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2447, 'Consolacao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2448, 'Contagem', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2449, 'Coqueiral', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2450, 'Coracao de Jesus', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2451, 'Cordisburgo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2452, 'Cordislandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2453, 'Corinto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2454, 'Coroaci', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2455, 'Coromandel', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2456, 'Coronel Fabriciano', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2457, 'Coronel Murta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2458, 'Coronel Pacheco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2459, 'Coronel Xavier Chaves', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2460, 'Corrego Danta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2461, 'Corrego do Bom Jesus', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2462, 'Corrego Fundo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2463, 'Corrego Novo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2464, 'Couto de Magalhaes de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2465, 'Crisolita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2466, 'Cristais', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2467, 'Cristalia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2468, 'Cristiano Otoni', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2469, 'Cristina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2470, 'Crucilandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2471, 'Cruzeiro da Fortaleza', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2472, 'Cruzilia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2473, 'Cuparaque', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2474, 'Curral de Dentro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2475, 'Curvelo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2476, 'Datas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2477, 'Delfim Moreira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2478, 'Delfinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2479, 'Delta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2480, 'Descoberto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2481, 'Desterro de Entre Rios', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2482, 'Desterro do Melo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2483, 'Diamantina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2484, 'Diogo de Vasconcelos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2485, 'Dionisio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2486, 'Divinesia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2487, 'Divino', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2488, 'Divino das Laranjeiras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2489, 'Divinolandia de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2490, 'Divinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2491, 'Divisa Alegre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2492, 'Divisa Nova', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2493, 'Divisopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2494, 'Dom Bosco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2495, 'Dom Cavati', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2496, 'Dom Joaquim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2497, 'Dom Silverio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2498, 'Dom Vicoso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2499, 'Dona Eusebia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2500, 'Dores de Campos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2501, 'Dores de Guanhaes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2502, 'Dores do Indaia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2503, 'Dores do Turvo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2504, 'Doresopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2505, 'Douradoquara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2506, 'Durande', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2507, 'Eloi Mendes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2508, 'Engenheiro Caldas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2509, 'Engenheiro Navarro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2510, 'Entre Folhas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2511, 'Entre Rios de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2512, 'Ervalia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2513, 'Esmeraldas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2514, 'Espera Feliz', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2515, 'Espinosa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2516, 'Espirito Santo do Dourado', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2517, 'Estiva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2518, 'Estrela Dalva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2519, 'Estrela do Indaia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2520, 'Estrela do Sul', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2521, 'Eugenopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2522, 'Ewbank da Camara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2523, 'Extrema', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2524, 'Fama', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2525, 'Faria Lemos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2526, 'Felicio dos Santos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2527, 'Felisburgo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2528, 'Felixlandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2529, 'Fernandes Tourinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2530, 'Ferros', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2531, 'Fervedouro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2532, 'Florestal', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2533, 'Formiga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2534, 'Formoso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2535, 'Fortaleza de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2536, 'Fortuna de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2537, 'Francisco Badaro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2538, 'Francisco Dumont', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2539, 'Francisco Sa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2540, 'Franciscopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2541, 'Frei Gaspar', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2542, 'Frei Inocencio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2543, 'Frei Lagonegro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2544, 'Fronteira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2545, 'Fronteira dos Vales', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2546, 'Fruta de Leite', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2547, 'Frutal', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2548, 'Funilandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2549, 'Galileia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2550, 'Gameleiras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2551, 'Glaucilandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2552, 'Goiabeira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2553, 'Goiana', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2554, 'Goncalves', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2555, 'Gonzaga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2556, 'Gouveia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2557, 'Governador Valadares', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2558, 'Grao Mogol', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2559, 'Grupiara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2560, 'Guanhaes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2561, 'Guape', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2562, 'Guaraciaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2563, 'Guaraciama', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2564, 'Guaranesia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2565, 'Guarani', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2566, 'Guarara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2567, 'Guarda-mor', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2568, 'Guaxupe', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2569, 'Guidoval', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2570, 'Guimarania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2571, 'Guiricema', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2572, 'Gurinhata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2573, 'Heliodora', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2574, 'Iapu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2575, 'Ibertioga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2576, 'Ibia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2577, 'Ibiai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2578, 'Ibiracatu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2579, 'Ibiraci', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2580, 'Ibirite', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2581, 'Ibitiura de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2582, 'Ibituruna', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2583, 'Icarai de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2584, 'Igarape', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2585, 'Igaratinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2586, 'Iguatama', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2587, 'Ijaci', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2588, 'Ilicinea', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2589, 'Imbe de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2590, 'Inconfidentes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2591, 'Indaiabira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2592, 'Indianopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2593, 'Ingai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2594, 'Inhapim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2595, 'Inhauma', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2596, 'Inimutaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2597, 'Ipaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2598, 'Ipanema', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2599, 'Ipatinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2600, 'Ipiacu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2601, 'Ipuiuna', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2602, 'Irai de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2603, 'Itabira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2604, 'Itabirinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2605, 'Itabirito', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2606, 'Itacambira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2607, 'Itacarambi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2608, 'Itaguara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2609, 'Itaipe', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2610, 'Itajuba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2611, 'Itamarandiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2612, 'Itamarati de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2613, 'Itambacuri', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2614, 'Itambe do Mato Dentro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2615, 'Itamogi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2616, 'Itamonte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2617, 'Itanhandu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2618, 'Itanhomi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2619, 'Itaobim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2620, 'Itapagipe', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2621, 'Itapecerica', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2622, 'Itapeva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2623, 'Itatiaiucu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2624, 'Itau de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2625, 'Itauna', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2626, 'Itaverava', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2627, 'Itinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2628, 'Itueta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2629, 'Ituiutaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2630, 'Itumirim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2631, 'Iturama', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2632, 'Itutinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2633, 'Jaboticatubas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2634, 'Jacinto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2635, 'Jacui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2636, 'Jacutinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2637, 'Jaguaracu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2638, 'Jaiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2639, 'Jampruca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2640, 'Janauba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2641, 'Januaria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2642, 'Japaraiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2643, 'Japonvar', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2644, 'Jeceaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2645, 'Jenipapo de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2646, 'Jequeri', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2647, 'Jequitai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2648, 'Jequitiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2649, 'Jequitinhonha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2650, 'Jesuania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2651, 'Joaima', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2652, 'Joanesia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2653, 'Joao Monlevade', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2654, 'Joao Pinheiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2655, 'Joaquim Felicio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2656, 'Jordania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2657, 'Jose Goncalves de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2658, 'Jose Raydan', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2659, 'Josenopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2660, 'Juatuba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2661, 'Juiz de Fora', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2662, 'Juramento', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2663, 'Juruaia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2664, 'Juvenilia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2665, 'Ladainha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2666, 'Lagamar', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2667, 'Lagoa da Prata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2668, 'Lagoa dos Patos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2669, 'Lagoa Dourada', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2670, 'Lagoa Formosa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2671, 'Lagoa Grande', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2672, 'Lagoa Santa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2673, 'Lajinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2674, 'Lambari', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2675, 'Lamim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2676, 'Laranjal', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2677, 'Lassance', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2678, 'Lavras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2679, 'Leandro Ferreira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2680, 'Leme do Prado', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2681, 'Leopoldina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2682, 'Liberdade', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2683, 'Lima Duarte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2684, 'Limeira do Oeste', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2685, 'Lontra', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2686, 'Luisburgo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2687, 'Luislandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2688, 'Luminarias', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2689, 'Luz', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2690, 'Machacalis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2691, 'Machado', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2692, 'Madre de Deus de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2693, 'Malacacheta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2694, 'Mamonas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2695, 'Manga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2696, 'Manhuacu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2697, 'Manhumirim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2698, 'Mantena', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2699, 'Mar de Espanha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2700, 'Maravilhas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2701, 'Maria da Fe', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2702, 'Mariana', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2703, 'Marilac', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2704, 'Mario Campos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2705, 'Maripa de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2706, 'Marlieria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2707, 'Marmelopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2708, 'Martinho Campos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2709, 'Martins Soares', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2710, 'Mata Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2711, 'Materlandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2712, 'Mateus Leme', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2713, 'Mathias Lobato', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2714, 'Matias Barbosa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2715, 'Matias Cardoso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2716, 'Matipo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2717, 'Mato Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2718, 'Matozinhos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2719, 'Matutina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2720, 'Medeiros', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2721, 'Medina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2722, 'Mendes Pimentel', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2723, 'Merces', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2724, 'Mesquita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2725, 'Minas Novas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2726, 'Minduri', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2727, 'Mirabela', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2728, 'Miradouro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2729, 'Mirai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2730, 'Miravania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2731, 'Moeda', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2732, 'Moema', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2733, 'Monjolos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2734, 'Monsenhor Paulo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2735, 'Montalvania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2736, 'Monte Alegre de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2737, 'Monte Azul', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2738, 'Monte Belo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2739, 'Monte Carmelo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2740, 'Monte Formoso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2741, 'Monte Santo de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2742, 'Monte Siao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2743, 'Montes Claros', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2744, 'Montezuma', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2745, 'Morada Nova de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2746, 'Morro da Garca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2747, 'Morro do Pilar', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2748, 'Munhoz', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2749, 'Muriae', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2750, 'Mutum', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2751, 'Muzambinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2752, 'Nacip Raydan', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2753, 'Nanuque', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2754, 'Naque', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2755, 'Natalandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2756, 'Natercia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2757, 'Nazareno', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2758, 'Nepomuceno', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2759, 'Ninheira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2760, 'Nova Belem', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2761, 'Nova Era', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2762, 'Nova Lima', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2763, 'Nova Modica', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2764, 'Nova Ponte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2765, 'Nova Porteirinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2766, 'Nova Resende', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2767, 'Nova Serrana', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2768, 'Nova Uniao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2769, 'Novo Cruzeiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2770, 'Novo Oriente de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2771, 'Novorizonte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2772, 'Olaria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2773, 'Olhos-d Agua', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2774, 'Olimpio Noronha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2775, 'Oliveira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2776, 'Oliveira Fortes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2777, 'Onca de Pitangui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2778, 'Oratorios', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2779, 'Orizania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2780, 'Ouro Branco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2781, 'Ouro Fino', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2782, 'Ouro Preto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2783, 'Ouro Verde de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2784, 'Padre Carvalho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2785, 'Padre Paraiso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2786, 'Pai Pedro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2787, 'Paineiras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2788, 'Pains', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2789, 'Paiva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2790, 'Palma', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2791, 'Palmopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2792, 'Papagaios', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2793, 'Para de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2794, 'Paracatu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2795, 'Paraguacu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2796, 'Paraisopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2797, 'Paraopeba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2798, 'Passa Quatro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2799, 'Passa Tempo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2800, 'Passa Vinte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2801, 'Passabem', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2802, 'Passos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2803, 'Patis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2804, 'Patos de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2805, 'Patrocinio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2806, 'Patrocinio do Muriae', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2807, 'Paula Candido', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2808, 'Paulistas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2809, 'Pavao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2810, 'Pecanha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2811, 'Pedra Azul', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2812, 'Pedra Bonita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2813, 'Pedra do Anta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2814, 'Pedra do Indaia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2815, 'Pedra Dourada', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2816, 'Pedralva', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2817, 'Pedras de Maria da Cruz', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2818, 'Pedrinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2819, 'Pedro Leopoldo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2820, 'Pedro Teixeira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2821, 'Pequeri', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2822, 'Pequi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2823, 'Perdigao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2824, 'Perdizes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2825, 'Perdoes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2826, 'Periquito', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2827, 'Pescador', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2828, 'Piau', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2829, 'Piedade de Caratinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2830, 'Piedade de Ponte Nova', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2831, 'Piedade do Rio Grande', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2832, 'Piedade dos Gerais', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2833, 'Pimenta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2834, 'Pingo D Agua', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2835, 'Pintopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2836, 'Piracema', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2837, 'Pirajuba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2838, 'Piranga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2839, 'Pirangucu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2840, 'Piranguinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2841, 'Pirapetinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2842, 'Pirapora', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2843, 'Pirauba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2844, 'Pitangui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2845, 'Piumhi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2846, 'Planura', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2847, 'Poco Fundo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2848, 'Pocos de Caldas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2849, 'Pocrane', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2850, 'Pompeu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2851, 'Ponte Nova', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2852, 'Ponto Chique', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2853, 'Ponto dos Volantes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2854, 'Porteirinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2855, 'Porto Firme', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2856, 'Pote', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2857, 'Pouso Alegre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2858, 'Pouso Alto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2859, 'Prados', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2860, 'Prata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2861, 'Pratapolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2862, 'Pratinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2863, 'Presidente Bernardes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2864, 'Presidente Juscelino', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2865, 'Presidente Kubitschek', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2866, 'Presidente Olegario', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2867, 'Prudente de Morais', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2868, 'Quartel Geral', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2869, 'Queluzito', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2870, 'Raposos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2871, 'Raul Soares', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2872, 'Recreio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2873, 'Reduto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2874, 'Resende Costa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2875, 'Resplendor', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2876, 'Ressaquinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2877, 'Riachinho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2878, 'Riacho dos Machados', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2879, 'Ribeirao das Neves', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2880, 'Ribeirao Vermelho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2881, 'Rio Acima', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2882, 'Rio Casca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2883, 'Rio do Prado', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2884, 'Rio Doce', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2885, 'Rio Espera', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2886, 'Rio Manso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2887, 'Rio Novo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2888, 'Rio Paranaiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2889, 'Rio Pardo de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2890, 'Rio Piracicaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2891, 'Rio Pomba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2892, 'Rio Preto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2893, 'Rio Vermelho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2894, 'Ritapolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2895, 'Rochedo de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2896, 'Rodeiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2897, 'Romaria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2898, 'Rosario da Limeira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2899, 'Rubelita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2900, 'Rubim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2901, 'Sabara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2902, 'Sabinopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2903, 'Sacramento', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2904, 'Salinas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2905, 'Salto da Divisa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2906, 'Santa Barbara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2907, 'Santa Barbara do Leste', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2908, 'Santa Barbara do Monte Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2909, 'Santa Barbara do Tugurio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2910, 'Santa Cruz de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2911, 'Santa Cruz de Salinas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2912, 'Santa Cruz do Escalvado', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2913, 'Santa Efigenia de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2914, 'Santa Fe de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2915, 'Santa Helena de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2916, 'Santa Juliana', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2917, 'Santa Luzia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2918, 'Santa Margarida', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2919, 'Santa Maria de Itabira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2920, 'Santa Maria do Salto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2921, 'Santa Maria do Suacui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2922, 'Santa Rita de Caldas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2923, 'Santa Rita de Ibitipoca', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2924, 'Santa Rita de Jacutinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2925, 'Santa Rita de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2926, 'Santa Rita do Itueto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2927, 'Santa Rita do Sapucai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2928, 'Santa Rosa da Serra', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2929, 'Santa Vitoria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2930, 'Santana da Vargem', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2931, 'Santana de Cataguases', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2932, 'Santana de Pirapama', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2933, 'Santana do Deserto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2934, 'Santana do Garambeu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2935, 'Santana do Jacare', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2936, 'Santana do Manhuacu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2937, 'Santana do Paraiso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2938, 'Santana do Riacho', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2939, 'Santana dos Montes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2940, 'Santo Antonio do Amparo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2941, 'Santo Antonio do Aventureiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2942, 'Santo Antonio do Grama', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2943, 'Santo Antonio do Itambe', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2944, 'Santo Antonio do Jacinto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2945, 'Santo Antonio do Monte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2946, 'Santo Antonio do Retiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2947, 'Santo Antonio do Rio Abaixo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2948, 'Santo Hipolito', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2949, 'Santos Dumont', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2950, 'Sao Bento Abade', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2951, 'Sao Bras do Suacui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2952, 'Sao Domingos das Dores', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2953, 'Sao Domingos do Prata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2954, 'Sao Felix de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2955, 'Sao Francisco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2956, 'Sao Francisco de Paula', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2957, 'Sao Francisco de Sales', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2958, 'Sao Francisco do Gloria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2959, 'Sao Geraldo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2960, 'Sao Geraldo da Piedade', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2961, 'Sao Geraldo do Baixio', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2962, 'Sao Goncalo do Abaete', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2963, 'Sao Goncalo do Para', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2964, 'Sao Goncalo do Rio Abaixo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2965, 'Sao Goncalo do Rio Preto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2966, 'Sao Goncalo do Sapucai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2967, 'Sao Gotardo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2968, 'Sao Joao Batista do Gloria', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2969, 'Sao Joao da Lagoa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2970, 'Sao Joao da Mata', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2971, 'Sao Joao da Ponte', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2972, 'Sao Joao das Missoes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2973, 'Sao Joao Del Rei', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2974, 'Sao Joao do Manhuacu', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2975, 'Sao Joao do Manteninha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2976, 'Sao Joao do Oriente', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2977, 'Sao Joao do Pacui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2978, 'Sao Joao do Paraiso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2979, 'Sao Joao Evangelista', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2980, 'Sao Joao Nepomuceno', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2981, 'Sao Joaquim de Bicas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2982, 'Sao Jose da Barra', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2983, 'Sao Jose da Lapa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2984, 'Sao Jose da Safira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2985, 'Sao Jose da Varginha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2986, 'Sao Jose do Alegre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2987, 'Sao Jose do Divino', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2988, 'Sao Jose do Goiabal', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2989, 'Sao Jose do Jacuri', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2990, 'Sao Jose do Mantimento', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2991, 'Sao Lourenco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2992, 'Sao Miguel do Anta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2993, 'Sao Pedro da Uniao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2994, 'Sao Pedro do Suacui', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2995, 'Sao Pedro dos Ferros', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2996, 'Sao Romao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2997, 'Sao Roque de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2998, 'Sao Sebastiao da Bela Vista', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (2999, 'Sao Sebastiao da Vargem Alegre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3000, 'Sao Sebastiao do Anta', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3001, 'Sao Sebastiao do Maranhao', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3002, 'Sao Sebastiao do Oeste', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3003, 'Sao Sebastiao do Paraiso', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3004, 'Sao Sebastiao do Rio Preto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3005, 'Sao Sebastiao do Rio Verde', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3006, 'Sao Thome das Letras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3007, 'Sao Tiago', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3008, 'Sao Tomas de Aquino', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3009, 'Sao Vicente de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3010, 'Sapucai-mirim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3011, 'Sardoa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3012, 'Sarzedo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3013, 'Sem Peixe', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3014, 'Senador Amaral', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3015, 'Senador Cortes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3016, 'Senador Firmino', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3017, 'Senador Jose Bento', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3018, 'Senador Modestino Goncalves', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3019, 'Senhora de Oliveira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3020, 'Senhora do Porto', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3021, 'Senhora dos Remedios', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3022, 'Sericita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3023, 'Seritinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3024, 'Serra Azul de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3025, 'Serra da Saudade', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3026, 'Serra do Salitre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3027, 'Serra dos Aimores', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3028, 'Serrania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3029, 'Serranopolis de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3030, 'Serranos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3031, 'Serro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3032, 'Sete Lagoas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3033, 'Setubinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3034, 'Silveirania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3035, 'Silvianopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3036, 'Simao Pereira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3037, 'Simonesia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3038, 'Sobralia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3039, 'Soledade de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3040, 'Tabuleiro', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3041, 'Taiobeiras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3042, 'Taparuba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3043, 'Tapira', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3044, 'Tapirai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3045, 'Taquaracu de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3046, 'Tarumirim', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3047, 'Teixeiras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3048, 'Teofilo Otoni', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3049, 'Timoteo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3050, 'Tiradentes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3051, 'Tiros', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3052, 'Tocantins', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3053, 'Tocos do Mogi', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3054, 'Toledo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3055, 'Tombos', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3056, 'Tres Coracoes', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3057, 'Tres Marias', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3058, 'Tres Pontas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3059, 'Tumiritinga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3060, 'Tupaciguara', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3061, 'Turmalina', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3062, 'Turvolandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3063, 'Uba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3064, 'Ubai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3065, 'Ubaporanga', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3066, 'Uberaba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3067, 'Uberlandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3068, 'Umburatiba', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3069, 'Unai', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3070, 'Uniao de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3071, 'Uruana de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3072, 'Urucania', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3073, 'Urucuia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3074, 'Vargem Alegre', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3075, 'Vargem Bonita', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3076, 'Vargem Grande do Rio Pardo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3077, 'Varginha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3078, 'Varjao de Minas', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3079, 'Varzea da Palma', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3080, 'Varzelandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3081, 'Vazante', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3082, 'Verdelandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3083, 'Veredinha', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3084, 'Verissimo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3085, 'Vermelho Novo', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3086, 'Vespasiano', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3087, 'Vicosa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3088, 'Vieiras', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3089, 'Virgem da Lapa', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3090, 'Virginia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3091, 'Virginopolis', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3092, 'Virgolandia', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3093, 'Visconde do Rio Branco', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3094, 'Volta Grande', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3095, 'Wenceslau Braz', 'MG', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3096, 'Afonso Claudio', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3097, 'Agua Doce do Norte', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3098, 'Aguia Branca', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3099, 'Alegre', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3100, 'Alfredo Chaves', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3101, 'Alto Rio Novo', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3102, 'Anchieta', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3103, 'Apiaca', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3104, 'Aracruz', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3105, 'Atilio Vivacqua', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3106, 'Baixo Guandu', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3107, 'Barra de Sao Francisco', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3108, 'Boa Esperanca', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3109, 'Bom Jesus do Norte', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3110, 'Brejetuba', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3111, 'Cachoeiro de Itapemirim', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3112, 'Cariacica', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3113, 'Castelo', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3114, 'Colatina', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3115, 'Conceicao da Barra', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3116, 'Conceicao do Castelo', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3117, 'Divino de Sao Lourenco', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3118, 'Domingos Martins', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3119, 'Dores do Rio Preto', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3120, 'Ecoporanga', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3121, 'Fundao', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3122, 'Governador Lindenberg', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3123, 'Guacui', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3124, 'Guarapari', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3125, 'Ibatiba', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3126, 'Ibiracu', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3127, 'Ibitirama', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3128, 'Iconha', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3129, 'Irupi', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3130, 'Itaguacu', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3131, 'Itapemirim', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3132, 'Itarana', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3133, 'Iuna', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3134, 'Jaguare', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3135, 'Jeronimo Monteiro', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3136, 'Joao Neiva', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3137, 'Laranja da Terra', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3138, 'Linhares', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3139, 'Mantenopolis', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3140, 'Marataizes', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3141, 'Marechal Floriano', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3142, 'Marilandia', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3143, 'Mimoso do Sul', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3144, 'Montanha', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3145, 'Mucurici', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3146, 'Muniz Freire', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3147, 'Muqui', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3148, 'Nova Venecia', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3149, 'Pancas', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3150, 'Pedro Canario', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3151, 'Pinheiros', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3152, 'Piuma', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3153, 'Ponto Belo', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3154, 'Presidente Kennedy', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3155, 'Rio Bananal', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3156, 'Rio Novo do Sul', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3157, 'Santa Leopoldina', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3158, 'Santa Maria de Jetiba', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3159, 'Santa Teresa', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3160, 'Sao Domingos do Norte', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3161, 'Sao Gabriel da Palha', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3162, 'Sao Jose do Calcado', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3163, 'Sao Mateus', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3164, 'Sao Roque do Canaa', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3165, 'Serra', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3166, 'Sooretama', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3167, 'Vargem Alta', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3168, 'Venda Nova do Imigrante', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3169, 'Viana', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3170, 'Vila Pavao', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3171, 'Vila Valerio', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3172, 'Vila Velha', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3173, 'Vitoria', 'ES', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3174, 'Angra dos Reis', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3175, 'Aperibe', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3176, 'Araruama', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3177, 'Areal', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3178, 'Armacao dos Buzios', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3179, 'Arraial do Cabo', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3180, 'Barra do Pirai', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3181, 'Barra Mansa', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3182, 'Belford Roxo', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3183, 'Bom Jardim', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3184, 'Bom Jesus do Itabapoana', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3185, 'Cabo Frio', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3186, 'Cachoeiras de Macacu', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3187, 'Cambuci', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3188, 'Campos dos Goytacazes', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3189, 'Cantagalo', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3190, 'Carapebus', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3191, 'Cardoso Moreira', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3192, 'Carmo', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3193, 'Casimiro de Abreu', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3194, 'Comendador Levy Gasparian', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3195, 'Conceicao de Macabu', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3196, 'Cordeiro', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3197, 'Duas Barras', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3198, 'Duque de Caxias', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3199, 'Engenheiro Paulo de Frontin', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3200, 'Guapimirim', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3201, 'Iguaba Grande', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3202, 'Itaborai', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3203, 'Itaguai', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3204, 'Italva', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3205, 'Itaocara', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3206, 'Itaperuna', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3207, 'Itatiaia', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3208, 'Japeri', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3209, 'Laje do Muriae', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3210, 'Macae', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3211, 'Macuco', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3212, 'Mage', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3213, 'Mangaratiba', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3214, 'Marica', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3215, 'Mendes', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3216, 'Mesquita', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3217, 'Miguel Pereira', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3218, 'Miracema', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3219, 'Natividade', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3220, 'Nilopolis', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3221, 'Niteroi', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3222, 'Nova Friburgo', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3223, 'Nova Iguacu', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3224, 'Paracambi', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3225, 'Paraiba do Sul', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3226, 'Parati', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3227, 'Paty do Alferes', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3228, 'Petropolis', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3229, 'Pinheiral', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3230, 'Pirai', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3231, 'Porciuncula', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3232, 'Porto Real', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3233, 'Quatis', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3234, 'Queimados', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3235, 'Quissama', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3236, 'Resende', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3237, 'Rio Bonito', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3238, 'Rio Claro', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3239, 'Rio das Flores', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3240, 'Rio das Ostras', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3241, 'Rio de Janeiro', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3242, 'Santa Maria Madalena', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3243, 'Santo Antonio de Padua', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3244, 'Sao Fidelis', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3245, 'Sao Francisco de Itabapoana', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3246, 'Sao Goncalo', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3247, 'Sao Joao da Barra', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3248, 'Sao Joao de Meriti', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3249, 'Sao Jose de Uba', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3250, 'Sao Jose do Vale do Rio Preto', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3251, 'Sao Pedro da Aldeia', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3252, 'Sao Sebastiao do Alto', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3253, 'Sapucaia', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3254, 'Saquarema', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3255, 'Seropedica', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3256, 'Silva Jardim', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3257, 'Sumidouro', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3258, 'Tangua', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3259, 'Teresopolis', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3260, 'Trajano de Morais', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3261, 'Tres Rios', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3262, 'Valenca', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3263, 'Varre-sai', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3264, 'Vassouras', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3265, 'Volta Redonda', 'RJ', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3266, 'Adamantina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3267, 'Adolfo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3268, 'Aguai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3269, 'Aguas da Prata', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3270, 'Aguas de Lindoia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3271, 'Aguas de Santa Barbara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3272, 'Aguas de Sao Pedro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3273, 'Agudos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3274, 'Alambari', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3275, 'Alfredo Marcondes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3276, 'Altair', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3277, 'Altinopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3278, 'Alto Alegre', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3279, 'Aluminio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3280, 'Alvares Florence', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3281, 'Alvares Machado', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3282, 'Alvaro de Carvalho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3283, 'Alvinlandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3284, 'Americana', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3285, 'Americo Brasiliense', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3286, 'Americo de Campos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3287, 'Amparo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3288, 'Analandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3289, 'Andradina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3290, 'Angatuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3291, 'Anhembi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3292, 'Anhumas', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3293, 'Aparecida', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3294, 'Aparecida D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3295, 'Apiai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3296, 'Aracariguama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3297, 'Aracatuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3298, 'Aracoiaba da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3299, 'Aramina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3300, 'Arandu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3301, 'Arapei', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3302, 'Araraquara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3303, 'Araras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3304, 'Arco-iris', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3305, 'Arealva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3306, 'Areias', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3307, 'Areiopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3308, 'Ariranha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3309, 'Artur Nogueira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3310, 'Aruja', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3311, 'Aspasia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3312, 'Assis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3313, 'Atibaia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3314, 'Auriflama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3315, 'Avai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3316, 'Avanhandava', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3317, 'Avare', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3318, 'Bady Bassitt', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3319, 'Balbinos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3320, 'Balsamo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3321, 'Bananal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3322, 'Barao de Antonina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3323, 'Barbosa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3324, 'Bariri', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3325, 'Barra Bonita', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3326, 'Barra do Chapeu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3327, 'Barra do Turvo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3328, 'Barretos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3329, 'Barrinha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3330, 'Barueri', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3331, 'Bastos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3332, 'Batatais', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3333, 'Bauru', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3334, 'Bebedouro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3335, 'Bento de Abreu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3336, 'Bernardino de Campos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3337, 'Bertioga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3338, 'Bilac', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3339, 'Birigui', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3340, 'Biritiba-mirim', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3341, 'Boa Esperanca do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3342, 'Bocaina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3343, 'Bofete', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3344, 'Boituva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3345, 'Bom Jesus dos Perdoes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3346, 'Bom Sucesso de Itarare', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3347, 'Bora', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3348, 'Boraceia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3349, 'Borborema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3350, 'Borebi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3351, 'Botucatu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3352, 'Braganca Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3353, 'Brauna', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3354, 'Brejo Alegre', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3355, 'Brodowski', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3356, 'Brotas', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3357, 'Buri', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3358, 'Buritama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3359, 'Buritizal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3360, 'Cabralia Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3361, 'Cabreuva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3362, 'Cacapava', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3363, 'Cachoeira Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3364, 'Caconde', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3365, 'Cafelandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3366, 'Caiabu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3367, 'Caieiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3368, 'Caiua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3369, 'Cajamar', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3370, 'Cajati', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3371, 'Cajobi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3372, 'Cajuru', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3373, 'Campina do Monte Alegre', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3374, 'Campinas', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3375, 'Campo Limpo Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3376, 'Campos do Jordao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3377, 'Campos Novos Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3378, 'Cananeia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3379, 'Canas', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3380, 'Candido Mota', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3381, 'Candido Rodrigues', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3382, 'Canitar', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3383, 'Capao Bonito', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3384, 'Capela do Alto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3385, 'Capivari', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3386, 'Caraguatatuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3387, 'Carapicuiba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3388, 'Cardoso', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3389, 'Casa Branca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3390, 'Cassia dos Coqueiros', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3391, 'Castilho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3392, 'Catanduva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3393, 'Catigua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3394, 'Cedral', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3395, 'Cerqueira Cesar', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3396, 'Cerquilho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3397, 'Cesario Lange', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3398, 'Charqueada', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3399, 'Chavantes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3400, 'Clementina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3401, 'Colina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3402, 'Colombia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3403, 'Conchal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3404, 'Conchas', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3405, 'Cordeiropolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3406, 'Coroados', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3407, 'Coronel Macedo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3408, 'Corumbatai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3409, 'Cosmopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3410, 'Cosmorama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3411, 'Cotia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3412, 'Cravinhos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3413, 'Cristais Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3414, 'Cruzalia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3415, 'Cruzeiro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3416, 'Cubatao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3417, 'Cunha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3418, 'Descalvado', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3419, 'Diadema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3420, 'Dirce Reis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3421, 'Divinolandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3422, 'Dobrada', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3423, 'Dois Corregos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3424, 'Dolcinopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3425, 'Dourado', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3426, 'Dracena', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3427, 'Duartina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3428, 'Dumont', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3429, 'Echapora', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3430, 'Eldorado', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3431, 'Elias Fausto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3432, 'Elisiario', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3433, 'Embauba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3434, 'Embu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3435, 'Embu-guacu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3436, 'Emilianopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3437, 'Engenheiro Coelho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3438, 'Espirito Santo do Pinhal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3439, 'Espirito Santo do Turvo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3440, 'Estiva Gerbi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3441, 'Estrela do Norte', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3442, 'Estrela D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3443, 'Euclides da Cunha Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3444, 'Fartura', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3445, 'Fernando Prestes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3446, 'Fernandopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3447, 'Fernao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3448, 'Ferraz de Vasconcelos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3449, 'Flora Rica', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3450, 'Floreal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3451, 'Florida Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3452, 'Florinia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3453, 'Franca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3454, 'Francisco Morato', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3455, 'Franco da Rocha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3456, 'Gabriel Monteiro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3457, 'Galia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3458, 'Garca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3459, 'Gastao Vidigal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3460, 'Gaviao Peixoto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3461, 'General Salgado', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3462, 'Getulina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3463, 'Glicerio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3464, 'Guaicara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3465, 'Guaimbe', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3466, 'Guaira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3467, 'Guapiacu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3468, 'Guapiara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3469, 'Guara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3470, 'Guaracai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3471, 'Guaraci', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3472, 'Guarani D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3473, 'Guaranta', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3474, 'Guararapes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3475, 'Guararema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3476, 'Guaratingueta', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3477, 'Guarei', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3478, 'Guariba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3479, 'Guaruja', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3480, 'Guarulhos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3481, 'Guatapara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3482, 'Guzolandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3483, 'Herculandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3484, 'Holambra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3485, 'Hortolandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3486, 'Iacanga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3487, 'Iacri', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3488, 'Iaras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3489, 'Ibate', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3490, 'Ibira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3491, 'Ibirarema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3492, 'Ibitinga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3493, 'Ibiuna', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3494, 'Icem', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3495, 'Iepe', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3496, 'Igaracu do Tiete', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3497, 'Igarapava', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3498, 'Igarata', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3499, 'Iguape', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3500, 'Ilha Comprida', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3501, 'Ilha Solteira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3502, 'Ilhabela', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3503, 'Indaiatuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3504, 'Indiana', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3505, 'Indiapora', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3506, 'Inubia Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3507, 'Ipaussu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3508, 'Ipero', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3509, 'Ipeuna', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3510, 'Ipigua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3511, 'Iporanga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3512, 'Ipua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3513, 'Iracemapolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3514, 'Irapua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3515, 'Irapuru', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3516, 'Itabera', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3517, 'Itai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3518, 'Itajobi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3519, 'Itaju', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3520, 'Itanhaem', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3521, 'Itaoca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3522, 'Itapecerica da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3523, 'Itapetininga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3524, 'Itapeva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3525, 'Itapevi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3526, 'Itapira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3527, 'Itapirapua Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3528, 'Itapolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3529, 'Itaporanga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3530, 'Itapui', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3531, 'Itapura', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3532, 'Itaquaquecetuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3533, 'Itarare', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3534, 'Itariri', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3535, 'Itatiba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3536, 'Itatinga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3537, 'Itirapina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3538, 'Itirapua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3539, 'Itobi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3540, 'Itu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3541, 'Itupeva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3542, 'Ituverava', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3543, 'Jaborandi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3544, 'Jaboticabal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3545, 'Jacarei', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3546, 'Jaci', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3547, 'Jacupiranga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3548, 'Jaguariuna', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3549, 'Jales', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3550, 'Jambeiro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3551, 'Jandira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3552, 'Jardinopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3553, 'Jarinu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3554, 'Jau', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3555, 'Jeriquara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3556, 'Joanopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3557, 'Joao Ramalho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3558, 'Jose Bonifacio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3559, 'Julio Mesquita', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3560, 'Jumirim', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3561, 'Jundiai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3562, 'Junqueiropolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3563, 'Juquia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3564, 'Juquitiba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3565, 'Lagoinha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5511, 'Quirinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3566, 'Laranjal Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3567, 'Lavinia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3568, 'Lavrinhas', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3569, 'Leme', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3570, 'Lencois Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3571, 'Limeira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3572, 'Lindoia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3573, 'Lins', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3574, 'Lorena', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3575, 'Lourdes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3576, 'Louveira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3577, 'Lucelia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3578, 'Lucianopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3579, 'Luis Antonio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3580, 'Luiziania', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3581, 'Lupercio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3582, 'Lutecia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3583, 'Macatuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3584, 'Macaubal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3585, 'Macedonia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3586, 'Magda', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3587, 'Mairinque', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3588, 'Mairipora', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3589, 'Manduri', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3590, 'Maraba Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3591, 'Maracai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3592, 'Marapoama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3593, 'Mariapolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3594, 'Marilia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3595, 'Marinopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3596, 'Martinopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3597, 'Matao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3598, 'Maua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3599, 'Mendonca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3600, 'Meridiano', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3601, 'Mesopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3602, 'Miguelopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3603, 'Mineiros do Tiete', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3604, 'Mira Estrela', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3605, 'Miracatu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3606, 'Mirandopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3607, 'Mirante do Paranapanema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3608, 'Mirassol', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3609, 'Mirassolandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3610, 'Mococa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3611, 'Mogi das Cruzes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3612, 'Mogi Guacu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3613, 'Mogi Mirim', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3614, 'Mombuca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3615, 'Moncoes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3616, 'Mongagua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3617, 'Monte Alegre do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3618, 'Monte Alto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3619, 'Monte Aprazivel', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3620, 'Monte Azul Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3621, 'Monte Castelo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3622, 'Monte Mor', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3623, 'Monteiro Lobato', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3624, 'Morro Agudo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3625, 'Morungaba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3626, 'Motuca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3627, 'Murutinga do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3628, 'Nantes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3629, 'Narandiba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3630, 'Natividade da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3631, 'Nazare Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3632, 'Neves Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3633, 'Nhandeara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3634, 'Nipoa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3635, 'Nova Alianca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3636, 'Nova Campina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3637, 'Nova Canaa Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3638, 'Nova Castilho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3639, 'Nova Europa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3640, 'Nova Granada', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3641, 'Nova Guataporanga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3642, 'Nova Independencia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3643, 'Nova Luzitania', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3644, 'Nova Odessa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3645, 'Novais', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3646, 'Novo Horizonte', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3647, 'Nuporanga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3648, 'Ocaucu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3649, 'Oleo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3650, 'Olimpia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3651, 'Onda Verde', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3652, 'Oriente', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3653, 'Orindiuva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3654, 'Orlandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3655, 'Osasco', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3656, 'Oscar Bressane', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3657, 'Osvaldo Cruz', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3658, 'Ourinhos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3659, 'Ouro Verde', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3660, 'Ouroeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3661, 'Pacaembu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3662, 'Palestina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3663, 'Palmares Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3664, 'Palmeira D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3665, 'Palmital', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3666, 'Panorama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3667, 'Paraguacu Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3668, 'Paraibuna', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3669, 'Paraiso', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3670, 'Paranapanema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3671, 'Paranapua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3672, 'Parapua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3673, 'Pardinho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3674, 'Pariquera-acu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3675, 'Parisi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3676, 'Patrocinio Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3677, 'Pauliceia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3678, 'Paulinia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3679, 'Paulistania', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3680, 'Paulo de Faria', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3681, 'Pederneiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3682, 'Pedra Bela', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3683, 'Pedranopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3684, 'Pedregulho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3685, 'Pedreira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3686, 'Pedrinhas Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3687, 'Pedro de Toledo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3688, 'Penapolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3689, 'Pereira Barreto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3690, 'Pereiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3691, 'Peruibe', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3692, 'Piacatu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3693, 'Piedade', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3694, 'Pilar do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3695, 'Pindamonhangaba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3696, 'Pindorama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3697, 'Pinhalzinho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3698, 'Piquerobi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3699, 'Piquete', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3700, 'Piracaia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3701, 'Piracicaba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3702, 'Piraju', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3703, 'Pirajui', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3704, 'Pirangi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3705, 'Pirapora do Bom Jesus', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3706, 'Pirapozinho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3707, 'Pirassununga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3708, 'Piratininga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3709, 'Pitangueiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3710, 'Planalto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3711, 'Platina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3712, 'Poa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3713, 'Poloni', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3714, 'Pompeia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3715, 'Pongai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3716, 'Pontal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3717, 'Pontalinda', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3718, 'Pontes Gestal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3719, 'Populina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3720, 'Porangaba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3721, 'Porto Feliz', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3722, 'Porto Ferreira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3723, 'Potim', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3724, 'Potirendaba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3725, 'Pracinha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3726, 'Pradopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3727, 'Praia Grande', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3728, 'Pratania', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3729, 'Presidente Alves', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3730, 'Presidente Bernardes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3731, 'Presidente Epitacio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3732, 'Presidente Prudente', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3733, 'Presidente Venceslau', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3734, 'Promissao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3735, 'Quadra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3736, 'Quata', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3737, 'Queiroz', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3738, 'Queluz', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3739, 'Quintana', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3740, 'Rafard', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3741, 'Rancharia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3742, 'Redencao da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3743, 'Regente Feijo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3744, 'Reginopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3745, 'Registro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3746, 'Restinga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3747, 'Ribeira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3748, 'Ribeirao Bonito', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3749, 'Ribeirao Branco', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3750, 'Ribeirao Corrente', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3751, 'Ribeirao do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3752, 'Ribeirao dos Indios', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3753, 'Ribeirao Grande', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3754, 'Ribeirao Pires', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3755, 'Ribeirao Preto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3756, 'Rifaina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3757, 'Rincao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3758, 'Rinopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3759, 'Rio Claro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3760, 'Rio das Pedras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3761, 'Rio Grande da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3762, 'Riolandia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3763, 'Riversul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3764, 'Rosana', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3765, 'Roseira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3766, 'Rubiacea', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3767, 'Rubineia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3768, 'Sabino', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3769, 'Sagres', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3770, 'Sales', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3771, 'Sales Oliveira', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3772, 'Salesopolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3773, 'Salmourao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3774, 'Saltinho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3775, 'Salto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3776, 'Salto de Pirapora', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3777, 'Salto Grande', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3778, 'Sandovalina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3779, 'Santa Adelia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3780, 'Santa Albertina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3781, 'Santa Barbara D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3782, 'Santa Branca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3783, 'Santa Clara D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3784, 'Santa Cruz da Conceicao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3785, 'Santa Cruz da Esperanca', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3786, 'Santa Cruz das Palmeiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3787, 'Santa Cruz do Rio Pardo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3788, 'Santa Ernestina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3789, 'Santa Fe do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3790, 'Santa Gertrudes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3791, 'Santa Isabel', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3792, 'Santa Lucia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3793, 'Santa Maria da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3794, 'Santa Mercedes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3795, 'Santa Rita do Passa Quatro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3796, 'Santa Rita D''oeste', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3797, 'Santa Rosa de Viterbo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3798, 'Santa Salete', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3799, 'Santana da Ponte Pensa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3800, 'Santana de Parnaiba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3801, 'Santo Anastacio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3802, 'Santo Andre', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3803, 'Santo Antonio da Alegria', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3804, 'Santo Antonio de Posse', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3805, 'Santo Antonio do Aracangua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3806, 'Santo Antonio do Jardim', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3807, 'Santo Antonio do Pinhal', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3808, 'Santo Expedito', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3809, 'Santopolis do Aguapei', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3810, 'Santos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3811, 'Sao Bento do Sapucai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3812, 'Sao Bernardo do Campo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3813, 'Sao Caetano do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3814, 'Sao Carlos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3815, 'Sao Francisco', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3816, 'Sao Joao da Boa Vista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3817, 'Sao Joao das Duas Pontes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3818, 'Sao Joao de Iracema', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3819, 'Sao Joao do Pau D''alho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3820, 'Sao Joaquim da Barra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3821, 'Sao Jose da Bela Vista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3822, 'Sao Jose do Barreiro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3823, 'Sao Jose do Rio Pardo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3824, 'Sao Jose do Rio Preto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3825, 'Sao Jose dos Campos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3826, 'Sao Lourenco da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3827, 'Sao Luis do Paraitinga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3828, 'Sao Manuel', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3829, 'Sao Miguel Arcanjo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3830, 'Sao Paulo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3831, 'Sao Pedro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3832, 'Sao Pedro do Turvo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3833, 'Sao Roque', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3834, 'Sao Sebastiao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3835, 'Sao Sebastiao da Grama', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3836, 'Sao Simao', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3837, 'Sao Vicente', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3838, 'Sarapui', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3839, 'Sarutaia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3840, 'Sebastianopolis do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3841, 'Serra Azul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3842, 'Serra Negra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3843, 'Serrana', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3844, 'Sertaozinho', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3845, 'Sete Barras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3846, 'Severinia', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3847, 'Silveiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3848, 'Socorro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3849, 'Sorocaba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3850, 'Sud Mennucci', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3851, 'Sumare', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3852, 'Suzanapolis', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3853, 'Suzano', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3854, 'Tabapua', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3855, 'Tabatinga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3856, 'Taboao da Serra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3857, 'Taciba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3858, 'Taguai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3859, 'Taiacu', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3860, 'Taiuva', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3861, 'Tambau', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3862, 'Tanabi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3863, 'Tapirai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3864, 'Tapiratiba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3865, 'Taquaral', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3866, 'Taquaritinga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3867, 'Taquarituba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3868, 'Taquarivai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3869, 'Tarabai', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3870, 'Taruma', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3871, 'Tatui', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3872, 'Taubate', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3873, 'Tejupa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3874, 'Teodoro Sampaio', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3875, 'Terra Roxa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3876, 'Tiete', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3877, 'Timburi', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3878, 'Torre de Pedra', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3879, 'Torrinha', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3880, 'Trabiju', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3881, 'Tremembe', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3882, 'Tres Fronteiras', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3883, 'Tuiuti', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3884, 'Tupa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3885, 'Tupi Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3886, 'Turiuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3887, 'Turmalina', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3888, 'Ubarana', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3889, 'Ubatuba', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3890, 'Ubirajara', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3891, 'Uchoa', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3892, 'Uniao Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3893, 'Urania', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3894, 'Uru', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3895, 'Urupes', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3896, 'Valentim Gentil', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3897, 'Valinhos', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3898, 'Valparaiso', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3899, 'Vargem', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3900, 'Vargem Grande do Sul', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3901, 'Vargem Grande Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3902, 'Varzea Paulista', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3903, 'Vera Cruz', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3904, 'Vinhedo', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3905, 'Viradouro', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3906, 'Vista Alegre do Alto', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3907, 'Vitoria Brasil', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3908, 'Votorantim', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3909, 'Votuporanga', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3910, 'Zacarias', 'SP', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3911, 'Abatia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3912, 'Adrianopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3913, 'Agudos do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3914, 'Almirante Tamandare', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3915, 'Altamira do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3916, 'Alto Paraiso', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3917, 'Alto Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3918, 'Alto Piquiri', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3919, 'Altonia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3920, 'Alvorada do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3921, 'Amapora', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3922, 'Ampere', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3923, 'Anahy', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3924, 'Andira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3925, 'Angulo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3926, 'Antonina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3927, 'Antonio Olinto', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3928, 'Apucarana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3929, 'Arapongas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3930, 'Arapoti', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3931, 'Arapua', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3932, 'Araruna', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3933, 'Araucaria', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3934, 'Ariranha do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3935, 'Assai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3936, 'Assis Chateaubriand', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3937, 'Astorga', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3938, 'Atalaia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3939, 'Balsa Nova', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3940, 'Bandeirantes', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3941, 'Barbosa Ferraz', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3942, 'Barra do Jacare', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3943, 'Barracao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3944, 'Bela Vista do Caroba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3945, 'Bela Vista do Paraiso', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3946, 'Bituruna', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3947, 'Boa Esperanca', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3948, 'Boa Esperanca do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3949, 'Boa Ventura de Sao Roque', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3950, 'Boa Vista da Aparecida', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3951, 'Bocaiuva do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3952, 'Bom Jesus do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3953, 'Bom Sucesso', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3954, 'Bom Sucesso do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3955, 'Borrazopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3956, 'Braganey', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3957, 'Brasilandia do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3958, 'Cafeara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3959, 'Cafelandia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3960, 'Cafezal do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3961, 'California', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3962, 'Cambara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3963, 'Cambe', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3964, 'Cambira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3965, 'Campina da Lagoa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3966, 'Campina do Simao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3967, 'Campina Grande do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3968, 'Campo Bonito', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3969, 'Campo do Tenente', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3970, 'Campo Largo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3971, 'Campo Magro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3972, 'Campo Mourao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3973, 'Candido de Abreu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3974, 'Candoi', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3975, 'Cantagalo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3976, 'Capanema', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3977, 'Capitao Leonidas Marques', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3978, 'Carambei', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3979, 'Carlopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3980, 'Cascavel', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3981, 'Castro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3982, 'Catanduvas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3983, 'Centenario do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3984, 'Cerro Azul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3985, 'Ceu Azul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3986, 'Chopinzinho', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3987, 'Cianorte', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3988, 'Cidade Gaucha', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3989, 'Clevelandia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3990, 'Colombo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3991, 'Colorado', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3992, 'Congonhinhas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3993, 'Conselheiro Mairinck', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3994, 'Contenda', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3995, 'Corbelia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3996, 'Cornelio Procopio', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3997, 'Coronel Domingos Soares', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3998, 'Coronel Vivida', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (3999, 'Corumbatai do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4000, 'Cruz Machado', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4001, 'Cruzeiro do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4002, 'Cruzeiro do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4003, 'Cruzeiro do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4004, 'Cruzmaltina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4005, 'Curitiba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4006, 'Curiuva', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4007, 'Diamante do Norte', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4008, 'Diamante do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4009, 'Diamante D''oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4010, 'Dois Vizinhos', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4011, 'Douradina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4012, 'Doutor Camargo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4013, 'Doutor Ulysses', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4014, 'Eneas Marques', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4015, 'Engenheiro Beltrao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4016, 'Entre Rios do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4017, 'Esperanca Nova', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4018, 'Espigao Alto do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4019, 'Farol', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4020, 'Faxinal', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4021, 'Fazenda Rio Grande', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4022, 'Fenix', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4023, 'Fernandes Pinheiro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4024, 'Figueira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4025, 'Flor da Serra do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4026, 'Florai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4027, 'Floresta', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4028, 'Florestopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4029, 'Florida', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4030, 'Formosa do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4031, 'Foz do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4032, 'Foz do Jordao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4033, 'Francisco Alves', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4034, 'Francisco Beltrao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4035, 'General Carneiro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4036, 'Godoy Moreira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4037, 'Goio-ere', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4038, 'Goioxim', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4039, 'Grandes Rios', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4040, 'Guaira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4041, 'Guairaca', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4042, 'Guamiranga', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4043, 'Guapirama', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4044, 'Guaporema', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4045, 'Guaraci', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4046, 'Guaraniacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4047, 'Guarapuava', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4048, 'Guaraquecaba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4049, 'Guaratuba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4050, 'Honorio Serpa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4051, 'Ibaiti', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4052, 'Ibema', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4053, 'Ibipora', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4054, 'Icaraima', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4055, 'Iguaracu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4056, 'Iguatu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4057, 'Imbau', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4058, 'Imbituva', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4059, 'Inacio Martins', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4060, 'Inaja', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4061, 'Indianopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4062, 'Ipiranga', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4063, 'Ipora', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4064, 'Iracema do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4065, 'Irati', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4066, 'Iretama', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4067, 'Itaguaje', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4068, 'Itaipulandia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4069, 'Itambaraca', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4070, 'Itambe', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4071, 'Itapejara D''oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4072, 'Itaperucu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4073, 'Itauna do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4074, 'Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4075, 'Ivaipora', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4076, 'Ivate', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4077, 'Ivatuba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4078, 'Jaboti', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4079, 'Jacarezinho', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4080, 'Jaguapita', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4081, 'Jaguariaiva', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4082, 'Jandaia do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4083, 'Janiopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4084, 'Japira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4085, 'Japura', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4086, 'Jardim Alegre', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4087, 'Jardim Olinda', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4088, 'Jataizinho', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4089, 'Jesuitas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4090, 'Joaquim Tavora', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4091, 'Jundiai do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4092, 'Juranda', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4093, 'Jussara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4094, 'Kalore', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4095, 'Lapa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4096, 'Laranjal', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4097, 'Laranjeiras do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4098, 'Leopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4099, 'Lidianopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4100, 'Lindoeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4101, 'Loanda', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4102, 'Lobato', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4103, 'Londrina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4104, 'Luiziana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4105, 'Lunardelli', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4106, 'Lupionopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4107, 'Mallet', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4108, 'Mambore', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4109, 'Mandaguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4110, 'Mandaguari', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4111, 'Mandirituba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4112, 'Manfrinopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4113, 'Mangueirinha', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4114, 'Manoel Ribas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4115, 'Marechal Candido Rondon', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4116, 'Maria Helena', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4117, 'Marialva', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4118, 'Marilandia do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4119, 'Marilena', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4120, 'Mariluz', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4121, 'Maringa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4122, 'Mariopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4123, 'Maripa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4124, 'Marmeleiro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4125, 'Marquinho', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4126, 'Marumbi', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4127, 'Matelandia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4128, 'Matinhos', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4129, 'Mato Rico', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4130, 'Maua da Serra', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4131, 'Medianeira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4132, 'Mercedes', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4133, 'Mirador', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4134, 'Miraselva', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4135, 'Missal', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4136, 'Moreira Sales', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4137, 'Morretes', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4138, 'Munhoz de Melo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4139, 'Nossa Senhora das Gracas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4140, 'Nova Alianca do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4141, 'Nova America da Colina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4142, 'Nova Aurora', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4143, 'Nova Cantu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4144, 'Nova Esperanca', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4145, 'Nova Esperanca do Sudoeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4146, 'Nova Fatima', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4147, 'Nova Laranjeiras', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4148, 'Nova Londrina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4149, 'Nova Olimpia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4150, 'Nova Prata do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4151, 'Nova Santa Barbara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4152, 'Nova Santa Rosa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4153, 'Nova Tebas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4154, 'Novo Itacolomi', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4155, 'Ortigueira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4156, 'Ourizona', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4157, 'Ouro Verde do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4158, 'Paicandu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4159, 'Palmas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4160, 'Palmeira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4161, 'Palmital', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4162, 'Palotina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4163, 'Paraiso do Norte', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4164, 'Paranacity', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4165, 'Paranagua', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4166, 'Paranapoema', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4167, 'Paranavai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4168, 'Pato Bragado', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4169, 'Pato Branco', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4170, 'Paula Freitas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4171, 'Paulo Frontin', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4172, 'Peabiru', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4173, 'Perobal', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4174, 'Perola', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4175, 'Perola D''oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4176, 'Pien', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4177, 'Pinhais', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4178, 'Pinhal de Sao Bento', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4179, 'Pinhalao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4180, 'Pinhao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4181, 'Pirai do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4182, 'Piraquara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4183, 'Pitanga', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4184, 'Pitangueiras', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4185, 'Planaltina do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4186, 'Planalto', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4187, 'Ponta Grossa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4188, 'Pontal do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4189, 'Porecatu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4190, 'Porto Amazonas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4191, 'Porto Barreiro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4192, 'Porto Rico', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4193, 'Porto Vitoria', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4194, 'Prado Ferreira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4195, 'Pranchita', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4196, 'Presidente Castelo Branco', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4197, 'Primeiro de Maio', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4198, 'Prudentopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4199, 'Quarto Centenario', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4200, 'Quatigua', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4201, 'Quatro Barras', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4202, 'Quatro Pontes', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4203, 'Quedas do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4204, 'Querencia do Norte', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4205, 'Quinta do Sol', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4206, 'Quitandinha', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4207, 'Ramilandia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4208, 'Rancho Alegre', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4209, 'Rancho Alegre D''oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4210, 'Realeza', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4211, 'Reboucas', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4212, 'Renascenca', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4213, 'Reserva', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4214, 'Reserva do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4215, 'Ribeirao Claro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4216, 'Ribeirao do Pinhal', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4217, 'Rio Azul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4218, 'Rio Bom', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4219, 'Rio Bonito do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4220, 'Rio Branco do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4221, 'Rio Branco do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4222, 'Rio Negro', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4223, 'Rolandia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4224, 'Roncador', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4225, 'Rondon', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4226, 'Rosario do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4227, 'Sabaudia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4228, 'Salgado Filho', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4229, 'Salto do Itarare', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4230, 'Salto do Lontra', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4231, 'Santa Amelia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4232, 'Santa Cecilia do Pavao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4233, 'Santa Cruz de Monte Castelo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4234, 'Santa Fe', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4235, 'Santa Helena', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4236, 'Santa Ines', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4237, 'Santa Isabel do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4238, 'Santa Izabel do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4239, 'Santa Lucia', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4240, 'Santa Maria do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4241, 'Santa Mariana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4242, 'Santa Monica', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4243, 'Santa Tereza do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4244, 'Santa Terezinha de Itaipu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4245, 'Santana do Itarare', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4246, 'Santo Antonio da Platina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4247, 'Santo Antonio do Caiua', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4248, 'Santo Antonio do Paraiso', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4249, 'Santo Antonio do Sudoeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4250, 'Santo Inacio', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4251, 'Sao Carlos do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4252, 'Sao Jeronimo da Serra', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4253, 'Sao Joao', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4254, 'Sao Joao do Caiua', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4255, 'Sao Joao do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4256, 'Sao Joao do Triunfo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4257, 'Sao Jorge do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4258, 'Sao Jorge do Patrocinio', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4259, 'Sao Jorge D''oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4260, 'Sao Jose da Boa Vista', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4261, 'Sao Jose das Palmeiras', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4262, 'Sao Jose dos Pinhais', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4263, 'Sao Manoel do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4264, 'Sao Mateus do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4265, 'Sao Miguel do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4266, 'Sao Pedro do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4267, 'Sao Pedro do Ivai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4268, 'Sao Pedro do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4269, 'Sao Sebastiao da Amoreira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4270, 'Sao Tome', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4271, 'Sapopema', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4272, 'Sarandi', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4273, 'Saudade do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4274, 'Senges', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4275, 'Serranopolis do Iguacu', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4276, 'Sertaneja', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4277, 'Sertanopolis', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4278, 'Siqueira Campos', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4279, 'Sulina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4280, 'Tamarana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4281, 'Tamboara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4282, 'Tapejara', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4283, 'Tapira', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4284, 'Teixeira Soares', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4285, 'Telemaco Borba', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4286, 'Terra Boa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4287, 'Terra Rica', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4288, 'Terra Roxa', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4289, 'Tibagi', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4290, 'Tijucas do Sul', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4291, 'Toledo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4292, 'Tomazina', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4293, 'Tres Barras do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4294, 'Tunas do Parana', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4295, 'Tuneiras do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4296, 'Tupassi', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4297, 'Turvo', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4298, 'Ubirata', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4299, 'Umuarama', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4300, 'Uniao da Vitoria', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4301, 'Uniflor', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4302, 'Urai', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4303, 'Ventania', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4304, 'Vera Cruz do Oeste', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4305, 'Vere', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4306, 'Virmond', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4307, 'Vitorino', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4308, 'Wenceslau Braz', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4309, 'Xambre', 'PR', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4310, 'Abdon Batista', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4311, 'Abelardo Luz', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4312, 'Agrolandia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4313, 'Agronomica', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4314, 'Agua Doce', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4315, 'Aguas de Chapeco', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4316, 'Aguas Frias', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4317, 'Aguas Mornas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4318, 'Alfredo Wagner', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4319, 'Alto Bela Vista', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4320, 'Anchieta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4321, 'Angelina', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4322, 'Anita Garibaldi', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4323, 'Anitapolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4324, 'Antonio Carlos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4325, 'Apiuna', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4326, 'Arabuta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4327, 'Araquari', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4328, 'Ararangua', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4329, 'Armazem', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4330, 'Arroio Trinta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4331, 'Arvoredo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4332, 'Ascurra', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4333, 'Atalanta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4334, 'Aurora', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4335, 'Balneario Arroio do Silva', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4336, 'Balneario Barra do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4337, 'Balneario Camboriu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4338, 'Balneario Gaivota', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4339, 'Bandeirante', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4340, 'Barra Bonita', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4341, 'Barra Velha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4342, 'Bela Vista do Toldo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4343, 'Belmonte', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4344, 'Benedito Novo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4345, 'Biguacu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4346, 'Blumenau', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4347, 'Bocaina do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4348, 'Bom Jardim da Serra', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4349, 'Bom Jesus', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4350, 'Bom Jesus do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4351, 'Bom Retiro', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4352, 'Bombinhas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4353, 'Botuvera', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4354, 'Braco do Norte', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4355, 'Braco do Trombudo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4356, 'Brunopolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4357, 'Brusque', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4358, 'Cacador', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4359, 'Caibi', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4360, 'Calmon', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4361, 'Camboriu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4362, 'Campo Alegre', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4363, 'Campo Belo do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4364, 'Campo Ere', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4365, 'Campos Novos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4366, 'Canelinha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4367, 'Canoinhas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4368, 'Capao Alto', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4369, 'Capinzal', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4370, 'Capivari de Baixo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4371, 'Catanduvas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4372, 'Caxambu do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4373, 'Celso Ramos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4374, 'Cerro Negro', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4375, 'Chapadao do Lageado', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4376, 'Chapeco', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4377, 'Cocal do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4378, 'Concordia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4379, 'Cordilheira Alta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4380, 'Coronel Freitas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4381, 'Coronel Martins', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4382, 'Correia Pinto', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4383, 'Corupa', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4384, 'Criciuma', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4385, 'Cunha Pora', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4386, 'Cunhatai', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4387, 'Curitibanos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4388, 'Descanso', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4389, 'Dionisio Cerqueira', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4390, 'Dona Emma', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4391, 'Doutor Pedrinho', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4392, 'Entre Rios', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4393, 'Ermo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4394, 'Erval Velho', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4395, 'Faxinal dos Guedes', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4396, 'Flor do Sertao', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4397, 'Florianopolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4398, 'Formosa do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4399, 'Forquilhinha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4400, 'Fraiburgo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4401, 'Frei Rogerio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4402, 'Galvao', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4403, 'Garopaba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4404, 'Garuva', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4405, 'Gaspar', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4406, 'Governador Celso Ramos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4407, 'Grao Para', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4408, 'Gravatal', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4409, 'Guabiruba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4410, 'Guaraciaba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4411, 'Guaramirim', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4412, 'Guaruja do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4413, 'Guatambu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4414, 'Herval D''oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4415, 'Ibiam', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4416, 'Ibicare', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4417, 'Ibirama', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4418, 'Icara', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4419, 'Ilhota', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4420, 'Imarui', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4421, 'Imbituba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4422, 'Imbuia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4423, 'Indaial', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4424, 'Iomere', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4425, 'Ipira', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4426, 'Ipora do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4427, 'Ipuacu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4428, 'Ipumirim', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4429, 'Iraceminha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4430, 'Irani', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4431, 'Irati', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4432, 'Irineopolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4433, 'Ita', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4434, 'Itaiopolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4435, 'Itajai', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4436, 'Itapema', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4437, 'Itapiranga', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4438, 'Itapoa', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4439, 'Ituporanga', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4440, 'Jabora', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4441, 'Jacinto Machado', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4442, 'Jaguaruna', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4443, 'Jaragua do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4444, 'Jardinopolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4445, 'Joacaba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4446, 'Joinville', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4447, 'Jose Boiteux', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4448, 'Jupia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4449, 'Lacerdopolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4450, 'Lages', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4451, 'Laguna', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4452, 'Lajeado Grande', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4453, 'Laurentino', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4454, 'Lauro Muller', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4455, 'Lebon Regis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4456, 'Leoberto Leal', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4457, 'Lindoia do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4458, 'Lontras', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4459, 'Luiz Alves', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4460, 'Luzerna', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4461, 'Macieira', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4462, 'Mafra', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4463, 'Major Gercino', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4464, 'Major Vieira', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4465, 'Maracaja', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4466, 'Maravilha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4467, 'Marema', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4468, 'Massaranduba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4469, 'Matos Costa', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4470, 'Meleiro', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4471, 'Mirim Doce', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4472, 'Modelo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4473, 'Mondai', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4474, 'Monte Carlo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4475, 'Monte Castelo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4476, 'Morro da Fumaca', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4477, 'Morro Grande', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4478, 'Navegantes', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4479, 'Nova Erechim', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4480, 'Nova Itaberaba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4481, 'Nova Trento', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4482, 'Nova Veneza', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4483, 'Novo Horizonte', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4484, 'Orleans', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4485, 'Otacilio Costa', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4486, 'Ouro', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4487, 'Ouro Verde', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4488, 'Paial', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4489, 'Painel', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4490, 'Palhoca', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4491, 'Palma Sola', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4492, 'Palmeira', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4493, 'Palmitos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4494, 'Papanduva', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4495, 'Paraiso', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4496, 'Passo de Torres', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4497, 'Passos Maia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4498, 'Paulo Lopes', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4499, 'Pedras Grandes', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4500, 'Penha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4501, 'Peritiba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4502, 'Petrolandia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4503, 'Picarras', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4504, 'Pinhalzinho', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4505, 'Pinheiro Preto', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4506, 'Piratuba', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4507, 'Planalto Alegre', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4508, 'Pomerode', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4509, 'Ponte Alta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4510, 'Ponte Alta do Norte', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4511, 'Ponte Serrada', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4512, 'Porto Belo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4513, 'Porto Uniao', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4514, 'Pouso Redondo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4515, 'Praia Grande', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4516, 'Presidente Castello Branco', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4517, 'Presidente Getulio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4518, 'Presidente Nereu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4519, 'Princesa', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4520, 'Quilombo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4521, 'Rancho Queimado', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4522, 'Rio das Antas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4523, 'Rio do Campo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4524, 'Rio do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4525, 'Rio do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4526, 'Rio dos Cedros', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4527, 'Rio Fortuna', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4528, 'Rio Negrinho', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4529, 'Rio Rufino', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4530, 'Riqueza', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4531, 'Rodeio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4532, 'Romelandia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4533, 'Salete', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4534, 'Saltinho', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4535, 'Salto Veloso', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4536, 'Sangao', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4537, 'Santa Cecilia', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4538, 'Santa Helena', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4539, 'Santa Rosa de Lima', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4540, 'Santa Rosa do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4541, 'Santa Terezinha', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4542, 'Santa Terezinha do Progresso', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4543, 'Santiago do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4544, 'Santo Amaro da Imperatriz', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4545, 'Sao Bento do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4546, 'Sao Bernardino', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4547, 'Sao Bonifacio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4548, 'Sao Carlos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4549, 'Sao Cristovao do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4550, 'Sao Domingos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4551, 'Sao Francisco do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4552, 'Sao Joao Batista', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4553, 'Sao Joao do Itaperiu', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4554, 'Sao Joao do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4555, 'Sao Joao do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4556, 'Sao Joaquim', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4557, 'Sao Jose', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4558, 'Sao Jose do Cedro', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4559, 'Sao Jose do Cerrito', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4560, 'Sao Lourenco do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4561, 'Sao Ludgero', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4562, 'Sao Martinho', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4563, 'Sao Miguel da Boa Vista', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4564, 'Sao Miguel do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4565, 'Sao Pedro de Alcantara', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4566, 'Saudades', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4567, 'Schroeder', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4568, 'Seara', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4569, 'Serra Alta', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4570, 'Sideropolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4571, 'Sombrio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4572, 'Sul Brasil', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4573, 'Taio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4574, 'Tangara', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4575, 'Tigrinhos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4576, 'Tijucas', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4577, 'Timbe do Sul', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4578, 'Timbo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4579, 'Timbo Grande', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4580, 'Tres Barras', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4581, 'Treviso', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4582, 'Treze de Maio', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4583, 'Treze Tilias', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4584, 'Trombudo Central', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4585, 'Tubarao', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4586, 'Tunapolis', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4587, 'Turvo', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4588, 'Uniao do Oeste', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4589, 'Urubici', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4590, 'Urupema', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4591, 'Urussanga', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4592, 'Vargeao', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4593, 'Vargem', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4594, 'Vargem Bonita', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4595, 'Vidal Ramos', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4596, 'Videira', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4597, 'Vitor Meireles', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4598, 'Witmarsum', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4599, 'Xanxere', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4600, 'Xavantina', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4601, 'Xaxim', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4602, 'Zortea', 'SC', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4603, 'Acegua', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4604, 'Agua Santa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4605, 'Agudo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4606, 'Ajuricaba', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4607, 'Alecrim', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4608, 'Alegrete', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4609, 'Alegria', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4610, 'Almirante Tamandare do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4611, 'Alpestre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4612, 'Alto Alegre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4613, 'Alto Feliz', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4614, 'Alvorada', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4615, 'Amaral Ferrador', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4616, 'Ametista do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4617, 'Andre da Rocha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4618, 'Anta Gorda', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4619, 'Antonio Prado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4620, 'Arambare', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4621, 'Ararica', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4622, 'Aratiba', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4623, 'Arroio do Meio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4624, 'Arroio do Padre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4625, 'Arroio do Sal', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4626, 'Arroio do Tigre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4627, 'Arroio dos Ratos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4628, 'Arroio Grande', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4629, 'Arvorezinha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4630, 'Augusto Pestana', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4631, 'Aurea', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4632, 'Bage', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4633, 'Balneario Pinhal', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4634, 'Barao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4635, 'Barao de Cotegipe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4636, 'Barao do Triunfo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4637, 'Barra do Guarita', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4638, 'Barra do Quarai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4639, 'Barra do Ribeiro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4640, 'Barra do Rio Azul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4641, 'Barra Funda', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4642, 'Barracao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4643, 'Barros Cassal', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4644, 'Benjamin Constant do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4645, 'Bento Goncalves', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4646, 'Boa Vista das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4647, 'Boa Vista do Burica', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4648, 'Boa Vista do Cadeado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4649, 'Boa Vista do Incra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4650, 'Boa Vista do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4651, 'Bom Jesus', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4652, 'Bom Principio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4653, 'Bom Progresso', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4654, 'Bom Retiro do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4655, 'Boqueirao do Leao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4656, 'Bossoroca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4657, 'Bozano', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4658, 'Braga', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4659, 'Brochier', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4660, 'Butia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4661, 'Cacapava do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4662, 'Cacequi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4663, 'Cachoeira do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4664, 'Cachoeirinha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4665, 'Cacique Doble', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4666, 'Caibate', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4667, 'Caicara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4668, 'Camaqua', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4669, 'Camargo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4670, 'Cambara do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4671, 'Campestre da Serra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4672, 'Campina das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4673, 'Campinas do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4674, 'Campo Bom', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4675, 'Campo Novo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4676, 'Campos Borges', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4677, 'Candelaria', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4678, 'Candido Godoi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4679, 'Candiota', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4680, 'Canela', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4681, 'Cangucu', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4682, 'Canoas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4683, 'Canudos do Vale', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4684, 'Capao Bonito do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4685, 'Capao da Canoa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4686, 'Capao do Cipo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4687, 'Capao do Leao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4688, 'Capela de Santana', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4689, 'Capitao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4690, 'Capivari do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4691, 'Caraa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4692, 'Carazinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4693, 'Carlos Barbosa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4694, 'Carlos Gomes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4695, 'Casca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4696, 'Caseiros', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4697, 'Catuipe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4698, 'Caxias do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4699, 'Centenario', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4700, 'Cerrito', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4701, 'Cerro Branco', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4702, 'Cerro Grande', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4703, 'Cerro Grande do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4704, 'Cerro Largo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4705, 'Chapada', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4706, 'Charqueadas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4707, 'Charrua', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4708, 'Chiapeta', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4709, 'Chui', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4710, 'Chuvisca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4711, 'Cidreira', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4712, 'Ciriaco', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4713, 'Colinas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4714, 'Colorado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4715, 'Condor', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4716, 'Constantina', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4717, 'Coqueiro Baixo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4718, 'Coqueiros do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4719, 'Coronel Barros', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4720, 'Coronel Bicaco', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4721, 'Coronel Pilar', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4722, 'Cotipora', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4723, 'Coxilha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4724, 'Crissiumal', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4725, 'Cristal', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4726, 'Cristal do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4727, 'Cruz Alta', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4728, 'Cruzaltense', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4729, 'Cruzeiro do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4730, 'David Canabarro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4731, 'Derrubadas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4732, 'Dezesseis de Novembro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4733, 'Dilermando de Aguiar', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4734, 'Dois Irmaos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4735, 'Dois Irmaos das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4736, 'Dois Lajeados', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4737, 'Dom Feliciano', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4738, 'Dom Pedrito', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4739, 'Dom Pedro de Alcantara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4740, 'Dona Francisca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4741, 'Doutor Mauricio Cardoso', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4742, 'Doutor Ricardo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4743, 'Eldorado do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4744, 'Encantado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4745, 'Encruzilhada do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4746, 'Engenho Velho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4747, 'Entre Rios do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4748, 'Entre-ijuis', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4749, 'Erebango', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4750, 'Erechim', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4751, 'Ernestina', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4752, 'Erval Grande', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4753, 'Erval Seco', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4754, 'Esmeralda', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4755, 'Esperanca do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4756, 'Espumoso', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4757, 'Estacao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4758, 'Estancia Velha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4759, 'Esteio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4760, 'Estrela', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4761, 'Estrela Velha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4762, 'Eugenio de Castro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4763, 'Fagundes Varela', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4764, 'Farroupilha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4765, 'Faxinal do Soturno', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4766, 'Faxinalzinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4767, 'Fazenda Vilanova', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4768, 'Feliz', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4769, 'Flores da Cunha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4770, 'Floriano Peixoto', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4771, 'Fontoura Xavier', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4772, 'Formigueiro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4773, 'Forquetinha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4774, 'Fortaleza dos Valos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4775, 'Frederico Westphalen', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4776, 'Garibaldi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4777, 'Garruchos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4778, 'Gaurama', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4779, 'General Camara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4780, 'Gentil', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4781, 'Getulio Vargas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4782, 'Girua', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4783, 'Glorinha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4784, 'Gramado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4785, 'Gramado dos Loureiros', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4786, 'Gramado Xavier', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4787, 'Gravatai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4788, 'Guabiju', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4789, 'Guaiba', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4790, 'Guapore', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4791, 'Guarani das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4792, 'Harmonia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4793, 'Herval', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4794, 'Herveiras', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4795, 'Horizontina', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4796, 'Hulha Negra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4797, 'Humaita', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4798, 'Ibarama', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4799, 'Ibiaca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4800, 'Ibiraiaras', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4801, 'Ibirapuita', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4802, 'Ibiruba', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4803, 'Igrejinha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4804, 'Ijui', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4805, 'Ilopolis', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4806, 'Imbe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4807, 'Imigrante', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4808, 'Independencia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4809, 'Inhacora', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4810, 'Ipe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4811, 'Ipiranga do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4812, 'Irai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4813, 'Itaara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4814, 'Itacurubi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4815, 'Itapuca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4816, 'Itaqui', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4817, 'Itati', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4818, 'Itatiba do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4819, 'Ivora', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4820, 'Ivoti', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4821, 'Jaboticaba', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4822, 'Jacuizinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4823, 'Jacutinga', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4824, 'Jaguarao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4825, 'Jaguari', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4826, 'Jaquirana', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4827, 'Jari', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4828, 'Joia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4829, 'Julio de Castilhos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4830, 'Lagoa Bonita do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4831, 'Lagoa dos Tres Cantos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4832, 'Lagoa Vermelha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4833, 'Lagoao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4834, 'Lajeado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4835, 'Lajeado do Bugre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4836, 'Lavras do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4837, 'Liberato Salzano', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4838, 'Lindolfo Collor', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4839, 'Linha Nova', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4840, 'Macambara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4841, 'Machadinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4842, 'Mampituba', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4843, 'Manoel Viana', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4844, 'Maquine', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4845, 'Marata', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4846, 'Marau', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4847, 'Marcelino Ramos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4848, 'Mariana Pimentel', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4849, 'Mariano Moro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4850, 'Marques de Souza', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4851, 'Mata', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4852, 'Mato Castelhano', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4853, 'Mato Leitao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4854, 'Mato Queimado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4855, 'Maximiliano de Almeida', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4856, 'Minas do Leao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4857, 'Miraguai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4858, 'Montauri', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4859, 'Monte Alegre dos Campos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4860, 'Monte Belo do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4861, 'Montenegro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4862, 'Mormaco', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4863, 'Morrinhos do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4864, 'Morro Redondo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4865, 'Morro Reuter', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4866, 'Mostardas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4867, 'Mucum', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4868, 'Muitos Capoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4869, 'Muliterno', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4870, 'Nao-me-toque', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4871, 'Nicolau Vergueiro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4872, 'Nonoai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4873, 'Nova Alvorada', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4874, 'Nova Araca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4875, 'Nova Bassano', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4876, 'Nova Boa Vista', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4877, 'Nova Brescia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4878, 'Nova Candelaria', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4879, 'Nova Esperanca do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4880, 'Nova Hartz', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4881, 'Nova Padua', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4882, 'Nova Palma', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4883, 'Nova Petropolis', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4884, 'Nova Prata', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4885, 'Nova Ramada', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4886, 'Nova Roma do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4887, 'Nova Santa Rita', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4888, 'Novo Barreiro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4889, 'Novo Cabrais', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4890, 'Novo Hamburgo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4891, 'Novo Machado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4892, 'Novo Tiradentes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4893, 'Novo Xingu', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4894, 'Osorio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4895, 'Paim Filho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4896, 'Palmares do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4897, 'Palmeira das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4898, 'Palmitinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4899, 'Panambi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4900, 'Pantano Grande', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4901, 'Parai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4902, 'Paraiso do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4903, 'Pareci Novo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4904, 'Parobe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4905, 'Passa Sete', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4906, 'Passo do Sobrado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4907, 'Passo Fundo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4908, 'Paulo Bento', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4909, 'Paverama', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4910, 'Pedras Altas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4911, 'Pedro Osorio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4912, 'Pejucara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4913, 'Pelotas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4914, 'Picada Cafe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4915, 'Pinhal', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4916, 'Pinhal da Serra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4917, 'Pinhal Grande', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4918, 'Pinheirinho do Vale', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4919, 'Pinheiro Machado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4920, 'Pirapo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4921, 'Piratini', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4922, 'Planalto', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4923, 'Poco das Antas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4924, 'Pontao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4925, 'Ponte Preta', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4926, 'Portao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4927, 'Porto Alegre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4928, 'Porto Lucena', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4929, 'Porto Maua', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4930, 'Porto Vera Cruz', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4931, 'Porto Xavier', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4932, 'Pouso Novo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4933, 'Presidente Lucena', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4934, 'Progresso', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4935, 'Protasio Alves', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4936, 'Putinga', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4937, 'Quarai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4938, 'Quatro Irmaos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4939, 'Quevedos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4940, 'Quinze de Novembro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4941, 'Redentora', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4942, 'Relvado', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4943, 'Restinga Seca', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4944, 'Rio dos Indios', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4945, 'Rio Grande', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4946, 'Rio Pardo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4947, 'Riozinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4948, 'Roca Sales', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4949, 'Rodeio Bonito', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4950, 'Rolador', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4951, 'Rolante', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4952, 'Ronda Alta', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4953, 'Rondinha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4954, 'Roque Gonzales', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4955, 'Rosario do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4956, 'Sagrada Familia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4957, 'Saldanha Marinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4958, 'Salto do Jacui', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4959, 'Salvador das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4960, 'Salvador do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4961, 'Sananduva', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4962, 'Santa Barbara do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4963, 'Santa Cecilia do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4964, 'Santa Clara do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4965, 'Santa Cruz do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4966, 'Santa Margarida do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4967, 'Santa Maria', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4968, 'Santa Maria do Herval', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4969, 'Santa Rosa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4970, 'Santa Tereza', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4971, 'Santa Vitoria do Palmar', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4972, 'Santana da Boa Vista', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4973, 'Santana do Livramento', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4974, 'Santiago', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4975, 'Santo Angelo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4976, 'Santo Antonio da Patrulha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4977, 'Santo Antonio das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4978, 'Santo Antonio do Palma', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4979, 'Santo Antonio do Planalto', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4980, 'Santo Augusto', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4981, 'Santo Cristo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4982, 'Santo Expedito do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4983, 'Sao Borja', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4984, 'Sao Domingos do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4985, 'Sao Francisco de Assis', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4986, 'Sao Francisco de Paula', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4987, 'Sao Gabriel', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4988, 'Sao Jeronimo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4989, 'Sao Joao da Urtiga', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4990, 'Sao Joao do Polesine', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4991, 'Sao Jorge', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4992, 'Sao Jose das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4993, 'Sao Jose do Herval', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4994, 'Sao Jose do Hortencio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4995, 'Sao Jose do Inhacora', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4996, 'Sao Jose do Norte', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4997, 'Sao Jose do Ouro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4998, 'Sao Jose do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (4999, 'Sao Jose dos Ausentes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5000, 'Sao Leopoldo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5001, 'Sao Lourenco do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5002, 'Sao Luiz Gonzaga', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5003, 'Sao Marcos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5004, 'Sao Martinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5005, 'Sao Martinho da Serra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5006, 'Sao Miguel das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5007, 'Sao Nicolau', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5008, 'Sao Paulo das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5009, 'Sao Pedro da Serra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5010, 'Sao Pedro das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5011, 'Sao Pedro do Butia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5012, 'Sao Pedro do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5013, 'Sao Sebastiao do Cai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5014, 'Sao Sepe', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5015, 'Sao Valentim', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5016, 'Sao Valentim do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5017, 'Sao Valerio do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5018, 'Sao Vendelino', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5019, 'Sao Vicente do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5020, 'Sapiranga', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5021, 'Sapucaia do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5022, 'Sarandi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5023, 'Seberi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5024, 'Sede Nova', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5025, 'Segredo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5026, 'Selbach', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5027, 'Senador Salgado Filho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5028, 'Sentinela do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5029, 'Serafina Correa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5030, 'Serio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5031, 'Sertao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5032, 'Sertao Santana', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5033, 'Sete de Setembro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5034, 'Severiano de Almeida', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5035, 'Silveira Martins', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5036, 'Sinimbu', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5037, 'Sobradinho', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5038, 'Soledade', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5039, 'Tabai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5040, 'Tapejara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5041, 'Tapera', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5042, 'Tapes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5043, 'Taquara', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5044, 'Taquari', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5045, 'Taquarucu do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5046, 'Tavares', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5047, 'Tenente Portela', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5048, 'Terra de Areia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5049, 'Teutonia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5050, 'Tio Hugo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5051, 'Tiradentes do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5052, 'Toropi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5053, 'Torres', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5054, 'Tramandai', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5055, 'Travesseiro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5056, 'Tres Arroios', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5057, 'Tres Cachoeiras', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5058, 'Tres Coroas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5059, 'Tres de Maio', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5060, 'Tres Forquilhas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5061, 'Tres Palmeiras', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5062, 'Tres Passos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5063, 'Trindade do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5064, 'Triunfo', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5065, 'Tucunduva', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5066, 'Tunas', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5067, 'Tupanci do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5068, 'Tupancireta', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5069, 'Tupandi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5070, 'Tuparendi', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5071, 'Turucu', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5072, 'Ubiretama', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5073, 'Uniao da Serra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5074, 'Unistalda', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5075, 'Uruguaiana', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5076, 'Vacaria', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5077, 'Vale do Sol', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5078, 'Vale Real', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5079, 'Vale Verde', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5080, 'Vanini', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5081, 'Venancio Aires', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5082, 'Vera Cruz', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5083, 'Veranopolis', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5084, 'Vespasiano Correa', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5085, 'Viadutos', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5086, 'Viamao', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5087, 'Vicente Dutra', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5088, 'Victor Graeff', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5089, 'Vila Flores', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5090, 'Vila Langaro', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5091, 'Vila Maria', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5092, 'Vila Nova do Sul', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5093, 'Vista Alegre', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5094, 'Vista Alegre do Prata', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5095, 'Vista Gaucha', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5096, 'Vitoria das Missoes', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5097, 'Westfalia', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5098, 'Xangri-la', 'RS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5099, 'Agua Clara', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5100, 'Alcinopolis', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5101, 'Amambai', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5102, 'Anastacio', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5103, 'Anaurilandia', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5104, 'Angelica', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5105, 'Antonio Joao', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5106, 'Aparecida do Taboado', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5107, 'Aquidauana', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5108, 'Aral Moreira', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5109, 'Bandeirantes', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5110, 'Bataguassu', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5111, 'Bataipora', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5112, 'Bela Vista', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5113, 'Bodoquena', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5114, 'Bonito', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5115, 'Brasilandia', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5116, 'Caarapo', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5117, 'Camapua', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5118, 'Campo Grande', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5119, 'Caracol', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5120, 'Cassilandia', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5121, 'Chapadao do Sul', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5122, 'Corguinho', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5123, 'Coronel Sapucaia', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5124, 'Corumba', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5125, 'Costa Rica', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5126, 'Coxim', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5127, 'Deodapolis', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5128, 'Dois Irmaos do Buriti', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5129, 'Douradina', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5130, 'Dourados', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5131, 'Eldorado', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5132, 'Fatima do Sul', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5133, 'Figueirao', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5134, 'Gloria de Dourados', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5135, 'Guia Lopes da Laguna', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5136, 'Iguatemi', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5137, 'Inocencia', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5138, 'Itapora', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5139, 'Itaquirai', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5140, 'Ivinhema', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5141, 'Japora', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5142, 'Jaraguari', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5143, 'Jardim', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5144, 'Jatei', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5145, 'Juti', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5146, 'Ladario', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5147, 'Laguna Carapa', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5148, 'Maracaju', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5149, 'Miranda', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5150, 'Mundo Novo', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5151, 'Navirai', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5152, 'Nioaque', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5153, 'Nova Alvorada do Sul', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5154, 'Nova Andradina', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5155, 'Novo Horizonte do Sul', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5156, 'Paranaiba', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5157, 'Paranhos', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5158, 'Pedro Gomes', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5159, 'Ponta Pora', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5160, 'Porto Murtinho', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5161, 'Ribas do Rio Pardo', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5162, 'Rio Brilhante', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5163, 'Rio Negro', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5164, 'Rio Verde de Mato Grosso', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5165, 'Rochedo', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5166, 'Santa Rita do Pardo', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5167, 'Sao Gabriel do Oeste', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5168, 'Selviria', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5169, 'Sete Quedas', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5170, 'Sidrolandia', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5171, 'Sonora', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5172, 'Tacuru', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5173, 'Taquarussu', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5174, 'Terenos', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5175, 'Tres Lagoas', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5176, 'Vicentina', 'MS', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5177, 'Acorizal', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5178, 'Agua Boa', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5179, 'Alta Floresta', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5180, 'Alto Araguaia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5181, 'Alto Boa Vista', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5182, 'Alto Garcas', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5183, 'Alto Paraguai', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5184, 'Alto Taquari', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5185, 'Apiacas', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5186, 'Araguaiana', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5187, 'Araguainha', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5188, 'Araputanga', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5189, 'Arenapolis', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5190, 'Aripuana', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5191, 'Barao de Melgaco', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5192, 'Barra do Bugres', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5193, 'Barra do Garcas', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5194, 'Bom Jesus do Araguaia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5195, 'Brasnorte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5196, 'Caceres', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5197, 'Campinapolis', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5198, 'Campo Novo do Parecis', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5199, 'Campo Verde', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5200, 'Campos de Julio', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5201, 'Canabrava do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5202, 'Canarana', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5203, 'Carlinda', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5204, 'Castanheira', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5205, 'Chapada dos Guimaraes', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5206, 'Claudia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5207, 'Cocalinho', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5208, 'Colider', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5209, 'Colniza', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5210, 'Comodoro', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5211, 'Confresa', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5212, 'Conquista D''oeste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5213, 'Cotriguacu', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5214, 'Cuiaba', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5215, 'Cuverlandia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5216, 'Denise', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5217, 'Diamantino', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5218, 'Dom Aquino', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5219, 'Feliz Natal', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5220, 'Figueiropolis D''oeste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5221, 'Gaucha do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5222, 'General Carneiro', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5223, 'Gloria D''oeste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5224, 'Guaranta do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5225, 'Guiratinga', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5226, 'Indiavai', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5227, 'Ipiranga do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5228, 'Itanhanga', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5229, 'Itauba', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5230, 'Itiquira', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5231, 'Jaciara', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5232, 'Jangada', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5233, 'Jauru', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5234, 'Juara', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5235, 'Juina', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5236, 'Juruena', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5237, 'Juscimeira', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5238, 'Lambari D''oeste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5239, 'Lucas do Rio Verde', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5240, 'Luciara', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5241, 'Marcelandia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5242, 'Matupa', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5243, 'Mirassol D''oeste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5244, 'Nobres', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5245, 'Nortelandia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5246, 'Nossa Senhora do Livramento', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5247, 'Nova Bandeirantes', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5248, 'Nova Brasilandia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5249, 'Nova Canaa do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5250, 'Nova Guarita', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5251, 'Nova Lacerda', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5252, 'Nova Marilandia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5253, 'Nova Maringa', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5254, 'Nova Monte Verde', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5255, 'Nova Mutum', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5256, 'Nova Nazare', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5257, 'Nova Olimpia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5258, 'Nova Santa Helena', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5259, 'Nova Ubirata', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5260, 'Nova Xavantina', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5261, 'Novo Horizonte do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5262, 'Novo Mundo', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5263, 'Novo Santo Antonio', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5264, 'Novo Sao Joaquim', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5265, 'Paranaita', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5266, 'Paranatinga', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5267, 'Pedra Preta', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5268, 'Peixoto de Azevedo', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5269, 'Planalto da Serra', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5270, 'Pocone', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5271, 'Pontal do Araguaia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5272, 'Ponte Branca', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5273, 'Pontes e Lacerda', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5274, 'Porto Alegre do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5275, 'Porto dos Gauchos', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5276, 'Porto Esperidiao', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5277, 'Porto Estrela', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5278, 'Poxoreo', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5279, 'Primavera do Leste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5280, 'Querencia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5281, 'Reserva do Cabacal', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5282, 'Ribeirao Cascalheira', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5283, 'Ribeiraozinho', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5284, 'Rio Branco', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5285, 'Rondolandia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5286, 'Rondonopolis', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5287, 'Rosario Oeste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5288, 'Salto do Ceu', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5289, 'Santa Carmem', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5290, 'Santa Cruz do Xingu', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5291, 'Santa Rita do Trivelato', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5292, 'Santa Terezinha', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5293, 'Santo Afonso', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5294, 'Santo Antonio do Leste', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5295, 'Santo Antonio do Leverger', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5296, 'Sao Felix do Araguaia', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5297, 'Sao Jose do Povo', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5298, 'Sao Jose do Rio Claro', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5299, 'Sao Jose do Xingu', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5300, 'Sao Jose dos Quatro Marcos', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5301, 'Sao Pedro da Cipa', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5302, 'Sapezal', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5303, 'Serra Nova Dourada', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5304, 'Sinop', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5305, 'Sorriso', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5306, 'Tabapora', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5307, 'Tangara da Serra', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5308, 'Tapurah', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5309, 'Terra Nova do Norte', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5310, 'Tesouro', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5311, 'Torixoreu', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5312, 'Uniao do Sul', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5313, 'Vale de Sao Domingos', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5314, 'Varzea Grande', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5315, 'Vera', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5513, 'Rianapolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5316, 'Vila Bela da Santissima Trindade', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5317, 'Vila Rica', 'MT', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5318, 'Abadia de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5319, 'Abadiania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5320, 'Acreuna', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5321, 'Adelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5322, 'Agua Fria de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5323, 'Agua Limpa', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5324, 'Aguas Lindas de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5325, 'Alexania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5326, 'Aloandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5327, 'Alto Horizonte', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5328, 'Alto Paraiso de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5329, 'Alvorada do Norte', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5330, 'Amaralina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5331, 'Americano do Brasil', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5332, 'Amorinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5333, 'Anapolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5334, 'Anhanguera', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5335, 'Anicuns', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5336, 'Aparecida de Goiania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5337, 'Aparecida do Rio Doce', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5338, 'Apore', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5339, 'Aracu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5340, 'Aragarcas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5341, 'Aragoiania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5342, 'Araguapaz', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5343, 'Arenopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5344, 'Aruana', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5345, 'Aurilandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5346, 'Avelinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5347, 'Baliza', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5348, 'Barro Alto', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5349, 'Bela Vista de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5350, 'Bom Jardim de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5351, 'Bom Jesus de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5352, 'Bonfinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5353, 'Bonopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5354, 'Brazabrantes', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5355, 'Britania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5356, 'Buriti Alegre', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5357, 'Buriti de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5358, 'Buritinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5359, 'Cabeceiras', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5360, 'Cachoeira Alta', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5361, 'Cachoeira de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5362, 'Cachoeira Dourada', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5363, 'Cacu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5364, 'Caiaponia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5365, 'Caldas Novas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5366, 'Caldazinha', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5367, 'Campestre de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5368, 'Campinacu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5369, 'Campinorte', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5370, 'Campo Alegre de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5371, 'Campo Limpo de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5372, 'Campos Belos', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5373, 'Campos Verdes', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5374, 'Carmo do Rio Verde', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5375, 'Castelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5376, 'Catalao', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5377, 'Caturai', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5378, 'Cavalcante', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5379, 'Ceres', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5380, 'Cezarina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5381, 'Chapadao do Ceu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5382, 'Cidade Ocidental', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5383, 'Cocalzinho de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5384, 'Colinas do Sul', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5385, 'Corrego do Ouro', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5386, 'Corumba de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5387, 'Corumbaiba', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5388, 'Cristalina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5389, 'Cristianopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5390, 'Crixas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5391, 'Crominia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5392, 'Cumari', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5393, 'Damianopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5394, 'Damolandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5395, 'Davinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5396, 'Diorama', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5397, 'Divinopolis de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5398, 'Doverlandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5399, 'Edealina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5400, 'Edeia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5401, 'Estrela do Norte', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5402, 'Faina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5403, 'Fazenda Nova', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5404, 'Firminopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5405, 'Flores de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5406, 'Formosa', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5407, 'Formoso', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5408, 'Gameleira de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5409, 'Goianapolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5410, 'Goiandira', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5411, 'Goianesia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5412, 'Goiania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5413, 'Goianira', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5414, 'Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5415, 'Goiatuba', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5416, 'Gouvelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5417, 'Guapo', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5418, 'Guaraita', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5419, 'Guarani de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5420, 'Guarinos', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5421, 'Heitorai', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5422, 'Hidrolandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5423, 'Hidrolina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5424, 'Iaciara', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5425, 'Inaciolandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5426, 'Indiara', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5427, 'Inhumas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5428, 'Ipameri', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5429, 'Ipiranga de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5430, 'Ipora', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5431, 'Israelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5432, 'Itaberai', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5433, 'Itaguari', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5434, 'Itaguaru', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5435, 'Itaja', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5436, 'Itapaci', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5437, 'Itapirapua', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5438, 'Itapuranga', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5439, 'Itaruma', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5440, 'Itaucu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5441, 'Itumbiara', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5442, 'Ivolandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5443, 'Jandaia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5444, 'Jaragua', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5445, 'Jatai', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5446, 'Jaupaci', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5447, 'Jesupolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5448, 'Joviania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5449, 'Jussara', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5450, 'Lagoa Santa', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5451, 'Leopoldo de Bulhoes', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5452, 'Luziania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5453, 'Mairipotaba', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5454, 'Mambai', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5455, 'Mara Rosa', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5456, 'Marzagao', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5457, 'Matrincha', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5458, 'Maurilandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5459, 'Mimoso de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5460, 'Minacu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5461, 'Mineiros', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5462, 'Moipora', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5463, 'Monte Alegre de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5464, 'Montes Claros de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5465, 'Montividiu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5466, 'Montividiu do Norte', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5467, 'Morrinhos', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5468, 'Morro Agudo de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5469, 'Mossamedes', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5470, 'Mozarlandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5471, 'Mundo Novo', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5472, 'Mutunopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5473, 'Nazario', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5474, 'Neropolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5475, 'Niquelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5476, 'Nova America', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5477, 'Nova Aurora', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5478, 'Nova Crixas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5479, 'Nova Gloria', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5480, 'Nova Iguacu de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5481, 'Nova Roma', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5482, 'Nova Veneza', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5483, 'Novo Brasil', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5484, 'Novo Gama', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5485, 'Novo Planalto', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5486, 'Orizona', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5487, 'Ouro Verde de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5488, 'Ouvidor', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5489, 'Padre Bernardo', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5490, 'Palestina de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5491, 'Palmeiras de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5492, 'Palmelo', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5493, 'Palminopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5494, 'Panama', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5495, 'Paranaiguara', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5496, 'Parauna', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5497, 'Perolandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5498, 'Petrolina de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5499, 'Pilar de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5500, 'Piracanjuba', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5501, 'Piranhas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5502, 'Pirenopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5503, 'Pires do Rio', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5504, 'Planaltina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5505, 'Pontalina', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5506, 'Porangatu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5507, 'Porteirao', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5508, 'Portelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5509, 'Posse', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5510, 'Professor Jamil', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5514, 'Rio Quente', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5515, 'Rio Verde', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5516, 'Rubiataba', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5517, 'Sanclerlandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5518, 'Santa Barbara de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5519, 'Santa Cruz de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5520, 'Santa Fe de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5521, 'Santa Helena de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5522, 'Santa Isabel', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5523, 'Santa Rita do Araguaia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5524, 'Santa Rita do Novo Destino', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5525, 'Santa Rosa de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5526, 'Santa Tereza de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5527, 'Santa Terezinha de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5528, 'Santo Antonio da Barra', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5529, 'Santo Antonio de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5530, 'Santo Antonio do Descoberto', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5531, 'Sao Domingos', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5532, 'Sao Francisco de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5533, 'Sao Joao da Parauna', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5534, 'Sao Joao D''alianca', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5535, 'Sao Luis de Montes Belos', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5536, 'Sao Luiz do Norte', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5537, 'Sao Miguel do Araguaia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5538, 'Sao Miguel do Passa Quatro', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5539, 'Sao Patricio', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5540, 'Sao Simao', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5541, 'Senador Canedo', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5542, 'Serranopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5543, 'Silvania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5544, 'Simolandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5545, 'Sitio D''abadia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5546, 'Taquaral de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5547, 'Teresina de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5548, 'Terezopolis de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5549, 'Tres Ranchos', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5550, 'Trindade', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5551, 'Trombas', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5552, 'Turvania', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5553, 'Turvelandia', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5554, 'Uirapuru', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5555, 'Uruacu', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5556, 'Uruana', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5557, 'Urutai', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5558, 'Valparaiso de Goias', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5559, 'Varjao', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5560, 'Vianopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5561, 'Vicentinopolis', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5562, 'Vila Boa', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5563, 'Vila Propicio', 'GO', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);
INSERT INTO municipio VALUES (5564, 'Brasilia', 'DF', NULL, NULL, NULL, NULL, NULL, 'M', NULL, NULL, 1, NULL, 'NOW()', 'M', 'I', NULL, 9);


--
-- Data for Name: pais; Type: TABLE DATA; Schema: public; Owner: -
--

INSERT INTO pais VALUES (1, 'Abissnia', NULL);
INSERT INTO pais VALUES (2, 'Aores', NULL);
INSERT INTO pais VALUES (3, 'Afar Francs', NULL);
INSERT INTO pais VALUES (4, 'Afeganisto', NULL);
INSERT INTO pais VALUES (5, 'Albnia', NULL);
INSERT INTO pais VALUES (6, 'Alemanha', NULL);
INSERT INTO pais VALUES (7, 'Alto Volta', NULL);
INSERT INTO pais VALUES (8, 'Andorra', NULL);
INSERT INTO pais VALUES (9, 'Angola', NULL);
INSERT INTO pais VALUES (10, 'Antrtica Francesa', NULL);
INSERT INTO pais VALUES (11, 'Antrtico Argentino', NULL);
INSERT INTO pais VALUES (12, 'Antrtico Britnico, Territrio', NULL);
INSERT INTO pais VALUES (13, 'Antrtico Chileno', NULL);
INSERT INTO pais VALUES (14, 'Antrtico Noruegues', NULL);
INSERT INTO pais VALUES (15, 'Antgua E. Dep. Barbuda', NULL);
INSERT INTO pais VALUES (16, 'Antilhas Holandesas', NULL);
INSERT INTO pais VALUES (17, 'Aptrida', NULL);
INSERT INTO pais VALUES (18, 'Arbia Saudita', NULL);
INSERT INTO pais VALUES (19, 'Arglia', NULL);
INSERT INTO pais VALUES (20, 'Argentina', NULL);
INSERT INTO pais VALUES (21, 'Armnia', NULL);
INSERT INTO pais VALUES (22, 'Arquiplago de Bismark', NULL);
INSERT INTO pais VALUES (23, 'Arquiplago Manahiki', NULL);
INSERT INTO pais VALUES (24, 'Arquiplago Midway', NULL);
INSERT INTO pais VALUES (25, 'Aruba', NULL);
INSERT INTO pais VALUES (26, 'Ascenso e Tristo da Cunha,is', NULL);
INSERT INTO pais VALUES (27, 'Ashmore e Cartier', NULL);
INSERT INTO pais VALUES (28, 'Austrlia', NULL);
INSERT INTO pais VALUES (29, 'ustria', NULL);
INSERT INTO pais VALUES (30, 'Azerbaijo', NULL);
INSERT INTO pais VALUES (31, 'Bahrein', NULL);
INSERT INTO pais VALUES (32, 'Bangladesh', NULL);
INSERT INTO pais VALUES (33, 'Barbados', NULL);
INSERT INTO pais VALUES (34, 'Bashkista', NULL);
INSERT INTO pais VALUES (35, 'Bechuanalndia', NULL);
INSERT INTO pais VALUES (36, 'Blgica', NULL);
INSERT INTO pais VALUES (37, 'Belize', NULL);
INSERT INTO pais VALUES (38, 'Benin', NULL);
INSERT INTO pais VALUES (39, 'Bermudas', NULL);
INSERT INTO pais VALUES (40, 'Bhutan', NULL);
INSERT INTO pais VALUES (41, 'Birmnia', NULL);
INSERT INTO pais VALUES (42, 'Bolvia', NULL);
INSERT INTO pais VALUES (43, 'Bsnia Herzegovina', NULL);
INSERT INTO pais VALUES (44, 'Botsuana', NULL);
INSERT INTO pais VALUES (45, 'Brasil', NULL);
INSERT INTO pais VALUES (46, 'Brunei', NULL);
INSERT INTO pais VALUES (47, 'Bulgria', NULL);
INSERT INTO pais VALUES (48, 'Burkina Fasso', NULL);
INSERT INTO pais VALUES (49, 'Burundi', NULL);
INSERT INTO pais VALUES (50, 'Buryat', NULL);
INSERT INTO pais VALUES (51, 'Cabo Verde', NULL);
INSERT INTO pais VALUES (52, 'Camares', NULL);
INSERT INTO pais VALUES (53, 'Canad', NULL);
INSERT INTO pais VALUES (54, 'Carlia', NULL);
INSERT INTO pais VALUES (55, 'Catar', NULL);
INSERT INTO pais VALUES (56, 'Cazaquisto', NULL);
INSERT INTO pais VALUES (57, 'Ceilo', NULL);
INSERT INTO pais VALUES (58, 'Ceuta e Melilla', NULL);
INSERT INTO pais VALUES (59, 'Chade', NULL);
INSERT INTO pais VALUES (60, 'Chechen Ingusth', NULL);
INSERT INTO pais VALUES (61, 'Chile', NULL);
INSERT INTO pais VALUES (62, 'China', NULL);
INSERT INTO pais VALUES (63, 'China (taiwan)', NULL);
INSERT INTO pais VALUES (64, 'Chipre', NULL);
INSERT INTO pais VALUES (65, 'Chuvash', NULL);
INSERT INTO pais VALUES (66, 'Cingapura', NULL);
INSERT INTO pais VALUES (67, 'Colmbia', NULL);
INSERT INTO pais VALUES (68, 'Comunidade das Bahamas', NULL);
INSERT INTO pais VALUES (69, 'Comunidade Dominicana', NULL);
INSERT INTO pais VALUES (70, 'Congo', NULL);
INSERT INTO pais VALUES (71, 'Coria', NULL);
INSERT INTO pais VALUES (72, 'Costa do Marfim', NULL);
INSERT INTO pais VALUES (73, 'Costa Rica', NULL);
INSERT INTO pais VALUES (74, 'Coveite', NULL);
INSERT INTO pais VALUES (75, 'Crocia', NULL);
INSERT INTO pais VALUES (76, 'Cuba', NULL);
INSERT INTO pais VALUES (77, 'Curaao', NULL);
INSERT INTO pais VALUES (78, 'Dagesta', NULL);
INSERT INTO pais VALUES (79, 'Daom', NULL);
INSERT INTO pais VALUES (80, 'Dependncia de Ross', NULL);
INSERT INTO pais VALUES (81, 'Dinamarca', NULL);
INSERT INTO pais VALUES (82, 'Djibuti', NULL);
INSERT INTO pais VALUES (83, 'Eire', NULL);
INSERT INTO pais VALUES (84, 'Emirados rabes Unidos', NULL);
INSERT INTO pais VALUES (85, 'Equador', NULL);
INSERT INTO pais VALUES (86, 'Esccia', NULL);
INSERT INTO pais VALUES (87, 'Eslovquia', NULL);
INSERT INTO pais VALUES (88, 'Eslovnia', NULL);
INSERT INTO pais VALUES (89, 'Espanha', NULL);
INSERT INTO pais VALUES (90, 'Estado da Cidade do Vaticano', NULL);
INSERT INTO pais VALUES (91, 'Estados Assoc. das Antilhas', NULL);
INSERT INTO pais VALUES (92, 'Estados Unidos da Amrica (eua)', NULL);
INSERT INTO pais VALUES (93, 'Estnia', NULL);
INSERT INTO pais VALUES (94, 'Etipia', NULL);
INSERT INTO pais VALUES (95, 'Filipinas', NULL);
INSERT INTO pais VALUES (96, 'Finlndia', NULL);
INSERT INTO pais VALUES (97, 'Frana', NULL);
INSERT INTO pais VALUES (98, 'Gmbia', NULL);
INSERT INTO pais VALUES (99, 'Gana', NULL);
INSERT INTO pais VALUES (100, 'Gaza', NULL);
INSERT INTO pais VALUES (101, 'Gergia', NULL);
INSERT INTO pais VALUES (102, 'Gibraltar', NULL);
INSERT INTO pais VALUES (103, 'Gorno Altai', NULL);
INSERT INTO pais VALUES (104, 'Gr-bretanha', NULL);
INSERT INTO pais VALUES (105, 'Granada', NULL);
INSERT INTO pais VALUES (106, 'Grcia', NULL);
INSERT INTO pais VALUES (107, 'Groenlndia', NULL);
INSERT INTO pais VALUES (108, 'Guam', NULL);
INSERT INTO pais VALUES (109, 'Guatemala', NULL);
INSERT INTO pais VALUES (110, 'Guiana Francesa', NULL);
INSERT INTO pais VALUES (111, 'Guin', NULL);
INSERT INTO pais VALUES (112, 'Guin Bissau', NULL);
INSERT INTO pais VALUES (113, 'Guin Equatorial', NULL);
INSERT INTO pais VALUES (114, 'Holanda', NULL);
INSERT INTO pais VALUES (115, 'Honduras', NULL);
INSERT INTO pais VALUES (116, 'Honduras Britnicas', NULL);
INSERT INTO pais VALUES (117, 'Hong-kong', NULL);
INSERT INTO pais VALUES (118, 'Hungria', NULL);
INSERT INTO pais VALUES (119, 'Iemen', NULL);
INSERT INTO pais VALUES (120, 'Iemen do Sul', NULL);
INSERT INTO pais VALUES (121, 'Ifni', NULL);
INSERT INTO pais VALUES (122, 'Ilha Johnston e Sand', NULL);
INSERT INTO pais VALUES (123, 'Ilha Milhos', NULL);
INSERT INTO pais VALUES (124, 'Ilhas Baker', NULL);
INSERT INTO pais VALUES (125, 'Ilhas Baleares', NULL);
INSERT INTO pais VALUES (126, 'Ilhas Canrias', NULL);
INSERT INTO pais VALUES (127, 'Ilhas Canto e Enderburg', NULL);
INSERT INTO pais VALUES (128, 'Ilhas Carolinas', NULL);
INSERT INTO pais VALUES (129, 'Ilhas Christmas', NULL);
INSERT INTO pais VALUES (130, 'Ilhas Comores', NULL);
INSERT INTO pais VALUES (131, 'Ilhas Cook', NULL);
INSERT INTO pais VALUES (132, 'Ilhas Cosmoledo (lomores)', NULL);
INSERT INTO pais VALUES (133, 'Ilhas de Man', NULL);
INSERT INTO pais VALUES (134, 'Ilhas do Canal', NULL);
INSERT INTO pais VALUES (135, 'Ilhas do Pacfico', NULL);
INSERT INTO pais VALUES (136, 'Ilhas Falklands', NULL);
INSERT INTO pais VALUES (137, 'Ilhas Faroes', NULL);
INSERT INTO pais VALUES (138, 'Ilhas Gilbert', NULL);
INSERT INTO pais VALUES (139, 'Ilhas Guadalupe', NULL);
INSERT INTO pais VALUES (140, 'Ilhas Howland e Jarvis', NULL);
INSERT INTO pais VALUES (141, 'Ilhas Kingman Reef', NULL);
INSERT INTO pais VALUES (142, 'Ilhas Linha', NULL);
INSERT INTO pais VALUES (143, 'Ilhas Macdonal e Heard', NULL);
INSERT INTO pais VALUES (144, 'Ilhas Macquaire', NULL);
INSERT INTO pais VALUES (145, 'Ilhas Malvinas', NULL);
INSERT INTO pais VALUES (146, 'Ilhas Marianas', NULL);
INSERT INTO pais VALUES (147, 'Ilhas Marshall', NULL);
INSERT INTO pais VALUES (148, 'Ilhas Niue', NULL);
INSERT INTO pais VALUES (149, 'Ilhas Norfolk', NULL);
INSERT INTO pais VALUES (150, 'Ilhas Nova Calednia', NULL);
INSERT INTO pais VALUES (151, 'Ilhas Novas Hebridas', NULL);
INSERT INTO pais VALUES (152, 'Ilhas Palau', NULL);
INSERT INTO pais VALUES (153, 'Ilhas Pscoa', NULL);
INSERT INTO pais VALUES (154, 'Ilhas Pitcairin', NULL);
INSERT INTO pais VALUES (155, 'Ilhas Salomo', NULL);
INSERT INTO pais VALUES (156, 'Ilhas Santa Cruz', NULL);
INSERT INTO pais VALUES (157, 'Ilhas Serranas', NULL);
INSERT INTO pais VALUES (158, 'Ilhas Tokelau', NULL);
INSERT INTO pais VALUES (159, 'Ilhas Turca', NULL);
INSERT INTO pais VALUES (160, 'Ilhas Turks e Caicos', NULL);
INSERT INTO pais VALUES (161, 'Ilhas Virgens Americanas', NULL);
INSERT INTO pais VALUES (162, 'Ilhas Virgens Britnicas', NULL);
INSERT INTO pais VALUES (163, 'Ilhas Wake', NULL);
INSERT INTO pais VALUES (164, 'Ilhas Wallis e Futuna', NULL);
INSERT INTO pais VALUES (165, 'ndia', NULL);
INSERT INTO pais VALUES (166, 'Indonsia', NULL);
INSERT INTO pais VALUES (167, 'Inglaterra', NULL);
INSERT INTO pais VALUES (168, 'Ir', NULL);
INSERT INTO pais VALUES (169, 'Iraque', NULL);
INSERT INTO pais VALUES (170, 'Irlanda', NULL);
INSERT INTO pais VALUES (171, 'Irlanda do Norte', NULL);
INSERT INTO pais VALUES (172, 'Islndia', NULL);
INSERT INTO pais VALUES (173, 'Israel', NULL);
INSERT INTO pais VALUES (174, 'Itlia', NULL);
INSERT INTO pais VALUES (175, 'Iugoslvia', NULL);
INSERT INTO pais VALUES (176, 'Jamaica', NULL);
INSERT INTO pais VALUES (177, 'Japo', NULL);
INSERT INTO pais VALUES (178, 'Jordnia', NULL);
INSERT INTO pais VALUES (179, 'Kabardino Balkar', NULL);
INSERT INTO pais VALUES (180, 'Kalimatan', NULL);
INSERT INTO pais VALUES (181, 'Kalmir', NULL);
INSERT INTO pais VALUES (182, 'Kara Kalpak', NULL);
INSERT INTO pais VALUES (183, 'Karachaevocherkess', NULL);
INSERT INTO pais VALUES (184, 'Khakass', NULL);
INSERT INTO pais VALUES (185, 'Kmer/camboja', NULL);
INSERT INTO pais VALUES (186, 'Komi', NULL);
INSERT INTO pais VALUES (187, 'Kuwait', NULL);
INSERT INTO pais VALUES (188, 'Laos', NULL);
INSERT INTO pais VALUES (189, 'Lesoto', NULL);
INSERT INTO pais VALUES (190, 'Letnia', NULL);
INSERT INTO pais VALUES (191, 'Lbano', NULL);
INSERT INTO pais VALUES (192, 'Libria', NULL);
INSERT INTO pais VALUES (193, 'Lbia', NULL);
INSERT INTO pais VALUES (194, 'Liechtenstein', NULL);
INSERT INTO pais VALUES (195, 'Litunia', NULL);
INSERT INTO pais VALUES (196, 'Luxemburgo', NULL);
INSERT INTO pais VALUES (197, 'Macau', NULL);
INSERT INTO pais VALUES (198, 'Madagascar', NULL);
INSERT INTO pais VALUES (199, 'Madeira', NULL);
INSERT INTO pais VALUES (200, 'Malsia', NULL);
INSERT INTO pais VALUES (201, 'Malawi', NULL);
INSERT INTO pais VALUES (202, 'Maldivas,is', NULL);
INSERT INTO pais VALUES (203, 'Mali', NULL);
INSERT INTO pais VALUES (204, 'Mari', NULL);
INSERT INTO pais VALUES (205, 'Marrocos', NULL);
INSERT INTO pais VALUES (206, 'Martinica', NULL);
INSERT INTO pais VALUES (207, 'Mascate', NULL);
INSERT INTO pais VALUES (208, 'Maurcio', NULL);
INSERT INTO pais VALUES (209, 'Mauritnia', NULL);
INSERT INTO pais VALUES (210, 'Mxico', NULL);
INSERT INTO pais VALUES (211, 'Mianma', NULL);
INSERT INTO pais VALUES (212, 'Moambique', NULL);
INSERT INTO pais VALUES (213, 'Moldvia', NULL);
INSERT INTO pais VALUES (214, 'Mnaco', NULL);
INSERT INTO pais VALUES (215, 'Monglia', NULL);
INSERT INTO pais VALUES (216, 'Monte Serrat', NULL);
INSERT INTO pais VALUES (217, 'Montenegro', NULL);
INSERT INTO pais VALUES (218, 'Nambia', NULL);
INSERT INTO pais VALUES (219, 'Nauru', NULL);
INSERT INTO pais VALUES (220, 'Nepal', NULL);
INSERT INTO pais VALUES (221, 'Nguane', NULL);
INSERT INTO pais VALUES (222, 'Nicargua', NULL);
INSERT INTO pais VALUES (223, 'Nigria', NULL);
INSERT INTO pais VALUES (224, 'Noruega', NULL);
INSERT INTO pais VALUES (225, 'Nova Guin', NULL);
INSERT INTO pais VALUES (226, 'Nova Zelndia', NULL);
INSERT INTO pais VALUES (227, 'Oman', NULL);
INSERT INTO pais VALUES (228, 'Ossetia Setentrional', NULL);
INSERT INTO pais VALUES (229, 'Pas de Gales', NULL);
INSERT INTO pais VALUES (230, 'Pases Baixos', NULL);
INSERT INTO pais VALUES (231, 'Palestina', NULL);
INSERT INTO pais VALUES (232, 'Panam', NULL);
INSERT INTO pais VALUES (233, 'Panam - Zona do Canal', NULL);
INSERT INTO pais VALUES (234, 'Papua Nova Guin', NULL);
INSERT INTO pais VALUES (235, 'Paquisto', NULL);
INSERT INTO pais VALUES (236, 'Paraguai', NULL);
INSERT INTO pais VALUES (237, 'Peru', NULL);
INSERT INTO pais VALUES (238, 'Polinsia Francesa', NULL);
INSERT INTO pais VALUES (239, 'Polnia', NULL);
INSERT INTO pais VALUES (240, 'Porto Rico', NULL);
INSERT INTO pais VALUES (241, 'Portugal', NULL);
INSERT INTO pais VALUES (242, 'Praas Norte Africanas', NULL);
INSERT INTO pais VALUES (243, 'Protetor do Sudoeste Africano', NULL);
INSERT INTO pais VALUES (244, 'Qunia', NULL);
INSERT INTO pais VALUES (245, 'Quirguisto', NULL);
INSERT INTO pais VALUES (246, 'Quitasueno', NULL);
INSERT INTO pais VALUES (247, 'Repblica rabe do Egito', NULL);
INSERT INTO pais VALUES (248, 'Repblica Centro Africana', NULL);
INSERT INTO pais VALUES (249, 'Repblica da frica do Sul', NULL);
INSERT INTO pais VALUES (250, 'Repblica da Bielorrssia', NULL);
INSERT INTO pais VALUES (251, 'Repblica da Macednia', NULL);
INSERT INTO pais VALUES (252, 'Repblica de El Salvador', NULL);
INSERT INTO pais VALUES (253, 'Repblica de Fiji', NULL);
INSERT INTO pais VALUES (254, 'Repblica de Malta', NULL);
INSERT INTO pais VALUES (255, 'Repblica do Gabo', NULL);
INSERT INTO pais VALUES (256, 'Repblica do Haiti', NULL);
INSERT INTO pais VALUES (257, 'Repblica do Nger', NULL);
INSERT INTO pais VALUES (258, 'Repblica Dominicana', NULL);
INSERT INTO pais VALUES (259, 'Repblica Guiana', NULL);
INSERT INTO pais VALUES (260, 'Repblica Tcheca', NULL);
INSERT INTO pais VALUES (261, 'Reservado', NULL);
INSERT INTO pais VALUES (262, 'Reservado', NULL);
INSERT INTO pais VALUES (263, 'Reservado', NULL);
INSERT INTO pais VALUES (264, 'Reservado', NULL);
INSERT INTO pais VALUES (265, 'Reunio', NULL);
INSERT INTO pais VALUES (266, 'Rodsia (zimbbwe)', NULL);
INSERT INTO pais VALUES (267, 'Romnia', NULL);
INSERT INTO pais VALUES (268, 'Roncador', NULL);
INSERT INTO pais VALUES (269, 'Ruanda', NULL);
INSERT INTO pais VALUES (270, 'Ruiquiu,is', NULL);
INSERT INTO pais VALUES (271, 'Rssia', NULL);
INSERT INTO pais VALUES (272, 'Saara Espanhol', NULL);
INSERT INTO pais VALUES (273, 'Sabah', NULL);
INSERT INTO pais VALUES (274, 'Samoa Americana', NULL);
INSERT INTO pais VALUES (275, 'Samoa Ocidental', NULL);
INSERT INTO pais VALUES (276, 'San Marino', NULL);
INSERT INTO pais VALUES (277, 'Santa Helena', NULL);
INSERT INTO pais VALUES (278, 'Santa Lcia', NULL);
INSERT INTO pais VALUES (279, 'So Cristvo', NULL);
INSERT INTO pais VALUES (280, 'So Tom e Prncipe', NULL);
INSERT INTO pais VALUES (281, 'So Vicente', NULL);
INSERT INTO pais VALUES (282, 'Sarawak', NULL);
INSERT INTO pais VALUES (283, 'Senegal', NULL);
INSERT INTO pais VALUES (284, 'Sequin', NULL);
INSERT INTO pais VALUES (285, 'Serra Leoa', NULL);
INSERT INTO pais VALUES (286, 'Srvia', NULL);
INSERT INTO pais VALUES (287, 'Seychelles', NULL);
INSERT INTO pais VALUES (288, 'Sria', NULL);
INSERT INTO pais VALUES (289, 'Somlia, Repblica', NULL);
INSERT INTO pais VALUES (290, 'Sri-lanka', NULL);
INSERT INTO pais VALUES (291, 'St. Pierre et Miquelon', NULL);
INSERT INTO pais VALUES (292, 'Suazilndia', NULL);
INSERT INTO pais VALUES (293, 'Sudo', NULL);
INSERT INTO pais VALUES (294, 'Sucia', NULL);
INSERT INTO pais VALUES (295, 'Sua', NULL);
INSERT INTO pais VALUES (296, 'Suriname', NULL);
INSERT INTO pais VALUES (297, 'Svalbard e Jan Mayer,is', NULL);
INSERT INTO pais VALUES (298, 'Tadjiquisto', NULL);
INSERT INTO pais VALUES (299, 'Tailndia', NULL);
INSERT INTO pais VALUES (300, 'Tanganica', NULL);
INSERT INTO pais VALUES (301, 'Tanznia', NULL);
INSERT INTO pais VALUES (302, 'Tartaria', NULL);
INSERT INTO pais VALUES (303, 'Tchecoslovquia', NULL);
INSERT INTO pais VALUES (304, 'Terr. Antrtico da Austrlia', NULL);
INSERT INTO pais VALUES (305, 'Terras Austrais', NULL);
INSERT INTO pais VALUES (306, 'Territ. Britnico do Oceano ndico', NULL);
INSERT INTO pais VALUES (307, 'Territrio de Cocos', NULL);
INSERT INTO pais VALUES (308, 'Territrio de Papua', NULL);
INSERT INTO pais VALUES (309, 'Timor', NULL);
INSERT INTO pais VALUES (310, 'Togo', NULL);
INSERT INTO pais VALUES (311, 'Tonga', NULL);
INSERT INTO pais VALUES (312, 'Transkei', NULL);
INSERT INTO pais VALUES (313, 'Trgua, Estado', NULL);
INSERT INTO pais VALUES (314, 'Trinidad e Tobago', NULL);
INSERT INTO pais VALUES (315, 'Tunsia', NULL);
INSERT INTO pais VALUES (316, 'Turcomenisto', NULL);
INSERT INTO pais VALUES (317, 'Turquia', NULL);
INSERT INTO pais VALUES (318, 'Tuvalu', NULL);
INSERT INTO pais VALUES (319, 'Tuvin', NULL);
INSERT INTO pais VALUES (320, 'Ucrnia', NULL);
INSERT INTO pais VALUES (321, 'Udmurt', NULL);
INSERT INTO pais VALUES (322, 'Uganda', NULL);
INSERT INTO pais VALUES (323, 'Unio Sovitica', NULL);
INSERT INTO pais VALUES (324, 'Uruguai', NULL);
INSERT INTO pais VALUES (325, 'Uzbequisto', NULL);
INSERT INTO pais VALUES (326, 'Venezuela', NULL);
INSERT INTO pais VALUES (327, 'Vietn do Norte', NULL);
INSERT INTO pais VALUES (328, 'Vietn do Sul', NULL);
INSERT INTO pais VALUES (329, 'Yakut', NULL);
INSERT INTO pais VALUES (330, 'Zaire', NULL);
INSERT INTO pais VALUES (331, 'Zmbia', NULL);
INSERT INTO pais VALUES (332, 'Zimbbwe', NULL);


--
-- Data for Name: regiao; Type: TABLE DATA; Schema: public; Owner: -
--



--
-- Data for Name: setor; Type: TABLE DATA; Schema: public; Owner: -
--



--
-- Data for Name: uf; Type: TABLE DATA; Schema: public; Owner: -
--

INSERT INTO uf VALUES ('RO', 'Rondnia', NULL, 45);
INSERT INTO uf VALUES ('AC', 'Acre', NULL, 45);
INSERT INTO uf VALUES ('AM', 'Amazonas', NULL, 45);
INSERT INTO uf VALUES ('RR', 'Roraima', NULL, 45);
INSERT INTO uf VALUES ('PA', 'Par', NULL, 45);
INSERT INTO uf VALUES ('AP', 'Amap', NULL, 45);
INSERT INTO uf VALUES ('TO', 'Tocantins', NULL, 45);
INSERT INTO uf VALUES ('MA', 'Maranho', NULL, 45);
INSERT INTO uf VALUES ('PI', 'Piau', NULL, 45);
INSERT INTO uf VALUES ('CE', 'Cear', NULL, 45);
INSERT INTO uf VALUES ('RN', 'Rio Grande do Norte', NULL, 45);
INSERT INTO uf VALUES ('PB', 'Paraba', NULL, 45);
INSERT INTO uf VALUES ('PE', 'Pernambuco', NULL, 45);
INSERT INTO uf VALUES ('AL', 'Alagoas', NULL, 45);
INSERT INTO uf VALUES ('SE', 'Sergipe', NULL, 45);
INSERT INTO uf VALUES ('BA', 'Bahia', NULL, 45);
INSERT INTO uf VALUES ('MG', 'Minas Gerais', NULL, 45);
INSERT INTO uf VALUES ('ES', 'Esprito Santo', NULL, 45);
INSERT INTO uf VALUES ('RJ', 'Rio de Janeiro', NULL, 45);
INSERT INTO uf VALUES ('SP', 'So Paulo', NULL, 45);
INSERT INTO uf VALUES ('PR', 'Paran', NULL, 45);
INSERT INTO uf VALUES ('SC', 'Santa Catarina', NULL, 45);
INSERT INTO uf VALUES ('RS', 'Rio Grande do Sul', NULL, 45);
INSERT INTO uf VALUES ('MS', 'Mato Grosso do Sul', NULL, 45);
INSERT INTO uf VALUES ('MT', 'Mato Grosso', NULL, 45);
INSERT INTO uf VALUES ('GO', 'Gois', NULL, 45);
INSERT INTO uf VALUES ('DF', 'Distrito Federal', NULL, 45);


--
-- Data for Name: vila; Type: TABLE DATA; Schema: public; Owner: -
--



SET search_path = urbano, pg_catalog;

--
-- Data for Name: cep_logradouro; Type: TABLE DATA; Schema: urbano; Owner: -
--



--
-- Data for Name: cep_logradouro_bairro; Type: TABLE DATA; Schema: urbano; Owner: -
--



--
-- Data for Name: tipo_logradouro; Type: TABLE DATA; Schema: urbano; Owner: -
--

INSERT INTO tipo_logradouro VALUES ('CORRE', 'Corredor');
INSERT INTO tipo_logradouro VALUES ('DESCI', 'Descida');
INSERT INTO tipo_logradouro VALUES ('AEROP', 'Aeroporto');
INSERT INTO tipo_logradouro VALUES ('DESVI', 'Desvio');
INSERT INTO tipo_logradouro VALUES ('ELEVA', 'Elevada');
INSERT INTO tipo_logradouro VALUES ('ENTRP', 'Entrada Particular');
INSERT INTO tipo_logradouro VALUES ('ESCA', 'Escada');
INSERT INTO tipo_logradouro VALUES ('ESCAD', 'Escadaria');
INSERT INTO tipo_logradouro VALUES ('ESTAC', 'Estacionamento');
INSERT INTO tipo_logradouro VALUES ('ESPLA', 'Esplanada');
INSERT INTO tipo_logradouro VALUES ('GALER', 'Galeria');
INSERT INTO tipo_logradouro VALUES ('ILHA', 'Ilha');
INSERT INTO tipo_logradouro VALUES ('JDNET', 'Jardinete');
INSERT INTO tipo_logradouro VALUES ('LAGO', 'Lago');
INSERT INTO tipo_logradouro VALUES ('LAGOA', 'Lagoa');
INSERT INTO tipo_logradouro VALUES ('MONTE', 'Monte');
INSERT INTO tipo_logradouro VALUES ('MARIN', 'Marina');
INSERT INTO tipo_logradouro VALUES ('VERED', 'Vereda');
INSERT INTO tipo_logradouro VALUES ('CORRG', 'Crrego');
INSERT INTO tipo_logradouro VALUES ('PASSE', 'Passeio');
INSERT INTO tipo_logradouro VALUES ('BULEV', 'Bulevar');
INSERT INTO tipo_logradouro VALUES ('PAES', 'Praa Esportes');
INSERT INTO tipo_logradouro VALUES ('CALA', 'Calada');
INSERT INTO tipo_logradouro VALUES ('COLON', 'Colnia');
INSERT INTO tipo_logradouro VALUES ('PATIO', 'Ptio');
INSERT INTO tipo_logradouro VALUES ('TUNEL', 'Tnel');
INSERT INTO tipo_logradouro VALUES ('ROTUL', 'Rtula');
INSERT INTO tipo_logradouro VALUES ('ESTAD', 'Estdio');
INSERT INTO tipo_logradouro VALUES ('MODUL', 'Mdulo');
INSERT INTO tipo_logradouro VALUES ('SERVI', 'Servido');
INSERT INTO tipo_logradouro VALUES ('ALM', 'Alameda');
INSERT INTO tipo_logradouro VALUES ('ALTO', 'Alto');
INSERT INTO tipo_logradouro VALUES ('AV', 'Avenida');
INSERT INTO tipo_logradouro VALUES ('BEC', 'Beco');
INSERT INTO tipo_logradouro VALUES ('CHC', 'Chcara');
INSERT INTO tipo_logradouro VALUES ('COND', 'Condomnio');
INSERT INTO tipo_logradouro VALUES ('CONJ', 'Conjunto');
INSERT INTO tipo_logradouro VALUES ('DIV', 'Divisa');
INSERT INTO tipo_logradouro VALUES ('EST', 'Estrada');
INSERT INTO tipo_logradouro VALUES ('FEIRA', 'Feira');
INSERT INTO tipo_logradouro VALUES ('JD', 'Jardim');
INSERT INTO tipo_logradouro VALUES ('LAD', 'Ladeira');
INSERT INTO tipo_logradouro VALUES ('LTM', 'Loteamento');
INSERT INTO tipo_logradouro VALUES ('LRG', 'Largo');
INSERT INTO tipo_logradouro VALUES ('MRG', 'Margem');
INSERT INTO tipo_logradouro VALUES ('PA', 'Praa');
INSERT INTO tipo_logradouro VALUES ('PORTO', 'Porto');
INSERT INTO tipo_logradouro VALUES ('PQE', 'Parque');
INSERT INTO tipo_logradouro VALUES ('PRAIA', 'Praia');
INSERT INTO tipo_logradouro VALUES ('QDA', 'Quadra');
INSERT INTO tipo_logradouro VALUES ('REC', 'Recanto');
INSERT INTO tipo_logradouro VALUES ('RES', 'Residencial');
INSERT INTO tipo_logradouro VALUES ('RET', 'Retorno');
INSERT INTO tipo_logradouro VALUES ('ROD', 'Rodovia');
INSERT INTO tipo_logradouro VALUES ('ROT', 'Rotatria');
INSERT INTO tipo_logradouro VALUES ('RUA', 'Rua');
INSERT INTO tipo_logradouro VALUES ('STIO', 'Stio');
INSERT INTO tipo_logradouro VALUES ('TER', 'Terminal');
INSERT INTO tipo_logradouro VALUES ('TRV', 'Travessa');
INSERT INTO tipo_logradouro VALUES ('UND', 'Unidade');
INSERT INTO tipo_logradouro VALUES ('VIA', 'Via');
INSERT INTO tipo_logradouro VALUES ('VLA', 'Vila');
INSERT INTO tipo_logradouro VALUES ('ACAMP', 'Acampamento');
INSERT INTO tipo_logradouro VALUES ('ACESS', 'Acesso');
INSERT INTO tipo_logradouro VALUES ('VIACE', 'Via de Acesso');
INSERT INTO tipo_logradouro VALUES ('ADRO', 'Adro');
INSERT INTO tipo_logradouro VALUES ('ANTES', 'Antiga Estrada');
INSERT INTO tipo_logradouro VALUES ('AREA', 'rea');
INSERT INTO tipo_logradouro VALUES ('ARESP', 'rea Especial');
INSERT INTO tipo_logradouro VALUES ('ATALH', 'Atalho');
INSERT INTO tipo_logradouro VALUES ('AVCON', 'Avenida Contorno');
INSERT INTO tipo_logradouro VALUES ('AVMAR', 'Avenida Marginal');
INSERT INTO tipo_logradouro VALUES ('AVVEL', 'Avenida Velha');
INSERT INTO tipo_logradouro VALUES ('PARAL', 'Paralela');
INSERT INTO tipo_logradouro VALUES ('ENTRQ', 'Entre Quadra');
INSERT INTO tipo_logradouro VALUES ('PRQMU', 'Parque Municipal');
INSERT INTO tipo_logradouro VALUES ('PRQRE', 'Parque Residencial');
INSERT INTO tipo_logradouro VALUES ('BAIXA', 'Baixa');
INSERT INTO tipo_logradouro VALUES ('PASSG', 'Passagem');
INSERT INTO tipo_logradouro VALUES ('PASPE', 'Passagem de Pedestres');
INSERT INTO tipo_logradouro VALUES ('PASSB', 'Passagem Subterrnea');
INSERT INTO tipo_logradouro VALUES ('BELVE', 'Belvedere');
INSERT INTO tipo_logradouro VALUES ('PASSA', 'Passarela');
INSERT INTO tipo_logradouro VALUES ('BLOCO', 'Bloco');
INSERT INTO tipo_logradouro VALUES ('BOSQU', 'Bosque');
INSERT INTO tipo_logradouro VALUES ('PONTA', 'Ponta');
INSERT INTO tipo_logradouro VALUES ('BOULE', 'Boulevard');
INSERT INTO tipo_logradouro VALUES ('ZIGUE', 'Zigue-Zague');
INSERT INTO tipo_logradouro VALUES ('VIELA', 'Viela');
INSERT INTO tipo_logradouro VALUES ('VIDUT', 'Viaduto');
INSERT INTO tipo_logradouro VALUES ('VIALO', 'Via Local');
INSERT INTO tipo_logradouro VALUES ('VIAPE', 'Via de Pedestre');
INSERT INTO tipo_logradouro VALUES ('VIALI', 'Via Litornea');
INSERT INTO tipo_logradouro VALUES ('VIAEL', 'Via Elevado');
INSERT INTO tipo_logradouro VALUES ('VIAEX', 'Via Expressa');
INSERT INTO tipo_logradouro VALUES ('VIACO', 'Via Coletora');
INSERT INTO tipo_logradouro VALUES ('CAMIN', 'Caminho');
INSERT INTO tipo_logradouro VALUES ('VALE', 'Vale');
INSERT INTO tipo_logradouro VALUES ('VALA', 'Vala');
INSERT INTO tipo_logradouro VALUES ('TRAVP', 'Travessa Particular');
INSERT INTO tipo_logradouro VALUES ('TRAVV', 'Travessa Velha');
INSERT INTO tipo_logradouro VALUES ('TRECH', 'Trecho');
INSERT INTO tipo_logradouro VALUES ('TREVO', 'Trevo');
INSERT INTO tipo_logradouro VALUES ('RUAPE', 'Rua de Pedestre');
INSERT INTO tipo_logradouro VALUES ('RUAPA', 'Rua Particular');
INSERT INTO tipo_logradouro VALUES ('RUAVE', 'Rua Velha');
INSERT INTO tipo_logradouro VALUES ('SUBID', 'Subida');
INSERT INTO tipo_logradouro VALUES ('PONTE', 'Ponte');
INSERT INTO tipo_logradouro VALUES ('RAMPA', 'Rampa');
INSERT INTO tipo_logradouro VALUES ('BURAC', 'Buraco');
INSERT INTO tipo_logradouro VALUES ('QUINT', 'Quinta');
INSERT INTO tipo_logradouro VALUES ('RETIR', 'Retiro');
INSERT INTO tipo_logradouro VALUES ('CAIS', 'Cais');
INSERT INTO tipo_logradouro VALUES ('FONTE', 'Fonte');
INSERT INTO tipo_logradouro VALUES ('PROLO', 'Prolongamento');
INSERT INTO tipo_logradouro VALUES ('PARAD', 'Parada');
INSERT INTO tipo_logradouro VALUES ('SETOR', 'Setor');
INSERT INTO tipo_logradouro VALUES ('RETA', 'Reta');
INSERT INTO tipo_logradouro VALUES ('MORRO', 'Morro');
INSERT INTO tipo_logradouro VALUES ('RODAN', 'Rodo Anel');
INSERT INTO tipo_logradouro VALUES ('RAMAL', 'Ramal');
INSERT INTO tipo_logradouro VALUES ('FERRO', 'Ferrovia');
INSERT INTO tipo_logradouro VALUES ('CANAL', 'Canal');
INSERT INTO tipo_logradouro VALUES ('FORT', 'Forte');
INSERT INTO tipo_logradouro VALUES ('DISTR', 'Distrito');
INSERT INTO tipo_logradouro VALUES ('ESTRE', 'Estrada Estadual');
INSERT INTO tipo_logradouro VALUES ('ESTRM', 'Estrada Municipal');
INSERT INTO tipo_logradouro VALUES ('ESTRP', 'Estrada Particular');
INSERT INTO tipo_logradouro VALUES ('ESTRV', 'Estrada Velha');
INSERT INTO tipo_logradouro VALUES ('ESTRC', 'Estrada Vicinal');
INSERT INTO tipo_logradouro VALUES ('CICLO', 'Ciclovia');
INSERT INTO tipo_logradouro VALUES ('GRANJ', 'Granja');
INSERT INTO tipo_logradouro VALUES ('FAZEN', 'Fazenda');
INSERT INTO tipo_logradouro VALUES ('CAMPO', 'Campo');
INSERT INTO tipo_logradouro VALUES ('FAVEL', 'Favela');
INSERT INTO tipo_logradouro VALUES ('CIRCU', 'Circular');
INSERT INTO tipo_logradouro VALUES ('CONTO', 'Contorno');
INSERT INTO tipo_logradouro VALUES ('RUALI', 'Rua de Ligao');
INSERT INTO tipo_logradouro VALUES ('ROTAT', 'Rotatria');
INSERT INTO tipo_logradouro VALUES ('NUCLE', 'Ncleo');
INSERT INTO tipo_logradouro VALUES ('ESTRL', 'Estrada de Ligao');
INSERT INTO tipo_logradouro VALUES ('ARTER', 'Artria');
INSERT INTO tipo_logradouro VALUES ('COMVI', 'Complexo Virio');
INSERT INTO tipo_logradouro VALUES ('CJOMU', 'Conjunto Mutiro');
INSERT INTO tipo_logradouro VALUES ('ESTA', 'Estao');
INSERT INTO tipo_logradouro VALUES ('BALAO', 'Balo');


SET search_path = acesso, pg_catalog;

--
-- Name: pk_funcao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY funcao
    ADD CONSTRAINT pk_funcao PRIMARY KEY (idfunc, idsis, idmen);


--
-- Name: pk_grupo; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupo
    ADD CONSTRAINT pk_grupo PRIMARY KEY (idgrp);


--
-- Name: pk_grupo_funcao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupo_funcao
    ADD CONSTRAINT pk_grupo_funcao PRIMARY KEY (idmen, idsis, idgrp, idfunc);


--
-- Name: pk_grupo_menu; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupo_menu
    ADD CONSTRAINT pk_grupo_menu PRIMARY KEY (idgrp, idsis, idmen);


--
-- Name: pk_grupo_operacao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupo_operacao
    ADD CONSTRAINT pk_grupo_operacao PRIMARY KEY (idfunc, idgrp, idsis, idmen, idope);


--
-- Name: pk_grupo_sistema; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupo_sistema
    ADD CONSTRAINT pk_grupo_sistema PRIMARY KEY (idsis, idgrp);


--
-- Name: pk_historico_senha; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY historico_senha
    ADD CONSTRAINT pk_historico_senha PRIMARY KEY (login, senha);


--
-- Name: pk_instituicao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY instituicao
    ADD CONSTRAINT pk_instituicao PRIMARY KEY (idins);


--
-- Name: pk_menu; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT pk_menu PRIMARY KEY (idsis, idmen);


--
-- Name: pk_operacao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY operacao
    ADD CONSTRAINT pk_operacao PRIMARY KEY (idope);


--
-- Name: pk_operacao_funcao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY operacao_funcao
    ADD CONSTRAINT pk_operacao_funcao PRIMARY KEY (idmen, idsis, idfunc, idope);


--
-- Name: pk_pessoa_instituicao; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa_instituicao
    ADD CONSTRAINT pk_pessoa_instituicao PRIMARY KEY (idins, idpes);


--
-- Name: pk_sistema; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY sistema
    ADD CONSTRAINT pk_sistema PRIMARY KEY (idsis);


--
-- Name: pk_usuario; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT pk_usuario PRIMARY KEY (login);


--
-- Name: pk_usuario_grupo; Type: CONSTRAINT; Schema: acesso; Owner: -; Tablespace:
--

ALTER TABLE ONLY usuario_grupo
    ADD CONSTRAINT pk_usuario_grupo PRIMARY KEY (idgrp, login);


SET search_path = alimentos, pg_catalog;

--
-- Name: pk_baixa_guia_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY baixa_guia_produto
    ADD CONSTRAINT pk_baixa_guia_produto PRIMARY KEY (idbap);


--
-- Name: pk_baixa_guia_remessa; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY baixa_guia_remessa
    ADD CONSTRAINT pk_baixa_guia_remessa PRIMARY KEY (idbai);


--
-- Name: pk_calendario; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario
    ADD CONSTRAINT pk_calendario PRIMARY KEY (idcad);


--
-- Name: pk_cardapio; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY cardapio
    ADD CONSTRAINT pk_cardapio PRIMARY KEY (idcar);


--
-- Name: pk_cardapio_faixa_unidade; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY cardapio_faixa_unidade
    ADD CONSTRAINT pk_cardapio_faixa_unidade PRIMARY KEY (idfeu, idcar);


--
-- Name: pk_cardapio_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY cardapio_produto
    ADD CONSTRAINT pk_cardapio_produto PRIMARY KEY (idcpr);


--
-- Name: pk_cardapio_receita; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY cardapio_receita
    ADD CONSTRAINT pk_cardapio_receita PRIMARY KEY (idcar, idrec);


--
-- Name: pk_cliente; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT pk_cliente PRIMARY KEY (idcli);


--
-- Name: pk_contrato; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY contrato
    ADD CONSTRAINT pk_contrato PRIMARY KEY (idcon);


--
-- Name: pk_contrato_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY contrato_produto
    ADD CONSTRAINT pk_contrato_produto PRIMARY KEY (idcop);


--
-- Name: pk_cp_quimico; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY composto_quimico
    ADD CONSTRAINT pk_cp_quimico PRIMARY KEY (idcom);


--
-- Name: pk_evento; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY evento
    ADD CONSTRAINT pk_evento PRIMARY KEY (ideve);


--
-- Name: pk_faixa_composto_quimico; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY faixa_composto_quimico
    ADD CONSTRAINT pk_faixa_composto_quimico PRIMARY KEY (idfcp);


--
-- Name: pk_faixa_etaria; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY faixa_etaria
    ADD CONSTRAINT pk_faixa_etaria PRIMARY KEY (idfae);


--
-- Name: pk_fornecedor; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY fornecedor
    ADD CONSTRAINT pk_fornecedor PRIMARY KEY (idfor);


--
-- Name: pk_grp_quimico; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupo_quimico
    ADD CONSTRAINT pk_grp_quimico PRIMARY KEY (idgrpq);


--
-- Name: pk_guia_produto_diario; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY guia_produto_diario
    ADD CONSTRAINT pk_guia_produto_diario PRIMARY KEY (idguiaprodiario);


--
-- Name: pk_guia_remessa; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT pk_guia_remessa PRIMARY KEY (idgui);


--
-- Name: pk_guia_remessa_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY guia_remessa_produto
    ADD CONSTRAINT pk_guia_remessa_produto PRIMARY KEY (idgup);


--
-- Name: pk_log_guia_remessa; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY log_guia_remessa
    ADD CONSTRAINT pk_log_guia_remessa PRIMARY KEY (idlogguia);


--
-- Name: pk_medidas_caseiras; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY medidas_caseiras
    ADD CONSTRAINT pk_medidas_caseiras PRIMARY KEY (idmedcas, idcli);


--
-- Name: pk_pessoa; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa
    ADD CONSTRAINT pk_pessoa PRIMARY KEY (idpes);


--
-- Name: pk_prod_cp_quimico; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY produto_composto_quimico
    ADD CONSTRAINT pk_prod_cp_quimico PRIMARY KEY (idpcq);


--
-- Name: pk_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY produto
    ADD CONSTRAINT pk_produto PRIMARY KEY (idpro);


--
-- Name: pk_produto_fornecedor; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY produto_fornecedor
    ADD CONSTRAINT pk_produto_fornecedor PRIMARY KEY (idprf);


--
-- Name: pk_produto_medida_caseira; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY produto_medida_caseira
    ADD CONSTRAINT pk_produto_medida_caseira PRIMARY KEY (idpmc);


--
-- Name: pk_rec_cp_quimico; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY receita_composto_quimico
    ADD CONSTRAINT pk_rec_cp_quimico PRIMARY KEY (idrcq);


--
-- Name: pk_rec_prod; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY receita_produto
    ADD CONSTRAINT pk_rec_prod PRIMARY KEY (idrpr);


--
-- Name: pk_receita; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY receita
    ADD CONSTRAINT pk_receita PRIMARY KEY (idrec);


--
-- Name: pk_tipo_unidade; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_unidade
    ADD CONSTRAINT pk_tipo_unidade PRIMARY KEY (idtip);


--
-- Name: pk_tp_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_produto
    ADD CONSTRAINT pk_tp_produto PRIMARY KEY (idtip);


--
-- Name: pk_tp_refeicao; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_refeicao
    ADD CONSTRAINT pk_tp_refeicao PRIMARY KEY (idtre);


--
-- Name: pk_uni_faixa_etaria; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY unidade_faixa_etaria
    ADD CONSTRAINT pk_uni_faixa_etaria PRIMARY KEY (idfeu);


--
-- Name: pk_uni_produto; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY unidade_produto
    ADD CONSTRAINT pk_uni_produto PRIMARY KEY (idunp, idcli);


--
-- Name: pk_unidade_atendida; Type: CONSTRAINT; Schema: alimentos; Owner: -; Tablespace:
--

ALTER TABLE ONLY unidade_atendida
    ADD CONSTRAINT pk_unidade_atendida PRIMARY KEY (iduni);


SET search_path = cadastro, pg_catalog;

--
-- Name: fisica_foto_pkey; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fisica_foto
    ADD CONSTRAINT fisica_foto_pkey PRIMARY KEY (idpes);


--
-- Name: fisica_sangue_pkey; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fisica_sangue
    ADD CONSTRAINT fisica_sangue_pkey PRIMARY KEY (idpes);


--
-- Name: pk_aviso_nome; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY aviso_nome
    ADD CONSTRAINT pk_aviso_nome PRIMARY KEY (idpes, aviso);


--
-- Name: pk_cadastro_escolaridade; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY deficiencia
    ADD CONSTRAINT pk_cadastro_escolaridade PRIMARY KEY (cod_deficiencia);


--
-- Name: pk_documento; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT pk_documento PRIMARY KEY (idpes);


--
-- Name: pk_endereco_externo; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT pk_endereco_externo PRIMARY KEY (idpes, tipo);


--
-- Name: pk_endereco_pessoa; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT pk_endereco_pessoa PRIMARY KEY (idpes, tipo);


--
-- Name: pk_escolaridade; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY escolaridade
    ADD CONSTRAINT pk_escolaridade PRIMARY KEY (idesco);


--
-- Name: pk_estado_civil; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY estado_civil
    ADD CONSTRAINT pk_estado_civil PRIMARY KEY (ideciv);


--
-- Name: pk_fisica; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT pk_fisica PRIMARY KEY (idpes);


--
-- Name: pk_fisica_cpf; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fisica_cpf
    ADD CONSTRAINT pk_fisica_cpf PRIMARY KEY (idpes);


--
-- Name: pk_fisica_deficiencia; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fisica_deficiencia
    ADD CONSTRAINT pk_fisica_deficiencia PRIMARY KEY (ref_idpes, ref_cod_deficiencia);


--
-- Name: pk_fisica_raca; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fisica_raca
    ADD CONSTRAINT pk_fisica_raca PRIMARY KEY (ref_idpes);


--
-- Name: pk_fone_pessoa; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY fone_pessoa
    ADD CONSTRAINT pk_fone_pessoa PRIMARY KEY (idpes, tipo);


--
-- Name: pk_funcionario; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT pk_funcionario PRIMARY KEY (matricula, idins);


--
-- Name: pk_juridica; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY juridica
    ADD CONSTRAINT pk_juridica PRIMARY KEY (idpes);


--
-- Name: pk_ocupacao; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY ocupacao
    ADD CONSTRAINT pk_ocupacao PRIMARY KEY (idocup);


--
-- Name: pk_orgao_emissor_rg; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY orgao_emissor_rg
    ADD CONSTRAINT pk_orgao_emissor_rg PRIMARY KEY (idorg_rg);


--
-- Name: pk_pessoa; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa
    ADD CONSTRAINT pk_pessoa PRIMARY KEY (idpes);


--
-- Name: pk_pessoa_fonetico; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa_fonetico
    ADD CONSTRAINT pk_pessoa_fonetico PRIMARY KEY (fonema, idpes);


--
-- Name: pk_socio; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT pk_socio PRIMARY KEY (idpes_juridica, idpes_fisica);


--
-- Name: raca_pkey; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY raca
    ADD CONSTRAINT raca_pkey PRIMARY KEY (cod_raca);


--
-- Name: religiao_pkey; Type: CONSTRAINT; Schema: cadastro; Owner: -; Tablespace:
--

ALTER TABLE ONLY religiao
    ADD CONSTRAINT religiao_pkey PRIMARY KEY (cod_religiao);


SET search_path = consistenciacao, pg_catalog;

--
-- Name: pk_campo_consistenciacao; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY campo_consistenciacao
    ADD CONSTRAINT pk_campo_consistenciacao PRIMARY KEY (idcam);


--
-- Name: pk_campo_metadado; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY campo_metadado
    ADD CONSTRAINT pk_campo_metadado PRIMARY KEY (id_campo_met);


--
-- Name: pk_confrontacao; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY confrontacao
    ADD CONSTRAINT pk_confrontacao PRIMARY KEY (idcon);


--
-- Name: pk_fonte; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY fonte
    ADD CONSTRAINT pk_fonte PRIMARY KEY (idfon);


--
-- Name: pk_historico_campo; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY historico_campo
    ADD CONSTRAINT pk_historico_campo PRIMARY KEY (idpes, idcam);


--
-- Name: pk_inc_pessoa_possivel; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY incoerencia_pessoa_possivel
    ADD CONSTRAINT pk_inc_pessoa_possivel PRIMARY KEY (idinc, idpes);


--
-- Name: pk_incoerencia; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY incoerencia
    ADD CONSTRAINT pk_incoerencia PRIMARY KEY (idinc);


--
-- Name: pk_incoerencia_documento; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY incoerencia_documento
    ADD CONSTRAINT pk_incoerencia_documento PRIMARY KEY (id_inc_doc);


--
-- Name: pk_incoerencia_endereco; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY incoerencia_endereco
    ADD CONSTRAINT pk_incoerencia_endereco PRIMARY KEY (id_inc_end);


--
-- Name: pk_incoerencia_fone; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY incoerencia_fone
    ADD CONSTRAINT pk_incoerencia_fone PRIMARY KEY (id_inc_fone);


--
-- Name: pk_incoerencia_tipo_incoerencia; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY incoerencia_tipo_incoerencia
    ADD CONSTRAINT pk_incoerencia_tipo_incoerencia PRIMARY KEY (id_tipo_inc, idinc);


--
-- Name: pk_metadado; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY metadado
    ADD CONSTRAINT pk_metadado PRIMARY KEY (idmet);


--
-- Name: pk_ocorrencia_regra_campo; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY ocorrencia_regra_campo
    ADD CONSTRAINT pk_ocorrencia_regra_campo PRIMARY KEY (idreg, conteudo_padrao);


--
-- Name: pk_regra_campo; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY regra_campo
    ADD CONSTRAINT pk_regra_campo PRIMARY KEY (idreg);


--
-- Name: pk_temp_cadastro_unificacao_cmf; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY temp_cadastro_unificacao_cmf
    ADD CONSTRAINT pk_temp_cadastro_unificacao_cmf PRIMARY KEY (idpes);


--
-- Name: pk_temp_cadastro_unificacao_siam; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY temp_cadastro_unificacao_siam
    ADD CONSTRAINT pk_temp_cadastro_unificacao_siam PRIMARY KEY (idpes);


--
-- Name: pk_tipo_incoerencia; Type: CONSTRAINT; Schema: consistenciacao; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_incoerencia
    ADD CONSTRAINT pk_tipo_incoerencia PRIMARY KEY (id_tipo_inc);


SET search_path = modules, pg_catalog;

--
-- Name: area_conhecimento_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY area_conhecimento
    ADD CONSTRAINT area_conhecimento_pkey PRIMARY KEY (id, instituicao_id);


--
-- Name: calendario_turma_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario_turma
    ADD CONSTRAINT calendario_turma_pk PRIMARY KEY (calendario_ano_letivo_id, ano, mes, dia, turma_id);


--
-- Name: componente_curricular_ano_escolar_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY componente_curricular_ano_escolar
    ADD CONSTRAINT componente_curricular_ano_escolar_pkey PRIMARY KEY (componente_curricular_id, ano_escolar_id);


--
-- Name: componente_curricular_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY componente_curricular
    ADD CONSTRAINT componente_curricular_pkey PRIMARY KEY (id, instituicao_id);


--
-- Name: componente_curricular_turma_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY componente_curricular_turma
    ADD CONSTRAINT componente_curricular_turma_pkey PRIMARY KEY (componente_curricular_id, turma_id);


--
-- Name: docente_licenciatura_curso_unique; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY docente_licenciatura
    ADD CONSTRAINT docente_licenciatura_curso_unique UNIQUE (servidor_id, curso_id, ies_id);


--
-- Name: docente_licenciatura_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY docente_licenciatura
    ADD CONSTRAINT docente_licenciatura_pk PRIMARY KEY (id);


--
-- Name: educacenso_cod_aluno_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY educacenso_cod_aluno
    ADD CONSTRAINT educacenso_cod_aluno_pk PRIMARY KEY (cod_aluno, cod_aluno_inep);


--
-- Name: educacenso_cod_docente_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY educacenso_cod_docente
    ADD CONSTRAINT educacenso_cod_docente_pk PRIMARY KEY (cod_servidor, cod_docente_inep);


--
-- Name: educacenso_curso_superior_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY educacenso_curso_superior
    ADD CONSTRAINT educacenso_curso_superior_pk PRIMARY KEY (id);


--
-- Name: educacenso_ies_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY educacenso_ies
    ADD CONSTRAINT educacenso_ies_pk PRIMARY KEY (id);


--
-- Name: falta_aluno_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY falta_aluno
    ADD CONSTRAINT falta_aluno_pkey PRIMARY KEY (id);


--
-- Name: falta_componente_curricular_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY falta_componente_curricular
    ADD CONSTRAINT falta_componente_curricular_pkey PRIMARY KEY (id);


--
-- Name: falta_geral_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY falta_geral
    ADD CONSTRAINT falta_geral_pkey PRIMARY KEY (id);


--
-- Name: formula_media_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY formula_media
    ADD CONSTRAINT formula_media_pkey PRIMARY KEY (id, instituicao_id);


--
-- Name: nota_aluno_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_pkey PRIMARY KEY (id);


--
-- Name: nota_componente_curricular_media_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY nota_componente_curricular_media
    ADD CONSTRAINT nota_componente_curricular_media_pkey PRIMARY KEY (nota_aluno_id, componente_curricular_id);


--
-- Name: nota_componente_curricular_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY nota_componente_curricular
    ADD CONSTRAINT nota_componente_curricular_pkey PRIMARY KEY (id);


--
-- Name: parecer_aluno_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY parecer_aluno
    ADD CONSTRAINT parecer_aluno_pkey PRIMARY KEY (id);


--
-- Name: parecer_componente_curricular_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY parecer_componente_curricular
    ADD CONSTRAINT parecer_componente_curricular_pkey PRIMARY KEY (id);


--
-- Name: parecer_geral_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY parecer_geral
    ADD CONSTRAINT parecer_geral_pkey PRIMARY KEY (id);


--
-- Name: regra_avaliacao_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY regra_avaliacao
    ADD CONSTRAINT regra_avaliacao_pkey PRIMARY KEY (id, instituicao_id);


--
-- Name: tabela_arredondamento_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY tabela_arredondamento
    ADD CONSTRAINT tabela_arredondamento_pkey PRIMARY KEY (id, instituicao_id);


--
-- Name: tabela_arredondamento_valor_pkey; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY tabela_arredondamento_valor
    ADD CONSTRAINT tabela_arredondamento_valor_pkey PRIMARY KEY (id);


--
-- Name: transporte_aluno_pk; Type: CONSTRAINT; Schema: modules; Owner: -; Tablespace:
--

ALTER TABLE ONLY transporte_aluno
    ADD CONSTRAINT transporte_aluno_pk PRIMARY KEY (aluno_id);


SET search_path = pmiacoes, pg_catalog;

--
-- Name: acao_governo_arquivo_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo_arquivo
    ADD CONSTRAINT acao_governo_arquivo_pkey PRIMARY KEY (cod_acao_governo_arquivo);


--
-- Name: acao_governo_categoria_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo_categoria
    ADD CONSTRAINT acao_governo_categoria_pkey PRIMARY KEY (ref_cod_categoria, ref_cod_acao_governo);


--
-- Name: acao_governo_foto_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo_foto
    ADD CONSTRAINT acao_governo_foto_pkey PRIMARY KEY (cod_acao_governo_foto);


--
-- Name: acao_governo_foto_portal_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo_foto_portal
    ADD CONSTRAINT acao_governo_foto_portal_pkey PRIMARY KEY (ref_cod_acao_governo, ref_cod_foto_portal);


--
-- Name: acao_governo_noticia_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo_noticia
    ADD CONSTRAINT acao_governo_noticia_pkey PRIMARY KEY (ref_cod_acao_governo, ref_cod_not_portal);


--
-- Name: acao_governo_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo
    ADD CONSTRAINT acao_governo_pkey PRIMARY KEY (cod_acao_governo);


--
-- Name: acao_governo_setor_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY acao_governo_setor
    ADD CONSTRAINT acao_governo_setor_pkey PRIMARY KEY (ref_cod_acao_governo, ref_cod_setor);


--
-- Name: categoria_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY categoria
    ADD CONSTRAINT categoria_pkey PRIMARY KEY (cod_categoria);


--
-- Name: secretaria_responsavel_pkey; Type: CONSTRAINT; Schema: pmiacoes; Owner: -; Tablespace:
--

ALTER TABLE ONLY secretaria_responsavel
    ADD CONSTRAINT secretaria_responsavel_pkey PRIMARY KEY (ref_cod_setor);


SET search_path = pmicontrolesis, pg_catalog;

--
-- Name: acontecimento_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY acontecimento
    ADD CONSTRAINT acontecimento_pkey PRIMARY KEY (cod_acontecimento);


--
-- Name: artigo_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY artigo
    ADD CONSTRAINT artigo_pkey PRIMARY KEY (cod_artigo);


--
-- Name: foto_evento_pk; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY foto_evento
    ADD CONSTRAINT foto_evento_pk PRIMARY KEY (cod_foto_evento);


--
-- Name: foto_vinc_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY foto_vinc
    ADD CONSTRAINT foto_vinc_pkey PRIMARY KEY (cod_foto_vinc);


--
-- Name: itinerario_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY itinerario
    ADD CONSTRAINT itinerario_pkey PRIMARY KEY (cod_itinerario);


--
-- Name: menu_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT menu_pkey PRIMARY KEY (cod_menu);


--
-- Name: menu_portal_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu_portal
    ADD CONSTRAINT menu_portal_pkey PRIMARY KEY (cod_menu_portal);


--
-- Name: portais_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY portais
    ADD CONSTRAINT portais_pkey PRIMARY KEY (cod_portais);


--
-- Name: servicos_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY servicos
    ADD CONSTRAINT servicos_pkey PRIMARY KEY (cod_servicos);


--
-- Name: sistema_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY sistema
    ADD CONSTRAINT sistema_pkey PRIMARY KEY (cod_sistema);


--
-- Name: submenu_portal_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY submenu_portal
    ADD CONSTRAINT submenu_portal_pkey PRIMARY KEY (cod_submenu_portal);


--
-- Name: telefones_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY telefones
    ADD CONSTRAINT telefones_pkey PRIMARY KEY (cod_telefones);


--
-- Name: tipo_acontecimento_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_acontecimento
    ADD CONSTRAINT tipo_acontecimento_pkey PRIMARY KEY (cod_tipo_acontecimento);


--
-- Name: topo_portal_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY topo_portal
    ADD CONSTRAINT topo_portal_pkey PRIMARY KEY (cod_topo_portal);


--
-- Name: tutormenu_pkey; Type: CONSTRAINT; Schema: pmicontrolesis; Owner: -; Tablespace:
--

ALTER TABLE ONLY tutormenu
    ADD CONSTRAINT tutormenu_pkey PRIMARY KEY (cod_tutormenu);


SET search_path = pmidrh, pg_catalog;

--
-- Name: diaria_grupo_pkey; Type: CONSTRAINT; Schema: pmidrh; Owner: -; Tablespace:
--

ALTER TABLE ONLY diaria_grupo
    ADD CONSTRAINT diaria_grupo_pkey PRIMARY KEY (cod_diaria_grupo);


--
-- Name: diaria_pkey; Type: CONSTRAINT; Schema: pmidrh; Owner: -; Tablespace:
--

ALTER TABLE ONLY diaria
    ADD CONSTRAINT diaria_pkey PRIMARY KEY (cod_diaria);


--
-- Name: diaria_valores_pkey; Type: CONSTRAINT; Schema: pmidrh; Owner: -; Tablespace:
--

ALTER TABLE ONLY diaria_valores
    ADD CONSTRAINT diaria_valores_pkey PRIMARY KEY (cod_diaria_valores);


--
-- Name: setor_pkey; Type: CONSTRAINT; Schema: pmidrh; Owner: -; Tablespace:
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT setor_pkey PRIMARY KEY (cod_setor);


SET search_path = pmieducar, pg_catalog;

--
-- Name: acervo_acervo_assunto_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_acervo_assunto
    ADD CONSTRAINT acervo_acervo_assunto_pkey PRIMARY KEY (ref_cod_acervo, ref_cod_acervo_assunto);


--
-- Name: acervo_acervo_autor_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_acervo_autor
    ADD CONSTRAINT acervo_acervo_autor_pkey PRIMARY KEY (ref_cod_acervo_autor, ref_cod_acervo);


--
-- Name: acervo_assunto_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_assunto
    ADD CONSTRAINT acervo_assunto_pkey PRIMARY KEY (cod_acervo_assunto);


--
-- Name: acervo_autor_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_autor
    ADD CONSTRAINT acervo_autor_pkey PRIMARY KEY (cod_acervo_autor);


--
-- Name: acervo_colecao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_colecao
    ADD CONSTRAINT acervo_colecao_pkey PRIMARY KEY (cod_acervo_colecao);


--
-- Name: acervo_editora_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_editora
    ADD CONSTRAINT acervo_editora_pkey PRIMARY KEY (cod_acervo_editora);


--
-- Name: acervo_idioma_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo_idioma
    ADD CONSTRAINT acervo_idioma_pkey PRIMARY KEY (cod_acervo_idioma);


--
-- Name: acervo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_pkey PRIMARY KEY (cod_acervo);


--
-- Name: aluno_beneficio_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY aluno_beneficio
    ADD CONSTRAINT aluno_beneficio_pkey PRIMARY KEY (cod_aluno_beneficio);


--
-- Name: aluno_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY aluno
    ADD CONSTRAINT aluno_pkey PRIMARY KEY (cod_aluno);


--
-- Name: aluno_ref_idpes_un; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY aluno
    ADD CONSTRAINT aluno_ref_idpes_un UNIQUE (ref_idpes);


--
-- Name: ano_letivo_modulo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY ano_letivo_modulo
    ADD CONSTRAINT ano_letivo_modulo_pkey PRIMARY KEY (ref_ano, ref_ref_cod_escola, sequencial, ref_cod_modulo);


--
-- Name: avaliacao_desempenho_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY avaliacao_desempenho
    ADD CONSTRAINT avaliacao_desempenho_pkey PRIMARY KEY (sequencial, ref_cod_servidor, ref_ref_cod_instituicao);


--
-- Name: biblioteca_dia_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY biblioteca_dia
    ADD CONSTRAINT biblioteca_dia_pkey PRIMARY KEY (ref_cod_biblioteca, dia);


--
-- Name: biblioteca_feriados_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY biblioteca_feriados
    ADD CONSTRAINT biblioteca_feriados_pkey PRIMARY KEY (cod_feriado);


--
-- Name: biblioteca_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY biblioteca
    ADD CONSTRAINT biblioteca_pkey PRIMARY KEY (cod_biblioteca);


--
-- Name: biblioteca_usuario_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY biblioteca_usuario
    ADD CONSTRAINT biblioteca_usuario_pkey PRIMARY KEY (ref_cod_biblioteca, ref_cod_usuario);


--
-- Name: calendario_ano_letivo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario_ano_letivo
    ADD CONSTRAINT calendario_ano_letivo_pkey PRIMARY KEY (cod_calendario_ano_letivo);


--
-- Name: calendario_anotacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario_anotacao
    ADD CONSTRAINT calendario_anotacao_pkey PRIMARY KEY (cod_calendario_anotacao);


--
-- Name: calendario_dia_anotacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario_dia_anotacao
    ADD CONSTRAINT calendario_dia_anotacao_pkey PRIMARY KEY (ref_dia, ref_mes, ref_ref_cod_calendario_ano_letivo, ref_cod_calendario_anotacao);


--
-- Name: calendario_dia_motivo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario_dia_motivo
    ADD CONSTRAINT calendario_dia_motivo_pkey PRIMARY KEY (cod_calendario_dia_motivo);


--
-- Name: calendario_dia_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY calendario_dia
    ADD CONSTRAINT calendario_dia_pkey PRIMARY KEY (ref_cod_calendario_ano_letivo, mes, dia);


--
-- Name: categoria_nivel_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY categoria_nivel
    ADD CONSTRAINT categoria_nivel_pkey PRIMARY KEY (cod_categoria_nivel);


--
-- Name: cliente_login_ukey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT cliente_login_ukey UNIQUE (login);


--
-- Name: cliente_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT cliente_pkey PRIMARY KEY (cod_cliente);


--
-- Name: cliente_ref_idpes_ukey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT cliente_ref_idpes_ukey UNIQUE (ref_idpes);


--
-- Name: cliente_suspensao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente_suspensao
    ADD CONSTRAINT cliente_suspensao_pkey PRIMARY KEY (sequencial, ref_cod_cliente, ref_cod_motivo_suspensao);


--
-- Name: cliente_tipo_cliente_pk; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente_tipo_cliente
    ADD CONSTRAINT cliente_tipo_cliente_pk PRIMARY KEY (ref_cod_cliente_tipo, ref_cod_cliente);


--
-- Name: cliente_tipo_exemplar_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente_tipo_exemplar_tipo
    ADD CONSTRAINT cliente_tipo_exemplar_tipo_pkey PRIMARY KEY (ref_cod_cliente_tipo, ref_cod_exemplar_tipo);


--
-- Name: cliente_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY cliente_tipo
    ADD CONSTRAINT cliente_tipo_pkey PRIMARY KEY (cod_cliente_tipo);


--
-- Name: cod_dispensa_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY dispensa_disciplina
    ADD CONSTRAINT cod_dispensa_pkey PRIMARY KEY (cod_dispensa);


--
-- Name: coffebreak_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY coffebreak_tipo
    ADD CONSTRAINT coffebreak_tipo_pkey PRIMARY KEY (cod_coffebreak_tipo);


--
-- Name: curso_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_pkey PRIMARY KEY (cod_curso);


--
-- Name: disciplina_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY disciplina
    ADD CONSTRAINT disciplina_pkey PRIMARY KEY (cod_disciplina);


--
-- Name: disciplina_serie_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY disciplina_serie
    ADD CONSTRAINT disciplina_serie_pkey PRIMARY KEY (ref_cod_disciplina, ref_cod_serie);


--
-- Name: disciplina_topico_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY disciplina_topico
    ADD CONSTRAINT disciplina_topico_pkey PRIMARY KEY (cod_disciplina_topico);


--
-- Name: escola_ano_letivo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_ano_letivo
    ADD CONSTRAINT escola_ano_letivo_pkey PRIMARY KEY (ref_cod_escola, ano);


--
-- Name: escola_complemento_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_complemento
    ADD CONSTRAINT escola_complemento_pkey PRIMARY KEY (ref_cod_escola);


--
-- Name: escola_curso_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_curso
    ADD CONSTRAINT escola_curso_pkey PRIMARY KEY (ref_cod_escola, ref_cod_curso);


--
-- Name: escola_localizacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_localizacao
    ADD CONSTRAINT escola_localizacao_pkey PRIMARY KEY (cod_escola_localizacao);


--
-- Name: escola_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_pkey PRIMARY KEY (cod_escola);


--
-- Name: escola_rede_ensino_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_rede_ensino
    ADD CONSTRAINT escola_rede_ensino_pkey PRIMARY KEY (cod_escola_rede_ensino);


--
-- Name: escola_serie_disciplina_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_serie_disciplina
    ADD CONSTRAINT escola_serie_disciplina_pkey PRIMARY KEY (ref_ref_cod_serie, ref_ref_cod_escola, ref_cod_disciplina);


--
-- Name: escola_serie_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY escola_serie
    ADD CONSTRAINT escola_serie_pkey PRIMARY KEY (ref_cod_escola, ref_cod_serie);


--
-- Name: exemplar_emprestimo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY exemplar_emprestimo
    ADD CONSTRAINT exemplar_emprestimo_pkey PRIMARY KEY (cod_emprestimo);


--
-- Name: exemplar_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_pkey PRIMARY KEY (cod_exemplar);


--
-- Name: exemplar_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY exemplar_tipo
    ADD CONSTRAINT exemplar_tipo_pkey PRIMARY KEY (cod_exemplar_tipo);


--
-- Name: falta_aluno_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY falta_aluno
    ADD CONSTRAINT falta_aluno_pkey PRIMARY KEY (cod_falta_aluno);


--
-- Name: falta_atraso_compensado_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY falta_atraso_compensado
    ADD CONSTRAINT falta_atraso_compensado_pkey PRIMARY KEY (cod_compensado);


--
-- Name: falta_atraso_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY falta_atraso
    ADD CONSTRAINT falta_atraso_pkey PRIMARY KEY (cod_falta_atraso);


--
-- Name: faltas_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY faltas
    ADD CONSTRAINT faltas_pkey PRIMARY KEY (ref_cod_matricula, sequencial);


--
-- Name: fonte_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY fonte
    ADD CONSTRAINT fonte_pkey PRIMARY KEY (cod_fonte);


--
-- Name: funcao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY funcao
    ADD CONSTRAINT funcao_pkey PRIMARY KEY (cod_funcao);


--
-- Name: habilitacao_curso_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY habilitacao_curso
    ADD CONSTRAINT habilitacao_curso_pkey PRIMARY KEY (ref_cod_habilitacao, ref_cod_curso);


--
-- Name: habilitacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY habilitacao
    ADD CONSTRAINT habilitacao_pkey PRIMARY KEY (cod_habilitacao);


--
-- Name: historico_disciplinas_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY historico_disciplinas
    ADD CONSTRAINT historico_disciplinas_pkey PRIMARY KEY (sequencial, ref_ref_cod_aluno, ref_sequencial);


--
-- Name: historico_escolar_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY historico_escolar
    ADD CONSTRAINT historico_escolar_pkey PRIMARY KEY (ref_cod_aluno, sequencial);


--
-- Name: infra_comodo_funcao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY infra_comodo_funcao
    ADD CONSTRAINT infra_comodo_funcao_pkey PRIMARY KEY (cod_infra_comodo_funcao);


--
-- Name: infra_predio_comodo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY infra_predio_comodo
    ADD CONSTRAINT infra_predio_comodo_pkey PRIMARY KEY (cod_infra_predio_comodo);


--
-- Name: infra_predio_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY infra_predio
    ADD CONSTRAINT infra_predio_pkey PRIMARY KEY (cod_infra_predio);


--
-- Name: instituicao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY instituicao
    ADD CONSTRAINT instituicao_pkey PRIMARY KEY (cod_instituicao);


--
-- Name: material_didatico_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY material_didatico
    ADD CONSTRAINT material_didatico_pkey PRIMARY KEY (cod_material_didatico);


--
-- Name: material_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY material_tipo
    ADD CONSTRAINT material_tipo_pkey PRIMARY KEY (cod_material_tipo);


--
-- Name: matricula_excessao_pk; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY matricula_excessao
    ADD CONSTRAINT matricula_excessao_pk PRIMARY KEY (cod_aluno_excessao);


--
-- Name: matricula_ocorrencia_disciplinar_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY matricula_ocorrencia_disciplinar
    ADD CONSTRAINT matricula_ocorrencia_disciplinar_pkey PRIMARY KEY (ref_cod_matricula, ref_cod_tipo_ocorrencia_disciplinar, sequencial);


--
-- Name: matricula_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY matricula
    ADD CONSTRAINT matricula_pkey PRIMARY KEY (cod_matricula);


--
-- Name: matricula_turma_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY matricula_turma
    ADD CONSTRAINT matricula_turma_pkey PRIMARY KEY (ref_cod_matricula, ref_cod_turma, sequencial);


--
-- Name: menu_tipo_usuario_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu_tipo_usuario
    ADD CONSTRAINT menu_tipo_usuario_pkey PRIMARY KEY (ref_cod_tipo_usuario, ref_cod_menu_submenu);


--
-- Name: modulo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY modulo
    ADD CONSTRAINT modulo_pkey PRIMARY KEY (cod_modulo);


--
-- Name: motivo_afastamento_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY motivo_afastamento
    ADD CONSTRAINT motivo_afastamento_pkey PRIMARY KEY (cod_motivo_afastamento);


--
-- Name: motivo_baixa_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY motivo_baixa
    ADD CONSTRAINT motivo_baixa_pkey PRIMARY KEY (cod_motivo_baixa);


--
-- Name: motivo_suspensao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY motivo_suspensao
    ADD CONSTRAINT motivo_suspensao_pkey PRIMARY KEY (cod_motivo_suspensao);


--
-- Name: nivel_ensino_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY nivel_ensino
    ADD CONSTRAINT nivel_ensino_pkey PRIMARY KEY (cod_nivel_ensino);


--
-- Name: nivel_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY nivel
    ADD CONSTRAINT nivel_pkey PRIMARY KEY (cod_nivel);


--
-- Name: nota_aluno_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_pkey PRIMARY KEY (cod_nota_aluno);


--
-- Name: operador_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY operador
    ADD CONSTRAINT operador_pkey PRIMARY KEY (cod_operador);


--
-- Name: pagamento_multa_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY pagamento_multa
    ADD CONSTRAINT pagamento_multa_pkey PRIMARY KEY (cod_pagamento_multa);


--
-- Name: pre_requisito_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY pre_requisito
    ADD CONSTRAINT pre_requisito_pkey PRIMARY KEY (cod_pre_requisito);


--
-- Name: quadro_horario_horarios_aux_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY quadro_horario_horarios_aux
    ADD CONSTRAINT quadro_horario_horarios_aux_pkey PRIMARY KEY (ref_cod_quadro_horario, sequencial);


--
-- Name: quadro_horario_horarios_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY quadro_horario_horarios
    ADD CONSTRAINT quadro_horario_horarios_pkey PRIMARY KEY (ref_cod_quadro_horario, sequencial);


--
-- Name: quadro_horario_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY quadro_horario
    ADD CONSTRAINT quadro_horario_pkey PRIMARY KEY (cod_quadro_horario);


--
-- Name: religiao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY religiao
    ADD CONSTRAINT religiao_pkey PRIMARY KEY (cod_religiao);


--
-- Name: reserva_vaga_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY reserva_vaga
    ADD CONSTRAINT reserva_vaga_pkey PRIMARY KEY (cod_reserva_vaga);


--
-- Name: reservas_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY reservas
    ADD CONSTRAINT reservas_pkey PRIMARY KEY (cod_reserva);


--
-- Name: sequencia_serie_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY sequencia_serie
    ADD CONSTRAINT sequencia_serie_pkey PRIMARY KEY (ref_serie_origem, ref_serie_destino);


--
-- Name: serie_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY serie
    ADD CONSTRAINT serie_pkey PRIMARY KEY (cod_serie);


--
-- Name: serie_pre_requisito_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY serie_pre_requisito
    ADD CONSTRAINT serie_pre_requisito_pkey PRIMARY KEY (ref_cod_pre_requisito, ref_cod_operador, ref_cod_serie);


--
-- Name: servidor_afastamento_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_afastamento
    ADD CONSTRAINT servidor_afastamento_pkey PRIMARY KEY (ref_cod_servidor, sequencial, ref_ref_cod_instituicao);


--
-- Name: servidor_alocacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_alocacao
    ADD CONSTRAINT servidor_alocacao_pkey PRIMARY KEY (cod_servidor_alocacao);


--
-- Name: servidor_curso_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_curso
    ADD CONSTRAINT servidor_curso_pkey PRIMARY KEY (cod_servidor_curso);


--
-- Name: servidor_cuso_ministra_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_curso_ministra
    ADD CONSTRAINT servidor_cuso_ministra_pkey PRIMARY KEY (ref_cod_curso, ref_ref_cod_instituicao, ref_cod_servidor);


--
-- Name: servidor_disciplina_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_disciplina
    ADD CONSTRAINT servidor_disciplina_pkey PRIMARY KEY (ref_cod_disciplina, ref_ref_cod_instituicao, ref_cod_servidor, ref_cod_curso);


--
-- Name: servidor_formacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_formacao
    ADD CONSTRAINT servidor_formacao_pkey PRIMARY KEY (cod_formacao);


--
-- Name: servidor_funcao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_funcao
    ADD CONSTRAINT servidor_funcao_pkey PRIMARY KEY (ref_ref_cod_instituicao, ref_cod_servidor, ref_cod_funcao);


--
-- Name: servidor_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor
    ADD CONSTRAINT servidor_pkey PRIMARY KEY (cod_servidor, ref_cod_instituicao);


--
-- Name: servidor_titulo_concurso_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY servidor_titulo_concurso
    ADD CONSTRAINT servidor_titulo_concurso_pkey PRIMARY KEY (cod_servidor_titulo);


--
-- Name: situacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY situacao
    ADD CONSTRAINT situacao_pkey PRIMARY KEY (cod_situacao);


--
-- Name: subnivel_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY subnivel
    ADD CONSTRAINT subnivel_pkey PRIMARY KEY (cod_subnivel);


--
-- Name: tipo_avaliacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_avaliacao
    ADD CONSTRAINT tipo_avaliacao_pkey PRIMARY KEY (cod_tipo_avaliacao);


--
-- Name: tipo_avaliacao_valores_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_avaliacao_valores
    ADD CONSTRAINT tipo_avaliacao_valores_pkey PRIMARY KEY (ref_cod_tipo_avaliacao, sequencial);


--
-- Name: tipo_dispensa_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_dispensa
    ADD CONSTRAINT tipo_dispensa_pkey PRIMARY KEY (cod_tipo_dispensa);


--
-- Name: tipo_ensino_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_ensino
    ADD CONSTRAINT tipo_ensino_pkey PRIMARY KEY (cod_tipo_ensino);


--
-- Name: tipo_ocorrencia_disciplinar_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_ocorrencia_disciplinar
    ADD CONSTRAINT tipo_ocorrencia_disciplinar_pkey PRIMARY KEY (cod_tipo_ocorrencia_disciplinar);


--
-- Name: tipo_regime_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_regime
    ADD CONSTRAINT tipo_regime_pkey PRIMARY KEY (cod_tipo_regime);


--
-- Name: tipo_usuario_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_usuario
    ADD CONSTRAINT tipo_usuario_pkey PRIMARY KEY (cod_tipo_usuario);


--
-- Name: transferencia_solicitacao_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY transferencia_solicitacao
    ADD CONSTRAINT transferencia_solicitacao_pkey PRIMARY KEY (cod_transferencia_solicitacao);


--
-- Name: transferencia_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY transferencia_tipo
    ADD CONSTRAINT transferencia_tipo_pkey PRIMARY KEY (cod_transferencia_tipo);


--
-- Name: turma_dia_semana_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY turma_dia_semana
    ADD CONSTRAINT turma_dia_semana_pkey PRIMARY KEY (dia_semana, ref_cod_turma);


--
-- Name: turma_modulo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY turma_modulo
    ADD CONSTRAINT turma_modulo_pkey PRIMARY KEY (ref_cod_turma, ref_cod_modulo, sequencial);


--
-- Name: turma_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_pkey PRIMARY KEY (cod_turma);


--
-- Name: turma_tipo_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY turma_tipo
    ADD CONSTRAINT turma_tipo_pkey PRIMARY KEY (cod_turma_tipo);


--
-- Name: usuario_pkey; Type: CONSTRAINT; Schema: pmieducar; Owner: -; Tablespace:
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_pkey PRIMARY KEY (cod_usuario);


SET search_path = pmiotopic, pg_catalog;

--
-- Name: funcionario_su_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY funcionario_su
    ADD CONSTRAINT funcionario_su_pkey PRIMARY KEY (ref_ref_cod_pessoa_fj);


--
-- Name: grupomoderador_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupomoderador
    ADD CONSTRAINT grupomoderador_pkey PRIMARY KEY (ref_ref_cod_pessoa_fj, ref_cod_grupos);


--
-- Name: grupopessoa_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupopessoa
    ADD CONSTRAINT grupopessoa_pkey PRIMARY KEY (ref_idpes, ref_cod_grupos);


--
-- Name: grupos_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY grupos
    ADD CONSTRAINT grupos_pkey PRIMARY KEY (cod_grupos);


--
-- Name: notas_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY notas
    ADD CONSTRAINT notas_pkey PRIMARY KEY (sequencial, ref_idpes);


--
-- Name: participante_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY participante
    ADD CONSTRAINT participante_pkey PRIMARY KEY (sequencial, ref_ref_cod_grupos, ref_ref_idpes, ref_cod_reuniao);


--
-- Name: reuniao_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY reuniao
    ADD CONSTRAINT reuniao_pkey PRIMARY KEY (cod_reuniao);


--
-- Name: topico_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY topico
    ADD CONSTRAINT topico_pkey PRIMARY KEY (cod_topico);


--
-- Name: topicoreuniao_pkey; Type: CONSTRAINT; Schema: pmiotopic; Owner: -; Tablespace:
--

ALTER TABLE ONLY topicoreuniao
    ADD CONSTRAINT topicoreuniao_pkey PRIMARY KEY (ref_cod_topico, ref_cod_reuniao);


SET search_path = portal, pg_catalog;

--
-- Name: acesso_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY acesso
    ADD CONSTRAINT acesso_pk PRIMARY KEY (cod_acesso);


--
-- Name: agenda_compromisso_pkey; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY agenda_compromisso
    ADD CONSTRAINT agenda_compromisso_pkey PRIMARY KEY (cod_agenda_compromisso, versao, ref_cod_agenda);


--
-- Name: agenda_pkey; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY agenda
    ADD CONSTRAINT agenda_pkey PRIMARY KEY (cod_agenda);


--
-- Name: agenda_pref_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY agenda_pref
    ADD CONSTRAINT agenda_pref_pk PRIMARY KEY (cod_comp);


--
-- Name: agenda_responsavel_pkey; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY agenda_responsavel
    ADD CONSTRAINT agenda_responsavel_pkey PRIMARY KEY (ref_cod_agenda, ref_ref_cod_pessoa_fj);


--
-- Name: compras_editais_editais_empresas_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_editais_editais_empresas
    ADD CONSTRAINT compras_editais_editais_empresas_pk PRIMARY KEY (ref_cod_compras_editais_editais, ref_cod_compras_editais_empresa, data_hora);


--
-- Name: compras_editais_editais_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_editais_editais
    ADD CONSTRAINT compras_editais_editais_pk PRIMARY KEY (cod_compras_editais_editais);


--
-- Name: compras_editais_empresa_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_editais_empresa
    ADD CONSTRAINT compras_editais_empresa_pk PRIMARY KEY (cod_compras_editais_empresa);


--
-- Name: compras_final_pregao_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_final_pregao
    ADD CONSTRAINT compras_final_pregao_pk PRIMARY KEY (cod_compras_final_pregao);


--
-- Name: compras_funcionarios_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_funcionarios
    ADD CONSTRAINT compras_funcionarios_pk PRIMARY KEY (ref_ref_cod_pessoa_fj);


--
-- Name: compras_licitacoes_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_licitacoes
    ADD CONSTRAINT compras_licitacoes_pk PRIMARY KEY (cod_compras_licitacoes);


--
-- Name: compras_modalidade_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_modalidade
    ADD CONSTRAINT compras_modalidade_pk PRIMARY KEY (cod_compras_modalidade);


--
-- Name: compras_pregao_execucao_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_pk PRIMARY KEY (cod_compras_pregao_execucao);


--
-- Name: compras_prestacao_contas_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY compras_prestacao_contas
    ADD CONSTRAINT compras_prestacao_contas_pk PRIMARY KEY (cod_compras_prestacao_contas);


--
-- Name: foto_portal_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY foto_portal
    ADD CONSTRAINT foto_portal_pk PRIMARY KEY (cod_foto_portal);


--
-- Name: foto_secao_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY foto_secao
    ADD CONSTRAINT foto_secao_pk PRIMARY KEY (cod_foto_secao);


--
-- Name: funcionario_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT funcionario_pk PRIMARY KEY (ref_cod_pessoa_fj);


--
-- Name: funcionario_vinculo_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY funcionario_vinculo
    ADD CONSTRAINT funcionario_vinculo_pk PRIMARY KEY (cod_funcionario_vinculo);


--
-- Name: imagem_pkey; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY imagem
    ADD CONSTRAINT imagem_pkey PRIMARY KEY (cod_imagem);


--
-- Name: imagem_tipo_pkey; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY imagem_tipo
    ADD CONSTRAINT imagem_tipo_pkey PRIMARY KEY (cod_imagem_tipo);


--
-- Name: intranet_segur_permissao_negada_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY intranet_segur_permissao_negada
    ADD CONSTRAINT intranet_segur_permissao_negada_pk PRIMARY KEY (cod_intranet_segur_permissao_negada);


--
-- Name: jor_arquivo_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY jor_arquivo
    ADD CONSTRAINT jor_arquivo_pk PRIMARY KEY (ref_cod_jor_edicao, jor_arquivo);


--
-- Name: jor_edicao_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY jor_edicao
    ADD CONSTRAINT jor_edicao_pk PRIMARY KEY (cod_jor_edicao);


--
-- Name: mailling_email_conteudo_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY mailling_email_conteudo
    ADD CONSTRAINT mailling_email_conteudo_pk PRIMARY KEY (cod_mailling_email_conteudo);


--
-- Name: mailling_email_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY mailling_email
    ADD CONSTRAINT mailling_email_pk PRIMARY KEY (cod_mailling_email);


--
-- Name: mailling_fila_envio_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY mailling_fila_envio
    ADD CONSTRAINT mailling_fila_envio_pk PRIMARY KEY (cod_mailling_fila_envio);


--
-- Name: mailling_grupo_email_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY mailling_grupo_email
    ADD CONSTRAINT mailling_grupo_email_pk PRIMARY KEY (ref_cod_mailling_email, ref_cod_mailling_grupo);


--
-- Name: mailling_grupo_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY mailling_grupo
    ADD CONSTRAINT mailling_grupo_pk PRIMARY KEY (cod_mailling_grupo);


--
-- Name: mailling_historico_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY mailling_historico
    ADD CONSTRAINT mailling_historico_pk PRIMARY KEY (cod_mailling_historico);


--
-- Name: menu_funcionario_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu_funcionario
    ADD CONSTRAINT menu_funcionario_pk PRIMARY KEY (ref_ref_cod_pessoa_fj, ref_cod_menu_submenu);


--
-- Name: menu_menu_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu_menu
    ADD CONSTRAINT menu_menu_pk PRIMARY KEY (cod_menu_menu);


--
-- Name: menu_submenu_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY menu_submenu
    ADD CONSTRAINT menu_submenu_pk PRIMARY KEY (cod_menu_submenu);


--
-- Name: not_portal_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY not_portal
    ADD CONSTRAINT not_portal_pk PRIMARY KEY (cod_not_portal);


--
-- Name: not_portal_tipo_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY not_portal_tipo
    ADD CONSTRAINT not_portal_tipo_pk PRIMARY KEY (ref_cod_not_portal, ref_cod_not_tipo);


--
-- Name: not_tipo_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY not_tipo
    ADD CONSTRAINT not_tipo_pk PRIMARY KEY (cod_not_tipo);


--
-- Name: not_vinc_portal_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY not_vinc_portal
    ADD CONSTRAINT not_vinc_portal_pk PRIMARY KEY (ref_cod_not_portal, vic_num);


--
-- Name: pessoa_atividade_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa_atividade
    ADD CONSTRAINT pessoa_atividade_pk PRIMARY KEY (cod_pessoa_atividade);


--
-- Name: pessoa_fj_pessoa_atividade_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa_fj_pessoa_atividade
    ADD CONSTRAINT pessoa_fj_pessoa_atividade_pk PRIMARY KEY (ref_cod_pessoa_atividade, ref_cod_pessoa_fj);


--
-- Name: pessoa_fj_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa_fj
    ADD CONSTRAINT pessoa_fj_pk PRIMARY KEY (cod_pessoa_fj);


--
-- Name: pessoa_ramo_atividade_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY pessoa_ramo_atividade
    ADD CONSTRAINT pessoa_ramo_atividade_pk PRIMARY KEY (cod_ramo_atividade);


--
-- Name: portal_banner_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY portal_banner
    ADD CONSTRAINT portal_banner_pk PRIMARY KEY (cod_portal_banner);


--
-- Name: portal_concurso_pk; Type: CONSTRAINT; Schema: portal; Owner: -; Tablespace:
--

ALTER TABLE ONLY portal_concurso
    ADD CONSTRAINT portal_concurso_pk PRIMARY KEY (cod_portal_concurso);


SET search_path = public, pg_catalog;

--
-- Name: bairro_regiao_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY bairro_regiao
    ADD CONSTRAINT bairro_regiao_pkey PRIMARY KEY (ref_cod_regiao, ref_idbai);


--
-- Name: pk_bairro; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY bairro
    ADD CONSTRAINT pk_bairro PRIMARY KEY (idbai);


--
-- Name: pk_logradouro; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY logradouro
    ADD CONSTRAINT pk_logradouro PRIMARY KEY (idlog);


--
-- Name: pk_logradouro_fonetico; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY logradouro_fonetico
    ADD CONSTRAINT pk_logradouro_fonetico PRIMARY KEY (fonema, idlog);


--
-- Name: pk_municipio; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT pk_municipio PRIMARY KEY (idmun);


--
-- Name: pk_pais; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY pais
    ADD CONSTRAINT pk_pais PRIMARY KEY (idpais);


--
-- Name: pk_setor; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT pk_setor PRIMARY KEY (idset);


--
-- Name: pk_uf; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY uf
    ADD CONSTRAINT pk_uf PRIMARY KEY (sigla_uf);


--
-- Name: pk_vila; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY vila
    ADD CONSTRAINT pk_vila PRIMARY KEY (idvil);


--
-- Name: pkchangelog; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY changelog
    ADD CONSTRAINT pkchangelog PRIMARY KEY (change_number, delta_set);


--
-- Name: regiao_pkey; Type: CONSTRAINT; Schema: public; Owner: -; Tablespace:
--

ALTER TABLE ONLY regiao
    ADD CONSTRAINT regiao_pkey PRIMARY KEY (cod_regiao);


SET search_path = urbano, pg_catalog;

--
-- Name: pk_cep_logradouro; Type: CONSTRAINT; Schema: urbano; Owner: -; Tablespace:
--

ALTER TABLE ONLY cep_logradouro
    ADD CONSTRAINT pk_cep_logradouro PRIMARY KEY (cep, idlog);


--
-- Name: pk_cep_logradouro_bairro; Type: CONSTRAINT; Schema: urbano; Owner: -; Tablespace:
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT pk_cep_logradouro_bairro PRIMARY KEY (idbai, idlog, cep);


--
-- Name: pk_tipo_logradouro; Type: CONSTRAINT; Schema: urbano; Owner: -; Tablespace:
--

ALTER TABLE ONLY tipo_logradouro
    ADD CONSTRAINT pk_tipo_logradouro PRIMARY KEY (idtlog);


SET search_path = acesso, pg_catalog;

--
-- Name: un_usuario_idpes; Type: INDEX; Schema: acesso; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_usuario_idpes ON usuario USING btree (idpes);


SET search_path = alimentos, pg_catalog;

--
-- Name: un_baixa_guia_remessa; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_baixa_guia_remessa ON baixa_guia_remessa USING btree (idgui, dt_recebimento);


--
-- Name: un_cardapio_produto; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_cardapio_produto ON cardapio_produto USING btree (idcar, idpro);


--
-- Name: un_cliente; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_cliente ON cliente USING btree (idcli, identificacao);


--
-- Name: un_contrato; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_contrato ON contrato USING btree (idcli, codigo, num_aditivo);


--
-- Name: un_contrato_produto; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_contrato_produto ON contrato_produto USING btree (idcon, idpro);


--
-- Name: un_evento; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_evento ON evento USING btree (idcad, mes, dia);


--
-- Name: un_faixa_cp_quimico; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_faixa_cp_quimico ON faixa_composto_quimico USING btree (idcom, idfae);


--
-- Name: un_fornecedor; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_fornecedor ON fornecedor USING btree (idcli, nome_fantasia);


--
-- Name: un_fornecedor_unidade_atend; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_fornecedor_unidade_atend ON fornecedor_unidade_atendida USING btree (iduni, idfor);


--
-- Name: un_guia_remessa; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_guia_remessa ON guia_remessa USING btree (idcli, ano, sequencial);


--
-- Name: un_guia_remessa_produto; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_guia_remessa_produto ON guia_remessa_produto USING btree (idgui, idpro);


--
-- Name: un_prod_cp_quimico; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_prod_cp_quimico ON produto_composto_quimico USING btree (idpro, idcom);


--
-- Name: un_produto; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_produto ON produto USING btree (idcli, nome_compra);


--
-- Name: un_produto_fornecedor; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_produto_fornecedor ON produto_fornecedor USING btree (idfor, idpro);


--
-- Name: un_produto_medida_caseira; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_produto_medida_caseira ON produto_medida_caseira USING btree (idmedcas, idcli, idpro);


--
-- Name: un_rec_cp_quimico; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_rec_cp_quimico ON receita_composto_quimico USING btree (idcom, idrec);


--
-- Name: un_rec_prod; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_rec_prod ON receita_produto USING btree (idpro, idrec);


--
-- Name: un_uni_faixa_etaria; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_uni_faixa_etaria ON unidade_faixa_etaria USING btree (iduni, idfae);


--
-- Name: un_unidade_atendida; Type: INDEX; Schema: alimentos; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_unidade_atendida ON unidade_atendida USING btree (idcli, codigo);


SET search_path = cadastro, pg_catalog;

--
-- Name: un_fisica_cpf; Type: INDEX; Schema: cadastro; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_fisica_cpf ON fisica_cpf USING btree (cpf);


--
-- Name: un_juridica_cnpj; Type: INDEX; Schema: cadastro; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX un_juridica_cnpj ON juridica USING btree (cnpj);


SET search_path = modules, pg_catalog;

--
-- Name: area_conhecimento_nome_key; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE INDEX area_conhecimento_nome_key ON area_conhecimento USING btree (nome);


--
-- Name: componente_curricular_area_conhecimento_key; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE INDEX componente_curricular_area_conhecimento_key ON componente_curricular USING btree (area_conhecimento_id);


--
-- Name: componente_curricular_id_key; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX componente_curricular_id_key ON componente_curricular USING btree (id);


--
-- Name: componente_curricular_turma_turma_idx; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE INDEX componente_curricular_turma_turma_idx ON componente_curricular_turma USING btree (turma_id);


--
-- Name: docente_licenciatura_ies_idx; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE INDEX docente_licenciatura_ies_idx ON docente_licenciatura USING btree (ies_id);


--
-- Name: regra_avaliacao_id_key; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX regra_avaliacao_id_key ON regra_avaliacao USING btree (id);


--
-- Name: tabela_arredondamento_id_key; Type: INDEX; Schema: modules; Owner: -; Tablespace:
--

CREATE UNIQUE INDEX tabela_arredondamento_id_key ON tabela_arredondamento USING btree (id);


SET search_path = pmieducar, pg_catalog;

--
-- Name: fki_biblioteca_usuario_ref_cod_biblioteca_fk; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX fki_biblioteca_usuario_ref_cod_biblioteca_fk ON biblioteca_usuario USING btree (ref_cod_biblioteca);


--
-- Name: fki_servidor_ref_cod_subnivel; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX fki_servidor_ref_cod_subnivel ON servidor USING btree (ref_cod_subnivel);


--
-- Name: fki_servidor_ref_cod_subnivel_; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX fki_servidor_ref_cod_subnivel_ ON servidor USING btree (ref_cod_subnivel);


--
-- Name: i_aluno_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_ativo ON aluno USING btree (ativo);


--
-- Name: i_aluno_beneficio_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_beneficio_ativo ON aluno_beneficio USING btree (ativo);


--
-- Name: i_aluno_beneficio_nm_beneficio; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_beneficio_nm_beneficio ON aluno_beneficio USING btree (nm_beneficio);


--
-- Name: i_aluno_beneficio_nm_beneficio_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_beneficio_nm_beneficio_asc ON aluno_beneficio USING btree (to_ascii((nm_beneficio)::text));


--
-- Name: i_aluno_beneficio_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_beneficio_ref_usuario_cad ON aluno_beneficio USING btree (ref_usuario_cad);


--
-- Name: i_aluno_ref_cod_aluno_beneficio; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_ref_cod_aluno_beneficio ON aluno USING btree (ref_cod_aluno_beneficio);


--
-- Name: i_aluno_ref_cod_religiao; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_ref_cod_religiao ON aluno USING btree (ref_cod_religiao);


--
-- Name: i_aluno_ref_idpes; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_ref_idpes ON aluno USING btree (ref_idpes);


--
-- Name: i_aluno_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_aluno_ref_usuario_cad ON aluno USING btree (ref_usuario_cad);


--
-- Name: i_calendario_ano_letivo_ano; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_ano_letivo_ano ON calendario_ano_letivo USING btree (ano);


--
-- Name: i_calendario_ano_letivo_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_ano_letivo_ativo ON calendario_ano_letivo USING btree (ativo);


--
-- Name: i_calendario_ano_letivo_ref_cod_escola; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_ano_letivo_ref_cod_escola ON calendario_ano_letivo USING btree (ref_cod_escola);


--
-- Name: i_calendario_ano_letivo_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_ano_letivo_ref_usuario_cad ON calendario_ano_letivo USING btree (ref_usuario_cad);


--
-- Name: i_calendario_dia_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_ativo ON calendario_dia USING btree (ativo);


--
-- Name: i_calendario_dia_dia; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_dia ON calendario_dia USING btree (dia);


--
-- Name: i_calendario_dia_mes; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_mes ON calendario_dia USING btree (mes);


--
-- Name: i_calendario_dia_motivo_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_motivo_ativo ON calendario_dia_motivo USING btree (ativo);


--
-- Name: i_calendario_dia_motivo_ref_cod_escola; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_motivo_ref_cod_escola ON calendario_dia_motivo USING btree (ref_cod_escola);


--
-- Name: i_calendario_dia_motivo_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_motivo_ref_usuario_cad ON calendario_dia_motivo USING btree (ref_usuario_cad);


--
-- Name: i_calendario_dia_motivo_sigla; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_motivo_sigla ON calendario_dia_motivo USING btree (sigla);


--
-- Name: i_calendario_dia_motivo_sigla_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_motivo_sigla_asc ON calendario_dia_motivo USING btree (to_ascii((sigla)::text));


--
-- Name: i_calendario_dia_motivo_tipo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_motivo_tipo ON calendario_dia_motivo USING btree (tipo);


--
-- Name: i_calendario_dia_ref_cod_calendario_dia_motivo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_ref_cod_calendario_dia_motivo ON calendario_dia USING btree (ref_cod_calendario_dia_motivo);


--
-- Name: i_calendario_dia_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_calendario_dia_ref_usuario_cad ON calendario_dia USING btree (ref_usuario_cad);


--
-- Name: i_coffebreak_tipo_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_coffebreak_tipo_ativo ON coffebreak_tipo USING btree (ativo);


--
-- Name: i_coffebreak_tipo_custo_unitario; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_coffebreak_tipo_custo_unitario ON coffebreak_tipo USING btree (custo_unitario);


--
-- Name: i_coffebreak_tipo_nm_tipo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_coffebreak_tipo_nm_tipo ON coffebreak_tipo USING btree (nm_tipo);


--
-- Name: i_coffebreak_tipo_nm_tipo_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_coffebreak_tipo_nm_tipo_asc ON coffebreak_tipo USING btree (to_ascii((nm_tipo)::text));


--
-- Name: i_coffebreak_tipo_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_coffebreak_tipo_ref_usuario_cad ON coffebreak_tipo USING btree (ref_usuario_cad);


--
-- Name: i_curso_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_ativo ON curso USING btree (ativo);


--
-- Name: i_curso_ato_poder_publico; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_ato_poder_publico ON curso USING btree (ato_poder_publico);


--
-- Name: i_curso_carga_horaria; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_carga_horaria ON curso USING btree (carga_horaria);


--
-- Name: i_curso_nm_curso; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_nm_curso ON curso USING btree (nm_curso);


--
-- Name: i_curso_nm_curso_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_nm_curso_asc ON curso USING btree (to_ascii((nm_curso)::text));


--
-- Name: i_curso_objetivo_curso; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_objetivo_curso ON curso USING btree (objetivo_curso);


--
-- Name: i_curso_objetivo_curso_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_objetivo_curso_asc ON curso USING btree (to_ascii(objetivo_curso));


--
-- Name: i_curso_qtd_etapas; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_qtd_etapas ON curso USING btree (qtd_etapas);


--
-- Name: i_curso_ref_cod_nivel_ensino; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_ref_cod_nivel_ensino ON curso USING btree (ref_cod_nivel_ensino);


--
-- Name: i_curso_ref_cod_tipo_ensino; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_ref_cod_tipo_ensino ON curso USING btree (ref_cod_tipo_ensino);


--
-- Name: i_curso_ref_cod_tipo_regime; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_ref_cod_tipo_regime ON curso USING btree (ref_cod_tipo_regime);


--
-- Name: i_curso_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_ref_usuario_cad ON curso USING btree (ref_usuario_cad);


--
-- Name: i_curso_sgl_curso; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_sgl_curso ON curso USING btree (sgl_curso);


--
-- Name: i_curso_sgl_curso_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_curso_sgl_curso_asc ON curso USING btree (to_ascii((sgl_curso)::text));


--
-- Name: i_disciplina_abreviatura; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_abreviatura ON disciplina USING btree (abreviatura);


--
-- Name: i_disciplina_abreviatura_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_abreviatura_asc ON disciplina USING btree (to_ascii((abreviatura)::text));


--
-- Name: i_disciplina_apura_falta; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_apura_falta ON disciplina USING btree (apura_falta);


--
-- Name: i_disciplina_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_ativo ON disciplina USING btree (ativo);


--
-- Name: i_disciplina_carga_horaria; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_carga_horaria ON disciplina USING btree (carga_horaria);


--
-- Name: i_disciplina_nm_disciplina; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_nm_disciplina ON disciplina USING btree (nm_disciplina);


--
-- Name: i_disciplina_nm_disciplina_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_nm_disciplina_asc ON disciplina USING btree (to_ascii((nm_disciplina)::text));


--
-- Name: i_disciplina_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_ref_usuario_cad ON disciplina USING btree (ref_usuario_cad);


--
-- Name: i_disciplina_topico_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_topico_ativo ON disciplina_topico USING btree (ativo);


--
-- Name: i_disciplina_topico_nm_topico; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_topico_nm_topico ON disciplina_topico USING btree (nm_topico);


--
-- Name: i_disciplina_topico_nm_topico_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_topico_nm_topico_asc ON disciplina_topico USING btree (to_ascii((nm_topico)::text));


--
-- Name: i_disciplina_topico_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_disciplina_topico_ref_usuario_cad ON disciplina_topico USING btree (ref_usuario_cad);


--
-- Name: i_dispensa_disciplina_ref_cod_matricula; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_dispensa_disciplina_ref_cod_matricula ON dispensa_disciplina USING btree (ref_cod_matricula);


--
-- Name: i_escola_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_ativo ON escola USING btree (ativo);


--
-- Name: i_escola_complemento_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_ativo ON escola_complemento USING btree (ativo);


--
-- Name: i_escola_complemento_bairro; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_bairro ON escola_complemento USING btree (bairro);


--
-- Name: i_escola_complemento_bairro_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_bairro_asc ON escola_complemento USING btree (to_ascii((bairro)::text));


--
-- Name: i_escola_complemento_cep; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_cep ON escola_complemento USING btree (cep);


--
-- Name: i_escola_complemento_cep_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_cep_asc ON escola_complemento USING btree (to_ascii((cep)::text));


--
-- Name: i_escola_complemento_complemento; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_complemento ON escola_complemento USING btree (complemento);


--
-- Name: i_escola_complemento_complemento_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_complemento_asc ON escola_complemento USING btree (to_ascii((complemento)::text));


--
-- Name: i_escola_complemento_email; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_email ON escola_complemento USING btree (email);


--
-- Name: i_escola_complemento_email_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_email_asc ON escola_complemento USING btree (to_ascii((email)::text));


--
-- Name: i_escola_complemento_logradouro; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_logradouro ON escola_complemento USING btree (logradouro);


--
-- Name: i_escola_complemento_logradouro_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_logradouro_asc ON escola_complemento USING btree (to_ascii((bairro)::text));


--
-- Name: i_escola_complemento_municipio; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_municipio ON escola_complemento USING btree (municipio);


--
-- Name: i_escola_complemento_municipio_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_municipio_asc ON escola_complemento USING btree (to_ascii((municipio)::text));


--
-- Name: i_escola_complemento_nm_escola; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_nm_escola ON escola_complemento USING btree (nm_escola);


--
-- Name: i_escola_complemento_nm_escola_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_nm_escola_asc ON escola_complemento USING btree (to_ascii((nm_escola)::text));


--
-- Name: i_escola_complemento_numero; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_numero ON escola_complemento USING btree (numero);


--
-- Name: i_escola_complemento_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_complemento_ref_usuario_cad ON escola_complemento USING btree (ref_usuario_cad);


--
-- Name: i_escola_curso_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_curso_ativo ON escola_curso USING btree (ativo);


--
-- Name: i_escola_curso_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_curso_ref_usuario_cad ON escola_curso USING btree (ref_usuario_cad);


--
-- Name: i_escola_localizacao_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_localizacao_ativo ON escola_localizacao USING btree (ativo);


--
-- Name: i_escola_localizacao_nm_localizacao; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_localizacao_nm_localizacao ON escola_localizacao USING btree (nm_localizacao);


--
-- Name: i_escola_localizacao_nm_localizacao_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_localizacao_nm_localizacao_asc ON escola_localizacao USING btree (to_ascii((nm_localizacao)::text));


--
-- Name: i_escola_localizacao_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_localizacao_ref_usuario_cad ON escola_localizacao USING btree (ref_usuario_cad);


--
-- Name: i_escola_rede_ensino_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_rede_ensino_ativo ON escola_rede_ensino USING btree (ativo);


--
-- Name: i_escola_rede_ensino_nm_rede; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_rede_ensino_nm_rede ON escola_rede_ensino USING btree (nm_rede);


--
-- Name: i_escola_rede_ensino_nm_rede_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_rede_ensino_nm_rede_asc ON escola_rede_ensino USING btree (to_ascii((nm_rede)::text));


--
-- Name: i_escola_rede_ensino_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_rede_ensino_ref_usuario_cad ON escola_rede_ensino USING btree (ref_usuario_cad);


--
-- Name: i_escola_ref_cod_escola_localizacao; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_ref_cod_escola_localizacao ON escola USING btree (ref_cod_escola_localizacao);


--
-- Name: i_escola_ref_cod_escola_rede_ensino; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_ref_cod_escola_rede_ensino ON escola USING btree (ref_cod_escola_rede_ensino);


--
-- Name: i_escola_ref_cod_instituicao; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_ref_cod_instituicao ON escola USING btree (ref_cod_instituicao);


--
-- Name: i_escola_ref_idpes; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_ref_idpes ON escola USING btree (ref_idpes);


--
-- Name: i_escola_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_ref_usuario_cad ON escola USING btree (ref_usuario_cad);


--
-- Name: i_escola_serie_ensino_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_serie_ensino_ativo ON escola_serie USING btree (ativo);


--
-- Name: i_escola_serie_hora_final; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_serie_hora_final ON escola_serie USING btree (hora_final);


--
-- Name: i_escola_serie_hora_inicial; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_serie_hora_inicial ON escola_serie USING btree (hora_inicial);


--
-- Name: i_escola_serie_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_serie_ref_usuario_cad ON escola_serie USING btree (ref_usuario_cad);


--
-- Name: i_escola_sigla; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_sigla ON escola USING btree (sigla);


--
-- Name: i_escola_sigla_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_escola_sigla_asc ON escola USING btree (to_ascii((sigla)::text));


--
-- Name: i_funcao_abreviatura; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_abreviatura ON funcao USING btree (abreviatura);


--
-- Name: i_funcao_abreviatura_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_abreviatura_asc ON funcao USING btree (to_ascii((abreviatura)::text));


--
-- Name: i_funcao_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_ativo ON funcao USING btree (ativo);


--
-- Name: i_funcao_nm_funcao; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_nm_funcao ON funcao USING btree (nm_funcao);


--
-- Name: i_funcao_nm_funcao_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_nm_funcao_asc ON funcao USING btree (to_ascii((nm_funcao)::text));


--
-- Name: i_funcao_professor; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_professor ON funcao USING btree (professor);


--
-- Name: i_funcao_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_funcao_ref_usuario_cad ON funcao USING btree (ref_usuario_cad);


--
-- Name: i_habilitacao_ativo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_habilitacao_ativo ON habilitacao USING btree (ativo);


--
-- Name: i_habilitacao_nm_tipo; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_habilitacao_nm_tipo ON habilitacao USING btree (nm_tipo);


--
-- Name: i_habilitacao_ref_usuario_cad; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_habilitacao_ref_usuario_cad ON habilitacao USING btree (ref_usuario_cad);


--
-- Name: i_habilitacaoo_nm_tipo_asc; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_habilitacaoo_nm_tipo_asc ON habilitacao USING btree (to_ascii((nm_tipo)::text));


--
-- Name: i_matricula_turma_ref_cod_turma; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_matricula_turma_ref_cod_turma ON matricula_turma USING btree (ref_cod_turma);


--
-- Name: i_nota_aluno_ref_cod_matricula; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_nota_aluno_ref_cod_matricula ON nota_aluno USING btree (ref_cod_matricula);


--
-- Name: i_turma_nm_turma; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX i_turma_nm_turma ON turma USING btree (nm_turma);


--
-- Name: quadro_horario_horarios_busca_horarios_idx; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX quadro_horario_horarios_busca_horarios_idx ON quadro_horario_horarios USING btree (ref_servidor, ref_cod_instituicao_servidor, dia_semana, hora_inicial, hora_final, ativo);


--
-- Name: INDEX quadro_horario_horarios_busca_horarios_idx; Type: COMMENT; Schema: pmieducar; Owner: -
--

COMMENT ON INDEX quadro_horario_horarios_busca_horarios_idx IS 'ndice para otimizar a busca por professores na criao de quadro de horrios.';


--
-- Name: servidor_alocacao_busca_horarios_idx; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX servidor_alocacao_busca_horarios_idx ON servidor_alocacao USING btree (ref_ref_cod_instituicao, ref_cod_escola, ativo, periodo, carga_horaria);


--
-- Name: INDEX servidor_alocacao_busca_horarios_idx; Type: COMMENT; Schema: pmieducar; Owner: -
--

COMMENT ON INDEX servidor_alocacao_busca_horarios_idx IS 'ndice para otimizar a busca por professores na criao de quadro de horrios.';


--
-- Name: servidor_idx; Type: INDEX; Schema: pmieducar; Owner: -; Tablespace:
--

CREATE INDEX servidor_idx ON servidor USING btree (cod_servidor, ref_cod_instituicao, ativo);


--
-- Name: INDEX servidor_idx; Type: COMMENT; Schema: pmieducar; Owner: -
--

COMMENT ON INDEX servidor_idx IS 'ndice para otimizao de acesso aos campos mais usados para queries na tabela.';


SET search_path = portal, pg_catalog;

--
-- Name: mailling_fila_envio_data_envio_idx; Type: INDEX; Schema: portal; Owner: -; Tablespace:
--

CREATE INDEX mailling_fila_envio_data_envio_idx ON mailling_fila_envio USING btree (data_envio);


--
-- Name: mailling_fila_envio_ref_cod_mailling_email; Type: INDEX; Schema: portal; Owner: -; Tablespace:
--

CREATE INDEX mailling_fila_envio_ref_cod_mailling_email ON mailling_fila_envio USING btree (ref_cod_mailling_email);


--
-- Name: mailling_fila_envio_ref_cod_mailling_email_conteudo; Type: INDEX; Schema: portal; Owner: -; Tablespace:
--

CREATE INDEX mailling_fila_envio_ref_cod_mailling_email_conteudo ON mailling_fila_envio USING btree (ref_cod_mailling_email_conteudo);


--
-- Name: mailling_fila_envio_ref_cod_mailling_fila_envio; Type: INDEX; Schema: portal; Owner: -; Tablespace:
--

CREATE INDEX mailling_fila_envio_ref_cod_mailling_fila_envio ON mailling_fila_envio USING btree (cod_mailling_fila_envio);


SET search_path = cadastro, pg_catalog;

--
-- Name: trg_aft_documento; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_documento
    AFTER INSERT OR UPDATE ON documento
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_documento();


--
-- Name: trg_aft_documento_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_documento_historico_campo
    AFTER INSERT OR UPDATE ON documento
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_documento_historico_campo();


--
-- Name: trg_aft_documento_provisorio; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_documento_provisorio
    AFTER INSERT OR UPDATE ON documento
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_documento_provisorio();


--
-- Name: trg_aft_endereco_externo_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_endereco_externo_historico_campo
    AFTER INSERT OR UPDATE ON endereco_externo
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_endereco_externo_historico_campo();


--
-- Name: trg_aft_endereco_pessoa_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_endereco_pessoa_historico_campo
    AFTER INSERT OR UPDATE ON endereco_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_endereco_pessoa_historico_campo();


--
-- Name: trg_aft_fisica; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_fisica
    AFTER INSERT OR UPDATE ON fisica
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_fisica();


--
-- Name: trg_aft_fisica_cpf_provisorio; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_fisica_cpf_provisorio
    AFTER INSERT OR UPDATE ON fisica_cpf
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_fisica_cpf_provisorio();


--
-- Name: trg_aft_fisica_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_fisica_historico_campo
    AFTER INSERT OR UPDATE ON fisica
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_fisica_historico_campo();


--
-- Name: trg_aft_fisica_provisorio; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_fisica_provisorio
    AFTER INSERT OR UPDATE ON fisica
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_fisica_provisorio();


--
-- Name: trg_aft_fone_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_fone_historico_campo
    AFTER INSERT OR UPDATE ON fone_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_fone_historico_campo();


--
-- Name: trg_aft_fone_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_fone_pessoa_historico
    AFTER DELETE ON fone_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_fone_pessoa();


--
-- Name: trg_aft_funcionario_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_funcionario_historico
    AFTER DELETE ON funcionario
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_funcionario();


--
-- Name: trg_aft_ins_endereco_externo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_ins_endereco_externo
    AFTER INSERT OR UPDATE ON endereco_externo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_ins_endereco_externo();


--
-- Name: trg_aft_ins_endereco_pessoa; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_ins_endereco_pessoa
    AFTER INSERT OR UPDATE ON endereco_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_ins_endereco_pessoa();


--
-- Name: trg_aft_juridica_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_juridica_historico_campo
    AFTER INSERT OR UPDATE ON juridica
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_juridica_historico_campo();


--
-- Name: trg_aft_pessoa_fonetiza; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_pessoa_fonetiza
    AFTER INSERT OR UPDATE ON pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE public.fcn_aft_pessoa_fonetiza();


--
-- Name: trg_aft_pessoa_historico_campo; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_aft_pessoa_historico_campo
    AFTER INSERT OR UPDATE ON pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE consistenciacao.fcn_pessoa_historico_campo();


--
-- Name: trg_bef_documento_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_documento_historico
    BEFORE UPDATE ON documento
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_documento();


--
-- Name: trg_bef_endereco_externo_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_endereco_externo_historico
    BEFORE UPDATE ON endereco_externo
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_endereco_externo();


--
-- Name: trg_bef_endereco_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_endereco_pessoa_historico
    BEFORE UPDATE ON endereco_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_endereco_pessoa();


--
-- Name: trg_bef_fisica_cpf_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_fisica_cpf_historico
    BEFORE UPDATE ON fisica_cpf
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_fisica_cpf();


--
-- Name: trg_bef_fisica_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_fisica_historico
    BEFORE UPDATE ON fisica
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_fisica();


--
-- Name: trg_bef_fone_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_fone_pessoa_historico
    BEFORE UPDATE ON fone_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_fone_pessoa();


--
-- Name: trg_bef_funcionario_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_funcionario_historico
    BEFORE UPDATE ON funcionario
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_funcionario();


--
-- Name: trg_bef_ins_fisica; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_ins_fisica
    BEFORE INSERT ON fisica
    FOR EACH ROW
    EXECUTE PROCEDURE public.fcn_bef_ins_fisica();


--
-- Name: trg_bef_ins_juridica; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_ins_juridica
    BEFORE INSERT ON juridica
    FOR EACH ROW
    EXECUTE PROCEDURE public.fcn_bef_ins_juridica();


--
-- Name: trg_bef_juridica_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_juridica_historico
    BEFORE UPDATE ON juridica
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_juridica();


--
-- Name: trg_bef_pessoa_fonetiza; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_pessoa_fonetiza
    BEFORE DELETE ON pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE public.fcn_bef_pessoa_fonetiza();


--
-- Name: trg_bef_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_pessoa_historico
    BEFORE UPDATE ON pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_pessoa();


--
-- Name: trg_bef_socio_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_bef_socio_historico
    BEFORE UPDATE ON socio
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_socio();


--
-- Name: trg_delete_documento_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_documento_historico
    AFTER DELETE ON documento
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_documento();


--
-- Name: trg_delete_endereco_externo_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_endereco_externo_historico
    AFTER DELETE ON endereco_externo
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_endereco_externo();


--
-- Name: trg_delete_endereco_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_endereco_pessoa_historico
    AFTER DELETE ON endereco_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_endereco_pessoa();


--
-- Name: trg_delete_fisica_cpf_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_fisica_cpf_historico
    AFTER DELETE ON fisica_cpf
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_fisica_cpf();


--
-- Name: trg_delete_fisica_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_fisica_historico
    AFTER DELETE ON fisica
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_fisica();


--
-- Name: trg_delete_fone_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_fone_pessoa_historico
    AFTER DELETE ON fone_pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_fone_pessoa();


--
-- Name: trg_delete_funcionario_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_funcionario_historico
    AFTER DELETE ON funcionario
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_funcionario();


--
-- Name: trg_delete_juridica_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_juridica_historico
    AFTER DELETE ON juridica
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_juridica();


--
-- Name: trg_delete_pessoa_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_pessoa_historico
    AFTER DELETE ON pessoa
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_pessoa();


--
-- Name: trg_delete_socio_historico; Type: TRIGGER; Schema: cadastro; Owner: -
--

CREATE TRIGGER trg_delete_socio_historico
    AFTER DELETE ON socio
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_socio();


SET search_path = pmieducar, pg_catalog;

--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON instituicao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_acervo_assunto
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_acervo_autor
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_assunto
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_autor
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_colecao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_editora
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON acervo_idioma
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON aluno
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON aluno_beneficio
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON ano_letivo_modulo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON avaliacao_desempenho
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON biblioteca
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON biblioteca_dia
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON biblioteca_feriados
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON biblioteca_usuario
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON calendario_ano_letivo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON calendario_anotacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON calendario_dia
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON calendario_dia_anotacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON calendario_dia_motivo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON cliente
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON cliente_suspensao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON cliente_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON cliente_tipo_cliente
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON cliente_tipo_exemplar_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON coffebreak_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON curso
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON disciplina
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON disciplina_topico
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola_ano_letivo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola_complemento
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola_curso
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola_localizacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola_rede_ensino
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON escola_serie
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON exemplar
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON exemplar_emprestimo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON exemplar_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON falta_atraso
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON falta_atraso_compensado
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON fonte
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON funcao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON habilitacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON habilitacao_curso
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON historico_disciplinas
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON historico_escolar
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON infra_comodo_funcao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON infra_predio
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON infra_predio_comodo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON material_didatico
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON material_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON matricula
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON matricula_ocorrencia_disciplinar
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON menu_tipo_usuario
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON modulo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON motivo_afastamento
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON motivo_baixa
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON motivo_suspensao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON nivel_ensino
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON operador
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON pagamento_multa
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON pre_requisito
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON quadro_horario
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON religiao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON reserva_vaga
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON reservas
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON sequencia_serie
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON serie
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON serie_pre_requisito
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON servidor
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON servidor_afastamento
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON servidor_alocacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON servidor_curso
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON servidor_formacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON servidor_titulo_concurso
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON situacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_avaliacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_avaliacao_valores
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_dispensa
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_ensino
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_ocorrencia_disciplinar
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_regime
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON tipo_usuario
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON transferencia_solicitacao
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON transferencia_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON turma
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON turma_dia_semana
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON turma_modulo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON turma_tipo
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


--
-- Name: fcn_aft_update; Type: TRIGGER; Schema: pmieducar; Owner: -
--

CREATE TRIGGER fcn_aft_update
    AFTER INSERT OR UPDATE ON usuario
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_update();


SET search_path = public, pg_catalog;

--
-- Name: trg_aft_logradouro_fonetiza; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_aft_logradouro_fonetiza
    AFTER INSERT OR UPDATE ON logradouro
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_aft_logradouro_fonetiza();


--
-- Name: trg_bef_bairro_historico; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_bef_bairro_historico
    BEFORE UPDATE ON bairro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_bairro();


--
-- Name: trg_bef_logradouro_fonetiza; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_bef_logradouro_fonetiza
    BEFORE DELETE ON logradouro
    FOR EACH ROW
    EXECUTE PROCEDURE fcn_bef_logradouro_fonetiza();


--
-- Name: trg_bef_logradouro_historico; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_bef_logradouro_historico
    BEFORE UPDATE ON logradouro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_logradouro();


--
-- Name: trg_bef_municipio_historico; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_bef_municipio_historico
    BEFORE UPDATE ON municipio
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_municipio();


--
-- Name: trg_delete_bairro_historico; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_delete_bairro_historico
    AFTER DELETE ON bairro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_bairro();


--
-- Name: trg_delete_logradouro_historico; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_delete_logradouro_historico
    AFTER DELETE ON logradouro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_logradouro();


--
-- Name: trg_delete_municipio_historico; Type: TRIGGER; Schema: public; Owner: -
--

CREATE TRIGGER trg_delete_municipio_historico
    AFTER DELETE ON municipio
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_municipio();


SET search_path = urbano, pg_catalog;

--
-- Name: trg_bef_cep_logradouro_bairro_historico; Type: TRIGGER; Schema: urbano; Owner: -
--

CREATE TRIGGER trg_bef_cep_logradouro_bairro_historico
    BEFORE UPDATE ON cep_logradouro_bairro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_cep_logradouro_bairro();


--
-- Name: trg_bef_cep_logradouro_historico; Type: TRIGGER; Schema: urbano; Owner: -
--

CREATE TRIGGER trg_bef_cep_logradouro_historico
    BEFORE UPDATE ON cep_logradouro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_grava_historico_cep_logradouro();


--
-- Name: trg_delete_cep_logradouro_bairro_historico; Type: TRIGGER; Schema: urbano; Owner: -
--

CREATE TRIGGER trg_delete_cep_logradouro_bairro_historico
    AFTER DELETE ON cep_logradouro_bairro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_cep_logradouro_bairro();


--
-- Name: trg_delete_cep_logradouro_historico; Type: TRIGGER; Schema: urbano; Owner: -
--

CREATE TRIGGER trg_delete_cep_logradouro_historico
    AFTER DELETE ON cep_logradouro
    FOR EACH ROW
    EXECUTE PROCEDURE historico.fcn_delete_grava_historico_cep_logradouro();


SET search_path = acesso, pg_catalog;

--
-- Name: fk_funcao_grp_funcao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_funcao
    ADD CONSTRAINT fk_funcao_grp_funcao FOREIGN KEY (idfunc, idsis, idmen) REFERENCES funcao(idfunc, idsis, idmen);


--
-- Name: fk_funcao_operacao_funcao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY operacao_funcao
    ADD CONSTRAINT fk_funcao_operacao_funcao FOREIGN KEY (idfunc, idsis, idmen) REFERENCES funcao(idfunc, idsis, idmen);


--
-- Name: fk_grp_fun_grp_operacao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_operacao
    ADD CONSTRAINT fk_grp_fun_grp_operacao FOREIGN KEY (idmen, idsis, idgrp, idfunc) REFERENCES grupo_funcao(idmen, idsis, idgrp, idfunc);


--
-- Name: fk_grp_menu_grp_funcao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_funcao
    ADD CONSTRAINT fk_grp_menu_grp_funcao FOREIGN KEY (idgrp, idsis, idmen) REFERENCES grupo_menu(idgrp, idsis, idmen);


--
-- Name: fk_grp_sis_grp_menu; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_menu
    ADD CONSTRAINT fk_grp_sis_grp_menu FOREIGN KEY (idsis, idgrp) REFERENCES grupo_sistema(idsis, idgrp);


--
-- Name: fk_grupo_grupo_sistema; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_sistema
    ADD CONSTRAINT fk_grupo_grupo_sistema FOREIGN KEY (idgrp) REFERENCES grupo(idgrp);


--
-- Name: fk_grupo_usuario_grupo; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY usuario_grupo
    ADD CONSTRAINT fk_grupo_usuario_grupo FOREIGN KEY (idgrp) REFERENCES grupo(idgrp);


--
-- Name: fk_inst_pessoa_instituicao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY pessoa_instituicao
    ADD CONSTRAINT fk_inst_pessoa_instituicao FOREIGN KEY (idins) REFERENCES instituicao(idins);


--
-- Name: fk_menu_funcao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY funcao
    ADD CONSTRAINT fk_menu_funcao FOREIGN KEY (idmen, idsis) REFERENCES menu(idmen, idsis);


--
-- Name: fk_menu_grp_menu; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_menu
    ADD CONSTRAINT fk_menu_grp_menu FOREIGN KEY (idmen, idsis) REFERENCES menu(idmen, idsis);


--
-- Name: fk_menu_menu; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT fk_menu_menu FOREIGN KEY (menu_idsis, menu_idmen) REFERENCES menu(idsis, idmen);


--
-- Name: fk_oper_func_grp_oper; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_operacao
    ADD CONSTRAINT fk_oper_func_grp_oper FOREIGN KEY (idmen, idsis, idfunc, idope) REFERENCES operacao_funcao(idmen, idsis, idfunc, idope);


--
-- Name: fk_operacao_operacao_funcao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY operacao_funcao
    ADD CONSTRAINT fk_operacao_operacao_funcao FOREIGN KEY (idope) REFERENCES operacao(idope);


--
-- Name: fk_pes_pessoa_instituicao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY pessoa_instituicao
    ADD CONSTRAINT fk_pes_pessoa_instituicao FOREIGN KEY (idpes) REFERENCES cadastro.pessoa(idpes);


--
-- Name: fk_pessoa_usuario; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT fk_pessoa_usuario FOREIGN KEY (idpes) REFERENCES cadastro.pessoa(idpes);


--
-- Name: fk_sistema_grupo_sistema; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY grupo_sistema
    ADD CONSTRAINT fk_sistema_grupo_sistema FOREIGN KEY (idsis) REFERENCES sistema(idsis);


--
-- Name: fk_sistema_menu; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT fk_sistema_menu FOREIGN KEY (idsis) REFERENCES sistema(idsis);


--
-- Name: fk_sistema_operacao; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY operacao
    ADD CONSTRAINT fk_sistema_operacao FOREIGN KEY (idsis) REFERENCES sistema(idsis);


--
-- Name: fk_usuario_usuario_grupo; Type: FK CONSTRAINT; Schema: acesso; Owner: -
--

ALTER TABLE ONLY usuario_grupo
    ADD CONSTRAINT fk_usuario_usuario_grupo FOREIGN KEY ("login") REFERENCES usuario("login");


SET search_path = alimentos, pg_catalog;

--
-- Name: fk_alterar_usuario_cardapio; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio
    ADD CONSTRAINT fk_alterar_usuario_cardapio FOREIGN KEY (login_alteracao) REFERENCES acesso.usuario("login");


--
-- Name: fk_baixa_guia_baixa_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY baixa_guia_produto
    ADD CONSTRAINT fk_baixa_guia_baixa_produto FOREIGN KEY (idbai) REFERENCES baixa_guia_remessa(idbai);


--
-- Name: fk_calendario_evento; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY evento
    ADD CONSTRAINT fk_calendario_evento FOREIGN KEY (idcad) REFERENCES calendario(idcad);


--
-- Name: fk_calendario_unidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_atendida
    ADD CONSTRAINT fk_calendario_unidade FOREIGN KEY (idcad) REFERENCES calendario(idcad);


--
-- Name: fk_cancelar_usuario_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT fk_cancelar_usuario_guia_remessa FOREIGN KEY (login_cancelamento) REFERENCES acesso.usuario("login");


--
-- Name: fk_cardapio_cardapio_faixa_unidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio_faixa_unidade
    ADD CONSTRAINT fk_cardapio_cardapio_faixa_unidade FOREIGN KEY (idcar) REFERENCES cardapio(idcar);


--
-- Name: fk_cardapio_cardapio_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio_produto
    ADD CONSTRAINT fk_cardapio_cardapio_produto FOREIGN KEY (idcar) REFERENCES cardapio(idcar);


--
-- Name: fk_cardapio_cardapio_receita; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio_receita
    ADD CONSTRAINT fk_cardapio_cardapio_receita FOREIGN KEY (idcar) REFERENCES cardapio(idcar);


--
-- Name: fk_cliente_calendario; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY calendario
    ADD CONSTRAINT fk_cliente_calendario FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_cardapio; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio
    ADD CONSTRAINT fk_cliente_cardapio FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_contrato; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY contrato
    ADD CONSTRAINT fk_cliente_contrato FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_cpquimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY composto_quimico
    ADD CONSTRAINT fk_cliente_cpquimico FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_fornecedor; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY fornecedor
    ADD CONSTRAINT fk_cliente_fornecedor FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_grpatencao; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY faixa_etaria
    ADD CONSTRAINT fk_cliente_grpatencao FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_grpquimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY grupo_quimico
    ADD CONSTRAINT fk_cliente_grpquimico FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT fk_cliente_guia_remessa FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_log_guia; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY log_guia_remessa
    ADD CONSTRAINT fk_cliente_log_guia FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_receita; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY receita
    ADD CONSTRAINT fk_cliente_receita FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_tpproduto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY tipo_produto
    ADD CONSTRAINT fk_cliente_tpproduto FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_tprefeicao; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY tipo_refeicao
    ADD CONSTRAINT fk_cliente_tprefeicao FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_tpunidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY tipo_unidade
    ADD CONSTRAINT fk_cliente_tpunidade FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_unidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_atendida
    ADD CONSTRAINT fk_cliente_unidade FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_cliente_uniproduto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_produto
    ADD CONSTRAINT fk_cliente_uniproduto FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_contrato_contrato_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY contrato_produto
    ADD CONSTRAINT fk_contrato_contrato_produto FOREIGN KEY (idcon) REFERENCES contrato(idcon);


--
-- Name: fk_contrato_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT fk_contrato_guia_remessa FOREIGN KEY (idcon) REFERENCES contrato(idcon);


--
-- Name: fk_cp_quimico_faixa_cp_quimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY faixa_composto_quimico
    ADD CONSTRAINT fk_cp_quimico_faixa_cp_quimico FOREIGN KEY (idcom) REFERENCES composto_quimico(idcom);


--
-- Name: fk_emitir_usuario_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT fk_emitir_usuario_guia_remessa FOREIGN KEY (login_emissao) REFERENCES acesso.usuario("login");


--
-- Name: fk_faixa_etaria_unidade_faixa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_faixa_etaria
    ADD CONSTRAINT fk_faixa_etaria_unidade_faixa FOREIGN KEY (idfae) REFERENCES faixa_etaria(idfae);


--
-- Name: fk_faixa_faixa_cp_quimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY faixa_composto_quimico
    ADD CONSTRAINT fk_faixa_faixa_cp_quimico FOREIGN KEY (idfae) REFERENCES faixa_etaria(idfae);


--
-- Name: fk_faixa_uni_cardapio_faixa_unidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio_faixa_unidade
    ADD CONSTRAINT fk_faixa_uni_cardapio_faixa_unidade FOREIGN KEY (idfeu) REFERENCES unidade_faixa_etaria(idfeu);


--
-- Name: fk_fornecedor_contrato; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY contrato
    ADD CONSTRAINT fk_fornecedor_contrato FOREIGN KEY (idfor) REFERENCES fornecedor(idfor);


--
-- Name: fk_fornecedor_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT fk_fornecedor_guia_remessa FOREIGN KEY (idfor) REFERENCES fornecedor(idfor);


--
-- Name: fk_fornecedor_produto_fornecedor; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_fornecedor
    ADD CONSTRAINT fk_fornecedor_produto_fornecedor FOREIGN KEY (idfor) REFERENCES fornecedor(idfor);


--
-- Name: fk_fornecedor_unidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY fornecedor_unidade_atendida
    ADD CONSTRAINT fk_fornecedor_unidade FOREIGN KEY (idfor) REFERENCES fornecedor(idfor);


--
-- Name: fk_grupo_cp_quimico_cp_quimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY composto_quimico
    ADD CONSTRAINT fk_grupo_cp_quimico_cp_quimico FOREIGN KEY (idgrpq) REFERENCES grupo_quimico(idgrpq);


--
-- Name: fk_guia_guia_remessa_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa_produto
    ADD CONSTRAINT fk_guia_guia_remessa_produto FOREIGN KEY (idgui) REFERENCES guia_remessa(idgui);


--
-- Name: fk_guia_produto_baixa_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY baixa_guia_produto
    ADD CONSTRAINT fk_guia_produto_baixa_produto FOREIGN KEY (idgup) REFERENCES guia_remessa_produto(idgup);


--
-- Name: fk_guia_remessa_baixa_guia; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY baixa_guia_remessa
    ADD CONSTRAINT fk_guia_remessa_baixa_guia FOREIGN KEY (idgui) REFERENCES guia_remessa(idgui);


--
-- Name: fk_guia_remessa_guia_pro_diario; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_produto_diario
    ADD CONSTRAINT fk_guia_remessa_guia_pro_diario FOREIGN KEY (idgui) REFERENCES guia_remessa(idgui);


--
-- Name: fk_incluir_usuario_cardapio; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio
    ADD CONSTRAINT fk_incluir_usuario_cardapio FOREIGN KEY (login_inclusao) REFERENCES acesso.usuario("login");


--
-- Name: fk_medidas_caseiras_cliente; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY medidas_caseiras
    ADD CONSTRAINT fk_medidas_caseiras_cliente FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_pessoa_cliente; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT fk_pessoa_cliente FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_pessoa_fornecedor; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY fornecedor
    ADD CONSTRAINT fk_pessoa_fornecedor FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_pessoa_unidade_atend; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_atendida
    ADD CONSTRAINT fk_pessoa_unidade_atend FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_prod_cp_quimico_cp_quimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_composto_quimico
    ADD CONSTRAINT fk_prod_cp_quimico_cp_quimico FOREIGN KEY (idcom) REFERENCES composto_quimico(idcom);


--
-- Name: fk_prod_cp_quimico_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_composto_quimico
    ADD CONSTRAINT fk_prod_cp_quimico_produto FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_cardapio_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio_produto
    ADD CONSTRAINT fk_produto_cardapio_produto FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_cliente; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto
    ADD CONSTRAINT fk_produto_cliente FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_produto_contrato_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY contrato_produto
    ADD CONSTRAINT fk_produto_contrato_produto FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_fornecedor; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto
    ADD CONSTRAINT fk_produto_fornecedor FOREIGN KEY (idfor) REFERENCES fornecedor(idfor);


--
-- Name: fk_produto_guia_pro_diario; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_produto_diario
    ADD CONSTRAINT fk_produto_guia_pro_diario FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_guia_remessa_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa_produto
    ADD CONSTRAINT fk_produto_guia_remessa_produto FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_medida_caseira_cliente; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_medida_caseira
    ADD CONSTRAINT fk_produto_medida_caseira_cliente FOREIGN KEY (idcli) REFERENCES cliente(idcli);


--
-- Name: fk_produto_medida_caseira_medidas; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_medida_caseira
    ADD CONSTRAINT fk_produto_medida_caseira_medidas FOREIGN KEY (idmedcas, idcli) REFERENCES medidas_caseiras(idmedcas, idcli);


--
-- Name: fk_produto_medida_caseira_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_medida_caseira
    ADD CONSTRAINT fk_produto_medida_caseira_produto FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_produto_fornecedor; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto_fornecedor
    ADD CONSTRAINT fk_produto_produto_fornecedor FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_produto_tipo; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto
    ADD CONSTRAINT fk_produto_tipo FOREIGN KEY (idtip) REFERENCES tipo_produto(idtip);


--
-- Name: fk_produto_unidade; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY produto
    ADD CONSTRAINT fk_produto_unidade FOREIGN KEY (idunp, idcli) REFERENCES unidade_produto(idunp, idcli);


--
-- Name: fk_rec_cp_quimico_cp_quimico; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY receita_composto_quimico
    ADD CONSTRAINT fk_rec_cp_quimico_cp_quimico FOREIGN KEY (idcom) REFERENCES composto_quimico(idcom);


--
-- Name: fk_rec_cp_quimico_receita; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY receita_composto_quimico
    ADD CONSTRAINT fk_rec_cp_quimico_receita FOREIGN KEY (idrec) REFERENCES receita(idrec);


--
-- Name: fk_rec_prod_produto; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY receita_produto
    ADD CONSTRAINT fk_rec_prod_produto FOREIGN KEY (idpro) REFERENCES produto(idpro);


--
-- Name: fk_rec_prod_receita; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY receita_produto
    ADD CONSTRAINT fk_rec_prod_receita FOREIGN KEY (idrec) REFERENCES receita(idrec);


--
-- Name: fk_receita_cardapio_receita; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio_receita
    ADD CONSTRAINT fk_receita_cardapio_receita FOREIGN KEY (idrec) REFERENCES receita(idrec);


--
-- Name: fk_tipo_uni_uni_atendida; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_atendida
    ADD CONSTRAINT fk_tipo_uni_uni_atendida FOREIGN KEY (idtip) REFERENCES tipo_unidade(idtip);


--
-- Name: fk_tp_refeicao_cardapio; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY cardapio
    ADD CONSTRAINT fk_tp_refeicao_cardapio FOREIGN KEY (idtre) REFERENCES tipo_refeicao(idtre);


--
-- Name: fk_uni_atend_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_remessa
    ADD CONSTRAINT fk_uni_atend_guia_remessa FOREIGN KEY (iduni) REFERENCES unidade_atendida(iduni);


--
-- Name: fk_uni_atend_uni_faixa_eta; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY unidade_faixa_etaria
    ADD CONSTRAINT fk_uni_atend_uni_faixa_eta FOREIGN KEY (iduni) REFERENCES unidade_atendida(iduni);


--
-- Name: fk_unidade_atendida_guia_pro_diario; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY guia_produto_diario
    ADD CONSTRAINT fk_unidade_atendida_guia_pro_diario FOREIGN KEY (iduni) REFERENCES unidade_atendida(iduni);


--
-- Name: fk_unidade_fornecedor; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY fornecedor_unidade_atendida
    ADD CONSTRAINT fk_unidade_fornecedor FOREIGN KEY (iduni) REFERENCES unidade_atendida(iduni);


--
-- Name: fk_usuario_baixa_guia; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY baixa_guia_produto
    ADD CONSTRAINT fk_usuario_baixa_guia FOREIGN KEY (login_baixa) REFERENCES acesso.usuario("login");


--
-- Name: fk_usuario_baixa_guia_remessa; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY baixa_guia_remessa
    ADD CONSTRAINT fk_usuario_baixa_guia_remessa FOREIGN KEY (login_baixa) REFERENCES acesso.usuario("login");


--
-- Name: fk_usuario_contrato; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY contrato
    ADD CONSTRAINT fk_usuario_contrato FOREIGN KEY ("login") REFERENCES acesso.usuario("login");


--
-- Name: fk_usuario_log_guia; Type: FK CONSTRAINT; Schema: alimentos; Owner: -
--

ALTER TABLE ONLY log_guia_remessa
    ADD CONSTRAINT fk_usuario_log_guia FOREIGN KEY ("login") REFERENCES acesso.usuario("login");


SET search_path = cadastro, pg_catalog;

--
-- Name: fisica_foto_idpes_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_foto
    ADD CONSTRAINT fisica_foto_idpes_fkey FOREIGN KEY (idpes) REFERENCES pessoa(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fisica_ref_cod_religiao; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fisica_ref_cod_religiao FOREIGN KEY (ref_cod_religiao) REFERENCES religiao(cod_religiao);


--
-- Name: fisica_sangue_idpes_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_sangue
    ADD CONSTRAINT fisica_sangue_idpes_fkey FOREIGN KEY (idpes) REFERENCES fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_aviso_nome_fisica; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY aviso_nome
    ADD CONSTRAINT fk_aviso_nome_fisica FOREIGN KEY (idpes) REFERENCES fisica(idpes) ON DELETE RESTRICT;


--
-- Name: fk_documento_fisica; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_fisica FOREIGN KEY (idpes) REFERENCES fisica(idpes);


--
-- Name: fk_documento_orgao_emissor_rg; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_orgao_emissor_rg FOREIGN KEY (idorg_exp_rg) REFERENCES orgao_emissor_rg(idorg_rg) ON DELETE RESTRICT;


--
-- Name: fk_documento_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_documento_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_documento_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_documento_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_documento_uf_cart_trabalho; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_uf_cart_trabalho FOREIGN KEY (sigla_uf_cart_trabalho) REFERENCES public.uf(sigla_uf);


--
-- Name: fk_documento_uf_cert_civil; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_uf_cert_civil FOREIGN KEY (sigla_uf_cert_civil) REFERENCES public.uf(sigla_uf);


--
-- Name: fk_documento_uf_rg; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY documento
    ADD CONSTRAINT fk_documento_uf_rg FOREIGN KEY (sigla_uf_exp_rg) REFERENCES public.uf(sigla_uf);


--
-- Name: fk_endereco_externo_pessoa; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_pessoa FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_endereco_externo_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_endereco_externo_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_endereco_externo_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_endereco_externo_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_endereco_externo_tipo_log; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_tipo_log FOREIGN KEY (idtlog) REFERENCES urbano.tipo_logradouro(idtlog);


--
-- Name: fk_endereco_externo_uf; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_externo
    ADD CONSTRAINT fk_endereco_externo_uf FOREIGN KEY (sigla_uf) REFERENCES public.uf(sigla_uf);


--
-- Name: fk_endereco_pes_cep_log_bai; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT fk_endereco_pes_cep_log_bai FOREIGN KEY (idbai, idlog, cep) REFERENCES urbano.cep_logradouro_bairro(idbai, idlog, cep);


--
-- Name: fk_endereco_pessoa_pessoa; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT fk_endereco_pessoa_pessoa FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_endereco_pessoa_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT fk_endereco_pessoa_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_endereco_pessoa_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT fk_endereco_pessoa_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_endereco_pessoa_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT fk_endereco_pessoa_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_endereco_pessoa_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY endereco_pessoa
    ADD CONSTRAINT fk_endereco_pessoa_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_fisica_cpf_fisica; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_cpf
    ADD CONSTRAINT fk_fisica_cpf_fisica FOREIGN KEY (idpes) REFERENCES fisica(idpes) ON DELETE RESTRICT;


--
-- Name: fk_fisica_cpf_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_cpf
    ADD CONSTRAINT fk_fisica_cpf_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_cpf_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_cpf
    ADD CONSTRAINT fk_fisica_cpf_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_cpf_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_cpf
    ADD CONSTRAINT fk_fisica_cpf_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_fisica_cpf_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_cpf
    ADD CONSTRAINT fk_fisica_cpf_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_fisica_escolaridade; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_escolaridade FOREIGN KEY (idesco) REFERENCES escolaridade(idesco);


--
-- Name: fk_fisica_estado_civil; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_estado_civil FOREIGN KEY (ideciv) REFERENCES estado_civil(ideciv);


--
-- Name: fk_fisica_municipio; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_municipio FOREIGN KEY (idmun_nascimento) REFERENCES public.municipio(idmun);


--
-- Name: fk_fisica_ocupacao; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_ocupacao FOREIGN KEY (idocup) REFERENCES ocupacao(idocup);


--
-- Name: fk_fisica_pais; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_pais FOREIGN KEY (idpais_estrangeiro) REFERENCES public.pais(idpais);


--
-- Name: fk_fisica_pessoa; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_pessoa FOREIGN KEY (idpes) REFERENCES pessoa(idpes) ON DELETE RESTRICT;


--
-- Name: fk_fisica_pessoa_conjuge; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_pessoa_conjuge FOREIGN KEY (idpes_con) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_pessoa_mae; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_pessoa_mae FOREIGN KEY (idpes_mae) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_pessoa_pai; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_pessoa_pai FOREIGN KEY (idpes_pai) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_pessoa_responsavel; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_pessoa_responsavel FOREIGN KEY (idpes_responsavel) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fisica_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_fisica_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica
    ADD CONSTRAINT fk_fisica_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_fone_pessoa_pessoa; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fone_pessoa
    ADD CONSTRAINT fk_fone_pessoa_pessoa FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_fone_pessoa_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fone_pessoa
    ADD CONSTRAINT fk_fone_pessoa_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fone_pessoa_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fone_pessoa
    ADD CONSTRAINT fk_fone_pessoa_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_fone_pessoa_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fone_pessoa
    ADD CONSTRAINT fk_fone_pessoa_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_fone_pessoa_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fone_pessoa
    ADD CONSTRAINT fk_fone_pessoa_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_funcionario_fisica; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_fisica FOREIGN KEY (idpes) REFERENCES fisica(idpes);


--
-- Name: fk_funcionario_instituicao; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_instituicao FOREIGN KEY (idins) REFERENCES acesso.instituicao(idins);


--
-- Name: fk_funcionario_setor; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_setor FOREIGN KEY (idset) REFERENCES public.setor(idset);


--
-- Name: fk_funcionario_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_funcionario_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_funcionario_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_funcionario_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_funcionario_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_hist_cartao_pes_cidadao; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY historico_cartao
    ADD CONSTRAINT fk_hist_cartao_pes_cidadao FOREIGN KEY (idpes_cidadao) REFERENCES pessoa(idpes);


--
-- Name: fk_hist_cartao_pes_emitiu; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY historico_cartao
    ADD CONSTRAINT fk_hist_cartao_pes_emitiu FOREIGN KEY (idpes_emitiu) REFERENCES pessoa(idpes);


--
-- Name: fk_juridica_pessoa; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY juridica
    ADD CONSTRAINT fk_juridica_pessoa FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_juridica_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY juridica
    ADD CONSTRAINT fk_juridica_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_juridica_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY juridica
    ADD CONSTRAINT fk_juridica_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_juridica_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY juridica
    ADD CONSTRAINT fk_juridica_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_juridica_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY juridica
    ADD CONSTRAINT fk_juridica_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_juridica_socio; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT fk_juridica_socio FOREIGN KEY (idpes_juridica) REFERENCES juridica(idpes);


--
-- Name: fk_pessoa_fonetico_pessoa; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY pessoa_fonetico
    ADD CONSTRAINT fk_pessoa_fonetico_pessoa FOREIGN KEY (idpes) REFERENCES pessoa(idpes);


--
-- Name: fk_pessoa_pessoa_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY pessoa
    ADD CONSTRAINT fk_pessoa_pessoa_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_pessoa_pessoa_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY pessoa
    ADD CONSTRAINT fk_pessoa_pessoa_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_pessoa_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY pessoa
    ADD CONSTRAINT fk_pessoa_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_pessoa_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY pessoa
    ADD CONSTRAINT fk_pessoa_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_pessoa_socio; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT fk_pessoa_socio FOREIGN KEY (idpes_fisica) REFERENCES pessoa(idpes);


--
-- Name: fk_socio_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT fk_socio_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_socio_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT fk_socio_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_socio_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT fk_socio_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_socio_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY socio
    ADD CONSTRAINT fk_socio_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: pessoa_deficiencia_ref_cod_deficiencia_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_deficiencia
    ADD CONSTRAINT pessoa_deficiencia_ref_cod_deficiencia_fkey FOREIGN KEY (ref_cod_deficiencia) REFERENCES deficiencia(cod_deficiencia) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pessoa_deficiencia_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_deficiencia
    ADD CONSTRAINT pessoa_deficiencia_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pessoa_raca_ref_cod_deficiencia_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_raca
    ADD CONSTRAINT pessoa_raca_ref_cod_deficiencia_fkey FOREIGN KEY (ref_cod_raca) REFERENCES raca(cod_raca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pessoa_raca_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY fisica_raca
    ADD CONSTRAINT pessoa_raca_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: religiao_idpes_cad_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY religiao
    ADD CONSTRAINT religiao_idpes_cad_fkey FOREIGN KEY (idpes_cad) REFERENCES fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: religiao_idpes_exc_fkey; Type: FK CONSTRAINT; Schema: cadastro; Owner: -
--

ALTER TABLE ONLY religiao
    ADD CONSTRAINT religiao_idpes_exc_fkey FOREIGN KEY (idpes_exc) REFERENCES fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


SET search_path = consistenciacao, pg_catalog;

--
-- Name: fk_campo_metadado_campo_consis; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY campo_metadado
    ADD CONSTRAINT fk_campo_metadado_campo_consis FOREIGN KEY (idcam) REFERENCES campo_consistenciacao(idcam);


--
-- Name: fk_campo_metadado_metadado; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY campo_metadado
    ADD CONSTRAINT fk_campo_metadado_metadado FOREIGN KEY (idmet) REFERENCES metadado(idmet);


--
-- Name: fk_campo_metadado_regra_campo; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY campo_metadado
    ADD CONSTRAINT fk_campo_metadado_regra_campo FOREIGN KEY (idreg) REFERENCES regra_campo(idreg);


--
-- Name: fk_confrontacao_metadado; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY confrontacao
    ADD CONSTRAINT fk_confrontacao_metadado FOREIGN KEY (idmet) REFERENCES metadado(idmet);


--
-- Name: fk_confrontacao_pessoa_instituicao; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY confrontacao
    ADD CONSTRAINT fk_confrontacao_pessoa_instituicao FOREIGN KEY (idins, idpes) REFERENCES acesso.pessoa_instituicao(idins, idpes);


--
-- Name: fk_hist_campo_campo_consist; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY historico_campo
    ADD CONSTRAINT fk_hist_campo_campo_consist FOREIGN KEY (idcam) REFERENCES campo_consistenciacao(idcam) ON DELETE CASCADE;


--
-- Name: fk_historico_campo_pessoa; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY historico_campo
    ADD CONSTRAINT fk_historico_campo_pessoa FOREIGN KEY (idpes) REFERENCES cadastro.pessoa(idpes) ON DELETE CASCADE;


--
-- Name: fk_inc_pessoa_possivel_incoerencia; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_pessoa_possivel
    ADD CONSTRAINT fk_inc_pessoa_possivel_incoerencia FOREIGN KEY (idinc) REFERENCES incoerencia(idinc) ON DELETE CASCADE;


--
-- Name: fk_inc_pessoa_possivel_pessoa; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_pessoa_possivel
    ADD CONSTRAINT fk_inc_pessoa_possivel_pessoa FOREIGN KEY (idpes) REFERENCES cadastro.pessoa(idpes);


--
-- Name: fk_inc_tipo_inc_incoerencia; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_tipo_incoerencia
    ADD CONSTRAINT fk_inc_tipo_inc_incoerencia FOREIGN KEY (idinc) REFERENCES incoerencia(idinc) ON DELETE CASCADE;


--
-- Name: fk_inc_tipo_inc_tipo_incoerencia; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_tipo_incoerencia
    ADD CONSTRAINT fk_inc_tipo_inc_tipo_incoerencia FOREIGN KEY (id_tipo_inc) REFERENCES tipo_incoerencia(id_tipo_inc);


--
-- Name: fk_incoerencia_confrontacao; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia
    ADD CONSTRAINT fk_incoerencia_confrontacao FOREIGN KEY (idcon) REFERENCES confrontacao(idcon);


--
-- Name: fk_incoerencia_documento_incoerencia; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_documento
    ADD CONSTRAINT fk_incoerencia_documento_incoerencia FOREIGN KEY (idinc) REFERENCES incoerencia(idinc) ON DELETE CASCADE;


--
-- Name: fk_incoerencia_endereco_incoerencia; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_endereco
    ADD CONSTRAINT fk_incoerencia_endereco_incoerencia FOREIGN KEY (idinc) REFERENCES incoerencia(idinc) ON DELETE CASCADE;


--
-- Name: fk_incoerencia_fone_incoerencia; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY incoerencia_fone
    ADD CONSTRAINT fk_incoerencia_fone_incoerencia FOREIGN KEY (idinc) REFERENCES incoerencia(idinc) ON DELETE CASCADE;


--
-- Name: fk_metadado_fonte; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY metadado
    ADD CONSTRAINT fk_metadado_fonte FOREIGN KEY (idfon) REFERENCES fonte(idfon);


--
-- Name: fk_oco_reg_cam_regra_campo; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY ocorrencia_regra_campo
    ADD CONSTRAINT fk_oco_reg_cam_regra_campo FOREIGN KEY (idreg) REFERENCES regra_campo(idreg);


--
-- Name: fk_tipo_incoerencia_campo_consis; Type: FK CONSTRAINT; Schema: consistenciacao; Owner: -
--

ALTER TABLE ONLY tipo_incoerencia
    ADD CONSTRAINT fk_tipo_incoerencia_campo_consis FOREIGN KEY (idcam) REFERENCES campo_consistenciacao(idcam);


SET search_path = modules, pg_catalog;

--
-- Name: calendario_turma_calendario_dia_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY calendario_turma
    ADD CONSTRAINT calendario_turma_calendario_dia_fk FOREIGN KEY (calendario_ano_letivo_id, mes, dia) REFERENCES pmieducar.calendario_dia(ref_cod_calendario_ano_letivo, mes, dia) MATCH FULL ON DELETE CASCADE;


--
-- Name: componente_curricular_ano_escolar_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY componente_curricular_ano_escolar
    ADD CONSTRAINT componente_curricular_ano_escolar_fk FOREIGN KEY (componente_curricular_id) REFERENCES componente_curricular(id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: componente_curricular_area_conhecimento_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY componente_curricular
    ADD CONSTRAINT componente_curricular_area_conhecimento_fk FOREIGN KEY (area_conhecimento_id, instituicao_id) REFERENCES area_conhecimento(id, instituicao_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: componente_curricular_turma_componente_curricular_fkey; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY componente_curricular_turma
    ADD CONSTRAINT componente_curricular_turma_componente_curricular_fkey FOREIGN KEY (componente_curricular_id) REFERENCES componente_curricular(id) ON DELETE RESTRICT;


--
-- Name: componente_curricular_turma_fkey; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY componente_curricular_turma
    ADD CONSTRAINT componente_curricular_turma_fkey FOREIGN KEY (turma_id) REFERENCES pmieducar.turma(cod_turma) ON DELETE CASCADE;


--
-- Name: docente_licenciatura_ies_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY docente_licenciatura
    ADD CONSTRAINT docente_licenciatura_ies_fk FOREIGN KEY (ies_id) REFERENCES educacenso_ies(id) ON DELETE RESTRICT;


--
-- Name: educacenso_cod_aluno_cod_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY educacenso_cod_aluno
    ADD CONSTRAINT educacenso_cod_aluno_cod_aluno_fk FOREIGN KEY (cod_aluno) REFERENCES pmieducar.aluno(cod_aluno) ON DELETE CASCADE;


--
-- Name: falta_componente_curricular_falta_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY falta_componente_curricular
    ADD CONSTRAINT falta_componente_curricular_falta_aluno_fk FOREIGN KEY (falta_aluno_id) REFERENCES falta_aluno(id) ON DELETE CASCADE;


--
-- Name: falta_geral_falta_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY falta_geral
    ADD CONSTRAINT falta_geral_falta_aluno_fk FOREIGN KEY (falta_aluno_id) REFERENCES falta_aluno(id) ON DELETE CASCADE;


--
-- Name: nota_componente_curricular_media_nota_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY nota_componente_curricular_media
    ADD CONSTRAINT nota_componente_curricular_media_nota_aluno_fk FOREIGN KEY (nota_aluno_id) REFERENCES nota_aluno(id) ON DELETE CASCADE;


--
-- Name: nota_componente_curricular_nota_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY nota_componente_curricular
    ADD CONSTRAINT nota_componente_curricular_nota_aluno_fk FOREIGN KEY (nota_aluno_id) REFERENCES nota_aluno(id) ON DELETE CASCADE;


--
-- Name: parecer_componente_curricular_parecer_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY parecer_componente_curricular
    ADD CONSTRAINT parecer_componente_curricular_parecer_aluno_fk FOREIGN KEY (parecer_aluno_id) REFERENCES parecer_aluno(id) ON DELETE CASCADE;


--
-- Name: parecer_geral_parecer_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY parecer_geral
    ADD CONSTRAINT parecer_geral_parecer_aluno_fk FOREIGN KEY (parecer_aluno_id) REFERENCES parecer_aluno(id) ON DELETE CASCADE;


--
-- Name: regra_avaliacao_formula_media_formula_media_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY regra_avaliacao
    ADD CONSTRAINT regra_avaliacao_formula_media_formula_media_fk FOREIGN KEY (formula_media_id, instituicao_id) REFERENCES formula_media(id, instituicao_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: regra_avaliacao_formula_media_formula_recuperacao_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY regra_avaliacao
    ADD CONSTRAINT regra_avaliacao_formula_media_formula_recuperacao_fk FOREIGN KEY (formula_recuperacao_id, instituicao_id) REFERENCES formula_media(id, instituicao_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: regra_avaliacao_tabela_arredondamento_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY regra_avaliacao
    ADD CONSTRAINT regra_avaliacao_tabela_arredondamento_fk FOREIGN KEY (tabela_arredondamento_id, instituicao_id) REFERENCES tabela_arredondamento(id, instituicao_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tabela_arredondamento_tabela_arredondamento_valor_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY tabela_arredondamento_valor
    ADD CONSTRAINT tabela_arredondamento_tabela_arredondamento_valor_fk FOREIGN KEY (tabela_arredondamento_id) REFERENCES tabela_arredondamento(id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transporte_aluno_aluno_fk; Type: FK CONSTRAINT; Schema: modules; Owner: -
--

ALTER TABLE ONLY transporte_aluno
    ADD CONSTRAINT transporte_aluno_aluno_fk FOREIGN KEY (aluno_id) REFERENCES pmieducar.aluno(cod_aluno) ON DELETE CASCADE;


SET search_path = pmiacoes, pg_catalog;

--
-- Name: acao_governo_arquivo_ref_cod_acao_governo_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_arquivo
    ADD CONSTRAINT acao_governo_arquivo_ref_cod_acao_governo_fkey FOREIGN KEY (ref_cod_acao_governo) REFERENCES acao_governo(cod_acao_governo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_arquivo_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_arquivo
    ADD CONSTRAINT acao_governo_arquivo_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_categoria_ref_cod_acao_governo_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_categoria
    ADD CONSTRAINT acao_governo_categoria_ref_cod_acao_governo_fkey FOREIGN KEY (ref_cod_acao_governo) REFERENCES acao_governo(cod_acao_governo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_categoria_ref_cod_categoria_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_categoria
    ADD CONSTRAINT acao_governo_categoria_ref_cod_categoria_fkey FOREIGN KEY (ref_cod_categoria) REFERENCES categoria(cod_categoria) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_foto_portal_ref_cod_acao_governo_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_foto_portal
    ADD CONSTRAINT acao_governo_foto_portal_ref_cod_acao_governo_fkey FOREIGN KEY (ref_cod_acao_governo) REFERENCES acao_governo(cod_acao_governo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_foto_portal_ref_cod_foto_portal_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_foto_portal
    ADD CONSTRAINT acao_governo_foto_portal_ref_cod_foto_portal_fkey FOREIGN KEY (ref_cod_foto_portal) REFERENCES portal.foto_portal(cod_foto_portal) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_foto_portal_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_foto_portal
    ADD CONSTRAINT acao_governo_foto_portal_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_foto_ref_cod_acao_governo_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_foto
    ADD CONSTRAINT acao_governo_foto_ref_cod_acao_governo_fkey FOREIGN KEY (ref_cod_acao_governo) REFERENCES acao_governo(cod_acao_governo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_foto_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_foto
    ADD CONSTRAINT acao_governo_foto_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_noticia_ref_cod_acao_governo_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_noticia
    ADD CONSTRAINT acao_governo_noticia_ref_cod_acao_governo_fkey FOREIGN KEY (ref_cod_acao_governo) REFERENCES acao_governo(cod_acao_governo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_noticia_ref_cod_not_portal_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_noticia
    ADD CONSTRAINT acao_governo_noticia_ref_cod_not_portal_fkey FOREIGN KEY (ref_cod_not_portal) REFERENCES portal.not_portal(cod_not_portal) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_noticia_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_noticia
    ADD CONSTRAINT acao_governo_noticia_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo
    ADD CONSTRAINT acao_governo_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_ref_funcionario_exc_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo
    ADD CONSTRAINT acao_governo_ref_funcionario_exc_fkey FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_setor_ref_cod_acao_governo_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_setor
    ADD CONSTRAINT acao_governo_setor_ref_cod_acao_governo_fkey FOREIGN KEY (ref_cod_acao_governo) REFERENCES acao_governo(cod_acao_governo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_setor_ref_cod_setor_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_setor
    ADD CONSTRAINT acao_governo_setor_ref_cod_setor_fkey FOREIGN KEY (ref_cod_setor) REFERENCES pmidrh.setor(cod_setor) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acao_governo_setor_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY acao_governo_setor
    ADD CONSTRAINT acao_governo_setor_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: categoria_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY categoria
    ADD CONSTRAINT categoria_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: categoria_ref_funcionario_exc_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY categoria
    ADD CONSTRAINT categoria_ref_funcionario_exc_fkey FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: secretaria_responsavel_ref_cod_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY secretaria_responsavel
    ADD CONSTRAINT secretaria_responsavel_ref_cod_funcionario_cad_fkey FOREIGN KEY (ref_cod_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: secretaria_responsavel_ref_cod_setor_fkey; Type: FK CONSTRAINT; Schema: pmiacoes; Owner: -
--

ALTER TABLE ONLY secretaria_responsavel
    ADD CONSTRAINT secretaria_responsavel_ref_cod_setor_fkey FOREIGN KEY (ref_cod_setor) REFERENCES pmidrh.setor(cod_setor) ON UPDATE RESTRICT ON DELETE RESTRICT;


SET search_path = pmicontrolesis, pg_catalog;

--
-- Name: acontecimento_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY acontecimento
    ADD CONSTRAINT acontecimento_ref_funcionario_cad_fk FOREIGN KEY (ref_cod_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: acontecimento_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY tipo_acontecimento
    ADD CONSTRAINT acontecimento_ref_funcionario_cad_fk FOREIGN KEY (ref_cod_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: acontecimento_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY acontecimento
    ADD CONSTRAINT acontecimento_ref_funcionario_exc_fk FOREIGN KEY (ref_cod_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: acontecimento_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY tipo_acontecimento
    ADD CONSTRAINT acontecimento_ref_funcionario_exc_fk FOREIGN KEY (ref_cod_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: fk_to_imagem_ico; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT fk_to_imagem_ico FOREIGN KEY (ref_cod_ico) REFERENCES portal.imagem(cod_imagem) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_to_tutor; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT fk_to_tutor FOREIGN KEY (ref_cod_tutormenu) REFERENCES tutormenu(cod_tutormenu);


--
-- Name: foto_evento_ibfk_1; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY foto_evento
    ADD CONSTRAINT foto_evento_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: foto_vinc_ref_cod_acontecimento_fkey; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY foto_vinc
    ADD CONSTRAINT foto_vinc_ref_cod_acontecimento_fkey FOREIGN KEY (ref_cod_acontecimento) REFERENCES acontecimento(cod_acontecimento);


--
-- Name: foto_vinc_ref_cod_foto_evento_fkey; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY foto_vinc
    ADD CONSTRAINT foto_vinc_ref_cod_foto_evento_fkey FOREIGN KEY (ref_cod_foto_evento) REFERENCES foto_evento(cod_foto_evento);


--
-- Name: itinerario_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY itinerario
    ADD CONSTRAINT itinerario_ref_funcionario_cad_fk FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: itinerario_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY itinerario
    ADD CONSTRAINT itinerario_ref_funcionario_exc_fk FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: menu_portal_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY menu_portal
    ADD CONSTRAINT menu_portal_ref_funcionario_cad_fk FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: menu_portal_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY menu_portal
    ADD CONSTRAINT menu_portal_ref_funcionario_exc_fk FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: menu_ref_cod_menu_pai_fkey; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT menu_ref_cod_menu_pai_fkey FOREIGN KEY (ref_cod_menu_pai) REFERENCES menu(cod_menu) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: menu_ref_cod_menu_submenu_fkey; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT menu_ref_cod_menu_submenu_fkey FOREIGN KEY (ref_cod_menu_submenu) REFERENCES portal.menu_submenu(cod_menu_submenu) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: portais_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY portais
    ADD CONSTRAINT portais_ref_funcionario_cad_fk FOREIGN KEY (ref_cod_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: portais_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY portais
    ADD CONSTRAINT portais_ref_funcionario_exc_fk FOREIGN KEY (ref_cod_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: servicos_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY servicos
    ADD CONSTRAINT servicos_ref_funcionario_cad_fk FOREIGN KEY (ref_cod_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: servicos_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY servicos
    ADD CONSTRAINT servicos_ref_funcionario_exc_fk FOREIGN KEY (ref_cod_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: sistema_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY sistema
    ADD CONSTRAINT sistema_ref_funcionario_cad_fk FOREIGN KEY (ref_cod_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: sistema_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY sistema
    ADD CONSTRAINT sistema_ref_funcionario_exc_fk FOREIGN KEY (ref_cod_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: submenu_portal_ref_cod_menu_portal_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY submenu_portal
    ADD CONSTRAINT submenu_portal_ref_cod_menu_portal_fk FOREIGN KEY (ref_cod_menu_portal) REFERENCES menu_portal(cod_menu_portal);


--
-- Name: submenu_portal_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY submenu_portal
    ADD CONSTRAINT submenu_portal_ref_funcionario_cad_fk FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: submenu_portal_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY topo_portal
    ADD CONSTRAINT submenu_portal_ref_funcionario_cad_fk FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: submenu_portal_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY submenu_portal
    ADD CONSTRAINT submenu_portal_ref_funcionario_exc_fk FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: submenu_portal_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY topo_portal
    ADD CONSTRAINT submenu_portal_ref_funcionario_exc_fk FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: telefones_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY telefones
    ADD CONSTRAINT telefones_ref_funcionario_cad_fk FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: telefones_ref_funcionario_exc_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY telefones
    ADD CONSTRAINT telefones_ref_funcionario_exc_fk FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj);


--
-- Name: tipo_acontecimento_ref_funcionario_cad_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY acontecimento
    ADD CONSTRAINT tipo_acontecimento_ref_funcionario_cad_fk FOREIGN KEY (ref_cod_tipo_acontecimento) REFERENCES tipo_acontecimento(cod_tipo_acontecimento);


--
-- Name: topo_portal_ref_cod_menu_portal_fk; Type: FK CONSTRAINT; Schema: pmicontrolesis; Owner: -
--

ALTER TABLE ONLY topo_portal
    ADD CONSTRAINT topo_portal_ref_cod_menu_portal_fk FOREIGN KEY (ref_cod_menu_portal) REFERENCES menu_portal(cod_menu_portal);


SET search_path = pmidrh, pg_catalog;

--
-- Name: diaria_ref_cod_diaria_grupo_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY diaria
    ADD CONSTRAINT diaria_ref_cod_diaria_grupo_fkey FOREIGN KEY (ref_cod_diaria_grupo) REFERENCES diaria_grupo(cod_diaria_grupo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: diaria_ref_cod_setor; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY diaria
    ADD CONSTRAINT diaria_ref_cod_setor FOREIGN KEY (ref_cod_setor) REFERENCES setor(cod_setor);


--
-- Name: diaria_ref_funcionario_cadastro_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY diaria
    ADD CONSTRAINT diaria_ref_funcionario_cadastro_fkey FOREIGN KEY (ref_funcionario_cadastro) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: diaria_ref_funcionario_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY diaria
    ADD CONSTRAINT diaria_ref_funcionario_fkey FOREIGN KEY (ref_funcionario) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: diaria_valores_ref_cod_diaria_grupo_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY diaria_valores
    ADD CONSTRAINT diaria_valores_ref_cod_diaria_grupo_fkey FOREIGN KEY (ref_cod_diaria_grupo) REFERENCES diaria_grupo(cod_diaria_grupo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: diaria_valores_ref_funcionario_cadastro_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY diaria_valores
    ADD CONSTRAINT diaria_valores_ref_funcionario_cadastro_fkey FOREIGN KEY (ref_funcionario_cadastro) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_setor_pai; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT fk_setor_pai FOREIGN KEY (ref_cod_setor) REFERENCES setor(cod_setor) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_to_idpes_resp; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT fk_to_idpes_resp FOREIGN KEY (ref_idpes_resp) REFERENCES cadastro.fisica(idpes);


--
-- Name: setor_ref_cod_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT setor_ref_cod_pessoa_cad_fkey FOREIGN KEY (ref_cod_pessoa_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: setor_ref_cod_pessoa_exc_fkey; Type: FK CONSTRAINT; Schema: pmidrh; Owner: -
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT setor_ref_cod_pessoa_exc_fkey FOREIGN KEY (ref_cod_pessoa_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


SET search_path = pmieducar, pg_catalog;

--
-- Name: acervo_acervo_assunto_ref_cod_acervo_assunto_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_acervo_assunto
    ADD CONSTRAINT acervo_acervo_assunto_ref_cod_acervo_assunto_fkey FOREIGN KEY (ref_cod_acervo_assunto) REFERENCES acervo_assunto(cod_acervo_assunto) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_acervo_assunto_ref_cod_acervo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_acervo_assunto
    ADD CONSTRAINT acervo_acervo_assunto_ref_cod_acervo_fkey FOREIGN KEY (ref_cod_acervo) REFERENCES acervo(cod_acervo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_acervo_autor_ref_cod_acervo_autor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_acervo_autor
    ADD CONSTRAINT acervo_acervo_autor_ref_cod_acervo_autor_fkey FOREIGN KEY (ref_cod_acervo_autor) REFERENCES acervo_autor(cod_acervo_autor) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_acervo_autor_ref_cod_acervo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_acervo_autor
    ADD CONSTRAINT acervo_acervo_autor_ref_cod_acervo_fkey FOREIGN KEY (ref_cod_acervo) REFERENCES acervo(cod_acervo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_assunto_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_assunto
    ADD CONSTRAINT acervo_assunto_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_assunto_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_assunto
    ADD CONSTRAINT acervo_assunto_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_assunto_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_assunto
    ADD CONSTRAINT acervo_assunto_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_autor_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_autor
    ADD CONSTRAINT acervo_autor_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_autor_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_autor
    ADD CONSTRAINT acervo_autor_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_colecao_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_colecao
    ADD CONSTRAINT acervo_colecao_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_colecao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_colecao
    ADD CONSTRAINT acervo_colecao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_colecao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_colecao
    ADD CONSTRAINT acervo_colecao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_editora_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_editora
    ADD CONSTRAINT acervo_editora_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_editora_ref_idtlog_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_editora
    ADD CONSTRAINT acervo_editora_ref_idtlog_fkey FOREIGN KEY (ref_idtlog) REFERENCES urbano.tipo_logradouro(idtlog) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_editora_ref_sigla_uf_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_editora
    ADD CONSTRAINT acervo_editora_ref_sigla_uf_fkey FOREIGN KEY (ref_sigla_uf) REFERENCES public.uf(sigla_uf) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_editora_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_editora
    ADD CONSTRAINT acervo_editora_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_editora_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_editora
    ADD CONSTRAINT acervo_editora_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_idioma_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_idioma
    ADD CONSTRAINT acervo_idioma_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_idioma_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_idioma
    ADD CONSTRAINT acervo_idioma_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_idioma_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_idioma
    ADD CONSTRAINT acervo_idioma_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_acervo_colecao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_cod_acervo_colecao_fkey FOREIGN KEY (ref_cod_acervo_colecao) REFERENCES acervo_colecao(cod_acervo_colecao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_acervo_editora_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_cod_acervo_editora_fkey FOREIGN KEY (ref_cod_acervo_editora) REFERENCES acervo_editora(cod_acervo_editora) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_acervo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_cod_acervo_fkey FOREIGN KEY (ref_cod_acervo) REFERENCES acervo(cod_acervo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_acervo_idioma_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_cod_acervo_idioma_fkey FOREIGN KEY (ref_cod_acervo_idioma) REFERENCES acervo_idioma(cod_acervo_idioma) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_biblioteca; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_cod_biblioteca FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo_autor
    ADD CONSTRAINT acervo_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_cod_exemplar_tipo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_cod_exemplar_tipo_fkey FOREIGN KEY (ref_cod_exemplar_tipo) REFERENCES exemplar_tipo(cod_exemplar_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: acervo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY acervo
    ADD CONSTRAINT acervo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: aluno_beneficio_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY aluno_beneficio
    ADD CONSTRAINT aluno_beneficio_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: aluno_beneficio_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY aluno_beneficio
    ADD CONSTRAINT aluno_beneficio_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: aluno_ref_cod_aluno_beneficio_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY aluno
    ADD CONSTRAINT aluno_ref_cod_aluno_beneficio_fkey FOREIGN KEY (ref_cod_aluno_beneficio) REFERENCES aluno_beneficio(cod_aluno_beneficio) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: aluno_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY aluno
    ADD CONSTRAINT aluno_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES cadastro.fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: aluno_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY aluno
    ADD CONSTRAINT aluno_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: aluno_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY aluno
    ADD CONSTRAINT aluno_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: ano_letivo_modulo_ref_cod_modulo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY ano_letivo_modulo
    ADD CONSTRAINT ano_letivo_modulo_ref_cod_modulo_fkey FOREIGN KEY (ref_cod_modulo) REFERENCES modulo(cod_modulo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: ano_letivo_modulo_ref_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY ano_letivo_modulo
    ADD CONSTRAINT ano_letivo_modulo_ref_ref_cod_escola_fkey FOREIGN KEY (ref_ref_cod_escola, ref_ano) REFERENCES escola_ano_letivo(ref_cod_escola, ano) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: avaliacao_desempenho_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY avaliacao_desempenho
    ADD CONSTRAINT avaliacao_desempenho_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: avaliacao_desempenho_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY avaliacao_desempenho
    ADD CONSTRAINT avaliacao_desempenho_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: avaliacao_desempenho_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY avaliacao_desempenho
    ADD CONSTRAINT avaliacao_desempenho_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: biblioteca_dia_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY biblioteca_dia
    ADD CONSTRAINT biblioteca_dia_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: biblioteca_feriados_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY biblioteca_feriados
    ADD CONSTRAINT biblioteca_feriados_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: biblioteca_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY biblioteca
    ADD CONSTRAINT biblioteca_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: biblioteca_ref_cod_instituicao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY biblioteca
    ADD CONSTRAINT biblioteca_ref_cod_instituicao_fkey FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: biblioteca_usuario_ref_cod_biblioteca_fk; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY biblioteca_usuario
    ADD CONSTRAINT biblioteca_usuario_ref_cod_biblioteca_fk FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_ano_letivo_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_ano_letivo
    ADD CONSTRAINT calendario_ano_letivo_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_ano_letivo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_ano_letivo
    ADD CONSTRAINT calendario_ano_letivo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_ano_letivo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_ano_letivo
    ADD CONSTRAINT calendario_ano_letivo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_anotacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_anotacao
    ADD CONSTRAINT calendario_anotacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_anotacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_anotacao
    ADD CONSTRAINT calendario_anotacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_anotacao_ref_cod_calendario_anotacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia_anotacao
    ADD CONSTRAINT calendario_dia_anotacao_ref_cod_calendario_anotacao_fkey FOREIGN KEY (ref_cod_calendario_anotacao) REFERENCES calendario_anotacao(cod_calendario_anotacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_anotacao_ref_ref_cod_calendario_ano_letivo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia_anotacao
    ADD CONSTRAINT calendario_dia_anotacao_ref_ref_cod_calendario_ano_letivo_fkey FOREIGN KEY (ref_ref_cod_calendario_ano_letivo, ref_mes, ref_dia) REFERENCES calendario_dia(ref_cod_calendario_ano_letivo, mes, dia) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_motivo_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia_motivo
    ADD CONSTRAINT calendario_dia_motivo_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_motivo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia_motivo
    ADD CONSTRAINT calendario_dia_motivo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_motivo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia_motivo
    ADD CONSTRAINT calendario_dia_motivo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_ref_cod_calendario_ano_letivo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia
    ADD CONSTRAINT calendario_dia_ref_cod_calendario_ano_letivo_fkey FOREIGN KEY (ref_cod_calendario_ano_letivo) REFERENCES calendario_ano_letivo(cod_calendario_ano_letivo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_ref_cod_calendario_dia_motivo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia
    ADD CONSTRAINT calendario_dia_ref_cod_calendario_dia_motivo_fkey FOREIGN KEY (ref_cod_calendario_dia_motivo) REFERENCES calendario_dia_motivo(cod_calendario_dia_motivo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia
    ADD CONSTRAINT calendario_dia_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: calendario_dia_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY calendario_dia
    ADD CONSTRAINT calendario_dia_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: categoria_nivel_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY categoria_nivel
    ADD CONSTRAINT categoria_nivel_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: categoria_nivel_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY categoria_nivel
    ADD CONSTRAINT categoria_nivel_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT cliente_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES cadastro.fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT cliente_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente
    ADD CONSTRAINT cliente_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_suspensao_ref_cod_cliente_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_suspensao
    ADD CONSTRAINT cliente_suspensao_ref_cod_cliente_fkey FOREIGN KEY (ref_cod_cliente) REFERENCES cliente(cod_cliente) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_suspensao_ref_cod_motivo_suspensao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_suspensao
    ADD CONSTRAINT cliente_suspensao_ref_cod_motivo_suspensao_fkey FOREIGN KEY (ref_cod_motivo_suspensao) REFERENCES motivo_suspensao(cod_motivo_suspensao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_suspensao_ref_usuario_libera_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_suspensao
    ADD CONSTRAINT cliente_suspensao_ref_usuario_libera_fkey FOREIGN KEY (ref_usuario_libera) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_suspensao_ref_usuario_suspende_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_suspensao
    ADD CONSTRAINT cliente_suspensao_ref_usuario_suspende_fkey FOREIGN KEY (ref_usuario_suspende) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_cliente_ibfk1; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo_cliente
    ADD CONSTRAINT cliente_tipo_cliente_ibfk1 FOREIGN KEY (ref_cod_cliente) REFERENCES cliente(cod_cliente) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_cliente_ibfk2; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo_cliente
    ADD CONSTRAINT cliente_tipo_cliente_ibfk2 FOREIGN KEY (ref_cod_cliente_tipo) REFERENCES cliente_tipo(cod_cliente_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_exemplar_tipo_ref_cod_cliente_tipo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo_exemplar_tipo
    ADD CONSTRAINT cliente_tipo_exemplar_tipo_ref_cod_cliente_tipo_fkey FOREIGN KEY (ref_cod_cliente_tipo) REFERENCES cliente_tipo(cod_cliente_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_exemplar_tipo_ref_cod_exemplar_tipo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo_exemplar_tipo
    ADD CONSTRAINT cliente_tipo_exemplar_tipo_ref_cod_exemplar_tipo_fkey FOREIGN KEY (ref_cod_exemplar_tipo) REFERENCES exemplar_tipo(cod_exemplar_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo
    ADD CONSTRAINT cliente_tipo_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo
    ADD CONSTRAINT cliente_tipo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: cliente_tipo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo
    ADD CONSTRAINT cliente_tipo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: coffebreak_tipo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY coffebreak_tipo
    ADD CONSTRAINT coffebreak_tipo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: coffebreak_tipo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY coffebreak_tipo
    ADD CONSTRAINT coffebreak_tipo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: curso_ref_cod_instituicao_fk; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_ref_cod_instituicao_fk FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: curso_ref_cod_nivel_ensino_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_ref_cod_nivel_ensino_fkey FOREIGN KEY (ref_cod_nivel_ensino) REFERENCES nivel_ensino(cod_nivel_ensino) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: curso_ref_cod_tipo_ensino_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_ref_cod_tipo_ensino_fkey FOREIGN KEY (ref_cod_tipo_ensino) REFERENCES tipo_ensino(cod_tipo_ensino) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: curso_ref_cod_tipo_regime_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_ref_cod_tipo_regime_fkey FOREIGN KEY (ref_cod_tipo_regime) REFERENCES tipo_regime(cod_tipo_regime) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: curso_ref_cod_usuario_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_ref_cod_usuario_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: curso_ref_usuario_exc_fk; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY curso
    ADD CONSTRAINT curso_ref_usuario_exc_fk FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_ref_cod_curso; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina
    ADD CONSTRAINT disciplina_ref_cod_curso FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina
    ADD CONSTRAINT disciplina_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina
    ADD CONSTRAINT disciplina_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_serie_ref_cod_disciplina_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina_serie
    ADD CONSTRAINT disciplina_serie_ref_cod_disciplina_fkey FOREIGN KEY (ref_cod_disciplina) REFERENCES disciplina(cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_serie_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina_serie
    ADD CONSTRAINT disciplina_serie_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie) REFERENCES serie(cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_topico_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina_topico
    ADD CONSTRAINT disciplina_topico_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: disciplina_topico_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY disciplina_topico
    ADD CONSTRAINT disciplina_topico_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: dispensa_disciplina_ref_cod_matricula_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY dispensa_disciplina
    ADD CONSTRAINT dispensa_disciplina_ref_cod_matricula_fkey FOREIGN KEY (ref_cod_matricula) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: dispensa_disciplina_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY dispensa_disciplina
    ADD CONSTRAINT dispensa_disciplina_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie, ref_cod_escola, ref_cod_disciplina) REFERENCES escola_serie_disciplina(ref_ref_cod_serie, ref_ref_cod_escola, ref_cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: dispensa_disciplina_ref_cod_tipo_dispensa_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY dispensa_disciplina
    ADD CONSTRAINT dispensa_disciplina_ref_cod_tipo_dispensa_fkey FOREIGN KEY (ref_cod_tipo_dispensa) REFERENCES tipo_dispensa(cod_tipo_dispensa) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: dispensa_disciplina_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY dispensa_disciplina
    ADD CONSTRAINT dispensa_disciplina_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: dispensa_disciplina_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY dispensa_disciplina
    ADD CONSTRAINT dispensa_disciplina_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ano_letivo_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_ano_letivo
    ADD CONSTRAINT escola_ano_letivo_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ano_letivo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_ano_letivo
    ADD CONSTRAINT escola_ano_letivo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ano_letivo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_ano_letivo
    ADD CONSTRAINT escola_ano_letivo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_complemento_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_complemento
    ADD CONSTRAINT escola_complemento_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_complemento_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_complemento
    ADD CONSTRAINT escola_complemento_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_curso_ref_cod_curso_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_curso
    ADD CONSTRAINT escola_curso_ref_cod_curso_fkey FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_curso_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_curso
    ADD CONSTRAINT escola_curso_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_curso_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_curso
    ADD CONSTRAINT escola_curso_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_curso_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_curso
    ADD CONSTRAINT escola_curso_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_localizacao_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_localizacao
    ADD CONSTRAINT escola_localizacao_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_localizacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_localizacao
    ADD CONSTRAINT escola_localizacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_localizacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_localizacao
    ADD CONSTRAINT escola_localizacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_rede_ensino_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_rede_ensino
    ADD CONSTRAINT escola_rede_ensino_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_rede_ensino_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_rede_ensino
    ADD CONSTRAINT escola_rede_ensino_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ref_cod_escola_localizacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_ref_cod_escola_localizacao_fkey FOREIGN KEY (ref_cod_escola_localizacao) REFERENCES escola_localizacao(cod_escola_localizacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ref_cod_escola_rede_ensino_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_ref_cod_escola_rede_ensino_fkey FOREIGN KEY (ref_cod_escola_rede_ensino) REFERENCES escola_rede_ensino(cod_escola_rede_ensino) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ref_cod_instituicao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_ref_cod_instituicao_fkey FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES cadastro.juridica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola
    ADD CONSTRAINT escola_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_serie_disciplina_ref_cod_disciplina_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_serie_disciplina
    ADD CONSTRAINT escola_serie_disciplina_ref_cod_disciplina_fkey FOREIGN KEY (ref_cod_disciplina) REFERENCES modules.componente_curricular(id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_serie_disciplina_ref_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_serie_disciplina
    ADD CONSTRAINT escola_serie_disciplina_ref_ref_cod_escola_fkey FOREIGN KEY (ref_ref_cod_escola, ref_ref_cod_serie) REFERENCES escola_serie(ref_cod_escola, ref_cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_serie_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_serie
    ADD CONSTRAINT escola_serie_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_serie_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_serie
    ADD CONSTRAINT escola_serie_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie) REFERENCES serie(cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_serie_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_serie
    ADD CONSTRAINT escola_serie_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: escola_serie_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY escola_serie
    ADD CONSTRAINT escola_serie_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_emprestimo_ref_cod_cliente_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_emprestimo
    ADD CONSTRAINT exemplar_emprestimo_ref_cod_cliente_fkey FOREIGN KEY (ref_cod_cliente) REFERENCES cliente(cod_cliente) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_emprestimo_ref_cod_exemplar_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_emprestimo
    ADD CONSTRAINT exemplar_emprestimo_ref_cod_exemplar_fkey FOREIGN KEY (ref_cod_exemplar) REFERENCES exemplar(cod_exemplar) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_emprestimo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_emprestimo
    ADD CONSTRAINT exemplar_emprestimo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_emprestimo_ref_usuario_devolucao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_emprestimo
    ADD CONSTRAINT exemplar_emprestimo_ref_usuario_devolucao_fkey FOREIGN KEY (ref_usuario_devolucao) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_ref_cod_acervo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_ref_cod_acervo_fkey FOREIGN KEY (ref_cod_acervo) REFERENCES acervo(cod_acervo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_ref_cod_fonte_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_ref_cod_fonte_fkey FOREIGN KEY (ref_cod_fonte) REFERENCES fonte(cod_fonte) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_ref_cod_motivo_baixa_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_ref_cod_motivo_baixa_fkey FOREIGN KEY (ref_cod_motivo_baixa) REFERENCES motivo_baixa(cod_motivo_baixa) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_ref_cod_situacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_ref_cod_situacao_fkey FOREIGN KEY (ref_cod_situacao) REFERENCES situacao(cod_situacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar
    ADD CONSTRAINT exemplar_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_tipo_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_tipo
    ADD CONSTRAINT exemplar_tipo_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_tipo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_tipo
    ADD CONSTRAINT exemplar_tipo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: exemplar_tipo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY exemplar_tipo
    ADD CONSTRAINT exemplar_tipo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_aluno_ref_cod_curso_disciplina; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_aluno
    ADD CONSTRAINT falta_aluno_ref_cod_curso_disciplina FOREIGN KEY (ref_cod_curso_disciplina) REFERENCES disciplina(cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_aluno_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_aluno
    ADD CONSTRAINT falta_aluno_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie, ref_cod_escola, ref_cod_disciplina) REFERENCES escola_serie_disciplina(ref_ref_cod_serie, ref_ref_cod_escola, ref_cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_aluno_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_aluno
    ADD CONSTRAINT falta_aluno_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_aluno_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_aluno
    ADD CONSTRAINT falta_aluno_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_compensado_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso_compensado
    ADD CONSTRAINT falta_atraso_compensado_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_compensado_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso_compensado
    ADD CONSTRAINT falta_atraso_compensado_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_compensado_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso_compensado
    ADD CONSTRAINT falta_atraso_compensado_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_compensado_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso_compensado
    ADD CONSTRAINT falta_atraso_compensado_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso
    ADD CONSTRAINT falta_atraso_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso
    ADD CONSTRAINT falta_atraso_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso
    ADD CONSTRAINT falta_atraso_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: falta_atraso_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY falta_atraso
    ADD CONSTRAINT falta_atraso_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: faltas_ref_cod_matricula_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY faltas
    ADD CONSTRAINT faltas_ref_cod_matricula_fkey FOREIGN KEY (ref_cod_matricula) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: faltas_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY faltas
    ADD CONSTRAINT faltas_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_ref_cod_biblioteca_cliente; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY cliente_tipo_cliente
    ADD CONSTRAINT fk_ref_cod_biblioteca_cliente FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_ref_cod_exemplar; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reservas
    ADD CONSTRAINT fk_ref_cod_exemplar FOREIGN KEY (ref_cod_exemplar) REFERENCES exemplar(cod_exemplar) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fonte_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY fonte
    ADD CONSTRAINT fonte_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fonte_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY fonte
    ADD CONSTRAINT fonte_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fonte_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY fonte
    ADD CONSTRAINT fonte_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: funca_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY funcao
    ADD CONSTRAINT funca_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: funcao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY funcao
    ADD CONSTRAINT funcao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: funcao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY funcao
    ADD CONSTRAINT funcao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: habilitacao_curso_ref_cod_curso_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY habilitacao_curso
    ADD CONSTRAINT habilitacao_curso_ref_cod_curso_fkey FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: habilitacao_curso_ref_cod_habilitacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY habilitacao_curso
    ADD CONSTRAINT habilitacao_curso_ref_cod_habilitacao_fkey FOREIGN KEY (ref_cod_habilitacao) REFERENCES habilitacao(cod_habilitacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: habilitacao_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY habilitacao
    ADD CONSTRAINT habilitacao_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: habilitacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY habilitacao
    ADD CONSTRAINT habilitacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: habilitacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY habilitacao
    ADD CONSTRAINT habilitacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: historico_disciplinas_ref_ref_cod_aluno_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY historico_disciplinas
    ADD CONSTRAINT historico_disciplinas_ref_ref_cod_aluno_fkey FOREIGN KEY (ref_ref_cod_aluno, ref_sequencial) REFERENCES historico_escolar(ref_cod_aluno, sequencial) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: historico_escolar_ref_cod_aluno_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY historico_escolar
    ADD CONSTRAINT historico_escolar_ref_cod_aluno_fkey FOREIGN KEY (ref_cod_aluno) REFERENCES aluno(cod_aluno) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: historico_escolar_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY historico_escolar
    ADD CONSTRAINT historico_escolar_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: historico_escolar_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY historico_escolar
    ADD CONSTRAINT historico_escolar_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_comodo_funcao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_comodo_funcao
    ADD CONSTRAINT infra_comodo_funcao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_comodo_funcao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_comodo_funcao
    ADD CONSTRAINT infra_comodo_funcao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_comodo_ref_cod_escola; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_comodo_funcao
    ADD CONSTRAINT infra_comodo_ref_cod_escola FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_comodo_ref_cod_infra_comodo_funcao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio_comodo
    ADD CONSTRAINT infra_predio_comodo_ref_cod_infra_comodo_funcao_fkey FOREIGN KEY (ref_cod_infra_comodo_funcao) REFERENCES infra_comodo_funcao(cod_infra_comodo_funcao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_comodo_ref_cod_infra_predio_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio_comodo
    ADD CONSTRAINT infra_predio_comodo_ref_cod_infra_predio_fkey FOREIGN KEY (ref_cod_infra_predio) REFERENCES infra_predio(cod_infra_predio) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_comodo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio_comodo
    ADD CONSTRAINT infra_predio_comodo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_comodo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio_comodo
    ADD CONSTRAINT infra_predio_comodo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio
    ADD CONSTRAINT infra_predio_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio
    ADD CONSTRAINT infra_predio_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: infra_predio_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY infra_predio
    ADD CONSTRAINT infra_predio_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: instituicao_ref_idtlog_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY instituicao
    ADD CONSTRAINT instituicao_ref_idtlog_fkey FOREIGN KEY (ref_idtlog) REFERENCES urbano.tipo_logradouro(idtlog) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: instituicao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY instituicao
    ADD CONSTRAINT instituicao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: instituicao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY instituicao
    ADD CONSTRAINT instituicao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_didatico_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_didatico
    ADD CONSTRAINT material_didatico_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_didatico_ref_cod_material_tipo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_didatico
    ADD CONSTRAINT material_didatico_ref_cod_material_tipo_fkey FOREIGN KEY (ref_cod_material_tipo) REFERENCES material_tipo(cod_material_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_didatico_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_didatico
    ADD CONSTRAINT material_didatico_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_didatico_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_didatico
    ADD CONSTRAINT material_didatico_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_tipo_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_tipo
    ADD CONSTRAINT material_tipo_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_tipo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_tipo
    ADD CONSTRAINT material_tipo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: material_tipo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY material_tipo
    ADD CONSTRAINT material_tipo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_excessao_ref_cod_disciplina_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_excessao
    ADD CONSTRAINT matricula_excessao_ref_cod_disciplina_fkey FOREIGN KEY (ref_cod_serie, ref_cod_escola, ref_cod_disciplina) REFERENCES escola_serie_disciplina(ref_ref_cod_serie, ref_ref_cod_escola, ref_cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_excessao_ref_cod_matricula_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_excessao
    ADD CONSTRAINT matricula_excessao_ref_cod_matricula_fkey FOREIGN KEY (ref_cod_matricula, ref_cod_turma, ref_sequencial) REFERENCES matricula_turma(ref_cod_matricula, ref_cod_turma, sequencial) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ocorrencia_discipli_ref_cod_tipo_ocorrencia_disc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_ocorrencia_disciplinar
    ADD CONSTRAINT matricula_ocorrencia_discipli_ref_cod_tipo_ocorrencia_disc_fkey FOREIGN KEY (ref_cod_tipo_ocorrencia_disciplinar) REFERENCES tipo_ocorrencia_disciplinar(cod_tipo_ocorrencia_disciplinar) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ocorrencia_disciplinar_ref_cod_matricula_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_ocorrencia_disciplinar
    ADD CONSTRAINT matricula_ocorrencia_disciplinar_ref_cod_matricula_fkey FOREIGN KEY (ref_cod_matricula) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ocorrencia_disciplinar_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_ocorrencia_disciplinar
    ADD CONSTRAINT matricula_ocorrencia_disciplinar_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ocorrencia_disciplinar_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_ocorrencia_disciplinar
    ADD CONSTRAINT matricula_ocorrencia_disciplinar_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ref_cod_aluno_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula
    ADD CONSTRAINT matricula_ref_cod_aluno_fkey FOREIGN KEY (ref_cod_aluno) REFERENCES aluno(cod_aluno) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ref_cod_curso; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula
    ADD CONSTRAINT matricula_ref_cod_curso FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ref_cod_reserva_vaga_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula
    ADD CONSTRAINT matricula_ref_cod_reserva_vaga_fkey FOREIGN KEY (ref_cod_reserva_vaga) REFERENCES reserva_vaga(cod_reserva_vaga) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula
    ADD CONSTRAINT matricula_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula
    ADD CONSTRAINT matricula_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_turma_ref_cod_matricula_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_turma
    ADD CONSTRAINT matricula_turma_ref_cod_matricula_fkey FOREIGN KEY (ref_cod_matricula) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_turma_ref_cod_turma_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_turma
    ADD CONSTRAINT matricula_turma_ref_cod_turma_fkey FOREIGN KEY (ref_cod_turma) REFERENCES turma(cod_turma) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_turma_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_turma
    ADD CONSTRAINT matricula_turma_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: matricula_turma_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY matricula_turma
    ADD CONSTRAINT matricula_turma_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: menu_tipo_usuario_ref_cod_menu_submenu_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY menu_tipo_usuario
    ADD CONSTRAINT menu_tipo_usuario_ref_cod_menu_submenu_fkey FOREIGN KEY (ref_cod_menu_submenu) REFERENCES portal.menu_submenu(cod_menu_submenu) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: menu_tipo_usuario_ref_cod_tipo_usuario_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY menu_tipo_usuario
    ADD CONSTRAINT menu_tipo_usuario_ref_cod_tipo_usuario_fkey FOREIGN KEY (ref_cod_tipo_usuario) REFERENCES tipo_usuario(cod_tipo_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: modulo_ref_cod_instituicao_fk; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY modulo
    ADD CONSTRAINT modulo_ref_cod_instituicao_fk FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: modulo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY modulo
    ADD CONSTRAINT modulo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: modulo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY modulo
    ADD CONSTRAINT modulo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_afastamento_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_afastamento
    ADD CONSTRAINT motivo_afastamento_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_afastamento_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_afastamento
    ADD CONSTRAINT motivo_afastamento_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_afastamento_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_afastamento
    ADD CONSTRAINT motivo_afastamento_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_baixa_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_baixa
    ADD CONSTRAINT motivo_baixa_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_baixa_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_baixa
    ADD CONSTRAINT motivo_baixa_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_baixa_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_baixa
    ADD CONSTRAINT motivo_baixa_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_suspensao_ref_cod_biblioteca_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_suspensao
    ADD CONSTRAINT motivo_suspensao_ref_cod_biblioteca_fkey FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_suspensao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_suspensao
    ADD CONSTRAINT motivo_suspensao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: motivo_suspensao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY motivo_suspensao
    ADD CONSTRAINT motivo_suspensao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ensino_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel_ensino
    ADD CONSTRAINT nivel_ensino_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ensino_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel_ensino
    ADD CONSTRAINT nivel_ensino_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ensino_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel_ensino
    ADD CONSTRAINT nivel_ensino_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ref_cod_categoria_nivel_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel
    ADD CONSTRAINT nivel_ref_cod_categoria_nivel_fkey FOREIGN KEY (ref_cod_categoria_nivel) REFERENCES categoria_nivel(cod_categoria_nivel) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ref_cod_nivel_anterior_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel
    ADD CONSTRAINT nivel_ref_cod_nivel_anterior_fkey FOREIGN KEY (ref_cod_nivel_anterior) REFERENCES nivel(cod_nivel) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel
    ADD CONSTRAINT nivel_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nivel_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nivel
    ADD CONSTRAINT nivel_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nota_aluno_ref_cod_curso_disciplina; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_ref_cod_curso_disciplina FOREIGN KEY (ref_cod_curso_disciplina) REFERENCES disciplina(cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nota_aluno_ref_cod_matricula_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_ref_cod_matricula_fkey FOREIGN KEY (ref_cod_matricula) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nota_aluno_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie, ref_cod_escola, ref_cod_disciplina) REFERENCES escola_serie_disciplina(ref_ref_cod_serie, ref_ref_cod_escola, ref_cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nota_aluno_ref_ref_cod_tipo_avaliacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_ref_ref_cod_tipo_avaliacao_fkey FOREIGN KEY (ref_ref_cod_tipo_avaliacao, ref_sequencial) REFERENCES tipo_avaliacao_valores(ref_cod_tipo_avaliacao, sequencial) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nota_aluno_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nota_aluno_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY nota_aluno
    ADD CONSTRAINT nota_aluno_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: operador_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY operador
    ADD CONSTRAINT operador_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: operador_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY operador
    ADD CONSTRAINT operador_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pagamento_divida_ref_cod_biblioteca; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY pagamento_multa
    ADD CONSTRAINT pagamento_divida_ref_cod_biblioteca FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pagamento_multa_ref_cod_cliente_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY pagamento_multa
    ADD CONSTRAINT pagamento_multa_ref_cod_cliente_fkey FOREIGN KEY (ref_cod_cliente) REFERENCES cliente(cod_cliente) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pagamento_multa_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY pagamento_multa
    ADD CONSTRAINT pagamento_multa_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pre_requisito_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY pre_requisito
    ADD CONSTRAINT pre_requisito_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pre_requisito_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY pre_requisito
    ADD CONSTRAINT pre_requisito_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_horarios_ref_cod_quadro_horario_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario_horarios_aux
    ADD CONSTRAINT quadro_horario_horarios_ref_cod_quadro_horario_fkey FOREIGN KEY (ref_cod_quadro_horario) REFERENCES quadro_horario(cod_quadro_horario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_horarios_ref_cod_quadro_horario_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario_horarios
    ADD CONSTRAINT quadro_horario_horarios_ref_cod_quadro_horario_fkey FOREIGN KEY (ref_cod_quadro_horario) REFERENCES quadro_horario(cod_quadro_horario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_horarios_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario_horarios_aux
    ADD CONSTRAINT quadro_horario_horarios_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie, ref_cod_escola, ref_cod_disciplina) REFERENCES escola_serie_disciplina(ref_ref_cod_serie, ref_ref_cod_escola, ref_cod_disciplina) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_horarios_ref_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario_horarios_aux
    ADD CONSTRAINT quadro_horario_horarios_ref_servidor_fkey FOREIGN KEY (ref_servidor, ref_cod_instituicao_servidor) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_horarios_ref_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario_horarios
    ADD CONSTRAINT quadro_horario_horarios_ref_servidor_fkey FOREIGN KEY (ref_servidor, ref_cod_instituicao_servidor) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_horarios_ref_servidor_substituto_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario_horarios
    ADD CONSTRAINT quadro_horario_horarios_ref_servidor_substituto_fkey FOREIGN KEY (ref_servidor_substituto, ref_cod_instituicao_substituto) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_ref_cod_turma_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario
    ADD CONSTRAINT quadro_horario_ref_cod_turma_fkey FOREIGN KEY (ref_cod_turma) REFERENCES turma(cod_turma) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario
    ADD CONSTRAINT quadro_horario_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: quadro_horario_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY quadro_horario
    ADD CONSTRAINT quadro_horario_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: religiao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY religiao
    ADD CONSTRAINT religiao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: religiao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY religiao
    ADD CONSTRAINT religiao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reserva_vaga_ref_cod_aluno_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reserva_vaga
    ADD CONSTRAINT reserva_vaga_ref_cod_aluno_fkey FOREIGN KEY (ref_cod_aluno) REFERENCES aluno(cod_aluno) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reserva_vaga_ref_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reserva_vaga
    ADD CONSTRAINT reserva_vaga_ref_ref_cod_serie_fkey FOREIGN KEY (ref_ref_cod_serie, ref_ref_cod_escola) REFERENCES escola_serie(ref_cod_serie, ref_cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reserva_vaga_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reserva_vaga
    ADD CONSTRAINT reserva_vaga_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reserva_vaga_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reserva_vaga
    ADD CONSTRAINT reserva_vaga_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reservas_ref_cod_cliente_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reservas
    ADD CONSTRAINT reservas_ref_cod_cliente_fkey FOREIGN KEY (ref_cod_cliente) REFERENCES cliente(cod_cliente) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reservas_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reservas
    ADD CONSTRAINT reservas_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reservas_ref_usuario_libera_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY reservas
    ADD CONSTRAINT reservas_ref_usuario_libera_fkey FOREIGN KEY (ref_usuario_libera) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: sequencia_serie_ref_serie_destino_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY sequencia_serie
    ADD CONSTRAINT sequencia_serie_ref_serie_destino_fkey FOREIGN KEY (ref_serie_destino) REFERENCES serie(cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: sequencia_serie_ref_serie_origem_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY sequencia_serie
    ADD CONSTRAINT sequencia_serie_ref_serie_origem_fkey FOREIGN KEY (ref_serie_origem) REFERENCES serie(cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: sequencia_serie_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY sequencia_serie
    ADD CONSTRAINT sequencia_serie_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: sequencia_serie_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY sequencia_serie
    ADD CONSTRAINT sequencia_serie_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_pre_requisito_ref_cod_operador_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie_pre_requisito
    ADD CONSTRAINT serie_pre_requisito_ref_cod_operador_fkey FOREIGN KEY (ref_cod_operador) REFERENCES operador(cod_operador) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_pre_requisito_ref_cod_pre_requisito_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie_pre_requisito
    ADD CONSTRAINT serie_pre_requisito_ref_cod_pre_requisito_fkey FOREIGN KEY (ref_cod_pre_requisito) REFERENCES pre_requisito(cod_pre_requisito) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_pre_requisito_ref_cod_serie_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie_pre_requisito
    ADD CONSTRAINT serie_pre_requisito_ref_cod_serie_fkey FOREIGN KEY (ref_cod_serie) REFERENCES serie(cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_ref_cod_curso_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie
    ADD CONSTRAINT serie_ref_cod_curso_fkey FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie
    ADD CONSTRAINT serie_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie
    ADD CONSTRAINT serie_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: serie_regra_avaliacao_fk; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY serie
    ADD CONSTRAINT serie_regra_avaliacao_fk FOREIGN KEY (regra_avaliacao_id) REFERENCES modules.regra_avaliacao(id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_afastamento_ref_cod_motivo_afastamento_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_afastamento
    ADD CONSTRAINT servidor_afastamento_ref_cod_motivo_afastamento_fkey FOREIGN KEY (ref_cod_motivo_afastamento) REFERENCES motivo_afastamento(cod_motivo_afastamento) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_afastamento_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_afastamento
    ADD CONSTRAINT servidor_afastamento_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_afastamento_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_afastamento
    ADD CONSTRAINT servidor_afastamento_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_afastamento_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_afastamento
    ADD CONSTRAINT servidor_afastamento_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_alocacao_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_alocacao
    ADD CONSTRAINT servidor_alocacao_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_alocacao_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_alocacao
    ADD CONSTRAINT servidor_alocacao_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_alocacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_alocacao
    ADD CONSTRAINT servidor_alocacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_alocacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_alocacao
    ADD CONSTRAINT servidor_alocacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor
    ADD CONSTRAINT servidor_cod_servidor_fkey FOREIGN KEY (cod_servidor) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_curso_ref_cod_formacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_curso
    ADD CONSTRAINT servidor_curso_ref_cod_formacao_fkey FOREIGN KEY (ref_cod_formacao) REFERENCES servidor_formacao(cod_formacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_cuso_ministra_ref_cod_curso_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_curso_ministra
    ADD CONSTRAINT servidor_cuso_ministra_ref_cod_curso_fkey FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_cuso_ministra_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_curso_ministra
    ADD CONSTRAINT servidor_cuso_ministra_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_disciplina_ref_cod_disciplina_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_disciplina
    ADD CONSTRAINT servidor_disciplina_ref_cod_disciplina_fkey FOREIGN KEY (ref_cod_disciplina) REFERENCES modules.componente_curricular(id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_disciplina_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_disciplina
    ADD CONSTRAINT servidor_disciplina_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_formacao_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_formacao
    ADD CONSTRAINT servidor_formacao_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_formacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_formacao
    ADD CONSTRAINT servidor_formacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_formacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_formacao
    ADD CONSTRAINT servidor_formacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_funcao_ref_cod_funcao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_funcao
    ADD CONSTRAINT servidor_funcao_ref_cod_funcao_fkey FOREIGN KEY (ref_cod_funcao) REFERENCES funcao(cod_funcao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_funcao_ref_cod_servidor_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_funcao
    ADD CONSTRAINT servidor_funcao_ref_cod_servidor_fkey FOREIGN KEY (ref_cod_servidor, ref_ref_cod_instituicao) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_ref_cod_deficiencia_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor
    ADD CONSTRAINT servidor_ref_cod_deficiencia_fkey FOREIGN KEY (ref_cod_deficiencia) REFERENCES cadastro.deficiencia(cod_deficiencia) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_ref_cod_instituicao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor
    ADD CONSTRAINT servidor_ref_cod_instituicao_fkey FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_ref_cod_subnivel_; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor
    ADD CONSTRAINT servidor_ref_cod_subnivel_ FOREIGN KEY (ref_cod_subnivel) REFERENCES subnivel(cod_subnivel) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_ref_idesco_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor
    ADD CONSTRAINT servidor_ref_idesco_fkey FOREIGN KEY (ref_idesco) REFERENCES cadastro.escolaridade(idesco) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: servidor_titulo_concurso_ref_cod_formacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY servidor_titulo_concurso
    ADD CONSTRAINT servidor_titulo_concurso_ref_cod_formacao_fkey FOREIGN KEY (ref_cod_formacao) REFERENCES servidor_formacao(cod_formacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: situacao_ref_cod_biblioteca; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY situacao
    ADD CONSTRAINT situacao_ref_cod_biblioteca FOREIGN KEY (ref_cod_biblioteca) REFERENCES biblioteca(cod_biblioteca) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: situacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY situacao
    ADD CONSTRAINT situacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: situacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY situacao
    ADD CONSTRAINT situacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: subnivel_ref_cod_nivel_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY subnivel
    ADD CONSTRAINT subnivel_ref_cod_nivel_fkey FOREIGN KEY (ref_cod_nivel) REFERENCES nivel(cod_nivel) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: subnivel_ref_cod_subnivel_anterior_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY subnivel
    ADD CONSTRAINT subnivel_ref_cod_subnivel_anterior_fkey FOREIGN KEY (ref_cod_subnivel_anterior) REFERENCES subnivel(cod_subnivel) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: subnivel_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY subnivel
    ADD CONSTRAINT subnivel_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: subnivel_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY subnivel
    ADD CONSTRAINT subnivel_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_avaliacao_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_avaliacao
    ADD CONSTRAINT tipo_avaliacao_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_avaliacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_avaliacao
    ADD CONSTRAINT tipo_avaliacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_avaliacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_avaliacao
    ADD CONSTRAINT tipo_avaliacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_avaliacao_valores_ref_cod_tipo_avaliacao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_avaliacao_valores
    ADD CONSTRAINT tipo_avaliacao_valores_ref_cod_tipo_avaliacao_fkey FOREIGN KEY (ref_cod_tipo_avaliacao) REFERENCES tipo_avaliacao(cod_tipo_avaliacao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_dispensa_ref_cod_instituicao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_dispensa
    ADD CONSTRAINT tipo_dispensa_ref_cod_instituicao_fkey FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_dispensa_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_dispensa
    ADD CONSTRAINT tipo_dispensa_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_dispensa_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_dispensa
    ADD CONSTRAINT tipo_dispensa_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_ensino_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_ensino
    ADD CONSTRAINT tipo_ensino_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_ensino_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_ensino
    ADD CONSTRAINT tipo_ensino_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_ensino_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_ensino
    ADD CONSTRAINT tipo_ensino_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_ocorrencia_disciplinar_ref_cod_instituicao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_ocorrencia_disciplinar
    ADD CONSTRAINT tipo_ocorrencia_disciplinar_ref_cod_instituicao_fkey FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_ocorrencia_disciplinar_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_ocorrencia_disciplinar
    ADD CONSTRAINT tipo_ocorrencia_disciplinar_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_ocorrencia_disciplinar_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_ocorrencia_disciplinar
    ADD CONSTRAINT tipo_ocorrencia_disciplinar_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_regime_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_regime
    ADD CONSTRAINT tipo_regime_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_regime_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_regime
    ADD CONSTRAINT tipo_regime_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_regime_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_regime
    ADD CONSTRAINT tipo_regime_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_usuario_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_usuario
    ADD CONSTRAINT tipo_usuario_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: tipo_usuario_ref_funcionario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY tipo_usuario
    ADD CONSTRAINT tipo_usuario_ref_funcionario_exc_fkey FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_solicitacao_ref_cod_matricula_entrada_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_solicitacao
    ADD CONSTRAINT transferencia_solicitacao_ref_cod_matricula_entrada_fkey FOREIGN KEY (ref_cod_matricula_entrada) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_solicitacao_ref_cod_matricula_saida_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_solicitacao
    ADD CONSTRAINT transferencia_solicitacao_ref_cod_matricula_saida_fkey FOREIGN KEY (ref_cod_matricula_saida) REFERENCES matricula(cod_matricula) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_solicitacao_ref_cod_transferencia_tipo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_solicitacao
    ADD CONSTRAINT transferencia_solicitacao_ref_cod_transferencia_tipo_fkey FOREIGN KEY (ref_cod_transferencia_tipo) REFERENCES transferencia_tipo(cod_transferencia_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_solicitacao_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_solicitacao
    ADD CONSTRAINT transferencia_solicitacao_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_solicitacao_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_solicitacao
    ADD CONSTRAINT transferencia_solicitacao_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_tipo_ref_cod_escola; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_tipo
    ADD CONSTRAINT transferencia_tipo_ref_cod_escola FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_tipo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_tipo
    ADD CONSTRAINT transferencia_tipo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: transferencia_tipo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY transferencia_tipo
    ADD CONSTRAINT transferencia_tipo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_dia_semana_ref_cod_turma_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma_dia_semana
    ADD CONSTRAINT turma_dia_semana_ref_cod_turma_fkey FOREIGN KEY (ref_cod_turma) REFERENCES turma(cod_turma) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_escola_serie_muil; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_escola_serie_muil FOREIGN KEY (ref_ref_cod_serie_mult, ref_ref_cod_escola_mult) REFERENCES escola_serie(ref_cod_serie, ref_cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_modulo_ref_cod_modulo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma_modulo
    ADD CONSTRAINT turma_modulo_ref_cod_modulo_fkey FOREIGN KEY (ref_cod_modulo) REFERENCES modulo(cod_modulo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_modulo_ref_cod_turma_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma_modulo
    ADD CONSTRAINT turma_modulo_ref_cod_turma_fkey FOREIGN KEY (ref_cod_turma) REFERENCES turma(cod_turma) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_cod_curso; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_cod_curso FOREIGN KEY (ref_cod_curso) REFERENCES curso(cod_curso) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_cod_infra_predio_comodo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_cod_infra_predio_comodo_fkey FOREIGN KEY (ref_cod_infra_predio_comodo) REFERENCES infra_predio_comodo(cod_infra_predio_comodo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_cod_instituicao; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_cod_instituicao FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_cod_regente; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_cod_regente FOREIGN KEY (ref_cod_regente, ref_cod_instituicao_regente) REFERENCES servidor(cod_servidor, ref_cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_ref_cod_escola_fkey FOREIGN KEY (ref_ref_cod_escola, ref_ref_cod_serie) REFERENCES escola_serie(ref_cod_escola, ref_cod_serie) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_tipo_ref_usuario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma_tipo
    ADD CONSTRAINT turma_tipo_ref_usuario_cad_fkey FOREIGN KEY (ref_usuario_cad) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_tipo_ref_usuario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma_tipo
    ADD CONSTRAINT turma_tipo_ref_usuario_exc_fkey FOREIGN KEY (ref_usuario_exc) REFERENCES usuario(cod_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: turma_turma_tipo_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY turma
    ADD CONSTRAINT turma_turma_tipo_fkey FOREIGN KEY (ref_cod_turma_tipo) REFERENCES turma_tipo(cod_turma_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: usuario_cod_usuario_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_cod_usuario_fkey FOREIGN KEY (cod_usuario) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: usuario_ref_cod_escola_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_ref_cod_escola_fkey FOREIGN KEY (ref_cod_escola) REFERENCES escola(cod_escola) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: usuario_ref_cod_instituicao_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_ref_cod_instituicao_fkey FOREIGN KEY (ref_cod_instituicao) REFERENCES instituicao(cod_instituicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: usuario_ref_cod_tipo_usuario_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_ref_cod_tipo_usuario_fkey FOREIGN KEY (ref_cod_tipo_usuario) REFERENCES tipo_usuario(cod_tipo_usuario) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: usuario_ref_funcionario_cad_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_ref_funcionario_cad_fkey FOREIGN KEY (ref_funcionario_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: usuario_ref_funcionario_exc_fkey; Type: FK CONSTRAINT; Schema: pmieducar; Owner: -
--

ALTER TABLE ONLY usuario
    ADD CONSTRAINT usuario_ref_funcionario_exc_fkey FOREIGN KEY (ref_funcionario_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


SET search_path = pmiotopic, pg_catalog;

--
-- Name: funcionario_su_ref_ref_cod_pessoa_fj_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY funcionario_su
    ADD CONSTRAINT funcionario_su_ref_ref_cod_pessoa_fj_fkey FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: grupomoderador_ref_cod_grupos_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupomoderador
    ADD CONSTRAINT grupomoderador_ref_cod_grupos_fkey FOREIGN KEY (ref_cod_grupos) REFERENCES grupos(cod_grupos) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupomoderador_ref_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupomoderador
    ADD CONSTRAINT grupomoderador_ref_pessoa_cad_fkey FOREIGN KEY (ref_pessoa_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupomoderador_ref_pessoa_exc_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupomoderador
    ADD CONSTRAINT grupomoderador_ref_pessoa_exc_fkey FOREIGN KEY (ref_pessoa_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupomoderador_ref_ref_cod_pessoa_fj_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupomoderador
    ADD CONSTRAINT grupomoderador_ref_ref_cod_pessoa_fj_fkey FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupopessoa_ref_cod_grupos_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupopessoa
    ADD CONSTRAINT grupopessoa_ref_cod_grupos_fkey FOREIGN KEY (ref_cod_grupos) REFERENCES grupos(cod_grupos) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupopessoa_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupopessoa
    ADD CONSTRAINT grupopessoa_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES cadastro.fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupopessoa_ref_pessoa_cadatro_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupopessoa
    ADD CONSTRAINT grupopessoa_ref_pessoa_cadatro_fkey FOREIGN KEY (ref_pessoa_cad, ref_grupos_cad) REFERENCES grupomoderador(ref_ref_cod_pessoa_fj, ref_cod_grupos) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupopessoa_ref_pessoa_exclusao_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupopessoa
    ADD CONSTRAINT grupopessoa_ref_pessoa_exclusao_fkey FOREIGN KEY (ref_pessoa_exc, ref_grupos_exc) REFERENCES grupomoderador(ref_ref_cod_pessoa_fj, ref_cod_grupos) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupos_ref_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupos
    ADD CONSTRAINT grupos_ref_pessoa_cad_fkey FOREIGN KEY (ref_pessoa_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: grupos_ref_pessoa_exc_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY grupos
    ADD CONSTRAINT grupos_ref_pessoa_exc_fkey FOREIGN KEY (ref_pessoa_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: notas_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY notas
    ADD CONSTRAINT notas_ref_idpes_fkey FOREIGN KEY (ref_idpes) REFERENCES cadastro.fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: notas_ref_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY notas
    ADD CONSTRAINT notas_ref_pessoa_cad_fkey FOREIGN KEY (ref_pessoa_cad) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: notas_ref_pessoa_exc_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY notas
    ADD CONSTRAINT notas_ref_pessoa_exc_fkey FOREIGN KEY (ref_pessoa_exc) REFERENCES portal.funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: participante_ref_cod_reuniao_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY participante
    ADD CONSTRAINT participante_ref_cod_reuniao_fkey FOREIGN KEY (ref_cod_reuniao) REFERENCES reuniao(cod_reuniao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: participante_ref_ref_idpes_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY participante
    ADD CONSTRAINT participante_ref_ref_idpes_fkey FOREIGN KEY (ref_ref_idpes, ref_ref_cod_grupos) REFERENCES grupopessoa(ref_idpes, ref_cod_grupos) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: reuniao_ref_moderador_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY reuniao
    ADD CONSTRAINT reuniao_ref_moderador_fkey FOREIGN KEY (ref_moderador, ref_grupos_moderador) REFERENCES grupomoderador(ref_ref_cod_pessoa_fj, ref_cod_grupos) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: topicoreuniao_ref_cod_reuniao_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY topicoreuniao
    ADD CONSTRAINT topicoreuniao_ref_cod_reuniao_fkey FOREIGN KEY (ref_cod_reuniao) REFERENCES reuniao(cod_reuniao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: topicoreuniao_ref_cod_topico_fkey; Type: FK CONSTRAINT; Schema: pmiotopic; Owner: -
--

ALTER TABLE ONLY topicoreuniao
    ADD CONSTRAINT topicoreuniao_ref_cod_topico_fkey FOREIGN KEY (ref_cod_topico) REFERENCES topico(cod_topico) ON UPDATE RESTRICT ON DELETE RESTRICT;


SET search_path = portal, pg_catalog;

--
-- Name: agenda_compromisso_ref_cod_agenda_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda_compromisso
    ADD CONSTRAINT agenda_compromisso_ref_cod_agenda_fkey FOREIGN KEY (ref_cod_agenda) REFERENCES agenda(cod_agenda) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: agenda_compromisso_ref_ref_cod_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda_compromisso
    ADD CONSTRAINT agenda_compromisso_ref_ref_cod_pessoa_cad_fkey FOREIGN KEY (ref_ref_cod_pessoa_cad) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: agenda_ref_ref_cod_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda
    ADD CONSTRAINT agenda_ref_ref_cod_pessoa_cad_fkey FOREIGN KEY (ref_ref_cod_pessoa_cad) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: agenda_ref_ref_cod_pessoa_exc_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda
    ADD CONSTRAINT agenda_ref_ref_cod_pessoa_exc_fkey FOREIGN KEY (ref_ref_cod_pessoa_exc) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: agenda_ref_ref_cod_pessoa_own_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda
    ADD CONSTRAINT agenda_ref_ref_cod_pessoa_own_fkey FOREIGN KEY (ref_ref_cod_pessoa_own) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: agenda_responsavel_ref_cod_agenda_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda_responsavel
    ADD CONSTRAINT agenda_responsavel_ref_cod_agenda_fkey FOREIGN KEY (ref_cod_agenda) REFERENCES agenda(cod_agenda) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: agenda_responsavel_ref_ref_cod_pessoa_fj_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY agenda_responsavel
    ADD CONSTRAINT agenda_responsavel_ref_ref_cod_pessoa_fj_fkey FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_editais_editais_empresas_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_editais_editais_empresas
    ADD CONSTRAINT compras_editais_editais_empresas_ibfk_1 FOREIGN KEY (ref_cod_compras_editais_editais) REFERENCES compras_editais_editais(cod_compras_editais_editais) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_editais_editais_empresas_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_editais_editais_empresas
    ADD CONSTRAINT compras_editais_editais_empresas_ibfk_2 FOREIGN KEY (ref_cod_compras_editais_empresa) REFERENCES compras_editais_empresa(cod_compras_editais_empresa) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_editais_editais_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_editais_editais
    ADD CONSTRAINT compras_editais_editais_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_editais_editais_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_editais_editais
    ADD CONSTRAINT compras_editais_editais_ibfk_2 FOREIGN KEY (ref_cod_compras_licitacoes) REFERENCES compras_licitacoes(cod_compras_licitacoes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_editais_empresa_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_editais_empresa
    ADD CONSTRAINT compras_editais_empresa_ibfk_1 FOREIGN KEY (ref_sigla_uf) REFERENCES public.uf(sigla_uf) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_funcionarios_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_funcionarios
    ADD CONSTRAINT compras_funcionarios_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_licitacoes_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_licitacoes
    ADD CONSTRAINT compras_licitacoes_ibfk_1 FOREIGN KEY (ref_cod_compras_modalidade) REFERENCES compras_modalidade(cod_compras_modalidade) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_licitacoes_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_licitacoes
    ADD CONSTRAINT compras_licitacoes_ibfk_2 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_pregao_execucao_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_ibfk_1 FOREIGN KEY (ref_equipe3) REFERENCES compras_funcionarios(ref_ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_pregao_execucao_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_ibfk_2 FOREIGN KEY (ref_pregoeiro) REFERENCES compras_funcionarios(ref_ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_pregao_execucao_ibfk_3; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_ibfk_3 FOREIGN KEY (ref_equipe1) REFERENCES compras_funcionarios(ref_ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_pregao_execucao_ibfk_4; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_ibfk_4 FOREIGN KEY (ref_equipe2) REFERENCES compras_funcionarios(ref_ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_pregao_execucao_ibfk_5; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_ibfk_5 FOREIGN KEY (ref_cod_compras_final_pregao) REFERENCES compras_final_pregao(cod_compras_final_pregao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: compras_pregao_execucao_ibfk_6; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY compras_pregao_execucao
    ADD CONSTRAINT compras_pregao_execucao_ibfk_6 FOREIGN KEY (ref_cod_compras_licitacoes) REFERENCES compras_licitacoes(cod_compras_licitacoes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_to_setor_new; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT fk_to_setor_new FOREIGN KEY (ref_cod_setor_new) REFERENCES pmidrh.setor(cod_setor) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: foto_portal_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY foto_portal
    ADD CONSTRAINT foto_portal_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: foto_portal_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY foto_portal
    ADD CONSTRAINT foto_portal_ibfk_2 FOREIGN KEY (ref_cod_credito) REFERENCES cadastro.fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: funcionario_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT funcionario_ibfk_1 FOREIGN KEY (ref_cod_pessoa_fj) REFERENCES cadastro.fisica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: funcionario_ibfk_5; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY funcionario
    ADD CONSTRAINT funcionario_ibfk_5 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: imagem_ref_cod_imagem_tipo_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY imagem
    ADD CONSTRAINT imagem_ref_cod_imagem_tipo_fkey FOREIGN KEY (ref_cod_imagem_tipo) REFERENCES imagem_tipo(cod_imagem_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: imagem_ref_cod_pessoa_cad_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY imagem
    ADD CONSTRAINT imagem_ref_cod_pessoa_cad_fkey FOREIGN KEY (ref_cod_pessoa_cad) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: imagem_ref_cod_pessoa_exc_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY imagem
    ADD CONSTRAINT imagem_ref_cod_pessoa_exc_fkey FOREIGN KEY (ref_cod_pessoa_exc) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: intranet_segur_permissao_negada_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY intranet_segur_permissao_negada
    ADD CONSTRAINT intranet_segur_permissao_negada_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: jor_arquivo_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY jor_arquivo
    ADD CONSTRAINT jor_arquivo_ibfk_1 FOREIGN KEY (ref_cod_jor_edicao) REFERENCES jor_edicao(cod_jor_edicao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: jor_edicao_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY jor_edicao
    ADD CONSTRAINT jor_edicao_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_email_conteudo_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_email_conteudo
    ADD CONSTRAINT mailling_email_conteudo_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_fila_envio_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_fila_envio
    ADD CONSTRAINT mailling_fila_envio_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_fila_envio_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_fila_envio
    ADD CONSTRAINT mailling_fila_envio_ibfk_2 FOREIGN KEY (ref_cod_mailling_email) REFERENCES mailling_email(cod_mailling_email);


--
-- Name: mailling_fila_envio_ibfk_3; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_fila_envio
    ADD CONSTRAINT mailling_fila_envio_ibfk_3 FOREIGN KEY (ref_cod_mailling_email_conteudo) REFERENCES mailling_email_conteudo(cod_mailling_email_conteudo);


--
-- Name: mailling_grupo_email_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_grupo_email
    ADD CONSTRAINT mailling_grupo_email_ibfk_1 FOREIGN KEY (ref_cod_mailling_email) REFERENCES mailling_email(cod_mailling_email) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_grupo_email_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_grupo_email
    ADD CONSTRAINT mailling_grupo_email_ibfk_2 FOREIGN KEY (ref_cod_mailling_grupo) REFERENCES mailling_grupo(cod_mailling_grupo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_historico_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_historico
    ADD CONSTRAINT mailling_historico_ibfk_1 FOREIGN KEY (ref_cod_not_portal) REFERENCES not_portal(cod_not_portal) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_historico_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_historico
    ADD CONSTRAINT mailling_historico_ibfk_2 FOREIGN KEY (ref_cod_mailling_grupo) REFERENCES mailling_grupo(cod_mailling_grupo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: mailling_historico_ibfk_3; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY mailling_historico
    ADD CONSTRAINT mailling_historico_ibfk_3 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: menu_funcionario_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY menu_funcionario
    ADD CONSTRAINT menu_funcionario_ibfk_1 FOREIGN KEY (ref_cod_menu_submenu) REFERENCES menu_submenu(cod_menu_submenu) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: menu_funcionario_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY menu_funcionario
    ADD CONSTRAINT menu_funcionario_ibfk_2 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: menu_submenu_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY menu_submenu
    ADD CONSTRAINT menu_submenu_ibfk_1 FOREIGN KEY (ref_cod_menu_menu) REFERENCES menu_menu(cod_menu_menu) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: not_portal_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY not_portal
    ADD CONSTRAINT not_portal_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: not_portal_tipo_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY not_portal_tipo
    ADD CONSTRAINT not_portal_tipo_ibfk_1 FOREIGN KEY (ref_cod_not_portal) REFERENCES not_portal(cod_not_portal) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: not_portal_tipo_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY not_portal_tipo
    ADD CONSTRAINT not_portal_tipo_ibfk_2 FOREIGN KEY (ref_cod_not_tipo) REFERENCES not_tipo(cod_not_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: not_vinc_portal_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY not_vinc_portal
    ADD CONSTRAINT not_vinc_portal_ibfk_1 FOREIGN KEY (ref_cod_not_portal) REFERENCES not_portal(cod_not_portal) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: notificacao_ref_cod_funcionario_fkey; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY notificacao
    ADD CONSTRAINT notificacao_ref_cod_funcionario_fkey FOREIGN KEY (ref_cod_funcionario) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pessoa_atividade_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY pessoa_atividade
    ADD CONSTRAINT pessoa_atividade_ibfk_1 FOREIGN KEY (ref_cod_ramo_atividade) REFERENCES pessoa_ramo_atividade(cod_ramo_atividade) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pessoa_fj_pessoa_atividade_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY pessoa_fj_pessoa_atividade
    ADD CONSTRAINT pessoa_fj_pessoa_atividade_ibfk_1 FOREIGN KEY (ref_cod_pessoa_atividade) REFERENCES pessoa_atividade(cod_pessoa_atividade) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: pessoa_fj_pessoa_atividade_ibfk_2; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY pessoa_fj_pessoa_atividade
    ADD CONSTRAINT pessoa_fj_pessoa_atividade_ibfk_2 FOREIGN KEY (ref_cod_pessoa_fj) REFERENCES cadastro.juridica(idpes) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: portal_banner_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY portal_banner
    ADD CONSTRAINT portal_banner_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: portal_concurso_ibfk_1; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY portal_concurso
    ADD CONSTRAINT portal_concurso_ibfk_1 FOREIGN KEY (ref_ref_cod_pessoa_fj) REFERENCES funcionario(ref_cod_pessoa_fj) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: ref_cod_menu_pai_fk; Type: FK CONSTRAINT; Schema: portal; Owner: -
--

ALTER TABLE ONLY menu_menu
    ADD CONSTRAINT ref_cod_menu_pai_fk FOREIGN KEY (ref_cod_menu_pai) REFERENCES menu_menu(cod_menu_menu) ON UPDATE RESTRICT ON DELETE RESTRICT;


SET search_path = public, pg_catalog;

--
-- Name: bairro_regiao_ref_cod_regiao_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro_regiao
    ADD CONSTRAINT bairro_regiao_ref_cod_regiao_fkey FOREIGN KEY (ref_cod_regiao) REFERENCES regiao(cod_regiao) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: bairro_regiao_ref_idbai_fkey; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro_regiao
    ADD CONSTRAINT bairro_regiao_ref_idbai_fkey FOREIGN KEY (ref_idbai) REFERENCES bairro(idbai) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: fk_bairro_municipio; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro
    ADD CONSTRAINT fk_bairro_municipio FOREIGN KEY (idmun) REFERENCES municipio(idmun);


--
-- Name: fk_bairro_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro
    ADD CONSTRAINT fk_bairro_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_bairro_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro
    ADD CONSTRAINT fk_bairro_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_bairro_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro
    ADD CONSTRAINT fk_bairro_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_bairro_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY bairro
    ADD CONSTRAINT fk_bairro_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_logr_logr_fonetico; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY logradouro_fonetico
    ADD CONSTRAINT fk_logr_logr_fonetico FOREIGN KEY (idlog) REFERENCES logradouro(idlog);


--
-- Name: fk_logradouro_municipio; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_logradouro_municipio FOREIGN KEY (idmun) REFERENCES municipio(idmun);


--
-- Name: fk_logradouro_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY logradouro
    ADD CONSTRAINT fk_logradouro_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_logradouro_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY logradouro
    ADD CONSTRAINT fk_logradouro_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_logradouro_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY logradouro
    ADD CONSTRAINT fk_logradouro_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_logradouro_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY logradouro
    ADD CONSTRAINT fk_logradouro_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_logradouro_tipo_log; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY logradouro
    ADD CONSTRAINT fk_logradouro_tipo_log FOREIGN KEY (idtlog) REFERENCES urbano.tipo_logradouro(idtlog);


--
-- Name: fk_municipio_municipiopai; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_municipio_municipiopai FOREIGN KEY (idmun_pai) REFERENCES municipio(idmun);


--
-- Name: fk_municipio_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_municipio_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_municipio_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_municipio_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_municipio_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_municipio_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_municipio_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_municipio_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_municipio_uf; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY municipio
    ADD CONSTRAINT fk_municipio_uf FOREIGN KEY (sigla_uf) REFERENCES uf(sigla_uf);


--
-- Name: fk_setor_idsetredir; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT fk_setor_idsetredir FOREIGN KEY (idsetredir) REFERENCES setor(idset) ON DELETE RESTRICT;


--
-- Name: fk_setor_idsetsub; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY setor
    ADD CONSTRAINT fk_setor_idsetsub FOREIGN KEY (idsetsub) REFERENCES setor(idset) ON DELETE CASCADE;


--
-- Name: fk_uf_pais; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY uf
    ADD CONSTRAINT fk_uf_pais FOREIGN KEY (idpais) REFERENCES pais(idpais);


--
-- Name: fk_vila_municipio; Type: FK CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY vila
    ADD CONSTRAINT fk_vila_municipio FOREIGN KEY (idmun) REFERENCES municipio(idmun);


SET search_path = urbano, pg_catalog;

--
-- Name: fk_cep_log_bairro_bai; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT fk_cep_log_bairro_bai FOREIGN KEY (idbai) REFERENCES public.bairro(idbai);


--
-- Name: fk_cep_log_bairro_cep_log; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT fk_cep_log_bairro_cep_log FOREIGN KEY (cep, idlog) REFERENCES cep_logradouro(cep, idlog) ON DELETE CASCADE;


--
-- Name: fk_cep_logradouro_bairro_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT fk_cep_logradouro_bairro_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_bairro_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT fk_cep_logradouro_bairro_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_bairro_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT fk_cep_logradouro_bairro_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_bairro_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro_bairro
    ADD CONSTRAINT fk_cep_logradouro_bairro_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_logradouro; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro
    ADD CONSTRAINT fk_cep_logradouro_logradouro FOREIGN KEY (idlog) REFERENCES public.logradouro(idlog) ON DELETE CASCADE;


--
-- Name: fk_cep_logradouro_sistema_idpes_cad; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro
    ADD CONSTRAINT fk_cep_logradouro_sistema_idpes_cad FOREIGN KEY (idpes_cad) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_sistema_idpes_rev; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro
    ADD CONSTRAINT fk_cep_logradouro_sistema_idpes_rev FOREIGN KEY (idpes_rev) REFERENCES cadastro.pessoa(idpes) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_sistema_idsis_cad; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro
    ADD CONSTRAINT fk_cep_logradouro_sistema_idsis_cad FOREIGN KEY (idsis_cad) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- Name: fk_cep_logradouro_sistema_idsis_rev; Type: FK CONSTRAINT; Schema: urbano; Owner: -
--

ALTER TABLE ONLY cep_logradouro
    ADD CONSTRAINT fk_cep_logradouro_sistema_idsis_rev FOREIGN KEY (idsis_rev) REFERENCES acesso.sistema(idsis) ON DELETE SET NULL;


--
-- PostgreSQL database dump complete
--

